function s1(r,a){for(var i=0;i<a.length;i++){const o=a[i];if(typeof o!="string"&&!Array.isArray(o)){for(const s in o)if(s!=="default"&&!(s in r)){const f=Object.getOwnPropertyDescriptor(o,s);f&&Object.defineProperty(r,s,f.get?f:{enumerable:!0,get:()=>o[s]})}}}return Object.freeze(Object.defineProperty(r,Symbol.toStringTag,{value:"Module"}))}(function(){const a=document.createElement("link").relList;if(a&&a.supports&&a.supports("modulepreload"))return;for(const s of document.querySelectorAll('link[rel="modulepreload"]'))o(s);new MutationObserver(s=>{for(const f of s)if(f.type==="childList")for(const d of f.addedNodes)d.tagName==="LINK"&&d.rel==="modulepreload"&&o(d)}).observe(document,{childList:!0,subtree:!0});function i(s){const f={};return s.integrity&&(f.integrity=s.integrity),s.referrerPolicy&&(f.referrerPolicy=s.referrerPolicy),s.crossOrigin==="use-credentials"?f.credentials="include":s.crossOrigin==="anonymous"?f.credentials="omit":f.credentials="same-origin",f}function o(s){if(s.ep)return;s.ep=!0;const f=i(s);fetch(s.href,f)}})();function c1(r){return r&&r.__esModule&&Object.prototype.hasOwnProperty.call(r,"default")?r.default:r}var Sc={exports:{}},Ya={};/**
 * @license React
 * react-jsx-runtime.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var zm;function f1(){if(zm)return Ya;zm=1;var r=Symbol.for("react.transitional.element"),a=Symbol.for("react.fragment");function i(o,s,f){var d=null;if(f!==void 0&&(d=""+f),s.key!==void 0&&(d=""+s.key),"key"in s){f={};for(var h in s)h!=="key"&&(f[h]=s[h])}else f=s;return s=f.ref,{$$typeof:r,type:o,key:d,ref:s!==void 0?s:null,props:f}}return Ya.Fragment=a,Ya.jsx=i,Ya.jsxs=i,Ya}var _m;function d1(){return _m||(_m=1,Sc.exports=f1()),Sc.exports}var G=d1(),Ec={exports:{}},xe={};/**
 * @license React
 * react.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Dm;function h1(){if(Dm)return xe;Dm=1;var r=Symbol.for("react.transitional.element"),a=Symbol.for("react.portal"),i=Symbol.for("react.fragment"),o=Symbol.for("react.strict_mode"),s=Symbol.for("react.profiler"),f=Symbol.for("react.consumer"),d=Symbol.for("react.context"),h=Symbol.for("react.forward_ref"),g=Symbol.for("react.suspense"),m=Symbol.for("react.memo"),y=Symbol.for("react.lazy"),v=Symbol.iterator;function w(E){return E===null||typeof E!="object"?null:(E=v&&E[v]||E["@@iterator"],typeof E=="function"?E:null)}var S={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},x=Object.assign,R={};function k(E,_,P){this.props=E,this.context=_,this.refs=R,this.updater=P||S}k.prototype.isReactComponent={},k.prototype.setState=function(E,_){if(typeof E!="object"&&typeof E!="function"&&E!=null)throw Error("takes an object of state variables to update or a function which returns an object of state variables.");this.updater.enqueueSetState(this,E,_,"setState")},k.prototype.forceUpdate=function(E){this.updater.enqueueForceUpdate(this,E,"forceUpdate")};function O(){}O.prototype=k.prototype;function M(E,_,P){this.props=E,this.context=_,this.refs=R,this.updater=P||S}var H=M.prototype=new O;H.constructor=M,x(H,k.prototype),H.isPureReactComponent=!0;var $=Array.isArray,V={H:null,A:null,T:null,S:null,V:null},W=Object.prototype.hasOwnProperty;function ee(E,_,P,F,oe,be){return P=be.ref,{$$typeof:r,type:E,key:_,ref:P!==void 0?P:null,props:be}}function fe(E,_){return ee(E.type,_,void 0,void 0,void 0,E.props)}function ne(E){return typeof E=="object"&&E!==null&&E.$$typeof===r}function se(E){var _={"=":"=0",":":"=2"};return"$"+E.replace(/[=:]/g,function(P){return _[P]})}var me=/\/+/g;function te(E,_){return typeof E=="object"&&E!==null&&E.key!=null?se(""+E.key):_.toString(36)}function ue(){}function X(E){switch(E.status){case"fulfilled":return E.value;case"rejected":throw E.reason;default:switch(typeof E.status=="string"?E.then(ue,ue):(E.status="pending",E.then(function(_){E.status==="pending"&&(E.status="fulfilled",E.value=_)},function(_){E.status==="pending"&&(E.status="rejected",E.reason=_)})),E.status){case"fulfilled":return E.value;case"rejected":throw E.reason}}throw E}function I(E,_,P,F,oe){var be=typeof E;(be==="undefined"||be==="boolean")&&(E=null);var ae=!1;if(E===null)ae=!0;else switch(be){case"bigint":case"string":case"number":ae=!0;break;case"object":switch(E.$$typeof){case r:case a:ae=!0;break;case y:return ae=E._init,I(ae(E._payload),_,P,F,oe)}}if(ae)return oe=oe(E),ae=F===""?"."+te(E,0):F,$(oe)?(P="",ae!=null&&(P=ae.replace(me,"$&/")+"/"),I(oe,_,P,"",function(Ce){return Ce})):oe!=null&&(ne(oe)&&(oe=fe(oe,P+(oe.key==null||E&&E.key===oe.key?"":(""+oe.key).replace(me,"$&/")+"/")+ae)),_.push(oe)),1;ae=0;var ke=F===""?".":F+":";if($(E))for(var de=0;de<E.length;de++)F=E[de],be=ke+te(F,de),ae+=I(F,_,P,be,oe);else if(de=w(E),typeof de=="function")for(E=de.call(E),de=0;!(F=E.next()).done;)F=F.value,be=ke+te(F,de++),ae+=I(F,_,P,be,oe);else if(be==="object"){if(typeof E.then=="function")return I(X(E),_,P,F,oe);throw _=String(E),Error("Objects are not valid as a React child (found: "+(_==="[object Object]"?"object with keys {"+Object.keys(E).join(", ")+"}":_)+"). If you meant to render a collection of children, use an array instead.")}return ae}function N(E,_,P){if(E==null)return E;var F=[],oe=0;return I(E,F,"","",function(be){return _.call(P,be,oe++)}),F}function q(E){if(E._status===-1){var _=E._result;_=_(),_.then(function(P){(E._status===0||E._status===-1)&&(E._status=1,E._result=P)},function(P){(E._status===0||E._status===-1)&&(E._status=2,E._result=P)}),E._status===-1&&(E._status=0,E._result=_)}if(E._status===1)return E._result.default;throw E._result}var L=typeof reportError=="function"?reportError:function(E){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var _=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof E=="object"&&E!==null&&typeof E.message=="string"?String(E.message):String(E),error:E});if(!window.dispatchEvent(_))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",E);return}console.error(E)};function K(){}return xe.Children={map:N,forEach:function(E,_,P){N(E,function(){_.apply(this,arguments)},P)},count:function(E){var _=0;return N(E,function(){_++}),_},toArray:function(E){return N(E,function(_){return _})||[]},only:function(E){if(!ne(E))throw Error("React.Children.only expected to receive a single React element child.");return E}},xe.Component=k,xe.Fragment=i,xe.Profiler=s,xe.PureComponent=M,xe.StrictMode=o,xe.Suspense=g,xe.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=V,xe.__COMPILER_RUNTIME={__proto__:null,c:function(E){return V.H.useMemoCache(E)}},xe.cache=function(E){return function(){return E.apply(null,arguments)}},xe.cloneElement=function(E,_,P){if(E==null)throw Error("The argument must be a React element, but you passed "+E+".");var F=x({},E.props),oe=E.key,be=void 0;if(_!=null)for(ae in _.ref!==void 0&&(be=void 0),_.key!==void 0&&(oe=""+_.key),_)!W.call(_,ae)||ae==="key"||ae==="__self"||ae==="__source"||ae==="ref"&&_.ref===void 0||(F[ae]=_[ae]);var ae=arguments.length-2;if(ae===1)F.children=P;else if(1<ae){for(var ke=Array(ae),de=0;de<ae;de++)ke[de]=arguments[de+2];F.children=ke}return ee(E.type,oe,void 0,void 0,be,F)},xe.createContext=function(E){return E={$$typeof:d,_currentValue:E,_currentValue2:E,_threadCount:0,Provider:null,Consumer:null},E.Provider=E,E.Consumer={$$typeof:f,_context:E},E},xe.createElement=function(E,_,P){var F,oe={},be=null;if(_!=null)for(F in _.key!==void 0&&(be=""+_.key),_)W.call(_,F)&&F!=="key"&&F!=="__self"&&F!=="__source"&&(oe[F]=_[F]);var ae=arguments.length-2;if(ae===1)oe.children=P;else if(1<ae){for(var ke=Array(ae),de=0;de<ae;de++)ke[de]=arguments[de+2];oe.children=ke}if(E&&E.defaultProps)for(F in ae=E.defaultProps,ae)oe[F]===void 0&&(oe[F]=ae[F]);return ee(E,be,void 0,void 0,null,oe)},xe.createRef=function(){return{current:null}},xe.forwardRef=function(E){return{$$typeof:h,render:E}},xe.isValidElement=ne,xe.lazy=function(E){return{$$typeof:y,_payload:{_status:-1,_result:E},_init:q}},xe.memo=function(E,_){return{$$typeof:m,type:E,compare:_===void 0?null:_}},xe.startTransition=function(E){var _=V.T,P={};V.T=P;try{var F=E(),oe=V.S;oe!==null&&oe(P,F),typeof F=="object"&&F!==null&&typeof F.then=="function"&&F.then(K,L)}catch(be){L(be)}finally{V.T=_}},xe.unstable_useCacheRefresh=function(){return V.H.useCacheRefresh()},xe.use=function(E){return V.H.use(E)},xe.useActionState=function(E,_,P){return V.H.useActionState(E,_,P)},xe.useCallback=function(E,_){return V.H.useCallback(E,_)},xe.useContext=function(E){return V.H.useContext(E)},xe.useDebugValue=function(){},xe.useDeferredValue=function(E,_){return V.H.useDeferredValue(E,_)},xe.useEffect=function(E,_,P){var F=V.H;if(typeof P=="function")throw Error("useEffect CRUD overload is not enabled in this build of React.");return F.useEffect(E,_)},xe.useId=function(){return V.H.useId()},xe.useImperativeHandle=function(E,_,P){return V.H.useImperativeHandle(E,_,P)},xe.useInsertionEffect=function(E,_){return V.H.useInsertionEffect(E,_)},xe.useLayoutEffect=function(E,_){return V.H.useLayoutEffect(E,_)},xe.useMemo=function(E,_){return V.H.useMemo(E,_)},xe.useOptimistic=function(E,_){return V.H.useOptimistic(E,_)},xe.useReducer=function(E,_,P){return V.H.useReducer(E,_,P)},xe.useRef=function(E){return V.H.useRef(E)},xe.useState=function(E){return V.H.useState(E)},xe.useSyncExternalStore=function(E,_,P){return V.H.useSyncExternalStore(E,_,P)},xe.useTransition=function(){return V.H.useTransition()},xe.version="19.1.0",xe}var Um;function cf(){return Um||(Um=1,Ec.exports=h1()),Ec.exports}var T=cf();const ir=c1(T),m1=s1({__proto__:null,default:ir},[T]);var Rc={exports:{}},Va={},Tc={exports:{}},Ac={};/**
 * @license React
 * scheduler.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var jm;function g1(){return jm||(jm=1,function(r){function a(N,q){var L=N.length;N.push(q);e:for(;0<L;){var K=L-1>>>1,E=N[K];if(0<s(E,q))N[K]=q,N[L]=E,L=K;else break e}}function i(N){return N.length===0?null:N[0]}function o(N){if(N.length===0)return null;var q=N[0],L=N.pop();if(L!==q){N[0]=L;e:for(var K=0,E=N.length,_=E>>>1;K<_;){var P=2*(K+1)-1,F=N[P],oe=P+1,be=N[oe];if(0>s(F,L))oe<E&&0>s(be,F)?(N[K]=be,N[oe]=L,K=oe):(N[K]=F,N[P]=L,K=P);else if(oe<E&&0>s(be,L))N[K]=be,N[oe]=L,K=oe;else break e}}return q}function s(N,q){var L=N.sortIndex-q.sortIndex;return L!==0?L:N.id-q.id}if(r.unstable_now=void 0,typeof performance=="object"&&typeof performance.now=="function"){var f=performance;r.unstable_now=function(){return f.now()}}else{var d=Date,h=d.now();r.unstable_now=function(){return d.now()-h}}var g=[],m=[],y=1,v=null,w=3,S=!1,x=!1,R=!1,k=!1,O=typeof setTimeout=="function"?setTimeout:null,M=typeof clearTimeout=="function"?clearTimeout:null,H=typeof setImmediate<"u"?setImmediate:null;function $(N){for(var q=i(m);q!==null;){if(q.callback===null)o(m);else if(q.startTime<=N)o(m),q.sortIndex=q.expirationTime,a(g,q);else break;q=i(m)}}function V(N){if(R=!1,$(N),!x)if(i(g)!==null)x=!0,W||(W=!0,te());else{var q=i(m);q!==null&&I(V,q.startTime-N)}}var W=!1,ee=-1,fe=5,ne=-1;function se(){return k?!0:!(r.unstable_now()-ne<fe)}function me(){if(k=!1,W){var N=r.unstable_now();ne=N;var q=!0;try{e:{x=!1,R&&(R=!1,M(ee),ee=-1),S=!0;var L=w;try{t:{for($(N),v=i(g);v!==null&&!(v.expirationTime>N&&se());){var K=v.callback;if(typeof K=="function"){v.callback=null,w=v.priorityLevel;var E=K(v.expirationTime<=N);if(N=r.unstable_now(),typeof E=="function"){v.callback=E,$(N),q=!0;break t}v===i(g)&&o(g),$(N)}else o(g);v=i(g)}if(v!==null)q=!0;else{var _=i(m);_!==null&&I(V,_.startTime-N),q=!1}}break e}finally{v=null,w=L,S=!1}q=void 0}}finally{q?te():W=!1}}}var te;if(typeof H=="function")te=function(){H(me)};else if(typeof MessageChannel<"u"){var ue=new MessageChannel,X=ue.port2;ue.port1.onmessage=me,te=function(){X.postMessage(null)}}else te=function(){O(me,0)};function I(N,q){ee=O(function(){N(r.unstable_now())},q)}r.unstable_IdlePriority=5,r.unstable_ImmediatePriority=1,r.unstable_LowPriority=4,r.unstable_NormalPriority=3,r.unstable_Profiling=null,r.unstable_UserBlockingPriority=2,r.unstable_cancelCallback=function(N){N.callback=null},r.unstable_forceFrameRate=function(N){0>N||125<N?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):fe=0<N?Math.floor(1e3/N):5},r.unstable_getCurrentPriorityLevel=function(){return w},r.unstable_next=function(N){switch(w){case 1:case 2:case 3:var q=3;break;default:q=w}var L=w;w=q;try{return N()}finally{w=L}},r.unstable_requestPaint=function(){k=!0},r.unstable_runWithPriority=function(N,q){switch(N){case 1:case 2:case 3:case 4:case 5:break;default:N=3}var L=w;w=N;try{return q()}finally{w=L}},r.unstable_scheduleCallback=function(N,q,L){var K=r.unstable_now();switch(typeof L=="object"&&L!==null?(L=L.delay,L=typeof L=="number"&&0<L?K+L:K):L=K,N){case 1:var E=-1;break;case 2:E=250;break;case 5:E=1073741823;break;case 4:E=1e4;break;default:E=5e3}return E=L+E,N={id:y++,callback:q,priorityLevel:N,startTime:L,expirationTime:E,sortIndex:-1},L>K?(N.sortIndex=L,a(m,N),i(g)===null&&N===i(m)&&(R?(M(ee),ee=-1):R=!0,I(V,L-K))):(N.sortIndex=E,a(g,N),x||S||(x=!0,W||(W=!0,te()))),N},r.unstable_shouldYield=se,r.unstable_wrapCallback=function(N){var q=w;return function(){var L=w;w=q;try{return N.apply(this,arguments)}finally{w=L}}}}(Ac)),Ac}var Lm;function p1(){return Lm||(Lm=1,Tc.exports=g1()),Tc.exports}var kc={exports:{}},ht={};/**
 * @license React
 * react-dom.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Bm;function y1(){if(Bm)return ht;Bm=1;var r=cf();function a(g){var m="https://react.dev/errors/"+g;if(1<arguments.length){m+="?args[]="+encodeURIComponent(arguments[1]);for(var y=2;y<arguments.length;y++)m+="&args[]="+encodeURIComponent(arguments[y])}return"Minified React error #"+g+"; visit "+m+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function i(){}var o={d:{f:i,r:function(){throw Error(a(522))},D:i,C:i,L:i,m:i,X:i,S:i,M:i},p:0,findDOMNode:null},s=Symbol.for("react.portal");function f(g,m,y){var v=3<arguments.length&&arguments[3]!==void 0?arguments[3]:null;return{$$typeof:s,key:v==null?null:""+v,children:g,containerInfo:m,implementation:y}}var d=r.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE;function h(g,m){if(g==="font")return"";if(typeof m=="string")return m==="use-credentials"?m:""}return ht.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=o,ht.createPortal=function(g,m){var y=2<arguments.length&&arguments[2]!==void 0?arguments[2]:null;if(!m||m.nodeType!==1&&m.nodeType!==9&&m.nodeType!==11)throw Error(a(299));return f(g,m,null,y)},ht.flushSync=function(g){var m=d.T,y=o.p;try{if(d.T=null,o.p=2,g)return g()}finally{d.T=m,o.p=y,o.d.f()}},ht.preconnect=function(g,m){typeof g=="string"&&(m?(m=m.crossOrigin,m=typeof m=="string"?m==="use-credentials"?m:"":void 0):m=null,o.d.C(g,m))},ht.prefetchDNS=function(g){typeof g=="string"&&o.d.D(g)},ht.preinit=function(g,m){if(typeof g=="string"&&m&&typeof m.as=="string"){var y=m.as,v=h(y,m.crossOrigin),w=typeof m.integrity=="string"?m.integrity:void 0,S=typeof m.fetchPriority=="string"?m.fetchPriority:void 0;y==="style"?o.d.S(g,typeof m.precedence=="string"?m.precedence:void 0,{crossOrigin:v,integrity:w,fetchPriority:S}):y==="script"&&o.d.X(g,{crossOrigin:v,integrity:w,fetchPriority:S,nonce:typeof m.nonce=="string"?m.nonce:void 0})}},ht.preinitModule=function(g,m){if(typeof g=="string")if(typeof m=="object"&&m!==null){if(m.as==null||m.as==="script"){var y=h(m.as,m.crossOrigin);o.d.M(g,{crossOrigin:y,integrity:typeof m.integrity=="string"?m.integrity:void 0,nonce:typeof m.nonce=="string"?m.nonce:void 0})}}else m==null&&o.d.M(g)},ht.preload=function(g,m){if(typeof g=="string"&&typeof m=="object"&&m!==null&&typeof m.as=="string"){var y=m.as,v=h(y,m.crossOrigin);o.d.L(g,y,{crossOrigin:v,integrity:typeof m.integrity=="string"?m.integrity:void 0,nonce:typeof m.nonce=="string"?m.nonce:void 0,type:typeof m.type=="string"?m.type:void 0,fetchPriority:typeof m.fetchPriority=="string"?m.fetchPriority:void 0,referrerPolicy:typeof m.referrerPolicy=="string"?m.referrerPolicy:void 0,imageSrcSet:typeof m.imageSrcSet=="string"?m.imageSrcSet:void 0,imageSizes:typeof m.imageSizes=="string"?m.imageSizes:void 0,media:typeof m.media=="string"?m.media:void 0})}},ht.preloadModule=function(g,m){if(typeof g=="string")if(m){var y=h(m.as,m.crossOrigin);o.d.m(g,{as:typeof m.as=="string"&&m.as!=="script"?m.as:void 0,crossOrigin:y,integrity:typeof m.integrity=="string"?m.integrity:void 0})}else o.d.m(g)},ht.requestFormReset=function(g){o.d.r(g)},ht.unstable_batchedUpdates=function(g,m){return g(m)},ht.useFormState=function(g,m,y){return d.H.useFormState(g,m,y)},ht.useFormStatus=function(){return d.H.useHostTransitionStatus()},ht.version="19.1.0",ht}var Hm;function Xg(){if(Hm)return kc.exports;Hm=1;function r(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(r)}catch(a){console.error(a)}}return r(),kc.exports=y1(),kc.exports}/**
 * @license React
 * react-dom-client.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var qm;function b1(){if(qm)return Va;qm=1;var r=p1(),a=cf(),i=Xg();function o(e){var t="https://react.dev/errors/"+e;if(1<arguments.length){t+="?args[]="+encodeURIComponent(arguments[1]);for(var n=2;n<arguments.length;n++)t+="&args[]="+encodeURIComponent(arguments[n])}return"Minified React error #"+e+"; visit "+t+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function s(e){return!(!e||e.nodeType!==1&&e.nodeType!==9&&e.nodeType!==11)}function f(e){var t=e,n=e;if(e.alternate)for(;t.return;)t=t.return;else{e=t;do t=e,(t.flags&4098)!==0&&(n=t.return),e=t.return;while(e)}return t.tag===3?n:null}function d(e){if(e.tag===13){var t=e.memoizedState;if(t===null&&(e=e.alternate,e!==null&&(t=e.memoizedState)),t!==null)return t.dehydrated}return null}function h(e){if(f(e)!==e)throw Error(o(188))}function g(e){var t=e.alternate;if(!t){if(t=f(e),t===null)throw Error(o(188));return t!==e?null:e}for(var n=e,l=t;;){var u=n.return;if(u===null)break;var c=u.alternate;if(c===null){if(l=u.return,l!==null){n=l;continue}break}if(u.child===c.child){for(c=u.child;c;){if(c===n)return h(u),e;if(c===l)return h(u),t;c=c.sibling}throw Error(o(188))}if(n.return!==l.return)n=u,l=c;else{for(var p=!1,b=u.child;b;){if(b===n){p=!0,n=u,l=c;break}if(b===l){p=!0,l=u,n=c;break}b=b.sibling}if(!p){for(b=c.child;b;){if(b===n){p=!0,n=c,l=u;break}if(b===l){p=!0,l=c,n=u;break}b=b.sibling}if(!p)throw Error(o(189))}}if(n.alternate!==l)throw Error(o(190))}if(n.tag!==3)throw Error(o(188));return n.stateNode.current===n?e:t}function m(e){var t=e.tag;if(t===5||t===26||t===27||t===6)return e;for(e=e.child;e!==null;){if(t=m(e),t!==null)return t;e=e.sibling}return null}var y=Object.assign,v=Symbol.for("react.element"),w=Symbol.for("react.transitional.element"),S=Symbol.for("react.portal"),x=Symbol.for("react.fragment"),R=Symbol.for("react.strict_mode"),k=Symbol.for("react.profiler"),O=Symbol.for("react.provider"),M=Symbol.for("react.consumer"),H=Symbol.for("react.context"),$=Symbol.for("react.forward_ref"),V=Symbol.for("react.suspense"),W=Symbol.for("react.suspense_list"),ee=Symbol.for("react.memo"),fe=Symbol.for("react.lazy"),ne=Symbol.for("react.activity"),se=Symbol.for("react.memo_cache_sentinel"),me=Symbol.iterator;function te(e){return e===null||typeof e!="object"?null:(e=me&&e[me]||e["@@iterator"],typeof e=="function"?e:null)}var ue=Symbol.for("react.client.reference");function X(e){if(e==null)return null;if(typeof e=="function")return e.$$typeof===ue?null:e.displayName||e.name||null;if(typeof e=="string")return e;switch(e){case x:return"Fragment";case k:return"Profiler";case R:return"StrictMode";case V:return"Suspense";case W:return"SuspenseList";case ne:return"Activity"}if(typeof e=="object")switch(e.$$typeof){case S:return"Portal";case H:return(e.displayName||"Context")+".Provider";case M:return(e._context.displayName||"Context")+".Consumer";case $:var t=e.render;return e=e.displayName,e||(e=t.displayName||t.name||"",e=e!==""?"ForwardRef("+e+")":"ForwardRef"),e;case ee:return t=e.displayName||null,t!==null?t:X(e.type)||"Memo";case fe:t=e._payload,e=e._init;try{return X(e(t))}catch{}}return null}var I=Array.isArray,N=a.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,q=i.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,L={pending:!1,data:null,method:null,action:null},K=[],E=-1;function _(e){return{current:e}}function P(e){0>E||(e.current=K[E],K[E]=null,E--)}function F(e,t){E++,K[E]=e.current,e.current=t}var oe=_(null),be=_(null),ae=_(null),ke=_(null);function de(e,t){switch(F(ae,t),F(be,e),F(oe,null),t.nodeType){case 9:case 11:e=(e=t.documentElement)&&(e=e.namespaceURI)?om(e):0;break;default:if(e=t.tagName,t=t.namespaceURI)t=om(t),e=im(t,e);else switch(e){case"svg":e=1;break;case"math":e=2;break;default:e=0}}P(oe),F(oe,e)}function Ce(){P(oe),P(be),P(ae)}function At(e){e.memoizedState!==null&&F(ke,e);var t=oe.current,n=im(t,e.type);t!==n&&(F(be,e),F(oe,n))}function pt(e){be.current===e&&(P(oe),P(be)),ke.current===e&&(P(ke),La._currentValue=L)}var jt=Object.prototype.hasOwnProperty,Vl=r.unstable_scheduleCallback,Xl=r.unstable_cancelCallback,cu=r.unstable_shouldYield,Yy=r.unstable_requestPaint,en=r.unstable_now,Vy=r.unstable_getCurrentPriorityLevel,qf=r.unstable_ImmediatePriority,Gf=r.unstable_UserBlockingPriority,oo=r.unstable_NormalPriority,Xy=r.unstable_LowPriority,Yf=r.unstable_IdlePriority,Qy=r.log,Zy=r.unstable_setDisableYieldValue,Ql=null,kt=null;function _n(e){if(typeof Qy=="function"&&Zy(e),kt&&typeof kt.setStrictMode=="function")try{kt.setStrictMode(Ql,e)}catch{}}var Ot=Math.clz32?Math.clz32:Jy,Ky=Math.log,$y=Math.LN2;function Jy(e){return e>>>=0,e===0?32:31-(Ky(e)/$y|0)|0}var io=256,uo=4194304;function dr(e){var t=e&42;if(t!==0)return t;switch(e&-e){case 1:return 1;case 2:return 2;case 4:return 4;case 8:return 8;case 16:return 16;case 32:return 32;case 64:return 64;case 128:return 128;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return e&4194048;case 4194304:case 8388608:case 16777216:case 33554432:return e&62914560;case 67108864:return 67108864;case 134217728:return 134217728;case 268435456:return 268435456;case 536870912:return 536870912;case 1073741824:return 0;default:return e}}function so(e,t,n){var l=e.pendingLanes;if(l===0)return 0;var u=0,c=e.suspendedLanes,p=e.pingedLanes;e=e.warmLanes;var b=l&134217727;return b!==0?(l=b&~c,l!==0?u=dr(l):(p&=b,p!==0?u=dr(p):n||(n=b&~e,n!==0&&(u=dr(n))))):(b=l&~c,b!==0?u=dr(b):p!==0?u=dr(p):n||(n=l&~e,n!==0&&(u=dr(n)))),u===0?0:t!==0&&t!==u&&(t&c)===0&&(c=u&-u,n=t&-t,c>=n||c===32&&(n&4194048)!==0)?t:u}function Zl(e,t){return(e.pendingLanes&~(e.suspendedLanes&~e.pingedLanes)&t)===0}function Py(e,t){switch(e){case 1:case 2:case 4:case 8:case 64:return t+250;case 16:case 32:case 128:case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return t+5e3;case 4194304:case 8388608:case 16777216:case 33554432:return-1;case 67108864:case 134217728:case 268435456:case 536870912:case 1073741824:return-1;default:return-1}}function Vf(){var e=io;return io<<=1,(io&4194048)===0&&(io=256),e}function Xf(){var e=uo;return uo<<=1,(uo&62914560)===0&&(uo=4194304),e}function fu(e){for(var t=[],n=0;31>n;n++)t.push(e);return t}function Kl(e,t){e.pendingLanes|=t,t!==268435456&&(e.suspendedLanes=0,e.pingedLanes=0,e.warmLanes=0)}function Fy(e,t,n,l,u,c){var p=e.pendingLanes;e.pendingLanes=n,e.suspendedLanes=0,e.pingedLanes=0,e.warmLanes=0,e.expiredLanes&=n,e.entangledLanes&=n,e.errorRecoveryDisabledLanes&=n,e.shellSuspendCounter=0;var b=e.entanglements,A=e.expirationTimes,U=e.hiddenUpdates;for(n=p&~n;0<n;){var Q=31-Ot(n),J=1<<Q;b[Q]=0,A[Q]=-1;var j=U[Q];if(j!==null)for(U[Q]=null,Q=0;Q<j.length;Q++){var B=j[Q];B!==null&&(B.lane&=-536870913)}n&=~J}l!==0&&Qf(e,l,0),c!==0&&u===0&&e.tag!==0&&(e.suspendedLanes|=c&~(p&~t))}function Qf(e,t,n){e.pendingLanes|=t,e.suspendedLanes&=~t;var l=31-Ot(t);e.entangledLanes|=t,e.entanglements[l]=e.entanglements[l]|1073741824|n&4194090}function Zf(e,t){var n=e.entangledLanes|=t;for(e=e.entanglements;n;){var l=31-Ot(n),u=1<<l;u&t|e[l]&t&&(e[l]|=t),n&=~u}}function du(e){switch(e){case 2:e=1;break;case 8:e=4;break;case 32:e=16;break;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:case 4194304:case 8388608:case 16777216:case 33554432:e=128;break;case 268435456:e=134217728;break;default:e=0}return e}function hu(e){return e&=-e,2<e?8<e?(e&134217727)!==0?32:268435456:8:2}function Kf(){var e=q.p;return e!==0?e:(e=window.event,e===void 0?32:Am(e.type))}function Wy(e,t){var n=q.p;try{return q.p=e,t()}finally{q.p=n}}var Dn=Math.random().toString(36).slice(2),ft="__reactFiber$"+Dn,yt="__reactProps$"+Dn,Hr="__reactContainer$"+Dn,mu="__reactEvents$"+Dn,Iy="__reactListeners$"+Dn,eb="__reactHandles$"+Dn,$f="__reactResources$"+Dn,$l="__reactMarker$"+Dn;function gu(e){delete e[ft],delete e[yt],delete e[mu],delete e[Iy],delete e[eb]}function qr(e){var t=e[ft];if(t)return t;for(var n=e.parentNode;n;){if(t=n[Hr]||n[ft]){if(n=t.alternate,t.child!==null||n!==null&&n.child!==null)for(e=fm(e);e!==null;){if(n=e[ft])return n;e=fm(e)}return t}e=n,n=e.parentNode}return null}function Gr(e){if(e=e[ft]||e[Hr]){var t=e.tag;if(t===5||t===6||t===13||t===26||t===27||t===3)return e}return null}function Jl(e){var t=e.tag;if(t===5||t===26||t===27||t===6)return e.stateNode;throw Error(o(33))}function Yr(e){var t=e[$f];return t||(t=e[$f]={hoistableStyles:new Map,hoistableScripts:new Map}),t}function et(e){e[$l]=!0}var Jf=new Set,Pf={};function hr(e,t){Vr(e,t),Vr(e+"Capture",t)}function Vr(e,t){for(Pf[e]=t,e=0;e<t.length;e++)Jf.add(t[e])}var tb=RegExp("^[:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD][:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*$"),Ff={},Wf={};function nb(e){return jt.call(Wf,e)?!0:jt.call(Ff,e)?!1:tb.test(e)?Wf[e]=!0:(Ff[e]=!0,!1)}function co(e,t,n){if(nb(t))if(n===null)e.removeAttribute(t);else{switch(typeof n){case"undefined":case"function":case"symbol":e.removeAttribute(t);return;case"boolean":var l=t.toLowerCase().slice(0,5);if(l!=="data-"&&l!=="aria-"){e.removeAttribute(t);return}}e.setAttribute(t,""+n)}}function fo(e,t,n){if(n===null)e.removeAttribute(t);else{switch(typeof n){case"undefined":case"function":case"symbol":case"boolean":e.removeAttribute(t);return}e.setAttribute(t,""+n)}}function fn(e,t,n,l){if(l===null)e.removeAttribute(n);else{switch(typeof l){case"undefined":case"function":case"symbol":case"boolean":e.removeAttribute(n);return}e.setAttributeNS(t,n,""+l)}}var pu,If;function Xr(e){if(pu===void 0)try{throw Error()}catch(n){var t=n.stack.trim().match(/\n( *(at )?)/);pu=t&&t[1]||"",If=-1<n.stack.indexOf(`
    at`)?" (<anonymous>)":-1<n.stack.indexOf("@")?"@unknown:0:0":""}return`
`+pu+e+If}var yu=!1;function bu(e,t){if(!e||yu)return"";yu=!0;var n=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{var l={DetermineComponentFrameRoot:function(){try{if(t){var J=function(){throw Error()};if(Object.defineProperty(J.prototype,"props",{set:function(){throw Error()}}),typeof Reflect=="object"&&Reflect.construct){try{Reflect.construct(J,[])}catch(B){var j=B}Reflect.construct(e,[],J)}else{try{J.call()}catch(B){j=B}e.call(J.prototype)}}else{try{throw Error()}catch(B){j=B}(J=e())&&typeof J.catch=="function"&&J.catch(function(){})}}catch(B){if(B&&j&&typeof B.stack=="string")return[B.stack,j.stack]}return[null,null]}};l.DetermineComponentFrameRoot.displayName="DetermineComponentFrameRoot";var u=Object.getOwnPropertyDescriptor(l.DetermineComponentFrameRoot,"name");u&&u.configurable&&Object.defineProperty(l.DetermineComponentFrameRoot,"name",{value:"DetermineComponentFrameRoot"});var c=l.DetermineComponentFrameRoot(),p=c[0],b=c[1];if(p&&b){var A=p.split(`
`),U=b.split(`
`);for(u=l=0;l<A.length&&!A[l].includes("DetermineComponentFrameRoot");)l++;for(;u<U.length&&!U[u].includes("DetermineComponentFrameRoot");)u++;if(l===A.length||u===U.length)for(l=A.length-1,u=U.length-1;1<=l&&0<=u&&A[l]!==U[u];)u--;for(;1<=l&&0<=u;l--,u--)if(A[l]!==U[u]){if(l!==1||u!==1)do if(l--,u--,0>u||A[l]!==U[u]){var Q=`
`+A[l].replace(" at new "," at ");return e.displayName&&Q.includes("<anonymous>")&&(Q=Q.replace("<anonymous>",e.displayName)),Q}while(1<=l&&0<=u);break}}}finally{yu=!1,Error.prepareStackTrace=n}return(n=e?e.displayName||e.name:"")?Xr(n):""}function rb(e){switch(e.tag){case 26:case 27:case 5:return Xr(e.type);case 16:return Xr("Lazy");case 13:return Xr("Suspense");case 19:return Xr("SuspenseList");case 0:case 15:return bu(e.type,!1);case 11:return bu(e.type.render,!1);case 1:return bu(e.type,!0);case 31:return Xr("Activity");default:return""}}function ed(e){try{var t="";do t+=rb(e),e=e.return;while(e);return t}catch(n){return`
Error generating stack: `+n.message+`
`+n.stack}}function Lt(e){switch(typeof e){case"bigint":case"boolean":case"number":case"string":case"undefined":return e;case"object":return e;default:return""}}function td(e){var t=e.type;return(e=e.nodeName)&&e.toLowerCase()==="input"&&(t==="checkbox"||t==="radio")}function lb(e){var t=td(e)?"checked":"value",n=Object.getOwnPropertyDescriptor(e.constructor.prototype,t),l=""+e[t];if(!e.hasOwnProperty(t)&&typeof n<"u"&&typeof n.get=="function"&&typeof n.set=="function"){var u=n.get,c=n.set;return Object.defineProperty(e,t,{configurable:!0,get:function(){return u.call(this)},set:function(p){l=""+p,c.call(this,p)}}),Object.defineProperty(e,t,{enumerable:n.enumerable}),{getValue:function(){return l},setValue:function(p){l=""+p},stopTracking:function(){e._valueTracker=null,delete e[t]}}}}function ho(e){e._valueTracker||(e._valueTracker=lb(e))}function nd(e){if(!e)return!1;var t=e._valueTracker;if(!t)return!0;var n=t.getValue(),l="";return e&&(l=td(e)?e.checked?"true":"false":e.value),e=l,e!==n?(t.setValue(e),!0):!1}function mo(e){if(e=e||(typeof document<"u"?document:void 0),typeof e>"u")return null;try{return e.activeElement||e.body}catch{return e.body}}var ab=/[\n"\\]/g;function Bt(e){return e.replace(ab,function(t){return"\\"+t.charCodeAt(0).toString(16)+" "})}function vu(e,t,n,l,u,c,p,b){e.name="",p!=null&&typeof p!="function"&&typeof p!="symbol"&&typeof p!="boolean"?e.type=p:e.removeAttribute("type"),t!=null?p==="number"?(t===0&&e.value===""||e.value!=t)&&(e.value=""+Lt(t)):e.value!==""+Lt(t)&&(e.value=""+Lt(t)):p!=="submit"&&p!=="reset"||e.removeAttribute("value"),t!=null?xu(e,p,Lt(t)):n!=null?xu(e,p,Lt(n)):l!=null&&e.removeAttribute("value"),u==null&&c!=null&&(e.defaultChecked=!!c),u!=null&&(e.checked=u&&typeof u!="function"&&typeof u!="symbol"),b!=null&&typeof b!="function"&&typeof b!="symbol"&&typeof b!="boolean"?e.name=""+Lt(b):e.removeAttribute("name")}function rd(e,t,n,l,u,c,p,b){if(c!=null&&typeof c!="function"&&typeof c!="symbol"&&typeof c!="boolean"&&(e.type=c),t!=null||n!=null){if(!(c!=="submit"&&c!=="reset"||t!=null))return;n=n!=null?""+Lt(n):"",t=t!=null?""+Lt(t):n,b||t===e.value||(e.value=t),e.defaultValue=t}l=l??u,l=typeof l!="function"&&typeof l!="symbol"&&!!l,e.checked=b?e.checked:!!l,e.defaultChecked=!!l,p!=null&&typeof p!="function"&&typeof p!="symbol"&&typeof p!="boolean"&&(e.name=p)}function xu(e,t,n){t==="number"&&mo(e.ownerDocument)===e||e.defaultValue===""+n||(e.defaultValue=""+n)}function Qr(e,t,n,l){if(e=e.options,t){t={};for(var u=0;u<n.length;u++)t["$"+n[u]]=!0;for(n=0;n<e.length;n++)u=t.hasOwnProperty("$"+e[n].value),e[n].selected!==u&&(e[n].selected=u),u&&l&&(e[n].defaultSelected=!0)}else{for(n=""+Lt(n),t=null,u=0;u<e.length;u++){if(e[u].value===n){e[u].selected=!0,l&&(e[u].defaultSelected=!0);return}t!==null||e[u].disabled||(t=e[u])}t!==null&&(t.selected=!0)}}function ld(e,t,n){if(t!=null&&(t=""+Lt(t),t!==e.value&&(e.value=t),n==null)){e.defaultValue!==t&&(e.defaultValue=t);return}e.defaultValue=n!=null?""+Lt(n):""}function ad(e,t,n,l){if(t==null){if(l!=null){if(n!=null)throw Error(o(92));if(I(l)){if(1<l.length)throw Error(o(93));l=l[0]}n=l}n==null&&(n=""),t=n}n=Lt(t),e.defaultValue=n,l=e.textContent,l===n&&l!==""&&l!==null&&(e.value=l)}function Zr(e,t){if(t){var n=e.firstChild;if(n&&n===e.lastChild&&n.nodeType===3){n.nodeValue=t;return}}e.textContent=t}var ob=new Set("animationIterationCount aspectRatio borderImageOutset borderImageSlice borderImageWidth boxFlex boxFlexGroup boxOrdinalGroup columnCount columns flex flexGrow flexPositive flexShrink flexNegative flexOrder gridArea gridRow gridRowEnd gridRowSpan gridRowStart gridColumn gridColumnEnd gridColumnSpan gridColumnStart fontWeight lineClamp lineHeight opacity order orphans scale tabSize widows zIndex zoom fillOpacity floodOpacity stopOpacity strokeDasharray strokeDashoffset strokeMiterlimit strokeOpacity strokeWidth MozAnimationIterationCount MozBoxFlex MozBoxFlexGroup MozLineClamp msAnimationIterationCount msFlex msZoom msFlexGrow msFlexNegative msFlexOrder msFlexPositive msFlexShrink msGridColumn msGridColumnSpan msGridRow msGridRowSpan WebkitAnimationIterationCount WebkitBoxFlex WebKitBoxFlexGroup WebkitBoxOrdinalGroup WebkitColumnCount WebkitColumns WebkitFlex WebkitFlexGrow WebkitFlexPositive WebkitFlexShrink WebkitLineClamp".split(" "));function od(e,t,n){var l=t.indexOf("--")===0;n==null||typeof n=="boolean"||n===""?l?e.setProperty(t,""):t==="float"?e.cssFloat="":e[t]="":l?e.setProperty(t,n):typeof n!="number"||n===0||ob.has(t)?t==="float"?e.cssFloat=n:e[t]=(""+n).trim():e[t]=n+"px"}function id(e,t,n){if(t!=null&&typeof t!="object")throw Error(o(62));if(e=e.style,n!=null){for(var l in n)!n.hasOwnProperty(l)||t!=null&&t.hasOwnProperty(l)||(l.indexOf("--")===0?e.setProperty(l,""):l==="float"?e.cssFloat="":e[l]="");for(var u in t)l=t[u],t.hasOwnProperty(u)&&n[u]!==l&&od(e,u,l)}else for(var c in t)t.hasOwnProperty(c)&&od(e,c,t[c])}function wu(e){if(e.indexOf("-")===-1)return!1;switch(e){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}var ib=new Map([["acceptCharset","accept-charset"],["htmlFor","for"],["httpEquiv","http-equiv"],["crossOrigin","crossorigin"],["accentHeight","accent-height"],["alignmentBaseline","alignment-baseline"],["arabicForm","arabic-form"],["baselineShift","baseline-shift"],["capHeight","cap-height"],["clipPath","clip-path"],["clipRule","clip-rule"],["colorInterpolation","color-interpolation"],["colorInterpolationFilters","color-interpolation-filters"],["colorProfile","color-profile"],["colorRendering","color-rendering"],["dominantBaseline","dominant-baseline"],["enableBackground","enable-background"],["fillOpacity","fill-opacity"],["fillRule","fill-rule"],["floodColor","flood-color"],["floodOpacity","flood-opacity"],["fontFamily","font-family"],["fontSize","font-size"],["fontSizeAdjust","font-size-adjust"],["fontStretch","font-stretch"],["fontStyle","font-style"],["fontVariant","font-variant"],["fontWeight","font-weight"],["glyphName","glyph-name"],["glyphOrientationHorizontal","glyph-orientation-horizontal"],["glyphOrientationVertical","glyph-orientation-vertical"],["horizAdvX","horiz-adv-x"],["horizOriginX","horiz-origin-x"],["imageRendering","image-rendering"],["letterSpacing","letter-spacing"],["lightingColor","lighting-color"],["markerEnd","marker-end"],["markerMid","marker-mid"],["markerStart","marker-start"],["overlinePosition","overline-position"],["overlineThickness","overline-thickness"],["paintOrder","paint-order"],["panose-1","panose-1"],["pointerEvents","pointer-events"],["renderingIntent","rendering-intent"],["shapeRendering","shape-rendering"],["stopColor","stop-color"],["stopOpacity","stop-opacity"],["strikethroughPosition","strikethrough-position"],["strikethroughThickness","strikethrough-thickness"],["strokeDasharray","stroke-dasharray"],["strokeDashoffset","stroke-dashoffset"],["strokeLinecap","stroke-linecap"],["strokeLinejoin","stroke-linejoin"],["strokeMiterlimit","stroke-miterlimit"],["strokeOpacity","stroke-opacity"],["strokeWidth","stroke-width"],["textAnchor","text-anchor"],["textDecoration","text-decoration"],["textRendering","text-rendering"],["transformOrigin","transform-origin"],["underlinePosition","underline-position"],["underlineThickness","underline-thickness"],["unicodeBidi","unicode-bidi"],["unicodeRange","unicode-range"],["unitsPerEm","units-per-em"],["vAlphabetic","v-alphabetic"],["vHanging","v-hanging"],["vIdeographic","v-ideographic"],["vMathematical","v-mathematical"],["vectorEffect","vector-effect"],["vertAdvY","vert-adv-y"],["vertOriginX","vert-origin-x"],["vertOriginY","vert-origin-y"],["wordSpacing","word-spacing"],["writingMode","writing-mode"],["xmlnsXlink","xmlns:xlink"],["xHeight","x-height"]]),ub=/^[\u0000-\u001F ]*j[\r\n\t]*a[\r\n\t]*v[\r\n\t]*a[\r\n\t]*s[\r\n\t]*c[\r\n\t]*r[\r\n\t]*i[\r\n\t]*p[\r\n\t]*t[\r\n\t]*:/i;function go(e){return ub.test(""+e)?"javascript:throw new Error('React has blocked a javascript: URL as a security precaution.')":e}var Su=null;function Eu(e){return e=e.target||e.srcElement||window,e.correspondingUseElement&&(e=e.correspondingUseElement),e.nodeType===3?e.parentNode:e}var Kr=null,$r=null;function ud(e){var t=Gr(e);if(t&&(e=t.stateNode)){var n=e[yt]||null;e:switch(e=t.stateNode,t.type){case"input":if(vu(e,n.value,n.defaultValue,n.defaultValue,n.checked,n.defaultChecked,n.type,n.name),t=n.name,n.type==="radio"&&t!=null){for(n=e;n.parentNode;)n=n.parentNode;for(n=n.querySelectorAll('input[name="'+Bt(""+t)+'"][type="radio"]'),t=0;t<n.length;t++){var l=n[t];if(l!==e&&l.form===e.form){var u=l[yt]||null;if(!u)throw Error(o(90));vu(l,u.value,u.defaultValue,u.defaultValue,u.checked,u.defaultChecked,u.type,u.name)}}for(t=0;t<n.length;t++)l=n[t],l.form===e.form&&nd(l)}break e;case"textarea":ld(e,n.value,n.defaultValue);break e;case"select":t=n.value,t!=null&&Qr(e,!!n.multiple,t,!1)}}}var Ru=!1;function sd(e,t,n){if(Ru)return e(t,n);Ru=!0;try{var l=e(t);return l}finally{if(Ru=!1,(Kr!==null||$r!==null)&&(ei(),Kr&&(t=Kr,e=$r,$r=Kr=null,ud(t),e)))for(t=0;t<e.length;t++)ud(e[t])}}function Pl(e,t){var n=e.stateNode;if(n===null)return null;var l=n[yt]||null;if(l===null)return null;n=l[t];e:switch(t){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(l=!l.disabled)||(e=e.type,l=!(e==="button"||e==="input"||e==="select"||e==="textarea")),e=!l;break e;default:e=!1}if(e)return null;if(n&&typeof n!="function")throw Error(o(231,t,typeof n));return n}var dn=!(typeof window>"u"||typeof window.document>"u"||typeof window.document.createElement>"u"),Tu=!1;if(dn)try{var Fl={};Object.defineProperty(Fl,"passive",{get:function(){Tu=!0}}),window.addEventListener("test",Fl,Fl),window.removeEventListener("test",Fl,Fl)}catch{Tu=!1}var Un=null,Au=null,po=null;function cd(){if(po)return po;var e,t=Au,n=t.length,l,u="value"in Un?Un.value:Un.textContent,c=u.length;for(e=0;e<n&&t[e]===u[e];e++);var p=n-e;for(l=1;l<=p&&t[n-l]===u[c-l];l++);return po=u.slice(e,1<l?1-l:void 0)}function yo(e){var t=e.keyCode;return"charCode"in e?(e=e.charCode,e===0&&t===13&&(e=13)):e=t,e===10&&(e=13),32<=e||e===13?e:0}function bo(){return!0}function fd(){return!1}function bt(e){function t(n,l,u,c,p){this._reactName=n,this._targetInst=u,this.type=l,this.nativeEvent=c,this.target=p,this.currentTarget=null;for(var b in e)e.hasOwnProperty(b)&&(n=e[b],this[b]=n?n(c):c[b]);return this.isDefaultPrevented=(c.defaultPrevented!=null?c.defaultPrevented:c.returnValue===!1)?bo:fd,this.isPropagationStopped=fd,this}return y(t.prototype,{preventDefault:function(){this.defaultPrevented=!0;var n=this.nativeEvent;n&&(n.preventDefault?n.preventDefault():typeof n.returnValue!="unknown"&&(n.returnValue=!1),this.isDefaultPrevented=bo)},stopPropagation:function(){var n=this.nativeEvent;n&&(n.stopPropagation?n.stopPropagation():typeof n.cancelBubble!="unknown"&&(n.cancelBubble=!0),this.isPropagationStopped=bo)},persist:function(){},isPersistent:bo}),t}var mr={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(e){return e.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},vo=bt(mr),Wl=y({},mr,{view:0,detail:0}),sb=bt(Wl),ku,Ou,Il,xo=y({},Wl,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:Mu,button:0,buttons:0,relatedTarget:function(e){return e.relatedTarget===void 0?e.fromElement===e.srcElement?e.toElement:e.fromElement:e.relatedTarget},movementX:function(e){return"movementX"in e?e.movementX:(e!==Il&&(Il&&e.type==="mousemove"?(ku=e.screenX-Il.screenX,Ou=e.screenY-Il.screenY):Ou=ku=0,Il=e),ku)},movementY:function(e){return"movementY"in e?e.movementY:Ou}}),dd=bt(xo),cb=y({},xo,{dataTransfer:0}),fb=bt(cb),db=y({},Wl,{relatedTarget:0}),Cu=bt(db),hb=y({},mr,{animationName:0,elapsedTime:0,pseudoElement:0}),mb=bt(hb),gb=y({},mr,{clipboardData:function(e){return"clipboardData"in e?e.clipboardData:window.clipboardData}}),pb=bt(gb),yb=y({},mr,{data:0}),hd=bt(yb),bb={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},vb={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},xb={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function wb(e){var t=this.nativeEvent;return t.getModifierState?t.getModifierState(e):(e=xb[e])?!!t[e]:!1}function Mu(){return wb}var Sb=y({},Wl,{key:function(e){if(e.key){var t=bb[e.key]||e.key;if(t!=="Unidentified")return t}return e.type==="keypress"?(e=yo(e),e===13?"Enter":String.fromCharCode(e)):e.type==="keydown"||e.type==="keyup"?vb[e.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:Mu,charCode:function(e){return e.type==="keypress"?yo(e):0},keyCode:function(e){return e.type==="keydown"||e.type==="keyup"?e.keyCode:0},which:function(e){return e.type==="keypress"?yo(e):e.type==="keydown"||e.type==="keyup"?e.keyCode:0}}),Eb=bt(Sb),Rb=y({},xo,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0}),md=bt(Rb),Tb=y({},Wl,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:Mu}),Ab=bt(Tb),kb=y({},mr,{propertyName:0,elapsedTime:0,pseudoElement:0}),Ob=bt(kb),Cb=y({},xo,{deltaX:function(e){return"deltaX"in e?e.deltaX:"wheelDeltaX"in e?-e.wheelDeltaX:0},deltaY:function(e){return"deltaY"in e?e.deltaY:"wheelDeltaY"in e?-e.wheelDeltaY:"wheelDelta"in e?-e.wheelDelta:0},deltaZ:0,deltaMode:0}),Mb=bt(Cb),Nb=y({},mr,{newState:0,oldState:0}),zb=bt(Nb),_b=[9,13,27,32],Nu=dn&&"CompositionEvent"in window,ea=null;dn&&"documentMode"in document&&(ea=document.documentMode);var Db=dn&&"TextEvent"in window&&!ea,gd=dn&&(!Nu||ea&&8<ea&&11>=ea),pd=" ",yd=!1;function bd(e,t){switch(e){case"keyup":return _b.indexOf(t.keyCode)!==-1;case"keydown":return t.keyCode!==229;case"keypress":case"mousedown":case"focusout":return!0;default:return!1}}function vd(e){return e=e.detail,typeof e=="object"&&"data"in e?e.data:null}var Jr=!1;function Ub(e,t){switch(e){case"compositionend":return vd(t);case"keypress":return t.which!==32?null:(yd=!0,pd);case"textInput":return e=t.data,e===pd&&yd?null:e;default:return null}}function jb(e,t){if(Jr)return e==="compositionend"||!Nu&&bd(e,t)?(e=cd(),po=Au=Un=null,Jr=!1,e):null;switch(e){case"paste":return null;case"keypress":if(!(t.ctrlKey||t.altKey||t.metaKey)||t.ctrlKey&&t.altKey){if(t.char&&1<t.char.length)return t.char;if(t.which)return String.fromCharCode(t.which)}return null;case"compositionend":return gd&&t.locale!=="ko"?null:t.data;default:return null}}var Lb={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function xd(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t==="input"?!!Lb[e.type]:t==="textarea"}function wd(e,t,n,l){Kr?$r?$r.push(l):$r=[l]:Kr=l,t=oi(t,"onChange"),0<t.length&&(n=new vo("onChange","change",null,n,l),e.push({event:n,listeners:t}))}var ta=null,na=null;function Bb(e){tm(e,0)}function wo(e){var t=Jl(e);if(nd(t))return e}function Sd(e,t){if(e==="change")return t}var Ed=!1;if(dn){var zu;if(dn){var _u="oninput"in document;if(!_u){var Rd=document.createElement("div");Rd.setAttribute("oninput","return;"),_u=typeof Rd.oninput=="function"}zu=_u}else zu=!1;Ed=zu&&(!document.documentMode||9<document.documentMode)}function Td(){ta&&(ta.detachEvent("onpropertychange",Ad),na=ta=null)}function Ad(e){if(e.propertyName==="value"&&wo(na)){var t=[];wd(t,na,e,Eu(e)),sd(Bb,t)}}function Hb(e,t,n){e==="focusin"?(Td(),ta=t,na=n,ta.attachEvent("onpropertychange",Ad)):e==="focusout"&&Td()}function qb(e){if(e==="selectionchange"||e==="keyup"||e==="keydown")return wo(na)}function Gb(e,t){if(e==="click")return wo(t)}function Yb(e,t){if(e==="input"||e==="change")return wo(t)}function Vb(e,t){return e===t&&(e!==0||1/e===1/t)||e!==e&&t!==t}var Ct=typeof Object.is=="function"?Object.is:Vb;function ra(e,t){if(Ct(e,t))return!0;if(typeof e!="object"||e===null||typeof t!="object"||t===null)return!1;var n=Object.keys(e),l=Object.keys(t);if(n.length!==l.length)return!1;for(l=0;l<n.length;l++){var u=n[l];if(!jt.call(t,u)||!Ct(e[u],t[u]))return!1}return!0}function kd(e){for(;e&&e.firstChild;)e=e.firstChild;return e}function Od(e,t){var n=kd(e);e=0;for(var l;n;){if(n.nodeType===3){if(l=e+n.textContent.length,e<=t&&l>=t)return{node:n,offset:t-e};e=l}e:{for(;n;){if(n.nextSibling){n=n.nextSibling;break e}n=n.parentNode}n=void 0}n=kd(n)}}function Cd(e,t){return e&&t?e===t?!0:e&&e.nodeType===3?!1:t&&t.nodeType===3?Cd(e,t.parentNode):"contains"in e?e.contains(t):e.compareDocumentPosition?!!(e.compareDocumentPosition(t)&16):!1:!1}function Md(e){e=e!=null&&e.ownerDocument!=null&&e.ownerDocument.defaultView!=null?e.ownerDocument.defaultView:window;for(var t=mo(e.document);t instanceof e.HTMLIFrameElement;){try{var n=typeof t.contentWindow.location.href=="string"}catch{n=!1}if(n)e=t.contentWindow;else break;t=mo(e.document)}return t}function Du(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t&&(t==="input"&&(e.type==="text"||e.type==="search"||e.type==="tel"||e.type==="url"||e.type==="password")||t==="textarea"||e.contentEditable==="true")}var Xb=dn&&"documentMode"in document&&11>=document.documentMode,Pr=null,Uu=null,la=null,ju=!1;function Nd(e,t,n){var l=n.window===n?n.document:n.nodeType===9?n:n.ownerDocument;ju||Pr==null||Pr!==mo(l)||(l=Pr,"selectionStart"in l&&Du(l)?l={start:l.selectionStart,end:l.selectionEnd}:(l=(l.ownerDocument&&l.ownerDocument.defaultView||window).getSelection(),l={anchorNode:l.anchorNode,anchorOffset:l.anchorOffset,focusNode:l.focusNode,focusOffset:l.focusOffset}),la&&ra(la,l)||(la=l,l=oi(Uu,"onSelect"),0<l.length&&(t=new vo("onSelect","select",null,t,n),e.push({event:t,listeners:l}),t.target=Pr)))}function gr(e,t){var n={};return n[e.toLowerCase()]=t.toLowerCase(),n["Webkit"+e]="webkit"+t,n["Moz"+e]="moz"+t,n}var Fr={animationend:gr("Animation","AnimationEnd"),animationiteration:gr("Animation","AnimationIteration"),animationstart:gr("Animation","AnimationStart"),transitionrun:gr("Transition","TransitionRun"),transitionstart:gr("Transition","TransitionStart"),transitioncancel:gr("Transition","TransitionCancel"),transitionend:gr("Transition","TransitionEnd")},Lu={},zd={};dn&&(zd=document.createElement("div").style,"AnimationEvent"in window||(delete Fr.animationend.animation,delete Fr.animationiteration.animation,delete Fr.animationstart.animation),"TransitionEvent"in window||delete Fr.transitionend.transition);function pr(e){if(Lu[e])return Lu[e];if(!Fr[e])return e;var t=Fr[e],n;for(n in t)if(t.hasOwnProperty(n)&&n in zd)return Lu[e]=t[n];return e}var _d=pr("animationend"),Dd=pr("animationiteration"),Ud=pr("animationstart"),Qb=pr("transitionrun"),Zb=pr("transitionstart"),Kb=pr("transitioncancel"),jd=pr("transitionend"),Ld=new Map,Bu="abort auxClick beforeToggle cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll toggle touchMove waiting wheel".split(" ");Bu.push("scrollEnd");function Kt(e,t){Ld.set(e,t),hr(t,[e])}var Bd=new WeakMap;function Ht(e,t){if(typeof e=="object"&&e!==null){var n=Bd.get(e);return n!==void 0?n:(t={value:e,source:t,stack:ed(t)},Bd.set(e,t),t)}return{value:e,source:t,stack:ed(t)}}var qt=[],Wr=0,Hu=0;function So(){for(var e=Wr,t=Hu=Wr=0;t<e;){var n=qt[t];qt[t++]=null;var l=qt[t];qt[t++]=null;var u=qt[t];qt[t++]=null;var c=qt[t];if(qt[t++]=null,l!==null&&u!==null){var p=l.pending;p===null?u.next=u:(u.next=p.next,p.next=u),l.pending=u}c!==0&&Hd(n,u,c)}}function Eo(e,t,n,l){qt[Wr++]=e,qt[Wr++]=t,qt[Wr++]=n,qt[Wr++]=l,Hu|=l,e.lanes|=l,e=e.alternate,e!==null&&(e.lanes|=l)}function qu(e,t,n,l){return Eo(e,t,n,l),Ro(e)}function Ir(e,t){return Eo(e,null,null,t),Ro(e)}function Hd(e,t,n){e.lanes|=n;var l=e.alternate;l!==null&&(l.lanes|=n);for(var u=!1,c=e.return;c!==null;)c.childLanes|=n,l=c.alternate,l!==null&&(l.childLanes|=n),c.tag===22&&(e=c.stateNode,e===null||e._visibility&1||(u=!0)),e=c,c=c.return;return e.tag===3?(c=e.stateNode,u&&t!==null&&(u=31-Ot(n),e=c.hiddenUpdates,l=e[u],l===null?e[u]=[t]:l.push(t),t.lane=n|536870912),c):null}function Ro(e){if(50<Ca)throw Ca=0,Zs=null,Error(o(185));for(var t=e.return;t!==null;)e=t,t=e.return;return e.tag===3?e.stateNode:null}var el={};function $b(e,t,n,l){this.tag=e,this.key=n,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.refCleanup=this.ref=null,this.pendingProps=t,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=l,this.subtreeFlags=this.flags=0,this.deletions=null,this.childLanes=this.lanes=0,this.alternate=null}function Mt(e,t,n,l){return new $b(e,t,n,l)}function Gu(e){return e=e.prototype,!(!e||!e.isReactComponent)}function hn(e,t){var n=e.alternate;return n===null?(n=Mt(e.tag,t,e.key,e.mode),n.elementType=e.elementType,n.type=e.type,n.stateNode=e.stateNode,n.alternate=e,e.alternate=n):(n.pendingProps=t,n.type=e.type,n.flags=0,n.subtreeFlags=0,n.deletions=null),n.flags=e.flags&65011712,n.childLanes=e.childLanes,n.lanes=e.lanes,n.child=e.child,n.memoizedProps=e.memoizedProps,n.memoizedState=e.memoizedState,n.updateQueue=e.updateQueue,t=e.dependencies,n.dependencies=t===null?null:{lanes:t.lanes,firstContext:t.firstContext},n.sibling=e.sibling,n.index=e.index,n.ref=e.ref,n.refCleanup=e.refCleanup,n}function qd(e,t){e.flags&=65011714;var n=e.alternate;return n===null?(e.childLanes=0,e.lanes=t,e.child=null,e.subtreeFlags=0,e.memoizedProps=null,e.memoizedState=null,e.updateQueue=null,e.dependencies=null,e.stateNode=null):(e.childLanes=n.childLanes,e.lanes=n.lanes,e.child=n.child,e.subtreeFlags=0,e.deletions=null,e.memoizedProps=n.memoizedProps,e.memoizedState=n.memoizedState,e.updateQueue=n.updateQueue,e.type=n.type,t=n.dependencies,e.dependencies=t===null?null:{lanes:t.lanes,firstContext:t.firstContext}),e}function To(e,t,n,l,u,c){var p=0;if(l=e,typeof e=="function")Gu(e)&&(p=1);else if(typeof e=="string")p=Pv(e,n,oe.current)?26:e==="html"||e==="head"||e==="body"?27:5;else e:switch(e){case ne:return e=Mt(31,n,t,u),e.elementType=ne,e.lanes=c,e;case x:return yr(n.children,u,c,t);case R:p=8,u|=24;break;case k:return e=Mt(12,n,t,u|2),e.elementType=k,e.lanes=c,e;case V:return e=Mt(13,n,t,u),e.elementType=V,e.lanes=c,e;case W:return e=Mt(19,n,t,u),e.elementType=W,e.lanes=c,e;default:if(typeof e=="object"&&e!==null)switch(e.$$typeof){case O:case H:p=10;break e;case M:p=9;break e;case $:p=11;break e;case ee:p=14;break e;case fe:p=16,l=null;break e}p=29,n=Error(o(130,e===null?"null":typeof e,"")),l=null}return t=Mt(p,n,t,u),t.elementType=e,t.type=l,t.lanes=c,t}function yr(e,t,n,l){return e=Mt(7,e,l,t),e.lanes=n,e}function Yu(e,t,n){return e=Mt(6,e,null,t),e.lanes=n,e}function Vu(e,t,n){return t=Mt(4,e.children!==null?e.children:[],e.key,t),t.lanes=n,t.stateNode={containerInfo:e.containerInfo,pendingChildren:null,implementation:e.implementation},t}var tl=[],nl=0,Ao=null,ko=0,Gt=[],Yt=0,br=null,mn=1,gn="";function vr(e,t){tl[nl++]=ko,tl[nl++]=Ao,Ao=e,ko=t}function Gd(e,t,n){Gt[Yt++]=mn,Gt[Yt++]=gn,Gt[Yt++]=br,br=e;var l=mn;e=gn;var u=32-Ot(l)-1;l&=~(1<<u),n+=1;var c=32-Ot(t)+u;if(30<c){var p=u-u%5;c=(l&(1<<p)-1).toString(32),l>>=p,u-=p,mn=1<<32-Ot(t)+u|n<<u|l,gn=c+e}else mn=1<<c|n<<u|l,gn=e}function Xu(e){e.return!==null&&(vr(e,1),Gd(e,1,0))}function Qu(e){for(;e===Ao;)Ao=tl[--nl],tl[nl]=null,ko=tl[--nl],tl[nl]=null;for(;e===br;)br=Gt[--Yt],Gt[Yt]=null,gn=Gt[--Yt],Gt[Yt]=null,mn=Gt[--Yt],Gt[Yt]=null}var gt=null,Ze=null,Ne=!1,xr=null,tn=!1,Zu=Error(o(519));function wr(e){var t=Error(o(418,""));throw ia(Ht(t,e)),Zu}function Yd(e){var t=e.stateNode,n=e.type,l=e.memoizedProps;switch(t[ft]=e,t[yt]=l,n){case"dialog":Te("cancel",t),Te("close",t);break;case"iframe":case"object":case"embed":Te("load",t);break;case"video":case"audio":for(n=0;n<Na.length;n++)Te(Na[n],t);break;case"source":Te("error",t);break;case"img":case"image":case"link":Te("error",t),Te("load",t);break;case"details":Te("toggle",t);break;case"input":Te("invalid",t),rd(t,l.value,l.defaultValue,l.checked,l.defaultChecked,l.type,l.name,!0),ho(t);break;case"select":Te("invalid",t);break;case"textarea":Te("invalid",t),ad(t,l.value,l.defaultValue,l.children),ho(t)}n=l.children,typeof n!="string"&&typeof n!="number"&&typeof n!="bigint"||t.textContent===""+n||l.suppressHydrationWarning===!0||am(t.textContent,n)?(l.popover!=null&&(Te("beforetoggle",t),Te("toggle",t)),l.onScroll!=null&&Te("scroll",t),l.onScrollEnd!=null&&Te("scrollend",t),l.onClick!=null&&(t.onclick=ii),t=!0):t=!1,t||wr(e)}function Vd(e){for(gt=e.return;gt;)switch(gt.tag){case 5:case 13:tn=!1;return;case 27:case 3:tn=!0;return;default:gt=gt.return}}function aa(e){if(e!==gt)return!1;if(!Ne)return Vd(e),Ne=!0,!1;var t=e.tag,n;if((n=t!==3&&t!==27)&&((n=t===5)&&(n=e.type,n=!(n!=="form"&&n!=="button")||uc(e.type,e.memoizedProps)),n=!n),n&&Ze&&wr(e),Vd(e),t===13){if(e=e.memoizedState,e=e!==null?e.dehydrated:null,!e)throw Error(o(317));e:{for(e=e.nextSibling,t=0;e;){if(e.nodeType===8)if(n=e.data,n==="/$"){if(t===0){Ze=Jt(e.nextSibling);break e}t--}else n!=="$"&&n!=="$!"&&n!=="$?"||t++;e=e.nextSibling}Ze=null}}else t===27?(t=Ze,Fn(e.type)?(e=dc,dc=null,Ze=e):Ze=t):Ze=gt?Jt(e.stateNode.nextSibling):null;return!0}function oa(){Ze=gt=null,Ne=!1}function Xd(){var e=xr;return e!==null&&(wt===null?wt=e:wt.push.apply(wt,e),xr=null),e}function ia(e){xr===null?xr=[e]:xr.push(e)}var Ku=_(null),Sr=null,pn=null;function jn(e,t,n){F(Ku,t._currentValue),t._currentValue=n}function yn(e){e._currentValue=Ku.current,P(Ku)}function $u(e,t,n){for(;e!==null;){var l=e.alternate;if((e.childLanes&t)!==t?(e.childLanes|=t,l!==null&&(l.childLanes|=t)):l!==null&&(l.childLanes&t)!==t&&(l.childLanes|=t),e===n)break;e=e.return}}function Ju(e,t,n,l){var u=e.child;for(u!==null&&(u.return=e);u!==null;){var c=u.dependencies;if(c!==null){var p=u.child;c=c.firstContext;e:for(;c!==null;){var b=c;c=u;for(var A=0;A<t.length;A++)if(b.context===t[A]){c.lanes|=n,b=c.alternate,b!==null&&(b.lanes|=n),$u(c.return,n,e),l||(p=null);break e}c=b.next}}else if(u.tag===18){if(p=u.return,p===null)throw Error(o(341));p.lanes|=n,c=p.alternate,c!==null&&(c.lanes|=n),$u(p,n,e),p=null}else p=u.child;if(p!==null)p.return=u;else for(p=u;p!==null;){if(p===e){p=null;break}if(u=p.sibling,u!==null){u.return=p.return,p=u;break}p=p.return}u=p}}function ua(e,t,n,l){e=null;for(var u=t,c=!1;u!==null;){if(!c){if((u.flags&524288)!==0)c=!0;else if((u.flags&262144)!==0)break}if(u.tag===10){var p=u.alternate;if(p===null)throw Error(o(387));if(p=p.memoizedProps,p!==null){var b=u.type;Ct(u.pendingProps.value,p.value)||(e!==null?e.push(b):e=[b])}}else if(u===ke.current){if(p=u.alternate,p===null)throw Error(o(387));p.memoizedState.memoizedState!==u.memoizedState.memoizedState&&(e!==null?e.push(La):e=[La])}u=u.return}e!==null&&Ju(t,e,n,l),t.flags|=262144}function Oo(e){for(e=e.firstContext;e!==null;){if(!Ct(e.context._currentValue,e.memoizedValue))return!0;e=e.next}return!1}function Er(e){Sr=e,pn=null,e=e.dependencies,e!==null&&(e.firstContext=null)}function dt(e){return Qd(Sr,e)}function Co(e,t){return Sr===null&&Er(e),Qd(e,t)}function Qd(e,t){var n=t._currentValue;if(t={context:t,memoizedValue:n,next:null},pn===null){if(e===null)throw Error(o(308));pn=t,e.dependencies={lanes:0,firstContext:t},e.flags|=524288}else pn=pn.next=t;return n}var Jb=typeof AbortController<"u"?AbortController:function(){var e=[],t=this.signal={aborted:!1,addEventListener:function(n,l){e.push(l)}};this.abort=function(){t.aborted=!0,e.forEach(function(n){return n()})}},Pb=r.unstable_scheduleCallback,Fb=r.unstable_NormalPriority,We={$$typeof:H,Consumer:null,Provider:null,_currentValue:null,_currentValue2:null,_threadCount:0};function Pu(){return{controller:new Jb,data:new Map,refCount:0}}function sa(e){e.refCount--,e.refCount===0&&Pb(Fb,function(){e.controller.abort()})}var ca=null,Fu=0,rl=0,ll=null;function Wb(e,t){if(ca===null){var n=ca=[];Fu=0,rl=Is(),ll={status:"pending",value:void 0,then:function(l){n.push(l)}}}return Fu++,t.then(Zd,Zd),t}function Zd(){if(--Fu===0&&ca!==null){ll!==null&&(ll.status="fulfilled");var e=ca;ca=null,rl=0,ll=null;for(var t=0;t<e.length;t++)(0,e[t])()}}function Ib(e,t){var n=[],l={status:"pending",value:null,reason:null,then:function(u){n.push(u)}};return e.then(function(){l.status="fulfilled",l.value=t;for(var u=0;u<n.length;u++)(0,n[u])(t)},function(u){for(l.status="rejected",l.reason=u,u=0;u<n.length;u++)(0,n[u])(void 0)}),l}var Kd=N.S;N.S=function(e,t){typeof t=="object"&&t!==null&&typeof t.then=="function"&&Wb(e,t),Kd!==null&&Kd(e,t)};var Rr=_(null);function Wu(){var e=Rr.current;return e!==null?e:qe.pooledCache}function Mo(e,t){t===null?F(Rr,Rr.current):F(Rr,t.pool)}function $d(){var e=Wu();return e===null?null:{parent:We._currentValue,pool:e}}var fa=Error(o(460)),Jd=Error(o(474)),No=Error(o(542)),Iu={then:function(){}};function Pd(e){return e=e.status,e==="fulfilled"||e==="rejected"}function zo(){}function Fd(e,t,n){switch(n=e[n],n===void 0?e.push(t):n!==t&&(t.then(zo,zo),t=n),t.status){case"fulfilled":return t.value;case"rejected":throw e=t.reason,Id(e),e;default:if(typeof t.status=="string")t.then(zo,zo);else{if(e=qe,e!==null&&100<e.shellSuspendCounter)throw Error(o(482));e=t,e.status="pending",e.then(function(l){if(t.status==="pending"){var u=t;u.status="fulfilled",u.value=l}},function(l){if(t.status==="pending"){var u=t;u.status="rejected",u.reason=l}})}switch(t.status){case"fulfilled":return t.value;case"rejected":throw e=t.reason,Id(e),e}throw da=t,fa}}var da=null;function Wd(){if(da===null)throw Error(o(459));var e=da;return da=null,e}function Id(e){if(e===fa||e===No)throw Error(o(483))}var Ln=!1;function es(e){e.updateQueue={baseState:e.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null,lanes:0,hiddenCallbacks:null},callbacks:null}}function ts(e,t){e=e.updateQueue,t.updateQueue===e&&(t.updateQueue={baseState:e.baseState,firstBaseUpdate:e.firstBaseUpdate,lastBaseUpdate:e.lastBaseUpdate,shared:e.shared,callbacks:null})}function Bn(e){return{lane:e,tag:0,payload:null,callback:null,next:null}}function Hn(e,t,n){var l=e.updateQueue;if(l===null)return null;if(l=l.shared,(ze&2)!==0){var u=l.pending;return u===null?t.next=t:(t.next=u.next,u.next=t),l.pending=t,t=Ro(e),Hd(e,null,n),t}return Eo(e,l,t,n),Ro(e)}function ha(e,t,n){if(t=t.updateQueue,t!==null&&(t=t.shared,(n&4194048)!==0)){var l=t.lanes;l&=e.pendingLanes,n|=l,t.lanes=n,Zf(e,n)}}function ns(e,t){var n=e.updateQueue,l=e.alternate;if(l!==null&&(l=l.updateQueue,n===l)){var u=null,c=null;if(n=n.firstBaseUpdate,n!==null){do{var p={lane:n.lane,tag:n.tag,payload:n.payload,callback:null,next:null};c===null?u=c=p:c=c.next=p,n=n.next}while(n!==null);c===null?u=c=t:c=c.next=t}else u=c=t;n={baseState:l.baseState,firstBaseUpdate:u,lastBaseUpdate:c,shared:l.shared,callbacks:l.callbacks},e.updateQueue=n;return}e=n.lastBaseUpdate,e===null?n.firstBaseUpdate=t:e.next=t,n.lastBaseUpdate=t}var rs=!1;function ma(){if(rs){var e=ll;if(e!==null)throw e}}function ga(e,t,n,l){rs=!1;var u=e.updateQueue;Ln=!1;var c=u.firstBaseUpdate,p=u.lastBaseUpdate,b=u.shared.pending;if(b!==null){u.shared.pending=null;var A=b,U=A.next;A.next=null,p===null?c=U:p.next=U,p=A;var Q=e.alternate;Q!==null&&(Q=Q.updateQueue,b=Q.lastBaseUpdate,b!==p&&(b===null?Q.firstBaseUpdate=U:b.next=U,Q.lastBaseUpdate=A))}if(c!==null){var J=u.baseState;p=0,Q=U=A=null,b=c;do{var j=b.lane&-536870913,B=j!==b.lane;if(B?(Oe&j)===j:(l&j)===j){j!==0&&j===rl&&(rs=!0),Q!==null&&(Q=Q.next={lane:0,tag:b.tag,payload:b.payload,callback:null,next:null});e:{var ye=e,he=b;j=t;var je=n;switch(he.tag){case 1:if(ye=he.payload,typeof ye=="function"){J=ye.call(je,J,j);break e}J=ye;break e;case 3:ye.flags=ye.flags&-65537|128;case 0:if(ye=he.payload,j=typeof ye=="function"?ye.call(je,J,j):ye,j==null)break e;J=y({},J,j);break e;case 2:Ln=!0}}j=b.callback,j!==null&&(e.flags|=64,B&&(e.flags|=8192),B=u.callbacks,B===null?u.callbacks=[j]:B.push(j))}else B={lane:j,tag:b.tag,payload:b.payload,callback:b.callback,next:null},Q===null?(U=Q=B,A=J):Q=Q.next=B,p|=j;if(b=b.next,b===null){if(b=u.shared.pending,b===null)break;B=b,b=B.next,B.next=null,u.lastBaseUpdate=B,u.shared.pending=null}}while(!0);Q===null&&(A=J),u.baseState=A,u.firstBaseUpdate=U,u.lastBaseUpdate=Q,c===null&&(u.shared.lanes=0),Kn|=p,e.lanes=p,e.memoizedState=J}}function eh(e,t){if(typeof e!="function")throw Error(o(191,e));e.call(t)}function th(e,t){var n=e.callbacks;if(n!==null)for(e.callbacks=null,e=0;e<n.length;e++)eh(n[e],t)}var al=_(null),_o=_(0);function nh(e,t){e=Rn,F(_o,e),F(al,t),Rn=e|t.baseLanes}function ls(){F(_o,Rn),F(al,al.current)}function as(){Rn=_o.current,P(al),P(_o)}var qn=0,Se=null,De=null,Pe=null,Do=!1,ol=!1,Tr=!1,Uo=0,pa=0,il=null,ev=0;function $e(){throw Error(o(321))}function os(e,t){if(t===null)return!1;for(var n=0;n<t.length&&n<e.length;n++)if(!Ct(e[n],t[n]))return!1;return!0}function is(e,t,n,l,u,c){return qn=c,Se=t,t.memoizedState=null,t.updateQueue=null,t.lanes=0,N.H=e===null||e.memoizedState===null?Hh:qh,Tr=!1,c=n(l,u),Tr=!1,ol&&(c=lh(t,n,l,u)),rh(e),c}function rh(e){N.H=Go;var t=De!==null&&De.next!==null;if(qn=0,Pe=De=Se=null,Do=!1,pa=0,il=null,t)throw Error(o(300));e===null||tt||(e=e.dependencies,e!==null&&Oo(e)&&(tt=!0))}function lh(e,t,n,l){Se=e;var u=0;do{if(ol&&(il=null),pa=0,ol=!1,25<=u)throw Error(o(301));if(u+=1,Pe=De=null,e.updateQueue!=null){var c=e.updateQueue;c.lastEffect=null,c.events=null,c.stores=null,c.memoCache!=null&&(c.memoCache.index=0)}N.H=iv,c=t(n,l)}while(ol);return c}function tv(){var e=N.H,t=e.useState()[0];return t=typeof t.then=="function"?ya(t):t,e=e.useState()[0],(De!==null?De.memoizedState:null)!==e&&(Se.flags|=1024),t}function us(){var e=Uo!==0;return Uo=0,e}function ss(e,t,n){t.updateQueue=e.updateQueue,t.flags&=-2053,e.lanes&=~n}function cs(e){if(Do){for(e=e.memoizedState;e!==null;){var t=e.queue;t!==null&&(t.pending=null),e=e.next}Do=!1}qn=0,Pe=De=Se=null,ol=!1,pa=Uo=0,il=null}function vt(){var e={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return Pe===null?Se.memoizedState=Pe=e:Pe=Pe.next=e,Pe}function Fe(){if(De===null){var e=Se.alternate;e=e!==null?e.memoizedState:null}else e=De.next;var t=Pe===null?Se.memoizedState:Pe.next;if(t!==null)Pe=t,De=e;else{if(e===null)throw Se.alternate===null?Error(o(467)):Error(o(310));De=e,e={memoizedState:De.memoizedState,baseState:De.baseState,baseQueue:De.baseQueue,queue:De.queue,next:null},Pe===null?Se.memoizedState=Pe=e:Pe=Pe.next=e}return Pe}function fs(){return{lastEffect:null,events:null,stores:null,memoCache:null}}function ya(e){var t=pa;return pa+=1,il===null&&(il=[]),e=Fd(il,e,t),t=Se,(Pe===null?t.memoizedState:Pe.next)===null&&(t=t.alternate,N.H=t===null||t.memoizedState===null?Hh:qh),e}function jo(e){if(e!==null&&typeof e=="object"){if(typeof e.then=="function")return ya(e);if(e.$$typeof===H)return dt(e)}throw Error(o(438,String(e)))}function ds(e){var t=null,n=Se.updateQueue;if(n!==null&&(t=n.memoCache),t==null){var l=Se.alternate;l!==null&&(l=l.updateQueue,l!==null&&(l=l.memoCache,l!=null&&(t={data:l.data.map(function(u){return u.slice()}),index:0})))}if(t==null&&(t={data:[],index:0}),n===null&&(n=fs(),Se.updateQueue=n),n.memoCache=t,n=t.data[t.index],n===void 0)for(n=t.data[t.index]=Array(e),l=0;l<e;l++)n[l]=se;return t.index++,n}function bn(e,t){return typeof t=="function"?t(e):t}function Lo(e){var t=Fe();return hs(t,De,e)}function hs(e,t,n){var l=e.queue;if(l===null)throw Error(o(311));l.lastRenderedReducer=n;var u=e.baseQueue,c=l.pending;if(c!==null){if(u!==null){var p=u.next;u.next=c.next,c.next=p}t.baseQueue=u=c,l.pending=null}if(c=e.baseState,u===null)e.memoizedState=c;else{t=u.next;var b=p=null,A=null,U=t,Q=!1;do{var J=U.lane&-536870913;if(J!==U.lane?(Oe&J)===J:(qn&J)===J){var j=U.revertLane;if(j===0)A!==null&&(A=A.next={lane:0,revertLane:0,action:U.action,hasEagerState:U.hasEagerState,eagerState:U.eagerState,next:null}),J===rl&&(Q=!0);else if((qn&j)===j){U=U.next,j===rl&&(Q=!0);continue}else J={lane:0,revertLane:U.revertLane,action:U.action,hasEagerState:U.hasEagerState,eagerState:U.eagerState,next:null},A===null?(b=A=J,p=c):A=A.next=J,Se.lanes|=j,Kn|=j;J=U.action,Tr&&n(c,J),c=U.hasEagerState?U.eagerState:n(c,J)}else j={lane:J,revertLane:U.revertLane,action:U.action,hasEagerState:U.hasEagerState,eagerState:U.eagerState,next:null},A===null?(b=A=j,p=c):A=A.next=j,Se.lanes|=J,Kn|=J;U=U.next}while(U!==null&&U!==t);if(A===null?p=c:A.next=b,!Ct(c,e.memoizedState)&&(tt=!0,Q&&(n=ll,n!==null)))throw n;e.memoizedState=c,e.baseState=p,e.baseQueue=A,l.lastRenderedState=c}return u===null&&(l.lanes=0),[e.memoizedState,l.dispatch]}function ms(e){var t=Fe(),n=t.queue;if(n===null)throw Error(o(311));n.lastRenderedReducer=e;var l=n.dispatch,u=n.pending,c=t.memoizedState;if(u!==null){n.pending=null;var p=u=u.next;do c=e(c,p.action),p=p.next;while(p!==u);Ct(c,t.memoizedState)||(tt=!0),t.memoizedState=c,t.baseQueue===null&&(t.baseState=c),n.lastRenderedState=c}return[c,l]}function ah(e,t,n){var l=Se,u=Fe(),c=Ne;if(c){if(n===void 0)throw Error(o(407));n=n()}else n=t();var p=!Ct((De||u).memoizedState,n);p&&(u.memoizedState=n,tt=!0),u=u.queue;var b=uh.bind(null,l,u,e);if(ba(2048,8,b,[e]),u.getSnapshot!==t||p||Pe!==null&&Pe.memoizedState.tag&1){if(l.flags|=2048,ul(9,Bo(),ih.bind(null,l,u,n,t),null),qe===null)throw Error(o(349));c||(qn&124)!==0||oh(l,t,n)}return n}function oh(e,t,n){e.flags|=16384,e={getSnapshot:t,value:n},t=Se.updateQueue,t===null?(t=fs(),Se.updateQueue=t,t.stores=[e]):(n=t.stores,n===null?t.stores=[e]:n.push(e))}function ih(e,t,n,l){t.value=n,t.getSnapshot=l,sh(t)&&ch(e)}function uh(e,t,n){return n(function(){sh(t)&&ch(e)})}function sh(e){var t=e.getSnapshot;e=e.value;try{var n=t();return!Ct(e,n)}catch{return!0}}function ch(e){var t=Ir(e,2);t!==null&&Ut(t,e,2)}function gs(e){var t=vt();if(typeof e=="function"){var n=e;if(e=n(),Tr){_n(!0);try{n()}finally{_n(!1)}}}return t.memoizedState=t.baseState=e,t.queue={pending:null,lanes:0,dispatch:null,lastRenderedReducer:bn,lastRenderedState:e},t}function fh(e,t,n,l){return e.baseState=n,hs(e,De,typeof l=="function"?l:bn)}function nv(e,t,n,l,u){if(qo(e))throw Error(o(485));if(e=t.action,e!==null){var c={payload:u,action:e,next:null,isTransition:!0,status:"pending",value:null,reason:null,listeners:[],then:function(p){c.listeners.push(p)}};N.T!==null?n(!0):c.isTransition=!1,l(c),n=t.pending,n===null?(c.next=t.pending=c,dh(t,c)):(c.next=n.next,t.pending=n.next=c)}}function dh(e,t){var n=t.action,l=t.payload,u=e.state;if(t.isTransition){var c=N.T,p={};N.T=p;try{var b=n(u,l),A=N.S;A!==null&&A(p,b),hh(e,t,b)}catch(U){ps(e,t,U)}finally{N.T=c}}else try{c=n(u,l),hh(e,t,c)}catch(U){ps(e,t,U)}}function hh(e,t,n){n!==null&&typeof n=="object"&&typeof n.then=="function"?n.then(function(l){mh(e,t,l)},function(l){return ps(e,t,l)}):mh(e,t,n)}function mh(e,t,n){t.status="fulfilled",t.value=n,gh(t),e.state=n,t=e.pending,t!==null&&(n=t.next,n===t?e.pending=null:(n=n.next,t.next=n,dh(e,n)))}function ps(e,t,n){var l=e.pending;if(e.pending=null,l!==null){l=l.next;do t.status="rejected",t.reason=n,gh(t),t=t.next;while(t!==l)}e.action=null}function gh(e){e=e.listeners;for(var t=0;t<e.length;t++)(0,e[t])()}function ph(e,t){return t}function yh(e,t){if(Ne){var n=qe.formState;if(n!==null){e:{var l=Se;if(Ne){if(Ze){t:{for(var u=Ze,c=tn;u.nodeType!==8;){if(!c){u=null;break t}if(u=Jt(u.nextSibling),u===null){u=null;break t}}c=u.data,u=c==="F!"||c==="F"?u:null}if(u){Ze=Jt(u.nextSibling),l=u.data==="F!";break e}}wr(l)}l=!1}l&&(t=n[0])}}return n=vt(),n.memoizedState=n.baseState=t,l={pending:null,lanes:0,dispatch:null,lastRenderedReducer:ph,lastRenderedState:t},n.queue=l,n=jh.bind(null,Se,l),l.dispatch=n,l=gs(!1),c=ws.bind(null,Se,!1,l.queue),l=vt(),u={state:t,dispatch:null,action:e,pending:null},l.queue=u,n=nv.bind(null,Se,u,c,n),u.dispatch=n,l.memoizedState=e,[t,n,!1]}function bh(e){var t=Fe();return vh(t,De,e)}function vh(e,t,n){if(t=hs(e,t,ph)[0],e=Lo(bn)[0],typeof t=="object"&&t!==null&&typeof t.then=="function")try{var l=ya(t)}catch(p){throw p===fa?No:p}else l=t;t=Fe();var u=t.queue,c=u.dispatch;return n!==t.memoizedState&&(Se.flags|=2048,ul(9,Bo(),rv.bind(null,u,n),null)),[l,c,e]}function rv(e,t){e.action=t}function xh(e){var t=Fe(),n=De;if(n!==null)return vh(t,n,e);Fe(),t=t.memoizedState,n=Fe();var l=n.queue.dispatch;return n.memoizedState=e,[t,l,!1]}function ul(e,t,n,l){return e={tag:e,create:n,deps:l,inst:t,next:null},t=Se.updateQueue,t===null&&(t=fs(),Se.updateQueue=t),n=t.lastEffect,n===null?t.lastEffect=e.next=e:(l=n.next,n.next=e,e.next=l,t.lastEffect=e),e}function Bo(){return{destroy:void 0,resource:void 0}}function wh(){return Fe().memoizedState}function Ho(e,t,n,l){var u=vt();l=l===void 0?null:l,Se.flags|=e,u.memoizedState=ul(1|t,Bo(),n,l)}function ba(e,t,n,l){var u=Fe();l=l===void 0?null:l;var c=u.memoizedState.inst;De!==null&&l!==null&&os(l,De.memoizedState.deps)?u.memoizedState=ul(t,c,n,l):(Se.flags|=e,u.memoizedState=ul(1|t,c,n,l))}function Sh(e,t){Ho(8390656,8,e,t)}function Eh(e,t){ba(2048,8,e,t)}function Rh(e,t){return ba(4,2,e,t)}function Th(e,t){return ba(4,4,e,t)}function Ah(e,t){if(typeof t=="function"){e=e();var n=t(e);return function(){typeof n=="function"?n():t(null)}}if(t!=null)return e=e(),t.current=e,function(){t.current=null}}function kh(e,t,n){n=n!=null?n.concat([e]):null,ba(4,4,Ah.bind(null,t,e),n)}function ys(){}function Oh(e,t){var n=Fe();t=t===void 0?null:t;var l=n.memoizedState;return t!==null&&os(t,l[1])?l[0]:(n.memoizedState=[e,t],e)}function Ch(e,t){var n=Fe();t=t===void 0?null:t;var l=n.memoizedState;if(t!==null&&os(t,l[1]))return l[0];if(l=e(),Tr){_n(!0);try{e()}finally{_n(!1)}}return n.memoizedState=[l,t],l}function bs(e,t,n){return n===void 0||(qn&1073741824)!==0?e.memoizedState=t:(e.memoizedState=n,e=z0(),Se.lanes|=e,Kn|=e,n)}function Mh(e,t,n,l){return Ct(n,t)?n:al.current!==null?(e=bs(e,n,l),Ct(e,t)||(tt=!0),e):(qn&42)===0?(tt=!0,e.memoizedState=n):(e=z0(),Se.lanes|=e,Kn|=e,t)}function Nh(e,t,n,l,u){var c=q.p;q.p=c!==0&&8>c?c:8;var p=N.T,b={};N.T=b,ws(e,!1,t,n);try{var A=u(),U=N.S;if(U!==null&&U(b,A),A!==null&&typeof A=="object"&&typeof A.then=="function"){var Q=Ib(A,l);va(e,t,Q,Dt(e))}else va(e,t,l,Dt(e))}catch(J){va(e,t,{then:function(){},status:"rejected",reason:J},Dt())}finally{q.p=c,N.T=p}}function lv(){}function vs(e,t,n,l){if(e.tag!==5)throw Error(o(476));var u=zh(e).queue;Nh(e,u,t,L,n===null?lv:function(){return _h(e),n(l)})}function zh(e){var t=e.memoizedState;if(t!==null)return t;t={memoizedState:L,baseState:L,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:bn,lastRenderedState:L},next:null};var n={};return t.next={memoizedState:n,baseState:n,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:bn,lastRenderedState:n},next:null},e.memoizedState=t,e=e.alternate,e!==null&&(e.memoizedState=t),t}function _h(e){var t=zh(e).next.queue;va(e,t,{},Dt())}function xs(){return dt(La)}function Dh(){return Fe().memoizedState}function Uh(){return Fe().memoizedState}function av(e){for(var t=e.return;t!==null;){switch(t.tag){case 24:case 3:var n=Dt();e=Bn(n);var l=Hn(t,e,n);l!==null&&(Ut(l,t,n),ha(l,t,n)),t={cache:Pu()},e.payload=t;return}t=t.return}}function ov(e,t,n){var l=Dt();n={lane:l,revertLane:0,action:n,hasEagerState:!1,eagerState:null,next:null},qo(e)?Lh(t,n):(n=qu(e,t,n,l),n!==null&&(Ut(n,e,l),Bh(n,t,l)))}function jh(e,t,n){var l=Dt();va(e,t,n,l)}function va(e,t,n,l){var u={lane:l,revertLane:0,action:n,hasEagerState:!1,eagerState:null,next:null};if(qo(e))Lh(t,u);else{var c=e.alternate;if(e.lanes===0&&(c===null||c.lanes===0)&&(c=t.lastRenderedReducer,c!==null))try{var p=t.lastRenderedState,b=c(p,n);if(u.hasEagerState=!0,u.eagerState=b,Ct(b,p))return Eo(e,t,u,0),qe===null&&So(),!1}catch{}finally{}if(n=qu(e,t,u,l),n!==null)return Ut(n,e,l),Bh(n,t,l),!0}return!1}function ws(e,t,n,l){if(l={lane:2,revertLane:Is(),action:l,hasEagerState:!1,eagerState:null,next:null},qo(e)){if(t)throw Error(o(479))}else t=qu(e,n,l,2),t!==null&&Ut(t,e,2)}function qo(e){var t=e.alternate;return e===Se||t!==null&&t===Se}function Lh(e,t){ol=Do=!0;var n=e.pending;n===null?t.next=t:(t.next=n.next,n.next=t),e.pending=t}function Bh(e,t,n){if((n&4194048)!==0){var l=t.lanes;l&=e.pendingLanes,n|=l,t.lanes=n,Zf(e,n)}}var Go={readContext:dt,use:jo,useCallback:$e,useContext:$e,useEffect:$e,useImperativeHandle:$e,useLayoutEffect:$e,useInsertionEffect:$e,useMemo:$e,useReducer:$e,useRef:$e,useState:$e,useDebugValue:$e,useDeferredValue:$e,useTransition:$e,useSyncExternalStore:$e,useId:$e,useHostTransitionStatus:$e,useFormState:$e,useActionState:$e,useOptimistic:$e,useMemoCache:$e,useCacheRefresh:$e},Hh={readContext:dt,use:jo,useCallback:function(e,t){return vt().memoizedState=[e,t===void 0?null:t],e},useContext:dt,useEffect:Sh,useImperativeHandle:function(e,t,n){n=n!=null?n.concat([e]):null,Ho(4194308,4,Ah.bind(null,t,e),n)},useLayoutEffect:function(e,t){return Ho(4194308,4,e,t)},useInsertionEffect:function(e,t){Ho(4,2,e,t)},useMemo:function(e,t){var n=vt();t=t===void 0?null:t;var l=e();if(Tr){_n(!0);try{e()}finally{_n(!1)}}return n.memoizedState=[l,t],l},useReducer:function(e,t,n){var l=vt();if(n!==void 0){var u=n(t);if(Tr){_n(!0);try{n(t)}finally{_n(!1)}}}else u=t;return l.memoizedState=l.baseState=u,e={pending:null,lanes:0,dispatch:null,lastRenderedReducer:e,lastRenderedState:u},l.queue=e,e=e.dispatch=ov.bind(null,Se,e),[l.memoizedState,e]},useRef:function(e){var t=vt();return e={current:e},t.memoizedState=e},useState:function(e){e=gs(e);var t=e.queue,n=jh.bind(null,Se,t);return t.dispatch=n,[e.memoizedState,n]},useDebugValue:ys,useDeferredValue:function(e,t){var n=vt();return bs(n,e,t)},useTransition:function(){var e=gs(!1);return e=Nh.bind(null,Se,e.queue,!0,!1),vt().memoizedState=e,[!1,e]},useSyncExternalStore:function(e,t,n){var l=Se,u=vt();if(Ne){if(n===void 0)throw Error(o(407));n=n()}else{if(n=t(),qe===null)throw Error(o(349));(Oe&124)!==0||oh(l,t,n)}u.memoizedState=n;var c={value:n,getSnapshot:t};return u.queue=c,Sh(uh.bind(null,l,c,e),[e]),l.flags|=2048,ul(9,Bo(),ih.bind(null,l,c,n,t),null),n},useId:function(){var e=vt(),t=qe.identifierPrefix;if(Ne){var n=gn,l=mn;n=(l&~(1<<32-Ot(l)-1)).toString(32)+n,t="«"+t+"R"+n,n=Uo++,0<n&&(t+="H"+n.toString(32)),t+="»"}else n=ev++,t="«"+t+"r"+n.toString(32)+"»";return e.memoizedState=t},useHostTransitionStatus:xs,useFormState:yh,useActionState:yh,useOptimistic:function(e){var t=vt();t.memoizedState=t.baseState=e;var n={pending:null,lanes:0,dispatch:null,lastRenderedReducer:null,lastRenderedState:null};return t.queue=n,t=ws.bind(null,Se,!0,n),n.dispatch=t,[e,t]},useMemoCache:ds,useCacheRefresh:function(){return vt().memoizedState=av.bind(null,Se)}},qh={readContext:dt,use:jo,useCallback:Oh,useContext:dt,useEffect:Eh,useImperativeHandle:kh,useInsertionEffect:Rh,useLayoutEffect:Th,useMemo:Ch,useReducer:Lo,useRef:wh,useState:function(){return Lo(bn)},useDebugValue:ys,useDeferredValue:function(e,t){var n=Fe();return Mh(n,De.memoizedState,e,t)},useTransition:function(){var e=Lo(bn)[0],t=Fe().memoizedState;return[typeof e=="boolean"?e:ya(e),t]},useSyncExternalStore:ah,useId:Dh,useHostTransitionStatus:xs,useFormState:bh,useActionState:bh,useOptimistic:function(e,t){var n=Fe();return fh(n,De,e,t)},useMemoCache:ds,useCacheRefresh:Uh},iv={readContext:dt,use:jo,useCallback:Oh,useContext:dt,useEffect:Eh,useImperativeHandle:kh,useInsertionEffect:Rh,useLayoutEffect:Th,useMemo:Ch,useReducer:ms,useRef:wh,useState:function(){return ms(bn)},useDebugValue:ys,useDeferredValue:function(e,t){var n=Fe();return De===null?bs(n,e,t):Mh(n,De.memoizedState,e,t)},useTransition:function(){var e=ms(bn)[0],t=Fe().memoizedState;return[typeof e=="boolean"?e:ya(e),t]},useSyncExternalStore:ah,useId:Dh,useHostTransitionStatus:xs,useFormState:xh,useActionState:xh,useOptimistic:function(e,t){var n=Fe();return De!==null?fh(n,De,e,t):(n.baseState=e,[e,n.queue.dispatch])},useMemoCache:ds,useCacheRefresh:Uh},sl=null,xa=0;function Yo(e){var t=xa;return xa+=1,sl===null&&(sl=[]),Fd(sl,e,t)}function wa(e,t){t=t.props.ref,e.ref=t!==void 0?t:null}function Vo(e,t){throw t.$$typeof===v?Error(o(525)):(e=Object.prototype.toString.call(t),Error(o(31,e==="[object Object]"?"object with keys {"+Object.keys(t).join(", ")+"}":e)))}function Gh(e){var t=e._init;return t(e._payload)}function Yh(e){function t(z,C){if(e){var D=z.deletions;D===null?(z.deletions=[C],z.flags|=16):D.push(C)}}function n(z,C){if(!e)return null;for(;C!==null;)t(z,C),C=C.sibling;return null}function l(z){for(var C=new Map;z!==null;)z.key!==null?C.set(z.key,z):C.set(z.index,z),z=z.sibling;return C}function u(z,C){return z=hn(z,C),z.index=0,z.sibling=null,z}function c(z,C,D){return z.index=D,e?(D=z.alternate,D!==null?(D=D.index,D<C?(z.flags|=67108866,C):D):(z.flags|=67108866,C)):(z.flags|=1048576,C)}function p(z){return e&&z.alternate===null&&(z.flags|=67108866),z}function b(z,C,D,Z){return C===null||C.tag!==6?(C=Yu(D,z.mode,Z),C.return=z,C):(C=u(C,D),C.return=z,C)}function A(z,C,D,Z){var ie=D.type;return ie===x?Q(z,C,D.props.children,Z,D.key):C!==null&&(C.elementType===ie||typeof ie=="object"&&ie!==null&&ie.$$typeof===fe&&Gh(ie)===C.type)?(C=u(C,D.props),wa(C,D),C.return=z,C):(C=To(D.type,D.key,D.props,null,z.mode,Z),wa(C,D),C.return=z,C)}function U(z,C,D,Z){return C===null||C.tag!==4||C.stateNode.containerInfo!==D.containerInfo||C.stateNode.implementation!==D.implementation?(C=Vu(D,z.mode,Z),C.return=z,C):(C=u(C,D.children||[]),C.return=z,C)}function Q(z,C,D,Z,ie){return C===null||C.tag!==7?(C=yr(D,z.mode,Z,ie),C.return=z,C):(C=u(C,D),C.return=z,C)}function J(z,C,D){if(typeof C=="string"&&C!==""||typeof C=="number"||typeof C=="bigint")return C=Yu(""+C,z.mode,D),C.return=z,C;if(typeof C=="object"&&C!==null){switch(C.$$typeof){case w:return D=To(C.type,C.key,C.props,null,z.mode,D),wa(D,C),D.return=z,D;case S:return C=Vu(C,z.mode,D),C.return=z,C;case fe:var Z=C._init;return C=Z(C._payload),J(z,C,D)}if(I(C)||te(C))return C=yr(C,z.mode,D,null),C.return=z,C;if(typeof C.then=="function")return J(z,Yo(C),D);if(C.$$typeof===H)return J(z,Co(z,C),D);Vo(z,C)}return null}function j(z,C,D,Z){var ie=C!==null?C.key:null;if(typeof D=="string"&&D!==""||typeof D=="number"||typeof D=="bigint")return ie!==null?null:b(z,C,""+D,Z);if(typeof D=="object"&&D!==null){switch(D.$$typeof){case w:return D.key===ie?A(z,C,D,Z):null;case S:return D.key===ie?U(z,C,D,Z):null;case fe:return ie=D._init,D=ie(D._payload),j(z,C,D,Z)}if(I(D)||te(D))return ie!==null?null:Q(z,C,D,Z,null);if(typeof D.then=="function")return j(z,C,Yo(D),Z);if(D.$$typeof===H)return j(z,C,Co(z,D),Z);Vo(z,D)}return null}function B(z,C,D,Z,ie){if(typeof Z=="string"&&Z!==""||typeof Z=="number"||typeof Z=="bigint")return z=z.get(D)||null,b(C,z,""+Z,ie);if(typeof Z=="object"&&Z!==null){switch(Z.$$typeof){case w:return z=z.get(Z.key===null?D:Z.key)||null,A(C,z,Z,ie);case S:return z=z.get(Z.key===null?D:Z.key)||null,U(C,z,Z,ie);case fe:var Ee=Z._init;return Z=Ee(Z._payload),B(z,C,D,Z,ie)}if(I(Z)||te(Z))return z=z.get(D)||null,Q(C,z,Z,ie,null);if(typeof Z.then=="function")return B(z,C,D,Yo(Z),ie);if(Z.$$typeof===H)return B(z,C,D,Co(C,Z),ie);Vo(C,Z)}return null}function ye(z,C,D,Z){for(var ie=null,Ee=null,ce=C,ge=C=0,rt=null;ce!==null&&ge<D.length;ge++){ce.index>ge?(rt=ce,ce=null):rt=ce.sibling;var Me=j(z,ce,D[ge],Z);if(Me===null){ce===null&&(ce=rt);break}e&&ce&&Me.alternate===null&&t(z,ce),C=c(Me,C,ge),Ee===null?ie=Me:Ee.sibling=Me,Ee=Me,ce=rt}if(ge===D.length)return n(z,ce),Ne&&vr(z,ge),ie;if(ce===null){for(;ge<D.length;ge++)ce=J(z,D[ge],Z),ce!==null&&(C=c(ce,C,ge),Ee===null?ie=ce:Ee.sibling=ce,Ee=ce);return Ne&&vr(z,ge),ie}for(ce=l(ce);ge<D.length;ge++)rt=B(ce,z,ge,D[ge],Z),rt!==null&&(e&&rt.alternate!==null&&ce.delete(rt.key===null?ge:rt.key),C=c(rt,C,ge),Ee===null?ie=rt:Ee.sibling=rt,Ee=rt);return e&&ce.forEach(function(nr){return t(z,nr)}),Ne&&vr(z,ge),ie}function he(z,C,D,Z){if(D==null)throw Error(o(151));for(var ie=null,Ee=null,ce=C,ge=C=0,rt=null,Me=D.next();ce!==null&&!Me.done;ge++,Me=D.next()){ce.index>ge?(rt=ce,ce=null):rt=ce.sibling;var nr=j(z,ce,Me.value,Z);if(nr===null){ce===null&&(ce=rt);break}e&&ce&&nr.alternate===null&&t(z,ce),C=c(nr,C,ge),Ee===null?ie=nr:Ee.sibling=nr,Ee=nr,ce=rt}if(Me.done)return n(z,ce),Ne&&vr(z,ge),ie;if(ce===null){for(;!Me.done;ge++,Me=D.next())Me=J(z,Me.value,Z),Me!==null&&(C=c(Me,C,ge),Ee===null?ie=Me:Ee.sibling=Me,Ee=Me);return Ne&&vr(z,ge),ie}for(ce=l(ce);!Me.done;ge++,Me=D.next())Me=B(ce,z,ge,Me.value,Z),Me!==null&&(e&&Me.alternate!==null&&ce.delete(Me.key===null?ge:Me.key),C=c(Me,C,ge),Ee===null?ie=Me:Ee.sibling=Me,Ee=Me);return e&&ce.forEach(function(u1){return t(z,u1)}),Ne&&vr(z,ge),ie}function je(z,C,D,Z){if(typeof D=="object"&&D!==null&&D.type===x&&D.key===null&&(D=D.props.children),typeof D=="object"&&D!==null){switch(D.$$typeof){case w:e:{for(var ie=D.key;C!==null;){if(C.key===ie){if(ie=D.type,ie===x){if(C.tag===7){n(z,C.sibling),Z=u(C,D.props.children),Z.return=z,z=Z;break e}}else if(C.elementType===ie||typeof ie=="object"&&ie!==null&&ie.$$typeof===fe&&Gh(ie)===C.type){n(z,C.sibling),Z=u(C,D.props),wa(Z,D),Z.return=z,z=Z;break e}n(z,C);break}else t(z,C);C=C.sibling}D.type===x?(Z=yr(D.props.children,z.mode,Z,D.key),Z.return=z,z=Z):(Z=To(D.type,D.key,D.props,null,z.mode,Z),wa(Z,D),Z.return=z,z=Z)}return p(z);case S:e:{for(ie=D.key;C!==null;){if(C.key===ie)if(C.tag===4&&C.stateNode.containerInfo===D.containerInfo&&C.stateNode.implementation===D.implementation){n(z,C.sibling),Z=u(C,D.children||[]),Z.return=z,z=Z;break e}else{n(z,C);break}else t(z,C);C=C.sibling}Z=Vu(D,z.mode,Z),Z.return=z,z=Z}return p(z);case fe:return ie=D._init,D=ie(D._payload),je(z,C,D,Z)}if(I(D))return ye(z,C,D,Z);if(te(D)){if(ie=te(D),typeof ie!="function")throw Error(o(150));return D=ie.call(D),he(z,C,D,Z)}if(typeof D.then=="function")return je(z,C,Yo(D),Z);if(D.$$typeof===H)return je(z,C,Co(z,D),Z);Vo(z,D)}return typeof D=="string"&&D!==""||typeof D=="number"||typeof D=="bigint"?(D=""+D,C!==null&&C.tag===6?(n(z,C.sibling),Z=u(C,D),Z.return=z,z=Z):(n(z,C),Z=Yu(D,z.mode,Z),Z.return=z,z=Z),p(z)):n(z,C)}return function(z,C,D,Z){try{xa=0;var ie=je(z,C,D,Z);return sl=null,ie}catch(ce){if(ce===fa||ce===No)throw ce;var Ee=Mt(29,ce,null,z.mode);return Ee.lanes=Z,Ee.return=z,Ee}finally{}}}var cl=Yh(!0),Vh=Yh(!1),Vt=_(null),nn=null;function Gn(e){var t=e.alternate;F(Ie,Ie.current&1),F(Vt,e),nn===null&&(t===null||al.current!==null||t.memoizedState!==null)&&(nn=e)}function Xh(e){if(e.tag===22){if(F(Ie,Ie.current),F(Vt,e),nn===null){var t=e.alternate;t!==null&&t.memoizedState!==null&&(nn=e)}}else Yn()}function Yn(){F(Ie,Ie.current),F(Vt,Vt.current)}function vn(e){P(Vt),nn===e&&(nn=null),P(Ie)}var Ie=_(0);function Xo(e){for(var t=e;t!==null;){if(t.tag===13){var n=t.memoizedState;if(n!==null&&(n=n.dehydrated,n===null||n.data==="$?"||fc(n)))return t}else if(t.tag===19&&t.memoizedProps.revealOrder!==void 0){if((t.flags&128)!==0)return t}else if(t.child!==null){t.child.return=t,t=t.child;continue}if(t===e)break;for(;t.sibling===null;){if(t.return===null||t.return===e)return null;t=t.return}t.sibling.return=t.return,t=t.sibling}return null}function Ss(e,t,n,l){t=e.memoizedState,n=n(l,t),n=n==null?t:y({},t,n),e.memoizedState=n,e.lanes===0&&(e.updateQueue.baseState=n)}var Es={enqueueSetState:function(e,t,n){e=e._reactInternals;var l=Dt(),u=Bn(l);u.payload=t,n!=null&&(u.callback=n),t=Hn(e,u,l),t!==null&&(Ut(t,e,l),ha(t,e,l))},enqueueReplaceState:function(e,t,n){e=e._reactInternals;var l=Dt(),u=Bn(l);u.tag=1,u.payload=t,n!=null&&(u.callback=n),t=Hn(e,u,l),t!==null&&(Ut(t,e,l),ha(t,e,l))},enqueueForceUpdate:function(e,t){e=e._reactInternals;var n=Dt(),l=Bn(n);l.tag=2,t!=null&&(l.callback=t),t=Hn(e,l,n),t!==null&&(Ut(t,e,n),ha(t,e,n))}};function Qh(e,t,n,l,u,c,p){return e=e.stateNode,typeof e.shouldComponentUpdate=="function"?e.shouldComponentUpdate(l,c,p):t.prototype&&t.prototype.isPureReactComponent?!ra(n,l)||!ra(u,c):!0}function Zh(e,t,n,l){e=t.state,typeof t.componentWillReceiveProps=="function"&&t.componentWillReceiveProps(n,l),typeof t.UNSAFE_componentWillReceiveProps=="function"&&t.UNSAFE_componentWillReceiveProps(n,l),t.state!==e&&Es.enqueueReplaceState(t,t.state,null)}function Ar(e,t){var n=t;if("ref"in t){n={};for(var l in t)l!=="ref"&&(n[l]=t[l])}if(e=e.defaultProps){n===t&&(n=y({},n));for(var u in e)n[u]===void 0&&(n[u]=e[u])}return n}var Qo=typeof reportError=="function"?reportError:function(e){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var t=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof e=="object"&&e!==null&&typeof e.message=="string"?String(e.message):String(e),error:e});if(!window.dispatchEvent(t))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",e);return}console.error(e)};function Kh(e){Qo(e)}function $h(e){console.error(e)}function Jh(e){Qo(e)}function Zo(e,t){try{var n=e.onUncaughtError;n(t.value,{componentStack:t.stack})}catch(l){setTimeout(function(){throw l})}}function Ph(e,t,n){try{var l=e.onCaughtError;l(n.value,{componentStack:n.stack,errorBoundary:t.tag===1?t.stateNode:null})}catch(u){setTimeout(function(){throw u})}}function Rs(e,t,n){return n=Bn(n),n.tag=3,n.payload={element:null},n.callback=function(){Zo(e,t)},n}function Fh(e){return e=Bn(e),e.tag=3,e}function Wh(e,t,n,l){var u=n.type.getDerivedStateFromError;if(typeof u=="function"){var c=l.value;e.payload=function(){return u(c)},e.callback=function(){Ph(t,n,l)}}var p=n.stateNode;p!==null&&typeof p.componentDidCatch=="function"&&(e.callback=function(){Ph(t,n,l),typeof u!="function"&&($n===null?$n=new Set([this]):$n.add(this));var b=l.stack;this.componentDidCatch(l.value,{componentStack:b!==null?b:""})})}function uv(e,t,n,l,u){if(n.flags|=32768,l!==null&&typeof l=="object"&&typeof l.then=="function"){if(t=n.alternate,t!==null&&ua(t,n,u,!0),n=Vt.current,n!==null){switch(n.tag){case 13:return nn===null?$s():n.alternate===null&&Ke===0&&(Ke=3),n.flags&=-257,n.flags|=65536,n.lanes=u,l===Iu?n.flags|=16384:(t=n.updateQueue,t===null?n.updateQueue=new Set([l]):t.add(l),Ps(e,l,u)),!1;case 22:return n.flags|=65536,l===Iu?n.flags|=16384:(t=n.updateQueue,t===null?(t={transitions:null,markerInstances:null,retryQueue:new Set([l])},n.updateQueue=t):(n=t.retryQueue,n===null?t.retryQueue=new Set([l]):n.add(l)),Ps(e,l,u)),!1}throw Error(o(435,n.tag))}return Ps(e,l,u),$s(),!1}if(Ne)return t=Vt.current,t!==null?((t.flags&65536)===0&&(t.flags|=256),t.flags|=65536,t.lanes=u,l!==Zu&&(e=Error(o(422),{cause:l}),ia(Ht(e,n)))):(l!==Zu&&(t=Error(o(423),{cause:l}),ia(Ht(t,n))),e=e.current.alternate,e.flags|=65536,u&=-u,e.lanes|=u,l=Ht(l,n),u=Rs(e.stateNode,l,u),ns(e,u),Ke!==4&&(Ke=2)),!1;var c=Error(o(520),{cause:l});if(c=Ht(c,n),Oa===null?Oa=[c]:Oa.push(c),Ke!==4&&(Ke=2),t===null)return!0;l=Ht(l,n),n=t;do{switch(n.tag){case 3:return n.flags|=65536,e=u&-u,n.lanes|=e,e=Rs(n.stateNode,l,e),ns(n,e),!1;case 1:if(t=n.type,c=n.stateNode,(n.flags&128)===0&&(typeof t.getDerivedStateFromError=="function"||c!==null&&typeof c.componentDidCatch=="function"&&($n===null||!$n.has(c))))return n.flags|=65536,u&=-u,n.lanes|=u,u=Fh(u),Wh(u,e,n,l),ns(n,u),!1}n=n.return}while(n!==null);return!1}var Ih=Error(o(461)),tt=!1;function ut(e,t,n,l){t.child=e===null?Vh(t,null,n,l):cl(t,e.child,n,l)}function e0(e,t,n,l,u){n=n.render;var c=t.ref;if("ref"in l){var p={};for(var b in l)b!=="ref"&&(p[b]=l[b])}else p=l;return Er(t),l=is(e,t,n,p,c,u),b=us(),e!==null&&!tt?(ss(e,t,u),xn(e,t,u)):(Ne&&b&&Xu(t),t.flags|=1,ut(e,t,l,u),t.child)}function t0(e,t,n,l,u){if(e===null){var c=n.type;return typeof c=="function"&&!Gu(c)&&c.defaultProps===void 0&&n.compare===null?(t.tag=15,t.type=c,n0(e,t,c,l,u)):(e=To(n.type,null,l,t,t.mode,u),e.ref=t.ref,e.return=t,t.child=e)}if(c=e.child,!zs(e,u)){var p=c.memoizedProps;if(n=n.compare,n=n!==null?n:ra,n(p,l)&&e.ref===t.ref)return xn(e,t,u)}return t.flags|=1,e=hn(c,l),e.ref=t.ref,e.return=t,t.child=e}function n0(e,t,n,l,u){if(e!==null){var c=e.memoizedProps;if(ra(c,l)&&e.ref===t.ref)if(tt=!1,t.pendingProps=l=c,zs(e,u))(e.flags&131072)!==0&&(tt=!0);else return t.lanes=e.lanes,xn(e,t,u)}return Ts(e,t,n,l,u)}function r0(e,t,n){var l=t.pendingProps,u=l.children,c=e!==null?e.memoizedState:null;if(l.mode==="hidden"){if((t.flags&128)!==0){if(l=c!==null?c.baseLanes|n:n,e!==null){for(u=t.child=e.child,c=0;u!==null;)c=c|u.lanes|u.childLanes,u=u.sibling;t.childLanes=c&~l}else t.childLanes=0,t.child=null;return l0(e,t,l,n)}if((n&536870912)!==0)t.memoizedState={baseLanes:0,cachePool:null},e!==null&&Mo(t,c!==null?c.cachePool:null),c!==null?nh(t,c):ls(),Xh(t);else return t.lanes=t.childLanes=536870912,l0(e,t,c!==null?c.baseLanes|n:n,n)}else c!==null?(Mo(t,c.cachePool),nh(t,c),Yn(),t.memoizedState=null):(e!==null&&Mo(t,null),ls(),Yn());return ut(e,t,u,n),t.child}function l0(e,t,n,l){var u=Wu();return u=u===null?null:{parent:We._currentValue,pool:u},t.memoizedState={baseLanes:n,cachePool:u},e!==null&&Mo(t,null),ls(),Xh(t),e!==null&&ua(e,t,l,!0),null}function Ko(e,t){var n=t.ref;if(n===null)e!==null&&e.ref!==null&&(t.flags|=4194816);else{if(typeof n!="function"&&typeof n!="object")throw Error(o(284));(e===null||e.ref!==n)&&(t.flags|=4194816)}}function Ts(e,t,n,l,u){return Er(t),n=is(e,t,n,l,void 0,u),l=us(),e!==null&&!tt?(ss(e,t,u),xn(e,t,u)):(Ne&&l&&Xu(t),t.flags|=1,ut(e,t,n,u),t.child)}function a0(e,t,n,l,u,c){return Er(t),t.updateQueue=null,n=lh(t,l,n,u),rh(e),l=us(),e!==null&&!tt?(ss(e,t,c),xn(e,t,c)):(Ne&&l&&Xu(t),t.flags|=1,ut(e,t,n,c),t.child)}function o0(e,t,n,l,u){if(Er(t),t.stateNode===null){var c=el,p=n.contextType;typeof p=="object"&&p!==null&&(c=dt(p)),c=new n(l,c),t.memoizedState=c.state!==null&&c.state!==void 0?c.state:null,c.updater=Es,t.stateNode=c,c._reactInternals=t,c=t.stateNode,c.props=l,c.state=t.memoizedState,c.refs={},es(t),p=n.contextType,c.context=typeof p=="object"&&p!==null?dt(p):el,c.state=t.memoizedState,p=n.getDerivedStateFromProps,typeof p=="function"&&(Ss(t,n,p,l),c.state=t.memoizedState),typeof n.getDerivedStateFromProps=="function"||typeof c.getSnapshotBeforeUpdate=="function"||typeof c.UNSAFE_componentWillMount!="function"&&typeof c.componentWillMount!="function"||(p=c.state,typeof c.componentWillMount=="function"&&c.componentWillMount(),typeof c.UNSAFE_componentWillMount=="function"&&c.UNSAFE_componentWillMount(),p!==c.state&&Es.enqueueReplaceState(c,c.state,null),ga(t,l,c,u),ma(),c.state=t.memoizedState),typeof c.componentDidMount=="function"&&(t.flags|=4194308),l=!0}else if(e===null){c=t.stateNode;var b=t.memoizedProps,A=Ar(n,b);c.props=A;var U=c.context,Q=n.contextType;p=el,typeof Q=="object"&&Q!==null&&(p=dt(Q));var J=n.getDerivedStateFromProps;Q=typeof J=="function"||typeof c.getSnapshotBeforeUpdate=="function",b=t.pendingProps!==b,Q||typeof c.UNSAFE_componentWillReceiveProps!="function"&&typeof c.componentWillReceiveProps!="function"||(b||U!==p)&&Zh(t,c,l,p),Ln=!1;var j=t.memoizedState;c.state=j,ga(t,l,c,u),ma(),U=t.memoizedState,b||j!==U||Ln?(typeof J=="function"&&(Ss(t,n,J,l),U=t.memoizedState),(A=Ln||Qh(t,n,A,l,j,U,p))?(Q||typeof c.UNSAFE_componentWillMount!="function"&&typeof c.componentWillMount!="function"||(typeof c.componentWillMount=="function"&&c.componentWillMount(),typeof c.UNSAFE_componentWillMount=="function"&&c.UNSAFE_componentWillMount()),typeof c.componentDidMount=="function"&&(t.flags|=4194308)):(typeof c.componentDidMount=="function"&&(t.flags|=4194308),t.memoizedProps=l,t.memoizedState=U),c.props=l,c.state=U,c.context=p,l=A):(typeof c.componentDidMount=="function"&&(t.flags|=4194308),l=!1)}else{c=t.stateNode,ts(e,t),p=t.memoizedProps,Q=Ar(n,p),c.props=Q,J=t.pendingProps,j=c.context,U=n.contextType,A=el,typeof U=="object"&&U!==null&&(A=dt(U)),b=n.getDerivedStateFromProps,(U=typeof b=="function"||typeof c.getSnapshotBeforeUpdate=="function")||typeof c.UNSAFE_componentWillReceiveProps!="function"&&typeof c.componentWillReceiveProps!="function"||(p!==J||j!==A)&&Zh(t,c,l,A),Ln=!1,j=t.memoizedState,c.state=j,ga(t,l,c,u),ma();var B=t.memoizedState;p!==J||j!==B||Ln||e!==null&&e.dependencies!==null&&Oo(e.dependencies)?(typeof b=="function"&&(Ss(t,n,b,l),B=t.memoizedState),(Q=Ln||Qh(t,n,Q,l,j,B,A)||e!==null&&e.dependencies!==null&&Oo(e.dependencies))?(U||typeof c.UNSAFE_componentWillUpdate!="function"&&typeof c.componentWillUpdate!="function"||(typeof c.componentWillUpdate=="function"&&c.componentWillUpdate(l,B,A),typeof c.UNSAFE_componentWillUpdate=="function"&&c.UNSAFE_componentWillUpdate(l,B,A)),typeof c.componentDidUpdate=="function"&&(t.flags|=4),typeof c.getSnapshotBeforeUpdate=="function"&&(t.flags|=1024)):(typeof c.componentDidUpdate!="function"||p===e.memoizedProps&&j===e.memoizedState||(t.flags|=4),typeof c.getSnapshotBeforeUpdate!="function"||p===e.memoizedProps&&j===e.memoizedState||(t.flags|=1024),t.memoizedProps=l,t.memoizedState=B),c.props=l,c.state=B,c.context=A,l=Q):(typeof c.componentDidUpdate!="function"||p===e.memoizedProps&&j===e.memoizedState||(t.flags|=4),typeof c.getSnapshotBeforeUpdate!="function"||p===e.memoizedProps&&j===e.memoizedState||(t.flags|=1024),l=!1)}return c=l,Ko(e,t),l=(t.flags&128)!==0,c||l?(c=t.stateNode,n=l&&typeof n.getDerivedStateFromError!="function"?null:c.render(),t.flags|=1,e!==null&&l?(t.child=cl(t,e.child,null,u),t.child=cl(t,null,n,u)):ut(e,t,n,u),t.memoizedState=c.state,e=t.child):e=xn(e,t,u),e}function i0(e,t,n,l){return oa(),t.flags|=256,ut(e,t,n,l),t.child}var As={dehydrated:null,treeContext:null,retryLane:0,hydrationErrors:null};function ks(e){return{baseLanes:e,cachePool:$d()}}function Os(e,t,n){return e=e!==null?e.childLanes&~n:0,t&&(e|=Xt),e}function u0(e,t,n){var l=t.pendingProps,u=!1,c=(t.flags&128)!==0,p;if((p=c)||(p=e!==null&&e.memoizedState===null?!1:(Ie.current&2)!==0),p&&(u=!0,t.flags&=-129),p=(t.flags&32)!==0,t.flags&=-33,e===null){if(Ne){if(u?Gn(t):Yn(),Ne){var b=Ze,A;if(A=b){e:{for(A=b,b=tn;A.nodeType!==8;){if(!b){b=null;break e}if(A=Jt(A.nextSibling),A===null){b=null;break e}}b=A}b!==null?(t.memoizedState={dehydrated:b,treeContext:br!==null?{id:mn,overflow:gn}:null,retryLane:536870912,hydrationErrors:null},A=Mt(18,null,null,0),A.stateNode=b,A.return=t,t.child=A,gt=t,Ze=null,A=!0):A=!1}A||wr(t)}if(b=t.memoizedState,b!==null&&(b=b.dehydrated,b!==null))return fc(b)?t.lanes=32:t.lanes=536870912,null;vn(t)}return b=l.children,l=l.fallback,u?(Yn(),u=t.mode,b=$o({mode:"hidden",children:b},u),l=yr(l,u,n,null),b.return=t,l.return=t,b.sibling=l,t.child=b,u=t.child,u.memoizedState=ks(n),u.childLanes=Os(e,p,n),t.memoizedState=As,l):(Gn(t),Cs(t,b))}if(A=e.memoizedState,A!==null&&(b=A.dehydrated,b!==null)){if(c)t.flags&256?(Gn(t),t.flags&=-257,t=Ms(e,t,n)):t.memoizedState!==null?(Yn(),t.child=e.child,t.flags|=128,t=null):(Yn(),u=l.fallback,b=t.mode,l=$o({mode:"visible",children:l.children},b),u=yr(u,b,n,null),u.flags|=2,l.return=t,u.return=t,l.sibling=u,t.child=l,cl(t,e.child,null,n),l=t.child,l.memoizedState=ks(n),l.childLanes=Os(e,p,n),t.memoizedState=As,t=u);else if(Gn(t),fc(b)){if(p=b.nextSibling&&b.nextSibling.dataset,p)var U=p.dgst;p=U,l=Error(o(419)),l.stack="",l.digest=p,ia({value:l,source:null,stack:null}),t=Ms(e,t,n)}else if(tt||ua(e,t,n,!1),p=(n&e.childLanes)!==0,tt||p){if(p=qe,p!==null&&(l=n&-n,l=(l&42)!==0?1:du(l),l=(l&(p.suspendedLanes|n))!==0?0:l,l!==0&&l!==A.retryLane))throw A.retryLane=l,Ir(e,l),Ut(p,e,l),Ih;b.data==="$?"||$s(),t=Ms(e,t,n)}else b.data==="$?"?(t.flags|=192,t.child=e.child,t=null):(e=A.treeContext,Ze=Jt(b.nextSibling),gt=t,Ne=!0,xr=null,tn=!1,e!==null&&(Gt[Yt++]=mn,Gt[Yt++]=gn,Gt[Yt++]=br,mn=e.id,gn=e.overflow,br=t),t=Cs(t,l.children),t.flags|=4096);return t}return u?(Yn(),u=l.fallback,b=t.mode,A=e.child,U=A.sibling,l=hn(A,{mode:"hidden",children:l.children}),l.subtreeFlags=A.subtreeFlags&65011712,U!==null?u=hn(U,u):(u=yr(u,b,n,null),u.flags|=2),u.return=t,l.return=t,l.sibling=u,t.child=l,l=u,u=t.child,b=e.child.memoizedState,b===null?b=ks(n):(A=b.cachePool,A!==null?(U=We._currentValue,A=A.parent!==U?{parent:U,pool:U}:A):A=$d(),b={baseLanes:b.baseLanes|n,cachePool:A}),u.memoizedState=b,u.childLanes=Os(e,p,n),t.memoizedState=As,l):(Gn(t),n=e.child,e=n.sibling,n=hn(n,{mode:"visible",children:l.children}),n.return=t,n.sibling=null,e!==null&&(p=t.deletions,p===null?(t.deletions=[e],t.flags|=16):p.push(e)),t.child=n,t.memoizedState=null,n)}function Cs(e,t){return t=$o({mode:"visible",children:t},e.mode),t.return=e,e.child=t}function $o(e,t){return e=Mt(22,e,null,t),e.lanes=0,e.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null},e}function Ms(e,t,n){return cl(t,e.child,null,n),e=Cs(t,t.pendingProps.children),e.flags|=2,t.memoizedState=null,e}function s0(e,t,n){e.lanes|=t;var l=e.alternate;l!==null&&(l.lanes|=t),$u(e.return,t,n)}function Ns(e,t,n,l,u){var c=e.memoizedState;c===null?e.memoizedState={isBackwards:t,rendering:null,renderingStartTime:0,last:l,tail:n,tailMode:u}:(c.isBackwards=t,c.rendering=null,c.renderingStartTime=0,c.last=l,c.tail=n,c.tailMode=u)}function c0(e,t,n){var l=t.pendingProps,u=l.revealOrder,c=l.tail;if(ut(e,t,l.children,n),l=Ie.current,(l&2)!==0)l=l&1|2,t.flags|=128;else{if(e!==null&&(e.flags&128)!==0)e:for(e=t.child;e!==null;){if(e.tag===13)e.memoizedState!==null&&s0(e,n,t);else if(e.tag===19)s0(e,n,t);else if(e.child!==null){e.child.return=e,e=e.child;continue}if(e===t)break e;for(;e.sibling===null;){if(e.return===null||e.return===t)break e;e=e.return}e.sibling.return=e.return,e=e.sibling}l&=1}switch(F(Ie,l),u){case"forwards":for(n=t.child,u=null;n!==null;)e=n.alternate,e!==null&&Xo(e)===null&&(u=n),n=n.sibling;n=u,n===null?(u=t.child,t.child=null):(u=n.sibling,n.sibling=null),Ns(t,!1,u,n,c);break;case"backwards":for(n=null,u=t.child,t.child=null;u!==null;){if(e=u.alternate,e!==null&&Xo(e)===null){t.child=u;break}e=u.sibling,u.sibling=n,n=u,u=e}Ns(t,!0,n,null,c);break;case"together":Ns(t,!1,null,null,void 0);break;default:t.memoizedState=null}return t.child}function xn(e,t,n){if(e!==null&&(t.dependencies=e.dependencies),Kn|=t.lanes,(n&t.childLanes)===0)if(e!==null){if(ua(e,t,n,!1),(n&t.childLanes)===0)return null}else return null;if(e!==null&&t.child!==e.child)throw Error(o(153));if(t.child!==null){for(e=t.child,n=hn(e,e.pendingProps),t.child=n,n.return=t;e.sibling!==null;)e=e.sibling,n=n.sibling=hn(e,e.pendingProps),n.return=t;n.sibling=null}return t.child}function zs(e,t){return(e.lanes&t)!==0?!0:(e=e.dependencies,!!(e!==null&&Oo(e)))}function sv(e,t,n){switch(t.tag){case 3:de(t,t.stateNode.containerInfo),jn(t,We,e.memoizedState.cache),oa();break;case 27:case 5:At(t);break;case 4:de(t,t.stateNode.containerInfo);break;case 10:jn(t,t.type,t.memoizedProps.value);break;case 13:var l=t.memoizedState;if(l!==null)return l.dehydrated!==null?(Gn(t),t.flags|=128,null):(n&t.child.childLanes)!==0?u0(e,t,n):(Gn(t),e=xn(e,t,n),e!==null?e.sibling:null);Gn(t);break;case 19:var u=(e.flags&128)!==0;if(l=(n&t.childLanes)!==0,l||(ua(e,t,n,!1),l=(n&t.childLanes)!==0),u){if(l)return c0(e,t,n);t.flags|=128}if(u=t.memoizedState,u!==null&&(u.rendering=null,u.tail=null,u.lastEffect=null),F(Ie,Ie.current),l)break;return null;case 22:case 23:return t.lanes=0,r0(e,t,n);case 24:jn(t,We,e.memoizedState.cache)}return xn(e,t,n)}function f0(e,t,n){if(e!==null)if(e.memoizedProps!==t.pendingProps)tt=!0;else{if(!zs(e,n)&&(t.flags&128)===0)return tt=!1,sv(e,t,n);tt=(e.flags&131072)!==0}else tt=!1,Ne&&(t.flags&1048576)!==0&&Gd(t,ko,t.index);switch(t.lanes=0,t.tag){case 16:e:{e=t.pendingProps;var l=t.elementType,u=l._init;if(l=u(l._payload),t.type=l,typeof l=="function")Gu(l)?(e=Ar(l,e),t.tag=1,t=o0(null,t,l,e,n)):(t.tag=0,t=Ts(null,t,l,e,n));else{if(l!=null){if(u=l.$$typeof,u===$){t.tag=11,t=e0(null,t,l,e,n);break e}else if(u===ee){t.tag=14,t=t0(null,t,l,e,n);break e}}throw t=X(l)||l,Error(o(306,t,""))}}return t;case 0:return Ts(e,t,t.type,t.pendingProps,n);case 1:return l=t.type,u=Ar(l,t.pendingProps),o0(e,t,l,u,n);case 3:e:{if(de(t,t.stateNode.containerInfo),e===null)throw Error(o(387));l=t.pendingProps;var c=t.memoizedState;u=c.element,ts(e,t),ga(t,l,null,n);var p=t.memoizedState;if(l=p.cache,jn(t,We,l),l!==c.cache&&Ju(t,[We],n,!0),ma(),l=p.element,c.isDehydrated)if(c={element:l,isDehydrated:!1,cache:p.cache},t.updateQueue.baseState=c,t.memoizedState=c,t.flags&256){t=i0(e,t,l,n);break e}else if(l!==u){u=Ht(Error(o(424)),t),ia(u),t=i0(e,t,l,n);break e}else{switch(e=t.stateNode.containerInfo,e.nodeType){case 9:e=e.body;break;default:e=e.nodeName==="HTML"?e.ownerDocument.body:e}for(Ze=Jt(e.firstChild),gt=t,Ne=!0,xr=null,tn=!0,n=Vh(t,null,l,n),t.child=n;n;)n.flags=n.flags&-3|4096,n=n.sibling}else{if(oa(),l===u){t=xn(e,t,n);break e}ut(e,t,l,n)}t=t.child}return t;case 26:return Ko(e,t),e===null?(n=gm(t.type,null,t.pendingProps,null))?t.memoizedState=n:Ne||(n=t.type,e=t.pendingProps,l=ui(ae.current).createElement(n),l[ft]=t,l[yt]=e,ct(l,n,e),et(l),t.stateNode=l):t.memoizedState=gm(t.type,e.memoizedProps,t.pendingProps,e.memoizedState),null;case 27:return At(t),e===null&&Ne&&(l=t.stateNode=dm(t.type,t.pendingProps,ae.current),gt=t,tn=!0,u=Ze,Fn(t.type)?(dc=u,Ze=Jt(l.firstChild)):Ze=u),ut(e,t,t.pendingProps.children,n),Ko(e,t),e===null&&(t.flags|=4194304),t.child;case 5:return e===null&&Ne&&((u=l=Ze)&&(l=Lv(l,t.type,t.pendingProps,tn),l!==null?(t.stateNode=l,gt=t,Ze=Jt(l.firstChild),tn=!1,u=!0):u=!1),u||wr(t)),At(t),u=t.type,c=t.pendingProps,p=e!==null?e.memoizedProps:null,l=c.children,uc(u,c)?l=null:p!==null&&uc(u,p)&&(t.flags|=32),t.memoizedState!==null&&(u=is(e,t,tv,null,null,n),La._currentValue=u),Ko(e,t),ut(e,t,l,n),t.child;case 6:return e===null&&Ne&&((e=n=Ze)&&(n=Bv(n,t.pendingProps,tn),n!==null?(t.stateNode=n,gt=t,Ze=null,e=!0):e=!1),e||wr(t)),null;case 13:return u0(e,t,n);case 4:return de(t,t.stateNode.containerInfo),l=t.pendingProps,e===null?t.child=cl(t,null,l,n):ut(e,t,l,n),t.child;case 11:return e0(e,t,t.type,t.pendingProps,n);case 7:return ut(e,t,t.pendingProps,n),t.child;case 8:return ut(e,t,t.pendingProps.children,n),t.child;case 12:return ut(e,t,t.pendingProps.children,n),t.child;case 10:return l=t.pendingProps,jn(t,t.type,l.value),ut(e,t,l.children,n),t.child;case 9:return u=t.type._context,l=t.pendingProps.children,Er(t),u=dt(u),l=l(u),t.flags|=1,ut(e,t,l,n),t.child;case 14:return t0(e,t,t.type,t.pendingProps,n);case 15:return n0(e,t,t.type,t.pendingProps,n);case 19:return c0(e,t,n);case 31:return l=t.pendingProps,n=t.mode,l={mode:l.mode,children:l.children},e===null?(n=$o(l,n),n.ref=t.ref,t.child=n,n.return=t,t=n):(n=hn(e.child,l),n.ref=t.ref,t.child=n,n.return=t,t=n),t;case 22:return r0(e,t,n);case 24:return Er(t),l=dt(We),e===null?(u=Wu(),u===null&&(u=qe,c=Pu(),u.pooledCache=c,c.refCount++,c!==null&&(u.pooledCacheLanes|=n),u=c),t.memoizedState={parent:l,cache:u},es(t),jn(t,We,u)):((e.lanes&n)!==0&&(ts(e,t),ga(t,null,null,n),ma()),u=e.memoizedState,c=t.memoizedState,u.parent!==l?(u={parent:l,cache:l},t.memoizedState=u,t.lanes===0&&(t.memoizedState=t.updateQueue.baseState=u),jn(t,We,l)):(l=c.cache,jn(t,We,l),l!==u.cache&&Ju(t,[We],n,!0))),ut(e,t,t.pendingProps.children,n),t.child;case 29:throw t.pendingProps}throw Error(o(156,t.tag))}function wn(e){e.flags|=4}function d0(e,t){if(t.type!=="stylesheet"||(t.state.loading&4)!==0)e.flags&=-16777217;else if(e.flags|=16777216,!xm(t)){if(t=Vt.current,t!==null&&((Oe&4194048)===Oe?nn!==null:(Oe&62914560)!==Oe&&(Oe&536870912)===0||t!==nn))throw da=Iu,Jd;e.flags|=8192}}function Jo(e,t){t!==null&&(e.flags|=4),e.flags&16384&&(t=e.tag!==22?Xf():536870912,e.lanes|=t,ml|=t)}function Sa(e,t){if(!Ne)switch(e.tailMode){case"hidden":t=e.tail;for(var n=null;t!==null;)t.alternate!==null&&(n=t),t=t.sibling;n===null?e.tail=null:n.sibling=null;break;case"collapsed":n=e.tail;for(var l=null;n!==null;)n.alternate!==null&&(l=n),n=n.sibling;l===null?t||e.tail===null?e.tail=null:e.tail.sibling=null:l.sibling=null}}function Qe(e){var t=e.alternate!==null&&e.alternate.child===e.child,n=0,l=0;if(t)for(var u=e.child;u!==null;)n|=u.lanes|u.childLanes,l|=u.subtreeFlags&65011712,l|=u.flags&65011712,u.return=e,u=u.sibling;else for(u=e.child;u!==null;)n|=u.lanes|u.childLanes,l|=u.subtreeFlags,l|=u.flags,u.return=e,u=u.sibling;return e.subtreeFlags|=l,e.childLanes=n,t}function cv(e,t,n){var l=t.pendingProps;switch(Qu(t),t.tag){case 31:case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return Qe(t),null;case 1:return Qe(t),null;case 3:return n=t.stateNode,l=null,e!==null&&(l=e.memoizedState.cache),t.memoizedState.cache!==l&&(t.flags|=2048),yn(We),Ce(),n.pendingContext&&(n.context=n.pendingContext,n.pendingContext=null),(e===null||e.child===null)&&(aa(t)?wn(t):e===null||e.memoizedState.isDehydrated&&(t.flags&256)===0||(t.flags|=1024,Xd())),Qe(t),null;case 26:return n=t.memoizedState,e===null?(wn(t),n!==null?(Qe(t),d0(t,n)):(Qe(t),t.flags&=-16777217)):n?n!==e.memoizedState?(wn(t),Qe(t),d0(t,n)):(Qe(t),t.flags&=-16777217):(e.memoizedProps!==l&&wn(t),Qe(t),t.flags&=-16777217),null;case 27:pt(t),n=ae.current;var u=t.type;if(e!==null&&t.stateNode!=null)e.memoizedProps!==l&&wn(t);else{if(!l){if(t.stateNode===null)throw Error(o(166));return Qe(t),null}e=oe.current,aa(t)?Yd(t):(e=dm(u,l,n),t.stateNode=e,wn(t))}return Qe(t),null;case 5:if(pt(t),n=t.type,e!==null&&t.stateNode!=null)e.memoizedProps!==l&&wn(t);else{if(!l){if(t.stateNode===null)throw Error(o(166));return Qe(t),null}if(e=oe.current,aa(t))Yd(t);else{switch(u=ui(ae.current),e){case 1:e=u.createElementNS("http://www.w3.org/2000/svg",n);break;case 2:e=u.createElementNS("http://www.w3.org/1998/Math/MathML",n);break;default:switch(n){case"svg":e=u.createElementNS("http://www.w3.org/2000/svg",n);break;case"math":e=u.createElementNS("http://www.w3.org/1998/Math/MathML",n);break;case"script":e=u.createElement("div"),e.innerHTML="<script><\/script>",e=e.removeChild(e.firstChild);break;case"select":e=typeof l.is=="string"?u.createElement("select",{is:l.is}):u.createElement("select"),l.multiple?e.multiple=!0:l.size&&(e.size=l.size);break;default:e=typeof l.is=="string"?u.createElement(n,{is:l.is}):u.createElement(n)}}e[ft]=t,e[yt]=l;e:for(u=t.child;u!==null;){if(u.tag===5||u.tag===6)e.appendChild(u.stateNode);else if(u.tag!==4&&u.tag!==27&&u.child!==null){u.child.return=u,u=u.child;continue}if(u===t)break e;for(;u.sibling===null;){if(u.return===null||u.return===t)break e;u=u.return}u.sibling.return=u.return,u=u.sibling}t.stateNode=e;e:switch(ct(e,n,l),n){case"button":case"input":case"select":case"textarea":e=!!l.autoFocus;break e;case"img":e=!0;break e;default:e=!1}e&&wn(t)}}return Qe(t),t.flags&=-16777217,null;case 6:if(e&&t.stateNode!=null)e.memoizedProps!==l&&wn(t);else{if(typeof l!="string"&&t.stateNode===null)throw Error(o(166));if(e=ae.current,aa(t)){if(e=t.stateNode,n=t.memoizedProps,l=null,u=gt,u!==null)switch(u.tag){case 27:case 5:l=u.memoizedProps}e[ft]=t,e=!!(e.nodeValue===n||l!==null&&l.suppressHydrationWarning===!0||am(e.nodeValue,n)),e||wr(t)}else e=ui(e).createTextNode(l),e[ft]=t,t.stateNode=e}return Qe(t),null;case 13:if(l=t.memoizedState,e===null||e.memoizedState!==null&&e.memoizedState.dehydrated!==null){if(u=aa(t),l!==null&&l.dehydrated!==null){if(e===null){if(!u)throw Error(o(318));if(u=t.memoizedState,u=u!==null?u.dehydrated:null,!u)throw Error(o(317));u[ft]=t}else oa(),(t.flags&128)===0&&(t.memoizedState=null),t.flags|=4;Qe(t),u=!1}else u=Xd(),e!==null&&e.memoizedState!==null&&(e.memoizedState.hydrationErrors=u),u=!0;if(!u)return t.flags&256?(vn(t),t):(vn(t),null)}if(vn(t),(t.flags&128)!==0)return t.lanes=n,t;if(n=l!==null,e=e!==null&&e.memoizedState!==null,n){l=t.child,u=null,l.alternate!==null&&l.alternate.memoizedState!==null&&l.alternate.memoizedState.cachePool!==null&&(u=l.alternate.memoizedState.cachePool.pool);var c=null;l.memoizedState!==null&&l.memoizedState.cachePool!==null&&(c=l.memoizedState.cachePool.pool),c!==u&&(l.flags|=2048)}return n!==e&&n&&(t.child.flags|=8192),Jo(t,t.updateQueue),Qe(t),null;case 4:return Ce(),e===null&&rc(t.stateNode.containerInfo),Qe(t),null;case 10:return yn(t.type),Qe(t),null;case 19:if(P(Ie),u=t.memoizedState,u===null)return Qe(t),null;if(l=(t.flags&128)!==0,c=u.rendering,c===null)if(l)Sa(u,!1);else{if(Ke!==0||e!==null&&(e.flags&128)!==0)for(e=t.child;e!==null;){if(c=Xo(e),c!==null){for(t.flags|=128,Sa(u,!1),e=c.updateQueue,t.updateQueue=e,Jo(t,e),t.subtreeFlags=0,e=n,n=t.child;n!==null;)qd(n,e),n=n.sibling;return F(Ie,Ie.current&1|2),t.child}e=e.sibling}u.tail!==null&&en()>Wo&&(t.flags|=128,l=!0,Sa(u,!1),t.lanes=4194304)}else{if(!l)if(e=Xo(c),e!==null){if(t.flags|=128,l=!0,e=e.updateQueue,t.updateQueue=e,Jo(t,e),Sa(u,!0),u.tail===null&&u.tailMode==="hidden"&&!c.alternate&&!Ne)return Qe(t),null}else 2*en()-u.renderingStartTime>Wo&&n!==536870912&&(t.flags|=128,l=!0,Sa(u,!1),t.lanes=4194304);u.isBackwards?(c.sibling=t.child,t.child=c):(e=u.last,e!==null?e.sibling=c:t.child=c,u.last=c)}return u.tail!==null?(t=u.tail,u.rendering=t,u.tail=t.sibling,u.renderingStartTime=en(),t.sibling=null,e=Ie.current,F(Ie,l?e&1|2:e&1),t):(Qe(t),null);case 22:case 23:return vn(t),as(),l=t.memoizedState!==null,e!==null?e.memoizedState!==null!==l&&(t.flags|=8192):l&&(t.flags|=8192),l?(n&536870912)!==0&&(t.flags&128)===0&&(Qe(t),t.subtreeFlags&6&&(t.flags|=8192)):Qe(t),n=t.updateQueue,n!==null&&Jo(t,n.retryQueue),n=null,e!==null&&e.memoizedState!==null&&e.memoizedState.cachePool!==null&&(n=e.memoizedState.cachePool.pool),l=null,t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(l=t.memoizedState.cachePool.pool),l!==n&&(t.flags|=2048),e!==null&&P(Rr),null;case 24:return n=null,e!==null&&(n=e.memoizedState.cache),t.memoizedState.cache!==n&&(t.flags|=2048),yn(We),Qe(t),null;case 25:return null;case 30:return null}throw Error(o(156,t.tag))}function fv(e,t){switch(Qu(t),t.tag){case 1:return e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 3:return yn(We),Ce(),e=t.flags,(e&65536)!==0&&(e&128)===0?(t.flags=e&-65537|128,t):null;case 26:case 27:case 5:return pt(t),null;case 13:if(vn(t),e=t.memoizedState,e!==null&&e.dehydrated!==null){if(t.alternate===null)throw Error(o(340));oa()}return e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 19:return P(Ie),null;case 4:return Ce(),null;case 10:return yn(t.type),null;case 22:case 23:return vn(t),as(),e!==null&&P(Rr),e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 24:return yn(We),null;case 25:return null;default:return null}}function h0(e,t){switch(Qu(t),t.tag){case 3:yn(We),Ce();break;case 26:case 27:case 5:pt(t);break;case 4:Ce();break;case 13:vn(t);break;case 19:P(Ie);break;case 10:yn(t.type);break;case 22:case 23:vn(t),as(),e!==null&&P(Rr);break;case 24:yn(We)}}function Ea(e,t){try{var n=t.updateQueue,l=n!==null?n.lastEffect:null;if(l!==null){var u=l.next;n=u;do{if((n.tag&e)===e){l=void 0;var c=n.create,p=n.inst;l=c(),p.destroy=l}n=n.next}while(n!==u)}}catch(b){He(t,t.return,b)}}function Vn(e,t,n){try{var l=t.updateQueue,u=l!==null?l.lastEffect:null;if(u!==null){var c=u.next;l=c;do{if((l.tag&e)===e){var p=l.inst,b=p.destroy;if(b!==void 0){p.destroy=void 0,u=t;var A=n,U=b;try{U()}catch(Q){He(u,A,Q)}}}l=l.next}while(l!==c)}}catch(Q){He(t,t.return,Q)}}function m0(e){var t=e.updateQueue;if(t!==null){var n=e.stateNode;try{th(t,n)}catch(l){He(e,e.return,l)}}}function g0(e,t,n){n.props=Ar(e.type,e.memoizedProps),n.state=e.memoizedState;try{n.componentWillUnmount()}catch(l){He(e,t,l)}}function Ra(e,t){try{var n=e.ref;if(n!==null){switch(e.tag){case 26:case 27:case 5:var l=e.stateNode;break;case 30:l=e.stateNode;break;default:l=e.stateNode}typeof n=="function"?e.refCleanup=n(l):n.current=l}}catch(u){He(e,t,u)}}function rn(e,t){var n=e.ref,l=e.refCleanup;if(n!==null)if(typeof l=="function")try{l()}catch(u){He(e,t,u)}finally{e.refCleanup=null,e=e.alternate,e!=null&&(e.refCleanup=null)}else if(typeof n=="function")try{n(null)}catch(u){He(e,t,u)}else n.current=null}function p0(e){var t=e.type,n=e.memoizedProps,l=e.stateNode;try{e:switch(t){case"button":case"input":case"select":case"textarea":n.autoFocus&&l.focus();break e;case"img":n.src?l.src=n.src:n.srcSet&&(l.srcset=n.srcSet)}}catch(u){He(e,e.return,u)}}function _s(e,t,n){try{var l=e.stateNode;zv(l,e.type,n,t),l[yt]=t}catch(u){He(e,e.return,u)}}function y0(e){return e.tag===5||e.tag===3||e.tag===26||e.tag===27&&Fn(e.type)||e.tag===4}function Ds(e){e:for(;;){for(;e.sibling===null;){if(e.return===null||y0(e.return))return null;e=e.return}for(e.sibling.return=e.return,e=e.sibling;e.tag!==5&&e.tag!==6&&e.tag!==18;){if(e.tag===27&&Fn(e.type)||e.flags&2||e.child===null||e.tag===4)continue e;e.child.return=e,e=e.child}if(!(e.flags&2))return e.stateNode}}function Us(e,t,n){var l=e.tag;if(l===5||l===6)e=e.stateNode,t?(n.nodeType===9?n.body:n.nodeName==="HTML"?n.ownerDocument.body:n).insertBefore(e,t):(t=n.nodeType===9?n.body:n.nodeName==="HTML"?n.ownerDocument.body:n,t.appendChild(e),n=n._reactRootContainer,n!=null||t.onclick!==null||(t.onclick=ii));else if(l!==4&&(l===27&&Fn(e.type)&&(n=e.stateNode,t=null),e=e.child,e!==null))for(Us(e,t,n),e=e.sibling;e!==null;)Us(e,t,n),e=e.sibling}function Po(e,t,n){var l=e.tag;if(l===5||l===6)e=e.stateNode,t?n.insertBefore(e,t):n.appendChild(e);else if(l!==4&&(l===27&&Fn(e.type)&&(n=e.stateNode),e=e.child,e!==null))for(Po(e,t,n),e=e.sibling;e!==null;)Po(e,t,n),e=e.sibling}function b0(e){var t=e.stateNode,n=e.memoizedProps;try{for(var l=e.type,u=t.attributes;u.length;)t.removeAttributeNode(u[0]);ct(t,l,n),t[ft]=e,t[yt]=n}catch(c){He(e,e.return,c)}}var Sn=!1,Je=!1,js=!1,v0=typeof WeakSet=="function"?WeakSet:Set,nt=null;function dv(e,t){if(e=e.containerInfo,oc=mi,e=Md(e),Du(e)){if("selectionStart"in e)var n={start:e.selectionStart,end:e.selectionEnd};else e:{n=(n=e.ownerDocument)&&n.defaultView||window;var l=n.getSelection&&n.getSelection();if(l&&l.rangeCount!==0){n=l.anchorNode;var u=l.anchorOffset,c=l.focusNode;l=l.focusOffset;try{n.nodeType,c.nodeType}catch{n=null;break e}var p=0,b=-1,A=-1,U=0,Q=0,J=e,j=null;t:for(;;){for(var B;J!==n||u!==0&&J.nodeType!==3||(b=p+u),J!==c||l!==0&&J.nodeType!==3||(A=p+l),J.nodeType===3&&(p+=J.nodeValue.length),(B=J.firstChild)!==null;)j=J,J=B;for(;;){if(J===e)break t;if(j===n&&++U===u&&(b=p),j===c&&++Q===l&&(A=p),(B=J.nextSibling)!==null)break;J=j,j=J.parentNode}J=B}n=b===-1||A===-1?null:{start:b,end:A}}else n=null}n=n||{start:0,end:0}}else n=null;for(ic={focusedElem:e,selectionRange:n},mi=!1,nt=t;nt!==null;)if(t=nt,e=t.child,(t.subtreeFlags&1024)!==0&&e!==null)e.return=t,nt=e;else for(;nt!==null;){switch(t=nt,c=t.alternate,e=t.flags,t.tag){case 0:break;case 11:case 15:break;case 1:if((e&1024)!==0&&c!==null){e=void 0,n=t,u=c.memoizedProps,c=c.memoizedState,l=n.stateNode;try{var ye=Ar(n.type,u,n.elementType===n.type);e=l.getSnapshotBeforeUpdate(ye,c),l.__reactInternalSnapshotBeforeUpdate=e}catch(he){He(n,n.return,he)}}break;case 3:if((e&1024)!==0){if(e=t.stateNode.containerInfo,n=e.nodeType,n===9)cc(e);else if(n===1)switch(e.nodeName){case"HEAD":case"HTML":case"BODY":cc(e);break;default:e.textContent=""}}break;case 5:case 26:case 27:case 6:case 4:case 17:break;default:if((e&1024)!==0)throw Error(o(163))}if(e=t.sibling,e!==null){e.return=t.return,nt=e;break}nt=t.return}}function x0(e,t,n){var l=n.flags;switch(n.tag){case 0:case 11:case 15:Xn(e,n),l&4&&Ea(5,n);break;case 1:if(Xn(e,n),l&4)if(e=n.stateNode,t===null)try{e.componentDidMount()}catch(p){He(n,n.return,p)}else{var u=Ar(n.type,t.memoizedProps);t=t.memoizedState;try{e.componentDidUpdate(u,t,e.__reactInternalSnapshotBeforeUpdate)}catch(p){He(n,n.return,p)}}l&64&&m0(n),l&512&&Ra(n,n.return);break;case 3:if(Xn(e,n),l&64&&(e=n.updateQueue,e!==null)){if(t=null,n.child!==null)switch(n.child.tag){case 27:case 5:t=n.child.stateNode;break;case 1:t=n.child.stateNode}try{th(e,t)}catch(p){He(n,n.return,p)}}break;case 27:t===null&&l&4&&b0(n);case 26:case 5:Xn(e,n),t===null&&l&4&&p0(n),l&512&&Ra(n,n.return);break;case 12:Xn(e,n);break;case 13:Xn(e,n),l&4&&E0(e,n),l&64&&(e=n.memoizedState,e!==null&&(e=e.dehydrated,e!==null&&(n=wv.bind(null,n),Hv(e,n))));break;case 22:if(l=n.memoizedState!==null||Sn,!l){t=t!==null&&t.memoizedState!==null||Je,u=Sn;var c=Je;Sn=l,(Je=t)&&!c?Qn(e,n,(n.subtreeFlags&8772)!==0):Xn(e,n),Sn=u,Je=c}break;case 30:break;default:Xn(e,n)}}function w0(e){var t=e.alternate;t!==null&&(e.alternate=null,w0(t)),e.child=null,e.deletions=null,e.sibling=null,e.tag===5&&(t=e.stateNode,t!==null&&gu(t)),e.stateNode=null,e.return=null,e.dependencies=null,e.memoizedProps=null,e.memoizedState=null,e.pendingProps=null,e.stateNode=null,e.updateQueue=null}var Ve=null,xt=!1;function En(e,t,n){for(n=n.child;n!==null;)S0(e,t,n),n=n.sibling}function S0(e,t,n){if(kt&&typeof kt.onCommitFiberUnmount=="function")try{kt.onCommitFiberUnmount(Ql,n)}catch{}switch(n.tag){case 26:Je||rn(n,t),En(e,t,n),n.memoizedState?n.memoizedState.count--:n.stateNode&&(n=n.stateNode,n.parentNode.removeChild(n));break;case 27:Je||rn(n,t);var l=Ve,u=xt;Fn(n.type)&&(Ve=n.stateNode,xt=!1),En(e,t,n),_a(n.stateNode),Ve=l,xt=u;break;case 5:Je||rn(n,t);case 6:if(l=Ve,u=xt,Ve=null,En(e,t,n),Ve=l,xt=u,Ve!==null)if(xt)try{(Ve.nodeType===9?Ve.body:Ve.nodeName==="HTML"?Ve.ownerDocument.body:Ve).removeChild(n.stateNode)}catch(c){He(n,t,c)}else try{Ve.removeChild(n.stateNode)}catch(c){He(n,t,c)}break;case 18:Ve!==null&&(xt?(e=Ve,cm(e.nodeType===9?e.body:e.nodeName==="HTML"?e.ownerDocument.body:e,n.stateNode),Ga(e)):cm(Ve,n.stateNode));break;case 4:l=Ve,u=xt,Ve=n.stateNode.containerInfo,xt=!0,En(e,t,n),Ve=l,xt=u;break;case 0:case 11:case 14:case 15:Je||Vn(2,n,t),Je||Vn(4,n,t),En(e,t,n);break;case 1:Je||(rn(n,t),l=n.stateNode,typeof l.componentWillUnmount=="function"&&g0(n,t,l)),En(e,t,n);break;case 21:En(e,t,n);break;case 22:Je=(l=Je)||n.memoizedState!==null,En(e,t,n),Je=l;break;default:En(e,t,n)}}function E0(e,t){if(t.memoizedState===null&&(e=t.alternate,e!==null&&(e=e.memoizedState,e!==null&&(e=e.dehydrated,e!==null))))try{Ga(e)}catch(n){He(t,t.return,n)}}function hv(e){switch(e.tag){case 13:case 19:var t=e.stateNode;return t===null&&(t=e.stateNode=new v0),t;case 22:return e=e.stateNode,t=e._retryCache,t===null&&(t=e._retryCache=new v0),t;default:throw Error(o(435,e.tag))}}function Ls(e,t){var n=hv(e);t.forEach(function(l){var u=Sv.bind(null,e,l);n.has(l)||(n.add(l),l.then(u,u))})}function Nt(e,t){var n=t.deletions;if(n!==null)for(var l=0;l<n.length;l++){var u=n[l],c=e,p=t,b=p;e:for(;b!==null;){switch(b.tag){case 27:if(Fn(b.type)){Ve=b.stateNode,xt=!1;break e}break;case 5:Ve=b.stateNode,xt=!1;break e;case 3:case 4:Ve=b.stateNode.containerInfo,xt=!0;break e}b=b.return}if(Ve===null)throw Error(o(160));S0(c,p,u),Ve=null,xt=!1,c=u.alternate,c!==null&&(c.return=null),u.return=null}if(t.subtreeFlags&13878)for(t=t.child;t!==null;)R0(t,e),t=t.sibling}var $t=null;function R0(e,t){var n=e.alternate,l=e.flags;switch(e.tag){case 0:case 11:case 14:case 15:Nt(t,e),zt(e),l&4&&(Vn(3,e,e.return),Ea(3,e),Vn(5,e,e.return));break;case 1:Nt(t,e),zt(e),l&512&&(Je||n===null||rn(n,n.return)),l&64&&Sn&&(e=e.updateQueue,e!==null&&(l=e.callbacks,l!==null&&(n=e.shared.hiddenCallbacks,e.shared.hiddenCallbacks=n===null?l:n.concat(l))));break;case 26:var u=$t;if(Nt(t,e),zt(e),l&512&&(Je||n===null||rn(n,n.return)),l&4){var c=n!==null?n.memoizedState:null;if(l=e.memoizedState,n===null)if(l===null)if(e.stateNode===null){e:{l=e.type,n=e.memoizedProps,u=u.ownerDocument||u;t:switch(l){case"title":c=u.getElementsByTagName("title")[0],(!c||c[$l]||c[ft]||c.namespaceURI==="http://www.w3.org/2000/svg"||c.hasAttribute("itemprop"))&&(c=u.createElement(l),u.head.insertBefore(c,u.querySelector("head > title"))),ct(c,l,n),c[ft]=e,et(c),l=c;break e;case"link":var p=bm("link","href",u).get(l+(n.href||""));if(p){for(var b=0;b<p.length;b++)if(c=p[b],c.getAttribute("href")===(n.href==null||n.href===""?null:n.href)&&c.getAttribute("rel")===(n.rel==null?null:n.rel)&&c.getAttribute("title")===(n.title==null?null:n.title)&&c.getAttribute("crossorigin")===(n.crossOrigin==null?null:n.crossOrigin)){p.splice(b,1);break t}}c=u.createElement(l),ct(c,l,n),u.head.appendChild(c);break;case"meta":if(p=bm("meta","content",u).get(l+(n.content||""))){for(b=0;b<p.length;b++)if(c=p[b],c.getAttribute("content")===(n.content==null?null:""+n.content)&&c.getAttribute("name")===(n.name==null?null:n.name)&&c.getAttribute("property")===(n.property==null?null:n.property)&&c.getAttribute("http-equiv")===(n.httpEquiv==null?null:n.httpEquiv)&&c.getAttribute("charset")===(n.charSet==null?null:n.charSet)){p.splice(b,1);break t}}c=u.createElement(l),ct(c,l,n),u.head.appendChild(c);break;default:throw Error(o(468,l))}c[ft]=e,et(c),l=c}e.stateNode=l}else vm(u,e.type,e.stateNode);else e.stateNode=ym(u,l,e.memoizedProps);else c!==l?(c===null?n.stateNode!==null&&(n=n.stateNode,n.parentNode.removeChild(n)):c.count--,l===null?vm(u,e.type,e.stateNode):ym(u,l,e.memoizedProps)):l===null&&e.stateNode!==null&&_s(e,e.memoizedProps,n.memoizedProps)}break;case 27:Nt(t,e),zt(e),l&512&&(Je||n===null||rn(n,n.return)),n!==null&&l&4&&_s(e,e.memoizedProps,n.memoizedProps);break;case 5:if(Nt(t,e),zt(e),l&512&&(Je||n===null||rn(n,n.return)),e.flags&32){u=e.stateNode;try{Zr(u,"")}catch(B){He(e,e.return,B)}}l&4&&e.stateNode!=null&&(u=e.memoizedProps,_s(e,u,n!==null?n.memoizedProps:u)),l&1024&&(js=!0);break;case 6:if(Nt(t,e),zt(e),l&4){if(e.stateNode===null)throw Error(o(162));l=e.memoizedProps,n=e.stateNode;try{n.nodeValue=l}catch(B){He(e,e.return,B)}}break;case 3:if(fi=null,u=$t,$t=si(t.containerInfo),Nt(t,e),$t=u,zt(e),l&4&&n!==null&&n.memoizedState.isDehydrated)try{Ga(t.containerInfo)}catch(B){He(e,e.return,B)}js&&(js=!1,T0(e));break;case 4:l=$t,$t=si(e.stateNode.containerInfo),Nt(t,e),zt(e),$t=l;break;case 12:Nt(t,e),zt(e);break;case 13:Nt(t,e),zt(e),e.child.flags&8192&&e.memoizedState!==null!=(n!==null&&n.memoizedState!==null)&&(Vs=en()),l&4&&(l=e.updateQueue,l!==null&&(e.updateQueue=null,Ls(e,l)));break;case 22:u=e.memoizedState!==null;var A=n!==null&&n.memoizedState!==null,U=Sn,Q=Je;if(Sn=U||u,Je=Q||A,Nt(t,e),Je=Q,Sn=U,zt(e),l&8192)e:for(t=e.stateNode,t._visibility=u?t._visibility&-2:t._visibility|1,u&&(n===null||A||Sn||Je||kr(e)),n=null,t=e;;){if(t.tag===5||t.tag===26){if(n===null){A=n=t;try{if(c=A.stateNode,u)p=c.style,typeof p.setProperty=="function"?p.setProperty("display","none","important"):p.display="none";else{b=A.stateNode;var J=A.memoizedProps.style,j=J!=null&&J.hasOwnProperty("display")?J.display:null;b.style.display=j==null||typeof j=="boolean"?"":(""+j).trim()}}catch(B){He(A,A.return,B)}}}else if(t.tag===6){if(n===null){A=t;try{A.stateNode.nodeValue=u?"":A.memoizedProps}catch(B){He(A,A.return,B)}}}else if((t.tag!==22&&t.tag!==23||t.memoizedState===null||t===e)&&t.child!==null){t.child.return=t,t=t.child;continue}if(t===e)break e;for(;t.sibling===null;){if(t.return===null||t.return===e)break e;n===t&&(n=null),t=t.return}n===t&&(n=null),t.sibling.return=t.return,t=t.sibling}l&4&&(l=e.updateQueue,l!==null&&(n=l.retryQueue,n!==null&&(l.retryQueue=null,Ls(e,n))));break;case 19:Nt(t,e),zt(e),l&4&&(l=e.updateQueue,l!==null&&(e.updateQueue=null,Ls(e,l)));break;case 30:break;case 21:break;default:Nt(t,e),zt(e)}}function zt(e){var t=e.flags;if(t&2){try{for(var n,l=e.return;l!==null;){if(y0(l)){n=l;break}l=l.return}if(n==null)throw Error(o(160));switch(n.tag){case 27:var u=n.stateNode,c=Ds(e);Po(e,c,u);break;case 5:var p=n.stateNode;n.flags&32&&(Zr(p,""),n.flags&=-33);var b=Ds(e);Po(e,b,p);break;case 3:case 4:var A=n.stateNode.containerInfo,U=Ds(e);Us(e,U,A);break;default:throw Error(o(161))}}catch(Q){He(e,e.return,Q)}e.flags&=-3}t&4096&&(e.flags&=-4097)}function T0(e){if(e.subtreeFlags&1024)for(e=e.child;e!==null;){var t=e;T0(t),t.tag===5&&t.flags&1024&&t.stateNode.reset(),e=e.sibling}}function Xn(e,t){if(t.subtreeFlags&8772)for(t=t.child;t!==null;)x0(e,t.alternate,t),t=t.sibling}function kr(e){for(e=e.child;e!==null;){var t=e;switch(t.tag){case 0:case 11:case 14:case 15:Vn(4,t,t.return),kr(t);break;case 1:rn(t,t.return);var n=t.stateNode;typeof n.componentWillUnmount=="function"&&g0(t,t.return,n),kr(t);break;case 27:_a(t.stateNode);case 26:case 5:rn(t,t.return),kr(t);break;case 22:t.memoizedState===null&&kr(t);break;case 30:kr(t);break;default:kr(t)}e=e.sibling}}function Qn(e,t,n){for(n=n&&(t.subtreeFlags&8772)!==0,t=t.child;t!==null;){var l=t.alternate,u=e,c=t,p=c.flags;switch(c.tag){case 0:case 11:case 15:Qn(u,c,n),Ea(4,c);break;case 1:if(Qn(u,c,n),l=c,u=l.stateNode,typeof u.componentDidMount=="function")try{u.componentDidMount()}catch(U){He(l,l.return,U)}if(l=c,u=l.updateQueue,u!==null){var b=l.stateNode;try{var A=u.shared.hiddenCallbacks;if(A!==null)for(u.shared.hiddenCallbacks=null,u=0;u<A.length;u++)eh(A[u],b)}catch(U){He(l,l.return,U)}}n&&p&64&&m0(c),Ra(c,c.return);break;case 27:b0(c);case 26:case 5:Qn(u,c,n),n&&l===null&&p&4&&p0(c),Ra(c,c.return);break;case 12:Qn(u,c,n);break;case 13:Qn(u,c,n),n&&p&4&&E0(u,c);break;case 22:c.memoizedState===null&&Qn(u,c,n),Ra(c,c.return);break;case 30:break;default:Qn(u,c,n)}t=t.sibling}}function Bs(e,t){var n=null;e!==null&&e.memoizedState!==null&&e.memoizedState.cachePool!==null&&(n=e.memoizedState.cachePool.pool),e=null,t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(e=t.memoizedState.cachePool.pool),e!==n&&(e!=null&&e.refCount++,n!=null&&sa(n))}function Hs(e,t){e=null,t.alternate!==null&&(e=t.alternate.memoizedState.cache),t=t.memoizedState.cache,t!==e&&(t.refCount++,e!=null&&sa(e))}function ln(e,t,n,l){if(t.subtreeFlags&10256)for(t=t.child;t!==null;)A0(e,t,n,l),t=t.sibling}function A0(e,t,n,l){var u=t.flags;switch(t.tag){case 0:case 11:case 15:ln(e,t,n,l),u&2048&&Ea(9,t);break;case 1:ln(e,t,n,l);break;case 3:ln(e,t,n,l),u&2048&&(e=null,t.alternate!==null&&(e=t.alternate.memoizedState.cache),t=t.memoizedState.cache,t!==e&&(t.refCount++,e!=null&&sa(e)));break;case 12:if(u&2048){ln(e,t,n,l),e=t.stateNode;try{var c=t.memoizedProps,p=c.id,b=c.onPostCommit;typeof b=="function"&&b(p,t.alternate===null?"mount":"update",e.passiveEffectDuration,-0)}catch(A){He(t,t.return,A)}}else ln(e,t,n,l);break;case 13:ln(e,t,n,l);break;case 23:break;case 22:c=t.stateNode,p=t.alternate,t.memoizedState!==null?c._visibility&2?ln(e,t,n,l):Ta(e,t):c._visibility&2?ln(e,t,n,l):(c._visibility|=2,fl(e,t,n,l,(t.subtreeFlags&10256)!==0)),u&2048&&Bs(p,t);break;case 24:ln(e,t,n,l),u&2048&&Hs(t.alternate,t);break;default:ln(e,t,n,l)}}function fl(e,t,n,l,u){for(u=u&&(t.subtreeFlags&10256)!==0,t=t.child;t!==null;){var c=e,p=t,b=n,A=l,U=p.flags;switch(p.tag){case 0:case 11:case 15:fl(c,p,b,A,u),Ea(8,p);break;case 23:break;case 22:var Q=p.stateNode;p.memoizedState!==null?Q._visibility&2?fl(c,p,b,A,u):Ta(c,p):(Q._visibility|=2,fl(c,p,b,A,u)),u&&U&2048&&Bs(p.alternate,p);break;case 24:fl(c,p,b,A,u),u&&U&2048&&Hs(p.alternate,p);break;default:fl(c,p,b,A,u)}t=t.sibling}}function Ta(e,t){if(t.subtreeFlags&10256)for(t=t.child;t!==null;){var n=e,l=t,u=l.flags;switch(l.tag){case 22:Ta(n,l),u&2048&&Bs(l.alternate,l);break;case 24:Ta(n,l),u&2048&&Hs(l.alternate,l);break;default:Ta(n,l)}t=t.sibling}}var Aa=8192;function dl(e){if(e.subtreeFlags&Aa)for(e=e.child;e!==null;)k0(e),e=e.sibling}function k0(e){switch(e.tag){case 26:dl(e),e.flags&Aa&&e.memoizedState!==null&&Wv($t,e.memoizedState,e.memoizedProps);break;case 5:dl(e);break;case 3:case 4:var t=$t;$t=si(e.stateNode.containerInfo),dl(e),$t=t;break;case 22:e.memoizedState===null&&(t=e.alternate,t!==null&&t.memoizedState!==null?(t=Aa,Aa=16777216,dl(e),Aa=t):dl(e));break;default:dl(e)}}function O0(e){var t=e.alternate;if(t!==null&&(e=t.child,e!==null)){t.child=null;do t=e.sibling,e.sibling=null,e=t;while(e!==null)}}function ka(e){var t=e.deletions;if((e.flags&16)!==0){if(t!==null)for(var n=0;n<t.length;n++){var l=t[n];nt=l,M0(l,e)}O0(e)}if(e.subtreeFlags&10256)for(e=e.child;e!==null;)C0(e),e=e.sibling}function C0(e){switch(e.tag){case 0:case 11:case 15:ka(e),e.flags&2048&&Vn(9,e,e.return);break;case 3:ka(e);break;case 12:ka(e);break;case 22:var t=e.stateNode;e.memoizedState!==null&&t._visibility&2&&(e.return===null||e.return.tag!==13)?(t._visibility&=-3,Fo(e)):ka(e);break;default:ka(e)}}function Fo(e){var t=e.deletions;if((e.flags&16)!==0){if(t!==null)for(var n=0;n<t.length;n++){var l=t[n];nt=l,M0(l,e)}O0(e)}for(e=e.child;e!==null;){switch(t=e,t.tag){case 0:case 11:case 15:Vn(8,t,t.return),Fo(t);break;case 22:n=t.stateNode,n._visibility&2&&(n._visibility&=-3,Fo(t));break;default:Fo(t)}e=e.sibling}}function M0(e,t){for(;nt!==null;){var n=nt;switch(n.tag){case 0:case 11:case 15:Vn(8,n,t);break;case 23:case 22:if(n.memoizedState!==null&&n.memoizedState.cachePool!==null){var l=n.memoizedState.cachePool.pool;l!=null&&l.refCount++}break;case 24:sa(n.memoizedState.cache)}if(l=n.child,l!==null)l.return=n,nt=l;else e:for(n=e;nt!==null;){l=nt;var u=l.sibling,c=l.return;if(w0(l),l===n){nt=null;break e}if(u!==null){u.return=c,nt=u;break e}nt=c}}}var mv={getCacheForType:function(e){var t=dt(We),n=t.data.get(e);return n===void 0&&(n=e(),t.data.set(e,n)),n}},gv=typeof WeakMap=="function"?WeakMap:Map,ze=0,qe=null,Re=null,Oe=0,_e=0,_t=null,Zn=!1,hl=!1,qs=!1,Rn=0,Ke=0,Kn=0,Or=0,Gs=0,Xt=0,ml=0,Oa=null,wt=null,Ys=!1,Vs=0,Wo=1/0,Io=null,$n=null,st=0,Jn=null,gl=null,pl=0,Xs=0,Qs=null,N0=null,Ca=0,Zs=null;function Dt(){if((ze&2)!==0&&Oe!==0)return Oe&-Oe;if(N.T!==null){var e=rl;return e!==0?e:Is()}return Kf()}function z0(){Xt===0&&(Xt=(Oe&536870912)===0||Ne?Vf():536870912);var e=Vt.current;return e!==null&&(e.flags|=32),Xt}function Ut(e,t,n){(e===qe&&(_e===2||_e===9)||e.cancelPendingCommit!==null)&&(yl(e,0),Pn(e,Oe,Xt,!1)),Kl(e,n),((ze&2)===0||e!==qe)&&(e===qe&&((ze&2)===0&&(Or|=n),Ke===4&&Pn(e,Oe,Xt,!1)),an(e))}function _0(e,t,n){if((ze&6)!==0)throw Error(o(327));var l=!n&&(t&124)===0&&(t&e.expiredLanes)===0||Zl(e,t),u=l?bv(e,t):Js(e,t,!0),c=l;do{if(u===0){hl&&!l&&Pn(e,t,0,!1);break}else{if(n=e.current.alternate,c&&!pv(n)){u=Js(e,t,!1),c=!1;continue}if(u===2){if(c=t,e.errorRecoveryDisabledLanes&c)var p=0;else p=e.pendingLanes&-536870913,p=p!==0?p:p&536870912?536870912:0;if(p!==0){t=p;e:{var b=e;u=Oa;var A=b.current.memoizedState.isDehydrated;if(A&&(yl(b,p).flags|=256),p=Js(b,p,!1),p!==2){if(qs&&!A){b.errorRecoveryDisabledLanes|=c,Or|=c,u=4;break e}c=wt,wt=u,c!==null&&(wt===null?wt=c:wt.push.apply(wt,c))}u=p}if(c=!1,u!==2)continue}}if(u===1){yl(e,0),Pn(e,t,0,!0);break}e:{switch(l=e,c=u,c){case 0:case 1:throw Error(o(345));case 4:if((t&4194048)!==t)break;case 6:Pn(l,t,Xt,!Zn);break e;case 2:wt=null;break;case 3:case 5:break;default:throw Error(o(329))}if((t&62914560)===t&&(u=Vs+300-en(),10<u)){if(Pn(l,t,Xt,!Zn),so(l,0,!0)!==0)break e;l.timeoutHandle=um(D0.bind(null,l,n,wt,Io,Ys,t,Xt,Or,ml,Zn,c,2,-0,0),u);break e}D0(l,n,wt,Io,Ys,t,Xt,Or,ml,Zn,c,0,-0,0)}}break}while(!0);an(e)}function D0(e,t,n,l,u,c,p,b,A,U,Q,J,j,B){if(e.timeoutHandle=-1,J=t.subtreeFlags,(J&8192||(J&16785408)===16785408)&&(ja={stylesheets:null,count:0,unsuspend:Fv},k0(t),J=Iv(),J!==null)){e.cancelPendingCommit=J(G0.bind(null,e,t,c,n,l,u,p,b,A,Q,1,j,B)),Pn(e,c,p,!U);return}G0(e,t,c,n,l,u,p,b,A)}function pv(e){for(var t=e;;){var n=t.tag;if((n===0||n===11||n===15)&&t.flags&16384&&(n=t.updateQueue,n!==null&&(n=n.stores,n!==null)))for(var l=0;l<n.length;l++){var u=n[l],c=u.getSnapshot;u=u.value;try{if(!Ct(c(),u))return!1}catch{return!1}}if(n=t.child,t.subtreeFlags&16384&&n!==null)n.return=t,t=n;else{if(t===e)break;for(;t.sibling===null;){if(t.return===null||t.return===e)return!0;t=t.return}t.sibling.return=t.return,t=t.sibling}}return!0}function Pn(e,t,n,l){t&=~Gs,t&=~Or,e.suspendedLanes|=t,e.pingedLanes&=~t,l&&(e.warmLanes|=t),l=e.expirationTimes;for(var u=t;0<u;){var c=31-Ot(u),p=1<<c;l[c]=-1,u&=~p}n!==0&&Qf(e,n,t)}function ei(){return(ze&6)===0?(Ma(0),!1):!0}function Ks(){if(Re!==null){if(_e===0)var e=Re.return;else e=Re,pn=Sr=null,cs(e),sl=null,xa=0,e=Re;for(;e!==null;)h0(e.alternate,e),e=e.return;Re=null}}function yl(e,t){var n=e.timeoutHandle;n!==-1&&(e.timeoutHandle=-1,Dv(n)),n=e.cancelPendingCommit,n!==null&&(e.cancelPendingCommit=null,n()),Ks(),qe=e,Re=n=hn(e.current,null),Oe=t,_e=0,_t=null,Zn=!1,hl=Zl(e,t),qs=!1,ml=Xt=Gs=Or=Kn=Ke=0,wt=Oa=null,Ys=!1,(t&8)!==0&&(t|=t&32);var l=e.entangledLanes;if(l!==0)for(e=e.entanglements,l&=t;0<l;){var u=31-Ot(l),c=1<<u;t|=e[u],l&=~c}return Rn=t,So(),n}function U0(e,t){Se=null,N.H=Go,t===fa||t===No?(t=Wd(),_e=3):t===Jd?(t=Wd(),_e=4):_e=t===Ih?8:t!==null&&typeof t=="object"&&typeof t.then=="function"?6:1,_t=t,Re===null&&(Ke=1,Zo(e,Ht(t,e.current)))}function j0(){var e=N.H;return N.H=Go,e===null?Go:e}function L0(){var e=N.A;return N.A=mv,e}function $s(){Ke=4,Zn||(Oe&4194048)!==Oe&&Vt.current!==null||(hl=!0),(Kn&134217727)===0&&(Or&134217727)===0||qe===null||Pn(qe,Oe,Xt,!1)}function Js(e,t,n){var l=ze;ze|=2;var u=j0(),c=L0();(qe!==e||Oe!==t)&&(Io=null,yl(e,t)),t=!1;var p=Ke;e:do try{if(_e!==0&&Re!==null){var b=Re,A=_t;switch(_e){case 8:Ks(),p=6;break e;case 3:case 2:case 9:case 6:Vt.current===null&&(t=!0);var U=_e;if(_e=0,_t=null,bl(e,b,A,U),n&&hl){p=0;break e}break;default:U=_e,_e=0,_t=null,bl(e,b,A,U)}}yv(),p=Ke;break}catch(Q){U0(e,Q)}while(!0);return t&&e.shellSuspendCounter++,pn=Sr=null,ze=l,N.H=u,N.A=c,Re===null&&(qe=null,Oe=0,So()),p}function yv(){for(;Re!==null;)B0(Re)}function bv(e,t){var n=ze;ze|=2;var l=j0(),u=L0();qe!==e||Oe!==t?(Io=null,Wo=en()+500,yl(e,t)):hl=Zl(e,t);e:do try{if(_e!==0&&Re!==null){t=Re;var c=_t;t:switch(_e){case 1:_e=0,_t=null,bl(e,t,c,1);break;case 2:case 9:if(Pd(c)){_e=0,_t=null,H0(t);break}t=function(){_e!==2&&_e!==9||qe!==e||(_e=7),an(e)},c.then(t,t);break e;case 3:_e=7;break e;case 4:_e=5;break e;case 7:Pd(c)?(_e=0,_t=null,H0(t)):(_e=0,_t=null,bl(e,t,c,7));break;case 5:var p=null;switch(Re.tag){case 26:p=Re.memoizedState;case 5:case 27:var b=Re;if(!p||xm(p)){_e=0,_t=null;var A=b.sibling;if(A!==null)Re=A;else{var U=b.return;U!==null?(Re=U,ti(U)):Re=null}break t}}_e=0,_t=null,bl(e,t,c,5);break;case 6:_e=0,_t=null,bl(e,t,c,6);break;case 8:Ks(),Ke=6;break e;default:throw Error(o(462))}}vv();break}catch(Q){U0(e,Q)}while(!0);return pn=Sr=null,N.H=l,N.A=u,ze=n,Re!==null?0:(qe=null,Oe=0,So(),Ke)}function vv(){for(;Re!==null&&!cu();)B0(Re)}function B0(e){var t=f0(e.alternate,e,Rn);e.memoizedProps=e.pendingProps,t===null?ti(e):Re=t}function H0(e){var t=e,n=t.alternate;switch(t.tag){case 15:case 0:t=a0(n,t,t.pendingProps,t.type,void 0,Oe);break;case 11:t=a0(n,t,t.pendingProps,t.type.render,t.ref,Oe);break;case 5:cs(t);default:h0(n,t),t=Re=qd(t,Rn),t=f0(n,t,Rn)}e.memoizedProps=e.pendingProps,t===null?ti(e):Re=t}function bl(e,t,n,l){pn=Sr=null,cs(t),sl=null,xa=0;var u=t.return;try{if(uv(e,u,t,n,Oe)){Ke=1,Zo(e,Ht(n,e.current)),Re=null;return}}catch(c){if(u!==null)throw Re=u,c;Ke=1,Zo(e,Ht(n,e.current)),Re=null;return}t.flags&32768?(Ne||l===1?e=!0:hl||(Oe&536870912)!==0?e=!1:(Zn=e=!0,(l===2||l===9||l===3||l===6)&&(l=Vt.current,l!==null&&l.tag===13&&(l.flags|=16384))),q0(t,e)):ti(t)}function ti(e){var t=e;do{if((t.flags&32768)!==0){q0(t,Zn);return}e=t.return;var n=cv(t.alternate,t,Rn);if(n!==null){Re=n;return}if(t=t.sibling,t!==null){Re=t;return}Re=t=e}while(t!==null);Ke===0&&(Ke=5)}function q0(e,t){do{var n=fv(e.alternate,e);if(n!==null){n.flags&=32767,Re=n;return}if(n=e.return,n!==null&&(n.flags|=32768,n.subtreeFlags=0,n.deletions=null),!t&&(e=e.sibling,e!==null)){Re=e;return}Re=e=n}while(e!==null);Ke=6,Re=null}function G0(e,t,n,l,u,c,p,b,A){e.cancelPendingCommit=null;do ni();while(st!==0);if((ze&6)!==0)throw Error(o(327));if(t!==null){if(t===e.current)throw Error(o(177));if(c=t.lanes|t.childLanes,c|=Hu,Fy(e,n,c,p,b,A),e===qe&&(Re=qe=null,Oe=0),gl=t,Jn=e,pl=n,Xs=c,Qs=u,N0=l,(t.subtreeFlags&10256)!==0||(t.flags&10256)!==0?(e.callbackNode=null,e.callbackPriority=0,Ev(oo,function(){return Z0(),null})):(e.callbackNode=null,e.callbackPriority=0),l=(t.flags&13878)!==0,(t.subtreeFlags&13878)!==0||l){l=N.T,N.T=null,u=q.p,q.p=2,p=ze,ze|=4;try{dv(e,t,n)}finally{ze=p,q.p=u,N.T=l}}st=1,Y0(),V0(),X0()}}function Y0(){if(st===1){st=0;var e=Jn,t=gl,n=(t.flags&13878)!==0;if((t.subtreeFlags&13878)!==0||n){n=N.T,N.T=null;var l=q.p;q.p=2;var u=ze;ze|=4;try{R0(t,e);var c=ic,p=Md(e.containerInfo),b=c.focusedElem,A=c.selectionRange;if(p!==b&&b&&b.ownerDocument&&Cd(b.ownerDocument.documentElement,b)){if(A!==null&&Du(b)){var U=A.start,Q=A.end;if(Q===void 0&&(Q=U),"selectionStart"in b)b.selectionStart=U,b.selectionEnd=Math.min(Q,b.value.length);else{var J=b.ownerDocument||document,j=J&&J.defaultView||window;if(j.getSelection){var B=j.getSelection(),ye=b.textContent.length,he=Math.min(A.start,ye),je=A.end===void 0?he:Math.min(A.end,ye);!B.extend&&he>je&&(p=je,je=he,he=p);var z=Od(b,he),C=Od(b,je);if(z&&C&&(B.rangeCount!==1||B.anchorNode!==z.node||B.anchorOffset!==z.offset||B.focusNode!==C.node||B.focusOffset!==C.offset)){var D=J.createRange();D.setStart(z.node,z.offset),B.removeAllRanges(),he>je?(B.addRange(D),B.extend(C.node,C.offset)):(D.setEnd(C.node,C.offset),B.addRange(D))}}}}for(J=[],B=b;B=B.parentNode;)B.nodeType===1&&J.push({element:B,left:B.scrollLeft,top:B.scrollTop});for(typeof b.focus=="function"&&b.focus(),b=0;b<J.length;b++){var Z=J[b];Z.element.scrollLeft=Z.left,Z.element.scrollTop=Z.top}}mi=!!oc,ic=oc=null}finally{ze=u,q.p=l,N.T=n}}e.current=t,st=2}}function V0(){if(st===2){st=0;var e=Jn,t=gl,n=(t.flags&8772)!==0;if((t.subtreeFlags&8772)!==0||n){n=N.T,N.T=null;var l=q.p;q.p=2;var u=ze;ze|=4;try{x0(e,t.alternate,t)}finally{ze=u,q.p=l,N.T=n}}st=3}}function X0(){if(st===4||st===3){st=0,Yy();var e=Jn,t=gl,n=pl,l=N0;(t.subtreeFlags&10256)!==0||(t.flags&10256)!==0?st=5:(st=0,gl=Jn=null,Q0(e,e.pendingLanes));var u=e.pendingLanes;if(u===0&&($n=null),hu(n),t=t.stateNode,kt&&typeof kt.onCommitFiberRoot=="function")try{kt.onCommitFiberRoot(Ql,t,void 0,(t.current.flags&128)===128)}catch{}if(l!==null){t=N.T,u=q.p,q.p=2,N.T=null;try{for(var c=e.onRecoverableError,p=0;p<l.length;p++){var b=l[p];c(b.value,{componentStack:b.stack})}}finally{N.T=t,q.p=u}}(pl&3)!==0&&ni(),an(e),u=e.pendingLanes,(n&4194090)!==0&&(u&42)!==0?e===Zs?Ca++:(Ca=0,Zs=e):Ca=0,Ma(0)}}function Q0(e,t){(e.pooledCacheLanes&=t)===0&&(t=e.pooledCache,t!=null&&(e.pooledCache=null,sa(t)))}function ni(e){return Y0(),V0(),X0(),Z0()}function Z0(){if(st!==5)return!1;var e=Jn,t=Xs;Xs=0;var n=hu(pl),l=N.T,u=q.p;try{q.p=32>n?32:n,N.T=null,n=Qs,Qs=null;var c=Jn,p=pl;if(st=0,gl=Jn=null,pl=0,(ze&6)!==0)throw Error(o(331));var b=ze;if(ze|=4,C0(c.current),A0(c,c.current,p,n),ze=b,Ma(0,!1),kt&&typeof kt.onPostCommitFiberRoot=="function")try{kt.onPostCommitFiberRoot(Ql,c)}catch{}return!0}finally{q.p=u,N.T=l,Q0(e,t)}}function K0(e,t,n){t=Ht(n,t),t=Rs(e.stateNode,t,2),e=Hn(e,t,2),e!==null&&(Kl(e,2),an(e))}function He(e,t,n){if(e.tag===3)K0(e,e,n);else for(;t!==null;){if(t.tag===3){K0(t,e,n);break}else if(t.tag===1){var l=t.stateNode;if(typeof t.type.getDerivedStateFromError=="function"||typeof l.componentDidCatch=="function"&&($n===null||!$n.has(l))){e=Ht(n,e),n=Fh(2),l=Hn(t,n,2),l!==null&&(Wh(n,l,t,e),Kl(l,2),an(l));break}}t=t.return}}function Ps(e,t,n){var l=e.pingCache;if(l===null){l=e.pingCache=new gv;var u=new Set;l.set(t,u)}else u=l.get(t),u===void 0&&(u=new Set,l.set(t,u));u.has(n)||(qs=!0,u.add(n),e=xv.bind(null,e,t,n),t.then(e,e))}function xv(e,t,n){var l=e.pingCache;l!==null&&l.delete(t),e.pingedLanes|=e.suspendedLanes&n,e.warmLanes&=~n,qe===e&&(Oe&n)===n&&(Ke===4||Ke===3&&(Oe&62914560)===Oe&&300>en()-Vs?(ze&2)===0&&yl(e,0):Gs|=n,ml===Oe&&(ml=0)),an(e)}function $0(e,t){t===0&&(t=Xf()),e=Ir(e,t),e!==null&&(Kl(e,t),an(e))}function wv(e){var t=e.memoizedState,n=0;t!==null&&(n=t.retryLane),$0(e,n)}function Sv(e,t){var n=0;switch(e.tag){case 13:var l=e.stateNode,u=e.memoizedState;u!==null&&(n=u.retryLane);break;case 19:l=e.stateNode;break;case 22:l=e.stateNode._retryCache;break;default:throw Error(o(314))}l!==null&&l.delete(t),$0(e,n)}function Ev(e,t){return Vl(e,t)}var ri=null,vl=null,Fs=!1,li=!1,Ws=!1,Cr=0;function an(e){e!==vl&&e.next===null&&(vl===null?ri=vl=e:vl=vl.next=e),li=!0,Fs||(Fs=!0,Tv())}function Ma(e,t){if(!Ws&&li){Ws=!0;do for(var n=!1,l=ri;l!==null;){if(e!==0){var u=l.pendingLanes;if(u===0)var c=0;else{var p=l.suspendedLanes,b=l.pingedLanes;c=(1<<31-Ot(42|e)+1)-1,c&=u&~(p&~b),c=c&201326741?c&201326741|1:c?c|2:0}c!==0&&(n=!0,W0(l,c))}else c=Oe,c=so(l,l===qe?c:0,l.cancelPendingCommit!==null||l.timeoutHandle!==-1),(c&3)===0||Zl(l,c)||(n=!0,W0(l,c));l=l.next}while(n);Ws=!1}}function Rv(){J0()}function J0(){li=Fs=!1;var e=0;Cr!==0&&(_v()&&(e=Cr),Cr=0);for(var t=en(),n=null,l=ri;l!==null;){var u=l.next,c=P0(l,t);c===0?(l.next=null,n===null?ri=u:n.next=u,u===null&&(vl=n)):(n=l,(e!==0||(c&3)!==0)&&(li=!0)),l=u}Ma(e)}function P0(e,t){for(var n=e.suspendedLanes,l=e.pingedLanes,u=e.expirationTimes,c=e.pendingLanes&-62914561;0<c;){var p=31-Ot(c),b=1<<p,A=u[p];A===-1?((b&n)===0||(b&l)!==0)&&(u[p]=Py(b,t)):A<=t&&(e.expiredLanes|=b),c&=~b}if(t=qe,n=Oe,n=so(e,e===t?n:0,e.cancelPendingCommit!==null||e.timeoutHandle!==-1),l=e.callbackNode,n===0||e===t&&(_e===2||_e===9)||e.cancelPendingCommit!==null)return l!==null&&l!==null&&Xl(l),e.callbackNode=null,e.callbackPriority=0;if((n&3)===0||Zl(e,n)){if(t=n&-n,t===e.callbackPriority)return t;switch(l!==null&&Xl(l),hu(n)){case 2:case 8:n=Gf;break;case 32:n=oo;break;case 268435456:n=Yf;break;default:n=oo}return l=F0.bind(null,e),n=Vl(n,l),e.callbackPriority=t,e.callbackNode=n,t}return l!==null&&l!==null&&Xl(l),e.callbackPriority=2,e.callbackNode=null,2}function F0(e,t){if(st!==0&&st!==5)return e.callbackNode=null,e.callbackPriority=0,null;var n=e.callbackNode;if(ni()&&e.callbackNode!==n)return null;var l=Oe;return l=so(e,e===qe?l:0,e.cancelPendingCommit!==null||e.timeoutHandle!==-1),l===0?null:(_0(e,l,t),P0(e,en()),e.callbackNode!=null&&e.callbackNode===n?F0.bind(null,e):null)}function W0(e,t){if(ni())return null;_0(e,t,!0)}function Tv(){Uv(function(){(ze&6)!==0?Vl(qf,Rv):J0()})}function Is(){return Cr===0&&(Cr=Vf()),Cr}function I0(e){return e==null||typeof e=="symbol"||typeof e=="boolean"?null:typeof e=="function"?e:go(""+e)}function em(e,t){var n=t.ownerDocument.createElement("input");return n.name=t.name,n.value=t.value,e.id&&n.setAttribute("form",e.id),t.parentNode.insertBefore(n,t),e=new FormData(e),n.parentNode.removeChild(n),e}function Av(e,t,n,l,u){if(t==="submit"&&n&&n.stateNode===u){var c=I0((u[yt]||null).action),p=l.submitter;p&&(t=(t=p[yt]||null)?I0(t.formAction):p.getAttribute("formAction"),t!==null&&(c=t,p=null));var b=new vo("action","action",null,l,u);e.push({event:b,listeners:[{instance:null,listener:function(){if(l.defaultPrevented){if(Cr!==0){var A=p?em(u,p):new FormData(u);vs(n,{pending:!0,data:A,method:u.method,action:c},null,A)}}else typeof c=="function"&&(b.preventDefault(),A=p?em(u,p):new FormData(u),vs(n,{pending:!0,data:A,method:u.method,action:c},c,A))},currentTarget:u}]})}}for(var ec=0;ec<Bu.length;ec++){var tc=Bu[ec],kv=tc.toLowerCase(),Ov=tc[0].toUpperCase()+tc.slice(1);Kt(kv,"on"+Ov)}Kt(_d,"onAnimationEnd"),Kt(Dd,"onAnimationIteration"),Kt(Ud,"onAnimationStart"),Kt("dblclick","onDoubleClick"),Kt("focusin","onFocus"),Kt("focusout","onBlur"),Kt(Qb,"onTransitionRun"),Kt(Zb,"onTransitionStart"),Kt(Kb,"onTransitionCancel"),Kt(jd,"onTransitionEnd"),Vr("onMouseEnter",["mouseout","mouseover"]),Vr("onMouseLeave",["mouseout","mouseover"]),Vr("onPointerEnter",["pointerout","pointerover"]),Vr("onPointerLeave",["pointerout","pointerover"]),hr("onChange","change click focusin focusout input keydown keyup selectionchange".split(" ")),hr("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" ")),hr("onBeforeInput",["compositionend","keypress","textInput","paste"]),hr("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" ")),hr("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" ")),hr("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var Na="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),Cv=new Set("beforetoggle cancel close invalid load scroll scrollend toggle".split(" ").concat(Na));function tm(e,t){t=(t&4)!==0;for(var n=0;n<e.length;n++){var l=e[n],u=l.event;l=l.listeners;e:{var c=void 0;if(t)for(var p=l.length-1;0<=p;p--){var b=l[p],A=b.instance,U=b.currentTarget;if(b=b.listener,A!==c&&u.isPropagationStopped())break e;c=b,u.currentTarget=U;try{c(u)}catch(Q){Qo(Q)}u.currentTarget=null,c=A}else for(p=0;p<l.length;p++){if(b=l[p],A=b.instance,U=b.currentTarget,b=b.listener,A!==c&&u.isPropagationStopped())break e;c=b,u.currentTarget=U;try{c(u)}catch(Q){Qo(Q)}u.currentTarget=null,c=A}}}}function Te(e,t){var n=t[mu];n===void 0&&(n=t[mu]=new Set);var l=e+"__bubble";n.has(l)||(nm(t,e,2,!1),n.add(l))}function nc(e,t,n){var l=0;t&&(l|=4),nm(n,e,l,t)}var ai="_reactListening"+Math.random().toString(36).slice(2);function rc(e){if(!e[ai]){e[ai]=!0,Jf.forEach(function(n){n!=="selectionchange"&&(Cv.has(n)||nc(n,!1,e),nc(n,!0,e))});var t=e.nodeType===9?e:e.ownerDocument;t===null||t[ai]||(t[ai]=!0,nc("selectionchange",!1,t))}}function nm(e,t,n,l){switch(Am(t)){case 2:var u=n1;break;case 8:u=r1;break;default:u=yc}n=u.bind(null,t,n,e),u=void 0,!Tu||t!=="touchstart"&&t!=="touchmove"&&t!=="wheel"||(u=!0),l?u!==void 0?e.addEventListener(t,n,{capture:!0,passive:u}):e.addEventListener(t,n,!0):u!==void 0?e.addEventListener(t,n,{passive:u}):e.addEventListener(t,n,!1)}function lc(e,t,n,l,u){var c=l;if((t&1)===0&&(t&2)===0&&l!==null)e:for(;;){if(l===null)return;var p=l.tag;if(p===3||p===4){var b=l.stateNode.containerInfo;if(b===u)break;if(p===4)for(p=l.return;p!==null;){var A=p.tag;if((A===3||A===4)&&p.stateNode.containerInfo===u)return;p=p.return}for(;b!==null;){if(p=qr(b),p===null)return;if(A=p.tag,A===5||A===6||A===26||A===27){l=c=p;continue e}b=b.parentNode}}l=l.return}sd(function(){var U=c,Q=Eu(n),J=[];e:{var j=Ld.get(e);if(j!==void 0){var B=vo,ye=e;switch(e){case"keypress":if(yo(n)===0)break e;case"keydown":case"keyup":B=Eb;break;case"focusin":ye="focus",B=Cu;break;case"focusout":ye="blur",B=Cu;break;case"beforeblur":case"afterblur":B=Cu;break;case"click":if(n.button===2)break e;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":B=dd;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":B=fb;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":B=Ab;break;case _d:case Dd:case Ud:B=mb;break;case jd:B=Ob;break;case"scroll":case"scrollend":B=sb;break;case"wheel":B=Mb;break;case"copy":case"cut":case"paste":B=pb;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":B=md;break;case"toggle":case"beforetoggle":B=zb}var he=(t&4)!==0,je=!he&&(e==="scroll"||e==="scrollend"),z=he?j!==null?j+"Capture":null:j;he=[];for(var C=U,D;C!==null;){var Z=C;if(D=Z.stateNode,Z=Z.tag,Z!==5&&Z!==26&&Z!==27||D===null||z===null||(Z=Pl(C,z),Z!=null&&he.push(za(C,Z,D))),je)break;C=C.return}0<he.length&&(j=new B(j,ye,null,n,Q),J.push({event:j,listeners:he}))}}if((t&7)===0){e:{if(j=e==="mouseover"||e==="pointerover",B=e==="mouseout"||e==="pointerout",j&&n!==Su&&(ye=n.relatedTarget||n.fromElement)&&(qr(ye)||ye[Hr]))break e;if((B||j)&&(j=Q.window===Q?Q:(j=Q.ownerDocument)?j.defaultView||j.parentWindow:window,B?(ye=n.relatedTarget||n.toElement,B=U,ye=ye?qr(ye):null,ye!==null&&(je=f(ye),he=ye.tag,ye!==je||he!==5&&he!==27&&he!==6)&&(ye=null)):(B=null,ye=U),B!==ye)){if(he=dd,Z="onMouseLeave",z="onMouseEnter",C="mouse",(e==="pointerout"||e==="pointerover")&&(he=md,Z="onPointerLeave",z="onPointerEnter",C="pointer"),je=B==null?j:Jl(B),D=ye==null?j:Jl(ye),j=new he(Z,C+"leave",B,n,Q),j.target=je,j.relatedTarget=D,Z=null,qr(Q)===U&&(he=new he(z,C+"enter",ye,n,Q),he.target=D,he.relatedTarget=je,Z=he),je=Z,B&&ye)t:{for(he=B,z=ye,C=0,D=he;D;D=xl(D))C++;for(D=0,Z=z;Z;Z=xl(Z))D++;for(;0<C-D;)he=xl(he),C--;for(;0<D-C;)z=xl(z),D--;for(;C--;){if(he===z||z!==null&&he===z.alternate)break t;he=xl(he),z=xl(z)}he=null}else he=null;B!==null&&rm(J,j,B,he,!1),ye!==null&&je!==null&&rm(J,je,ye,he,!0)}}e:{if(j=U?Jl(U):window,B=j.nodeName&&j.nodeName.toLowerCase(),B==="select"||B==="input"&&j.type==="file")var ie=Sd;else if(xd(j))if(Ed)ie=Yb;else{ie=qb;var Ee=Hb}else B=j.nodeName,!B||B.toLowerCase()!=="input"||j.type!=="checkbox"&&j.type!=="radio"?U&&wu(U.elementType)&&(ie=Sd):ie=Gb;if(ie&&(ie=ie(e,U))){wd(J,ie,n,Q);break e}Ee&&Ee(e,j,U),e==="focusout"&&U&&j.type==="number"&&U.memoizedProps.value!=null&&xu(j,"number",j.value)}switch(Ee=U?Jl(U):window,e){case"focusin":(xd(Ee)||Ee.contentEditable==="true")&&(Pr=Ee,Uu=U,la=null);break;case"focusout":la=Uu=Pr=null;break;case"mousedown":ju=!0;break;case"contextmenu":case"mouseup":case"dragend":ju=!1,Nd(J,n,Q);break;case"selectionchange":if(Xb)break;case"keydown":case"keyup":Nd(J,n,Q)}var ce;if(Nu)e:{switch(e){case"compositionstart":var ge="onCompositionStart";break e;case"compositionend":ge="onCompositionEnd";break e;case"compositionupdate":ge="onCompositionUpdate";break e}ge=void 0}else Jr?bd(e,n)&&(ge="onCompositionEnd"):e==="keydown"&&n.keyCode===229&&(ge="onCompositionStart");ge&&(gd&&n.locale!=="ko"&&(Jr||ge!=="onCompositionStart"?ge==="onCompositionEnd"&&Jr&&(ce=cd()):(Un=Q,Au="value"in Un?Un.value:Un.textContent,Jr=!0)),Ee=oi(U,ge),0<Ee.length&&(ge=new hd(ge,e,null,n,Q),J.push({event:ge,listeners:Ee}),ce?ge.data=ce:(ce=vd(n),ce!==null&&(ge.data=ce)))),(ce=Db?Ub(e,n):jb(e,n))&&(ge=oi(U,"onBeforeInput"),0<ge.length&&(Ee=new hd("onBeforeInput","beforeinput",null,n,Q),J.push({event:Ee,listeners:ge}),Ee.data=ce)),Av(J,e,U,n,Q)}tm(J,t)})}function za(e,t,n){return{instance:e,listener:t,currentTarget:n}}function oi(e,t){for(var n=t+"Capture",l=[];e!==null;){var u=e,c=u.stateNode;if(u=u.tag,u!==5&&u!==26&&u!==27||c===null||(u=Pl(e,n),u!=null&&l.unshift(za(e,u,c)),u=Pl(e,t),u!=null&&l.push(za(e,u,c))),e.tag===3)return l;e=e.return}return[]}function xl(e){if(e===null)return null;do e=e.return;while(e&&e.tag!==5&&e.tag!==27);return e||null}function rm(e,t,n,l,u){for(var c=t._reactName,p=[];n!==null&&n!==l;){var b=n,A=b.alternate,U=b.stateNode;if(b=b.tag,A!==null&&A===l)break;b!==5&&b!==26&&b!==27||U===null||(A=U,u?(U=Pl(n,c),U!=null&&p.unshift(za(n,U,A))):u||(U=Pl(n,c),U!=null&&p.push(za(n,U,A)))),n=n.return}p.length!==0&&e.push({event:t,listeners:p})}var Mv=/\r\n?/g,Nv=/\u0000|\uFFFD/g;function lm(e){return(typeof e=="string"?e:""+e).replace(Mv,`
`).replace(Nv,"")}function am(e,t){return t=lm(t),lm(e)===t}function ii(){}function Ue(e,t,n,l,u,c){switch(n){case"children":typeof l=="string"?t==="body"||t==="textarea"&&l===""||Zr(e,l):(typeof l=="number"||typeof l=="bigint")&&t!=="body"&&Zr(e,""+l);break;case"className":fo(e,"class",l);break;case"tabIndex":fo(e,"tabindex",l);break;case"dir":case"role":case"viewBox":case"width":case"height":fo(e,n,l);break;case"style":id(e,l,c);break;case"data":if(t!=="object"){fo(e,"data",l);break}case"src":case"href":if(l===""&&(t!=="a"||n!=="href")){e.removeAttribute(n);break}if(l==null||typeof l=="function"||typeof l=="symbol"||typeof l=="boolean"){e.removeAttribute(n);break}l=go(""+l),e.setAttribute(n,l);break;case"action":case"formAction":if(typeof l=="function"){e.setAttribute(n,"javascript:throw new Error('A React form was unexpectedly submitted. If you called form.submit() manually, consider using form.requestSubmit() instead. If you\\'re trying to use event.stopPropagation() in a submit event handler, consider also calling event.preventDefault().')");break}else typeof c=="function"&&(n==="formAction"?(t!=="input"&&Ue(e,t,"name",u.name,u,null),Ue(e,t,"formEncType",u.formEncType,u,null),Ue(e,t,"formMethod",u.formMethod,u,null),Ue(e,t,"formTarget",u.formTarget,u,null)):(Ue(e,t,"encType",u.encType,u,null),Ue(e,t,"method",u.method,u,null),Ue(e,t,"target",u.target,u,null)));if(l==null||typeof l=="symbol"||typeof l=="boolean"){e.removeAttribute(n);break}l=go(""+l),e.setAttribute(n,l);break;case"onClick":l!=null&&(e.onclick=ii);break;case"onScroll":l!=null&&Te("scroll",e);break;case"onScrollEnd":l!=null&&Te("scrollend",e);break;case"dangerouslySetInnerHTML":if(l!=null){if(typeof l!="object"||!("__html"in l))throw Error(o(61));if(n=l.__html,n!=null){if(u.children!=null)throw Error(o(60));e.innerHTML=n}}break;case"multiple":e.multiple=l&&typeof l!="function"&&typeof l!="symbol";break;case"muted":e.muted=l&&typeof l!="function"&&typeof l!="symbol";break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"defaultValue":case"defaultChecked":case"innerHTML":case"ref":break;case"autoFocus":break;case"xlinkHref":if(l==null||typeof l=="function"||typeof l=="boolean"||typeof l=="symbol"){e.removeAttribute("xlink:href");break}n=go(""+l),e.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href",n);break;case"contentEditable":case"spellCheck":case"draggable":case"value":case"autoReverse":case"externalResourcesRequired":case"focusable":case"preserveAlpha":l!=null&&typeof l!="function"&&typeof l!="symbol"?e.setAttribute(n,""+l):e.removeAttribute(n);break;case"inert":case"allowFullScreen":case"async":case"autoPlay":case"controls":case"default":case"defer":case"disabled":case"disablePictureInPicture":case"disableRemotePlayback":case"formNoValidate":case"hidden":case"loop":case"noModule":case"noValidate":case"open":case"playsInline":case"readOnly":case"required":case"reversed":case"scoped":case"seamless":case"itemScope":l&&typeof l!="function"&&typeof l!="symbol"?e.setAttribute(n,""):e.removeAttribute(n);break;case"capture":case"download":l===!0?e.setAttribute(n,""):l!==!1&&l!=null&&typeof l!="function"&&typeof l!="symbol"?e.setAttribute(n,l):e.removeAttribute(n);break;case"cols":case"rows":case"size":case"span":l!=null&&typeof l!="function"&&typeof l!="symbol"&&!isNaN(l)&&1<=l?e.setAttribute(n,l):e.removeAttribute(n);break;case"rowSpan":case"start":l==null||typeof l=="function"||typeof l=="symbol"||isNaN(l)?e.removeAttribute(n):e.setAttribute(n,l);break;case"popover":Te("beforetoggle",e),Te("toggle",e),co(e,"popover",l);break;case"xlinkActuate":fn(e,"http://www.w3.org/1999/xlink","xlink:actuate",l);break;case"xlinkArcrole":fn(e,"http://www.w3.org/1999/xlink","xlink:arcrole",l);break;case"xlinkRole":fn(e,"http://www.w3.org/1999/xlink","xlink:role",l);break;case"xlinkShow":fn(e,"http://www.w3.org/1999/xlink","xlink:show",l);break;case"xlinkTitle":fn(e,"http://www.w3.org/1999/xlink","xlink:title",l);break;case"xlinkType":fn(e,"http://www.w3.org/1999/xlink","xlink:type",l);break;case"xmlBase":fn(e,"http://www.w3.org/XML/1998/namespace","xml:base",l);break;case"xmlLang":fn(e,"http://www.w3.org/XML/1998/namespace","xml:lang",l);break;case"xmlSpace":fn(e,"http://www.w3.org/XML/1998/namespace","xml:space",l);break;case"is":co(e,"is",l);break;case"innerText":case"textContent":break;default:(!(2<n.length)||n[0]!=="o"&&n[0]!=="O"||n[1]!=="n"&&n[1]!=="N")&&(n=ib.get(n)||n,co(e,n,l))}}function ac(e,t,n,l,u,c){switch(n){case"style":id(e,l,c);break;case"dangerouslySetInnerHTML":if(l!=null){if(typeof l!="object"||!("__html"in l))throw Error(o(61));if(n=l.__html,n!=null){if(u.children!=null)throw Error(o(60));e.innerHTML=n}}break;case"children":typeof l=="string"?Zr(e,l):(typeof l=="number"||typeof l=="bigint")&&Zr(e,""+l);break;case"onScroll":l!=null&&Te("scroll",e);break;case"onScrollEnd":l!=null&&Te("scrollend",e);break;case"onClick":l!=null&&(e.onclick=ii);break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"innerHTML":case"ref":break;case"innerText":case"textContent":break;default:if(!Pf.hasOwnProperty(n))e:{if(n[0]==="o"&&n[1]==="n"&&(u=n.endsWith("Capture"),t=n.slice(2,u?n.length-7:void 0),c=e[yt]||null,c=c!=null?c[n]:null,typeof c=="function"&&e.removeEventListener(t,c,u),typeof l=="function")){typeof c!="function"&&c!==null&&(n in e?e[n]=null:e.hasAttribute(n)&&e.removeAttribute(n)),e.addEventListener(t,l,u);break e}n in e?e[n]=l:l===!0?e.setAttribute(n,""):co(e,n,l)}}}function ct(e,t,n){switch(t){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"img":Te("error",e),Te("load",e);var l=!1,u=!1,c;for(c in n)if(n.hasOwnProperty(c)){var p=n[c];if(p!=null)switch(c){case"src":l=!0;break;case"srcSet":u=!0;break;case"children":case"dangerouslySetInnerHTML":throw Error(o(137,t));default:Ue(e,t,c,p,n,null)}}u&&Ue(e,t,"srcSet",n.srcSet,n,null),l&&Ue(e,t,"src",n.src,n,null);return;case"input":Te("invalid",e);var b=c=p=u=null,A=null,U=null;for(l in n)if(n.hasOwnProperty(l)){var Q=n[l];if(Q!=null)switch(l){case"name":u=Q;break;case"type":p=Q;break;case"checked":A=Q;break;case"defaultChecked":U=Q;break;case"value":c=Q;break;case"defaultValue":b=Q;break;case"children":case"dangerouslySetInnerHTML":if(Q!=null)throw Error(o(137,t));break;default:Ue(e,t,l,Q,n,null)}}rd(e,c,b,A,U,p,u,!1),ho(e);return;case"select":Te("invalid",e),l=p=c=null;for(u in n)if(n.hasOwnProperty(u)&&(b=n[u],b!=null))switch(u){case"value":c=b;break;case"defaultValue":p=b;break;case"multiple":l=b;default:Ue(e,t,u,b,n,null)}t=c,n=p,e.multiple=!!l,t!=null?Qr(e,!!l,t,!1):n!=null&&Qr(e,!!l,n,!0);return;case"textarea":Te("invalid",e),c=u=l=null;for(p in n)if(n.hasOwnProperty(p)&&(b=n[p],b!=null))switch(p){case"value":l=b;break;case"defaultValue":u=b;break;case"children":c=b;break;case"dangerouslySetInnerHTML":if(b!=null)throw Error(o(91));break;default:Ue(e,t,p,b,n,null)}ad(e,l,u,c),ho(e);return;case"option":for(A in n)if(n.hasOwnProperty(A)&&(l=n[A],l!=null))switch(A){case"selected":e.selected=l&&typeof l!="function"&&typeof l!="symbol";break;default:Ue(e,t,A,l,n,null)}return;case"dialog":Te("beforetoggle",e),Te("toggle",e),Te("cancel",e),Te("close",e);break;case"iframe":case"object":Te("load",e);break;case"video":case"audio":for(l=0;l<Na.length;l++)Te(Na[l],e);break;case"image":Te("error",e),Te("load",e);break;case"details":Te("toggle",e);break;case"embed":case"source":case"link":Te("error",e),Te("load",e);case"area":case"base":case"br":case"col":case"hr":case"keygen":case"meta":case"param":case"track":case"wbr":case"menuitem":for(U in n)if(n.hasOwnProperty(U)&&(l=n[U],l!=null))switch(U){case"children":case"dangerouslySetInnerHTML":throw Error(o(137,t));default:Ue(e,t,U,l,n,null)}return;default:if(wu(t)){for(Q in n)n.hasOwnProperty(Q)&&(l=n[Q],l!==void 0&&ac(e,t,Q,l,n,void 0));return}}for(b in n)n.hasOwnProperty(b)&&(l=n[b],l!=null&&Ue(e,t,b,l,n,null))}function zv(e,t,n,l){switch(t){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"input":var u=null,c=null,p=null,b=null,A=null,U=null,Q=null;for(B in n){var J=n[B];if(n.hasOwnProperty(B)&&J!=null)switch(B){case"checked":break;case"value":break;case"defaultValue":A=J;default:l.hasOwnProperty(B)||Ue(e,t,B,null,l,J)}}for(var j in l){var B=l[j];if(J=n[j],l.hasOwnProperty(j)&&(B!=null||J!=null))switch(j){case"type":c=B;break;case"name":u=B;break;case"checked":U=B;break;case"defaultChecked":Q=B;break;case"value":p=B;break;case"defaultValue":b=B;break;case"children":case"dangerouslySetInnerHTML":if(B!=null)throw Error(o(137,t));break;default:B!==J&&Ue(e,t,j,B,l,J)}}vu(e,p,b,A,U,Q,c,u);return;case"select":B=p=b=j=null;for(c in n)if(A=n[c],n.hasOwnProperty(c)&&A!=null)switch(c){case"value":break;case"multiple":B=A;default:l.hasOwnProperty(c)||Ue(e,t,c,null,l,A)}for(u in l)if(c=l[u],A=n[u],l.hasOwnProperty(u)&&(c!=null||A!=null))switch(u){case"value":j=c;break;case"defaultValue":b=c;break;case"multiple":p=c;default:c!==A&&Ue(e,t,u,c,l,A)}t=b,n=p,l=B,j!=null?Qr(e,!!n,j,!1):!!l!=!!n&&(t!=null?Qr(e,!!n,t,!0):Qr(e,!!n,n?[]:"",!1));return;case"textarea":B=j=null;for(b in n)if(u=n[b],n.hasOwnProperty(b)&&u!=null&&!l.hasOwnProperty(b))switch(b){case"value":break;case"children":break;default:Ue(e,t,b,null,l,u)}for(p in l)if(u=l[p],c=n[p],l.hasOwnProperty(p)&&(u!=null||c!=null))switch(p){case"value":j=u;break;case"defaultValue":B=u;break;case"children":break;case"dangerouslySetInnerHTML":if(u!=null)throw Error(o(91));break;default:u!==c&&Ue(e,t,p,u,l,c)}ld(e,j,B);return;case"option":for(var ye in n)if(j=n[ye],n.hasOwnProperty(ye)&&j!=null&&!l.hasOwnProperty(ye))switch(ye){case"selected":e.selected=!1;break;default:Ue(e,t,ye,null,l,j)}for(A in l)if(j=l[A],B=n[A],l.hasOwnProperty(A)&&j!==B&&(j!=null||B!=null))switch(A){case"selected":e.selected=j&&typeof j!="function"&&typeof j!="symbol";break;default:Ue(e,t,A,j,l,B)}return;case"img":case"link":case"area":case"base":case"br":case"col":case"embed":case"hr":case"keygen":case"meta":case"param":case"source":case"track":case"wbr":case"menuitem":for(var he in n)j=n[he],n.hasOwnProperty(he)&&j!=null&&!l.hasOwnProperty(he)&&Ue(e,t,he,null,l,j);for(U in l)if(j=l[U],B=n[U],l.hasOwnProperty(U)&&j!==B&&(j!=null||B!=null))switch(U){case"children":case"dangerouslySetInnerHTML":if(j!=null)throw Error(o(137,t));break;default:Ue(e,t,U,j,l,B)}return;default:if(wu(t)){for(var je in n)j=n[je],n.hasOwnProperty(je)&&j!==void 0&&!l.hasOwnProperty(je)&&ac(e,t,je,void 0,l,j);for(Q in l)j=l[Q],B=n[Q],!l.hasOwnProperty(Q)||j===B||j===void 0&&B===void 0||ac(e,t,Q,j,l,B);return}}for(var z in n)j=n[z],n.hasOwnProperty(z)&&j!=null&&!l.hasOwnProperty(z)&&Ue(e,t,z,null,l,j);for(J in l)j=l[J],B=n[J],!l.hasOwnProperty(J)||j===B||j==null&&B==null||Ue(e,t,J,j,l,B)}var oc=null,ic=null;function ui(e){return e.nodeType===9?e:e.ownerDocument}function om(e){switch(e){case"http://www.w3.org/2000/svg":return 1;case"http://www.w3.org/1998/Math/MathML":return 2;default:return 0}}function im(e,t){if(e===0)switch(t){case"svg":return 1;case"math":return 2;default:return 0}return e===1&&t==="foreignObject"?0:e}function uc(e,t){return e==="textarea"||e==="noscript"||typeof t.children=="string"||typeof t.children=="number"||typeof t.children=="bigint"||typeof t.dangerouslySetInnerHTML=="object"&&t.dangerouslySetInnerHTML!==null&&t.dangerouslySetInnerHTML.__html!=null}var sc=null;function _v(){var e=window.event;return e&&e.type==="popstate"?e===sc?!1:(sc=e,!0):(sc=null,!1)}var um=typeof setTimeout=="function"?setTimeout:void 0,Dv=typeof clearTimeout=="function"?clearTimeout:void 0,sm=typeof Promise=="function"?Promise:void 0,Uv=typeof queueMicrotask=="function"?queueMicrotask:typeof sm<"u"?function(e){return sm.resolve(null).then(e).catch(jv)}:um;function jv(e){setTimeout(function(){throw e})}function Fn(e){return e==="head"}function cm(e,t){var n=t,l=0,u=0;do{var c=n.nextSibling;if(e.removeChild(n),c&&c.nodeType===8)if(n=c.data,n==="/$"){if(0<l&&8>l){n=l;var p=e.ownerDocument;if(n&1&&_a(p.documentElement),n&2&&_a(p.body),n&4)for(n=p.head,_a(n),p=n.firstChild;p;){var b=p.nextSibling,A=p.nodeName;p[$l]||A==="SCRIPT"||A==="STYLE"||A==="LINK"&&p.rel.toLowerCase()==="stylesheet"||n.removeChild(p),p=b}}if(u===0){e.removeChild(c),Ga(t);return}u--}else n==="$"||n==="$?"||n==="$!"?u++:l=n.charCodeAt(0)-48;else l=0;n=c}while(n);Ga(t)}function cc(e){var t=e.firstChild;for(t&&t.nodeType===10&&(t=t.nextSibling);t;){var n=t;switch(t=t.nextSibling,n.nodeName){case"HTML":case"HEAD":case"BODY":cc(n),gu(n);continue;case"SCRIPT":case"STYLE":continue;case"LINK":if(n.rel.toLowerCase()==="stylesheet")continue}e.removeChild(n)}}function Lv(e,t,n,l){for(;e.nodeType===1;){var u=n;if(e.nodeName.toLowerCase()!==t.toLowerCase()){if(!l&&(e.nodeName!=="INPUT"||e.type!=="hidden"))break}else if(l){if(!e[$l])switch(t){case"meta":if(!e.hasAttribute("itemprop"))break;return e;case"link":if(c=e.getAttribute("rel"),c==="stylesheet"&&e.hasAttribute("data-precedence"))break;if(c!==u.rel||e.getAttribute("href")!==(u.href==null||u.href===""?null:u.href)||e.getAttribute("crossorigin")!==(u.crossOrigin==null?null:u.crossOrigin)||e.getAttribute("title")!==(u.title==null?null:u.title))break;return e;case"style":if(e.hasAttribute("data-precedence"))break;return e;case"script":if(c=e.getAttribute("src"),(c!==(u.src==null?null:u.src)||e.getAttribute("type")!==(u.type==null?null:u.type)||e.getAttribute("crossorigin")!==(u.crossOrigin==null?null:u.crossOrigin))&&c&&e.hasAttribute("async")&&!e.hasAttribute("itemprop"))break;return e;default:return e}}else if(t==="input"&&e.type==="hidden"){var c=u.name==null?null:""+u.name;if(u.type==="hidden"&&e.getAttribute("name")===c)return e}else return e;if(e=Jt(e.nextSibling),e===null)break}return null}function Bv(e,t,n){if(t==="")return null;for(;e.nodeType!==3;)if((e.nodeType!==1||e.nodeName!=="INPUT"||e.type!=="hidden")&&!n||(e=Jt(e.nextSibling),e===null))return null;return e}function fc(e){return e.data==="$!"||e.data==="$?"&&e.ownerDocument.readyState==="complete"}function Hv(e,t){var n=e.ownerDocument;if(e.data!=="$?"||n.readyState==="complete")t();else{var l=function(){t(),n.removeEventListener("DOMContentLoaded",l)};n.addEventListener("DOMContentLoaded",l),e._reactRetry=l}}function Jt(e){for(;e!=null;e=e.nextSibling){var t=e.nodeType;if(t===1||t===3)break;if(t===8){if(t=e.data,t==="$"||t==="$!"||t==="$?"||t==="F!"||t==="F")break;if(t==="/$")return null}}return e}var dc=null;function fm(e){e=e.previousSibling;for(var t=0;e;){if(e.nodeType===8){var n=e.data;if(n==="$"||n==="$!"||n==="$?"){if(t===0)return e;t--}else n==="/$"&&t++}e=e.previousSibling}return null}function dm(e,t,n){switch(t=ui(n),e){case"html":if(e=t.documentElement,!e)throw Error(o(452));return e;case"head":if(e=t.head,!e)throw Error(o(453));return e;case"body":if(e=t.body,!e)throw Error(o(454));return e;default:throw Error(o(451))}}function _a(e){for(var t=e.attributes;t.length;)e.removeAttributeNode(t[0]);gu(e)}var Qt=new Map,hm=new Set;function si(e){return typeof e.getRootNode=="function"?e.getRootNode():e.nodeType===9?e:e.ownerDocument}var Tn=q.d;q.d={f:qv,r:Gv,D:Yv,C:Vv,L:Xv,m:Qv,X:Kv,S:Zv,M:$v};function qv(){var e=Tn.f(),t=ei();return e||t}function Gv(e){var t=Gr(e);t!==null&&t.tag===5&&t.type==="form"?_h(t):Tn.r(e)}var wl=typeof document>"u"?null:document;function mm(e,t,n){var l=wl;if(l&&typeof t=="string"&&t){var u=Bt(t);u='link[rel="'+e+'"][href="'+u+'"]',typeof n=="string"&&(u+='[crossorigin="'+n+'"]'),hm.has(u)||(hm.add(u),e={rel:e,crossOrigin:n,href:t},l.querySelector(u)===null&&(t=l.createElement("link"),ct(t,"link",e),et(t),l.head.appendChild(t)))}}function Yv(e){Tn.D(e),mm("dns-prefetch",e,null)}function Vv(e,t){Tn.C(e,t),mm("preconnect",e,t)}function Xv(e,t,n){Tn.L(e,t,n);var l=wl;if(l&&e&&t){var u='link[rel="preload"][as="'+Bt(t)+'"]';t==="image"&&n&&n.imageSrcSet?(u+='[imagesrcset="'+Bt(n.imageSrcSet)+'"]',typeof n.imageSizes=="string"&&(u+='[imagesizes="'+Bt(n.imageSizes)+'"]')):u+='[href="'+Bt(e)+'"]';var c=u;switch(t){case"style":c=Sl(e);break;case"script":c=El(e)}Qt.has(c)||(e=y({rel:"preload",href:t==="image"&&n&&n.imageSrcSet?void 0:e,as:t},n),Qt.set(c,e),l.querySelector(u)!==null||t==="style"&&l.querySelector(Da(c))||t==="script"&&l.querySelector(Ua(c))||(t=l.createElement("link"),ct(t,"link",e),et(t),l.head.appendChild(t)))}}function Qv(e,t){Tn.m(e,t);var n=wl;if(n&&e){var l=t&&typeof t.as=="string"?t.as:"script",u='link[rel="modulepreload"][as="'+Bt(l)+'"][href="'+Bt(e)+'"]',c=u;switch(l){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":c=El(e)}if(!Qt.has(c)&&(e=y({rel:"modulepreload",href:e},t),Qt.set(c,e),n.querySelector(u)===null)){switch(l){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":if(n.querySelector(Ua(c)))return}l=n.createElement("link"),ct(l,"link",e),et(l),n.head.appendChild(l)}}}function Zv(e,t,n){Tn.S(e,t,n);var l=wl;if(l&&e){var u=Yr(l).hoistableStyles,c=Sl(e);t=t||"default";var p=u.get(c);if(!p){var b={loading:0,preload:null};if(p=l.querySelector(Da(c)))b.loading=5;else{e=y({rel:"stylesheet",href:e,"data-precedence":t},n),(n=Qt.get(c))&&hc(e,n);var A=p=l.createElement("link");et(A),ct(A,"link",e),A._p=new Promise(function(U,Q){A.onload=U,A.onerror=Q}),A.addEventListener("load",function(){b.loading|=1}),A.addEventListener("error",function(){b.loading|=2}),b.loading|=4,ci(p,t,l)}p={type:"stylesheet",instance:p,count:1,state:b},u.set(c,p)}}}function Kv(e,t){Tn.X(e,t);var n=wl;if(n&&e){var l=Yr(n).hoistableScripts,u=El(e),c=l.get(u);c||(c=n.querySelector(Ua(u)),c||(e=y({src:e,async:!0},t),(t=Qt.get(u))&&mc(e,t),c=n.createElement("script"),et(c),ct(c,"link",e),n.head.appendChild(c)),c={type:"script",instance:c,count:1,state:null},l.set(u,c))}}function $v(e,t){Tn.M(e,t);var n=wl;if(n&&e){var l=Yr(n).hoistableScripts,u=El(e),c=l.get(u);c||(c=n.querySelector(Ua(u)),c||(e=y({src:e,async:!0,type:"module"},t),(t=Qt.get(u))&&mc(e,t),c=n.createElement("script"),et(c),ct(c,"link",e),n.head.appendChild(c)),c={type:"script",instance:c,count:1,state:null},l.set(u,c))}}function gm(e,t,n,l){var u=(u=ae.current)?si(u):null;if(!u)throw Error(o(446));switch(e){case"meta":case"title":return null;case"style":return typeof n.precedence=="string"&&typeof n.href=="string"?(t=Sl(n.href),n=Yr(u).hoistableStyles,l=n.get(t),l||(l={type:"style",instance:null,count:0,state:null},n.set(t,l)),l):{type:"void",instance:null,count:0,state:null};case"link":if(n.rel==="stylesheet"&&typeof n.href=="string"&&typeof n.precedence=="string"){e=Sl(n.href);var c=Yr(u).hoistableStyles,p=c.get(e);if(p||(u=u.ownerDocument||u,p={type:"stylesheet",instance:null,count:0,state:{loading:0,preload:null}},c.set(e,p),(c=u.querySelector(Da(e)))&&!c._p&&(p.instance=c,p.state.loading=5),Qt.has(e)||(n={rel:"preload",as:"style",href:n.href,crossOrigin:n.crossOrigin,integrity:n.integrity,media:n.media,hrefLang:n.hrefLang,referrerPolicy:n.referrerPolicy},Qt.set(e,n),c||Jv(u,e,n,p.state))),t&&l===null)throw Error(o(528,""));return p}if(t&&l!==null)throw Error(o(529,""));return null;case"script":return t=n.async,n=n.src,typeof n=="string"&&t&&typeof t!="function"&&typeof t!="symbol"?(t=El(n),n=Yr(u).hoistableScripts,l=n.get(t),l||(l={type:"script",instance:null,count:0,state:null},n.set(t,l)),l):{type:"void",instance:null,count:0,state:null};default:throw Error(o(444,e))}}function Sl(e){return'href="'+Bt(e)+'"'}function Da(e){return'link[rel="stylesheet"]['+e+"]"}function pm(e){return y({},e,{"data-precedence":e.precedence,precedence:null})}function Jv(e,t,n,l){e.querySelector('link[rel="preload"][as="style"]['+t+"]")?l.loading=1:(t=e.createElement("link"),l.preload=t,t.addEventListener("load",function(){return l.loading|=1}),t.addEventListener("error",function(){return l.loading|=2}),ct(t,"link",n),et(t),e.head.appendChild(t))}function El(e){return'[src="'+Bt(e)+'"]'}function Ua(e){return"script[async]"+e}function ym(e,t,n){if(t.count++,t.instance===null)switch(t.type){case"style":var l=e.querySelector('style[data-href~="'+Bt(n.href)+'"]');if(l)return t.instance=l,et(l),l;var u=y({},n,{"data-href":n.href,"data-precedence":n.precedence,href:null,precedence:null});return l=(e.ownerDocument||e).createElement("style"),et(l),ct(l,"style",u),ci(l,n.precedence,e),t.instance=l;case"stylesheet":u=Sl(n.href);var c=e.querySelector(Da(u));if(c)return t.state.loading|=4,t.instance=c,et(c),c;l=pm(n),(u=Qt.get(u))&&hc(l,u),c=(e.ownerDocument||e).createElement("link"),et(c);var p=c;return p._p=new Promise(function(b,A){p.onload=b,p.onerror=A}),ct(c,"link",l),t.state.loading|=4,ci(c,n.precedence,e),t.instance=c;case"script":return c=El(n.src),(u=e.querySelector(Ua(c)))?(t.instance=u,et(u),u):(l=n,(u=Qt.get(c))&&(l=y({},n),mc(l,u)),e=e.ownerDocument||e,u=e.createElement("script"),et(u),ct(u,"link",l),e.head.appendChild(u),t.instance=u);case"void":return null;default:throw Error(o(443,t.type))}else t.type==="stylesheet"&&(t.state.loading&4)===0&&(l=t.instance,t.state.loading|=4,ci(l,n.precedence,e));return t.instance}function ci(e,t,n){for(var l=n.querySelectorAll('link[rel="stylesheet"][data-precedence],style[data-precedence]'),u=l.length?l[l.length-1]:null,c=u,p=0;p<l.length;p++){var b=l[p];if(b.dataset.precedence===t)c=b;else if(c!==u)break}c?c.parentNode.insertBefore(e,c.nextSibling):(t=n.nodeType===9?n.head:n,t.insertBefore(e,t.firstChild))}function hc(e,t){e.crossOrigin==null&&(e.crossOrigin=t.crossOrigin),e.referrerPolicy==null&&(e.referrerPolicy=t.referrerPolicy),e.title==null&&(e.title=t.title)}function mc(e,t){e.crossOrigin==null&&(e.crossOrigin=t.crossOrigin),e.referrerPolicy==null&&(e.referrerPolicy=t.referrerPolicy),e.integrity==null&&(e.integrity=t.integrity)}var fi=null;function bm(e,t,n){if(fi===null){var l=new Map,u=fi=new Map;u.set(n,l)}else u=fi,l=u.get(n),l||(l=new Map,u.set(n,l));if(l.has(e))return l;for(l.set(e,null),n=n.getElementsByTagName(e),u=0;u<n.length;u++){var c=n[u];if(!(c[$l]||c[ft]||e==="link"&&c.getAttribute("rel")==="stylesheet")&&c.namespaceURI!=="http://www.w3.org/2000/svg"){var p=c.getAttribute(t)||"";p=e+p;var b=l.get(p);b?b.push(c):l.set(p,[c])}}return l}function vm(e,t,n){e=e.ownerDocument||e,e.head.insertBefore(n,t==="title"?e.querySelector("head > title"):null)}function Pv(e,t,n){if(n===1||t.itemProp!=null)return!1;switch(e){case"meta":case"title":return!0;case"style":if(typeof t.precedence!="string"||typeof t.href!="string"||t.href==="")break;return!0;case"link":if(typeof t.rel!="string"||typeof t.href!="string"||t.href===""||t.onLoad||t.onError)break;switch(t.rel){case"stylesheet":return e=t.disabled,typeof t.precedence=="string"&&e==null;default:return!0}case"script":if(t.async&&typeof t.async!="function"&&typeof t.async!="symbol"&&!t.onLoad&&!t.onError&&t.src&&typeof t.src=="string")return!0}return!1}function xm(e){return!(e.type==="stylesheet"&&(e.state.loading&3)===0)}var ja=null;function Fv(){}function Wv(e,t,n){if(ja===null)throw Error(o(475));var l=ja;if(t.type==="stylesheet"&&(typeof n.media!="string"||matchMedia(n.media).matches!==!1)&&(t.state.loading&4)===0){if(t.instance===null){var u=Sl(n.href),c=e.querySelector(Da(u));if(c){e=c._p,e!==null&&typeof e=="object"&&typeof e.then=="function"&&(l.count++,l=di.bind(l),e.then(l,l)),t.state.loading|=4,t.instance=c,et(c);return}c=e.ownerDocument||e,n=pm(n),(u=Qt.get(u))&&hc(n,u),c=c.createElement("link"),et(c);var p=c;p._p=new Promise(function(b,A){p.onload=b,p.onerror=A}),ct(c,"link",n),t.instance=c}l.stylesheets===null&&(l.stylesheets=new Map),l.stylesheets.set(t,e),(e=t.state.preload)&&(t.state.loading&3)===0&&(l.count++,t=di.bind(l),e.addEventListener("load",t),e.addEventListener("error",t))}}function Iv(){if(ja===null)throw Error(o(475));var e=ja;return e.stylesheets&&e.count===0&&gc(e,e.stylesheets),0<e.count?function(t){var n=setTimeout(function(){if(e.stylesheets&&gc(e,e.stylesheets),e.unsuspend){var l=e.unsuspend;e.unsuspend=null,l()}},6e4);return e.unsuspend=t,function(){e.unsuspend=null,clearTimeout(n)}}:null}function di(){if(this.count--,this.count===0){if(this.stylesheets)gc(this,this.stylesheets);else if(this.unsuspend){var e=this.unsuspend;this.unsuspend=null,e()}}}var hi=null;function gc(e,t){e.stylesheets=null,e.unsuspend!==null&&(e.count++,hi=new Map,t.forEach(e1,e),hi=null,di.call(e))}function e1(e,t){if(!(t.state.loading&4)){var n=hi.get(e);if(n)var l=n.get(null);else{n=new Map,hi.set(e,n);for(var u=e.querySelectorAll("link[data-precedence],style[data-precedence]"),c=0;c<u.length;c++){var p=u[c];(p.nodeName==="LINK"||p.getAttribute("media")!=="not all")&&(n.set(p.dataset.precedence,p),l=p)}l&&n.set(null,l)}u=t.instance,p=u.getAttribute("data-precedence"),c=n.get(p)||l,c===l&&n.set(null,u),n.set(p,u),this.count++,l=di.bind(this),u.addEventListener("load",l),u.addEventListener("error",l),c?c.parentNode.insertBefore(u,c.nextSibling):(e=e.nodeType===9?e.head:e,e.insertBefore(u,e.firstChild)),t.state.loading|=4}}var La={$$typeof:H,Provider:null,Consumer:null,_currentValue:L,_currentValue2:L,_threadCount:0};function t1(e,t,n,l,u,c,p,b){this.tag=1,this.containerInfo=e,this.pingCache=this.current=this.pendingChildren=null,this.timeoutHandle=-1,this.callbackNode=this.next=this.pendingContext=this.context=this.cancelPendingCommit=null,this.callbackPriority=0,this.expirationTimes=fu(-1),this.entangledLanes=this.shellSuspendCounter=this.errorRecoveryDisabledLanes=this.expiredLanes=this.warmLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0,this.entanglements=fu(0),this.hiddenUpdates=fu(null),this.identifierPrefix=l,this.onUncaughtError=u,this.onCaughtError=c,this.onRecoverableError=p,this.pooledCache=null,this.pooledCacheLanes=0,this.formState=b,this.incompleteTransitions=new Map}function wm(e,t,n,l,u,c,p,b,A,U,Q,J){return e=new t1(e,t,n,p,b,A,U,J),t=1,c===!0&&(t|=24),c=Mt(3,null,null,t),e.current=c,c.stateNode=e,t=Pu(),t.refCount++,e.pooledCache=t,t.refCount++,c.memoizedState={element:l,isDehydrated:n,cache:t},es(c),e}function Sm(e){return e?(e=el,e):el}function Em(e,t,n,l,u,c){u=Sm(u),l.context===null?l.context=u:l.pendingContext=u,l=Bn(t),l.payload={element:n},c=c===void 0?null:c,c!==null&&(l.callback=c),n=Hn(e,l,t),n!==null&&(Ut(n,e,t),ha(n,e,t))}function Rm(e,t){if(e=e.memoizedState,e!==null&&e.dehydrated!==null){var n=e.retryLane;e.retryLane=n!==0&&n<t?n:t}}function pc(e,t){Rm(e,t),(e=e.alternate)&&Rm(e,t)}function Tm(e){if(e.tag===13){var t=Ir(e,67108864);t!==null&&Ut(t,e,67108864),pc(e,67108864)}}var mi=!0;function n1(e,t,n,l){var u=N.T;N.T=null;var c=q.p;try{q.p=2,yc(e,t,n,l)}finally{q.p=c,N.T=u}}function r1(e,t,n,l){var u=N.T;N.T=null;var c=q.p;try{q.p=8,yc(e,t,n,l)}finally{q.p=c,N.T=u}}function yc(e,t,n,l){if(mi){var u=bc(l);if(u===null)lc(e,t,l,gi,n),km(e,l);else if(a1(u,e,t,n,l))l.stopPropagation();else if(km(e,l),t&4&&-1<l1.indexOf(e)){for(;u!==null;){var c=Gr(u);if(c!==null)switch(c.tag){case 3:if(c=c.stateNode,c.current.memoizedState.isDehydrated){var p=dr(c.pendingLanes);if(p!==0){var b=c;for(b.pendingLanes|=2,b.entangledLanes|=2;p;){var A=1<<31-Ot(p);b.entanglements[1]|=A,p&=~A}an(c),(ze&6)===0&&(Wo=en()+500,Ma(0))}}break;case 13:b=Ir(c,2),b!==null&&Ut(b,c,2),ei(),pc(c,2)}if(c=bc(l),c===null&&lc(e,t,l,gi,n),c===u)break;u=c}u!==null&&l.stopPropagation()}else lc(e,t,l,null,n)}}function bc(e){return e=Eu(e),vc(e)}var gi=null;function vc(e){if(gi=null,e=qr(e),e!==null){var t=f(e);if(t===null)e=null;else{var n=t.tag;if(n===13){if(e=d(t),e!==null)return e;e=null}else if(n===3){if(t.stateNode.current.memoizedState.isDehydrated)return t.tag===3?t.stateNode.containerInfo:null;e=null}else t!==e&&(e=null)}}return gi=e,null}function Am(e){switch(e){case"beforetoggle":case"cancel":case"click":case"close":case"contextmenu":case"copy":case"cut":case"auxclick":case"dblclick":case"dragend":case"dragstart":case"drop":case"focusin":case"focusout":case"input":case"invalid":case"keydown":case"keypress":case"keyup":case"mousedown":case"mouseup":case"paste":case"pause":case"play":case"pointercancel":case"pointerdown":case"pointerup":case"ratechange":case"reset":case"resize":case"seeked":case"submit":case"toggle":case"touchcancel":case"touchend":case"touchstart":case"volumechange":case"change":case"selectionchange":case"textInput":case"compositionstart":case"compositionend":case"compositionupdate":case"beforeblur":case"afterblur":case"beforeinput":case"blur":case"fullscreenchange":case"focus":case"hashchange":case"popstate":case"select":case"selectstart":return 2;case"drag":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"mousemove":case"mouseout":case"mouseover":case"pointermove":case"pointerout":case"pointerover":case"scroll":case"touchmove":case"wheel":case"mouseenter":case"mouseleave":case"pointerenter":case"pointerleave":return 8;case"message":switch(Vy()){case qf:return 2;case Gf:return 8;case oo:case Xy:return 32;case Yf:return 268435456;default:return 32}default:return 32}}var xc=!1,Wn=null,In=null,er=null,Ba=new Map,Ha=new Map,tr=[],l1="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset".split(" ");function km(e,t){switch(e){case"focusin":case"focusout":Wn=null;break;case"dragenter":case"dragleave":In=null;break;case"mouseover":case"mouseout":er=null;break;case"pointerover":case"pointerout":Ba.delete(t.pointerId);break;case"gotpointercapture":case"lostpointercapture":Ha.delete(t.pointerId)}}function qa(e,t,n,l,u,c){return e===null||e.nativeEvent!==c?(e={blockedOn:t,domEventName:n,eventSystemFlags:l,nativeEvent:c,targetContainers:[u]},t!==null&&(t=Gr(t),t!==null&&Tm(t)),e):(e.eventSystemFlags|=l,t=e.targetContainers,u!==null&&t.indexOf(u)===-1&&t.push(u),e)}function a1(e,t,n,l,u){switch(t){case"focusin":return Wn=qa(Wn,e,t,n,l,u),!0;case"dragenter":return In=qa(In,e,t,n,l,u),!0;case"mouseover":return er=qa(er,e,t,n,l,u),!0;case"pointerover":var c=u.pointerId;return Ba.set(c,qa(Ba.get(c)||null,e,t,n,l,u)),!0;case"gotpointercapture":return c=u.pointerId,Ha.set(c,qa(Ha.get(c)||null,e,t,n,l,u)),!0}return!1}function Om(e){var t=qr(e.target);if(t!==null){var n=f(t);if(n!==null){if(t=n.tag,t===13){if(t=d(n),t!==null){e.blockedOn=t,Wy(e.priority,function(){if(n.tag===13){var l=Dt();l=du(l);var u=Ir(n,l);u!==null&&Ut(u,n,l),pc(n,l)}});return}}else if(t===3&&n.stateNode.current.memoizedState.isDehydrated){e.blockedOn=n.tag===3?n.stateNode.containerInfo:null;return}}}e.blockedOn=null}function pi(e){if(e.blockedOn!==null)return!1;for(var t=e.targetContainers;0<t.length;){var n=bc(e.nativeEvent);if(n===null){n=e.nativeEvent;var l=new n.constructor(n.type,n);Su=l,n.target.dispatchEvent(l),Su=null}else return t=Gr(n),t!==null&&Tm(t),e.blockedOn=n,!1;t.shift()}return!0}function Cm(e,t,n){pi(e)&&n.delete(t)}function o1(){xc=!1,Wn!==null&&pi(Wn)&&(Wn=null),In!==null&&pi(In)&&(In=null),er!==null&&pi(er)&&(er=null),Ba.forEach(Cm),Ha.forEach(Cm)}function yi(e,t){e.blockedOn===t&&(e.blockedOn=null,xc||(xc=!0,r.unstable_scheduleCallback(r.unstable_NormalPriority,o1)))}var bi=null;function Mm(e){bi!==e&&(bi=e,r.unstable_scheduleCallback(r.unstable_NormalPriority,function(){bi===e&&(bi=null);for(var t=0;t<e.length;t+=3){var n=e[t],l=e[t+1],u=e[t+2];if(typeof l!="function"){if(vc(l||n)===null)continue;break}var c=Gr(n);c!==null&&(e.splice(t,3),t-=3,vs(c,{pending:!0,data:u,method:n.method,action:l},l,u))}}))}function Ga(e){function t(A){return yi(A,e)}Wn!==null&&yi(Wn,e),In!==null&&yi(In,e),er!==null&&yi(er,e),Ba.forEach(t),Ha.forEach(t);for(var n=0;n<tr.length;n++){var l=tr[n];l.blockedOn===e&&(l.blockedOn=null)}for(;0<tr.length&&(n=tr[0],n.blockedOn===null);)Om(n),n.blockedOn===null&&tr.shift();if(n=(e.ownerDocument||e).$$reactFormReplay,n!=null)for(l=0;l<n.length;l+=3){var u=n[l],c=n[l+1],p=u[yt]||null;if(typeof c=="function")p||Mm(n);else if(p){var b=null;if(c&&c.hasAttribute("formAction")){if(u=c,p=c[yt]||null)b=p.formAction;else if(vc(u)!==null)continue}else b=p.action;typeof b=="function"?n[l+1]=b:(n.splice(l,3),l-=3),Mm(n)}}}function wc(e){this._internalRoot=e}vi.prototype.render=wc.prototype.render=function(e){var t=this._internalRoot;if(t===null)throw Error(o(409));var n=t.current,l=Dt();Em(n,l,e,t,null,null)},vi.prototype.unmount=wc.prototype.unmount=function(){var e=this._internalRoot;if(e!==null){this._internalRoot=null;var t=e.containerInfo;Em(e.current,2,null,e,null,null),ei(),t[Hr]=null}};function vi(e){this._internalRoot=e}vi.prototype.unstable_scheduleHydration=function(e){if(e){var t=Kf();e={blockedOn:null,target:e,priority:t};for(var n=0;n<tr.length&&t!==0&&t<tr[n].priority;n++);tr.splice(n,0,e),n===0&&Om(e)}};var Nm=a.version;if(Nm!=="19.1.0")throw Error(o(527,Nm,"19.1.0"));q.findDOMNode=function(e){var t=e._reactInternals;if(t===void 0)throw typeof e.render=="function"?Error(o(188)):(e=Object.keys(e).join(","),Error(o(268,e)));return e=g(t),e=e!==null?m(e):null,e=e===null?null:e.stateNode,e};var i1={bundleType:0,version:"19.1.0",rendererPackageName:"react-dom",currentDispatcherRef:N,reconcilerVersion:"19.1.0"};if(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__<"u"){var xi=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!xi.isDisabled&&xi.supportsFiber)try{Ql=xi.inject(i1),kt=xi}catch{}}return Va.createRoot=function(e,t){if(!s(e))throw Error(o(299));var n=!1,l="",u=Kh,c=$h,p=Jh,b=null;return t!=null&&(t.unstable_strictMode===!0&&(n=!0),t.identifierPrefix!==void 0&&(l=t.identifierPrefix),t.onUncaughtError!==void 0&&(u=t.onUncaughtError),t.onCaughtError!==void 0&&(c=t.onCaughtError),t.onRecoverableError!==void 0&&(p=t.onRecoverableError),t.unstable_transitionCallbacks!==void 0&&(b=t.unstable_transitionCallbacks)),t=wm(e,1,!1,null,null,n,l,u,c,p,b,null),e[Hr]=t.current,rc(e),new wc(t)},Va.hydrateRoot=function(e,t,n){if(!s(e))throw Error(o(299));var l=!1,u="",c=Kh,p=$h,b=Jh,A=null,U=null;return n!=null&&(n.unstable_strictMode===!0&&(l=!0),n.identifierPrefix!==void 0&&(u=n.identifierPrefix),n.onUncaughtError!==void 0&&(c=n.onUncaughtError),n.onCaughtError!==void 0&&(p=n.onCaughtError),n.onRecoverableError!==void 0&&(b=n.onRecoverableError),n.unstable_transitionCallbacks!==void 0&&(A=n.unstable_transitionCallbacks),n.formState!==void 0&&(U=n.formState)),t=wm(e,1,!0,t,n??null,l,u,c,p,b,A,U),t.context=Sm(null),n=t.current,l=Dt(),l=du(l),u=Bn(l),u.callback=null,Hn(n,u,l),n=l,t.current.lanes=n,Kl(t,n),an(t),e[Hr]=t.current,rc(e),new vi(t)},Va.version="19.1.0",Va}var Gm;function v1(){if(Gm)return Rc.exports;Gm=1;function r(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(r)}catch(a){console.error(a)}}return r(),Rc.exports=b1(),Rc.exports}var x1=v1(),Xa={},Ym;function w1(){if(Ym)return Xa;Ym=1,Object.defineProperty(Xa,"__esModule",{value:!0}),Xa.parse=d,Xa.serialize=m;const r=/^[\u0021-\u003A\u003C\u003E-\u007E]+$/,a=/^[\u0021-\u003A\u003C-\u007E]*$/,i=/^([.]?[a-z0-9]([a-z0-9-]{0,61}[a-z0-9])?)([.][a-z0-9]([a-z0-9-]{0,61}[a-z0-9])?)*$/i,o=/^[\u0020-\u003A\u003D-\u007E]*$/,s=Object.prototype.toString,f=(()=>{const w=function(){};return w.prototype=Object.create(null),w})();function d(w,S){const x=new f,R=w.length;if(R<2)return x;const k=(S==null?void 0:S.decode)||y;let O=0;do{const M=w.indexOf("=",O);if(M===-1)break;const H=w.indexOf(";",O),$=H===-1?R:H;if(M>$){O=w.lastIndexOf(";",M-1)+1;continue}const V=h(w,O,M),W=g(w,M,V),ee=w.slice(V,W);if(x[ee]===void 0){let fe=h(w,M+1,$),ne=g(w,$,fe);const se=k(w.slice(fe,ne));x[ee]=se}O=$+1}while(O<R);return x}function h(w,S,x){do{const R=w.charCodeAt(S);if(R!==32&&R!==9)return S}while(++S<x);return x}function g(w,S,x){for(;S>x;){const R=w.charCodeAt(--S);if(R!==32&&R!==9)return S+1}return x}function m(w,S,x){const R=(x==null?void 0:x.encode)||encodeURIComponent;if(!r.test(w))throw new TypeError(`argument name is invalid: ${w}`);const k=R(S);if(!a.test(k))throw new TypeError(`argument val is invalid: ${S}`);let O=w+"="+k;if(!x)return O;if(x.maxAge!==void 0){if(!Number.isInteger(x.maxAge))throw new TypeError(`option maxAge is invalid: ${x.maxAge}`);O+="; Max-Age="+x.maxAge}if(x.domain){if(!i.test(x.domain))throw new TypeError(`option domain is invalid: ${x.domain}`);O+="; Domain="+x.domain}if(x.path){if(!o.test(x.path))throw new TypeError(`option path is invalid: ${x.path}`);O+="; Path="+x.path}if(x.expires){if(!v(x.expires)||!Number.isFinite(x.expires.valueOf()))throw new TypeError(`option expires is invalid: ${x.expires}`);O+="; Expires="+x.expires.toUTCString()}if(x.httpOnly&&(O+="; HttpOnly"),x.secure&&(O+="; Secure"),x.partitioned&&(O+="; Partitioned"),x.priority)switch(typeof x.priority=="string"?x.priority.toLowerCase():void 0){case"low":O+="; Priority=Low";break;case"medium":O+="; Priority=Medium";break;case"high":O+="; Priority=High";break;default:throw new TypeError(`option priority is invalid: ${x.priority}`)}if(x.sameSite)switch(typeof x.sameSite=="string"?x.sameSite.toLowerCase():x.sameSite){case!0:case"strict":O+="; SameSite=Strict";break;case"lax":O+="; SameSite=Lax";break;case"none":O+="; SameSite=None";break;default:throw new TypeError(`option sameSite is invalid: ${x.sameSite}`)}return O}function y(w){if(w.indexOf("%")===-1)return w;try{return decodeURIComponent(w)}catch{return w}}function v(w){return s.call(w)==="[object Date]"}return Xa}w1();var Vm="popstate";function S1(r={}){function a(o,s){let{pathname:f,search:d,hash:h}=o.location;return Vc("",{pathname:f,search:d,hash:h},s.state&&s.state.usr||null,s.state&&s.state.key||"default")}function i(o,s){return typeof s=="string"?s:Fa(s)}return R1(a,i,null,r)}function Xe(r,a){if(r===!1||r===null||typeof r>"u")throw new Error(a)}function Pt(r,a){if(!r){typeof console<"u"&&console.warn(a);try{throw new Error(a)}catch{}}}function E1(){return Math.random().toString(36).substring(2,10)}function Xm(r,a){return{usr:r.state,key:r.key,idx:a}}function Vc(r,a,i=null,o){return{pathname:typeof r=="string"?r:r.pathname,search:"",hash:"",...typeof a=="string"?zl(a):a,state:i,key:a&&a.key||o||E1()}}function Fa({pathname:r="/",search:a="",hash:i=""}){return a&&a!=="?"&&(r+=a.charAt(0)==="?"?a:"?"+a),i&&i!=="#"&&(r+=i.charAt(0)==="#"?i:"#"+i),r}function zl(r){let a={};if(r){let i=r.indexOf("#");i>=0&&(a.hash=r.substring(i),r=r.substring(0,i));let o=r.indexOf("?");o>=0&&(a.search=r.substring(o),r=r.substring(0,o)),r&&(a.pathname=r)}return a}function R1(r,a,i,o={}){let{window:s=document.defaultView,v5Compat:f=!1}=o,d=s.history,h="POP",g=null,m=y();m==null&&(m=0,d.replaceState({...d.state,idx:m},""));function y(){return(d.state||{idx:null}).idx}function v(){h="POP";let k=y(),O=k==null?null:k-m;m=k,g&&g({action:h,location:R.location,delta:O})}function w(k,O){h="PUSH";let M=Vc(R.location,k,O);m=y()+1;let H=Xm(M,m),$=R.createHref(M);try{d.pushState(H,"",$)}catch(V){if(V instanceof DOMException&&V.name==="DataCloneError")throw V;s.location.assign($)}f&&g&&g({action:h,location:R.location,delta:1})}function S(k,O){h="REPLACE";let M=Vc(R.location,k,O);m=y();let H=Xm(M,m),$=R.createHref(M);d.replaceState(H,"",$),f&&g&&g({action:h,location:R.location,delta:0})}function x(k){return T1(k)}let R={get action(){return h},get location(){return r(s,d)},listen(k){if(g)throw new Error("A history only accepts one active listener");return s.addEventListener(Vm,v),g=k,()=>{s.removeEventListener(Vm,v),g=null}},createHref(k){return a(s,k)},createURL:x,encodeLocation(k){let O=x(k);return{pathname:O.pathname,search:O.search,hash:O.hash}},push:w,replace:S,go(k){return d.go(k)}};return R}function T1(r,a=!1){let i="http://localhost";typeof window<"u"&&(i=window.location.origin!=="null"?window.location.origin:window.location.href),Xe(i,"No window.location.(origin|href) available to create URL");let o=typeof r=="string"?r:Fa(r);return o=o.replace(/ $/,"%20"),!a&&o.startsWith("//")&&(o=i+o),new URL(o,i)}function Qg(r,a,i="/"){return A1(r,a,i,!1)}function A1(r,a,i,o){let s=typeof a=="string"?zl(a):a,f=Mn(s.pathname||"/",i);if(f==null)return null;let d=Zg(r);k1(d);let h=null;for(let g=0;h==null&&g<d.length;++g){let m=B1(f);h=j1(d[g],m,o)}return h}function Zg(r,a=[],i=[],o=""){let s=(f,d,h)=>{let g={relativePath:h===void 0?f.path||"":h,caseSensitive:f.caseSensitive===!0,childrenIndex:d,route:f};g.relativePath.startsWith("/")&&(Xe(g.relativePath.startsWith(o),`Absolute route path "${g.relativePath}" nested under path "${o}" is not valid. An absolute child route path must start with the combined path of all its parent routes.`),g.relativePath=g.relativePath.slice(o.length));let m=On([o,g.relativePath]),y=i.concat(g);f.children&&f.children.length>0&&(Xe(f.index!==!0,`Index routes must not have child routes. Please remove all child routes from route path "${m}".`),Zg(f.children,a,y,m)),!(f.path==null&&!f.index)&&a.push({path:m,score:D1(m,f.index),routesMeta:y})};return r.forEach((f,d)=>{var h;if(f.path===""||!((h=f.path)!=null&&h.includes("?")))s(f,d);else for(let g of Kg(f.path))s(f,d,g)}),a}function Kg(r){let a=r.split("/");if(a.length===0)return[];let[i,...o]=a,s=i.endsWith("?"),f=i.replace(/\?$/,"");if(o.length===0)return s?[f,""]:[f];let d=Kg(o.join("/")),h=[];return h.push(...d.map(g=>g===""?f:[f,g].join("/"))),s&&h.push(...d),h.map(g=>r.startsWith("/")&&g===""?"/":g)}function k1(r){r.sort((a,i)=>a.score!==i.score?i.score-a.score:U1(a.routesMeta.map(o=>o.childrenIndex),i.routesMeta.map(o=>o.childrenIndex)))}var O1=/^:[\w-]+$/,C1=3,M1=2,N1=1,z1=10,_1=-2,Qm=r=>r==="*";function D1(r,a){let i=r.split("/"),o=i.length;return i.some(Qm)&&(o+=_1),a&&(o+=M1),i.filter(s=>!Qm(s)).reduce((s,f)=>s+(O1.test(f)?C1:f===""?N1:z1),o)}function U1(r,a){return r.length===a.length&&r.slice(0,-1).every((o,s)=>o===a[s])?r[r.length-1]-a[a.length-1]:0}function j1(r,a,i=!1){let{routesMeta:o}=r,s={},f="/",d=[];for(let h=0;h<o.length;++h){let g=o[h],m=h===o.length-1,y=f==="/"?a:a.slice(f.length)||"/",v=Li({path:g.relativePath,caseSensitive:g.caseSensitive,end:m},y),w=g.route;if(!v&&m&&i&&!o[o.length-1].route.index&&(v=Li({path:g.relativePath,caseSensitive:g.caseSensitive,end:!1},y)),!v)return null;Object.assign(s,v.params),d.push({params:s,pathname:On([f,v.pathname]),pathnameBase:Y1(On([f,v.pathnameBase])),route:w}),v.pathnameBase!=="/"&&(f=On([f,v.pathnameBase]))}return d}function Li(r,a){typeof r=="string"&&(r={path:r,caseSensitive:!1,end:!0});let[i,o]=L1(r.path,r.caseSensitive,r.end),s=a.match(i);if(!s)return null;let f=s[0],d=f.replace(/(.)\/+$/,"$1"),h=s.slice(1);return{params:o.reduce((m,{paramName:y,isOptional:v},w)=>{if(y==="*"){let x=h[w]||"";d=f.slice(0,f.length-x.length).replace(/(.)\/+$/,"$1")}const S=h[w];return v&&!S?m[y]=void 0:m[y]=(S||"").replace(/%2F/g,"/"),m},{}),pathname:f,pathnameBase:d,pattern:r}}function L1(r,a=!1,i=!0){Pt(r==="*"||!r.endsWith("*")||r.endsWith("/*"),`Route path "${r}" will be treated as if it were "${r.replace(/\*$/,"/*")}" because the \`*\` character must always follow a \`/\` in the pattern. To get rid of this warning, please change the route path to "${r.replace(/\*$/,"/*")}".`);let o=[],s="^"+r.replace(/\/*\*?$/,"").replace(/^\/*/,"/").replace(/[\\.*+^${}|()[\]]/g,"\\$&").replace(/\/:([\w-]+)(\?)?/g,(d,h,g)=>(o.push({paramName:h,isOptional:g!=null}),g?"/?([^\\/]+)?":"/([^\\/]+)"));return r.endsWith("*")?(o.push({paramName:"*"}),s+=r==="*"||r==="/*"?"(.*)$":"(?:\\/(.+)|\\/*)$"):i?s+="\\/*$":r!==""&&r!=="/"&&(s+="(?:(?=\\/|$))"),[new RegExp(s,a?void 0:"i"),o]}function B1(r){try{return r.split("/").map(a=>decodeURIComponent(a).replace(/\//g,"%2F")).join("/")}catch(a){return Pt(!1,`The URL path "${r}" could not be decoded because it is a malformed URL segment. This is probably due to a bad percent encoding (${a}).`),r}}function Mn(r,a){if(a==="/")return r;if(!r.toLowerCase().startsWith(a.toLowerCase()))return null;let i=a.endsWith("/")?a.length-1:a.length,o=r.charAt(i);return o&&o!=="/"?null:r.slice(i)||"/"}function H1(r,a="/"){let{pathname:i,search:o="",hash:s=""}=typeof r=="string"?zl(r):r;return{pathname:i?i.startsWith("/")?i:q1(i,a):a,search:V1(o),hash:X1(s)}}function q1(r,a){let i=a.replace(/\/+$/,"").split("/");return r.split("/").forEach(s=>{s===".."?i.length>1&&i.pop():s!=="."&&i.push(s)}),i.length>1?i.join("/"):"/"}function Oc(r,a,i,o){return`Cannot include a '${r}' character in a manually specified \`to.${a}\` field [${JSON.stringify(o)}].  Please separate it out to the \`to.${i}\` field. Alternatively you may provide the full path as a string in <Link to="..."> and the router will parse it for you.`}function G1(r){return r.filter((a,i)=>i===0||a.route.path&&a.route.path.length>0)}function ff(r){let a=G1(r);return a.map((i,o)=>o===a.length-1?i.pathname:i.pathnameBase)}function df(r,a,i,o=!1){let s;typeof r=="string"?s=zl(r):(s={...r},Xe(!s.pathname||!s.pathname.includes("?"),Oc("?","pathname","search",s)),Xe(!s.pathname||!s.pathname.includes("#"),Oc("#","pathname","hash",s)),Xe(!s.search||!s.search.includes("#"),Oc("#","search","hash",s)));let f=r===""||s.pathname==="",d=f?"/":s.pathname,h;if(d==null)h=i;else{let v=a.length-1;if(!o&&d.startsWith("..")){let w=d.split("/");for(;w[0]==="..";)w.shift(),v-=1;s.pathname=w.join("/")}h=v>=0?a[v]:"/"}let g=H1(s,h),m=d&&d!=="/"&&d.endsWith("/"),y=(f||d===".")&&i.endsWith("/");return!g.pathname.endsWith("/")&&(m||y)&&(g.pathname+="/"),g}var On=r=>r.join("/").replace(/\/\/+/g,"/"),Y1=r=>r.replace(/\/+$/,"").replace(/^\/*/,"/"),V1=r=>!r||r==="?"?"":r.startsWith("?")?r:"?"+r,X1=r=>!r||r==="#"?"":r.startsWith("#")?r:"#"+r;function Q1(r){return r!=null&&typeof r.status=="number"&&typeof r.statusText=="string"&&typeof r.internal=="boolean"&&"data"in r}var $g=["POST","PUT","PATCH","DELETE"];new Set($g);var Z1=["GET",...$g];new Set(Z1);var _l=T.createContext(null);_l.displayName="DataRouter";var $i=T.createContext(null);$i.displayName="DataRouterState";var Jg=T.createContext({isTransitioning:!1});Jg.displayName="ViewTransition";var K1=T.createContext(new Map);K1.displayName="Fetchers";var $1=T.createContext(null);$1.displayName="Await";var Ft=T.createContext(null);Ft.displayName="Navigation";var eo=T.createContext(null);eo.displayName="Location";var Wt=T.createContext({outlet:null,matches:[],isDataRoute:!1});Wt.displayName="Route";var hf=T.createContext(null);hf.displayName="RouteError";function J1(r,{relative:a}={}){Xe(Dl(),"useHref() may be used only in the context of a <Router> component.");let{basename:i,navigator:o}=T.useContext(Ft),{hash:s,pathname:f,search:d}=to(r,{relative:a}),h=f;return i!=="/"&&(h=f==="/"?i:On([i,f])),o.createHref({pathname:h,search:d,hash:s})}function Dl(){return T.useContext(eo)!=null}function cr(){return Xe(Dl(),"useLocation() may be used only in the context of a <Router> component."),T.useContext(eo).location}var Pg="You should call navigate() in a React.useEffect(), not when your component is first rendered.";function Fg(r){T.useContext(Ft).static||T.useLayoutEffect(r)}function mf(){let{isDataRoute:r}=T.useContext(Wt);return r?cx():P1()}function P1(){Xe(Dl(),"useNavigate() may be used only in the context of a <Router> component.");let r=T.useContext(_l),{basename:a,navigator:i}=T.useContext(Ft),{matches:o}=T.useContext(Wt),{pathname:s}=cr(),f=JSON.stringify(ff(o)),d=T.useRef(!1);return Fg(()=>{d.current=!0}),T.useCallback((g,m={})=>{if(Pt(d.current,Pg),!d.current)return;if(typeof g=="number"){i.go(g);return}let y=df(g,JSON.parse(f),s,m.relative==="path");r==null&&a!=="/"&&(y.pathname=y.pathname==="/"?a:On([a,y.pathname])),(m.replace?i.replace:i.push)(y,m.state,m)},[a,i,f,s,r])}var F1=T.createContext(null);function W1(r){let a=T.useContext(Wt).outlet;return a&&T.createElement(F1.Provider,{value:r},a)}function to(r,{relative:a}={}){let{matches:i}=T.useContext(Wt),{pathname:o}=cr(),s=JSON.stringify(ff(i));return T.useMemo(()=>df(r,JSON.parse(s),o,a==="path"),[r,s,o,a])}function I1(r,a){return Wg(r,a)}function Wg(r,a,i,o){var O;Xe(Dl(),"useRoutes() may be used only in the context of a <Router> component.");let{navigator:s}=T.useContext(Ft),{matches:f}=T.useContext(Wt),d=f[f.length-1],h=d?d.params:{},g=d?d.pathname:"/",m=d?d.pathnameBase:"/",y=d&&d.route;{let M=y&&y.path||"";Ig(g,!y||M.endsWith("*")||M.endsWith("*?"),`You rendered descendant <Routes> (or called \`useRoutes()\`) at "${g}" (under <Route path="${M}">) but the parent route path has no trailing "*". This means if you navigate deeper, the parent won't match anymore and therefore the child routes will never render.

Please change the parent <Route path="${M}"> to <Route path="${M==="/"?"*":`${M}/*`}">.`)}let v=cr(),w;if(a){let M=typeof a=="string"?zl(a):a;Xe(m==="/"||((O=M.pathname)==null?void 0:O.startsWith(m)),`When overriding the location using \`<Routes location>\` or \`useRoutes(routes, location)\`, the location pathname must begin with the portion of the URL pathname that was matched by all parent routes. The current pathname base is "${m}" but pathname "${M.pathname}" was given in the \`location\` prop.`),w=M}else w=v;let S=w.pathname||"/",x=S;if(m!=="/"){let M=m.replace(/^\//,"").split("/");x="/"+S.replace(/^\//,"").split("/").slice(M.length).join("/")}let R=Qg(r,{pathname:x});Pt(y||R!=null,`No routes matched location "${w.pathname}${w.search}${w.hash}" `),Pt(R==null||R[R.length-1].route.element!==void 0||R[R.length-1].route.Component!==void 0||R[R.length-1].route.lazy!==void 0,`Matched leaf route at location "${w.pathname}${w.search}${w.hash}" does not have an element or Component. This means it will render an <Outlet /> with a null value by default resulting in an "empty" page.`);let k=lx(R&&R.map(M=>Object.assign({},M,{params:Object.assign({},h,M.params),pathname:On([m,s.encodeLocation?s.encodeLocation(M.pathname).pathname:M.pathname]),pathnameBase:M.pathnameBase==="/"?m:On([m,s.encodeLocation?s.encodeLocation(M.pathnameBase).pathname:M.pathnameBase])})),f,i,o);return a&&k?T.createElement(eo.Provider,{value:{location:{pathname:"/",search:"",hash:"",state:null,key:"default",...w},navigationType:"POP"}},k):k}function ex(){let r=sx(),a=Q1(r)?`${r.status} ${r.statusText}`:r instanceof Error?r.message:JSON.stringify(r),i=r instanceof Error?r.stack:null,o="rgba(200,200,200, 0.5)",s={padding:"0.5rem",backgroundColor:o},f={padding:"2px 4px",backgroundColor:o},d=null;return console.error("Error handled by React Router default ErrorBoundary:",r),d=T.createElement(T.Fragment,null,T.createElement("p",null,"💿 Hey developer 👋"),T.createElement("p",null,"You can provide a way better UX than this when your app throws errors by providing your own ",T.createElement("code",{style:f},"ErrorBoundary")," or"," ",T.createElement("code",{style:f},"errorElement")," prop on your route.")),T.createElement(T.Fragment,null,T.createElement("h2",null,"Unexpected Application Error!"),T.createElement("h3",{style:{fontStyle:"italic"}},a),i?T.createElement("pre",{style:s},i):null,d)}var tx=T.createElement(ex,null),nx=class extends T.Component{constructor(r){super(r),this.state={location:r.location,revalidation:r.revalidation,error:r.error}}static getDerivedStateFromError(r){return{error:r}}static getDerivedStateFromProps(r,a){return a.location!==r.location||a.revalidation!=="idle"&&r.revalidation==="idle"?{error:r.error,location:r.location,revalidation:r.revalidation}:{error:r.error!==void 0?r.error:a.error,location:a.location,revalidation:r.revalidation||a.revalidation}}componentDidCatch(r,a){console.error("React Router caught the following error during render",r,a)}render(){return this.state.error!==void 0?T.createElement(Wt.Provider,{value:this.props.routeContext},T.createElement(hf.Provider,{value:this.state.error,children:this.props.component})):this.props.children}};function rx({routeContext:r,match:a,children:i}){let o=T.useContext(_l);return o&&o.static&&o.staticContext&&(a.route.errorElement||a.route.ErrorBoundary)&&(o.staticContext._deepestRenderedBoundaryId=a.route.id),T.createElement(Wt.Provider,{value:r},i)}function lx(r,a=[],i=null,o=null){if(r==null){if(!i)return null;if(i.errors)r=i.matches;else if(a.length===0&&!i.initialized&&i.matches.length>0)r=i.matches;else return null}let s=r,f=i==null?void 0:i.errors;if(f!=null){let g=s.findIndex(m=>m.route.id&&(f==null?void 0:f[m.route.id])!==void 0);Xe(g>=0,`Could not find a matching route for errors on route IDs: ${Object.keys(f).join(",")}`),s=s.slice(0,Math.min(s.length,g+1))}let d=!1,h=-1;if(i)for(let g=0;g<s.length;g++){let m=s[g];if((m.route.HydrateFallback||m.route.hydrateFallbackElement)&&(h=g),m.route.id){let{loaderData:y,errors:v}=i,w=m.route.loader&&!y.hasOwnProperty(m.route.id)&&(!v||v[m.route.id]===void 0);if(m.route.lazy||w){d=!0,h>=0?s=s.slice(0,h+1):s=[s[0]];break}}}return s.reduceRight((g,m,y)=>{let v,w=!1,S=null,x=null;i&&(v=f&&m.route.id?f[m.route.id]:void 0,S=m.route.errorElement||tx,d&&(h<0&&y===0?(Ig("route-fallback",!1,"No `HydrateFallback` element provided to render during initial hydration"),w=!0,x=null):h===y&&(w=!0,x=m.route.hydrateFallbackElement||null)));let R=a.concat(s.slice(0,y+1)),k=()=>{let O;return v?O=S:w?O=x:m.route.Component?O=T.createElement(m.route.Component,null):m.route.element?O=m.route.element:O=g,T.createElement(rx,{match:m,routeContext:{outlet:g,matches:R,isDataRoute:i!=null},children:O})};return i&&(m.route.ErrorBoundary||m.route.errorElement||y===0)?T.createElement(nx,{location:i.location,revalidation:i.revalidation,component:S,error:v,children:k(),routeContext:{outlet:null,matches:R,isDataRoute:!0}}):k()},null)}function gf(r){return`${r} must be used within a data router.  See https://reactrouter.com/en/main/routers/picking-a-router.`}function ax(r){let a=T.useContext(_l);return Xe(a,gf(r)),a}function ox(r){let a=T.useContext($i);return Xe(a,gf(r)),a}function ix(r){let a=T.useContext(Wt);return Xe(a,gf(r)),a}function pf(r){let a=ix(r),i=a.matches[a.matches.length-1];return Xe(i.route.id,`${r} can only be used on routes that contain a unique "id"`),i.route.id}function ux(){return pf("useRouteId")}function sx(){var o;let r=T.useContext(hf),a=ox("useRouteError"),i=pf("useRouteError");return r!==void 0?r:(o=a.errors)==null?void 0:o[i]}function cx(){let{router:r}=ax("useNavigate"),a=pf("useNavigate"),i=T.useRef(!1);return Fg(()=>{i.current=!0}),T.useCallback(async(s,f={})=>{Pt(i.current,Pg),i.current&&(typeof s=="number"?r.navigate(s):await r.navigate(s,{fromRouteId:a,...f}))},[r,a])}var Zm={};function Ig(r,a,i){!a&&!Zm[r]&&(Zm[r]=!0,Pt(!1,i))}T.memo(fx);function fx({routes:r,future:a,state:i}){return Wg(r,void 0,i,a)}function yf({to:r,replace:a,state:i,relative:o}){Xe(Dl(),"<Navigate> may be used only in the context of a <Router> component.");let{static:s}=T.useContext(Ft);Pt(!s,"<Navigate> must not be used on the initial render in a <StaticRouter>. This is a no-op, but you should modify your code so the <Navigate> is only ever rendered in response to some user interaction or state change.");let{matches:f}=T.useContext(Wt),{pathname:d}=cr(),h=mf(),g=df(r,ff(f),d,o==="path"),m=JSON.stringify(g);return T.useEffect(()=>{h(JSON.parse(m),{replace:a,state:i,relative:o})},[h,m,o,a,i]),null}function ep(r){return W1(r.context)}function Tl(r){Xe(!1,"A <Route> is only ever to be used as the child of <Routes> element, never rendered directly. Please wrap your <Route> in a <Routes>.")}function dx({basename:r="/",children:a=null,location:i,navigationType:o="POP",navigator:s,static:f=!1}){Xe(!Dl(),"You cannot render a <Router> inside another <Router>. You should never have more than one in your app.");let d=r.replace(/^\/*/,"/"),h=T.useMemo(()=>({basename:d,navigator:s,static:f,future:{}}),[d,s,f]);typeof i=="string"&&(i=zl(i));let{pathname:g="/",search:m="",hash:y="",state:v=null,key:w="default"}=i,S=T.useMemo(()=>{let x=Mn(g,d);return x==null?null:{location:{pathname:x,search:m,hash:y,state:v,key:w},navigationType:o}},[d,g,m,y,v,w,o]);return Pt(S!=null,`<Router basename="${d}"> is not able to match the URL "${g}${m}${y}" because it does not start with the basename, so the <Router> won't render anything.`),S==null?null:T.createElement(Ft.Provider,{value:h},T.createElement(eo.Provider,{children:a,value:S}))}function hx({children:r,location:a}){return I1(Xc(r),a)}function Xc(r,a=[]){let i=[];return T.Children.forEach(r,(o,s)=>{if(!T.isValidElement(o))return;let f=[...a,s];if(o.type===T.Fragment){i.push.apply(i,Xc(o.props.children,f));return}Xe(o.type===Tl,`[${typeof o.type=="string"?o.type:o.type.name}] is not a <Route> component. All component children of <Routes> must be a <Route> or <React.Fragment>`),Xe(!o.props.index||!o.props.children,"An index route cannot have child routes.");let d={id:o.props.id||f.join("-"),caseSensitive:o.props.caseSensitive,element:o.props.element,Component:o.props.Component,index:o.props.index,path:o.props.path,loader:o.props.loader,action:o.props.action,hydrateFallbackElement:o.props.hydrateFallbackElement,HydrateFallback:o.props.HydrateFallback,errorElement:o.props.errorElement,ErrorBoundary:o.props.ErrorBoundary,hasErrorBoundary:o.props.hasErrorBoundary===!0||o.props.ErrorBoundary!=null||o.props.errorElement!=null,shouldRevalidate:o.props.shouldRevalidate,handle:o.props.handle,lazy:o.props.lazy};o.props.children&&(d.children=Xc(o.props.children,f)),i.push(d)}),i}var ki="get",Oi="application/x-www-form-urlencoded";function Ji(r){return r!=null&&typeof r.tagName=="string"}function mx(r){return Ji(r)&&r.tagName.toLowerCase()==="button"}function gx(r){return Ji(r)&&r.tagName.toLowerCase()==="form"}function px(r){return Ji(r)&&r.tagName.toLowerCase()==="input"}function yx(r){return!!(r.metaKey||r.altKey||r.ctrlKey||r.shiftKey)}function bx(r,a){return r.button===0&&(!a||a==="_self")&&!yx(r)}var wi=null;function vx(){if(wi===null)try{new FormData(document.createElement("form"),0),wi=!1}catch{wi=!0}return wi}var xx=new Set(["application/x-www-form-urlencoded","multipart/form-data","text/plain"]);function Cc(r){return r!=null&&!xx.has(r)?(Pt(!1,`"${r}" is not a valid \`encType\` for \`<Form>\`/\`<fetcher.Form>\` and will default to "${Oi}"`),null):r}function wx(r,a){let i,o,s,f,d;if(gx(r)){let h=r.getAttribute("action");o=h?Mn(h,a):null,i=r.getAttribute("method")||ki,s=Cc(r.getAttribute("enctype"))||Oi,f=new FormData(r)}else if(mx(r)||px(r)&&(r.type==="submit"||r.type==="image")){let h=r.form;if(h==null)throw new Error('Cannot submit a <button> or <input type="submit"> without a <form>');let g=r.getAttribute("formaction")||h.getAttribute("action");if(o=g?Mn(g,a):null,i=r.getAttribute("formmethod")||h.getAttribute("method")||ki,s=Cc(r.getAttribute("formenctype"))||Cc(h.getAttribute("enctype"))||Oi,f=new FormData(h,r),!vx()){let{name:m,type:y,value:v}=r;if(y==="image"){let w=m?`${m}.`:"";f.append(`${w}x`,"0"),f.append(`${w}y`,"0")}else m&&f.append(m,v)}}else{if(Ji(r))throw new Error('Cannot submit element that is not <form>, <button>, or <input type="submit|image">');i=ki,o=null,s=Oi,d=r}return f&&s==="text/plain"&&(d=f,f=void 0),{action:o,method:i.toLowerCase(),encType:s,formData:f,body:d}}function bf(r,a){if(r===!1||r===null||typeof r>"u")throw new Error(a)}async function Sx(r,a){if(r.id in a)return a[r.id];try{let i=await import(r.module);return a[r.id]=i,i}catch(i){return console.error(`Error loading route module \`${r.module}\`, reloading page...`),console.error(i),window.__reactRouterContext&&window.__reactRouterContext.isSpaMode,window.location.reload(),new Promise(()=>{})}}function Ex(r){return r==null?!1:r.href==null?r.rel==="preload"&&typeof r.imageSrcSet=="string"&&typeof r.imageSizes=="string":typeof r.rel=="string"&&typeof r.href=="string"}async function Rx(r,a,i){let o=await Promise.all(r.map(async s=>{let f=a.routes[s.route.id];if(f){let d=await Sx(f,i);return d.links?d.links():[]}return[]}));return Ox(o.flat(1).filter(Ex).filter(s=>s.rel==="stylesheet"||s.rel==="preload").map(s=>s.rel==="stylesheet"?{...s,rel:"prefetch",as:"style"}:{...s,rel:"prefetch"}))}function Km(r,a,i,o,s,f){let d=(g,m)=>i[m]?g.route.id!==i[m].route.id:!0,h=(g,m)=>{var y;return i[m].pathname!==g.pathname||((y=i[m].route.path)==null?void 0:y.endsWith("*"))&&i[m].params["*"]!==g.params["*"]};return f==="assets"?a.filter((g,m)=>d(g,m)||h(g,m)):f==="data"?a.filter((g,m)=>{var v;let y=o.routes[g.route.id];if(!y||!y.hasLoader)return!1;if(d(g,m)||h(g,m))return!0;if(g.route.shouldRevalidate){let w=g.route.shouldRevalidate({currentUrl:new URL(s.pathname+s.search+s.hash,window.origin),currentParams:((v=i[0])==null?void 0:v.params)||{},nextUrl:new URL(r,window.origin),nextParams:g.params,defaultShouldRevalidate:!0});if(typeof w=="boolean")return w}return!0}):[]}function Tx(r,a,{includeHydrateFallback:i}={}){return Ax(r.map(o=>{let s=a.routes[o.route.id];if(!s)return[];let f=[s.module];return s.clientActionModule&&(f=f.concat(s.clientActionModule)),s.clientLoaderModule&&(f=f.concat(s.clientLoaderModule)),i&&s.hydrateFallbackModule&&(f=f.concat(s.hydrateFallbackModule)),s.imports&&(f=f.concat(s.imports)),f}).flat(1))}function Ax(r){return[...new Set(r)]}function kx(r){let a={},i=Object.keys(r).sort();for(let o of i)a[o]=r[o];return a}function Ox(r,a){let i=new Set;return new Set(a),r.reduce((o,s)=>{let f=JSON.stringify(kx(s));return i.has(f)||(i.add(f),o.push({key:f,link:s})),o},[])}Object.getOwnPropertyNames(Object.prototype).sort().join("\0");var Cx=new Set([100,101,204,205]);function Mx(r,a){let i=typeof r=="string"?new URL(r,typeof window>"u"?"server://singlefetch/":window.location.origin):r;return i.pathname==="/"?i.pathname="_root.data":a&&Mn(i.pathname,a)==="/"?i.pathname=`${a.replace(/\/$/,"")}/_root.data`:i.pathname=`${i.pathname.replace(/\/$/,"")}.data`,i}function tp(){let r=T.useContext(_l);return bf(r,"You must render this element inside a <DataRouterContext.Provider> element"),r}function Nx(){let r=T.useContext($i);return bf(r,"You must render this element inside a <DataRouterStateContext.Provider> element"),r}var vf=T.createContext(void 0);vf.displayName="FrameworkContext";function np(){let r=T.useContext(vf);return bf(r,"You must render this element inside a <HydratedRouter> element"),r}function zx(r,a){let i=T.useContext(vf),[o,s]=T.useState(!1),[f,d]=T.useState(!1),{onFocus:h,onBlur:g,onMouseEnter:m,onMouseLeave:y,onTouchStart:v}=a,w=T.useRef(null);T.useEffect(()=>{if(r==="render"&&d(!0),r==="viewport"){let R=O=>{O.forEach(M=>{d(M.isIntersecting)})},k=new IntersectionObserver(R,{threshold:.5});return w.current&&k.observe(w.current),()=>{k.disconnect()}}},[r]),T.useEffect(()=>{if(o){let R=setTimeout(()=>{d(!0)},100);return()=>{clearTimeout(R)}}},[o]);let S=()=>{s(!0)},x=()=>{s(!1),d(!1)};return i?r!=="intent"?[f,w,{}]:[f,w,{onFocus:Qa(h,S),onBlur:Qa(g,x),onMouseEnter:Qa(m,S),onMouseLeave:Qa(y,x),onTouchStart:Qa(v,S)}]:[!1,w,{}]}function Qa(r,a){return i=>{r&&r(i),i.defaultPrevented||a(i)}}function _x({page:r,...a}){let{router:i}=tp(),o=T.useMemo(()=>Qg(i.routes,r,i.basename),[i.routes,r,i.basename]);return o?T.createElement(Ux,{page:r,matches:o,...a}):null}function Dx(r){let{manifest:a,routeModules:i}=np(),[o,s]=T.useState([]);return T.useEffect(()=>{let f=!1;return Rx(r,a,i).then(d=>{f||s(d)}),()=>{f=!0}},[r,a,i]),o}function Ux({page:r,matches:a,...i}){let o=cr(),{manifest:s,routeModules:f}=np(),{basename:d}=tp(),{loaderData:h,matches:g}=Nx(),m=T.useMemo(()=>Km(r,a,g,s,o,"data"),[r,a,g,s,o]),y=T.useMemo(()=>Km(r,a,g,s,o,"assets"),[r,a,g,s,o]),v=T.useMemo(()=>{if(r===o.pathname+o.search+o.hash)return[];let x=new Set,R=!1;if(a.forEach(O=>{var H;let M=s.routes[O.route.id];!M||!M.hasLoader||(!m.some($=>$.route.id===O.route.id)&&O.route.id in h&&((H=f[O.route.id])!=null&&H.shouldRevalidate)||M.hasClientLoader?R=!0:x.add(O.route.id))}),x.size===0)return[];let k=Mx(r,d);return R&&x.size>0&&k.searchParams.set("_routes",a.filter(O=>x.has(O.route.id)).map(O=>O.route.id).join(",")),[k.pathname+k.search]},[d,h,o,s,m,a,r,f]),w=T.useMemo(()=>Tx(y,s),[y,s]),S=Dx(y);return T.createElement(T.Fragment,null,v.map(x=>T.createElement("link",{key:x,rel:"prefetch",as:"fetch",href:x,...i})),w.map(x=>T.createElement("link",{key:x,rel:"modulepreload",href:x,...i})),S.map(({key:x,link:R})=>T.createElement("link",{key:x,...R})))}function jx(...r){return a=>{r.forEach(i=>{typeof i=="function"?i(a):i!=null&&(i.current=a)})}}var rp=typeof window<"u"&&typeof window.document<"u"&&typeof window.document.createElement<"u";try{rp&&(window.__reactRouterVersion="7.6.3")}catch{}function Lx({basename:r,children:a,window:i}){let o=T.useRef();o.current==null&&(o.current=S1({window:i,v5Compat:!0}));let s=o.current,[f,d]=T.useState({action:s.action,location:s.location}),h=T.useCallback(g=>{T.startTransition(()=>d(g))},[d]);return T.useLayoutEffect(()=>s.listen(h),[s,h]),T.createElement(dx,{basename:r,children:a,location:f.location,navigationType:f.action,navigator:s})}var lp=/^(?:[a-z][a-z0-9+.-]*:|\/\/)/i,ap=T.forwardRef(function({onClick:a,discover:i="render",prefetch:o="none",relative:s,reloadDocument:f,replace:d,state:h,target:g,to:m,preventScrollReset:y,viewTransition:v,...w},S){let{basename:x}=T.useContext(Ft),R=typeof m=="string"&&lp.test(m),k,O=!1;if(typeof m=="string"&&R&&(k=m,rp))try{let ne=new URL(window.location.href),se=m.startsWith("//")?new URL(ne.protocol+m):new URL(m),me=Mn(se.pathname,x);se.origin===ne.origin&&me!=null?m=me+se.search+se.hash:O=!0}catch{Pt(!1,`<Link to="${m}"> contains an invalid URL which will probably break when clicked - please update to a valid URL path.`)}let M=J1(m,{relative:s}),[H,$,V]=zx(o,w),W=Gx(m,{replace:d,state:h,target:g,preventScrollReset:y,relative:s,viewTransition:v});function ee(ne){a&&a(ne),ne.defaultPrevented||W(ne)}let fe=T.createElement("a",{...w,...V,href:k||M,onClick:O||f?a:ee,ref:jx(S,$),target:g,"data-discover":!R&&i==="render"?"true":void 0});return H&&!R?T.createElement(T.Fragment,null,fe,T.createElement(_x,{page:M})):fe});ap.displayName="Link";var Bx=T.forwardRef(function({"aria-current":a="page",caseSensitive:i=!1,className:o="",end:s=!1,style:f,to:d,viewTransition:h,children:g,...m},y){let v=to(d,{relative:m.relative}),w=cr(),S=T.useContext($i),{navigator:x,basename:R}=T.useContext(Ft),k=S!=null&&Zx(v)&&h===!0,O=x.encodeLocation?x.encodeLocation(v).pathname:v.pathname,M=w.pathname,H=S&&S.navigation&&S.navigation.location?S.navigation.location.pathname:null;i||(M=M.toLowerCase(),H=H?H.toLowerCase():null,O=O.toLowerCase()),H&&R&&(H=Mn(H,R)||H);const $=O!=="/"&&O.endsWith("/")?O.length-1:O.length;let V=M===O||!s&&M.startsWith(O)&&M.charAt($)==="/",W=H!=null&&(H===O||!s&&H.startsWith(O)&&H.charAt(O.length)==="/"),ee={isActive:V,isPending:W,isTransitioning:k},fe=V?a:void 0,ne;typeof o=="function"?ne=o(ee):ne=[o,V?"active":null,W?"pending":null,k?"transitioning":null].filter(Boolean).join(" ");let se=typeof f=="function"?f(ee):f;return T.createElement(ap,{...m,"aria-current":fe,className:ne,ref:y,style:se,to:d,viewTransition:h},typeof g=="function"?g(ee):g)});Bx.displayName="NavLink";var Hx=T.forwardRef(({discover:r="render",fetcherKey:a,navigate:i,reloadDocument:o,replace:s,state:f,method:d=ki,action:h,onSubmit:g,relative:m,preventScrollReset:y,viewTransition:v,...w},S)=>{let x=Xx(),R=Qx(h,{relative:m}),k=d.toLowerCase()==="get"?"get":"post",O=typeof h=="string"&&lp.test(h),M=H=>{if(g&&g(H),H.defaultPrevented)return;H.preventDefault();let $=H.nativeEvent.submitter,V=($==null?void 0:$.getAttribute("formmethod"))||d;x($||H.currentTarget,{fetcherKey:a,method:V,navigate:i,replace:s,state:f,relative:m,preventScrollReset:y,viewTransition:v})};return T.createElement("form",{ref:S,method:k,action:R,onSubmit:o?g:M,...w,"data-discover":!O&&r==="render"?"true":void 0})});Hx.displayName="Form";function qx(r){return`${r} must be used within a data router.  See https://reactrouter.com/en/main/routers/picking-a-router.`}function op(r){let a=T.useContext(_l);return Xe(a,qx(r)),a}function Gx(r,{target:a,replace:i,state:o,preventScrollReset:s,relative:f,viewTransition:d}={}){let h=mf(),g=cr(),m=to(r,{relative:f});return T.useCallback(y=>{if(bx(y,a)){y.preventDefault();let v=i!==void 0?i:Fa(g)===Fa(m);h(r,{replace:v,state:o,preventScrollReset:s,relative:f,viewTransition:d})}},[g,h,m,i,o,a,r,s,f,d])}var Yx=0,Vx=()=>`__${String(++Yx)}__`;function Xx(){let{router:r}=op("useSubmit"),{basename:a}=T.useContext(Ft),i=ux();return T.useCallback(async(o,s={})=>{let{action:f,method:d,encType:h,formData:g,body:m}=wx(o,a);if(s.navigate===!1){let y=s.fetcherKey||Vx();await r.fetch(y,i,s.action||f,{preventScrollReset:s.preventScrollReset,formData:g,body:m,formMethod:s.method||d,formEncType:s.encType||h,flushSync:s.flushSync})}else await r.navigate(s.action||f,{preventScrollReset:s.preventScrollReset,formData:g,body:m,formMethod:s.method||d,formEncType:s.encType||h,replace:s.replace,state:s.state,fromRouteId:i,flushSync:s.flushSync,viewTransition:s.viewTransition})},[r,a,i])}function Qx(r,{relative:a}={}){let{basename:i}=T.useContext(Ft),o=T.useContext(Wt);Xe(o,"useFormAction must be used inside a RouteContext");let[s]=o.matches.slice(-1),f={...to(r||".",{relative:a})},d=cr();if(r==null){f.search=d.search;let h=new URLSearchParams(f.search),g=h.getAll("index");if(g.some(y=>y==="")){h.delete("index"),g.filter(v=>v).forEach(v=>h.append("index",v));let y=h.toString();f.search=y?`?${y}`:""}}return(!r||r===".")&&s.route.index&&(f.search=f.search?f.search.replace(/^\?/,"?index&"):"?index"),i!=="/"&&(f.pathname=f.pathname==="/"?i:On([i,f.pathname])),Fa(f)}function Zx(r,a={}){let i=T.useContext(Jg);Xe(i!=null,"`useViewTransitionState` must be used within `react-router-dom`'s `RouterProvider`.  Did you accidentally import `RouterProvider` from `react-router`?");let{basename:o}=op("useViewTransitionState"),s=to(r,{relative:a.relative});if(!i.isTransitioning)return!1;let f=Mn(i.currentLocation.pathname,o)||i.currentLocation.pathname,d=Mn(i.nextLocation.pathname,o)||i.nextLocation.pathname;return Li(s.pathname,d)!=null||Li(s.pathname,f)!=null}[...Cx];var Kx=Xg();function pe(r,a){const i=a.split(".");let o=r;for(const s of i){if(typeof o=="boolean"||typeof o=="string")return o;if(o==null||typeof o!="object")return;o=o[s]}return o}function $x(r){const{theme:a,clearTheme:i,applyTheme:o,...s}=r;return s}function at(r,a){let i=$x(r);return a&&(i={...a,...r}),i}const fr={defaultMerge:Symbol("deepmerge-ts: default merge"),skip:Symbol("deepmerge-ts: skip")};fr.defaultMerge;function Jx(r,a){return a}function Px(r,a){return r.filter(i=>i!==void 0)}var $m;(function(r){r[r.NOT=0]="NOT",r[r.RECORD=1]="RECORD",r[r.ARRAY=2]="ARRAY",r[r.SET=3]="SET",r[r.MAP=4]="MAP",r[r.OTHER=5]="OTHER"})($m||($m={}));function Jm(r){return typeof r!="object"||r===null?0:Array.isArray(r)?2:Ix(r)?1:r instanceof Set?3:r instanceof Map?4:5}function Fx(r){const a=new Set;for(const i of r)for(const o of[...Object.keys(i),...Object.getOwnPropertySymbols(i)])a.add(o);return a}function Wx(r,a){return typeof r=="object"&&Object.prototype.propertyIsEnumerable.call(r,a)}function ip(r){var o;let a=0,i=(o=r[0])==null?void 0:o[Symbol.iterator]();return{[Symbol.iterator](){return{next(){var s;do{if(i===void 0)return{done:!0,value:void 0};const f=i.next();if(f.done===!0){a+=1,i=(s=r[a])==null?void 0:s[Symbol.iterator]();continue}return{done:!1,value:f.value}}while(!0)}}}}}const Pm=["[object Object]","[object Module]"];function Ix(r){if(!Pm.includes(Object.prototype.toString.call(r)))return!1;const{constructor:a}=r;if(a===void 0)return!0;const i=a.prototype;return!(i===null||typeof i!="object"||!Pm.includes(Object.prototype.toString.call(i))||!i.hasOwnProperty("isPrototypeOf"))}function ew(r,a,i){const o={};for(const s of Fx(r)){const f=[];for(const g of r)Wx(g,s)&&f.push(g[s]);if(f.length===0)continue;const d=a.metaDataUpdater(i,{key:s,parents:r}),h=sp(f,a,d);h!==fr.skip&&(s==="__proto__"?Object.defineProperty(o,s,{value:h,configurable:!0,enumerable:!0,writable:!0}):o[s]=h)}return o}function tw(r){return r.flat()}function nw(r){return new Set(ip(r))}function rw(r){return new Map(ip(r))}function lw(r){return r.at(-1)}const Si={mergeRecords:ew,mergeArrays:tw,mergeSets:nw,mergeMaps:rw,mergeOthers:lw};function aw(...r){return up({})(...r)}function up(r,a){const i=ow(r,o);function o(...s){return sp(s,i,a)}return o}function ow(r,a){return{defaultMergeFunctions:Si,mergeFunctions:{...Si,...Object.fromEntries(Object.entries(r).filter(([i,o])=>Object.hasOwn(Si,i)).map(([i,o])=>o===!1?[i,Si.mergeOthers]:[i,o]))},metaDataUpdater:r.metaDataUpdater??Jx,deepmerge:a,useImplicitDefaultMerging:r.enableImplicitDefaultMerging??!1,filterValues:r.filterValues===!1?void 0:r.filterValues??Px,actions:fr}}function sp(r,a,i){var f;const o=((f=a.filterValues)==null?void 0:f.call(a,r,i))??r;if(o.length===0)return;if(o.length===1)return Mc(o,a,i);const s=Jm(o[0]);if(s!==0&&s!==5){for(let d=1;d<o.length;d++)if(Jm(o[d])!==s)return Mc(o,a,i)}switch(s){case 1:return iw(o,a,i);case 2:return uw(o,a,i);case 3:return sw(o,a,i);case 4:return cw(o,a,i);default:return Mc(o,a,i)}}function iw(r,a,i){const o=a.mergeFunctions.mergeRecords(r,a,i);return o===fr.defaultMerge||a.useImplicitDefaultMerging&&o===void 0&&a.mergeFunctions.mergeRecords!==a.defaultMergeFunctions.mergeRecords?a.defaultMergeFunctions.mergeRecords(r,a,i):o}function uw(r,a,i){const o=a.mergeFunctions.mergeArrays(r,a,i);return o===fr.defaultMerge||a.useImplicitDefaultMerging&&o===void 0&&a.mergeFunctions.mergeArrays!==a.defaultMergeFunctions.mergeArrays?a.defaultMergeFunctions.mergeArrays(r):o}function sw(r,a,i){const o=a.mergeFunctions.mergeSets(r,a,i);return o===fr.defaultMerge||a.useImplicitDefaultMerging&&o===void 0&&a.mergeFunctions.mergeSets!==a.defaultMergeFunctions.mergeSets?a.defaultMergeFunctions.mergeSets(r):o}function cw(r,a,i){const o=a.mergeFunctions.mergeMaps(r,a,i);return o===fr.defaultMerge||a.useImplicitDefaultMerging&&o===void 0&&a.mergeFunctions.mergeMaps!==a.defaultMergeFunctions.mergeMaps?a.defaultMergeFunctions.mergeMaps(r):o}function Mc(r,a,i){const o=a.mergeFunctions.mergeOthers(r,a,i);return o===fr.defaultMerge||a.useImplicitDefaultMerging&&o===void 0&&a.mergeFunctions.mergeOthers!==a.defaultMergeFunctions.mergeOthers?a.defaultMergeFunctions.mergeOthers(r):o}function Ci(r){var a,i,o;if(Array.isArray(r)){for(i=Array(a=r.length);a--;)i[a]=(o=r[a])&&typeof o=="object"?Ci(o):o;return i}if(Object.prototype.toString.call(r)==="[object Object]"){i={};for(a in r)a==="__proto__"?Object.defineProperty(i,a,{value:Ci(r[a]),configurable:!0,enumerable:!0,writable:!0}):i[a]=(o=r[a])&&typeof o=="object"?Ci(o):o;return i}return r}const xf={dark:void 0,mode:void 0,prefix:void 0};function fw(){return xf.dark}function dw(){return xf.mode}function cp(){return xf.prefix}const Fm=new Map;function hw(r){if(!r.trim().length)return r;const a=r,i=Fm.get(a);if(i)return i;const s=r.split(/(\s+)/).map(f=>{if(/^\s+$/.test(f))return f;const d=f,h=d.match(/^([^:]+:)?(.+)$/);if(h){const[,g="",m]=h;for(const[y,v]of mw)if(y.test(m))return g+m.replace(y,v)}return d}).join("");return Fm.set(a,s),s}const mw=[[/^shadow-sm$/,"shadow-xs"],[/^shadow$/,"shadow-sm"],[/^drop-shadow-sm$/,"drop-shadow-xs"],[/^drop-shadow$/,"drop-shadow-sm"],[/^blur-sm$/,"blur-xs"],[/^blur$/,"blur-sm"],[/^rounded-sm$/,"rounded-xs"],[/^rounded$/,"rounded-sm"],[/^ring$/,"ring-3"]];function gw(r){return up({mergeOthers:(a,i)=>{if(a.some(o=>typeof o=="string")){const o=a.filter(d=>typeof d=="string"),s=new Set,f=[];for(const d of o){const h=[...new Set(d.split(/\s+/))];f.push(h.filter(g=>!s.has(g)).join(" "));for(const g of h)s.add(g)}return r(f)}return i.actions.defaultMerge}})}const pw="4.1.11";function fp(){try{return parseInt(pw.split(".")[0],10)}catch{return}}function Qc(r,a){if(r===a)return!0;if(r&&a&&typeof r=="object"&&typeof a=="object"){if(r.constructor!==a.constructor)return!1;if(Array.isArray(r))return r.length!==a.length?!1:r.every((o,s)=>Qc(o,a[s]));if(r.constructor===RegExp)return r.source===a.source&&r.flags===a.flags;if(r.valueOf!==Object.prototype.valueOf)return r.valueOf()===a.valueOf();if(r.toString!==Object.prototype.toString)return r.toString()===a.toString();const i=Object.keys(r);return i.length!==Object.keys(a).length?!1:i.every(o=>Object.prototype.hasOwnProperty.call(a,o)&&Qc(r[o],a[o]))}return r!==r&&a!==a}const wf="-",yw=r=>{const a=vw(r),{conflictingClassGroups:i,conflictingClassGroupModifiers:o}=r;return{getClassGroupId:d=>{const h=d.split(wf);return h[0]===""&&h.length!==1&&h.shift(),dp(h,a)||bw(d)},getConflictingClassGroupIds:(d,h)=>{const g=i[d]||[];return h&&o[d]?[...g,...o[d]]:g}}},dp=(r,a)=>{var d;if(r.length===0)return a.classGroupId;const i=r[0],o=a.nextPart.get(i),s=o?dp(r.slice(1),o):void 0;if(s)return s;if(a.validators.length===0)return;const f=r.join(wf);return(d=a.validators.find(({validator:h})=>h(f)))==null?void 0:d.classGroupId},Wm=/^\[(.+)\]$/,bw=r=>{if(Wm.test(r)){const a=Wm.exec(r)[1],i=a==null?void 0:a.substring(0,a.indexOf(":"));if(i)return"arbitrary.."+i}},vw=r=>{const{theme:a,prefix:i}=r,o={nextPart:new Map,validators:[]};return ww(Object.entries(r.classGroups),i).forEach(([f,d])=>{Zc(d,o,f,a)}),o},Zc=(r,a,i,o)=>{r.forEach(s=>{if(typeof s=="string"){const f=s===""?a:Im(a,s);f.classGroupId=i;return}if(typeof s=="function"){if(xw(s)){Zc(s(o),a,i,o);return}a.validators.push({validator:s,classGroupId:i});return}Object.entries(s).forEach(([f,d])=>{Zc(d,Im(a,f),i,o)})})},Im=(r,a)=>{let i=r;return a.split(wf).forEach(o=>{i.nextPart.has(o)||i.nextPart.set(o,{nextPart:new Map,validators:[]}),i=i.nextPart.get(o)}),i},xw=r=>r.isThemeGetter,ww=(r,a)=>a?r.map(([i,o])=>{const s=o.map(f=>typeof f=="string"?a+f:typeof f=="object"?Object.fromEntries(Object.entries(f).map(([d,h])=>[a+d,h])):f);return[i,s]}):r,Sw=r=>{if(r<1)return{get:()=>{},set:()=>{}};let a=0,i=new Map,o=new Map;const s=(f,d)=>{i.set(f,d),a++,a>r&&(a=0,o=i,i=new Map)};return{get(f){let d=i.get(f);if(d!==void 0)return d;if((d=o.get(f))!==void 0)return s(f,d),d},set(f,d){i.has(f)?i.set(f,d):s(f,d)}}},hp="!",Ew=r=>{const{separator:a,experimentalParseClassName:i}=r,o=a.length===1,s=a[0],f=a.length,d=h=>{const g=[];let m=0,y=0,v;for(let k=0;k<h.length;k++){let O=h[k];if(m===0){if(O===s&&(o||h.slice(k,k+f)===a)){g.push(h.slice(y,k)),y=k+f;continue}if(O==="/"){v=k;continue}}O==="["?m++:O==="]"&&m--}const w=g.length===0?h:h.substring(y),S=w.startsWith(hp),x=S?w.substring(1):w,R=v&&v>y?v-y:void 0;return{modifiers:g,hasImportantModifier:S,baseClassName:x,maybePostfixModifierPosition:R}};return i?h=>i({className:h,parseClassName:d}):d},Rw=r=>{if(r.length<=1)return r;const a=[];let i=[];return r.forEach(o=>{o[0]==="["?(a.push(...i.sort(),o),i=[]):i.push(o)}),a.push(...i.sort()),a},Tw=r=>({cache:Sw(r.cacheSize),parseClassName:Ew(r),...yw(r)}),Aw=/\s+/,kw=(r,a)=>{const{parseClassName:i,getClassGroupId:o,getConflictingClassGroupIds:s}=a,f=[],d=r.trim().split(Aw);let h="";for(let g=d.length-1;g>=0;g-=1){const m=d[g],{modifiers:y,hasImportantModifier:v,baseClassName:w,maybePostfixModifierPosition:S}=i(m);let x=!!S,R=o(x?w.substring(0,S):w);if(!R){if(!x){h=m+(h.length>0?" "+h:h);continue}if(R=o(w),!R){h=m+(h.length>0?" "+h:h);continue}x=!1}const k=Rw(y).join(":"),O=v?k+hp:k,M=O+R;if(f.includes(M))continue;f.push(M);const H=s(R,x);for(let $=0;$<H.length;++$){const V=H[$];f.push(O+V)}h=m+(h.length>0?" "+h:h)}return h};function Ow(){let r=0,a,i,o="";for(;r<arguments.length;)(a=arguments[r++])&&(i=mp(a))&&(o&&(o+=" "),o+=i);return o}const mp=r=>{if(typeof r=="string")return r;let a,i="";for(let o=0;o<r.length;o++)r[o]&&(a=mp(r[o]))&&(i&&(i+=" "),i+=a);return i};function eg(r,...a){let i,o,s,f=d;function d(g){const m=a.reduce((y,v)=>v(y),r());return i=Tw(m),o=i.cache.get,s=i.cache.set,f=h,h(g)}function h(g){const m=o(g);if(m)return m;const y=kw(g,i);return s(g,y),y}return function(){return f(Ow.apply(null,arguments))}}const Ye=r=>{const a=i=>i[r]||[];return a.isThemeGetter=!0,a},gp=/^\[(?:([a-z-]+):)?(.+)\]$/i,Cw=/^\d+\/\d+$/,Mw=new Set(["px","full","screen"]),Nw=/^(\d+(\.\d+)?)?(xs|sm|md|lg|xl)$/,zw=/\d+(%|px|r?em|[sdl]?v([hwib]|min|max)|pt|pc|in|cm|mm|cap|ch|ex|r?lh|cq(w|h|i|b|min|max))|\b(calc|min|max|clamp)\(.+\)|^0$/,_w=/^(rgba?|hsla?|hwb|(ok)?(lab|lch))\(.+\)$/,Dw=/^(inset_)?-?((\d+)?\.?(\d+)[a-z]+|0)_-?((\d+)?\.?(\d+)[a-z]+|0)/,Uw=/^(url|image|image-set|cross-fade|element|(repeating-)?(linear|radial|conic)-gradient)\(.+\)$/,An=r=>kl(r)||Mw.has(r)||Cw.test(r),rr=r=>Ul(r,"length",Vw),kl=r=>!!r&&!Number.isNaN(Number(r)),Nc=r=>Ul(r,"number",kl),Za=r=>!!r&&Number.isInteger(Number(r)),jw=r=>r.endsWith("%")&&kl(r.slice(0,-1)),we=r=>gp.test(r),lr=r=>Nw.test(r),Lw=new Set(["length","size","percentage"]),Bw=r=>Ul(r,Lw,pp),Hw=r=>Ul(r,"position",pp),qw=new Set(["image","url"]),Gw=r=>Ul(r,qw,Qw),Yw=r=>Ul(r,"",Xw),Ka=()=>!0,Ul=(r,a,i)=>{const o=gp.exec(r);return o?o[1]?typeof a=="string"?o[1]===a:a.has(o[1]):i(o[2]):!1},Vw=r=>zw.test(r)&&!_w.test(r),pp=()=>!1,Xw=r=>Dw.test(r),Qw=r=>Uw.test(r),tg=()=>{const r=Ye("colors"),a=Ye("spacing"),i=Ye("blur"),o=Ye("brightness"),s=Ye("borderColor"),f=Ye("borderRadius"),d=Ye("borderSpacing"),h=Ye("borderWidth"),g=Ye("contrast"),m=Ye("grayscale"),y=Ye("hueRotate"),v=Ye("invert"),w=Ye("gap"),S=Ye("gradientColorStops"),x=Ye("gradientColorStopPositions"),R=Ye("inset"),k=Ye("margin"),O=Ye("opacity"),M=Ye("padding"),H=Ye("saturate"),$=Ye("scale"),V=Ye("sepia"),W=Ye("skew"),ee=Ye("space"),fe=Ye("translate"),ne=()=>["auto","contain","none"],se=()=>["auto","hidden","clip","visible","scroll"],me=()=>["auto",we,a],te=()=>[we,a],ue=()=>["",An,rr],X=()=>["auto",kl,we],I=()=>["bottom","center","left","left-bottom","left-top","right","right-bottom","right-top","top"],N=()=>["solid","dashed","dotted","double","none"],q=()=>["normal","multiply","screen","overlay","darken","lighten","color-dodge","color-burn","hard-light","soft-light","difference","exclusion","hue","saturation","color","luminosity"],L=()=>["start","end","center","between","around","evenly","stretch"],K=()=>["","0",we],E=()=>["auto","avoid","all","avoid-page","page","left","right","column"],_=()=>[kl,we];return{cacheSize:500,separator:":",theme:{colors:[Ka],spacing:[An,rr],blur:["none","",lr,we],brightness:_(),borderColor:[r],borderRadius:["none","","full",lr,we],borderSpacing:te(),borderWidth:ue(),contrast:_(),grayscale:K(),hueRotate:_(),invert:K(),gap:te(),gradientColorStops:[r],gradientColorStopPositions:[jw,rr],inset:me(),margin:me(),opacity:_(),padding:te(),saturate:_(),scale:_(),sepia:K(),skew:_(),space:te(),translate:te()},classGroups:{aspect:[{aspect:["auto","square","video",we]}],container:["container"],columns:[{columns:[lr]}],"break-after":[{"break-after":E()}],"break-before":[{"break-before":E()}],"break-inside":[{"break-inside":["auto","avoid","avoid-page","avoid-column"]}],"box-decoration":[{"box-decoration":["slice","clone"]}],box:[{box:["border","content"]}],display:["block","inline-block","inline","flex","inline-flex","table","inline-table","table-caption","table-cell","table-column","table-column-group","table-footer-group","table-header-group","table-row-group","table-row","flow-root","grid","inline-grid","contents","list-item","hidden"],float:[{float:["right","left","none","start","end"]}],clear:[{clear:["left","right","both","none","start","end"]}],isolation:["isolate","isolation-auto"],"object-fit":[{object:["contain","cover","fill","none","scale-down"]}],"object-position":[{object:[...I(),we]}],overflow:[{overflow:se()}],"overflow-x":[{"overflow-x":se()}],"overflow-y":[{"overflow-y":se()}],overscroll:[{overscroll:ne()}],"overscroll-x":[{"overscroll-x":ne()}],"overscroll-y":[{"overscroll-y":ne()}],position:["static","fixed","absolute","relative","sticky"],inset:[{inset:[R]}],"inset-x":[{"inset-x":[R]}],"inset-y":[{"inset-y":[R]}],start:[{start:[R]}],end:[{end:[R]}],top:[{top:[R]}],right:[{right:[R]}],bottom:[{bottom:[R]}],left:[{left:[R]}],visibility:["visible","invisible","collapse"],z:[{z:["auto",Za,we]}],basis:[{basis:me()}],"flex-direction":[{flex:["row","row-reverse","col","col-reverse"]}],"flex-wrap":[{flex:["wrap","wrap-reverse","nowrap"]}],flex:[{flex:["1","auto","initial","none",we]}],grow:[{grow:K()}],shrink:[{shrink:K()}],order:[{order:["first","last","none",Za,we]}],"grid-cols":[{"grid-cols":[Ka]}],"col-start-end":[{col:["auto",{span:["full",Za,we]},we]}],"col-start":[{"col-start":X()}],"col-end":[{"col-end":X()}],"grid-rows":[{"grid-rows":[Ka]}],"row-start-end":[{row:["auto",{span:[Za,we]},we]}],"row-start":[{"row-start":X()}],"row-end":[{"row-end":X()}],"grid-flow":[{"grid-flow":["row","col","dense","row-dense","col-dense"]}],"auto-cols":[{"auto-cols":["auto","min","max","fr",we]}],"auto-rows":[{"auto-rows":["auto","min","max","fr",we]}],gap:[{gap:[w]}],"gap-x":[{"gap-x":[w]}],"gap-y":[{"gap-y":[w]}],"justify-content":[{justify:["normal",...L()]}],"justify-items":[{"justify-items":["start","end","center","stretch"]}],"justify-self":[{"justify-self":["auto","start","end","center","stretch"]}],"align-content":[{content:["normal",...L(),"baseline"]}],"align-items":[{items:["start","end","center","baseline","stretch"]}],"align-self":[{self:["auto","start","end","center","stretch","baseline"]}],"place-content":[{"place-content":[...L(),"baseline"]}],"place-items":[{"place-items":["start","end","center","baseline","stretch"]}],"place-self":[{"place-self":["auto","start","end","center","stretch"]}],p:[{p:[M]}],px:[{px:[M]}],py:[{py:[M]}],ps:[{ps:[M]}],pe:[{pe:[M]}],pt:[{pt:[M]}],pr:[{pr:[M]}],pb:[{pb:[M]}],pl:[{pl:[M]}],m:[{m:[k]}],mx:[{mx:[k]}],my:[{my:[k]}],ms:[{ms:[k]}],me:[{me:[k]}],mt:[{mt:[k]}],mr:[{mr:[k]}],mb:[{mb:[k]}],ml:[{ml:[k]}],"space-x":[{"space-x":[ee]}],"space-x-reverse":["space-x-reverse"],"space-y":[{"space-y":[ee]}],"space-y-reverse":["space-y-reverse"],w:[{w:["auto","min","max","fit","svw","lvw","dvw",we,a]}],"min-w":[{"min-w":[we,a,"min","max","fit"]}],"max-w":[{"max-w":[we,a,"none","full","min","max","fit","prose",{screen:[lr]},lr]}],h:[{h:[we,a,"auto","min","max","fit","svh","lvh","dvh"]}],"min-h":[{"min-h":[we,a,"min","max","fit","svh","lvh","dvh"]}],"max-h":[{"max-h":[we,a,"min","max","fit","svh","lvh","dvh"]}],size:[{size:[we,a,"auto","min","max","fit"]}],"font-size":[{text:["base",lr,rr]}],"font-smoothing":["antialiased","subpixel-antialiased"],"font-style":["italic","not-italic"],"font-weight":[{font:["thin","extralight","light","normal","medium","semibold","bold","extrabold","black",Nc]}],"font-family":[{font:[Ka]}],"fvn-normal":["normal-nums"],"fvn-ordinal":["ordinal"],"fvn-slashed-zero":["slashed-zero"],"fvn-figure":["lining-nums","oldstyle-nums"],"fvn-spacing":["proportional-nums","tabular-nums"],"fvn-fraction":["diagonal-fractions","stacked-fractions"],tracking:[{tracking:["tighter","tight","normal","wide","wider","widest",we]}],"line-clamp":[{"line-clamp":["none",kl,Nc]}],leading:[{leading:["none","tight","snug","normal","relaxed","loose",An,we]}],"list-image":[{"list-image":["none",we]}],"list-style-type":[{list:["none","disc","decimal",we]}],"list-style-position":[{list:["inside","outside"]}],"placeholder-color":[{placeholder:[r]}],"placeholder-opacity":[{"placeholder-opacity":[O]}],"text-alignment":[{text:["left","center","right","justify","start","end"]}],"text-color":[{text:[r]}],"text-opacity":[{"text-opacity":[O]}],"text-decoration":["underline","overline","line-through","no-underline"],"text-decoration-style":[{decoration:[...N(),"wavy"]}],"text-decoration-thickness":[{decoration:["auto","from-font",An,rr]}],"underline-offset":[{"underline-offset":["auto",An,we]}],"text-decoration-color":[{decoration:[r]}],"text-transform":["uppercase","lowercase","capitalize","normal-case"],"text-overflow":["truncate","text-ellipsis","text-clip"],"text-wrap":[{text:["wrap","nowrap","balance","pretty"]}],indent:[{indent:te()}],"vertical-align":[{align:["baseline","top","middle","bottom","text-top","text-bottom","sub","super",we]}],whitespace:[{whitespace:["normal","nowrap","pre","pre-line","pre-wrap","break-spaces"]}],break:[{break:["normal","words","all","keep"]}],hyphens:[{hyphens:["none","manual","auto"]}],content:[{content:["none",we]}],"bg-attachment":[{bg:["fixed","local","scroll"]}],"bg-clip":[{"bg-clip":["border","padding","content","text"]}],"bg-opacity":[{"bg-opacity":[O]}],"bg-origin":[{"bg-origin":["border","padding","content"]}],"bg-position":[{bg:[...I(),Hw]}],"bg-repeat":[{bg:["no-repeat",{repeat:["","x","y","round","space"]}]}],"bg-size":[{bg:["auto","cover","contain",Bw]}],"bg-image":[{bg:["none",{"gradient-to":["t","tr","r","br","b","bl","l","tl"]},Gw]}],"bg-color":[{bg:[r]}],"gradient-from-pos":[{from:[x]}],"gradient-via-pos":[{via:[x]}],"gradient-to-pos":[{to:[x]}],"gradient-from":[{from:[S]}],"gradient-via":[{via:[S]}],"gradient-to":[{to:[S]}],rounded:[{rounded:[f]}],"rounded-s":[{"rounded-s":[f]}],"rounded-e":[{"rounded-e":[f]}],"rounded-t":[{"rounded-t":[f]}],"rounded-r":[{"rounded-r":[f]}],"rounded-b":[{"rounded-b":[f]}],"rounded-l":[{"rounded-l":[f]}],"rounded-ss":[{"rounded-ss":[f]}],"rounded-se":[{"rounded-se":[f]}],"rounded-ee":[{"rounded-ee":[f]}],"rounded-es":[{"rounded-es":[f]}],"rounded-tl":[{"rounded-tl":[f]}],"rounded-tr":[{"rounded-tr":[f]}],"rounded-br":[{"rounded-br":[f]}],"rounded-bl":[{"rounded-bl":[f]}],"border-w":[{border:[h]}],"border-w-x":[{"border-x":[h]}],"border-w-y":[{"border-y":[h]}],"border-w-s":[{"border-s":[h]}],"border-w-e":[{"border-e":[h]}],"border-w-t":[{"border-t":[h]}],"border-w-r":[{"border-r":[h]}],"border-w-b":[{"border-b":[h]}],"border-w-l":[{"border-l":[h]}],"border-opacity":[{"border-opacity":[O]}],"border-style":[{border:[...N(),"hidden"]}],"divide-x":[{"divide-x":[h]}],"divide-x-reverse":["divide-x-reverse"],"divide-y":[{"divide-y":[h]}],"divide-y-reverse":["divide-y-reverse"],"divide-opacity":[{"divide-opacity":[O]}],"divide-style":[{divide:N()}],"border-color":[{border:[s]}],"border-color-x":[{"border-x":[s]}],"border-color-y":[{"border-y":[s]}],"border-color-s":[{"border-s":[s]}],"border-color-e":[{"border-e":[s]}],"border-color-t":[{"border-t":[s]}],"border-color-r":[{"border-r":[s]}],"border-color-b":[{"border-b":[s]}],"border-color-l":[{"border-l":[s]}],"divide-color":[{divide:[s]}],"outline-style":[{outline:["",...N()]}],"outline-offset":[{"outline-offset":[An,we]}],"outline-w":[{outline:[An,rr]}],"outline-color":[{outline:[r]}],"ring-w":[{ring:ue()}],"ring-w-inset":["ring-inset"],"ring-color":[{ring:[r]}],"ring-opacity":[{"ring-opacity":[O]}],"ring-offset-w":[{"ring-offset":[An,rr]}],"ring-offset-color":[{"ring-offset":[r]}],shadow:[{shadow:["","inner","none",lr,Yw]}],"shadow-color":[{shadow:[Ka]}],opacity:[{opacity:[O]}],"mix-blend":[{"mix-blend":[...q(),"plus-lighter","plus-darker"]}],"bg-blend":[{"bg-blend":q()}],filter:[{filter:["","none"]}],blur:[{blur:[i]}],brightness:[{brightness:[o]}],contrast:[{contrast:[g]}],"drop-shadow":[{"drop-shadow":["","none",lr,we]}],grayscale:[{grayscale:[m]}],"hue-rotate":[{"hue-rotate":[y]}],invert:[{invert:[v]}],saturate:[{saturate:[H]}],sepia:[{sepia:[V]}],"backdrop-filter":[{"backdrop-filter":["","none"]}],"backdrop-blur":[{"backdrop-blur":[i]}],"backdrop-brightness":[{"backdrop-brightness":[o]}],"backdrop-contrast":[{"backdrop-contrast":[g]}],"backdrop-grayscale":[{"backdrop-grayscale":[m]}],"backdrop-hue-rotate":[{"backdrop-hue-rotate":[y]}],"backdrop-invert":[{"backdrop-invert":[v]}],"backdrop-opacity":[{"backdrop-opacity":[O]}],"backdrop-saturate":[{"backdrop-saturate":[H]}],"backdrop-sepia":[{"backdrop-sepia":[V]}],"border-collapse":[{border:["collapse","separate"]}],"border-spacing":[{"border-spacing":[d]}],"border-spacing-x":[{"border-spacing-x":[d]}],"border-spacing-y":[{"border-spacing-y":[d]}],"table-layout":[{table:["auto","fixed"]}],caption:[{caption:["top","bottom"]}],transition:[{transition:["none","all","","colors","opacity","shadow","transform",we]}],duration:[{duration:_()}],ease:[{ease:["linear","in","out","in-out",we]}],delay:[{delay:_()}],animate:[{animate:["none","spin","ping","pulse","bounce",we]}],transform:[{transform:["","gpu","none"]}],scale:[{scale:[$]}],"scale-x":[{"scale-x":[$]}],"scale-y":[{"scale-y":[$]}],rotate:[{rotate:[Za,we]}],"translate-x":[{"translate-x":[fe]}],"translate-y":[{"translate-y":[fe]}],"skew-x":[{"skew-x":[W]}],"skew-y":[{"skew-y":[W]}],"transform-origin":[{origin:["center","top","top-right","right","bottom-right","bottom","bottom-left","left","top-left",we]}],accent:[{accent:["auto",r]}],appearance:[{appearance:["none","auto"]}],cursor:[{cursor:["auto","default","pointer","wait","text","move","help","not-allowed","none","context-menu","progress","cell","crosshair","vertical-text","alias","copy","no-drop","grab","grabbing","all-scroll","col-resize","row-resize","n-resize","e-resize","s-resize","w-resize","ne-resize","nw-resize","se-resize","sw-resize","ew-resize","ns-resize","nesw-resize","nwse-resize","zoom-in","zoom-out",we]}],"caret-color":[{caret:[r]}],"pointer-events":[{"pointer-events":["none","auto"]}],resize:[{resize:["none","y","x",""]}],"scroll-behavior":[{scroll:["auto","smooth"]}],"scroll-m":[{"scroll-m":te()}],"scroll-mx":[{"scroll-mx":te()}],"scroll-my":[{"scroll-my":te()}],"scroll-ms":[{"scroll-ms":te()}],"scroll-me":[{"scroll-me":te()}],"scroll-mt":[{"scroll-mt":te()}],"scroll-mr":[{"scroll-mr":te()}],"scroll-mb":[{"scroll-mb":te()}],"scroll-ml":[{"scroll-ml":te()}],"scroll-p":[{"scroll-p":te()}],"scroll-px":[{"scroll-px":te()}],"scroll-py":[{"scroll-py":te()}],"scroll-ps":[{"scroll-ps":te()}],"scroll-pe":[{"scroll-pe":te()}],"scroll-pt":[{"scroll-pt":te()}],"scroll-pr":[{"scroll-pr":te()}],"scroll-pb":[{"scroll-pb":te()}],"scroll-pl":[{"scroll-pl":te()}],"snap-align":[{snap:["start","end","center","align-none"]}],"snap-stop":[{snap:["normal","always"]}],"snap-type":[{snap:["none","x","y","both"]}],"snap-strictness":[{snap:["mandatory","proximity"]}],touch:[{touch:["auto","none","manipulation"]}],"touch-x":[{"touch-pan":["x","left","right"]}],"touch-y":[{"touch-pan":["y","up","down"]}],"touch-pz":["touch-pinch-zoom"],select:[{select:["none","text","all","auto"]}],"will-change":[{"will-change":["auto","scroll","contents","transform",we]}],fill:[{fill:[r,"none"]}],"stroke-w":[{stroke:[An,rr,Nc]}],stroke:[{stroke:[r,"none"]}],sr:["sr-only","not-sr-only"],"forced-color-adjust":[{"forced-color-adjust":["auto","none"]}]},conflictingClassGroups:{overflow:["overflow-x","overflow-y"],overscroll:["overscroll-x","overscroll-y"],inset:["inset-x","inset-y","start","end","top","right","bottom","left"],"inset-x":["right","left"],"inset-y":["top","bottom"],flex:["basis","grow","shrink"],gap:["gap-x","gap-y"],p:["px","py","ps","pe","pt","pr","pb","pl"],px:["pr","pl"],py:["pt","pb"],m:["mx","my","ms","me","mt","mr","mb","ml"],mx:["mr","ml"],my:["mt","mb"],size:["w","h"],"font-size":["leading"],"fvn-normal":["fvn-ordinal","fvn-slashed-zero","fvn-figure","fvn-spacing","fvn-fraction"],"fvn-ordinal":["fvn-normal"],"fvn-slashed-zero":["fvn-normal"],"fvn-figure":["fvn-normal"],"fvn-spacing":["fvn-normal"],"fvn-fraction":["fvn-normal"],"line-clamp":["display","overflow"],rounded:["rounded-s","rounded-e","rounded-t","rounded-r","rounded-b","rounded-l","rounded-ss","rounded-se","rounded-ee","rounded-es","rounded-tl","rounded-tr","rounded-br","rounded-bl"],"rounded-s":["rounded-ss","rounded-es"],"rounded-e":["rounded-se","rounded-ee"],"rounded-t":["rounded-tl","rounded-tr"],"rounded-r":["rounded-tr","rounded-br"],"rounded-b":["rounded-br","rounded-bl"],"rounded-l":["rounded-tl","rounded-bl"],"border-spacing":["border-spacing-x","border-spacing-y"],"border-w":["border-w-s","border-w-e","border-w-t","border-w-r","border-w-b","border-w-l"],"border-w-x":["border-w-r","border-w-l"],"border-w-y":["border-w-t","border-w-b"],"border-color":["border-color-s","border-color-e","border-color-t","border-color-r","border-color-b","border-color-l"],"border-color-x":["border-color-r","border-color-l"],"border-color-y":["border-color-t","border-color-b"],"scroll-m":["scroll-mx","scroll-my","scroll-ms","scroll-me","scroll-mt","scroll-mr","scroll-mb","scroll-ml"],"scroll-mx":["scroll-mr","scroll-ml"],"scroll-my":["scroll-mt","scroll-mb"],"scroll-p":["scroll-px","scroll-py","scroll-ps","scroll-pe","scroll-pt","scroll-pr","scroll-pb","scroll-pl"],"scroll-px":["scroll-pr","scroll-pl"],"scroll-py":["scroll-pt","scroll-pb"],touch:["touch-x","touch-y","touch-pz"],"touch-x":["touch"],"touch-y":["touch"],"touch-pz":["touch"]},conflictingClassGroupModifiers:{"font-size":["leading"]}}},Zw=(r,{cacheSize:a,prefix:i,separator:o,experimentalParseClassName:s,extend:f={},override:d={}})=>{Ja(r,"cacheSize",a),Ja(r,"prefix",i),Ja(r,"separator",o),Ja(r,"experimentalParseClassName",s);for(const h in d)Kw(r[h],d[h]);for(const h in f)$w(r[h],f[h]);return r},Ja=(r,a,i)=>{i!==void 0&&(r[a]=i)},Kw=(r,a)=>{if(a)for(const i in a)Ja(r,i,a[i])},$w=(r,a)=>{if(a)for(const i in a){const o=a[i];o!==void 0&&(r[i]=(r[i]||[]).concat(o))}},Jw=(r,...a)=>typeof r=="function"?eg(tg,r,...a):eg(()=>Zw(tg(),r),...a),Sf="-",Pw=r=>{const a=Ww(r),{conflictingClassGroups:i,conflictingClassGroupModifiers:o}=r;return{getClassGroupId:d=>{const h=d.split(Sf);return h[0]===""&&h.length!==1&&h.shift(),yp(h,a)||Fw(d)},getConflictingClassGroupIds:(d,h)=>{const g=i[d]||[];return h&&o[d]?[...g,...o[d]]:g}}},yp=(r,a)=>{var d;if(r.length===0)return a.classGroupId;const i=r[0],o=a.nextPart.get(i),s=o?yp(r.slice(1),o):void 0;if(s)return s;if(a.validators.length===0)return;const f=r.join(Sf);return(d=a.validators.find(({validator:h})=>h(f)))==null?void 0:d.classGroupId},ng=/^\[(.+)\]$/,Fw=r=>{if(ng.test(r)){const a=ng.exec(r)[1],i=a==null?void 0:a.substring(0,a.indexOf(":"));if(i)return"arbitrary.."+i}},Ww=r=>{const{theme:a,classGroups:i}=r,o={nextPart:new Map,validators:[]};for(const s in i)Kc(i[s],o,s,a);return o},Kc=(r,a,i,o)=>{r.forEach(s=>{if(typeof s=="string"){const f=s===""?a:rg(a,s);f.classGroupId=i;return}if(typeof s=="function"){if(Iw(s)){Kc(s(o),a,i,o);return}a.validators.push({validator:s,classGroupId:i});return}Object.entries(s).forEach(([f,d])=>{Kc(d,rg(a,f),i,o)})})},rg=(r,a)=>{let i=r;return a.split(Sf).forEach(o=>{i.nextPart.has(o)||i.nextPart.set(o,{nextPart:new Map,validators:[]}),i=i.nextPart.get(o)}),i},Iw=r=>r.isThemeGetter,eS=r=>{if(r<1)return{get:()=>{},set:()=>{}};let a=0,i=new Map,o=new Map;const s=(f,d)=>{i.set(f,d),a++,a>r&&(a=0,o=i,i=new Map)};return{get(f){let d=i.get(f);if(d!==void 0)return d;if((d=o.get(f))!==void 0)return s(f,d),d},set(f,d){i.has(f)?i.set(f,d):s(f,d)}}},$c="!",Jc=":",tS=Jc.length,nS=r=>{const{prefix:a,experimentalParseClassName:i}=r;let o=s=>{const f=[];let d=0,h=0,g=0,m;for(let x=0;x<s.length;x++){let R=s[x];if(d===0&&h===0){if(R===Jc){f.push(s.slice(g,x)),g=x+tS;continue}if(R==="/"){m=x;continue}}R==="["?d++:R==="]"?d--:R==="("?h++:R===")"&&h--}const y=f.length===0?s:s.substring(g),v=rS(y),w=v!==y,S=m&&m>g?m-g:void 0;return{modifiers:f,hasImportantModifier:w,baseClassName:v,maybePostfixModifierPosition:S}};if(a){const s=a+Jc,f=o;o=d=>d.startsWith(s)?f(d.substring(s.length)):{isExternal:!0,modifiers:[],hasImportantModifier:!1,baseClassName:d,maybePostfixModifierPosition:void 0}}if(i){const s=o;o=f=>i({className:f,parseClassName:s})}return o},rS=r=>r.endsWith($c)?r.substring(0,r.length-1):r.startsWith($c)?r.substring(1):r,lS=r=>{const a=Object.fromEntries(r.orderSensitiveModifiers.map(o=>[o,!0]));return o=>{if(o.length<=1)return o;const s=[];let f=[];return o.forEach(d=>{d[0]==="["||a[d]?(s.push(...f.sort(),d),f=[]):f.push(d)}),s.push(...f.sort()),s}},aS=r=>({cache:eS(r.cacheSize),parseClassName:nS(r),sortModifiers:lS(r),...Pw(r)}),oS=/\s+/,iS=(r,a)=>{const{parseClassName:i,getClassGroupId:o,getConflictingClassGroupIds:s,sortModifiers:f}=a,d=[],h=r.trim().split(oS);let g="";for(let m=h.length-1;m>=0;m-=1){const y=h[m],{isExternal:v,modifiers:w,hasImportantModifier:S,baseClassName:x,maybePostfixModifierPosition:R}=i(y);if(v){g=y+(g.length>0?" "+g:g);continue}let k=!!R,O=o(k?x.substring(0,R):x);if(!O){if(!k){g=y+(g.length>0?" "+g:g);continue}if(O=o(x),!O){g=y+(g.length>0?" "+g:g);continue}k=!1}const M=f(w).join(":"),H=S?M+$c:M,$=H+O;if(d.includes($))continue;d.push($);const V=s(O,k);for(let W=0;W<V.length;++W){const ee=V[W];d.push(H+ee)}g=y+(g.length>0?" "+g:g)}return g};function uS(){let r=0,a,i,o="";for(;r<arguments.length;)(a=arguments[r++])&&(i=bp(a))&&(o&&(o+=" "),o+=i);return o}const bp=r=>{if(typeof r=="string")return r;let a,i="";for(let o=0;o<r.length;o++)r[o]&&(a=bp(r[o]))&&(i&&(i+=" "),i+=a);return i};function lg(r,...a){let i,o,s,f=d;function d(g){const m=a.reduce((y,v)=>v(y),r());return i=aS(m),o=i.cache.get,s=i.cache.set,f=h,h(g)}function h(g){const m=o(g);if(m)return m;const y=iS(g,i);return s(g,y),y}return function(){return f(uS.apply(null,arguments))}}const lt=r=>{const a=i=>i[r]||[];return a.isThemeGetter=!0,a},vp=/^\[(?:(\w[\w-]*):)?(.+)\]$/i,xp=/^\((?:(\w[\w-]*):)?(.+)\)$/i,sS=/^\d+\/\d+$/,cS=/^(\d+(\.\d+)?)?(xs|sm|md|lg|xl)$/,fS=/\d+(%|px|r?em|[sdl]?v([hwib]|min|max)|pt|pc|in|cm|mm|cap|ch|ex|r?lh|cq(w|h|i|b|min|max))|\b(calc|min|max|clamp)\(.+\)|^0$/,dS=/^(rgba?|hsla?|hwb|(ok)?(lab|lch))\(.+\)$/,hS=/^(inset_)?-?((\d+)?\.?(\d+)[a-z]+|0)_-?((\d+)?\.?(\d+)[a-z]+|0)/,mS=/^(url|image|image-set|cross-fade|element|(repeating-)?(linear|radial|conic)-gradient)\(.+\)$/,Rl=r=>sS.test(r),Ae=r=>!!r&&!Number.isNaN(Number(r)),Mr=r=>!!r&&Number.isInteger(Number(r)),ag=r=>r.endsWith("%")&&Ae(r.slice(0,-1)),ar=r=>cS.test(r),gS=()=>!0,pS=r=>fS.test(r)&&!dS.test(r),Ef=()=>!1,yS=r=>hS.test(r),bS=r=>mS.test(r),vS=r=>!re(r)&&!le(r),xS=r=>jl(r,Ep,Ef),re=r=>vp.test(r),Nr=r=>jl(r,Rp,pS),zc=r=>jl(r,NS,Ae),wS=r=>jl(r,wp,Ef),SS=r=>jl(r,Sp,bS),ES=r=>jl(r,Ef,yS),le=r=>xp.test(r),Ei=r=>Ll(r,Rp),RS=r=>Ll(r,zS),TS=r=>Ll(r,wp),AS=r=>Ll(r,Ep),kS=r=>Ll(r,Sp),OS=r=>Ll(r,_S,!0),jl=(r,a,i)=>{const o=vp.exec(r);return o?o[1]?a(o[1]):i(o[2]):!1},Ll=(r,a,i=!1)=>{const o=xp.exec(r);return o?o[1]?a(o[1]):i:!1},wp=r=>r==="position",CS=new Set(["image","url"]),Sp=r=>CS.has(r),MS=new Set(["length","size","percentage"]),Ep=r=>MS.has(r),Rp=r=>r==="length",NS=r=>r==="number",zS=r=>r==="family-name",_S=r=>r==="shadow",og=()=>{const r=lt("color"),a=lt("font"),i=lt("text"),o=lt("font-weight"),s=lt("tracking"),f=lt("leading"),d=lt("breakpoint"),h=lt("container"),g=lt("spacing"),m=lt("radius"),y=lt("shadow"),v=lt("inset-shadow"),w=lt("drop-shadow"),S=lt("blur"),x=lt("perspective"),R=lt("aspect"),k=lt("ease"),O=lt("animate"),M=()=>["auto","avoid","all","avoid-page","page","left","right","column"],H=()=>["bottom","center","left","left-bottom","left-top","right","right-bottom","right-top","top"],$=()=>["auto","hidden","clip","visible","scroll"],V=()=>["auto","contain","none"],W=()=>[Rl,"px","full","auto",le,re,g],ee=()=>[Mr,"none","subgrid",le,re],fe=()=>["auto",{span:["full",Mr,le,re]},le,re],ne=()=>[Mr,"auto",le,re],se=()=>["auto","min","max","fr",le,re],me=()=>[le,re,g],te=()=>["start","end","center","between","around","evenly","stretch","baseline"],ue=()=>["start","end","center","stretch"],X=()=>[le,re,g],I=()=>["px",...X()],N=()=>["px","auto",...X()],q=()=>[Rl,"auto","px","full","dvw","dvh","lvw","lvh","svw","svh","min","max","fit",le,re,g],L=()=>[r,le,re],K=()=>[ag,Nr],E=()=>["","none","full",m,le,re],_=()=>["",Ae,Ei,Nr],P=()=>["solid","dashed","dotted","double"],F=()=>["normal","multiply","screen","overlay","darken","lighten","color-dodge","color-burn","hard-light","soft-light","difference","exclusion","hue","saturation","color","luminosity"],oe=()=>["","none",S,le,re],be=()=>["center","top","top-right","right","bottom-right","bottom","bottom-left","left","top-left",le,re],ae=()=>["none",Ae,le,re],ke=()=>["none",Ae,le,re],de=()=>[Ae,le,re],Ce=()=>[Rl,"full","px",le,re,g];return{cacheSize:500,theme:{animate:["spin","ping","pulse","bounce"],aspect:["video"],blur:[ar],breakpoint:[ar],color:[gS],container:[ar],"drop-shadow":[ar],ease:["in","out","in-out"],font:[vS],"font-weight":["thin","extralight","light","normal","medium","semibold","bold","extrabold","black"],"inset-shadow":[ar],leading:["none","tight","snug","normal","relaxed","loose"],perspective:["dramatic","near","normal","midrange","distant","none"],radius:[ar],shadow:[ar],spacing:[Ae],text:[ar],tracking:["tighter","tight","normal","wide","wider","widest"]},classGroups:{aspect:[{aspect:["auto","square",Rl,re,le,R]}],container:["container"],columns:[{columns:[Ae,re,le,h]}],"break-after":[{"break-after":M()}],"break-before":[{"break-before":M()}],"break-inside":[{"break-inside":["auto","avoid","avoid-page","avoid-column"]}],"box-decoration":[{"box-decoration":["slice","clone"]}],box:[{box:["border","content"]}],display:["block","inline-block","inline","flex","inline-flex","table","inline-table","table-caption","table-cell","table-column","table-column-group","table-footer-group","table-header-group","table-row-group","table-row","flow-root","grid","inline-grid","contents","list-item","hidden"],sr:["sr-only","not-sr-only"],float:[{float:["right","left","none","start","end"]}],clear:[{clear:["left","right","both","none","start","end"]}],isolation:["isolate","isolation-auto"],"object-fit":[{object:["contain","cover","fill","none","scale-down"]}],"object-position":[{object:[...H(),re,le]}],overflow:[{overflow:$()}],"overflow-x":[{"overflow-x":$()}],"overflow-y":[{"overflow-y":$()}],overscroll:[{overscroll:V()}],"overscroll-x":[{"overscroll-x":V()}],"overscroll-y":[{"overscroll-y":V()}],position:["static","fixed","absolute","relative","sticky"],inset:[{inset:W()}],"inset-x":[{"inset-x":W()}],"inset-y":[{"inset-y":W()}],start:[{start:W()}],end:[{end:W()}],top:[{top:W()}],right:[{right:W()}],bottom:[{bottom:W()}],left:[{left:W()}],visibility:["visible","invisible","collapse"],z:[{z:[Mr,"auto",le,re]}],basis:[{basis:[Rl,"full","auto",le,re,h,g]}],"flex-direction":[{flex:["row","row-reverse","col","col-reverse"]}],"flex-wrap":[{flex:["nowrap","wrap","wrap-reverse"]}],flex:[{flex:[Ae,Rl,"auto","initial","none",re]}],grow:[{grow:["",Ae,le,re]}],shrink:[{shrink:["",Ae,le,re]}],order:[{order:[Mr,"first","last","none",le,re]}],"grid-cols":[{"grid-cols":ee()}],"col-start-end":[{col:fe()}],"col-start":[{"col-start":ne()}],"col-end":[{"col-end":ne()}],"grid-rows":[{"grid-rows":ee()}],"row-start-end":[{row:fe()}],"row-start":[{"row-start":ne()}],"row-end":[{"row-end":ne()}],"grid-flow":[{"grid-flow":["row","col","dense","row-dense","col-dense"]}],"auto-cols":[{"auto-cols":se()}],"auto-rows":[{"auto-rows":se()}],gap:[{gap:me()}],"gap-x":[{"gap-x":me()}],"gap-y":[{"gap-y":me()}],"justify-content":[{justify:[...te(),"normal"]}],"justify-items":[{"justify-items":[...ue(),"normal"]}],"justify-self":[{"justify-self":["auto",...ue()]}],"align-content":[{content:["normal",...te()]}],"align-items":[{items:[...ue(),"baseline"]}],"align-self":[{self:["auto",...ue(),"baseline"]}],"place-content":[{"place-content":te()}],"place-items":[{"place-items":[...ue(),"baseline"]}],"place-self":[{"place-self":["auto",...ue()]}],p:[{p:I()}],px:[{px:I()}],py:[{py:I()}],ps:[{ps:I()}],pe:[{pe:I()}],pt:[{pt:I()}],pr:[{pr:I()}],pb:[{pb:I()}],pl:[{pl:I()}],m:[{m:N()}],mx:[{mx:N()}],my:[{my:N()}],ms:[{ms:N()}],me:[{me:N()}],mt:[{mt:N()}],mr:[{mr:N()}],mb:[{mb:N()}],ml:[{ml:N()}],"space-x":[{"space-x":X()}],"space-x-reverse":["space-x-reverse"],"space-y":[{"space-y":X()}],"space-y-reverse":["space-y-reverse"],size:[{size:q()}],w:[{w:[h,"screen",...q()]}],"min-w":[{"min-w":[h,"screen","none",...q()]}],"max-w":[{"max-w":[h,"screen","none","prose",{screen:[d]},...q()]}],h:[{h:["screen",...q()]}],"min-h":[{"min-h":["screen","none",...q()]}],"max-h":[{"max-h":["screen",...q()]}],"font-size":[{text:["base",i,Ei,Nr]}],"font-smoothing":["antialiased","subpixel-antialiased"],"font-style":["italic","not-italic"],"font-weight":[{font:[o,le,zc]}],"font-stretch":[{"font-stretch":["ultra-condensed","extra-condensed","condensed","semi-condensed","normal","semi-expanded","expanded","extra-expanded","ultra-expanded",ag,re]}],"font-family":[{font:[RS,re,a]}],"fvn-normal":["normal-nums"],"fvn-ordinal":["ordinal"],"fvn-slashed-zero":["slashed-zero"],"fvn-figure":["lining-nums","oldstyle-nums"],"fvn-spacing":["proportional-nums","tabular-nums"],"fvn-fraction":["diagonal-fractions","stacked-fractions"],tracking:[{tracking:[s,le,re]}],"line-clamp":[{"line-clamp":[Ae,"none",le,zc]}],leading:[{leading:[le,re,f,g]}],"list-image":[{"list-image":["none",le,re]}],"list-style-position":[{list:["inside","outside"]}],"list-style-type":[{list:["disc","decimal","none",le,re]}],"text-alignment":[{text:["left","center","right","justify","start","end"]}],"placeholder-color":[{placeholder:L()}],"text-color":[{text:L()}],"text-decoration":["underline","overline","line-through","no-underline"],"text-decoration-style":[{decoration:[...P(),"wavy"]}],"text-decoration-thickness":[{decoration:[Ae,"from-font","auto",le,Nr]}],"text-decoration-color":[{decoration:L()}],"underline-offset":[{"underline-offset":[Ae,"auto",le,re]}],"text-transform":["uppercase","lowercase","capitalize","normal-case"],"text-overflow":["truncate","text-ellipsis","text-clip"],"text-wrap":[{text:["wrap","nowrap","balance","pretty"]}],indent:[{indent:["px",...X()]}],"vertical-align":[{align:["baseline","top","middle","bottom","text-top","text-bottom","sub","super",le,re]}],whitespace:[{whitespace:["normal","nowrap","pre","pre-line","pre-wrap","break-spaces"]}],break:[{break:["normal","words","all","keep"]}],hyphens:[{hyphens:["none","manual","auto"]}],content:[{content:["none",le,re]}],"bg-attachment":[{bg:["fixed","local","scroll"]}],"bg-clip":[{"bg-clip":["border","padding","content","text"]}],"bg-origin":[{"bg-origin":["border","padding","content"]}],"bg-position":[{bg:[...H(),TS,wS]}],"bg-repeat":[{bg:["no-repeat",{repeat:["","x","y","space","round"]}]}],"bg-size":[{bg:["auto","cover","contain",AS,xS]}],"bg-image":[{bg:["none",{linear:[{to:["t","tr","r","br","b","bl","l","tl"]},Mr,le,re],radial:["",le,re],conic:[Mr,le,re]},kS,SS]}],"bg-color":[{bg:L()}],"gradient-from-pos":[{from:K()}],"gradient-via-pos":[{via:K()}],"gradient-to-pos":[{to:K()}],"gradient-from":[{from:L()}],"gradient-via":[{via:L()}],"gradient-to":[{to:L()}],rounded:[{rounded:E()}],"rounded-s":[{"rounded-s":E()}],"rounded-e":[{"rounded-e":E()}],"rounded-t":[{"rounded-t":E()}],"rounded-r":[{"rounded-r":E()}],"rounded-b":[{"rounded-b":E()}],"rounded-l":[{"rounded-l":E()}],"rounded-ss":[{"rounded-ss":E()}],"rounded-se":[{"rounded-se":E()}],"rounded-ee":[{"rounded-ee":E()}],"rounded-es":[{"rounded-es":E()}],"rounded-tl":[{"rounded-tl":E()}],"rounded-tr":[{"rounded-tr":E()}],"rounded-br":[{"rounded-br":E()}],"rounded-bl":[{"rounded-bl":E()}],"border-w":[{border:_()}],"border-w-x":[{"border-x":_()}],"border-w-y":[{"border-y":_()}],"border-w-s":[{"border-s":_()}],"border-w-e":[{"border-e":_()}],"border-w-t":[{"border-t":_()}],"border-w-r":[{"border-r":_()}],"border-w-b":[{"border-b":_()}],"border-w-l":[{"border-l":_()}],"divide-x":[{"divide-x":_()}],"divide-x-reverse":["divide-x-reverse"],"divide-y":[{"divide-y":_()}],"divide-y-reverse":["divide-y-reverse"],"border-style":[{border:[...P(),"hidden","none"]}],"divide-style":[{divide:[...P(),"hidden","none"]}],"border-color":[{border:L()}],"border-color-x":[{"border-x":L()}],"border-color-y":[{"border-y":L()}],"border-color-s":[{"border-s":L()}],"border-color-e":[{"border-e":L()}],"border-color-t":[{"border-t":L()}],"border-color-r":[{"border-r":L()}],"border-color-b":[{"border-b":L()}],"border-color-l":[{"border-l":L()}],"divide-color":[{divide:L()}],"outline-style":[{outline:[...P(),"none","hidden"]}],"outline-offset":[{"outline-offset":[Ae,le,re]}],"outline-w":[{outline:["",Ae,Ei,Nr]}],"outline-color":[{outline:[r]}],shadow:[{shadow:["","none",y,OS,ES]}],"shadow-color":[{shadow:L()}],"inset-shadow":[{"inset-shadow":["none",le,re,v]}],"inset-shadow-color":[{"inset-shadow":L()}],"ring-w":[{ring:_()}],"ring-w-inset":["ring-inset"],"ring-color":[{ring:L()}],"ring-offset-w":[{"ring-offset":[Ae,Nr]}],"ring-offset-color":[{"ring-offset":L()}],"inset-ring-w":[{"inset-ring":_()}],"inset-ring-color":[{"inset-ring":L()}],opacity:[{opacity:[Ae,le,re]}],"mix-blend":[{"mix-blend":[...F(),"plus-darker","plus-lighter"]}],"bg-blend":[{"bg-blend":F()}],filter:[{filter:["","none",le,re]}],blur:[{blur:oe()}],brightness:[{brightness:[Ae,le,re]}],contrast:[{contrast:[Ae,le,re]}],"drop-shadow":[{"drop-shadow":["","none",w,le,re]}],grayscale:[{grayscale:["",Ae,le,re]}],"hue-rotate":[{"hue-rotate":[Ae,le,re]}],invert:[{invert:["",Ae,le,re]}],saturate:[{saturate:[Ae,le,re]}],sepia:[{sepia:["",Ae,le,re]}],"backdrop-filter":[{"backdrop-filter":["","none",le,re]}],"backdrop-blur":[{"backdrop-blur":oe()}],"backdrop-brightness":[{"backdrop-brightness":[Ae,le,re]}],"backdrop-contrast":[{"backdrop-contrast":[Ae,le,re]}],"backdrop-grayscale":[{"backdrop-grayscale":["",Ae,le,re]}],"backdrop-hue-rotate":[{"backdrop-hue-rotate":[Ae,le,re]}],"backdrop-invert":[{"backdrop-invert":["",Ae,le,re]}],"backdrop-opacity":[{"backdrop-opacity":[Ae,le,re]}],"backdrop-saturate":[{"backdrop-saturate":[Ae,le,re]}],"backdrop-sepia":[{"backdrop-sepia":["",Ae,le,re]}],"border-collapse":[{border:["collapse","separate"]}],"border-spacing":[{"border-spacing":X()}],"border-spacing-x":[{"border-spacing-x":X()}],"border-spacing-y":[{"border-spacing-y":X()}],"table-layout":[{table:["auto","fixed"]}],caption:[{caption:["top","bottom"]}],transition:[{transition:["","all","colors","opacity","shadow","transform","none",le,re]}],"transition-behavior":[{transition:["normal","discrete"]}],duration:[{duration:[Ae,"initial",le,re]}],ease:[{ease:["linear","initial",k,le,re]}],delay:[{delay:[Ae,le,re]}],animate:[{animate:["none",O,le,re]}],backface:[{backface:["hidden","visible"]}],perspective:[{perspective:[x,le,re]}],"perspective-origin":[{"perspective-origin":be()}],rotate:[{rotate:ae()}],"rotate-x":[{"rotate-x":ae()}],"rotate-y":[{"rotate-y":ae()}],"rotate-z":[{"rotate-z":ae()}],scale:[{scale:ke()}],"scale-x":[{"scale-x":ke()}],"scale-y":[{"scale-y":ke()}],"scale-z":[{"scale-z":ke()}],"scale-3d":["scale-3d"],skew:[{skew:de()}],"skew-x":[{"skew-x":de()}],"skew-y":[{"skew-y":de()}],transform:[{transform:[le,re,"","none","gpu","cpu"]}],"transform-origin":[{origin:be()}],"transform-style":[{transform:["3d","flat"]}],translate:[{translate:Ce()}],"translate-x":[{"translate-x":Ce()}],"translate-y":[{"translate-y":Ce()}],"translate-z":[{"translate-z":Ce()}],"translate-none":["translate-none"],accent:[{accent:L()}],appearance:[{appearance:["none","auto"]}],"caret-color":[{caret:L()}],"color-scheme":[{scheme:["normal","dark","light","light-dark","only-dark","only-light"]}],cursor:[{cursor:["auto","default","pointer","wait","text","move","help","not-allowed","none","context-menu","progress","cell","crosshair","vertical-text","alias","copy","no-drop","grab","grabbing","all-scroll","col-resize","row-resize","n-resize","e-resize","s-resize","w-resize","ne-resize","nw-resize","se-resize","sw-resize","ew-resize","ns-resize","nesw-resize","nwse-resize","zoom-in","zoom-out",le,re]}],"field-sizing":[{"field-sizing":["fixed","content"]}],"pointer-events":[{"pointer-events":["auto","none"]}],resize:[{resize:["none","","y","x"]}],"scroll-behavior":[{scroll:["auto","smooth"]}],"scroll-m":[{"scroll-m":X()}],"scroll-mx":[{"scroll-mx":X()}],"scroll-my":[{"scroll-my":X()}],"scroll-ms":[{"scroll-ms":X()}],"scroll-me":[{"scroll-me":X()}],"scroll-mt":[{"scroll-mt":X()}],"scroll-mr":[{"scroll-mr":X()}],"scroll-mb":[{"scroll-mb":X()}],"scroll-ml":[{"scroll-ml":X()}],"scroll-p":[{"scroll-p":X()}],"scroll-px":[{"scroll-px":X()}],"scroll-py":[{"scroll-py":X()}],"scroll-ps":[{"scroll-ps":X()}],"scroll-pe":[{"scroll-pe":X()}],"scroll-pt":[{"scroll-pt":X()}],"scroll-pr":[{"scroll-pr":X()}],"scroll-pb":[{"scroll-pb":X()}],"scroll-pl":[{"scroll-pl":X()}],"snap-align":[{snap:["start","end","center","align-none"]}],"snap-stop":[{snap:["normal","always"]}],"snap-type":[{snap:["none","x","y","both"]}],"snap-strictness":[{snap:["mandatory","proximity"]}],touch:[{touch:["auto","none","manipulation"]}],"touch-x":[{"touch-pan":["x","left","right"]}],"touch-y":[{"touch-pan":["y","up","down"]}],"touch-pz":["touch-pinch-zoom"],select:[{select:["none","text","all","auto"]}],"will-change":[{"will-change":["auto","scroll","contents","transform",le,re]}],fill:[{fill:["none",...L()]}],"stroke-w":[{stroke:[Ae,Ei,Nr,zc]}],stroke:[{stroke:["none",...L()]}],"forced-color-adjust":[{"forced-color-adjust":["auto","none"]}]},conflictingClassGroups:{overflow:["overflow-x","overflow-y"],overscroll:["overscroll-x","overscroll-y"],inset:["inset-x","inset-y","start","end","top","right","bottom","left"],"inset-x":["right","left"],"inset-y":["top","bottom"],flex:["basis","grow","shrink"],gap:["gap-x","gap-y"],p:["px","py","ps","pe","pt","pr","pb","pl"],px:["pr","pl"],py:["pt","pb"],m:["mx","my","ms","me","mt","mr","mb","ml"],mx:["mr","ml"],my:["mt","mb"],size:["w","h"],"font-size":["leading"],"fvn-normal":["fvn-ordinal","fvn-slashed-zero","fvn-figure","fvn-spacing","fvn-fraction"],"fvn-ordinal":["fvn-normal"],"fvn-slashed-zero":["fvn-normal"],"fvn-figure":["fvn-normal"],"fvn-spacing":["fvn-normal"],"fvn-fraction":["fvn-normal"],"line-clamp":["display","overflow"],rounded:["rounded-s","rounded-e","rounded-t","rounded-r","rounded-b","rounded-l","rounded-ss","rounded-se","rounded-ee","rounded-es","rounded-tl","rounded-tr","rounded-br","rounded-bl"],"rounded-s":["rounded-ss","rounded-es"],"rounded-e":["rounded-se","rounded-ee"],"rounded-t":["rounded-tl","rounded-tr"],"rounded-r":["rounded-tr","rounded-br"],"rounded-b":["rounded-br","rounded-bl"],"rounded-l":["rounded-tl","rounded-bl"],"border-spacing":["border-spacing-x","border-spacing-y"],"border-w":["border-w-s","border-w-e","border-w-t","border-w-r","border-w-b","border-w-l"],"border-w-x":["border-w-r","border-w-l"],"border-w-y":["border-w-t","border-w-b"],"border-color":["border-color-s","border-color-e","border-color-t","border-color-r","border-color-b","border-color-l"],"border-color-x":["border-color-r","border-color-l"],"border-color-y":["border-color-t","border-color-b"],translate:["translate-x","translate-y","translate-none"],"translate-none":["translate","translate-x","translate-y","translate-z"],"scroll-m":["scroll-mx","scroll-my","scroll-ms","scroll-me","scroll-mt","scroll-mr","scroll-mb","scroll-ml"],"scroll-mx":["scroll-mr","scroll-ml"],"scroll-my":["scroll-mt","scroll-mb"],"scroll-p":["scroll-px","scroll-py","scroll-ps","scroll-pe","scroll-pt","scroll-pr","scroll-pb","scroll-pl"],"scroll-px":["scroll-pr","scroll-pl"],"scroll-py":["scroll-pt","scroll-pb"],touch:["touch-x","touch-y","touch-pz"],"touch-x":["touch"],"touch-y":["touch"],"touch-pz":["touch"]},conflictingClassGroupModifiers:{"font-size":["leading"]},orderSensitiveModifiers:["before","after","placeholder","file","marker","selection","first-line","first-letter","backdrop","*","**"]}},DS=(r,{cacheSize:a,prefix:i,experimentalParseClassName:o,extend:s={},override:f={}})=>(Pa(r,"cacheSize",a),Pa(r,"prefix",i),Pa(r,"experimentalParseClassName",o),Ri(r.theme,f.theme),Ri(r.classGroups,f.classGroups),Ri(r.conflictingClassGroups,f.conflictingClassGroups),Ri(r.conflictingClassGroupModifiers,f.conflictingClassGroupModifiers),Pa(r,"orderSensitiveModifiers",f.orderSensitiveModifiers),Ti(r.theme,s.theme),Ti(r.classGroups,s.classGroups),Ti(r.conflictingClassGroups,s.conflictingClassGroups),Ti(r.conflictingClassGroupModifiers,s.conflictingClassGroupModifiers),Tp(r,s,"orderSensitiveModifiers"),r),Pa=(r,a,i)=>{i!==void 0&&(r[a]=i)},Ri=(r,a)=>{if(a)for(const i in a)Pa(r,i,a[i])},Ti=(r,a)=>{if(a)for(const i in a)Tp(r,a,i)},Tp=(r,a,i)=>{const o=a[i];o!==void 0&&(r[i]=r[i]?r[i].concat(o):o)},US=(r,...a)=>typeof r=="function"?lg(og,r,...a):lg(()=>DS(og(),r),...a),ig=new Map;function Be(...r){const a=cp(),i=fp(),o=`${a}.${i}`,s=ig.get(o);if(s)return s(...r);const f=(i===3?Jw:US)({extend:{classGroups:{"bg-image":["bg-arrow-down-icon","bg-check-icon","bg-dash-icon","bg-dot-icon"],shadow:["shadow-sm-light"]}},prefix:a});return ig.set(o,f),f(...r)}function ot(...r){return jS(()=>LS(...r),r)}function jS(r,a){const i=T.useRef(),o=T.useRef();return(!i.current||!Qc(i.current,a))&&(i.current=a,o.current=r()),o.current}function LS([r,...a],i,o){const s=fw(),f=cp(),d=fp(),h=a!=null&&a.length?a==null?void 0:a.filter(w=>w!==void 0):void 0,g=i!=null&&i.length?i==null?void 0:i.filter(w=>w!==void 0):void 0,m=o!=null&&o.length?o==null?void 0:o.filter(w=>w!==void 0):void 0,y=g!=null&&g.length||s===!1||d===4||f?Ci(r):r;if(g!=null&&g.length){const w=Pc(y,!1);let S=!1;for(const x of g)x&&(S=!0),BS(w,x);S&&qS(y,w)}(d===4||f)&&YS(y,w=>(d===4&&(w=hw(w)),w));let v=y;if(h!=null&&h.length&&(v=gw(Be)(y,...h)),m!=null&&m.length&&(h!=null&&h.length)){const w=Pc(y,"merge");let S=!1;for(const x of m)x!=="merge"&&(S=!0),HS(w,x);S&&GS(v,aw(y,...a),w)}return v}function BS(r,a){function i(o,s){if(typeof s=="boolean")if(typeof o=="object"&&o!==null)for(const f in o)o[f]=i(o[f],s);else return s;if(typeof s=="object"&&s!==null)for(const f in s)o[f]=i(o[f],s[f]);return o}i(r,a)}function HS(r,a){function i(o,s){if(typeof s=="string")if(typeof o=="object"&&o!==null)for(const f in o)o[f]=i(o[f],s);else return s;if(typeof s=="object"&&s!==null)for(const f in s)o[f]=i(o[f],s[f]);return o}i(r,a)}function qS(r,a){function i(o,s){if(s===!0)if(typeof o=="object"&&o!==null)for(const f in o)o[f]=i(o[f],s);else return"";if(typeof s=="object"&&s!==null)for(const f in s)o[f]=i(o[f],s[f]);return o}i(r,a)}function GS(r,a,i){function o(s,f,d){if(d==="replace")if(typeof s=="object"&&s!==null)for(const h in s)s[h]=o(s[h],f[h],d);else return f;if(typeof d=="object"&&d!==null)for(const h in d)s[h]=o(s[h],f[h],d[h]);return s}o(r,a,i)}function YS(r,a){function i(o){if(typeof o=="string")return a(o);if(Array.isArray(o))for(let s=0;s<o.length;s++)o[s]=i(o[s]);else if(typeof o=="object"&&o!==null)for(const s in o)o[s]=i(o[s]);return o}i(r)}function Pc(r,a){if(r===null||typeof r!="object")return a;const i={};for(const o in r)i[o]=Pc(r[o],a);return i}const VS=T.createContext(void 0);function it(){return T.useContext(VS)??{}}const Ap=T.forwardRef((r,a)=>G.jsx("svg",{xmlns:"http://www.w3.org/2000/svg",width:"1em",height:"1em",fill:"currentColor",stroke:"currentColor",strokeWidth:0,viewBox:"0 0 20 20",ref:a,...r,children:G.jsx("path",{fillRule:"evenodd",stroke:"none",d:"M4.293 4.293a1 1 0 0 1 1.414 0L10 8.586l4.293-4.293a1 1 0 1 1 1.414 1.414L11.414 10l4.293 4.293a1 1 0 0 1-1.414 1.414L10 11.414l-4.293 4.293a1 1 0 0 1-1.414-1.414L8.586 10 4.293 5.707a1 1 0 0 1 0-1.414z",clipRule:"evenodd"})}));Ap.displayName="XIcon";const XS={base:"flex flex-col gap-2 p-4 text-sm",borderAccent:"border-t-4",closeButton:{base:"-m-1.5 ml-auto inline-flex h-8 w-8 rounded-lg p-1.5 focus:ring-2",icon:"h-5 w-5",color:{info:"bg-cyan-100 text-cyan-500 hover:bg-cyan-200 focus:ring-cyan-400 dark:bg-cyan-200 dark:text-cyan-600 dark:hover:bg-cyan-300",gray:"bg-gray-100 text-gray-500 hover:bg-gray-200 focus:ring-gray-400 dark:bg-gray-700 dark:text-gray-300 dark:hover:bg-gray-800 dark:hover:text-white",failure:"bg-red-100 text-red-500 hover:bg-red-200 focus:ring-red-400 dark:bg-red-200 dark:text-red-600 dark:hover:bg-red-300",success:"bg-green-100 text-green-500 hover:bg-green-200 focus:ring-green-400 dark:bg-green-200 dark:text-green-600 dark:hover:bg-green-300",warning:"bg-yellow-100 text-yellow-500 hover:bg-yellow-200 focus:ring-yellow-400 dark:bg-yellow-200 dark:text-yellow-600 dark:hover:bg-yellow-300",red:"bg-red-100 text-red-500 hover:bg-red-200 focus:ring-red-400 dark:bg-red-200 dark:text-red-600 dark:hover:bg-red-300",green:"bg-green-100 text-green-500 hover:bg-green-200 focus:ring-green-400 dark:bg-green-200 dark:text-green-600 dark:hover:bg-green-300",yellow:"bg-yellow-100 text-yellow-500 hover:bg-yellow-200 focus:ring-yellow-400 dark:bg-yellow-200 dark:text-yellow-600 dark:hover:bg-yellow-300",blue:"bg-blue-100 text-blue-500 hover:bg-blue-200 focus:ring-blue-400 dark:bg-blue-200 dark:text-blue-600 dark:hover:bg-blue-300",cyan:"bg-cyan-100 text-cyan-500 hover:bg-cyan-200 focus:ring-cyan-400 dark:bg-cyan-200 dark:text-cyan-600 dark:hover:bg-cyan-300",pink:"bg-pink-100 text-pink-500 hover:bg-pink-200 focus:ring-pink-400 dark:bg-pink-200 dark:text-pink-600 dark:hover:bg-pink-300",lime:"bg-lime-100 text-lime-500 hover:bg-lime-200 focus:ring-lime-400 dark:bg-lime-200 dark:text-lime-600 dark:hover:bg-lime-300",dark:"bg-gray-100 text-gray-500 hover:bg-gray-200 focus:ring-gray-400 dark:bg-gray-200 dark:text-gray-600 dark:hover:bg-gray-300",indigo:"bg-indigo-100 text-indigo-500 hover:bg-indigo-200 focus:ring-indigo-400 dark:bg-indigo-200 dark:text-indigo-600 dark:hover:bg-indigo-300",purple:"bg-purple-100 text-purple-500 hover:bg-purple-200 focus:ring-purple-400 dark:bg-purple-200 dark:text-purple-600 dark:hover:bg-purple-300",teal:"bg-teal-100 text-teal-500 hover:bg-teal-200 focus:ring-teal-400 dark:bg-teal-200 dark:text-teal-600 dark:hover:bg-teal-300",light:"bg-gray-50 text-gray-500 hover:bg-gray-100 focus:ring-gray-200 dark:bg-gray-600 dark:text-gray-200 dark:hover:bg-gray-700 dark:hover:text-white"}},color:{info:"border-cyan-500 bg-cyan-100 text-cyan-700 dark:bg-cyan-200 dark:text-cyan-800",gray:"border-gray-500 bg-gray-100 text-gray-700 dark:bg-gray-700 dark:text-gray-300",failure:"border-red-500 bg-red-100 text-red-700 dark:bg-red-200 dark:text-red-800",success:"border-green-500 bg-green-100 text-green-700 dark:bg-green-200 dark:text-green-800",warning:"border-yellow-500 bg-yellow-100 text-yellow-700 dark:bg-yellow-200 dark:text-yellow-800",red:"border-red-500 bg-red-100 text-red-700 dark:bg-red-200 dark:text-red-800",green:"border-green-500 bg-green-100 text-green-700 dark:bg-green-200 dark:text-green-800",yellow:"border-yellow-500 bg-yellow-100 text-yellow-700 dark:bg-yellow-200 dark:text-yellow-800",blue:"border-blue-500 bg-blue-100 text-blue-700 dark:bg-blue-200 dark:text-blue-800",cyan:"border-cyan-500 bg-cyan-100 text-cyan-700 dark:bg-cyan-200 dark:text-cyan-800",pink:"border-pink-500 bg-pink-100 text-pink-700 dark:bg-pink-200 dark:text-pink-800",lime:"border-lime-500 bg-lime-100 text-lime-700 dark:bg-lime-200 dark:text-lime-800",dark:"border-gray-600 bg-gray-800 text-gray-200 dark:bg-gray-900 dark:text-gray-300",indigo:"border-indigo-500 bg-indigo-100 text-indigo-700 dark:bg-indigo-200 dark:text-indigo-800",purple:"border-purple-500 bg-purple-100 text-purple-700 dark:bg-purple-200 dark:text-purple-800",teal:"border-teal-500 bg-teal-100 text-teal-700 dark:bg-teal-200 dark:text-teal-800",light:"border-gray-400 bg-gray-50 text-gray-600 dark:bg-gray-500 dark:text-gray-200"},icon:"mr-3 inline h-5 w-5 shrink-0",rounded:"rounded-lg",wrapper:"flex items-center"},Rf=T.forwardRef((r,a)=>{var S,x;const i=it(),o=ot([XS,(S=i.theme)==null?void 0:S.alert,r.theme],[pe(i.clearTheme,"alert"),r.clearTheme],[pe(i.applyTheme,"alert"),r.applyTheme]),{additionalContent:s,children:f,className:d,color:h="info",icon:g,onDismiss:m,rounded:y=!0,withBorderAccent:v,...w}=at(r,(x=i.props)==null?void 0:x.alert);return G.jsxs("div",{ref:a,className:Be(o.base,o.color[h],y&&o.rounded,v&&o.borderAccent,d),role:"alert",...w,children:[G.jsxs("div",{className:o.wrapper,"data-testid":"flowbite-alert-wrapper",children:[g&&G.jsx(g,{className:o.icon,"data-testid":"flowbite-alert-icon"}),G.jsx("div",{children:f}),typeof m=="function"&&G.jsx("button",{"aria-label":"Dismiss",className:Be(o.closeButton.base,o.closeButton.color[h]),onClick:m,type:"button",children:G.jsx(Ap,{"aria-hidden":!0,className:o.closeButton.icon})})]}),s&&G.jsx("div",{children:s})]})});Rf.displayName="Alert";const QS={root:{base:"flex h-fit items-center gap-1 font-semibold",color:{info:"bg-cyan-100 text-cyan-800 hover:bg-cyan-200 dark:bg-cyan-200 dark:text-cyan-800 dark:hover:bg-cyan-300",gray:"bg-gray-100 text-gray-800 hover:bg-gray-200 dark:bg-gray-700 dark:text-gray-300 dark:hover:bg-gray-600",failure:"bg-red-100 text-red-800 hover:bg-red-200 dark:bg-red-200 dark:text-red-900 dark:hover:bg-red-300",success:"bg-green-100 text-green-800 hover:bg-green-200 dark:bg-green-200 dark:text-green-900 dark:hover:bg-green-300",warning:"bg-yellow-100 text-yellow-800 hover:bg-yellow-200 dark:bg-yellow-200 dark:text-yellow-900 dark:hover:bg-yellow-300",indigo:"bg-indigo-100 text-indigo-800 hover:bg-indigo-200 dark:bg-indigo-200 dark:text-indigo-900 dark:hover:bg-indigo-300",purple:"bg-purple-100 text-purple-800 hover:bg-purple-200 dark:bg-purple-200 dark:text-purple-900 dark:hover:bg-purple-300",pink:"bg-pink-100 text-pink-800 hover:bg-pink-200 dark:bg-pink-200 dark:text-pink-900 dark:hover:bg-pink-300",blue:"bg-blue-100 text-blue-800 hover:bg-blue-200 dark:bg-blue-200 dark:text-blue-900 dark:hover:bg-blue-300",cyan:"bg-cyan-100 text-cyan-800 hover:bg-cyan-200 dark:bg-cyan-200 dark:text-cyan-900 dark:hover:bg-cyan-300",dark:"bg-gray-600 text-gray-100 hover:bg-gray-500 dark:bg-gray-900 dark:text-gray-200 dark:hover:bg-gray-700",light:"bg-gray-200 text-gray-800 hover:bg-gray-300 dark:bg-gray-400 dark:text-gray-900 dark:hover:bg-gray-500",green:"bg-green-100 text-green-800 hover:bg-green-200 dark:bg-green-200 dark:text-green-900 dark:hover:bg-green-300",lime:"bg-lime-100 text-lime-800 hover:bg-lime-200 dark:bg-lime-200 dark:text-lime-900 dark:hover:bg-lime-300",red:"bg-red-100 text-red-800 hover:bg-red-200 dark:bg-red-200 dark:text-red-900 dark:hover:bg-red-300",teal:"bg-teal-100 text-teal-800 hover:bg-teal-200 dark:bg-teal-200 dark:text-teal-900 dark:hover:bg-teal-300",yellow:"bg-yellow-100 text-yellow-800 hover:bg-yellow-200 dark:bg-yellow-200 dark:text-yellow-900 dark:hover:bg-yellow-300"},size:{xs:"p-1 text-xs",sm:"p-1.5 text-sm"}},icon:{off:"rounded px-2 py-0.5",on:"rounded-full p-1.5",size:{xs:"h-3 w-3",sm:"h-3.5 w-3.5"}}},kp=T.forwardRef((r,a)=>{var y,v;const i=it(),o=ot([QS,(y=i.theme)==null?void 0:y.badge,r.theme],[pe(i.clearTheme,"badge"),r.clearTheme],[pe(i.applyTheme,"badge"),r.applyTheme]),{children:s,color:f="info",icon:d,size:h="xs",className:g,...m}=at(r,(v=i.props)==null?void 0:v.badge);return G.jsxs("span",{ref:a,className:Be(o.root.base,o.root.color[f],o.root.size[h],o.icon[d?"on":"off"],g),"data-testid":"flowbite-badge",...m,children:[d&&G.jsx(d,{"aria-hidden":!0,className:o.icon.size[h],"data-testid":"flowbite-badge-icon"}),s&&G.jsx("span",{children:s})]})});kp.displayName="Badge";const Op=T.forwardRef(({children:r,as:a,href:i,type:o="button",...s},f)=>{const d=a||(i?"a":"button");return T.createElement(d,{ref:f,href:i,type:o,...s},r)});Op.displayName="ButtonBase";const ZS=T.createContext(void 0);function KS(){return T.useContext(ZS)}const $S={base:"relative flex items-center justify-center rounded-lg text-center font-medium focus:outline-none focus:ring-4",disabled:"pointer-events-none opacity-50",fullSized:"w-full",grouped:"rounded-none border-l-0 first:rounded-s-lg first:border-l last:rounded-e-lg focus:ring-2",pill:"rounded-full",size:{xs:"h-8 px-3 text-xs",sm:"h-9 px-3 text-sm",md:"h-10 px-5 text-sm",lg:"h-12 px-5 text-base",xl:"h-[52px] px-6 text-base"},color:{default:"bg-primary-700 text-white hover:bg-primary-800 focus:ring-primary-300 dark:bg-primary-600 dark:hover:bg-primary-700 dark:focus:ring-primary-800",alternative:"border border-gray-200 bg-white text-gray-900 hover:bg-gray-100 hover:text-primary-700 focus:ring-gray-100 dark:border-gray-600 dark:bg-gray-800 dark:text-gray-400 dark:hover:bg-gray-700 dark:hover:text-white dark:focus:ring-gray-700",blue:"bg-blue-700 text-white hover:bg-blue-800 focus:ring-blue-300 dark:bg-blue-600 dark:hover:bg-blue-700 dark:focus:ring-blue-800",cyan:"bg-cyan-700 text-white hover:bg-cyan-800 focus:ring-cyan-300 dark:bg-cyan-600 dark:hover:bg-cyan-700 dark:focus:ring-cyan-800",dark:"bg-gray-800 text-white hover:bg-gray-900 focus:ring-gray-300 dark:bg-gray-800 dark:hover:bg-gray-700 dark:focus:ring-gray-700",gray:"bg-gray-700 text-white hover:bg-gray-800 focus:ring-gray-300 dark:bg-gray-600 dark:hover:bg-gray-700 dark:focus:ring-gray-800",green:"bg-green-700 text-white hover:bg-green-800 focus:ring-green-300 dark:bg-green-600 dark:hover:bg-green-700 dark:focus:ring-green-800",indigo:"bg-indigo-700 text-white hover:bg-indigo-800 focus:ring-indigo-300 dark:bg-indigo-600 dark:hover:bg-indigo-700 dark:focus:ring-indigo-800",light:"border border-gray-300 bg-white text-gray-900 hover:bg-gray-100 focus:ring-gray-100 dark:border-gray-600 dark:bg-gray-800 dark:text-white dark:hover:border-gray-600 dark:hover:bg-gray-700 dark:focus:ring-gray-700",lime:"bg-lime-700 text-white hover:bg-lime-800 focus:ring-lime-300 dark:bg-lime-600 dark:hover:bg-lime-700 dark:focus:ring-lime-800",pink:"bg-pink-700 text-white hover:bg-pink-800 focus:ring-pink-300 dark:bg-pink-600 dark:hover:bg-pink-700 dark:focus:ring-pink-800",purple:"bg-purple-700 text-white hover:bg-purple-800 focus:ring-purple-300 dark:bg-purple-600 dark:hover:bg-purple-700 dark:focus:ring-purple-800",red:"bg-red-700 text-white hover:bg-red-800 focus:ring-red-300 dark:bg-red-600 dark:hover:bg-red-700 dark:focus:ring-red-800",teal:"bg-teal-700 text-white hover:bg-teal-800 focus:ring-teal-300 dark:bg-teal-600 dark:hover:bg-teal-700 dark:focus:ring-teal-800",yellow:"bg-yellow-400 text-white hover:bg-yellow-500 focus:ring-yellow-300 dark:bg-yellow-600 dark:hover:bg-yellow-400 dark:focus:ring-yellow-900"},outlineColor:{default:"border border-primary-700 text-primary-700 hover:border-primary-800 hover:bg-primary-800 hover:text-white focus:ring-primary-300 dark:border-primary-600 dark:text-primary-500 dark:hover:border-primary-700 dark:hover:bg-primary-700 dark:hover:text-white dark:focus:ring-primary-800",blue:"border border-blue-700 text-blue-700 hover:border-blue-800 hover:bg-blue-800 hover:text-white focus:ring-blue-300 dark:border-blue-500 dark:text-blue-500 dark:hover:border-blue-700 dark:hover:bg-blue-700 dark:hover:text-white dark:focus:ring-blue-800",cyan:"border border-cyan-700 text-cyan-700 hover:border-cyan-800 hover:bg-cyan-800 hover:text-white focus:ring-cyan-300 dark:border-cyan-500 dark:text-cyan-500 dark:hover:border-cyan-700 dark:hover:bg-cyan-700 dark:hover:text-white dark:focus:ring-cyan-800",dark:"border border-gray-800 text-gray-800 hover:border-gray-900 hover:bg-gray-900 hover:text-white focus:ring-gray-300 dark:border-gray-600 dark:text-gray-400 dark:hover:bg-gray-600 dark:hover:text-white dark:focus:ring-gray-800",gray:"border border-gray-700 text-gray-700 hover:border-gray-800 hover:bg-gray-800 hover:text-white focus:ring-gray-300 dark:border-gray-600 dark:text-gray-400 dark:hover:border-gray-700 dark:hover:bg-gray-700 dark:hover:text-white dark:focus:ring-gray-800",green:"border border-green-700 text-green-700 hover:border-green-800 hover:bg-green-800 hover:text-white focus:ring-green-300 dark:border-green-600 dark:text-green-500 dark:hover:border-green-700 dark:hover:bg-green-700 dark:hover:text-white dark:focus:ring-green-800",indigo:"border border-indigo-700 text-indigo-700 hover:border-indigo-800 hover:bg-indigo-800 hover:text-white focus:ring-indigo-300 dark:border-indigo-600 dark:text-indigo-400 dark:hover:border-indigo-700 dark:hover:bg-indigo-700 dark:hover:text-white dark:focus:ring-indigo-800",lime:"border border-lime-700 text-lime-700 hover:border-lime-800 hover:bg-lime-800 hover:text-white focus:ring-lime-300 dark:border-lime-600 dark:text-lime-500 dark:hover:border-lime-700 dark:hover:bg-lime-700 dark:hover:text-white dark:focus:ring-lime-800",pink:"border border-pink-700 text-pink-700 hover:border-pink-800 hover:bg-pink-800 hover:text-white focus:ring-pink-300 dark:border-pink-600 dark:text-pink-500 dark:hover:border-pink-700 dark:hover:bg-pink-700 dark:hover:text-white dark:focus:ring-pink-800",purple:"border border-purple-700 text-purple-700 hover:border-purple-800 hover:bg-purple-800 hover:text-white focus:ring-purple-300 dark:border-purple-600 dark:text-purple-400 dark:hover:border-purple-700 dark:hover:bg-purple-700 dark:hover:text-white dark:focus:ring-purple-800",red:"border border-red-700 text-red-700 hover:border-red-800 hover:bg-red-800 hover:text-white focus:ring-red-300 dark:border-red-600 dark:text-red-500 dark:hover:border-red-700 dark:hover:bg-red-700 dark:hover:text-white dark:focus:ring-red-800",teal:"border border-teal-700 text-teal-700 hover:border-teal-800 hover:bg-teal-800 hover:text-white focus:ring-teal-300 dark:border-teal-600 dark:text-teal-400 dark:hover:border-teal-700 dark:hover:bg-teal-700 dark:hover:text-white dark:focus:ring-teal-800",yellow:"border border-yellow-400 text-yellow-400 hover:border-yellow-500 hover:bg-yellow-500 hover:text-white focus:ring-yellow-300 dark:border-yellow-300 dark:text-yellow-300 dark:hover:border-yellow-400 dark:hover:bg-yellow-400 dark:hover:text-white dark:focus:ring-yellow-900"}},Bi=T.forwardRef((r,a)=>{var k,O;const i=it(),o=ot([$S,(k=i.theme)==null?void 0:k.button,r.theme],[pe(i.clearTheme,"button"),r.clearTheme],[pe(i.applyTheme,"button"),r.applyTheme]),{children:s,className:f,color:d="default",disabled:h,fullSized:g,outline:m,pill:y,size:v="md",...w}=at(r,(O=i.props)==null?void 0:O.button),S=KS(),x=(S==null?void 0:S.outline)??m,R=(S==null?void 0:S.pill)??y;return G.jsx(Op,{ref:a,disabled:h,className:Be(o.base,o.size[v],R&&o.pill,h&&o.disabled,g&&o.fullSized,x?o.outlineColor[d]:o.color[d],S&&o.grouped,f),...w,children:s})});Bi.displayName="Button";const JS={root:{base:"flex rounded-lg border border-gray-200 bg-white shadow-md dark:border-gray-700 dark:bg-gray-800",children:"flex h-full flex-col justify-center gap-4 p-6",horizontal:{off:"flex-col",on:"flex-col md:max-w-xl md:flex-row"},href:"hover:bg-gray-100 dark:hover:bg-gray-700"},img:{base:"",horizontal:{off:"rounded-t-lg",on:"h-96 w-full rounded-t-lg object-cover md:h-auto md:w-48 md:rounded-none md:rounded-l-lg"}}},Cp=T.forwardRef((r,a)=>{var S,x;const i=it(),o=ot([JS,(S=i.theme)==null?void 0:S.card,r.theme],[pe(i.clearTheme,"card"),r.clearTheme],[pe(i.applyTheme,"card"),r.applyTheme]),{children:s,className:f,horizontal:d,href:h,imgAlt:g,imgSrc:m,renderImage:y,...v}=at(r,(x=i.props)==null?void 0:x.card),w=typeof h>"u"?"div":"a";return G.jsxs(w,{ref:a,"data-testid":"flowbite-card",href:h,className:Be(o.root.base,o.root.horizontal[d?"on":"off"],h&&o.root.href,f),...v,children:[(y==null?void 0:y(o,!!d))??(m&&G.jsx("img",{"data-testid":"flowbite-card-image",alt:g??"",src:m,className:Be(o.img.base,o.img.horizontal[r.horizontal?"on":"off"])})),G.jsx("div",{className:o.root.children,children:s})]})});Cp.displayName="Card";function Mp(){return typeof window<"u"}const PS={base:"h-4 w-4 appearance-none rounded border border-gray-300 bg-gray-100 bg-[length:0.55em_0.55em] bg-center bg-no-repeat checked:border-transparent checked:bg-current checked:bg-check-icon focus:outline-none focus:ring-2 focus:ring-offset-2 dark:border-gray-600 dark:bg-gray-700 dark:checked:border-transparent dark:checked:bg-current",color:{default:"text-primary-600 focus:ring-primary-600 dark:ring-offset-gray-800 dark:focus:ring-primary-600",dark:"text-gray-800 focus:ring-gray-800 dark:ring-offset-gray-800 dark:focus:ring-gray-800",failure:"text-red-900 focus:ring-red-900 dark:ring-offset-red-900 dark:focus:ring-red-900",gray:"text-gray-900 focus:ring-gray-900 dark:ring-offset-gray-900 dark:focus:ring-gray-900",info:"text-cyan-800 focus:ring-cyan-800 dark:ring-offset-gray-800 dark:focus:ring-cyan-800",light:"text-gray-900 focus:ring-gray-900 dark:ring-offset-gray-900 dark:focus:ring-gray-900",purple:"text-purple-600 focus:ring-purple-600 dark:ring-offset-purple-600 dark:focus:ring-purple-600",success:"text-green-800 focus:ring-green-800 dark:ring-offset-green-800 dark:focus:ring-green-800",warning:"text-yellow-400 focus:ring-yellow-400 dark:ring-offset-yellow-400 dark:focus:ring-yellow-400",blue:"text-blue-700 focus:ring-blue-600 dark:ring-offset-blue-700 dark:focus:ring-blue-700",cyan:"text-cyan-600 focus:ring-cyan-600 dark:ring-offset-cyan-600 dark:focus:ring-cyan-600",green:"text-green-600 focus:ring-green-600 dark:ring-offset-green-600 dark:focus:ring-green-600",indigo:"text-indigo-700 focus:ring-indigo-700 dark:ring-offset-indigo-700 dark:focus:ring-indigo-700",lime:"text-lime-700 focus:ring-lime-700 dark:ring-offset-lime-700 dark:focus:ring-lime-700",pink:"text-pink-600 focus:ring-pink-600 dark:ring-offset-pink-600 dark:focus:ring-pink-600",red:"text-red-600 focus:ring-red-600 dark:ring-offset-red-600 dark:focus:ring-red-600",teal:"text-teal-600 focus:ring-teal-600 dark:ring-offset-teal-600 dark:focus:ring-teal-600",yellow:"text-yellow-400 focus:ring-yellow-400 dark:ring-offset-yellow-400 dark:focus:ring-yellow-400"},indeterminate:"border-transparent bg-current bg-dash-icon dark:border-transparent dark:bg-current"},Np=T.forwardRef((r,a)=>{var g,m;const i=it(),o=ot([PS,(g=i.theme)==null?void 0:g.checkbox,r.theme],[pe(i.clearTheme,"checkbox"),r.clearTheme],[pe(i.applyTheme,"checkbox"),r.applyTheme]),{className:s,color:f="default",indeterminate:d,...h}=at(r,(m=i.props)==null?void 0:m.checkbox);return G.jsx("input",{ref:a,type:"checkbox",className:Be(o.base,o.color[f],d&&o.indeterminate,s),...h})});Np.displayName="Checkbox";function Pi(){return typeof window<"u"}function Bl(r){return zp(r)?(r.nodeName||"").toLowerCase():"#document"}function St(r){var a;return(r==null||(a=r.ownerDocument)==null?void 0:a.defaultView)||window}function cn(r){var a;return(a=(zp(r)?r.ownerDocument:r.document)||window.document)==null?void 0:a.documentElement}function zp(r){return Pi()?r instanceof Node||r instanceof St(r).Node:!1}function Ge(r){return Pi()?r instanceof Element||r instanceof St(r).Element:!1}function Tt(r){return Pi()?r instanceof HTMLElement||r instanceof St(r).HTMLElement:!1}function Fc(r){return!Pi()||typeof ShadowRoot>"u"?!1:r instanceof ShadowRoot||r instanceof St(r).ShadowRoot}const FS=new Set(["inline","contents"]);function no(r){const{overflow:a,overflowX:i,overflowY:o,display:s}=Zt(r);return/auto|scroll|overlay|hidden|clip/.test(a+o+i)&&!FS.has(s)}const WS=new Set(["table","td","th"]);function IS(r){return WS.has(Bl(r))}const eE=[":popover-open",":modal"];function Fi(r){return eE.some(a=>{try{return r.matches(a)}catch{return!1}})}const tE=["transform","translate","scale","rotate","perspective"],nE=["transform","translate","scale","rotate","perspective","filter"],rE=["paint","layout","strict","content"];function Tf(r){const a=Wi(),i=Ge(r)?Zt(r):r;return tE.some(o=>i[o]?i[o]!=="none":!1)||(i.containerType?i.containerType!=="normal":!1)||!a&&(i.backdropFilter?i.backdropFilter!=="none":!1)||!a&&(i.filter?i.filter!=="none":!1)||nE.some(o=>(i.willChange||"").includes(o))||rE.some(o=>(i.contain||"").includes(o))}function lE(r){let a=Nn(r);for(;Tt(a)&&!Cn(a);){if(Tf(a))return a;if(Fi(a))return null;a=Nn(a)}return null}function Wi(){return typeof CSS>"u"||!CSS.supports?!1:CSS.supports("-webkit-backdrop-filter","none")}const aE=new Set(["html","body","#document"]);function Cn(r){return aE.has(Bl(r))}function Zt(r){return St(r).getComputedStyle(r)}function Ii(r){return Ge(r)?{scrollLeft:r.scrollLeft,scrollTop:r.scrollTop}:{scrollLeft:r.scrollX,scrollTop:r.scrollY}}function Nn(r){if(Bl(r)==="html")return r;const a=r.assignedSlot||r.parentNode||Fc(r)&&r.host||cn(r);return Fc(a)?a.host:a}function _p(r){const a=Nn(r);return Cn(a)?r.ownerDocument?r.ownerDocument.body:r.body:Tt(a)&&no(a)?a:_p(a)}function ur(r,a,i){var o;a===void 0&&(a=[]),i===void 0&&(i=!0);const s=_p(r),f=s===((o=r.ownerDocument)==null?void 0:o.body),d=St(s);if(f){const h=Wc(d);return a.concat(d,d.visualViewport||[],no(s)?s:[],h&&i?ur(h):[])}return a.concat(s,ur(s,[],i))}function Wc(r){return r.parent&&Object.getPrototypeOf(r.parent)?r.frameElement:null}function ug(r){let a=r.activeElement;for(;((i=a)==null||(i=i.shadowRoot)==null?void 0:i.activeElement)!=null;){var i;a=a.shadowRoot.activeElement}return a}function sr(r,a){if(!r||!a)return!1;const i=a.getRootNode==null?void 0:a.getRootNode();if(r.contains(a))return!0;if(i&&Fc(i)){let o=a;for(;o;){if(r===o)return!0;o=o.parentNode||o.host}}return!1}function Dp(){const r=navigator.userAgentData;return r!=null&&r.platform?r.platform:navigator.platform}function Up(){const r=navigator.userAgentData;return r&&Array.isArray(r.brands)?r.brands.map(a=>{let{brand:i,version:o}=a;return i+"/"+o}).join(" "):navigator.userAgent}function oE(r){return sE()?!1:!sg()&&r.width===0&&r.height===0||sg()&&r.width===1&&r.height===1&&r.pressure===0&&r.detail===0&&r.pointerType==="mouse"||r.width<1&&r.height<1&&r.pressure===0&&r.detail===0&&r.pointerType==="touch"}function iE(){return/apple/i.test(navigator.vendor)}function sg(){const r=/android/i;return r.test(Dp())||r.test(Up())}function uE(){return Dp().toLowerCase().startsWith("mac")&&!navigator.maxTouchPoints}function sE(){return Up().includes("jsdom/")}function Wa(r,a){const i=["mouse","pen"];return a||i.push("",void 0),i.includes(r)}function cE(r){return"nativeEvent"in r}function fE(r){return r.matches("html,body")}function _r(r){return(r==null?void 0:r.ownerDocument)||document}function _c(r,a){if(a==null)return!1;if("composedPath"in r)return r.composedPath().includes(a);const i=r;return i.target!=null&&a.contains(i.target)}function zr(r){return"composedPath"in r?r.composedPath()[0]:r.target}const dE="input:not([type='hidden']):not([disabled]),[contenteditable]:not([contenteditable='false']),textarea:not([disabled])";function jp(r){return Tt(r)&&r.matches(dE)}const hE=["top","right","bottom","left"],cg=["start","end"],fg=hE.reduce((r,a)=>r.concat(a,a+"-"+cg[0],a+"-"+cg[1]),[]),Cl=Math.min,Ur=Math.max,Hi=Math.round,Ai=Math.floor,un=r=>({x:r,y:r}),mE={left:"right",right:"left",bottom:"top",top:"bottom"},gE={start:"end",end:"start"};function Ic(r,a,i){return Ur(r,Cl(a,i))}function Hl(r,a){return typeof r=="function"?r(a):r}function zn(r){return r.split("-")[0]}function sn(r){return r.split("-")[1]}function Lp(r){return r==="x"?"y":"x"}function Af(r){return r==="y"?"height":"width"}const pE=new Set(["top","bottom"]);function or(r){return pE.has(zn(r))?"y":"x"}function kf(r){return Lp(or(r))}function Bp(r,a,i){i===void 0&&(i=!1);const o=sn(r),s=kf(r),f=Af(s);let d=s==="x"?o===(i?"end":"start")?"right":"left":o==="start"?"bottom":"top";return a.reference[f]>a.floating[f]&&(d=Gi(d)),[d,Gi(d)]}function yE(r){const a=Gi(r);return[qi(r),a,qi(a)]}function qi(r){return r.replace(/start|end/g,a=>gE[a])}const dg=["left","right"],hg=["right","left"],bE=["top","bottom"],vE=["bottom","top"];function xE(r,a,i){switch(r){case"top":case"bottom":return i?a?hg:dg:a?dg:hg;case"left":case"right":return a?bE:vE;default:return[]}}function wE(r,a,i,o){const s=sn(r);let f=xE(zn(r),i==="start",o);return s&&(f=f.map(d=>d+"-"+s),a&&(f=f.concat(f.map(qi)))),f}function Gi(r){return r.replace(/left|right|bottom|top/g,a=>mE[a])}function SE(r){return{top:0,right:0,bottom:0,left:0,...r}}function Hp(r){return typeof r!="number"?SE(r):{top:r,right:r,bottom:r,left:r}}function Yi(r){const{x:a,y:i,width:o,height:s}=r;return{width:o,height:s,top:i,left:a,right:a+o,bottom:i+s,x:a,y:i}}function mg(r,a,i){let{reference:o,floating:s}=r;const f=or(a),d=kf(a),h=Af(d),g=zn(a),m=f==="y",y=o.x+o.width/2-s.width/2,v=o.y+o.height/2-s.height/2,w=o[h]/2-s[h]/2;let S;switch(g){case"top":S={x:y,y:o.y-s.height};break;case"bottom":S={x:y,y:o.y+o.height};break;case"right":S={x:o.x+o.width,y:v};break;case"left":S={x:o.x-s.width,y:v};break;default:S={x:o.x,y:o.y}}switch(sn(a)){case"start":S[d]-=w*(i&&m?-1:1);break;case"end":S[d]+=w*(i&&m?-1:1);break}return S}const EE=async(r,a,i)=>{const{placement:o="bottom",strategy:s="absolute",middleware:f=[],platform:d}=i,h=f.filter(Boolean),g=await(d.isRTL==null?void 0:d.isRTL(a));let m=await d.getElementRects({reference:r,floating:a,strategy:s}),{x:y,y:v}=mg(m,o,g),w=o,S={},x=0;for(let R=0;R<h.length;R++){const{name:k,fn:O}=h[R],{x:M,y:H,data:$,reset:V}=await O({x:y,y:v,initialPlacement:o,placement:w,strategy:s,middlewareData:S,rects:m,platform:d,elements:{reference:r,floating:a}});y=M??y,v=H??v,S={...S,[k]:{...S[k],...$}},V&&x<=50&&(x++,typeof V=="object"&&(V.placement&&(w=V.placement),V.rects&&(m=V.rects===!0?await d.getElementRects({reference:r,floating:a,strategy:s}):V.rects),{x:y,y:v}=mg(m,w,g)),R=-1)}return{x:y,y:v,placement:w,strategy:s,middlewareData:S}};async function Of(r,a){var i;a===void 0&&(a={});const{x:o,y:s,platform:f,rects:d,elements:h,strategy:g}=r,{boundary:m="clippingAncestors",rootBoundary:y="viewport",elementContext:v="floating",altBoundary:w=!1,padding:S=0}=Hl(a,r),x=Hp(S),k=h[w?v==="floating"?"reference":"floating":v],O=Yi(await f.getClippingRect({element:(i=await(f.isElement==null?void 0:f.isElement(k)))==null||i?k:k.contextElement||await(f.getDocumentElement==null?void 0:f.getDocumentElement(h.floating)),boundary:m,rootBoundary:y,strategy:g})),M=v==="floating"?{x:o,y:s,width:d.floating.width,height:d.floating.height}:d.reference,H=await(f.getOffsetParent==null?void 0:f.getOffsetParent(h.floating)),$=await(f.isElement==null?void 0:f.isElement(H))?await(f.getScale==null?void 0:f.getScale(H))||{x:1,y:1}:{x:1,y:1},V=Yi(f.convertOffsetParentRelativeRectToViewportRelativeRect?await f.convertOffsetParentRelativeRectToViewportRelativeRect({elements:h,rect:M,offsetParent:H,strategy:g}):M);return{top:(O.top-V.top+x.top)/$.y,bottom:(V.bottom-O.bottom+x.bottom)/$.y,left:(O.left-V.left+x.left)/$.x,right:(V.right-O.right+x.right)/$.x}}const RE=r=>({name:"arrow",options:r,async fn(a){const{x:i,y:o,placement:s,rects:f,platform:d,elements:h,middlewareData:g}=a,{element:m,padding:y=0}=Hl(r,a)||{};if(m==null)return{};const v=Hp(y),w={x:i,y:o},S=kf(s),x=Af(S),R=await d.getDimensions(m),k=S==="y",O=k?"top":"left",M=k?"bottom":"right",H=k?"clientHeight":"clientWidth",$=f.reference[x]+f.reference[S]-w[S]-f.floating[x],V=w[S]-f.reference[S],W=await(d.getOffsetParent==null?void 0:d.getOffsetParent(m));let ee=W?W[H]:0;(!ee||!await(d.isElement==null?void 0:d.isElement(W)))&&(ee=h.floating[H]||f.floating[x]);const fe=$/2-V/2,ne=ee/2-R[x]/2-1,se=Cl(v[O],ne),me=Cl(v[M],ne),te=se,ue=ee-R[x]-me,X=ee/2-R[x]/2+fe,I=Ic(te,X,ue),N=!g.arrow&&sn(s)!=null&&X!==I&&f.reference[x]/2-(X<te?se:me)-R[x]/2<0,q=N?X<te?X-te:X-ue:0;return{[S]:w[S]+q,data:{[S]:I,centerOffset:X-I-q,...N&&{alignmentOffset:q}},reset:N}}});function TE(r,a,i){return(r?[...i.filter(s=>sn(s)===r),...i.filter(s=>sn(s)!==r)]:i.filter(s=>zn(s)===s)).filter(s=>r?sn(s)===r||(a?qi(s)!==s:!1):!0)}const AE=function(r){return r===void 0&&(r={}),{name:"autoPlacement",options:r,async fn(a){var i,o,s;const{rects:f,middlewareData:d,placement:h,platform:g,elements:m}=a,{crossAxis:y=!1,alignment:v,allowedPlacements:w=fg,autoAlignment:S=!0,...x}=Hl(r,a),R=v!==void 0||w===fg?TE(v||null,S,w):w,k=await Of(a,x),O=((i=d.autoPlacement)==null?void 0:i.index)||0,M=R[O];if(M==null)return{};const H=Bp(M,f,await(g.isRTL==null?void 0:g.isRTL(m.floating)));if(h!==M)return{reset:{placement:R[0]}};const $=[k[zn(M)],k[H[0]],k[H[1]]],V=[...((o=d.autoPlacement)==null?void 0:o.overflows)||[],{placement:M,overflows:$}],W=R[O+1];if(W)return{data:{index:O+1,overflows:V},reset:{placement:W}};const ee=V.map(se=>{const me=sn(se.placement);return[se.placement,me&&y?se.overflows.slice(0,2).reduce((te,ue)=>te+ue,0):se.overflows[0],se.overflows]}).sort((se,me)=>se[1]-me[1]),ne=((s=ee.filter(se=>se[2].slice(0,sn(se[0])?2:3).every(me=>me<=0))[0])==null?void 0:s[0])||ee[0][0];return ne!==h?{data:{index:O+1,overflows:V},reset:{placement:ne}}:{}}}},kE=function(r){return r===void 0&&(r={}),{name:"flip",options:r,async fn(a){var i,o;const{placement:s,middlewareData:f,rects:d,initialPlacement:h,platform:g,elements:m}=a,{mainAxis:y=!0,crossAxis:v=!0,fallbackPlacements:w,fallbackStrategy:S="bestFit",fallbackAxisSideDirection:x="none",flipAlignment:R=!0,...k}=Hl(r,a);if((i=f.arrow)!=null&&i.alignmentOffset)return{};const O=zn(s),M=or(h),H=zn(h)===h,$=await(g.isRTL==null?void 0:g.isRTL(m.floating)),V=w||(H||!R?[Gi(h)]:yE(h)),W=x!=="none";!w&&W&&V.push(...wE(h,R,x,$));const ee=[h,...V],fe=await Of(a,k),ne=[];let se=((o=f.flip)==null?void 0:o.overflows)||[];if(y&&ne.push(fe[O]),v){const X=Bp(s,d,$);ne.push(fe[X[0]],fe[X[1]])}if(se=[...se,{placement:s,overflows:ne}],!ne.every(X=>X<=0)){var me,te;const X=(((me=f.flip)==null?void 0:me.index)||0)+1,I=ee[X];if(I&&(!(v==="alignment"?M!==or(I):!1)||se.every(L=>L.overflows[0]>0&&or(L.placement)===M)))return{data:{index:X,overflows:se},reset:{placement:I}};let N=(te=se.filter(q=>q.overflows[0]<=0).sort((q,L)=>q.overflows[1]-L.overflows[1])[0])==null?void 0:te.placement;if(!N)switch(S){case"bestFit":{var ue;const q=(ue=se.filter(L=>{if(W){const K=or(L.placement);return K===M||K==="y"}return!0}).map(L=>[L.placement,L.overflows.filter(K=>K>0).reduce((K,E)=>K+E,0)]).sort((L,K)=>L[1]-K[1])[0])==null?void 0:ue[0];q&&(N=q);break}case"initialPlacement":N=h;break}if(s!==N)return{reset:{placement:N}}}return{}}}},OE=new Set(["left","top"]);async function CE(r,a){const{placement:i,platform:o,elements:s}=r,f=await(o.isRTL==null?void 0:o.isRTL(s.floating)),d=zn(i),h=sn(i),g=or(i)==="y",m=OE.has(d)?-1:1,y=f&&g?-1:1,v=Hl(a,r);let{mainAxis:w,crossAxis:S,alignmentAxis:x}=typeof v=="number"?{mainAxis:v,crossAxis:0,alignmentAxis:null}:{mainAxis:v.mainAxis||0,crossAxis:v.crossAxis||0,alignmentAxis:v.alignmentAxis};return h&&typeof x=="number"&&(S=h==="end"?x*-1:x),g?{x:S*y,y:w*m}:{x:w*m,y:S*y}}const ME=function(r){return r===void 0&&(r=0),{name:"offset",options:r,async fn(a){var i,o;const{x:s,y:f,placement:d,middlewareData:h}=a,g=await CE(a,r);return d===((i=h.offset)==null?void 0:i.placement)&&(o=h.arrow)!=null&&o.alignmentOffset?{}:{x:s+g.x,y:f+g.y,data:{...g,placement:d}}}}},NE=function(r){return r===void 0&&(r={}),{name:"shift",options:r,async fn(a){const{x:i,y:o,placement:s}=a,{mainAxis:f=!0,crossAxis:d=!1,limiter:h={fn:k=>{let{x:O,y:M}=k;return{x:O,y:M}}},...g}=Hl(r,a),m={x:i,y:o},y=await Of(a,g),v=or(zn(s)),w=Lp(v);let S=m[w],x=m[v];if(f){const k=w==="y"?"top":"left",O=w==="y"?"bottom":"right",M=S+y[k],H=S-y[O];S=Ic(M,S,H)}if(d){const k=v==="y"?"top":"left",O=v==="y"?"bottom":"right",M=x+y[k],H=x-y[O];x=Ic(M,x,H)}const R=h.fn({...a,[w]:S,[v]:x});return{...R,data:{x:R.x-i,y:R.y-o,enabled:{[w]:f,[v]:d}}}}}};function qp(r){const a=Zt(r);let i=parseFloat(a.width)||0,o=parseFloat(a.height)||0;const s=Tt(r),f=s?r.offsetWidth:i,d=s?r.offsetHeight:o,h=Hi(i)!==f||Hi(o)!==d;return h&&(i=f,o=d),{width:i,height:o,$:h}}function Cf(r){return Ge(r)?r:r.contextElement}function Ol(r){const a=Cf(r);if(!Tt(a))return un(1);const i=a.getBoundingClientRect(),{width:o,height:s,$:f}=qp(a);let d=(f?Hi(i.width):i.width)/o,h=(f?Hi(i.height):i.height)/s;return(!d||!Number.isFinite(d))&&(d=1),(!h||!Number.isFinite(h))&&(h=1),{x:d,y:h}}const zE=un(0);function Gp(r){const a=St(r);return!Wi()||!a.visualViewport?zE:{x:a.visualViewport.offsetLeft,y:a.visualViewport.offsetTop}}function _E(r,a,i){return a===void 0&&(a=!1),!i||a&&i!==St(r)?!1:a}function Lr(r,a,i,o){a===void 0&&(a=!1),i===void 0&&(i=!1);const s=r.getBoundingClientRect(),f=Cf(r);let d=un(1);a&&(o?Ge(o)&&(d=Ol(o)):d=Ol(r));const h=_E(f,i,o)?Gp(f):un(0);let g=(s.left+h.x)/d.x,m=(s.top+h.y)/d.y,y=s.width/d.x,v=s.height/d.y;if(f){const w=St(f),S=o&&Ge(o)?St(o):o;let x=w,R=Wc(x);for(;R&&o&&S!==x;){const k=Ol(R),O=R.getBoundingClientRect(),M=Zt(R),H=O.left+(R.clientLeft+parseFloat(M.paddingLeft))*k.x,$=O.top+(R.clientTop+parseFloat(M.paddingTop))*k.y;g*=k.x,m*=k.y,y*=k.x,v*=k.y,g+=H,m+=$,x=St(R),R=Wc(x)}}return Yi({width:y,height:v,x:g,y:m})}function Mf(r,a){const i=Ii(r).scrollLeft;return a?a.left+i:Lr(cn(r)).left+i}function Yp(r,a,i){i===void 0&&(i=!1);const o=r.getBoundingClientRect(),s=o.left+a.scrollLeft-(i?0:Mf(r,o)),f=o.top+a.scrollTop;return{x:s,y:f}}function DE(r){let{elements:a,rect:i,offsetParent:o,strategy:s}=r;const f=s==="fixed",d=cn(o),h=a?Fi(a.floating):!1;if(o===d||h&&f)return i;let g={scrollLeft:0,scrollTop:0},m=un(1);const y=un(0),v=Tt(o);if((v||!v&&!f)&&((Bl(o)!=="body"||no(d))&&(g=Ii(o)),Tt(o))){const S=Lr(o);m=Ol(o),y.x=S.x+o.clientLeft,y.y=S.y+o.clientTop}const w=d&&!v&&!f?Yp(d,g,!0):un(0);return{width:i.width*m.x,height:i.height*m.y,x:i.x*m.x-g.scrollLeft*m.x+y.x+w.x,y:i.y*m.y-g.scrollTop*m.y+y.y+w.y}}function UE(r){return Array.from(r.getClientRects())}function jE(r){const a=cn(r),i=Ii(r),o=r.ownerDocument.body,s=Ur(a.scrollWidth,a.clientWidth,o.scrollWidth,o.clientWidth),f=Ur(a.scrollHeight,a.clientHeight,o.scrollHeight,o.clientHeight);let d=-i.scrollLeft+Mf(r);const h=-i.scrollTop;return Zt(o).direction==="rtl"&&(d+=Ur(a.clientWidth,o.clientWidth)-s),{width:s,height:f,x:d,y:h}}function LE(r,a){const i=St(r),o=cn(r),s=i.visualViewport;let f=o.clientWidth,d=o.clientHeight,h=0,g=0;if(s){f=s.width,d=s.height;const m=Wi();(!m||m&&a==="fixed")&&(h=s.offsetLeft,g=s.offsetTop)}return{width:f,height:d,x:h,y:g}}const BE=new Set(["absolute","fixed"]);function HE(r,a){const i=Lr(r,!0,a==="fixed"),o=i.top+r.clientTop,s=i.left+r.clientLeft,f=Tt(r)?Ol(r):un(1),d=r.clientWidth*f.x,h=r.clientHeight*f.y,g=s*f.x,m=o*f.y;return{width:d,height:h,x:g,y:m}}function gg(r,a,i){let o;if(a==="viewport")o=LE(r,i);else if(a==="document")o=jE(cn(r));else if(Ge(a))o=HE(a,i);else{const s=Gp(r);o={x:a.x-s.x,y:a.y-s.y,width:a.width,height:a.height}}return Yi(o)}function Vp(r,a){const i=Nn(r);return i===a||!Ge(i)||Cn(i)?!1:Zt(i).position==="fixed"||Vp(i,a)}function qE(r,a){const i=a.get(r);if(i)return i;let o=ur(r,[],!1).filter(h=>Ge(h)&&Bl(h)!=="body"),s=null;const f=Zt(r).position==="fixed";let d=f?Nn(r):r;for(;Ge(d)&&!Cn(d);){const h=Zt(d),g=Tf(d);!g&&h.position==="fixed"&&(s=null),(f?!g&&!s:!g&&h.position==="static"&&!!s&&BE.has(s.position)||no(d)&&!g&&Vp(r,d))?o=o.filter(y=>y!==d):s=h,d=Nn(d)}return a.set(r,o),o}function GE(r){let{element:a,boundary:i,rootBoundary:o,strategy:s}=r;const d=[...i==="clippingAncestors"?Fi(a)?[]:qE(a,this._c):[].concat(i),o],h=d[0],g=d.reduce((m,y)=>{const v=gg(a,y,s);return m.top=Ur(v.top,m.top),m.right=Cl(v.right,m.right),m.bottom=Cl(v.bottom,m.bottom),m.left=Ur(v.left,m.left),m},gg(a,h,s));return{width:g.right-g.left,height:g.bottom-g.top,x:g.left,y:g.top}}function YE(r){const{width:a,height:i}=qp(r);return{width:a,height:i}}function VE(r,a,i){const o=Tt(a),s=cn(a),f=i==="fixed",d=Lr(r,!0,f,a);let h={scrollLeft:0,scrollTop:0};const g=un(0);function m(){g.x=Mf(s)}if(o||!o&&!f)if((Bl(a)!=="body"||no(s))&&(h=Ii(a)),o){const S=Lr(a,!0,f,a);g.x=S.x+a.clientLeft,g.y=S.y+a.clientTop}else s&&m();f&&!o&&s&&m();const y=s&&!o&&!f?Yp(s,h):un(0),v=d.left+h.scrollLeft-g.x-y.x,w=d.top+h.scrollTop-g.y-y.y;return{x:v,y:w,width:d.width,height:d.height}}function Dc(r){return Zt(r).position==="static"}function pg(r,a){if(!Tt(r)||Zt(r).position==="fixed")return null;if(a)return a(r);let i=r.offsetParent;return cn(r)===i&&(i=i.ownerDocument.body),i}function Xp(r,a){const i=St(r);if(Fi(r))return i;if(!Tt(r)){let s=Nn(r);for(;s&&!Cn(s);){if(Ge(s)&&!Dc(s))return s;s=Nn(s)}return i}let o=pg(r,a);for(;o&&IS(o)&&Dc(o);)o=pg(o,a);return o&&Cn(o)&&Dc(o)&&!Tf(o)?i:o||lE(r)||i}const XE=async function(r){const a=this.getOffsetParent||Xp,i=this.getDimensions,o=await i(r.floating);return{reference:VE(r.reference,await a(r.floating),r.strategy),floating:{x:0,y:0,width:o.width,height:o.height}}};function QE(r){return Zt(r).direction==="rtl"}const ZE={convertOffsetParentRelativeRectToViewportRelativeRect:DE,getDocumentElement:cn,getClippingRect:GE,getOffsetParent:Xp,getElementRects:XE,getClientRects:UE,getDimensions:YE,getScale:Ol,isElement:Ge,isRTL:QE};function Qp(r,a){return r.x===a.x&&r.y===a.y&&r.width===a.width&&r.height===a.height}function KE(r,a){let i=null,o;const s=cn(r);function f(){var h;clearTimeout(o),(h=i)==null||h.disconnect(),i=null}function d(h,g){h===void 0&&(h=!1),g===void 0&&(g=1),f();const m=r.getBoundingClientRect(),{left:y,top:v,width:w,height:S}=m;if(h||a(),!w||!S)return;const x=Ai(v),R=Ai(s.clientWidth-(y+w)),k=Ai(s.clientHeight-(v+S)),O=Ai(y),H={rootMargin:-x+"px "+-R+"px "+-k+"px "+-O+"px",threshold:Ur(0,Cl(1,g))||1};let $=!0;function V(W){const ee=W[0].intersectionRatio;if(ee!==g){if(!$)return d();ee?d(!1,ee):o=setTimeout(()=>{d(!1,1e-7)},1e3)}ee===1&&!Qp(m,r.getBoundingClientRect())&&d(),$=!1}try{i=new IntersectionObserver(V,{...H,root:s.ownerDocument})}catch{i=new IntersectionObserver(V,H)}i.observe(r)}return d(!0),f}function Zp(r,a,i,o){o===void 0&&(o={});const{ancestorScroll:s=!0,ancestorResize:f=!0,elementResize:d=typeof ResizeObserver=="function",layoutShift:h=typeof IntersectionObserver=="function",animationFrame:g=!1}=o,m=Cf(r),y=s||f?[...m?ur(m):[],...ur(a)]:[];y.forEach(O=>{s&&O.addEventListener("scroll",i,{passive:!0}),f&&O.addEventListener("resize",i)});const v=m&&h?KE(m,i):null;let w=-1,S=null;d&&(S=new ResizeObserver(O=>{let[M]=O;M&&M.target===m&&S&&(S.unobserve(a),cancelAnimationFrame(w),w=requestAnimationFrame(()=>{var H;(H=S)==null||H.observe(a)})),i()}),m&&!g&&S.observe(m),S.observe(a));let x,R=g?Lr(r):null;g&&k();function k(){const O=Lr(r);R&&!Qp(R,O)&&i(),R=O,x=requestAnimationFrame(k)}return i(),()=>{var O;y.forEach(M=>{s&&M.removeEventListener("scroll",i),f&&M.removeEventListener("resize",i)}),v==null||v(),(O=S)==null||O.disconnect(),S=null,g&&cancelAnimationFrame(x)}}const $E=ME,JE=AE,PE=NE,FE=kE,yg=RE,WE=(r,a,i)=>{const o=new Map,s={platform:ZE,...i},f={...s.platform,_c:o};return EE(r,a,{...s,platform:f})};var IE=typeof document<"u",eR=function(){},Mi=IE?T.useLayoutEffect:eR;function Vi(r,a){if(r===a)return!0;if(typeof r!=typeof a)return!1;if(typeof r=="function"&&r.toString()===a.toString())return!0;let i,o,s;if(r&&a&&typeof r=="object"){if(Array.isArray(r)){if(i=r.length,i!==a.length)return!1;for(o=i;o--!==0;)if(!Vi(r[o],a[o]))return!1;return!0}if(s=Object.keys(r),i=s.length,i!==Object.keys(a).length)return!1;for(o=i;o--!==0;)if(!{}.hasOwnProperty.call(a,s[o]))return!1;for(o=i;o--!==0;){const f=s[o];if(!(f==="_owner"&&r.$$typeof)&&!Vi(r[f],a[f]))return!1}return!0}return r!==r&&a!==a}function Kp(r){return typeof window>"u"?1:(r.ownerDocument.defaultView||window).devicePixelRatio||1}function bg(r,a){const i=Kp(r);return Math.round(a*i)/i}function Uc(r){const a=T.useRef(r);return Mi(()=>{a.current=r}),a}function tR(r){r===void 0&&(r={});const{placement:a="bottom",strategy:i="absolute",middleware:o=[],platform:s,elements:{reference:f,floating:d}={},transform:h=!0,whileElementsMounted:g,open:m}=r,[y,v]=T.useState({x:0,y:0,strategy:i,placement:a,middlewareData:{},isPositioned:!1}),[w,S]=T.useState(o);Vi(w,o)||S(o);const[x,R]=T.useState(null),[k,O]=T.useState(null),M=T.useCallback(L=>{L!==W.current&&(W.current=L,R(L))},[]),H=T.useCallback(L=>{L!==ee.current&&(ee.current=L,O(L))},[]),$=f||x,V=d||k,W=T.useRef(null),ee=T.useRef(null),fe=T.useRef(y),ne=g!=null,se=Uc(g),me=Uc(s),te=Uc(m),ue=T.useCallback(()=>{if(!W.current||!ee.current)return;const L={placement:a,strategy:i,middleware:w};me.current&&(L.platform=me.current),WE(W.current,ee.current,L).then(K=>{const E={...K,isPositioned:te.current!==!1};X.current&&!Vi(fe.current,E)&&(fe.current=E,Kx.flushSync(()=>{v(E)}))})},[w,a,i,me,te]);Mi(()=>{m===!1&&fe.current.isPositioned&&(fe.current.isPositioned=!1,v(L=>({...L,isPositioned:!1})))},[m]);const X=T.useRef(!1);Mi(()=>(X.current=!0,()=>{X.current=!1}),[]),Mi(()=>{if($&&(W.current=$),V&&(ee.current=V),$&&V){if(se.current)return se.current($,V,ue);ue()}},[$,V,ue,se,ne]);const I=T.useMemo(()=>({reference:W,floating:ee,setReference:M,setFloating:H}),[M,H]),N=T.useMemo(()=>({reference:$,floating:V}),[$,V]),q=T.useMemo(()=>{const L={position:i,left:0,top:0};if(!N.floating)return L;const K=bg(N.floating,y.x),E=bg(N.floating,y.y);return h?{...L,transform:"translate("+K+"px, "+E+"px)",...Kp(N.floating)>=1.5&&{willChange:"transform"}}:{position:i,left:K,top:E}},[i,h,N.floating,y.x,y.y]);return T.useMemo(()=>({...y,update:ue,refs:I,elements:N,floatingStyles:q}),[y,ue,I,N,q])}const nR=r=>{function a(i){return{}.hasOwnProperty.call(i,"current")}return{name:"arrow",options:r,fn(i){const{element:o,padding:s}=typeof r=="function"?r(i):r;return o&&a(o)?o.current!=null?yg({element:o.current,padding:s}).fn(i):{}:o?yg({element:o,padding:s}).fn(i):{}}}},rR=(r,a)=>({...$E(r),options:[r,a]}),lR=(r,a)=>({...PE(r),options:[r,a]}),aR=(r,a)=>({...FE(r),options:[r,a]}),oR=(r,a)=>({...JE(r),options:[r,a]}),iR=(r,a)=>({...nR(r),options:[r,a]}),$p={...m1},uR=$p.useInsertionEffect,sR=uR||(r=>r());function kn(r){const a=T.useRef(()=>{});return sR(()=>{a.current=r}),T.useCallback(function(){for(var i=arguments.length,o=new Array(i),s=0;s<i;s++)o[s]=arguments[s];return a.current==null?void 0:a.current(...o)},[])}var Ml=typeof document<"u"?T.useLayoutEffect:T.useEffect;let vg=!1,cR=0;const xg=()=>"floating-ui-"+Math.random().toString(36).slice(2,6)+cR++;function fR(){const[r,a]=T.useState(()=>vg?xg():void 0);return Ml(()=>{r==null&&a(xg())},[]),T.useEffect(()=>{vg=!0},[]),r}const dR=$p.useId,Jp=dR||fR;function hR(){const r=new Map;return{emit(a,i){var o;(o=r.get(a))==null||o.forEach(s=>s(i))},on(a,i){r.set(a,[...r.get(a)||[],i])},off(a,i){var o;r.set(a,((o=r.get(a))==null?void 0:o.filter(s=>s!==i))||[])}}}const mR=T.createContext(null),gR=T.createContext(null),Nf=()=>{var r;return((r=T.useContext(mR))==null?void 0:r.id)||null},zf=()=>T.useContext(gR);function _f(r){return"data-floating-ui-"+r}function jc(r){const a=T.useRef(r);return Ml(()=>{a.current=r}),a}const wg=_f("safe-polygon");function Lc(r,a,i){return i&&!Wa(i)?0:typeof r=="number"?r:r==null?void 0:r[a]}function pR(r,a){a===void 0&&(a={});const{open:i,onOpenChange:o,dataRef:s,events:f,elements:d}=r,{enabled:h=!0,delay:g=0,handleClose:m=null,mouseOnly:y=!1,restMs:v=0,move:w=!0}=a,S=zf(),x=Nf(),R=jc(m),k=jc(g),O=jc(i),M=T.useRef(),H=T.useRef(-1),$=T.useRef(),V=T.useRef(-1),W=T.useRef(!0),ee=T.useRef(!1),fe=T.useRef(()=>{}),ne=T.useRef(!1),se=T.useCallback(()=>{var q;const L=(q=s.current.openEvent)==null?void 0:q.type;return(L==null?void 0:L.includes("mouse"))&&L!=="mousedown"},[s]);T.useEffect(()=>{if(!h)return;function q(L){let{open:K}=L;K||(clearTimeout(H.current),clearTimeout(V.current),W.current=!0,ne.current=!1)}return f.on("openchange",q),()=>{f.off("openchange",q)}},[h,f]),T.useEffect(()=>{if(!h||!R.current||!i)return;function q(K){se()&&o(!1,K,"hover")}const L=_r(d.floating).documentElement;return L.addEventListener("mouseleave",q),()=>{L.removeEventListener("mouseleave",q)}},[d.floating,i,o,h,R,se]);const me=T.useCallback(function(q,L,K){L===void 0&&(L=!0),K===void 0&&(K="hover");const E=Lc(k.current,"close",M.current);E&&!$.current?(clearTimeout(H.current),H.current=window.setTimeout(()=>o(!1,q,K),E)):L&&(clearTimeout(H.current),o(!1,q,K))},[k,o]),te=kn(()=>{fe.current(),$.current=void 0}),ue=kn(()=>{if(ee.current){const q=_r(d.floating).body;q.style.pointerEvents="",q.removeAttribute(wg),ee.current=!1}}),X=kn(()=>s.current.openEvent?["click","mousedown"].includes(s.current.openEvent.type):!1);T.useEffect(()=>{if(!h)return;function q(_){if(clearTimeout(H.current),W.current=!1,y&&!Wa(M.current)||v>0&&!Lc(k.current,"open"))return;const P=Lc(k.current,"open",M.current);P?H.current=window.setTimeout(()=>{O.current||o(!0,_,"hover")},P):i||o(!0,_,"hover")}function L(_){if(X())return;fe.current();const P=_r(d.floating);if(clearTimeout(V.current),ne.current=!1,R.current&&s.current.floatingContext){i||clearTimeout(H.current),$.current=R.current({...s.current.floatingContext,tree:S,x:_.clientX,y:_.clientY,onClose(){ue(),te(),X()||me(_,!0,"safe-polygon")}});const oe=$.current;P.addEventListener("mousemove",oe),fe.current=()=>{P.removeEventListener("mousemove",oe)};return}(M.current==="touch"?!sr(d.floating,_.relatedTarget):!0)&&me(_)}function K(_){X()||s.current.floatingContext&&(R.current==null||R.current({...s.current.floatingContext,tree:S,x:_.clientX,y:_.clientY,onClose(){ue(),te(),X()||me(_)}})(_))}if(Ge(d.domReference)){var E;const _=d.domReference;return i&&_.addEventListener("mouseleave",K),(E=d.floating)==null||E.addEventListener("mouseleave",K),w&&_.addEventListener("mousemove",q,{once:!0}),_.addEventListener("mouseenter",q),_.addEventListener("mouseleave",L),()=>{var P;i&&_.removeEventListener("mouseleave",K),(P=d.floating)==null||P.removeEventListener("mouseleave",K),w&&_.removeEventListener("mousemove",q),_.removeEventListener("mouseenter",q),_.removeEventListener("mouseleave",L)}}},[d,h,r,y,v,w,me,te,ue,o,i,O,S,k,R,s,X]),Ml(()=>{var q;if(h&&i&&(q=R.current)!=null&&q.__options.blockPointerEvents&&se()){ee.current=!0;const K=d.floating;if(Ge(d.domReference)&&K){var L;const E=_r(d.floating).body;E.setAttribute(wg,"");const _=d.domReference,P=S==null||(L=S.nodesRef.current.find(F=>F.id===x))==null||(L=L.context)==null?void 0:L.elements.floating;return P&&(P.style.pointerEvents=""),E.style.pointerEvents="none",_.style.pointerEvents="auto",K.style.pointerEvents="auto",()=>{E.style.pointerEvents="",_.style.pointerEvents="",K.style.pointerEvents=""}}}},[h,i,x,d,S,R,se]),Ml(()=>{i||(M.current=void 0,ne.current=!1,te(),ue())},[i,te,ue]),T.useEffect(()=>()=>{te(),clearTimeout(H.current),clearTimeout(V.current),ue()},[h,d.domReference,te,ue]);const I=T.useMemo(()=>{function q(L){M.current=L.pointerType}return{onPointerDown:q,onPointerEnter:q,onMouseMove(L){const{nativeEvent:K}=L;function E(){!W.current&&!O.current&&o(!0,K,"hover")}y&&!Wa(M.current)||i||v===0||ne.current&&L.movementX**2+L.movementY**2<2||(clearTimeout(V.current),M.current==="touch"?E():(ne.current=!0,V.current=window.setTimeout(E,v)))}}},[y,o,i,O,v]),N=T.useMemo(()=>({onMouseEnter(){clearTimeout(H.current)},onMouseLeave(q){X()||me(q.nativeEvent,!1)}}),[me,X]);return T.useMemo(()=>h?{reference:I,floating:N}:{},[h,I,N])}function Ni(r,a){let i=r.filter(s=>{var f;return s.parentId===a&&((f=s.context)==null?void 0:f.open)}),o=i;for(;o.length;)o=r.filter(s=>{var f;return(f=o)==null?void 0:f.some(d=>{var h;return s.parentId===d.id&&((h=s.context)==null?void 0:h.open)})}),i=i.concat(o);return i}const yR="data-floating-ui-focusable";function Sg(r){return Tt(r.target)&&r.target.tagName==="BUTTON"}function Eg(r){return jp(r)}function bR(r,a){a===void 0&&(a={});const{open:i,onOpenChange:o,dataRef:s,elements:{domReference:f}}=r,{enabled:d=!0,event:h="click",toggle:g=!0,ignoreMouse:m=!1,keyboardHandlers:y=!0,stickIfOpen:v=!0}=a,w=T.useRef(),S=T.useRef(!1),x=T.useMemo(()=>({onPointerDown(R){w.current=R.pointerType},onMouseDown(R){const k=w.current;R.button===0&&h!=="click"&&(Wa(k,!0)&&m||(i&&g&&(!(s.current.openEvent&&v)||s.current.openEvent.type==="mousedown")?o(!1,R.nativeEvent,"click"):(R.preventDefault(),o(!0,R.nativeEvent,"click"))))},onClick(R){const k=w.current;if(h==="mousedown"&&w.current){w.current=void 0;return}Wa(k,!0)&&m||(i&&g&&(!(s.current.openEvent&&v)||s.current.openEvent.type==="click")?o(!1,R.nativeEvent,"click"):o(!0,R.nativeEvent,"click"))},onKeyDown(R){w.current=void 0,!(R.defaultPrevented||!y||Sg(R))&&(R.key===" "&&!Eg(f)&&(R.preventDefault(),S.current=!0),R.key==="Enter"&&o(!(i&&g),R.nativeEvent,"click"))},onKeyUp(R){R.defaultPrevented||!y||Sg(R)||Eg(f)||R.key===" "&&S.current&&(S.current=!1,o(!(i&&g),R.nativeEvent,"click"))}}),[s,f,h,m,y,o,i,v,g]);return T.useMemo(()=>d?{reference:x}:{},[d,x])}const vR={pointerdown:"onPointerDown",mousedown:"onMouseDown",click:"onClick"},xR={pointerdown:"onPointerDownCapture",mousedown:"onMouseDownCapture",click:"onClickCapture"},Rg=r=>{var a,i;return{escapeKey:typeof r=="boolean"?r:(a=r==null?void 0:r.escapeKey)!=null?a:!1,outsidePress:typeof r=="boolean"?r:(i=r==null?void 0:r.outsidePress)!=null?i:!0}};function wR(r,a){a===void 0&&(a={});const{open:i,onOpenChange:o,elements:s,dataRef:f}=r,{enabled:d=!0,escapeKey:h=!0,outsidePress:g=!0,outsidePressEvent:m="pointerdown",referencePress:y=!1,referencePressEvent:v="pointerdown",ancestorScroll:w=!1,bubbles:S,capture:x}=a,R=zf(),k=kn(typeof g=="function"?g:()=>!1),O=typeof g=="function"?k:g,M=T.useRef(!1),H=T.useRef(!1),{escapeKey:$,outsidePress:V}=Rg(S),{escapeKey:W,outsidePress:ee}=Rg(x),fe=T.useRef(!1),ne=kn(I=>{var N;if(!i||!d||!h||I.key!=="Escape"||fe.current)return;const q=(N=f.current.floatingContext)==null?void 0:N.nodeId,L=R?Ni(R.nodesRef.current,q):[];if(!$&&(I.stopPropagation(),L.length>0)){let K=!0;if(L.forEach(E=>{var _;if((_=E.context)!=null&&_.open&&!E.context.dataRef.current.__escapeKeyBubbles){K=!1;return}}),!K)return}o(!1,cE(I)?I.nativeEvent:I,"escape-key")}),se=kn(I=>{var N;const q=()=>{var L;ne(I),(L=zr(I))==null||L.removeEventListener("keydown",q)};(N=zr(I))==null||N.addEventListener("keydown",q)}),me=kn(I=>{var N;const q=M.current;M.current=!1;const L=H.current;if(H.current=!1,m==="click"&&L||q||typeof O=="function"&&!O(I))return;const K=zr(I),E="["+_f("inert")+"]",_=_r(s.floating).querySelectorAll(E);let P=Ge(K)?K:null;for(;P&&!Cn(P);){const ae=Nn(P);if(Cn(ae)||!Ge(ae))break;P=ae}if(_.length&&Ge(K)&&!fE(K)&&!sr(K,s.floating)&&Array.from(_).every(ae=>!sr(P,ae)))return;if(Tt(K)&&X){const ae=Cn(K),ke=Zt(K),de=/auto|scroll/,Ce=ae||de.test(ke.overflowX),At=ae||de.test(ke.overflowY),pt=Ce&&K.clientWidth>0&&K.scrollWidth>K.clientWidth,jt=At&&K.clientHeight>0&&K.scrollHeight>K.clientHeight,Vl=ke.direction==="rtl",Xl=jt&&(Vl?I.offsetX<=K.offsetWidth-K.clientWidth:I.offsetX>K.clientWidth),cu=pt&&I.offsetY>K.clientHeight;if(Xl||cu)return}const F=(N=f.current.floatingContext)==null?void 0:N.nodeId,oe=R&&Ni(R.nodesRef.current,F).some(ae=>{var ke;return _c(I,(ke=ae.context)==null?void 0:ke.elements.floating)});if(_c(I,s.floating)||_c(I,s.domReference)||oe)return;const be=R?Ni(R.nodesRef.current,F):[];if(be.length>0){let ae=!0;if(be.forEach(ke=>{var de;if((de=ke.context)!=null&&de.open&&!ke.context.dataRef.current.__outsidePressBubbles){ae=!1;return}}),!ae)return}o(!1,I,"outside-press")}),te=kn(I=>{var N;const q=()=>{var L;me(I),(L=zr(I))==null||L.removeEventListener(m,q)};(N=zr(I))==null||N.addEventListener(m,q)});T.useEffect(()=>{if(!i||!d)return;f.current.__escapeKeyBubbles=$,f.current.__outsidePressBubbles=V;let I=-1;function N(_){o(!1,_,"ancestor-scroll")}function q(){window.clearTimeout(I),fe.current=!0}function L(){I=window.setTimeout(()=>{fe.current=!1},Wi()?5:0)}const K=_r(s.floating);h&&(K.addEventListener("keydown",W?se:ne,W),K.addEventListener("compositionstart",q),K.addEventListener("compositionend",L)),O&&K.addEventListener(m,ee?te:me,ee);let E=[];return w&&(Ge(s.domReference)&&(E=ur(s.domReference)),Ge(s.floating)&&(E=E.concat(ur(s.floating))),!Ge(s.reference)&&s.reference&&s.reference.contextElement&&(E=E.concat(ur(s.reference.contextElement)))),E=E.filter(_=>{var P;return _!==((P=K.defaultView)==null?void 0:P.visualViewport)}),E.forEach(_=>{_.addEventListener("scroll",N,{passive:!0})}),()=>{h&&(K.removeEventListener("keydown",W?se:ne,W),K.removeEventListener("compositionstart",q),K.removeEventListener("compositionend",L)),O&&K.removeEventListener(m,ee?te:me,ee),E.forEach(_=>{_.removeEventListener("scroll",N)}),window.clearTimeout(I)}},[f,s,h,O,m,i,o,w,d,$,V,ne,W,se,me,ee,te]),T.useEffect(()=>{M.current=!1},[O,m]);const ue=T.useMemo(()=>({onKeyDown:ne,...y&&{[vR[v]]:I=>{o(!1,I.nativeEvent,"reference-press")},...v!=="click"&&{onClick(I){o(!1,I.nativeEvent,"reference-press")}}}}),[ne,o,y,v]),X=T.useMemo(()=>({onKeyDown:ne,onMouseDown(){H.current=!0},onMouseUp(){H.current=!0},[xR[m]]:()=>{M.current=!0}}),[ne,m]);return T.useMemo(()=>d?{reference:ue,floating:X}:{},[d,ue,X])}function SR(r){const{open:a=!1,onOpenChange:i,elements:o}=r,s=Jp(),f=T.useRef({}),[d]=T.useState(()=>hR()),h=Nf()!=null,[g,m]=T.useState(o.reference),y=kn((S,x,R)=>{f.current.openEvent=S?x:void 0,d.emit("openchange",{open:S,event:x,reason:R,nested:h}),i==null||i(S,x,R)}),v=T.useMemo(()=>({setPositionReference:m}),[]),w=T.useMemo(()=>({reference:g||o.reference||null,floating:o.floating||null,domReference:o.reference}),[g,o.reference,o.floating]);return T.useMemo(()=>({dataRef:f,open:a,onOpenChange:y,elements:w,events:d,floatingId:s,refs:v}),[a,y,w,d,s,v])}function ER(r){r===void 0&&(r={});const{nodeId:a}=r,i=SR({...r,elements:{reference:null,floating:null,...r.elements}}),o=r.rootContext||i,s=o.elements,[f,d]=T.useState(null),[h,g]=T.useState(null),y=(s==null?void 0:s.domReference)||f,v=T.useRef(null),w=zf();Ml(()=>{y&&(v.current=y)},[y]);const S=tR({...r,elements:{...s,...h&&{reference:h}}}),x=T.useCallback(H=>{const $=Ge(H)?{getBoundingClientRect:()=>H.getBoundingClientRect(),contextElement:H}:H;g($),S.refs.setReference($)},[S.refs]),R=T.useCallback(H=>{(Ge(H)||H===null)&&(v.current=H,d(H)),(Ge(S.refs.reference.current)||S.refs.reference.current===null||H!==null&&!Ge(H))&&S.refs.setReference(H)},[S.refs]),k=T.useMemo(()=>({...S.refs,setReference:R,setPositionReference:x,domReference:v}),[S.refs,R,x]),O=T.useMemo(()=>({...S.elements,domReference:y}),[S.elements,y]),M=T.useMemo(()=>({...S,...o,refs:k,elements:O,nodeId:a}),[S,k,O,a,o]);return Ml(()=>{o.dataRef.current.floatingContext=M;const H=w==null?void 0:w.nodesRef.current.find($=>$.id===a);H&&(H.context=M)}),T.useMemo(()=>({...S,context:M,refs:k,elements:O}),[S,k,O,M])}function RR(r,a){a===void 0&&(a={});const{open:i,onOpenChange:o,events:s,dataRef:f,elements:d}=r,{enabled:h=!0,visibleOnly:g=!0}=a,m=T.useRef(!1),y=T.useRef(),v=T.useRef(!0);T.useEffect(()=>{if(!h)return;const S=St(d.domReference);function x(){!i&&Tt(d.domReference)&&d.domReference===ug(_r(d.domReference))&&(m.current=!0)}function R(){v.current=!0}return S.addEventListener("blur",x),S.addEventListener("keydown",R,!0),()=>{S.removeEventListener("blur",x),S.removeEventListener("keydown",R,!0)}},[d.domReference,i,h]),T.useEffect(()=>{if(!h)return;function S(x){let{reason:R}=x;(R==="reference-press"||R==="escape-key")&&(m.current=!0)}return s.on("openchange",S),()=>{s.off("openchange",S)}},[s,h]),T.useEffect(()=>()=>{clearTimeout(y.current)},[]);const w=T.useMemo(()=>({onPointerDown(S){oE(S.nativeEvent)||(v.current=!1)},onMouseLeave(){m.current=!1},onFocus(S){if(m.current)return;const x=zr(S.nativeEvent);if(g&&Ge(x))try{if(iE()&&uE())throw Error();if(!x.matches(":focus-visible"))return}catch{if(!v.current&&!jp(x))return}o(!0,S.nativeEvent,"focus")},onBlur(S){m.current=!1;const x=S.relatedTarget,R=S.nativeEvent,k=Ge(x)&&x.hasAttribute(_f("focus-guard"))&&x.getAttribute("data-type")==="outside";y.current=window.setTimeout(()=>{var O;const M=ug(d.domReference?d.domReference.ownerDocument:document);!x&&M===d.domReference||sr((O=f.current.floatingContext)==null?void 0:O.refs.floating.current,M)||sr(d.domReference,M)||k||o(!1,R,"focus")})}}),[f,d.domReference,o,g]);return T.useMemo(()=>h?{reference:w}:{},[h,w])}const Tg="active",Ag="selected";function Bc(r,a,i){const o=new Map,s=i==="item";let f=r;if(s&&r){const{[Tg]:d,[Ag]:h,...g}=r;f=g}return{...i==="floating"&&{tabIndex:-1,[yR]:""},...f,...a.map(d=>{const h=d?d[i]:null;return typeof h=="function"?r?h(r):null:h}).concat(r).reduce((d,h)=>(h&&Object.entries(h).forEach(g=>{let[m,y]=g;if(!(s&&[Tg,Ag].includes(m)))if(m.indexOf("on")===0){if(o.has(m)||o.set(m,[]),typeof y=="function"){var v;(v=o.get(m))==null||v.push(y),d[m]=function(){for(var w,S=arguments.length,x=new Array(S),R=0;R<S;R++)x[R]=arguments[R];return(w=o.get(m))==null?void 0:w.map(k=>k(...x)).find(k=>k!==void 0)}}}else d[m]=y}),d),{})}}function TR(r){r===void 0&&(r=[]);const a=r.map(h=>h==null?void 0:h.reference),i=r.map(h=>h==null?void 0:h.floating),o=r.map(h=>h==null?void 0:h.item),s=T.useCallback(h=>Bc(h,r,"reference"),a),f=T.useCallback(h=>Bc(h,r,"floating"),i),d=T.useCallback(h=>Bc(h,r,"item"),o);return T.useMemo(()=>({getReferenceProps:s,getFloatingProps:f,getItemProps:d}),[s,f,d])}const AR=new Map([["select","listbox"],["combobox","listbox"],["label",!1]]);function kR(r,a){var i;a===void 0&&(a={});const{open:o,floatingId:s}=r,{enabled:f=!0,role:d="dialog"}=a,h=(i=AR.get(d))!=null?i:d,g=Jp(),y=Nf()!=null,v=T.useMemo(()=>h==="tooltip"||d==="label"?{["aria-"+(d==="label"?"labelledby":"describedby")]:o?s:void 0}:{"aria-expanded":o?"true":"false","aria-haspopup":h==="alertdialog"?"dialog":h,"aria-controls":o?s:void 0,...h==="listbox"&&{role:"combobox"},...h==="menu"&&{id:g},...h==="menu"&&y&&{role:"menuitem"},...d==="select"&&{"aria-autocomplete":"none"},...d==="combobox"&&{"aria-autocomplete":"list"}},[h,s,y,o,g,d]),w=T.useMemo(()=>{const x={id:s,...h&&{role:h}};return h==="tooltip"||d==="label"?x:{...x,...h==="menu"&&{"aria-labelledby":g}}},[h,s,g,d]),S=T.useCallback(x=>{let{active:R,selected:k}=x;const O={role:"option",...R&&{id:s+"-option"}};switch(d){case"select":return{...O,"aria-selected":R&&k};case"combobox":return{...O,...R&&{"aria-selected":!0}}}return{}},[s,d]);return T.useMemo(()=>f?{reference:v,floating:w,item:S}:{},[f,v,w,S])}function kg(r,a){const[i,o]=r;let s=!1;const f=a.length;for(let d=0,h=f-1;d<f;h=d++){const[g,m]=a[d]||[0,0],[y,v]=a[h]||[0,0];m>=o!=v>=o&&i<=(y-g)*(o-m)/(v-m)+g&&(s=!s)}return s}function OR(r,a){return r[0]>=a.x&&r[0]<=a.x+a.width&&r[1]>=a.y&&r[1]<=a.y+a.height}function CR(r){r===void 0&&(r={});const{buffer:a=.5,blockPointerEvents:i=!1,requireIntent:o=!0}=r;let s,f=!1,d=null,h=null,g=performance.now();function m(v,w){const S=performance.now(),x=S-g;if(d===null||h===null||x===0)return d=v,h=w,g=S,null;const R=v-d,k=w-h,M=Math.sqrt(R*R+k*k)/x;return d=v,h=w,g=S,M}const y=v=>{let{x:w,y:S,placement:x,elements:R,onClose:k,nodeId:O,tree:M}=v;return function($){function V(){clearTimeout(s),k()}if(clearTimeout(s),!R.domReference||!R.floating||x==null||w==null||S==null)return;const{clientX:W,clientY:ee}=$,fe=[W,ee],ne=zr($),se=$.type==="mouseleave",me=sr(R.floating,ne),te=sr(R.domReference,ne),ue=R.domReference.getBoundingClientRect(),X=R.floating.getBoundingClientRect(),I=x.split("-")[0],N=w>X.right-X.width/2,q=S>X.bottom-X.height/2,L=OR(fe,ue),K=X.width>ue.width,E=X.height>ue.height,_=(K?ue:X).left,P=(K?ue:X).right,F=(E?ue:X).top,oe=(E?ue:X).bottom;if(me&&(f=!0,!se))return;if(te&&(f=!1),te&&!se){f=!0;return}if(se&&Ge($.relatedTarget)&&sr(R.floating,$.relatedTarget)||M&&Ni(M.nodesRef.current,O).some(ke=>{let{context:de}=ke;return de==null?void 0:de.open}))return;if(I==="top"&&S>=ue.bottom-1||I==="bottom"&&S<=ue.top+1||I==="left"&&w>=ue.right-1||I==="right"&&w<=ue.left+1)return V();let be=[];switch(I){case"top":be=[[_,ue.top+1],[_,X.bottom-1],[P,X.bottom-1],[P,ue.top+1]];break;case"bottom":be=[[_,X.top+1],[_,ue.bottom-1],[P,ue.bottom-1],[P,X.top+1]];break;case"left":be=[[X.right-1,oe],[X.right-1,F],[ue.left+1,F],[ue.left+1,oe]];break;case"right":be=[[ue.right-1,oe],[ue.right-1,F],[X.left+1,F],[X.left+1,oe]];break}function ae(ke){let[de,Ce]=ke;switch(I){case"top":{const At=[K?de+a/2:N?de+a*4:de-a*4,Ce+a+1],pt=[K?de-a/2:N?de+a*4:de-a*4,Ce+a+1],jt=[[X.left,N||K?X.bottom-a:X.top],[X.right,N?K?X.bottom-a:X.top:X.bottom-a]];return[At,pt,...jt]}case"bottom":{const At=[K?de+a/2:N?de+a*4:de-a*4,Ce-a],pt=[K?de-a/2:N?de+a*4:de-a*4,Ce-a],jt=[[X.left,N||K?X.top+a:X.bottom],[X.right,N?K?X.top+a:X.bottom:X.top+a]];return[At,pt,...jt]}case"left":{const At=[de+a+1,E?Ce+a/2:q?Ce+a*4:Ce-a*4],pt=[de+a+1,E?Ce-a/2:q?Ce+a*4:Ce-a*4];return[...[[q||E?X.right-a:X.left,X.top],[q?E?X.right-a:X.left:X.right-a,X.bottom]],At,pt]}case"right":{const At=[de-a,E?Ce+a/2:q?Ce+a*4:Ce-a*4],pt=[de-a,E?Ce-a/2:q?Ce+a*4:Ce-a*4],jt=[[q||E?X.left+a:X.right,X.top],[q?E?X.left+a:X.right:X.left+a,X.bottom]];return[At,pt,...jt]}}}if(!kg([W,ee],be)){if(f&&!L)return V();if(!se&&o){const ke=m($.clientX,$.clientY);if(ke!==null&&ke<.1)return V()}kg([W,ee],ae([w,S]))?!f&&o&&(s=window.setTimeout(V,40)):V()}}};return y.__options={blockPointerEvents:i},y}const MR=({arrowRef:r,placement:a})=>{const i=[];return i.push(rR(8)),i.push(a==="auto"?oR():aR()),i.push(lR({padding:8})),r!=null&&r.current&&i.push(iR({element:r.current})),i},NR=({placement:r})=>r==="auto"?void 0:r,zR=({placement:r})=>({top:"bottom",right:"left",bottom:"top",left:"right"})[r.split("-")[0]],_R=({open:r,arrowRef:a,placement:i="top",setOpen:o})=>ER({placement:NR({placement:i}),open:r,onOpenChange:o,whileElementsMounted:Zp,middleware:MR({placement:i,arrowRef:a})}),DR=({context:r,trigger:a,role:i="tooltip",interactions:o=[]})=>TR([bR(r,{enabled:a==="click"}),pR(r,{enabled:a==="hover",handleClose:CR()}),wR(r),kR(r,{role:i}),...o]);function Pp({animation:r="duration-300",arrow:a=!0,children:i,className:o,content:s,placement:f="top",style:d="dark",theme:h,trigger:g="hover",minWidth:m,...y}){const v=T.useRef(null),[w,S]=T.useState(!1),x=_R({open:w,placement:f,arrowRef:v,setOpen:S}),{context:R,middlewareData:{arrow:{x:k,y:O}={}},refs:M,strategy:H,update:$,x:V,y:W}=x,ee=RR(R),{getFloatingProps:fe,getReferenceProps:ne}=DR({context:R,role:"tooltip",trigger:g,interactions:[ee]});return T.useEffect(()=>{if(M.reference.current&&M.floating.current&&w)return Zp(M.reference.current,M.floating.current,$)},[w,M.floating,M.reference,$]),G.jsxs(G.Fragment,{children:[G.jsx("div",{ref:M.setReference,className:h.target,"data-testid":"flowbite-tooltip-target",...ne(),children:i}),G.jsxs("div",{ref:M.setFloating,"data-testid":"flowbite-tooltip",...fe({className:Be(h.base,r&&`${h.animation} ${r}`,!w&&h.hidden,h.style[d],o),style:{position:H,top:W??" ",left:V??" ",minWidth:m},...y}),children:[G.jsx("div",{className:h.content,children:s}),a&&G.jsx("div",{className:Be(h.arrow.base,d==="dark"&&h.arrow.style.dark,d==="light"&&h.arrow.style.light,d==="auto"&&h.arrow.style.auto),"data-testid":"flowbite-tooltip-arrow",ref:v,style:{top:O??" ",left:k??" ",right:" ",bottom:" ",[zR({placement:x.placement})]:h.arrow.placement},children:" "})]})]})}Pp.displayName="Floating";const UR={target:"w-fit",animation:"transition-opacity",arrow:{base:"absolute z-10 h-2 w-2 rotate-45",style:{dark:"bg-gray-900 dark:bg-gray-700",light:"bg-white",auto:"bg-white dark:bg-gray-700"},placement:"-4px"},base:"absolute z-10 inline-block rounded-lg px-3 py-2 text-sm font-medium shadow-sm",hidden:"invisible opacity-0",style:{dark:"bg-gray-900 text-white dark:bg-gray-700",light:"border border-gray-200 bg-white text-gray-900",auto:"border border-gray-200 bg-white text-gray-900 dark:border-none dark:bg-gray-700 dark:text-white"},content:"relative z-20"};function Fp(r){var w,S;const a=it(),i=ot([UR,(w=a.theme)==null?void 0:w.tooltip,r.theme],[pe(a.clearTheme,"tooltip"),r.clearTheme],[pe(a.applyTheme,"tooltip"),r.applyTheme]),{animation:o="duration-300",arrow:s=!0,children:f,className:d,content:h,placement:g="top",style:m="dark",trigger:y="hover",...v}=at(r,(S=a.props)==null?void 0:S.tooltip);return G.jsx(Pp,{animation:o,arrow:s,content:h,placement:g,style:m,theme:i,trigger:y,className:d,...v,children:f})}Fp.displayName="Tooltip";function jR({key:r,onChange:a}){function i({key:o,newValue:s}){o===r&&a(s)}T.useEffect(()=>(window.addEventListener("storage",i),()=>window.removeEventListener("storage",i)),[])}const Nl="auto",Df="flowbite-theme-mode",ef="flowbite-theme-mode-sync";function LR(){const[r,a]=T.useState(GR(dw()));jR({key:Df,onChange(f){a(Wp(f??Nl))}}),BR(f=>a(f));function i(f){a(f),HR(f),qR(f),document.dispatchEvent(new CustomEvent(ef,{detail:f}))}function o(){let f=r;f==="auto"&&(f=tf(f)),f=f==="dark"?"light":"dark",i(f)}function s(){i(r??Nl)}return{mode:r,computedMode:tf(r),setMode:i,toggleMode:o,clearMode:s}}function BR(r){T.useEffect(()=>{function a(i){const o=i.detail;r(o)}return document.addEventListener(ef,a),()=>document.removeEventListener(ef,a)},[])}function HR(r){localStorage.setItem(Df,r)}function qR(r){const a="";tf(r)==="dark"?document.documentElement.classList.add(`${a}dark`):document.documentElement.classList.remove(`${a}dark`)}function GR(r){if(!Mp())return Nl;const a=localStorage.getItem(Df);return Wp(a??r??Nl)}function tf(r){return Mp()?r==="auto"?YR():r:Nl}function YR(){var r;return(r=window.matchMedia)!=null&&r.call(window,"(prefers-color-scheme: dark)").matches?"dark":"light"}function Wp(r){return["light","dark","auto"].includes(r)?r:Nl}const Ip=T.forwardRef((r,a)=>G.jsx("svg",{xmlns:"http://www.w3.org/2000/svg",width:"1em",height:"1em",fill:"currentColor",stroke:"currentColor",strokeWidth:0,viewBox:"0 0 20 20",ref:a,...r,children:G.jsx("path",{stroke:"none",d:"M17.293 13.293A8 8 0 0 1 6.707 2.707a8.001 8.001 0 1 0 10.586 10.586z"})}));Ip.displayName="MoonIcon";const ey=T.forwardRef((r,a)=>G.jsx("svg",{xmlns:"http://www.w3.org/2000/svg",width:"1em",height:"1em",fill:"currentColor",stroke:"currentColor",strokeWidth:0,viewBox:"0 0 20 20",ref:a,...r,children:G.jsx("path",{fillRule:"evenodd",stroke:"none",d:"M10 2a1 1 0 0 1 1 1v1a1 1 0 1 1-2 0V3a1 1 0 0 1 1-1zm4 8a4 4 0 1 1-8 0 4 4 0 0 1 8 0zm-.464 4.95.707.707a1 1 0 0 0 1.414-1.414l-.707-.707a1 1 0 0 0-1.414 1.414zm2.12-10.607a1 1 0 0 1 0 1.414l-.706.707a1 1 0 1 1-1.414-1.414l.707-.707a1 1 0 0 1 1.414 0zM17 11a1 1 0 1 0 0-2h-1a1 1 0 1 0 0 2h1zm-7 4a1 1 0 0 1 1 1v1a1 1 0 1 1-2 0v-1a1 1 0 0 1 1-1zM5.05 6.464A1 1 0 1 0 6.465 5.05l-.708-.707a1 1 0 0 0-1.414 1.414l.707.707zm1.414 8.486-.707.707a1 1 0 0 1-1.414-1.414l.707-.707a1 1 0 0 1 1.414 1.414zM4 11a1 1 0 1 0 0-2H3a1 1 0 0 0 0 2h1z",clipRule:"evenodd"})}));ey.displayName="SunIcon";const VR={root:{base:"rounded-lg p-2.5 text-sm text-gray-500 hover:bg-gray-100 focus:outline-none focus:ring-4 focus:ring-gray-200 dark:text-gray-400 dark:hover:bg-gray-700 dark:focus:ring-gray-700",icon:{base:"h-5 w-5",dark:"hidden dark:block",light:"dark:hidden"}}},ty=T.forwardRef((r,a)=>{var m,y;const i=it(),o=ot([VR,(m=i.theme)==null?void 0:m.darkThemeToggle,r.theme],[pe(i.clearTheme,"darkThemeToggle"),r.clearTheme],[pe(i.applyTheme,"darkThemeToggle"),r.applyTheme]),{className:s,iconDark:f=ey,iconLight:d=Ip,...h}=at(r,(y=i.props)==null?void 0:y.darkThemeToggle),{toggleMode:g}=LR();return G.jsxs("button",{ref:a,type:"button","aria-label":"Toggle dark mode","data-testid":"dark-theme-toggle",className:Be(o.root.base,s),onClick:g,...h,children:[G.jsx(f,{"aria-label":"Currently dark mode",className:Be(o.root.icon.base,o.root.icon.dark)}),G.jsx(d,{"aria-label":"Currently light mode",className:Be(o.root.icon.base,o.root.icon.light)})]})});ty.displayName="DarkThemeToggle";const XR={base:"flex",addon:"inline-flex items-center rounded-l-md border border-r-0 border-gray-300 bg-gray-200 px-3 text-sm text-gray-900 dark:border-gray-600 dark:bg-gray-600 dark:text-gray-400",field:{base:"relative w-full",icon:{base:"pointer-events-none absolute inset-y-0 left-0 flex items-center pl-3",svg:"h-5 w-5 text-gray-500 dark:text-gray-400"},rightIcon:{base:"pointer-events-none absolute inset-y-0 right-0 flex items-center pr-3",svg:"h-5 w-5 text-gray-500 dark:text-gray-400"},input:{base:"block w-full border focus:outline-none focus:ring-1 disabled:cursor-not-allowed disabled:opacity-50",sizes:{sm:"p-2 sm:text-xs",md:"p-2.5 text-sm",lg:"p-4 sm:text-base"},colors:{gray:"border-gray-300 bg-gray-50 text-gray-900 placeholder-gray-500 focus:border-primary-500 focus:ring-primary-500 dark:border-gray-600 dark:bg-gray-700 dark:text-white dark:placeholder-gray-400 dark:focus:border-primary-500 dark:focus:ring-primary-500",info:"border-cyan-500 bg-cyan-50 text-cyan-900 placeholder-cyan-700 focus:border-cyan-500 focus:ring-cyan-500 dark:border-cyan-400 dark:bg-cyan-100 dark:focus:border-cyan-500 dark:focus:ring-cyan-500",failure:"border-red-500 bg-red-50 text-red-900 placeholder-red-700 focus:border-red-500 focus:ring-red-500 dark:border-red-400 dark:bg-red-100 dark:focus:border-red-500 dark:focus:ring-red-500",warning:"border-yellow-500 bg-yellow-50 text-yellow-900 placeholder-yellow-700 focus:border-yellow-500 focus:ring-yellow-500 dark:border-yellow-400 dark:bg-yellow-100 dark:focus:border-yellow-500 dark:focus:ring-yellow-500",success:"border-green-500 bg-green-50 text-green-900 placeholder-green-700 focus:border-green-500 focus:ring-green-500 dark:border-green-400 dark:bg-green-100 dark:focus:border-green-500 dark:focus:ring-green-500"},withRightIcon:{on:"pr-10",off:""},withIcon:{on:"pl-10",off:""},withAddon:{on:"rounded-r-lg",off:"rounded-lg"},withShadow:{on:"shadow-sm dark:shadow-sm-light",off:""}}}},nf=T.forwardRef((r,a)=>{var S,x;const i=it(),o=ot([XR,(S=i.theme)==null?void 0:S.textInput,r.theme],[pe(i.clearTheme,"textInput"),r.clearTheme],[pe(i.applyTheme,"textInput"),r.applyTheme]),{addon:s,className:f,color:d="gray",icon:h,rightIcon:g,shadow:m,sizing:y="md",type:v="text",...w}=at(r,(x=i.props)==null?void 0:x.textInput);return G.jsxs("div",{className:Be(o.base,f),children:[s&&G.jsx("span",{className:o.addon,children:s}),G.jsxs("div",{className:o.field.base,children:[h&&G.jsx("div",{className:o.field.icon.base,children:G.jsx(h,{className:o.field.icon.svg})}),g&&G.jsx("div",{"data-testid":"right-icon",className:o.field.rightIcon.base,children:G.jsx(g,{className:o.field.rightIcon.svg})}),G.jsx("input",{className:Be(o.field.input.base,o.field.input.colors[d],o.field.input.sizes[y],o.field.input.withIcon[h?"on":"off"],o.field.input.withRightIcon[g?"on":"off"],o.field.input.withAddon[s?"on":"off"],o.field.input.withShadow[m?"on":"off"]),type:v,...w,ref:a})]})]})});nf.displayName="TextInput";const QR={root:{base:"text-sm font-medium",disabled:"opacity-50",colors:{default:"text-gray-900 dark:text-white",info:"text-cyan-500 dark:text-cyan-600",failure:"text-red-700 dark:text-red-500",warning:"text-yellow-500 dark:text-yellow-600",success:"text-green-700 dark:text-green-500"}}},zi=T.forwardRef((r,a)=>{var g,m;const i=it(),o=ot([QR,(g=i.theme)==null?void 0:g.label,r.theme],[pe(i.clearTheme,"label"),r.clearTheme],[pe(i.applyTheme,"label"),r.applyTheme]),{className:s,color:f="default",disabled:d=!1,...h}=at(r,(m=i.props)==null?void 0:m.label);return G.jsx("label",{ref:a,className:Be(o.root.base,o.root.colors[f],d&&o.root.disabled,s),"data-testid":"flowbite-label",...h})});zi.displayName="Label";const ny=T.createContext(void 0);function eu(){const r=T.useContext(ny);if(!r)throw new Error("useNavBarContext should be used within the NavbarContext provider!");return r}const ro={root:{base:"bg-white px-2 py-2.5 sm:px-4 dark:border-gray-700 dark:bg-gray-800",rounded:{on:"rounded",off:""},bordered:{on:"border",off:""},inner:{base:"mx-auto flex flex-wrap items-center justify-between",fluid:{on:"",off:"container"}}},brand:{base:"flex items-center"},collapse:{base:"w-full md:block md:w-auto",list:"mt-4 flex flex-col md:mt-0 md:flex-row md:space-x-8 md:text-sm md:font-medium",hidden:{on:"hidden",off:""}},link:{base:"block py-2 pl-3 pr-4 md:p-0",active:{on:"bg-primary-700 text-white md:bg-transparent md:text-primary-700 dark:text-white",off:"border-b border-gray-100 text-gray-700 hover:bg-gray-50 md:border-0 md:hover:bg-transparent md:hover:text-primary-700 dark:border-gray-700 dark:text-gray-400 dark:hover:bg-gray-700 dark:hover:text-white md:dark:hover:bg-transparent md:dark:hover:text-white"},disabled:{on:"text-gray-400 hover:cursor-not-allowed dark:text-gray-600",off:""}},toggle:{base:"inline-flex items-center rounded-lg p-2 text-sm text-gray-500 hover:bg-gray-100 focus:outline-none focus:ring-2 focus:ring-gray-200 md:hidden dark:text-gray-400 dark:hover:bg-gray-700 dark:focus:ring-gray-600",icon:"h-6 w-6 shrink-0",title:"sr-only"}},ry=T.forwardRef((r,a)=>{var S,x;const i=it(),o=ot([ro,(S=i.theme)==null?void 0:S.navbar,r.theme],[pe(i.clearTheme,"navbar"),r.clearTheme],[pe(i.applyTheme,"navbar"),r.applyTheme]),{border:s,children:f,className:d,fluid:h=!1,menuOpen:g,rounded:m,...y}=at(r,(x=i.props)==null?void 0:x.navbar),[v,w]=T.useState(g);return G.jsx(ny.Provider,{value:{theme:r.theme,clearTheme:r.clearTheme,applyTheme:r.applyTheme,isOpen:v,setIsOpen:w},children:G.jsx("nav",{ref:a,className:Be(o.root.base,o.root.bordered[s?"on":"off"],o.root.rounded[m?"on":"off"],d),...y,children:G.jsx("div",{className:Be(o.root.inner.base,o.root.inner.fluid[h?"on":"off"]),children:f})})})});ry.displayName="Navbar";const ly=T.forwardRef((r,a)=>{var y,v,w;const{theme:i,clearTheme:o,applyTheme:s}=eu(),f=it(),d=ot([ro.brand,(v=(y=f.theme)==null?void 0:y.navbar)==null?void 0:v.brand,i==null?void 0:i.brand,r.theme],[pe(f.clearTheme,"navbar.brand"),pe(o,"brand"),r.clearTheme],[pe(f.applyTheme,"navbar.brand"),pe(s,"brand"),r.applyTheme]),{as:h="a",className:g,...m}=at(r,(w=f.props)==null?void 0:w.navbarBrand);return G.jsx(h,{ref:a,className:Be(d.base,g),...m})});ly.displayName="NavbarBrand";const ay=T.forwardRef((r,a)=>{var v,w,S;const{theme:i,clearTheme:o,applyTheme:s,isOpen:f}=eu(),d=it(),h=ot([ro.collapse,(w=(v=d.theme)==null?void 0:v.navbar)==null?void 0:w.collapse,i==null?void 0:i.collapse,r.theme],[pe(d.clearTheme,"navbar.collapse"),pe(o,"collapse"),r.clearTheme],[pe(d.applyTheme,"navbar.collapse"),pe(s,"collapse"),r.applyTheme]),{children:g,className:m,...y}=at(r,(S=d.props)==null?void 0:S.navbarCollapse);return G.jsx("div",{ref:a,"data-testid":"flowbite-navbar-collapse",className:Be(h.base,h.hidden[f?"off":"on"],m),...y,children:G.jsx("ul",{className:h.list,children:g})})});ay.displayName="NavbarCollapse";const _i=T.forwardRef((r,a)=>{var k,O,M;const{theme:i,clearTheme:o,applyTheme:s,setIsOpen:f}=eu(),d=it(),h=ot([ro.link,(O=(k=d.theme)==null?void 0:k.navbar)==null?void 0:O.link,i==null?void 0:i.link,r.theme],[pe(d.clearTheme,"navbar.link"),pe(o,"link"),r.clearTheme],[pe(d.applyTheme,"navbar.link"),pe(s,"link"),r.applyTheme]),{active:g,as:m="a",disabled:y,children:v,className:w,onClick:S,...x}=at(r,(M=d.props)==null?void 0:M.navbarLink);function R(H){f(!1),S==null||S(H)}return G.jsx("li",{ref:a,children:G.jsx(m,{className:Be(h.base,g&&h.active.on,!g&&!y&&h.active.off,h.disabled[y?"on":"off"],w),onClick:R,...x,children:v})})});_i.displayName="NavbarLink";const oy=T.forwardRef((r,a)=>G.jsx("svg",{xmlns:"http://www.w3.org/2000/svg",width:"1em",height:"1em",fill:"currentColor",stroke:"currentColor",strokeWidth:0,viewBox:"0 0 448 512",ref:a,...r,children:G.jsx("path",{stroke:"none",d:"M16 132h416c8.837 0 16-7.163 16-16V76c0-8.837-7.163-16-16-16H16C7.163 60 0 67.163 0 76v40c0 8.837 7.163 16 16 16zm0 160h416c8.837 0 16-7.163 16-16v-40c0-8.837-7.163-16-16-16H16c-8.837 0-16 7.163-16 16v40c0 8.837 7.163 16 16 16zm0 160h416c8.837 0 16-7.163 16-16v-40c0-8.837-7.163-16-16-16H16c-8.837 0-16 7.163-16 16v40c0 8.837 7.163 16 16 16z"})}));oy.displayName="BarsIcon";const iy=T.forwardRef((r,a)=>{var S,x,R;const{theme:i,clearTheme:o,applyTheme:s,isOpen:f,setIsOpen:d}=eu(),h=it(),g=ot([ro.toggle,(x=(S=h.theme)==null?void 0:S.navbar)==null?void 0:x.toggle,i==null?void 0:i.toggle,r.theme],[pe(h.clearTheme,"navbar.toggle"),pe(o,"toggle"),r.clearTheme],[pe(h.applyTheme,"navbar.toggle"),pe(s,"toggle"),r.applyTheme]),{barIcon:m=oy,className:y,...v}=at(r,(R=h.props)==null?void 0:R.navbarToggle);function w(){d(!f)}return G.jsxs("button",{ref:a,"data-testid":"flowbite-navbar-toggle",onClick:w,className:Be(g.base,y),...v,children:[G.jsx("span",{className:g.title,children:"Open main menu"}),G.jsx(m,{"aria-hidden":!0,className:g.icon})]})});iy.displayName="NavbarToggle";const uy=T.createContext(void 0);function Uf(){const r=T.useContext(uy);if(!r)throw new Error("useSidebarContext should be used within the SidebarContext provider!");return r}const tu={root:{base:"h-full",collapsed:{on:"w-16",off:"w-64"},inner:"h-full overflow-y-auto overflow-x-hidden rounded bg-gray-50 px-3 py-4 dark:bg-gray-800"},collapse:{button:"group flex w-full items-center rounded-lg p-2 text-base font-normal text-gray-900 transition duration-75 hover:bg-gray-100 dark:text-white dark:hover:bg-gray-700",icon:{base:"h-6 w-6 text-gray-500 transition duration-75 group-hover:text-gray-900 dark:text-gray-400 dark:group-hover:text-white",open:{off:"",on:"text-gray-900"}},label:{base:"ml-3 flex-1 whitespace-nowrap text-left",title:"sr-only",icon:{base:"h-6 w-6 transition delay-0 ease-in-out",open:{on:"rotate-180",off:""}}},list:"space-y-2 py-2"},cta:{base:"mt-6 rounded-lg bg-gray-100 p-4 dark:bg-gray-700",color:{blue:"bg-cyan-50 dark:bg-cyan-900",dark:"bg-dark-50 dark:bg-dark-900",failure:"bg-red-50 dark:bg-red-900",gray:"bg-gray-50 dark:bg-gray-900",green:"bg-green-50 dark:bg-green-900",light:"bg-light-50 dark:bg-light-900",red:"bg-red-50 dark:bg-red-900",purple:"bg-purple-50 dark:bg-purple-900",success:"bg-green-50 dark:bg-green-900",yellow:"bg-yellow-50 dark:bg-yellow-900",warning:"bg-yellow-50 dark:bg-yellow-900"}},item:{base:"flex items-center justify-center rounded-lg p-2 text-base font-normal text-gray-900 hover:bg-gray-100 dark:text-white dark:hover:bg-gray-700",active:"bg-gray-100 dark:bg-gray-700",collapsed:{insideCollapse:"group w-full pl-8 transition duration-75",noIcon:"font-bold"},content:{base:"flex-1 whitespace-nowrap px-3"},icon:{base:"h-6 w-6 shrink-0 text-gray-500 transition duration-75 group-hover:text-gray-900 dark:text-gray-400 dark:group-hover:text-white",active:"text-gray-700 dark:text-gray-100"},label:"",listItem:""},items:{base:""},itemGroup:{base:"mt-4 space-y-2 border-t border-gray-200 pt-4 first:mt-0 first:border-t-0 first:pt-0 dark:border-gray-700"},logo:{base:"mb-5 flex items-center pl-2.5",collapsed:{on:"hidden",off:"self-center whitespace-nowrap text-xl font-semibold dark:text-white"},img:"mr-3 h-6 sm:h-7"}},sy=T.forwardRef((r,a)=>{var y,v;const i=it(),o=ot([tu,(y=i.theme)==null?void 0:y.sidebar,r.theme],[pe(i.clearTheme,"sidebar"),r.clearTheme],[pe(i.applyTheme,"sidebar"),r.applyTheme]),{as:s="nav",children:f,className:d,collapseBehavior:h="collapse",collapsed:g=!1,...m}=at(r,(v=i.props)==null?void 0:v.sidebar);return G.jsx(uy.Provider,{value:{theme:r.theme,clearTheme:r.clearTheme,applyTheme:r.applyTheme,isCollapsed:g},children:G.jsx(s,{ref:a,"aria-label":"Sidebar",hidden:g&&h==="hide",className:Be(o.root.base,o.root.collapsed[g?"on":"off"],d),...m,children:G.jsx("div",{className:o.root.inner,children:f})})})});sy.displayName="Sidebar";const cy=T.createContext(void 0);function ZR(){const r=T.useContext(cy);if(!r)throw new Error("useSidebarItemContext should be used within the SidebarItemContext provider!");return r}const Al=T.forwardRef((r,a)=>{var M,H,$;const i=T.useId(),{theme:o,clearTheme:s,applyTheme:f,isCollapsed:d}=Uf(),{isInsideCollapse:h}=ZR(),g=it(),m=ot([tu.item,(H=(M=g.theme)==null?void 0:M.sidebar)==null?void 0:H.item,o==null?void 0:o.item,r.theme],[pe(g.clearTheme,"sidebar.item"),pe(s,"item"),r.clearTheme],[pe(g.applyTheme,"sidebar.item"),pe(f,"item"),r.applyTheme]),{active:y,as:v="a",children:w,className:S,icon:x,label:R,labelColor:k="info",...O}=at(r,($=g.props)==null?void 0:$.sidebarItem);return G.jsx(jf,{theme:m,className:m.listItem,id:i,isCollapsed:d,tooltipChildren:w,children:G.jsxs(v,{"aria-labelledby":`flowbite-sidebar-item-${i}`,ref:a,className:Be(m.base,y&&m.active,!d&&h&&m.collapsed.insideCollapse,S),...O,children:[x&&G.jsx(x,{"aria-hidden":!0,"data-testid":"flowbite-sidebar-item-icon",className:Be(m.icon.base,y&&m.icon.active)}),d&&!x&&G.jsx("span",{className:m.collapsed.noIcon,children:w.charAt(0).toLocaleUpperCase()??"?"}),!d&&G.jsx(fy,{id:i,theme:m,children:w}),!d&&R&&G.jsx(kp,{color:k,"data-testid":"flowbite-sidebar-label",hidden:d,className:m.label,children:R})]})})});Al.displayName="SidebarItem";function jf({id:r,theme:a,isCollapsed:i,tooltipChildren:o,children:s,...f}){return G.jsx("li",{...f,children:i?G.jsx(Fp,{content:G.jsx(fy,{id:r,theme:a,children:o}),placement:"right",children:s}):s})}jf.displayName="SidebarItem.ListItem";function fy({id:r,theme:a,children:i}){return G.jsx("span",{"data-testid":"flowbite-sidebar-item-content",id:`flowbite-sidebar-item-${r}`,className:Be(a.content.base),children:i})}jf.displayName="SidebarItem.Children";const dy=T.forwardRef((r,a)=>{var m,y,v;const{theme:i,clearTheme:o,applyTheme:s}=Uf(),f=it(),d=ot([tu.itemGroup,(y=(m=f.theme)==null?void 0:m.sidebar)==null?void 0:y.itemGroup,i==null?void 0:i.itemGroup,r.theme],[pe(f.clearTheme,"sidebar.itemGroup"),pe(o,"itemGroup"),r.clearTheme],[pe(f.applyTheme,"sidebar.itemGroup"),pe(s,"itemGroup"),r.applyTheme]),{className:h,...g}=at(r,(v=f.props)==null?void 0:v.sidebarItemGroup);return G.jsx(cy.Provider,{value:{isInsideCollapse:!1},children:G.jsx("ul",{ref:a,"data-testid":"flowbite-sidebar-item-group",className:Be(d.base,h),...g})})});dy.displayName="SidebarItemGroup";const hy=T.forwardRef((r,a)=>{var m,y,v;const{theme:i,clearTheme:o,applyTheme:s}=Uf(),f=it(),d=ot([tu.items,(y=(m=f.theme)==null?void 0:m.sidebar)==null?void 0:y.items,i==null?void 0:i.items,r.theme],[pe(f.clearTheme,"sidebar.items"),pe(o,"items"),r.clearTheme],[pe(f.applyTheme,"sidebar.items"),pe(s,"items"),r.applyTheme]),{className:h,...g}=at(r,(v=f.props)==null?void 0:v.sidebarItems);return G.jsx("div",{ref:a,className:Be(d.base,h),"data-testid":"flowbite-sidebar-items",...g})});hy.displayName="SidebarItems";const Hc={defaultMode:"auto",localStorageKey:"flowbite-theme-mode",prefix:""};function KR(r={}){const{mode:a,defaultMode:i=Hc.defaultMode,localStorageKey:o=Hc.localStorageKey,prefix:s=Hc.prefix}=r;try{const f=window.localStorage.getItem(o),h=(f==="light"||f==="dark"||f==="auto"?f:null)??a??i;(h==="auto"?window.matchMedia("(prefers-color-scheme: dark)").matches?"dark":"light":h)==="dark"?document.documentElement.classList.add(`${s}dark`):document.documentElement.classList.remove(`${s}dark`),localStorage.setItem(o,h),window.matchMedia("(prefers-color-scheme: dark)").addEventListener("change",y=>{const v=window.localStorage.getItem(o);(v==="light"||v==="dark"||v==="auto"?v:i)==="auto"&&(y.matches?document.documentElement.classList.add(`${s}dark`):document.documentElement.classList.remove(`${s}dark`))}),window.addEventListener("storage",y=>{if(y.key===o){const v=y.newValue,S=v==="light"||v==="dark"||v==="auto"?v:i;S==="dark"||S==="auto"&&window.matchMedia("(prefers-color-scheme: dark)").matches?document.documentElement.classList.add(`${s}dark`):document.documentElement.classList.remove(`${s}dark`)}})}catch{}}function my(r,a){return function(){return r.apply(a,arguments)}}const{toString:$R}=Object.prototype,{getPrototypeOf:Lf}=Object,{iterator:nu,toStringTag:gy}=Symbol,ru=(r=>a=>{const i=$R.call(a);return r[i]||(r[i]=i.slice(8,-1).toLowerCase())})(Object.create(null)),It=r=>(r=r.toLowerCase(),a=>ru(a)===r),lu=r=>a=>typeof a===r,{isArray:ql}=Array,Ia=lu("undefined");function JR(r){return r!==null&&!Ia(r)&&r.constructor!==null&&!Ia(r.constructor)&&Et(r.constructor.isBuffer)&&r.constructor.isBuffer(r)}const py=It("ArrayBuffer");function PR(r){let a;return typeof ArrayBuffer<"u"&&ArrayBuffer.isView?a=ArrayBuffer.isView(r):a=r&&r.buffer&&py(r.buffer),a}const FR=lu("string"),Et=lu("function"),yy=lu("number"),au=r=>r!==null&&typeof r=="object",WR=r=>r===!0||r===!1,Di=r=>{if(ru(r)!=="object")return!1;const a=Lf(r);return(a===null||a===Object.prototype||Object.getPrototypeOf(a)===null)&&!(gy in r)&&!(nu in r)},IR=It("Date"),e2=It("File"),t2=It("Blob"),n2=It("FileList"),r2=r=>au(r)&&Et(r.pipe),l2=r=>{let a;return r&&(typeof FormData=="function"&&r instanceof FormData||Et(r.append)&&((a=ru(r))==="formdata"||a==="object"&&Et(r.toString)&&r.toString()==="[object FormData]"))},a2=It("URLSearchParams"),[o2,i2,u2,s2]=["ReadableStream","Request","Response","Headers"].map(It),c2=r=>r.trim?r.trim():r.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g,"");function lo(r,a,{allOwnKeys:i=!1}={}){if(r===null||typeof r>"u")return;let o,s;if(typeof r!="object"&&(r=[r]),ql(r))for(o=0,s=r.length;o<s;o++)a.call(null,r[o],o,r);else{const f=i?Object.getOwnPropertyNames(r):Object.keys(r),d=f.length;let h;for(o=0;o<d;o++)h=f[o],a.call(null,r[h],h,r)}}function by(r,a){a=a.toLowerCase();const i=Object.keys(r);let o=i.length,s;for(;o-- >0;)if(s=i[o],a===s.toLowerCase())return s;return null}const Dr=typeof globalThis<"u"?globalThis:typeof self<"u"?self:typeof window<"u"?window:global,vy=r=>!Ia(r)&&r!==Dr;function rf(){const{caseless:r}=vy(this)&&this||{},a={},i=(o,s)=>{const f=r&&by(a,s)||s;Di(a[f])&&Di(o)?a[f]=rf(a[f],o):Di(o)?a[f]=rf({},o):ql(o)?a[f]=o.slice():a[f]=o};for(let o=0,s=arguments.length;o<s;o++)arguments[o]&&lo(arguments[o],i);return a}const f2=(r,a,i,{allOwnKeys:o}={})=>(lo(a,(s,f)=>{i&&Et(s)?r[f]=my(s,i):r[f]=s},{allOwnKeys:o}),r),d2=r=>(r.charCodeAt(0)===65279&&(r=r.slice(1)),r),h2=(r,a,i,o)=>{r.prototype=Object.create(a.prototype,o),r.prototype.constructor=r,Object.defineProperty(r,"super",{value:a.prototype}),i&&Object.assign(r.prototype,i)},m2=(r,a,i,o)=>{let s,f,d;const h={};if(a=a||{},r==null)return a;do{for(s=Object.getOwnPropertyNames(r),f=s.length;f-- >0;)d=s[f],(!o||o(d,r,a))&&!h[d]&&(a[d]=r[d],h[d]=!0);r=i!==!1&&Lf(r)}while(r&&(!i||i(r,a))&&r!==Object.prototype);return a},g2=(r,a,i)=>{r=String(r),(i===void 0||i>r.length)&&(i=r.length),i-=a.length;const o=r.indexOf(a,i);return o!==-1&&o===i},p2=r=>{if(!r)return null;if(ql(r))return r;let a=r.length;if(!yy(a))return null;const i=new Array(a);for(;a-- >0;)i[a]=r[a];return i},y2=(r=>a=>r&&a instanceof r)(typeof Uint8Array<"u"&&Lf(Uint8Array)),b2=(r,a)=>{const o=(r&&r[nu]).call(r);let s;for(;(s=o.next())&&!s.done;){const f=s.value;a.call(r,f[0],f[1])}},v2=(r,a)=>{let i;const o=[];for(;(i=r.exec(a))!==null;)o.push(i);return o},x2=It("HTMLFormElement"),w2=r=>r.toLowerCase().replace(/[-_\s]([a-z\d])(\w*)/g,function(i,o,s){return o.toUpperCase()+s}),Og=(({hasOwnProperty:r})=>(a,i)=>r.call(a,i))(Object.prototype),S2=It("RegExp"),xy=(r,a)=>{const i=Object.getOwnPropertyDescriptors(r),o={};lo(i,(s,f)=>{let d;(d=a(s,f,r))!==!1&&(o[f]=d||s)}),Object.defineProperties(r,o)},E2=r=>{xy(r,(a,i)=>{if(Et(r)&&["arguments","caller","callee"].indexOf(i)!==-1)return!1;const o=r[i];if(Et(o)){if(a.enumerable=!1,"writable"in a){a.writable=!1;return}a.set||(a.set=()=>{throw Error("Can not rewrite read-only method '"+i+"'")})}})},R2=(r,a)=>{const i={},o=s=>{s.forEach(f=>{i[f]=!0})};return ql(r)?o(r):o(String(r).split(a)),i},T2=()=>{},A2=(r,a)=>r!=null&&Number.isFinite(r=+r)?r:a;function k2(r){return!!(r&&Et(r.append)&&r[gy]==="FormData"&&r[nu])}const O2=r=>{const a=new Array(10),i=(o,s)=>{if(au(o)){if(a.indexOf(o)>=0)return;if(!("toJSON"in o)){a[s]=o;const f=ql(o)?[]:{};return lo(o,(d,h)=>{const g=i(d,s+1);!Ia(g)&&(f[h]=g)}),a[s]=void 0,f}}return o};return i(r,0)},C2=It("AsyncFunction"),M2=r=>r&&(au(r)||Et(r))&&Et(r.then)&&Et(r.catch),wy=((r,a)=>r?setImmediate:a?((i,o)=>(Dr.addEventListener("message",({source:s,data:f})=>{s===Dr&&f===i&&o.length&&o.shift()()},!1),s=>{o.push(s),Dr.postMessage(i,"*")}))(`axios@${Math.random()}`,[]):i=>setTimeout(i))(typeof setImmediate=="function",Et(Dr.postMessage)),N2=typeof queueMicrotask<"u"?queueMicrotask.bind(Dr):typeof process<"u"&&process.nextTick||wy,z2=r=>r!=null&&Et(r[nu]),Y={isArray:ql,isArrayBuffer:py,isBuffer:JR,isFormData:l2,isArrayBufferView:PR,isString:FR,isNumber:yy,isBoolean:WR,isObject:au,isPlainObject:Di,isReadableStream:o2,isRequest:i2,isResponse:u2,isHeaders:s2,isUndefined:Ia,isDate:IR,isFile:e2,isBlob:t2,isRegExp:S2,isFunction:Et,isStream:r2,isURLSearchParams:a2,isTypedArray:y2,isFileList:n2,forEach:lo,merge:rf,extend:f2,trim:c2,stripBOM:d2,inherits:h2,toFlatObject:m2,kindOf:ru,kindOfTest:It,endsWith:g2,toArray:p2,forEachEntry:b2,matchAll:v2,isHTMLForm:x2,hasOwnProperty:Og,hasOwnProp:Og,reduceDescriptors:xy,freezeMethods:E2,toObjectSet:R2,toCamelCase:w2,noop:T2,toFiniteNumber:A2,findKey:by,global:Dr,isContextDefined:vy,isSpecCompliantForm:k2,toJSONObject:O2,isAsyncFn:C2,isThenable:M2,setImmediate:wy,asap:N2,isIterable:z2};function ve(r,a,i,o,s){Error.call(this),Error.captureStackTrace?Error.captureStackTrace(this,this.constructor):this.stack=new Error().stack,this.message=r,this.name="AxiosError",a&&(this.code=a),i&&(this.config=i),o&&(this.request=o),s&&(this.response=s,this.status=s.status?s.status:null)}Y.inherits(ve,Error,{toJSON:function(){return{message:this.message,name:this.name,description:this.description,number:this.number,fileName:this.fileName,lineNumber:this.lineNumber,columnNumber:this.columnNumber,stack:this.stack,config:Y.toJSONObject(this.config),code:this.code,status:this.status}}});const Sy=ve.prototype,Ey={};["ERR_BAD_OPTION_VALUE","ERR_BAD_OPTION","ECONNABORTED","ETIMEDOUT","ERR_NETWORK","ERR_FR_TOO_MANY_REDIRECTS","ERR_DEPRECATED","ERR_BAD_RESPONSE","ERR_BAD_REQUEST","ERR_CANCELED","ERR_NOT_SUPPORT","ERR_INVALID_URL"].forEach(r=>{Ey[r]={value:r}});Object.defineProperties(ve,Ey);Object.defineProperty(Sy,"isAxiosError",{value:!0});ve.from=(r,a,i,o,s,f)=>{const d=Object.create(Sy);return Y.toFlatObject(r,d,function(g){return g!==Error.prototype},h=>h!=="isAxiosError"),ve.call(d,r.message,a,i,o,s),d.cause=r,d.name=r.name,f&&Object.assign(d,f),d};const _2=null;function lf(r){return Y.isPlainObject(r)||Y.isArray(r)}function Ry(r){return Y.endsWith(r,"[]")?r.slice(0,-2):r}function Cg(r,a,i){return r?r.concat(a).map(function(s,f){return s=Ry(s),!i&&f?"["+s+"]":s}).join(i?".":""):a}function D2(r){return Y.isArray(r)&&!r.some(lf)}const U2=Y.toFlatObject(Y,{},null,function(a){return/^is[A-Z]/.test(a)});function ou(r,a,i){if(!Y.isObject(r))throw new TypeError("target must be an object");a=a||new FormData,i=Y.toFlatObject(i,{metaTokens:!0,dots:!1,indexes:!1},!1,function(R,k){return!Y.isUndefined(k[R])});const o=i.metaTokens,s=i.visitor||y,f=i.dots,d=i.indexes,g=(i.Blob||typeof Blob<"u"&&Blob)&&Y.isSpecCompliantForm(a);if(!Y.isFunction(s))throw new TypeError("visitor must be a function");function m(x){if(x===null)return"";if(Y.isDate(x))return x.toISOString();if(Y.isBoolean(x))return x.toString();if(!g&&Y.isBlob(x))throw new ve("Blob is not supported. Use a Buffer instead.");return Y.isArrayBuffer(x)||Y.isTypedArray(x)?g&&typeof Blob=="function"?new Blob([x]):Buffer.from(x):x}function y(x,R,k){let O=x;if(x&&!k&&typeof x=="object"){if(Y.endsWith(R,"{}"))R=o?R:R.slice(0,-2),x=JSON.stringify(x);else if(Y.isArray(x)&&D2(x)||(Y.isFileList(x)||Y.endsWith(R,"[]"))&&(O=Y.toArray(x)))return R=Ry(R),O.forEach(function(H,$){!(Y.isUndefined(H)||H===null)&&a.append(d===!0?Cg([R],$,f):d===null?R:R+"[]",m(H))}),!1}return lf(x)?!0:(a.append(Cg(k,R,f),m(x)),!1)}const v=[],w=Object.assign(U2,{defaultVisitor:y,convertValue:m,isVisitable:lf});function S(x,R){if(!Y.isUndefined(x)){if(v.indexOf(x)!==-1)throw Error("Circular reference detected in "+R.join("."));v.push(x),Y.forEach(x,function(O,M){(!(Y.isUndefined(O)||O===null)&&s.call(a,O,Y.isString(M)?M.trim():M,R,w))===!0&&S(O,R?R.concat(M):[M])}),v.pop()}}if(!Y.isObject(r))throw new TypeError("data must be an object");return S(r),a}function Mg(r){const a={"!":"%21","'":"%27","(":"%28",")":"%29","~":"%7E","%20":"+","%00":"\0"};return encodeURIComponent(r).replace(/[!'()~]|%20|%00/g,function(o){return a[o]})}function Bf(r,a){this._pairs=[],r&&ou(r,this,a)}const Ty=Bf.prototype;Ty.append=function(a,i){this._pairs.push([a,i])};Ty.toString=function(a){const i=a?function(o){return a.call(this,o,Mg)}:Mg;return this._pairs.map(function(s){return i(s[0])+"="+i(s[1])},"").join("&")};function j2(r){return encodeURIComponent(r).replace(/%3A/gi,":").replace(/%24/g,"$").replace(/%2C/gi,",").replace(/%20/g,"+").replace(/%5B/gi,"[").replace(/%5D/gi,"]")}function Ay(r,a,i){if(!a)return r;const o=i&&i.encode||j2;Y.isFunction(i)&&(i={serialize:i});const s=i&&i.serialize;let f;if(s?f=s(a,i):f=Y.isURLSearchParams(a)?a.toString():new Bf(a,i).toString(o),f){const d=r.indexOf("#");d!==-1&&(r=r.slice(0,d)),r+=(r.indexOf("?")===-1?"?":"&")+f}return r}class Ng{constructor(){this.handlers=[]}use(a,i,o){return this.handlers.push({fulfilled:a,rejected:i,synchronous:o?o.synchronous:!1,runWhen:o?o.runWhen:null}),this.handlers.length-1}eject(a){this.handlers[a]&&(this.handlers[a]=null)}clear(){this.handlers&&(this.handlers=[])}forEach(a){Y.forEach(this.handlers,function(o){o!==null&&a(o)})}}const ky={silentJSONParsing:!0,forcedJSONParsing:!0,clarifyTimeoutError:!1},L2=typeof URLSearchParams<"u"?URLSearchParams:Bf,B2=typeof FormData<"u"?FormData:null,H2=typeof Blob<"u"?Blob:null,q2={isBrowser:!0,classes:{URLSearchParams:L2,FormData:B2,Blob:H2},protocols:["http","https","file","blob","url","data"]},Hf=typeof window<"u"&&typeof document<"u",af=typeof navigator=="object"&&navigator||void 0,G2=Hf&&(!af||["ReactNative","NativeScript","NS"].indexOf(af.product)<0),Y2=typeof WorkerGlobalScope<"u"&&self instanceof WorkerGlobalScope&&typeof self.importScripts=="function",V2=Hf&&window.location.href||"http://localhost",X2=Object.freeze(Object.defineProperty({__proto__:null,hasBrowserEnv:Hf,hasStandardBrowserEnv:G2,hasStandardBrowserWebWorkerEnv:Y2,navigator:af,origin:V2},Symbol.toStringTag,{value:"Module"})),mt={...X2,...q2};function Q2(r,a){return ou(r,new mt.classes.URLSearchParams,Object.assign({visitor:function(i,o,s,f){return mt.isNode&&Y.isBuffer(i)?(this.append(o,i.toString("base64")),!1):f.defaultVisitor.apply(this,arguments)}},a))}function Z2(r){return Y.matchAll(/\w+|\[(\w*)]/g,r).map(a=>a[0]==="[]"?"":a[1]||a[0])}function K2(r){const a={},i=Object.keys(r);let o;const s=i.length;let f;for(o=0;o<s;o++)f=i[o],a[f]=r[f];return a}function Oy(r){function a(i,o,s,f){let d=i[f++];if(d==="__proto__")return!0;const h=Number.isFinite(+d),g=f>=i.length;return d=!d&&Y.isArray(s)?s.length:d,g?(Y.hasOwnProp(s,d)?s[d]=[s[d],o]:s[d]=o,!h):((!s[d]||!Y.isObject(s[d]))&&(s[d]=[]),a(i,o,s[d],f)&&Y.isArray(s[d])&&(s[d]=K2(s[d])),!h)}if(Y.isFormData(r)&&Y.isFunction(r.entries)){const i={};return Y.forEachEntry(r,(o,s)=>{a(Z2(o),s,i,0)}),i}return null}function $2(r,a,i){if(Y.isString(r))try{return(a||JSON.parse)(r),Y.trim(r)}catch(o){if(o.name!=="SyntaxError")throw o}return(i||JSON.stringify)(r)}const ao={transitional:ky,adapter:["xhr","http","fetch"],transformRequest:[function(a,i){const o=i.getContentType()||"",s=o.indexOf("application/json")>-1,f=Y.isObject(a);if(f&&Y.isHTMLForm(a)&&(a=new FormData(a)),Y.isFormData(a))return s?JSON.stringify(Oy(a)):a;if(Y.isArrayBuffer(a)||Y.isBuffer(a)||Y.isStream(a)||Y.isFile(a)||Y.isBlob(a)||Y.isReadableStream(a))return a;if(Y.isArrayBufferView(a))return a.buffer;if(Y.isURLSearchParams(a))return i.setContentType("application/x-www-form-urlencoded;charset=utf-8",!1),a.toString();let h;if(f){if(o.indexOf("application/x-www-form-urlencoded")>-1)return Q2(a,this.formSerializer).toString();if((h=Y.isFileList(a))||o.indexOf("multipart/form-data")>-1){const g=this.env&&this.env.FormData;return ou(h?{"files[]":a}:a,g&&new g,this.formSerializer)}}return f||s?(i.setContentType("application/json",!1),$2(a)):a}],transformResponse:[function(a){const i=this.transitional||ao.transitional,o=i&&i.forcedJSONParsing,s=this.responseType==="json";if(Y.isResponse(a)||Y.isReadableStream(a))return a;if(a&&Y.isString(a)&&(o&&!this.responseType||s)){const d=!(i&&i.silentJSONParsing)&&s;try{return JSON.parse(a)}catch(h){if(d)throw h.name==="SyntaxError"?ve.from(h,ve.ERR_BAD_RESPONSE,this,null,this.response):h}}return a}],timeout:0,xsrfCookieName:"XSRF-TOKEN",xsrfHeaderName:"X-XSRF-TOKEN",maxContentLength:-1,maxBodyLength:-1,env:{FormData:mt.classes.FormData,Blob:mt.classes.Blob},validateStatus:function(a){return a>=200&&a<300},headers:{common:{Accept:"application/json, text/plain, */*","Content-Type":void 0}}};Y.forEach(["delete","get","head","post","put","patch"],r=>{ao.headers[r]={}});const J2=Y.toObjectSet(["age","authorization","content-length","content-type","etag","expires","from","host","if-modified-since","if-unmodified-since","last-modified","location","max-forwards","proxy-authorization","referer","retry-after","user-agent"]),P2=r=>{const a={};let i,o,s;return r&&r.split(`
`).forEach(function(d){s=d.indexOf(":"),i=d.substring(0,s).trim().toLowerCase(),o=d.substring(s+1).trim(),!(!i||a[i]&&J2[i])&&(i==="set-cookie"?a[i]?a[i].push(o):a[i]=[o]:a[i]=a[i]?a[i]+", "+o:o)}),a},zg=Symbol("internals");function $a(r){return r&&String(r).trim().toLowerCase()}function Ui(r){return r===!1||r==null?r:Y.isArray(r)?r.map(Ui):String(r)}function F2(r){const a=Object.create(null),i=/([^\s,;=]+)\s*(?:=\s*([^,;]+))?/g;let o;for(;o=i.exec(r);)a[o[1]]=o[2];return a}const W2=r=>/^[-_a-zA-Z0-9^`|~,!#$%&'*+.]+$/.test(r.trim());function qc(r,a,i,o,s){if(Y.isFunction(o))return o.call(this,a,i);if(s&&(a=i),!!Y.isString(a)){if(Y.isString(o))return a.indexOf(o)!==-1;if(Y.isRegExp(o))return o.test(a)}}function I2(r){return r.trim().toLowerCase().replace(/([a-z\d])(\w*)/g,(a,i,o)=>i.toUpperCase()+o)}function eT(r,a){const i=Y.toCamelCase(" "+a);["get","set","has"].forEach(o=>{Object.defineProperty(r,o+i,{value:function(s,f,d){return this[o].call(this,a,s,f,d)},configurable:!0})})}let Rt=class{constructor(a){a&&this.set(a)}set(a,i,o){const s=this;function f(h,g,m){const y=$a(g);if(!y)throw new Error("header name must be a non-empty string");const v=Y.findKey(s,y);(!v||s[v]===void 0||m===!0||m===void 0&&s[v]!==!1)&&(s[v||g]=Ui(h))}const d=(h,g)=>Y.forEach(h,(m,y)=>f(m,y,g));if(Y.isPlainObject(a)||a instanceof this.constructor)d(a,i);else if(Y.isString(a)&&(a=a.trim())&&!W2(a))d(P2(a),i);else if(Y.isObject(a)&&Y.isIterable(a)){let h={},g,m;for(const y of a){if(!Y.isArray(y))throw TypeError("Object iterator must return a key-value pair");h[m=y[0]]=(g=h[m])?Y.isArray(g)?[...g,y[1]]:[g,y[1]]:y[1]}d(h,i)}else a!=null&&f(i,a,o);return this}get(a,i){if(a=$a(a),a){const o=Y.findKey(this,a);if(o){const s=this[o];if(!i)return s;if(i===!0)return F2(s);if(Y.isFunction(i))return i.call(this,s,o);if(Y.isRegExp(i))return i.exec(s);throw new TypeError("parser must be boolean|regexp|function")}}}has(a,i){if(a=$a(a),a){const o=Y.findKey(this,a);return!!(o&&this[o]!==void 0&&(!i||qc(this,this[o],o,i)))}return!1}delete(a,i){const o=this;let s=!1;function f(d){if(d=$a(d),d){const h=Y.findKey(o,d);h&&(!i||qc(o,o[h],h,i))&&(delete o[h],s=!0)}}return Y.isArray(a)?a.forEach(f):f(a),s}clear(a){const i=Object.keys(this);let o=i.length,s=!1;for(;o--;){const f=i[o];(!a||qc(this,this[f],f,a,!0))&&(delete this[f],s=!0)}return s}normalize(a){const i=this,o={};return Y.forEach(this,(s,f)=>{const d=Y.findKey(o,f);if(d){i[d]=Ui(s),delete i[f];return}const h=a?I2(f):String(f).trim();h!==f&&delete i[f],i[h]=Ui(s),o[h]=!0}),this}concat(...a){return this.constructor.concat(this,...a)}toJSON(a){const i=Object.create(null);return Y.forEach(this,(o,s)=>{o!=null&&o!==!1&&(i[s]=a&&Y.isArray(o)?o.join(", "):o)}),i}[Symbol.iterator](){return Object.entries(this.toJSON())[Symbol.iterator]()}toString(){return Object.entries(this.toJSON()).map(([a,i])=>a+": "+i).join(`
`)}getSetCookie(){return this.get("set-cookie")||[]}get[Symbol.toStringTag](){return"AxiosHeaders"}static from(a){return a instanceof this?a:new this(a)}static concat(a,...i){const o=new this(a);return i.forEach(s=>o.set(s)),o}static accessor(a){const o=(this[zg]=this[zg]={accessors:{}}).accessors,s=this.prototype;function f(d){const h=$a(d);o[h]||(eT(s,d),o[h]=!0)}return Y.isArray(a)?a.forEach(f):f(a),this}};Rt.accessor(["Content-Type","Content-Length","Accept","Accept-Encoding","User-Agent","Authorization"]);Y.reduceDescriptors(Rt.prototype,({value:r},a)=>{let i=a[0].toUpperCase()+a.slice(1);return{get:()=>r,set(o){this[i]=o}}});Y.freezeMethods(Rt);function Gc(r,a){const i=this||ao,o=a||i,s=Rt.from(o.headers);let f=o.data;return Y.forEach(r,function(h){f=h.call(i,f,s.normalize(),a?a.status:void 0)}),s.normalize(),f}function Cy(r){return!!(r&&r.__CANCEL__)}function Gl(r,a,i){ve.call(this,r??"canceled",ve.ERR_CANCELED,a,i),this.name="CanceledError"}Y.inherits(Gl,ve,{__CANCEL__:!0});function My(r,a,i){const o=i.config.validateStatus;!i.status||!o||o(i.status)?r(i):a(new ve("Request failed with status code "+i.status,[ve.ERR_BAD_REQUEST,ve.ERR_BAD_RESPONSE][Math.floor(i.status/100)-4],i.config,i.request,i))}function tT(r){const a=/^([-+\w]{1,25})(:?\/\/|:)/.exec(r);return a&&a[1]||""}function nT(r,a){r=r||10;const i=new Array(r),o=new Array(r);let s=0,f=0,d;return a=a!==void 0?a:1e3,function(g){const m=Date.now(),y=o[f];d||(d=m),i[s]=g,o[s]=m;let v=f,w=0;for(;v!==s;)w+=i[v++],v=v%r;if(s=(s+1)%r,s===f&&(f=(f+1)%r),m-d<a)return;const S=y&&m-y;return S?Math.round(w*1e3/S):void 0}}function rT(r,a){let i=0,o=1e3/a,s,f;const d=(m,y=Date.now())=>{i=y,s=null,f&&(clearTimeout(f),f=null),r.apply(null,m)};return[(...m)=>{const y=Date.now(),v=y-i;v>=o?d(m,y):(s=m,f||(f=setTimeout(()=>{f=null,d(s)},o-v)))},()=>s&&d(s)]}const Xi=(r,a,i=3)=>{let o=0;const s=nT(50,250);return rT(f=>{const d=f.loaded,h=f.lengthComputable?f.total:void 0,g=d-o,m=s(g),y=d<=h;o=d;const v={loaded:d,total:h,progress:h?d/h:void 0,bytes:g,rate:m||void 0,estimated:m&&h&&y?(h-d)/m:void 0,event:f,lengthComputable:h!=null,[a?"download":"upload"]:!0};r(v)},i)},_g=(r,a)=>{const i=r!=null;return[o=>a[0]({lengthComputable:i,total:r,loaded:o}),a[1]]},Dg=r=>(...a)=>Y.asap(()=>r(...a)),lT=mt.hasStandardBrowserEnv?((r,a)=>i=>(i=new URL(i,mt.origin),r.protocol===i.protocol&&r.host===i.host&&(a||r.port===i.port)))(new URL(mt.origin),mt.navigator&&/(msie|trident)/i.test(mt.navigator.userAgent)):()=>!0,aT=mt.hasStandardBrowserEnv?{write(r,a,i,o,s,f){const d=[r+"="+encodeURIComponent(a)];Y.isNumber(i)&&d.push("expires="+new Date(i).toGMTString()),Y.isString(o)&&d.push("path="+o),Y.isString(s)&&d.push("domain="+s),f===!0&&d.push("secure"),document.cookie=d.join("; ")},read(r){const a=document.cookie.match(new RegExp("(^|;\\s*)("+r+")=([^;]*)"));return a?decodeURIComponent(a[3]):null},remove(r){this.write(r,"",Date.now()-864e5)}}:{write(){},read(){return null},remove(){}};function oT(r){return/^([a-z][a-z\d+\-.]*:)?\/\//i.test(r)}function iT(r,a){return a?r.replace(/\/?\/$/,"")+"/"+a.replace(/^\/+/,""):r}function Ny(r,a,i){let o=!oT(a);return r&&(o||i==!1)?iT(r,a):a}const Ug=r=>r instanceof Rt?{...r}:r;function Br(r,a){a=a||{};const i={};function o(m,y,v,w){return Y.isPlainObject(m)&&Y.isPlainObject(y)?Y.merge.call({caseless:w},m,y):Y.isPlainObject(y)?Y.merge({},y):Y.isArray(y)?y.slice():y}function s(m,y,v,w){if(Y.isUndefined(y)){if(!Y.isUndefined(m))return o(void 0,m,v,w)}else return o(m,y,v,w)}function f(m,y){if(!Y.isUndefined(y))return o(void 0,y)}function d(m,y){if(Y.isUndefined(y)){if(!Y.isUndefined(m))return o(void 0,m)}else return o(void 0,y)}function h(m,y,v){if(v in a)return o(m,y);if(v in r)return o(void 0,m)}const g={url:f,method:f,data:f,baseURL:d,transformRequest:d,transformResponse:d,paramsSerializer:d,timeout:d,timeoutMessage:d,withCredentials:d,withXSRFToken:d,adapter:d,responseType:d,xsrfCookieName:d,xsrfHeaderName:d,onUploadProgress:d,onDownloadProgress:d,decompress:d,maxContentLength:d,maxBodyLength:d,beforeRedirect:d,transport:d,httpAgent:d,httpsAgent:d,cancelToken:d,socketPath:d,responseEncoding:d,validateStatus:h,headers:(m,y,v)=>s(Ug(m),Ug(y),v,!0)};return Y.forEach(Object.keys(Object.assign({},r,a)),function(y){const v=g[y]||s,w=v(r[y],a[y],y);Y.isUndefined(w)&&v!==h||(i[y]=w)}),i}const zy=r=>{const a=Br({},r);let{data:i,withXSRFToken:o,xsrfHeaderName:s,xsrfCookieName:f,headers:d,auth:h}=a;a.headers=d=Rt.from(d),a.url=Ay(Ny(a.baseURL,a.url,a.allowAbsoluteUrls),r.params,r.paramsSerializer),h&&d.set("Authorization","Basic "+btoa((h.username||"")+":"+(h.password?unescape(encodeURIComponent(h.password)):"")));let g;if(Y.isFormData(i)){if(mt.hasStandardBrowserEnv||mt.hasStandardBrowserWebWorkerEnv)d.setContentType(void 0);else if((g=d.getContentType())!==!1){const[m,...y]=g?g.split(";").map(v=>v.trim()).filter(Boolean):[];d.setContentType([m||"multipart/form-data",...y].join("; "))}}if(mt.hasStandardBrowserEnv&&(o&&Y.isFunction(o)&&(o=o(a)),o||o!==!1&&lT(a.url))){const m=s&&f&&aT.read(f);m&&d.set(s,m)}return a},uT=typeof XMLHttpRequest<"u",sT=uT&&function(r){return new Promise(function(i,o){const s=zy(r);let f=s.data;const d=Rt.from(s.headers).normalize();let{responseType:h,onUploadProgress:g,onDownloadProgress:m}=s,y,v,w,S,x;function R(){S&&S(),x&&x(),s.cancelToken&&s.cancelToken.unsubscribe(y),s.signal&&s.signal.removeEventListener("abort",y)}let k=new XMLHttpRequest;k.open(s.method.toUpperCase(),s.url,!0),k.timeout=s.timeout;function O(){if(!k)return;const H=Rt.from("getAllResponseHeaders"in k&&k.getAllResponseHeaders()),V={data:!h||h==="text"||h==="json"?k.responseText:k.response,status:k.status,statusText:k.statusText,headers:H,config:r,request:k};My(function(ee){i(ee),R()},function(ee){o(ee),R()},V),k=null}"onloadend"in k?k.onloadend=O:k.onreadystatechange=function(){!k||k.readyState!==4||k.status===0&&!(k.responseURL&&k.responseURL.indexOf("file:")===0)||setTimeout(O)},k.onabort=function(){k&&(o(new ve("Request aborted",ve.ECONNABORTED,r,k)),k=null)},k.onerror=function(){o(new ve("Network Error",ve.ERR_NETWORK,r,k)),k=null},k.ontimeout=function(){let $=s.timeout?"timeout of "+s.timeout+"ms exceeded":"timeout exceeded";const V=s.transitional||ky;s.timeoutErrorMessage&&($=s.timeoutErrorMessage),o(new ve($,V.clarifyTimeoutError?ve.ETIMEDOUT:ve.ECONNABORTED,r,k)),k=null},f===void 0&&d.setContentType(null),"setRequestHeader"in k&&Y.forEach(d.toJSON(),function($,V){k.setRequestHeader(V,$)}),Y.isUndefined(s.withCredentials)||(k.withCredentials=!!s.withCredentials),h&&h!=="json"&&(k.responseType=s.responseType),m&&([w,x]=Xi(m,!0),k.addEventListener("progress",w)),g&&k.upload&&([v,S]=Xi(g),k.upload.addEventListener("progress",v),k.upload.addEventListener("loadend",S)),(s.cancelToken||s.signal)&&(y=H=>{k&&(o(!H||H.type?new Gl(null,r,k):H),k.abort(),k=null)},s.cancelToken&&s.cancelToken.subscribe(y),s.signal&&(s.signal.aborted?y():s.signal.addEventListener("abort",y)));const M=tT(s.url);if(M&&mt.protocols.indexOf(M)===-1){o(new ve("Unsupported protocol "+M+":",ve.ERR_BAD_REQUEST,r));return}k.send(f||null)})},cT=(r,a)=>{const{length:i}=r=r?r.filter(Boolean):[];if(a||i){let o=new AbortController,s;const f=function(m){if(!s){s=!0,h();const y=m instanceof Error?m:this.reason;o.abort(y instanceof ve?y:new Gl(y instanceof Error?y.message:y))}};let d=a&&setTimeout(()=>{d=null,f(new ve(`timeout ${a} of ms exceeded`,ve.ETIMEDOUT))},a);const h=()=>{r&&(d&&clearTimeout(d),d=null,r.forEach(m=>{m.unsubscribe?m.unsubscribe(f):m.removeEventListener("abort",f)}),r=null)};r.forEach(m=>m.addEventListener("abort",f));const{signal:g}=o;return g.unsubscribe=()=>Y.asap(h),g}},fT=function*(r,a){let i=r.byteLength;if(i<a){yield r;return}let o=0,s;for(;o<i;)s=o+a,yield r.slice(o,s),o=s},dT=async function*(r,a){for await(const i of hT(r))yield*fT(i,a)},hT=async function*(r){if(r[Symbol.asyncIterator]){yield*r;return}const a=r.getReader();try{for(;;){const{done:i,value:o}=await a.read();if(i)break;yield o}}finally{await a.cancel()}},jg=(r,a,i,o)=>{const s=dT(r,a);let f=0,d,h=g=>{d||(d=!0,o&&o(g))};return new ReadableStream({async pull(g){try{const{done:m,value:y}=await s.next();if(m){h(),g.close();return}let v=y.byteLength;if(i){let w=f+=v;i(w)}g.enqueue(new Uint8Array(y))}catch(m){throw h(m),m}},cancel(g){return h(g),s.return()}},{highWaterMark:2})},iu=typeof fetch=="function"&&typeof Request=="function"&&typeof Response=="function",_y=iu&&typeof ReadableStream=="function",mT=iu&&(typeof TextEncoder=="function"?(r=>a=>r.encode(a))(new TextEncoder):async r=>new Uint8Array(await new Response(r).arrayBuffer())),Dy=(r,...a)=>{try{return!!r(...a)}catch{return!1}},gT=_y&&Dy(()=>{let r=!1;const a=new Request(mt.origin,{body:new ReadableStream,method:"POST",get duplex(){return r=!0,"half"}}).headers.has("Content-Type");return r&&!a}),Lg=64*1024,of=_y&&Dy(()=>Y.isReadableStream(new Response("").body)),Qi={stream:of&&(r=>r.body)};iu&&(r=>{["text","arrayBuffer","blob","formData","stream"].forEach(a=>{!Qi[a]&&(Qi[a]=Y.isFunction(r[a])?i=>i[a]():(i,o)=>{throw new ve(`Response type '${a}' is not supported`,ve.ERR_NOT_SUPPORT,o)})})})(new Response);const pT=async r=>{if(r==null)return 0;if(Y.isBlob(r))return r.size;if(Y.isSpecCompliantForm(r))return(await new Request(mt.origin,{method:"POST",body:r}).arrayBuffer()).byteLength;if(Y.isArrayBufferView(r)||Y.isArrayBuffer(r))return r.byteLength;if(Y.isURLSearchParams(r)&&(r=r+""),Y.isString(r))return(await mT(r)).byteLength},yT=async(r,a)=>{const i=Y.toFiniteNumber(r.getContentLength());return i??pT(a)},bT=iu&&(async r=>{let{url:a,method:i,data:o,signal:s,cancelToken:f,timeout:d,onDownloadProgress:h,onUploadProgress:g,responseType:m,headers:y,withCredentials:v="same-origin",fetchOptions:w}=zy(r);m=m?(m+"").toLowerCase():"text";let S=cT([s,f&&f.toAbortSignal()],d),x;const R=S&&S.unsubscribe&&(()=>{S.unsubscribe()});let k;try{if(g&&gT&&i!=="get"&&i!=="head"&&(k=await yT(y,o))!==0){let V=new Request(a,{method:"POST",body:o,duplex:"half"}),W;if(Y.isFormData(o)&&(W=V.headers.get("content-type"))&&y.setContentType(W),V.body){const[ee,fe]=_g(k,Xi(Dg(g)));o=jg(V.body,Lg,ee,fe)}}Y.isString(v)||(v=v?"include":"omit");const O="credentials"in Request.prototype;x=new Request(a,{...w,signal:S,method:i.toUpperCase(),headers:y.normalize().toJSON(),body:o,duplex:"half",credentials:O?v:void 0});let M=await fetch(x,w);const H=of&&(m==="stream"||m==="response");if(of&&(h||H&&R)){const V={};["status","statusText","headers"].forEach(ne=>{V[ne]=M[ne]});const W=Y.toFiniteNumber(M.headers.get("content-length")),[ee,fe]=h&&_g(W,Xi(Dg(h),!0))||[];M=new Response(jg(M.body,Lg,ee,()=>{fe&&fe(),R&&R()}),V)}m=m||"text";let $=await Qi[Y.findKey(Qi,m)||"text"](M,r);return!H&&R&&R(),await new Promise((V,W)=>{My(V,W,{data:$,headers:Rt.from(M.headers),status:M.status,statusText:M.statusText,config:r,request:x})})}catch(O){throw R&&R(),O&&O.name==="TypeError"&&/Load failed|fetch/i.test(O.message)?Object.assign(new ve("Network Error",ve.ERR_NETWORK,r,x),{cause:O.cause||O}):ve.from(O,O&&O.code,r,x)}}),uf={http:_2,xhr:sT,fetch:bT};Y.forEach(uf,(r,a)=>{if(r){try{Object.defineProperty(r,"name",{value:a})}catch{}Object.defineProperty(r,"adapterName",{value:a})}});const Bg=r=>`- ${r}`,vT=r=>Y.isFunction(r)||r===null||r===!1,Uy={getAdapter:r=>{r=Y.isArray(r)?r:[r];const{length:a}=r;let i,o;const s={};for(let f=0;f<a;f++){i=r[f];let d;if(o=i,!vT(i)&&(o=uf[(d=String(i)).toLowerCase()],o===void 0))throw new ve(`Unknown adapter '${d}'`);if(o)break;s[d||"#"+f]=o}if(!o){const f=Object.entries(s).map(([h,g])=>`adapter ${h} `+(g===!1?"is not supported by the environment":"is not available in the build"));let d=a?f.length>1?`since :
`+f.map(Bg).join(`
`):" "+Bg(f[0]):"as no adapter specified";throw new ve("There is no suitable adapter to dispatch the request "+d,"ERR_NOT_SUPPORT")}return o},adapters:uf};function Yc(r){if(r.cancelToken&&r.cancelToken.throwIfRequested(),r.signal&&r.signal.aborted)throw new Gl(null,r)}function Hg(r){return Yc(r),r.headers=Rt.from(r.headers),r.data=Gc.call(r,r.transformRequest),["post","put","patch"].indexOf(r.method)!==-1&&r.headers.setContentType("application/x-www-form-urlencoded",!1),Uy.getAdapter(r.adapter||ao.adapter)(r).then(function(o){return Yc(r),o.data=Gc.call(r,r.transformResponse,o),o.headers=Rt.from(o.headers),o},function(o){return Cy(o)||(Yc(r),o&&o.response&&(o.response.data=Gc.call(r,r.transformResponse,o.response),o.response.headers=Rt.from(o.response.headers))),Promise.reject(o)})}const jy="1.10.0",uu={};["object","boolean","number","function","string","symbol"].forEach((r,a)=>{uu[r]=function(o){return typeof o===r||"a"+(a<1?"n ":" ")+r}});const qg={};uu.transitional=function(a,i,o){function s(f,d){return"[Axios v"+jy+"] Transitional option '"+f+"'"+d+(o?". "+o:"")}return(f,d,h)=>{if(a===!1)throw new ve(s(d," has been removed"+(i?" in "+i:"")),ve.ERR_DEPRECATED);return i&&!qg[d]&&(qg[d]=!0,console.warn(s(d," has been deprecated since v"+i+" and will be removed in the near future"))),a?a(f,d,h):!0}};uu.spelling=function(a){return(i,o)=>(console.warn(`${o} is likely a misspelling of ${a}`),!0)};function xT(r,a,i){if(typeof r!="object")throw new ve("options must be an object",ve.ERR_BAD_OPTION_VALUE);const o=Object.keys(r);let s=o.length;for(;s-- >0;){const f=o[s],d=a[f];if(d){const h=r[f],g=h===void 0||d(h,f,r);if(g!==!0)throw new ve("option "+f+" must be "+g,ve.ERR_BAD_OPTION_VALUE);continue}if(i!==!0)throw new ve("Unknown option "+f,ve.ERR_BAD_OPTION)}}const ji={assertOptions:xT,validators:uu},on=ji.validators;let jr=class{constructor(a){this.defaults=a||{},this.interceptors={request:new Ng,response:new Ng}}async request(a,i){try{return await this._request(a,i)}catch(o){if(o instanceof Error){let s={};Error.captureStackTrace?Error.captureStackTrace(s):s=new Error;const f=s.stack?s.stack.replace(/^.+\n/,""):"";try{o.stack?f&&!String(o.stack).endsWith(f.replace(/^.+\n.+\n/,""))&&(o.stack+=`
`+f):o.stack=f}catch{}}throw o}}_request(a,i){typeof a=="string"?(i=i||{},i.url=a):i=a||{},i=Br(this.defaults,i);const{transitional:o,paramsSerializer:s,headers:f}=i;o!==void 0&&ji.assertOptions(o,{silentJSONParsing:on.transitional(on.boolean),forcedJSONParsing:on.transitional(on.boolean),clarifyTimeoutError:on.transitional(on.boolean)},!1),s!=null&&(Y.isFunction(s)?i.paramsSerializer={serialize:s}:ji.assertOptions(s,{encode:on.function,serialize:on.function},!0)),i.allowAbsoluteUrls!==void 0||(this.defaults.allowAbsoluteUrls!==void 0?i.allowAbsoluteUrls=this.defaults.allowAbsoluteUrls:i.allowAbsoluteUrls=!0),ji.assertOptions(i,{baseUrl:on.spelling("baseURL"),withXsrfToken:on.spelling("withXSRFToken")},!0),i.method=(i.method||this.defaults.method||"get").toLowerCase();let d=f&&Y.merge(f.common,f[i.method]);f&&Y.forEach(["delete","get","head","post","put","patch","common"],x=>{delete f[x]}),i.headers=Rt.concat(d,f);const h=[];let g=!0;this.interceptors.request.forEach(function(R){typeof R.runWhen=="function"&&R.runWhen(i)===!1||(g=g&&R.synchronous,h.unshift(R.fulfilled,R.rejected))});const m=[];this.interceptors.response.forEach(function(R){m.push(R.fulfilled,R.rejected)});let y,v=0,w;if(!g){const x=[Hg.bind(this),void 0];for(x.unshift.apply(x,h),x.push.apply(x,m),w=x.length,y=Promise.resolve(i);v<w;)y=y.then(x[v++],x[v++]);return y}w=h.length;let S=i;for(v=0;v<w;){const x=h[v++],R=h[v++];try{S=x(S)}catch(k){R.call(this,k);break}}try{y=Hg.call(this,S)}catch(x){return Promise.reject(x)}for(v=0,w=m.length;v<w;)y=y.then(m[v++],m[v++]);return y}getUri(a){a=Br(this.defaults,a);const i=Ny(a.baseURL,a.url,a.allowAbsoluteUrls);return Ay(i,a.params,a.paramsSerializer)}};Y.forEach(["delete","get","head","options"],function(a){jr.prototype[a]=function(i,o){return this.request(Br(o||{},{method:a,url:i,data:(o||{}).data}))}});Y.forEach(["post","put","patch"],function(a){function i(o){return function(f,d,h){return this.request(Br(h||{},{method:a,headers:o?{"Content-Type":"multipart/form-data"}:{},url:f,data:d}))}}jr.prototype[a]=i(),jr.prototype[a+"Form"]=i(!0)});let wT=class Ly{constructor(a){if(typeof a!="function")throw new TypeError("executor must be a function.");let i;this.promise=new Promise(function(f){i=f});const o=this;this.promise.then(s=>{if(!o._listeners)return;let f=o._listeners.length;for(;f-- >0;)o._listeners[f](s);o._listeners=null}),this.promise.then=s=>{let f;const d=new Promise(h=>{o.subscribe(h),f=h}).then(s);return d.cancel=function(){o.unsubscribe(f)},d},a(function(f,d,h){o.reason||(o.reason=new Gl(f,d,h),i(o.reason))})}throwIfRequested(){if(this.reason)throw this.reason}subscribe(a){if(this.reason){a(this.reason);return}this._listeners?this._listeners.push(a):this._listeners=[a]}unsubscribe(a){if(!this._listeners)return;const i=this._listeners.indexOf(a);i!==-1&&this._listeners.splice(i,1)}toAbortSignal(){const a=new AbortController,i=o=>{a.abort(o)};return this.subscribe(i),a.signal.unsubscribe=()=>this.unsubscribe(i),a.signal}static source(){let a;return{token:new Ly(function(s){a=s}),cancel:a}}};function ST(r){return function(i){return r.apply(null,i)}}function ET(r){return Y.isObject(r)&&r.isAxiosError===!0}const sf={Continue:100,SwitchingProtocols:101,Processing:102,EarlyHints:103,Ok:200,Created:201,Accepted:202,NonAuthoritativeInformation:203,NoContent:204,ResetContent:205,PartialContent:206,MultiStatus:207,AlreadyReported:208,ImUsed:226,MultipleChoices:300,MovedPermanently:301,Found:302,SeeOther:303,NotModified:304,UseProxy:305,Unused:306,TemporaryRedirect:307,PermanentRedirect:308,BadRequest:400,Unauthorized:401,PaymentRequired:402,Forbidden:403,NotFound:404,MethodNotAllowed:405,NotAcceptable:406,ProxyAuthenticationRequired:407,RequestTimeout:408,Conflict:409,Gone:410,LengthRequired:411,PreconditionFailed:412,PayloadTooLarge:413,UriTooLong:414,UnsupportedMediaType:415,RangeNotSatisfiable:416,ExpectationFailed:417,ImATeapot:418,MisdirectedRequest:421,UnprocessableEntity:422,Locked:423,FailedDependency:424,TooEarly:425,UpgradeRequired:426,PreconditionRequired:428,TooManyRequests:429,RequestHeaderFieldsTooLarge:431,UnavailableForLegalReasons:451,InternalServerError:500,NotImplemented:501,BadGateway:502,ServiceUnavailable:503,GatewayTimeout:504,HttpVersionNotSupported:505,VariantAlsoNegotiates:506,InsufficientStorage:507,LoopDetected:508,NotExtended:510,NetworkAuthenticationRequired:511};Object.entries(sf).forEach(([r,a])=>{sf[a]=r});function By(r){const a=new jr(r),i=my(jr.prototype.request,a);return Y.extend(i,jr.prototype,a,{allOwnKeys:!0}),Y.extend(i,a,null,{allOwnKeys:!0}),i.create=function(s){return By(Br(r,s))},i}const Le=By(ao);Le.Axios=jr;Le.CanceledError=Gl;Le.CancelToken=wT;Le.isCancel=Cy;Le.VERSION=jy;Le.toFormData=ou;Le.AxiosError=ve;Le.Cancel=Le.CanceledError;Le.all=function(a){return Promise.all(a)};Le.spread=ST;Le.isAxiosError=ET;Le.mergeConfig=Br;Le.AxiosHeaders=Rt;Le.formToJSON=r=>Oy(Y.isHTMLForm(r)?new FormData(r):r);Le.getAdapter=Uy.getAdapter;Le.HttpStatusCode=sf;Le.default=Le;const{Axios:QT,AxiosError:ZT,CanceledError:KT,isCancel:$T,CancelToken:JT,VERSION:PT,all:FT,Cancel:WT,isAxiosError:IT,spread:eA,toFormData:tA,AxiosHeaders:nA,HttpStatusCode:rA,formToJSON:lA,getAdapter:aA,mergeConfig:oA}=Le,Hy=T.createContext({token:null,user:null,isAuthenticated:!1,isLoading:!0,login:async()=>!1,logout:()=>{}}),su=()=>T.useContext(Hy),RT=({children:r})=>{const[a,i]=T.useState({token:null,user:null,isAuthenticated:!1,isLoading:!0});T.useEffect(()=>{(async()=>{const d=localStorage.getItem("auth_token"),h=localStorage.getItem("auth_user");if(d&&h)try{Le.defaults.headers.common.Authorization=`Bearer ${d}`,i({token:d,user:JSON.parse(h),isAuthenticated:!0,isLoading:!1})}catch{localStorage.removeItem("auth_token"),localStorage.removeItem("auth_user"),i({token:null,user:null,isAuthenticated:!1,isLoading:!1})}else i(g=>({...g,isLoading:!1}))})()},[]);const o=async(f,d)=>{var h,g;try{console.log("Attempting login with:",{username:f});const m=new URLSearchParams;m.append("username",f),m.append("password",d),console.log("API URL:",window.location.origin+"/api/login"),console.log("Sending request to:","/api/login");const y=await Le.post("/api/login",m,{headers:{"Content-Type":"application/x-www-form-urlencoded"}});console.log("Login response:",y.data);const{access_token:v,token_type:w,...S}=y.data;return localStorage.setItem("auth_token",v),localStorage.setItem("auth_user",JSON.stringify(S)),Le.defaults.headers.common.Authorization=`Bearer ${v}`,i({token:v,user:S,isAuthenticated:!0,isLoading:!1}),!0}catch(m){return console.error("Login failed:",m),Le.isAxiosError(m)&&(console.error("Error details:",((h=m.response)==null?void 0:h.data)||"No response data"),console.error("Status code:",((g=m.response)==null?void 0:g.status)||"No status code")),!1}},s=()=>{localStorage.removeItem("auth_token"),localStorage.removeItem("auth_user"),delete Le.defaults.headers.common.Authorization,i({token:null,user:null,isAuthenticated:!1,isLoading:!1}),Le.post("/api/logout").catch(f=>{console.error("Logout API call failed:",f)})};return G.jsx(Hy.Provider,{value:{...a,login:o,logout:s},children:r})},TT=()=>{const{isAuthenticated:r,isLoading:a}=su();return a?G.jsx("div",{className:"flex items-center justify-center min-h-screen",children:G.jsx("div",{className:"animate-spin w-12 h-12 border-4 border-blue-600 rounded-full border-t-transparent"})}):r?G.jsx(ep,{}):G.jsx(yf,{to:"/login",replace:!0})};var qy={color:void 0,size:void 0,className:void 0,style:void 0,attr:void 0},Gg=ir.createContext&&ir.createContext(qy),AT=["attr","size","title"];function kT(r,a){if(r==null)return{};var i=OT(r,a),o,s;if(Object.getOwnPropertySymbols){var f=Object.getOwnPropertySymbols(r);for(s=0;s<f.length;s++)o=f[s],!(a.indexOf(o)>=0)&&Object.prototype.propertyIsEnumerable.call(r,o)&&(i[o]=r[o])}return i}function OT(r,a){if(r==null)return{};var i={};for(var o in r)if(Object.prototype.hasOwnProperty.call(r,o)){if(a.indexOf(o)>=0)continue;i[o]=r[o]}return i}function Zi(){return Zi=Object.assign?Object.assign.bind():function(r){for(var a=1;a<arguments.length;a++){var i=arguments[a];for(var o in i)Object.prototype.hasOwnProperty.call(i,o)&&(r[o]=i[o])}return r},Zi.apply(this,arguments)}function Yg(r,a){var i=Object.keys(r);if(Object.getOwnPropertySymbols){var o=Object.getOwnPropertySymbols(r);a&&(o=o.filter(function(s){return Object.getOwnPropertyDescriptor(r,s).enumerable})),i.push.apply(i,o)}return i}function Ki(r){for(var a=1;a<arguments.length;a++){var i=arguments[a]!=null?arguments[a]:{};a%2?Yg(Object(i),!0).forEach(function(o){CT(r,o,i[o])}):Object.getOwnPropertyDescriptors?Object.defineProperties(r,Object.getOwnPropertyDescriptors(i)):Yg(Object(i)).forEach(function(o){Object.defineProperty(r,o,Object.getOwnPropertyDescriptor(i,o))})}return r}function CT(r,a,i){return a=MT(a),a in r?Object.defineProperty(r,a,{value:i,enumerable:!0,configurable:!0,writable:!0}):r[a]=i,r}function MT(r){var a=NT(r,"string");return typeof a=="symbol"?a:a+""}function NT(r,a){if(typeof r!="object"||!r)return r;var i=r[Symbol.toPrimitive];if(i!==void 0){var o=i.call(r,a);if(typeof o!="object")return o;throw new TypeError("@@toPrimitive must return a primitive value.")}return(a==="string"?String:Number)(r)}function Gy(r){return r&&r.map((a,i)=>ir.createElement(a.tag,Ki({key:i},a.attr),Gy(a.child)))}function Yl(r){return a=>ir.createElement(zT,Zi({attr:Ki({},r.attr)},a),Gy(r.child))}function zT(r){var a=i=>{var{attr:o,size:s,title:f}=r,d=kT(r,AT),h=s||i.size||"1em",g;return i.className&&(g=i.className),r.className&&(g=(g?g+" ":"")+r.className),ir.createElement("svg",Zi({stroke:"currentColor",fill:"currentColor",strokeWidth:"0"},i.attr,o,d,{className:g,style:Ki(Ki({color:r.color||i.color},i.style),r.style),height:h,width:h,xmlns:"http://www.w3.org/2000/svg"}),f&&ir.createElement("title",null,f),r.children)};return Gg!==void 0?ir.createElement(Gg.Consumer,null,i=>a(i)):a(qy)}function _T(r){return Yl({attr:{viewBox:"0 0 20 20",fill:"currentColor","aria-hidden":"true"},child:[{tag:"path",attr:{d:"M2 10a8 8 0 018-8v8h8a8 8 0 11-16 0z"},child:[]},{tag:"path",attr:{d:"M12 2.252A8.014 8.014 0 0117.748 8H12V2.252z"},child:[]}]})(r)}function DT(r){return Yl({attr:{viewBox:"0 0 20 20",fill:"currentColor","aria-hidden":"true"},child:[{tag:"path",attr:{fillRule:"evenodd",d:"M5 3a2 2 0 00-2 2v10a2 2 0 002 2h10a2 2 0 002-2V5a2 2 0 00-2-2H5zm0 2h10v7h-2l-1 2H8l-1-2H5V5z",clipRule:"evenodd"},child:[]}]})(r)}function UT(r){return Yl({attr:{viewBox:"0 0 20 20",fill:"currentColor","aria-hidden":"true"},child:[{tag:"path",attr:{fillRule:"evenodd",d:"M3 3a1 1 0 00-1 1v12a1 1 0 102 0V4a1 1 0 00-1-1zm10.293 9.293a1 1 0 001.414 1.414l3-3a1 1 0 000-1.414l-3-3a1 1 0 10-1.414 1.414L14.586 9H7a1 1 0 100 2h7.586l-1.293 1.293z",clipRule:"evenodd"},child:[]}]})(r)}function jT(r){return Yl({attr:{viewBox:"0 0 20 20",fill:"currentColor","aria-hidden":"true"},child:[{tag:"path",attr:{fillRule:"evenodd",d:"M10 2a4 4 0 00-4 4v1H5a1 1 0 00-.994.89l-1 9A1 1 0 004 18h12a1 1 0 00.994-1.11l-1-9A1 1 0 0015 7h-1V6a4 4 0 00-4-4zm2 5V6a2 2 0 10-4 0v1h4zm-6 3a1 1 0 112 0 1 1 0 01-2 0zm7-1a1 1 0 100 2 1 1 0 000-2z",clipRule:"evenodd"},child:[]}]})(r)}function LT(r){return Yl({attr:{viewBox:"0 0 20 20",fill:"currentColor","aria-hidden":"true"},child:[{tag:"path",attr:{fillRule:"evenodd",d:"M10 9a3 3 0 100-6 3 3 0 000 6zm-7 9a7 7 0 1114 0H3z",clipRule:"evenodd"},child:[]}]})(r)}function BT(r){return Yl({attr:{viewBox:"0 0 20 20",fill:"currentColor","aria-hidden":"true"},child:[{tag:"path",attr:{d:"M2 4a1 1 0 011-1h2a1 1 0 011 1v12a1 1 0 01-1 1H3a1 1 0 01-1-1V4zM8 4a1 1 0 011-1h2a1 1 0 011 1v12a1 1 0 01-1 1H9a1 1 0 01-1-1V4zM15 3a1 1 0 00-1 1v12a1 1 0 001 1h2a1 1 0 001-1V4a1 1 0 00-1-1h-2z"},child:[]}]})(r)}const HT=()=>{var d;const[r,a]=T.useState(!1),{user:i,logout:o}=su(),s=()=>{a(!r)},f=()=>{o()};return G.jsxs("div",{className:"flex flex-col min-h-screen bg-gray-50 dark:bg-gray-900",children:[G.jsxs(ry,{fluid:!0,rounded:!0,className:"border-b border-gray-200 bg-white dark:border-gray-700 dark:bg-gray-800",children:[G.jsxs(ly,{href:"/",children:[G.jsx("img",{src:"/flowbite-react.svg",className:"mr-3 h-6 sm:h-9",alt:"Flowbite Logo"}),G.jsx("span",{className:"self-center whitespace-nowrap text-xl font-semibold dark:text-white",children:"VitalGraph"})]}),G.jsxs("div",{className:"flex md:order-2 items-center",children:[i&&G.jsxs("div",{className:"mr-4 flex items-center",children:[i.profile_image?G.jsx("img",{src:i.profile_image,alt:i.full_name,className:"w-8 h-8 rounded-full mr-2 object-cover border border-gray-200 dark:border-gray-600",onError:h=>{const g=h.currentTarget;g.style.display="none";const m=document.querySelector(".fallback-avatar");m instanceof HTMLElement&&(m.style.display="flex")}}):null,G.jsx("div",{className:`w-8 h-8 rounded-full mr-2 bg-blue-600 flex items-center justify-center text-white font-medium fallback-avatar ${i.profile_image?"hidden":""}`,title:i.full_name,children:((d=i.full_name)==null?void 0:d.charAt(0).toUpperCase())||"U"}),G.jsx("span",{className:"hidden md:inline text-sm font-medium dark:text-white",children:i.full_name})]}),G.jsx(ty,{className:"mr-2"}),G.jsxs(Bi,{color:"failure",onClick:f,size:"sm",children:[G.jsx(UT,{className:"mr-2 h-5 w-5"}),G.jsx("span",{className:"hidden sm:inline",children:"Logout"})]}),G.jsx(iy,{className:"ml-2"})]}),G.jsxs(ay,{children:[G.jsx(_i,{href:"/",active:!0,children:"Home"}),G.jsx(_i,{href:"/spaces",children:"Spaces"}),G.jsx(_i,{href:"/sparql",children:"SPARQL"})]})]}),G.jsxs("div",{className:"flex flex-1",children:[G.jsx(sy,{"aria-label":"Application sidebar",collapsed:!r,className:"w-fit",children:G.jsx(hy,{children:G.jsxs(dy,{children:[G.jsx(Al,{href:"/",icon:_T,children:"Home"}),G.jsx(Al,{href:"/spaces",icon:BT,label:"Pro",labelColor:"dark",children:"Spaces"}),G.jsx(Al,{href:"/sparql",icon:DT,label:"3",children:"SPARQL"}),G.jsx(Al,{href:"/users",icon:LT,children:"Users"}),G.jsx(Al,{href:"/products",icon:jT,children:"Products"})]})})}),G.jsx("main",{className:"flex-1 p-4",children:G.jsxs("div",{className:"rounded-lg border border-gray-200 bg-white p-4 shadow-sm dark:border-gray-700 dark:bg-gray-800 sm:p-6 lg:p-8",children:[G.jsx(ep,{}),G.jsx(Bi,{onClick:s,className:"mt-4",children:"Toggle Sidebar"})]})})]})]})},qT=()=>{const[r,a]=T.useState(""),[i,o]=T.useState(""),[s,f]=T.useState(!1),[d,h]=T.useState(null),[g,m]=T.useState(!1),{login:y,isAuthenticated:v}=su(),w=mf();if(v)return G.jsx(yf,{to:"/"});const S=async x=>{if(x.preventDefault(),!r||!i){h("Please enter both username and password");return}m(!0),h(null);try{await y(r,i)?w("/"):h("Invalid username or password")}catch(R){h("An error occurred during login. Please try again."),console.error("Login error:",R)}finally{m(!1)}};return G.jsx("div",{className:"flex items-center justify-center min-h-screen bg-gray-100 dark:bg-gray-900 px-4",children:G.jsxs(Cp,{className:"w-full max-w-md",children:[G.jsx("div",{className:"flex justify-center mb-4",children:G.jsx("img",{src:"/flowbite-react.svg",className:"h-12",alt:"VitalGraph Logo"})}),G.jsx("h1",{className:"text-xl font-bold text-center mb-6 dark:text-white",children:"Sign in to VitalGraph"}),d&&G.jsx(Rf,{color:"failure",className:"mb-4",children:d}),G.jsxs("form",{className:"flex flex-col gap-4",onSubmit:S,children:[G.jsxs("div",{children:[G.jsx("div",{className:"mb-2 block",children:G.jsx(zi,{htmlFor:"username",children:"Username"})}),G.jsx(nf,{id:"username",placeholder:"Enter your username",required:!0,value:r,onChange:x=>a(x.target.value),disabled:g})]}),G.jsxs("div",{children:[G.jsx("div",{className:"mb-2 block",children:G.jsx(zi,{htmlFor:"password",children:"Password"})}),G.jsx(nf,{id:"password",type:"password",placeholder:"••••••••",required:!0,value:i,onChange:x=>o(x.target.value),disabled:g})]}),G.jsxs("div",{className:"flex items-center gap-2",children:[G.jsx(Np,{id:"remember",checked:s,onChange:()=>f(!s),disabled:g}),G.jsx(zi,{htmlFor:"remember",children:"Remember me"})]}),G.jsx(Bi,{type:"submit",disabled:g,children:g?"Signing in...":"Sign in"})]})]})})},GT=()=>{const{user:r}=su();return G.jsxs("div",{children:[G.jsx("h1",{className:"mb-4 text-xl font-bold text-gray-900 dark:text-white",children:"Welcome to VitalGraph"}),G.jsxs(Rf,{color:"info",className:"mb-4",children:[G.jsxs("div",{className:"font-medium",children:["Hello, ",(r==null?void 0:r.full_name)||"User","!"]}),G.jsxs("div",{children:["You are successfully logged in as ",(r==null?void 0:r.role)||"User","."]})]}),G.jsx("p",{className:"text-base text-gray-500 dark:text-gray-400",children:"This is the authenticated home page of your VitalGraph application. You now have access to all the features and data available for your account."})]})};KR();function YT(){return G.jsx(RT,{children:G.jsx(Lx,{children:G.jsxs(hx,{children:[G.jsx(Tl,{path:"/login",element:G.jsx(qT,{})}),G.jsx(Tl,{element:G.jsx(TT,{}),children:G.jsx(Tl,{element:G.jsx(HT,{}),children:G.jsx(Tl,{path:"/",element:G.jsx(GT,{})})})}),G.jsx(Tl,{path:"*",element:G.jsx(yf,{to:"/"})})]})})})}Le.defaults.baseURL=window.location.origin;const Vg=localStorage.getItem("auth_token");Vg&&(Le.defaults.headers.common.Authorization=`Bearer ${Vg}`);x1.createRoot(document.getElementById("root")).render(G.jsx(T.StrictMode,{children:G.jsx(YT,{})}));
