(function(){const s=document.createElement("link").relList;if(s&&s.supports&&s.supports("modulepreload"))return;for(const h of document.querySelectorAll('link[rel="modulepreload"]'))f(h);new MutationObserver(h=>{for(const g of h)if(g.type==="childList")for(const A of g.addedNodes)A.tagName==="LINK"&&A.rel==="modulepreload"&&f(A)}).observe(document,{childList:!0,subtree:!0});function o(h){const g={};return h.integrity&&(g.integrity=h.integrity),h.referrerPolicy&&(g.referrerPolicy=h.referrerPolicy),h.crossOrigin==="use-credentials"?g.credentials="include":h.crossOrigin==="anonymous"?g.credentials="omit":g.credentials="same-origin",g}function f(h){if(h.ep)return;h.ep=!0;const g=o(h);fetch(h.href,g)}})();function Fd(c){return c&&c.__esModule&&Object.prototype.hasOwnProperty.call(c,"default")?c.default:c}var Er={exports:{}},zu={};/**
 * @license React
 * react-jsx-runtime.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Cd;function Hm(){if(Cd)return zu;Cd=1;var c=Symbol.for("react.transitional.element"),s=Symbol.for("react.fragment");function o(f,h,g){var A=null;if(g!==void 0&&(A=""+g),h.key!==void 0&&(A=""+h.key),"key"in h){g={};for(var z in h)z!=="key"&&(g[z]=h[z])}else g=h;return h=g.ref,{$$typeof:c,type:f,key:A,ref:h!==void 0?h:null,props:g}}return zu.Fragment=s,zu.jsx=o,zu.jsxs=o,zu}var Hd;function Bm(){return Hd||(Hd=1,Er.exports=Hm()),Er.exports}var d=Bm(),Ar={exports:{}},te={};/**
 * @license React
 * react.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Bd;function qm(){if(Bd)return te;Bd=1;var c=Symbol.for("react.transitional.element"),s=Symbol.for("react.portal"),o=Symbol.for("react.fragment"),f=Symbol.for("react.strict_mode"),h=Symbol.for("react.profiler"),g=Symbol.for("react.consumer"),A=Symbol.for("react.context"),z=Symbol.for("react.forward_ref"),b=Symbol.for("react.suspense"),m=Symbol.for("react.memo"),R=Symbol.for("react.lazy"),H=Symbol.iterator;function D(v){return v===null||typeof v!="object"?null:(v=H&&v[H]||v["@@iterator"],typeof v=="function"?v:null)}var L={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},C=Object.assign,Q={};function X(v,w,G){this.props=v,this.context=w,this.refs=Q,this.updater=G||L}X.prototype.isReactComponent={},X.prototype.setState=function(v,w){if(typeof v!="object"&&typeof v!="function"&&v!=null)throw Error("takes an object of state variables to update or a function which returns an object of state variables.");this.updater.enqueueSetState(this,v,w,"setState")},X.prototype.forceUpdate=function(v){this.updater.enqueueForceUpdate(this,v,"forceUpdate")};function B(){}B.prototype=X.prototype;function ee(v,w,G){this.props=v,this.context=w,this.refs=Q,this.updater=G||L}var P=ee.prototype=new B;P.constructor=ee,C(P,X.prototype),P.isPureReactComponent=!0;var de=Array.isArray,F={H:null,A:null,T:null,S:null,V:null},Me=Object.prototype.hasOwnProperty;function Te(v,w,G,q,V,ce){return G=ce.ref,{$$typeof:c,type:v,key:w,ref:G!==void 0?G:null,props:ce}}function je(v,w){return Te(v.type,w,void 0,void 0,void 0,v.props)}function be(v){return typeof v=="object"&&v!==null&&v.$$typeof===c}function Ke(v){var w={"=":"=0",":":"=2"};return"$"+v.replace(/[=:]/g,function(G){return w[G]})}var ot=/\/+/g;function Qe(v,w){return typeof v=="object"&&v!==null&&v.key!=null?Ke(""+v.key):w.toString(36)}function Al(){}function Tl(v){switch(v.status){case"fulfilled":return v.value;case"rejected":throw v.reason;default:switch(typeof v.status=="string"?v.then(Al,Al):(v.status="pending",v.then(function(w){v.status==="pending"&&(v.status="fulfilled",v.value=w)},function(w){v.status==="pending"&&(v.status="rejected",v.reason=w)})),v.status){case"fulfilled":return v.value;case"rejected":throw v.reason}}throw v}function ke(v,w,G,q,V){var ce=typeof v;(ce==="undefined"||ce==="boolean")&&(v=null);var I=!1;if(v===null)I=!0;else switch(ce){case"bigint":case"string":case"number":I=!0;break;case"object":switch(v.$$typeof){case c:case s:I=!0;break;case R:return I=v._init,ke(I(v._payload),w,G,q,V)}}if(I)return V=V(v),I=q===""?"."+Qe(v,0):q,de(V)?(G="",I!=null&&(G=I.replace(ot,"$&/")+"/"),ke(V,w,G,"",function(Ft){return Ft})):V!=null&&(be(V)&&(V=je(V,G+(V.key==null||v&&v.key===V.key?"":(""+V.key).replace(ot,"$&/")+"/")+I)),w.push(V)),1;I=0;var tt=q===""?".":q+":";if(de(v))for(var xe=0;xe<v.length;xe++)q=v[xe],ce=tt+Qe(q,xe),I+=ke(q,w,G,ce,V);else if(xe=D(v),typeof xe=="function")for(v=xe.call(v),xe=0;!(q=v.next()).done;)q=q.value,ce=tt+Qe(q,xe++),I+=ke(q,w,G,ce,V);else if(ce==="object"){if(typeof v.then=="function")return ke(Tl(v),w,G,q,V);throw w=String(v),Error("Objects are not valid as a React child (found: "+(w==="[object Object]"?"object with keys {"+Object.keys(v).join(", ")+"}":w)+"). If you meant to render a collection of children, use an array instead.")}return I}function M(v,w,G){if(v==null)return v;var q=[],V=0;return ke(v,q,"","",function(ce){return w.call(G,ce,V++)}),q}function Y(v){if(v._status===-1){var w=v._result;w=w(),w.then(function(G){(v._status===0||v._status===-1)&&(v._status=1,v._result=G)},function(G){(v._status===0||v._status===-1)&&(v._status=2,v._result=G)}),v._status===-1&&(v._status=0,v._result=w)}if(v._status===1)return v._result.default;throw v._result}var $=typeof reportError=="function"?reportError:function(v){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var w=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof v=="object"&&v!==null&&typeof v.message=="string"?String(v.message):String(v),error:v});if(!window.dispatchEvent(w))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",v);return}console.error(v)};function ge(){}return te.Children={map:M,forEach:function(v,w,G){M(v,function(){w.apply(this,arguments)},G)},count:function(v){var w=0;return M(v,function(){w++}),w},toArray:function(v){return M(v,function(w){return w})||[]},only:function(v){if(!be(v))throw Error("React.Children.only expected to receive a single React element child.");return v}},te.Component=X,te.Fragment=o,te.Profiler=h,te.PureComponent=ee,te.StrictMode=f,te.Suspense=b,te.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=F,te.__COMPILER_RUNTIME={__proto__:null,c:function(v){return F.H.useMemoCache(v)}},te.cache=function(v){return function(){return v.apply(null,arguments)}},te.cloneElement=function(v,w,G){if(v==null)throw Error("The argument must be a React element, but you passed "+v+".");var q=C({},v.props),V=v.key,ce=void 0;if(w!=null)for(I in w.ref!==void 0&&(ce=void 0),w.key!==void 0&&(V=""+w.key),w)!Me.call(w,I)||I==="key"||I==="__self"||I==="__source"||I==="ref"&&w.ref===void 0||(q[I]=w[I]);var I=arguments.length-2;if(I===1)q.children=G;else if(1<I){for(var tt=Array(I),xe=0;xe<I;xe++)tt[xe]=arguments[xe+2];q.children=tt}return Te(v.type,V,void 0,void 0,ce,q)},te.createContext=function(v){return v={$$typeof:A,_currentValue:v,_currentValue2:v,_threadCount:0,Provider:null,Consumer:null},v.Provider=v,v.Consumer={$$typeof:g,_context:v},v},te.createElement=function(v,w,G){var q,V={},ce=null;if(w!=null)for(q in w.key!==void 0&&(ce=""+w.key),w)Me.call(w,q)&&q!=="key"&&q!=="__self"&&q!=="__source"&&(V[q]=w[q]);var I=arguments.length-2;if(I===1)V.children=G;else if(1<I){for(var tt=Array(I),xe=0;xe<I;xe++)tt[xe]=arguments[xe+2];V.children=tt}if(v&&v.defaultProps)for(q in I=v.defaultProps,I)V[q]===void 0&&(V[q]=I[q]);return Te(v,ce,void 0,void 0,null,V)},te.createRef=function(){return{current:null}},te.forwardRef=function(v){return{$$typeof:z,render:v}},te.isValidElement=be,te.lazy=function(v){return{$$typeof:R,_payload:{_status:-1,_result:v},_init:Y}},te.memo=function(v,w){return{$$typeof:m,type:v,compare:w===void 0?null:w}},te.startTransition=function(v){var w=F.T,G={};F.T=G;try{var q=v(),V=F.S;V!==null&&V(G,q),typeof q=="object"&&q!==null&&typeof q.then=="function"&&q.then(ge,$)}catch(ce){$(ce)}finally{F.T=w}},te.unstable_useCacheRefresh=function(){return F.H.useCacheRefresh()},te.use=function(v){return F.H.use(v)},te.useActionState=function(v,w,G){return F.H.useActionState(v,w,G)},te.useCallback=function(v,w){return F.H.useCallback(v,w)},te.useContext=function(v){return F.H.useContext(v)},te.useDebugValue=function(){},te.useDeferredValue=function(v,w){return F.H.useDeferredValue(v,w)},te.useEffect=function(v,w,G){var q=F.H;if(typeof G=="function")throw Error("useEffect CRUD overload is not enabled in this build of React.");return q.useEffect(v,w)},te.useId=function(){return F.H.useId()},te.useImperativeHandle=function(v,w,G){return F.H.useImperativeHandle(v,w,G)},te.useInsertionEffect=function(v,w){return F.H.useInsertionEffect(v,w)},te.useLayoutEffect=function(v,w){return F.H.useLayoutEffect(v,w)},te.useMemo=function(v,w){return F.H.useMemo(v,w)},te.useOptimistic=function(v,w){return F.H.useOptimistic(v,w)},te.useReducer=function(v,w,G){return F.H.useReducer(v,w,G)},te.useRef=function(v){return F.H.useRef(v)},te.useState=function(v){return F.H.useState(v)},te.useSyncExternalStore=function(v,w,G){return F.H.useSyncExternalStore(v,w,G)},te.useTransition=function(){return F.H.useTransition()},te.version="19.1.0",te}var qd;function Dr(){return qd||(qd=1,Ar.exports=qm()),Ar.exports}var j=Dr();const Ym=Fd(j);var Tr={exports:{}},Ou={},Nr={exports:{}},Rr={};/**
 * @license React
 * scheduler.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Yd;function Lm(){return Yd||(Yd=1,function(c){function s(M,Y){var $=M.length;M.push(Y);e:for(;0<$;){var ge=$-1>>>1,v=M[ge];if(0<h(v,Y))M[ge]=Y,M[$]=v,$=ge;else break e}}function o(M){return M.length===0?null:M[0]}function f(M){if(M.length===0)return null;var Y=M[0],$=M.pop();if($!==Y){M[0]=$;e:for(var ge=0,v=M.length,w=v>>>1;ge<w;){var G=2*(ge+1)-1,q=M[G],V=G+1,ce=M[V];if(0>h(q,$))V<v&&0>h(ce,q)?(M[ge]=ce,M[V]=$,ge=V):(M[ge]=q,M[G]=$,ge=G);else if(V<v&&0>h(ce,$))M[ge]=ce,M[V]=$,ge=V;else break e}}return Y}function h(M,Y){var $=M.sortIndex-Y.sortIndex;return $!==0?$:M.id-Y.id}if(c.unstable_now=void 0,typeof performance=="object"&&typeof performance.now=="function"){var g=performance;c.unstable_now=function(){return g.now()}}else{var A=Date,z=A.now();c.unstable_now=function(){return A.now()-z}}var b=[],m=[],R=1,H=null,D=3,L=!1,C=!1,Q=!1,X=!1,B=typeof setTimeout=="function"?setTimeout:null,ee=typeof clearTimeout=="function"?clearTimeout:null,P=typeof setImmediate<"u"?setImmediate:null;function de(M){for(var Y=o(m);Y!==null;){if(Y.callback===null)f(m);else if(Y.startTime<=M)f(m),Y.sortIndex=Y.expirationTime,s(b,Y);else break;Y=o(m)}}function F(M){if(Q=!1,de(M),!C)if(o(b)!==null)C=!0,Me||(Me=!0,Qe());else{var Y=o(m);Y!==null&&ke(F,Y.startTime-M)}}var Me=!1,Te=-1,je=5,be=-1;function Ke(){return X?!0:!(c.unstable_now()-be<je)}function ot(){if(X=!1,Me){var M=c.unstable_now();be=M;var Y=!0;try{e:{C=!1,Q&&(Q=!1,ee(Te),Te=-1),L=!0;var $=D;try{t:{for(de(M),H=o(b);H!==null&&!(H.expirationTime>M&&Ke());){var ge=H.callback;if(typeof ge=="function"){H.callback=null,D=H.priorityLevel;var v=ge(H.expirationTime<=M);if(M=c.unstable_now(),typeof v=="function"){H.callback=v,de(M),Y=!0;break t}H===o(b)&&f(b),de(M)}else f(b);H=o(b)}if(H!==null)Y=!0;else{var w=o(m);w!==null&&ke(F,w.startTime-M),Y=!1}}break e}finally{H=null,D=$,L=!1}Y=void 0}}finally{Y?Qe():Me=!1}}}var Qe;if(typeof P=="function")Qe=function(){P(ot)};else if(typeof MessageChannel<"u"){var Al=new MessageChannel,Tl=Al.port2;Al.port1.onmessage=ot,Qe=function(){Tl.postMessage(null)}}else Qe=function(){B(ot,0)};function ke(M,Y){Te=B(function(){M(c.unstable_now())},Y)}c.unstable_IdlePriority=5,c.unstable_ImmediatePriority=1,c.unstable_LowPriority=4,c.unstable_NormalPriority=3,c.unstable_Profiling=null,c.unstable_UserBlockingPriority=2,c.unstable_cancelCallback=function(M){M.callback=null},c.unstable_forceFrameRate=function(M){0>M||125<M?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):je=0<M?Math.floor(1e3/M):5},c.unstable_getCurrentPriorityLevel=function(){return D},c.unstable_next=function(M){switch(D){case 1:case 2:case 3:var Y=3;break;default:Y=D}var $=D;D=Y;try{return M()}finally{D=$}},c.unstable_requestPaint=function(){X=!0},c.unstable_runWithPriority=function(M,Y){switch(M){case 1:case 2:case 3:case 4:case 5:break;default:M=3}var $=D;D=M;try{return Y()}finally{D=$}},c.unstable_scheduleCallback=function(M,Y,$){var ge=c.unstable_now();switch(typeof $=="object"&&$!==null?($=$.delay,$=typeof $=="number"&&0<$?ge+$:ge):$=ge,M){case 1:var v=-1;break;case 2:v=250;break;case 5:v=1073741823;break;case 4:v=1e4;break;default:v=5e3}return v=$+v,M={id:R++,callback:Y,priorityLevel:M,startTime:$,expirationTime:v,sortIndex:-1},$>ge?(M.sortIndex=$,s(m,M),o(b)===null&&M===o(m)&&(Q?(ee(Te),Te=-1):Q=!0,ke(F,$-ge))):(M.sortIndex=v,s(b,M),C||L||(C=!0,Me||(Me=!0,Qe()))),M},c.unstable_shouldYield=Ke,c.unstable_wrapCallback=function(M){var Y=D;return function(){var $=D;D=Y;try{return M.apply(this,arguments)}finally{D=$}}}}(Rr)),Rr}var Ld;function Gm(){return Ld||(Ld=1,Nr.exports=Lm()),Nr.exports}var jr={exports:{}},Ve={};/**
 * @license React
 * react-dom.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Gd;function Qm(){if(Gd)return Ve;Gd=1;var c=Dr();function s(b){var m="https://react.dev/errors/"+b;if(1<arguments.length){m+="?args[]="+encodeURIComponent(arguments[1]);for(var R=2;R<arguments.length;R++)m+="&args[]="+encodeURIComponent(arguments[R])}return"Minified React error #"+b+"; visit "+m+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function o(){}var f={d:{f:o,r:function(){throw Error(s(522))},D:o,C:o,L:o,m:o,X:o,S:o,M:o},p:0,findDOMNode:null},h=Symbol.for("react.portal");function g(b,m,R){var H=3<arguments.length&&arguments[3]!==void 0?arguments[3]:null;return{$$typeof:h,key:H==null?null:""+H,children:b,containerInfo:m,implementation:R}}var A=c.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE;function z(b,m){if(b==="font")return"";if(typeof m=="string")return m==="use-credentials"?m:""}return Ve.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=f,Ve.createPortal=function(b,m){var R=2<arguments.length&&arguments[2]!==void 0?arguments[2]:null;if(!m||m.nodeType!==1&&m.nodeType!==9&&m.nodeType!==11)throw Error(s(299));return g(b,m,null,R)},Ve.flushSync=function(b){var m=A.T,R=f.p;try{if(A.T=null,f.p=2,b)return b()}finally{A.T=m,f.p=R,f.d.f()}},Ve.preconnect=function(b,m){typeof b=="string"&&(m?(m=m.crossOrigin,m=typeof m=="string"?m==="use-credentials"?m:"":void 0):m=null,f.d.C(b,m))},Ve.prefetchDNS=function(b){typeof b=="string"&&f.d.D(b)},Ve.preinit=function(b,m){if(typeof b=="string"&&m&&typeof m.as=="string"){var R=m.as,H=z(R,m.crossOrigin),D=typeof m.integrity=="string"?m.integrity:void 0,L=typeof m.fetchPriority=="string"?m.fetchPriority:void 0;R==="style"?f.d.S(b,typeof m.precedence=="string"?m.precedence:void 0,{crossOrigin:H,integrity:D,fetchPriority:L}):R==="script"&&f.d.X(b,{crossOrigin:H,integrity:D,fetchPriority:L,nonce:typeof m.nonce=="string"?m.nonce:void 0})}},Ve.preinitModule=function(b,m){if(typeof b=="string")if(typeof m=="object"&&m!==null){if(m.as==null||m.as==="script"){var R=z(m.as,m.crossOrigin);f.d.M(b,{crossOrigin:R,integrity:typeof m.integrity=="string"?m.integrity:void 0,nonce:typeof m.nonce=="string"?m.nonce:void 0})}}else m==null&&f.d.M(b)},Ve.preload=function(b,m){if(typeof b=="string"&&typeof m=="object"&&m!==null&&typeof m.as=="string"){var R=m.as,H=z(R,m.crossOrigin);f.d.L(b,R,{crossOrigin:H,integrity:typeof m.integrity=="string"?m.integrity:void 0,nonce:typeof m.nonce=="string"?m.nonce:void 0,type:typeof m.type=="string"?m.type:void 0,fetchPriority:typeof m.fetchPriority=="string"?m.fetchPriority:void 0,referrerPolicy:typeof m.referrerPolicy=="string"?m.referrerPolicy:void 0,imageSrcSet:typeof m.imageSrcSet=="string"?m.imageSrcSet:void 0,imageSizes:typeof m.imageSizes=="string"?m.imageSizes:void 0,media:typeof m.media=="string"?m.media:void 0})}},Ve.preloadModule=function(b,m){if(typeof b=="string")if(m){var R=z(m.as,m.crossOrigin);f.d.m(b,{as:typeof m.as=="string"&&m.as!=="script"?m.as:void 0,crossOrigin:R,integrity:typeof m.integrity=="string"?m.integrity:void 0})}else f.d.m(b)},Ve.requestFormReset=function(b){f.d.r(b)},Ve.unstable_batchedUpdates=function(b,m){return b(m)},Ve.useFormState=function(b,m,R){return A.H.useFormState(b,m,R)},Ve.useFormStatus=function(){return A.H.useHostTransitionStatus()},Ve.version="19.1.0",Ve}var Qd;function km(){if(Qd)return jr.exports;Qd=1;function c(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(c)}catch(s){console.error(s)}}return c(),jr.exports=Qm(),jr.exports}/**
 * @license React
 * react-dom-client.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var kd;function Xm(){if(kd)return Ou;kd=1;var c=Gm(),s=Dr(),o=km();function f(e){var t="https://react.dev/errors/"+e;if(1<arguments.length){t+="?args[]="+encodeURIComponent(arguments[1]);for(var l=2;l<arguments.length;l++)t+="&args[]="+encodeURIComponent(arguments[l])}return"Minified React error #"+e+"; visit "+t+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function h(e){return!(!e||e.nodeType!==1&&e.nodeType!==9&&e.nodeType!==11)}function g(e){var t=e,l=e;if(e.alternate)for(;t.return;)t=t.return;else{e=t;do t=e,(t.flags&4098)!==0&&(l=t.return),e=t.return;while(e)}return t.tag===3?l:null}function A(e){if(e.tag===13){var t=e.memoizedState;if(t===null&&(e=e.alternate,e!==null&&(t=e.memoizedState)),t!==null)return t.dehydrated}return null}function z(e){if(g(e)!==e)throw Error(f(188))}function b(e){var t=e.alternate;if(!t){if(t=g(e),t===null)throw Error(f(188));return t!==e?null:e}for(var l=e,a=t;;){var u=l.return;if(u===null)break;var n=u.alternate;if(n===null){if(a=u.return,a!==null){l=a;continue}break}if(u.child===n.child){for(n=u.child;n;){if(n===l)return z(u),e;if(n===a)return z(u),t;n=n.sibling}throw Error(f(188))}if(l.return!==a.return)l=u,a=n;else{for(var i=!1,r=u.child;r;){if(r===l){i=!0,l=u,a=n;break}if(r===a){i=!0,a=u,l=n;break}r=r.sibling}if(!i){for(r=n.child;r;){if(r===l){i=!0,l=n,a=u;break}if(r===a){i=!0,a=n,l=u;break}r=r.sibling}if(!i)throw Error(f(189))}}if(l.alternate!==a)throw Error(f(190))}if(l.tag!==3)throw Error(f(188));return l.stateNode.current===l?e:t}function m(e){var t=e.tag;if(t===5||t===26||t===27||t===6)return e;for(e=e.child;e!==null;){if(t=m(e),t!==null)return t;e=e.sibling}return null}var R=Object.assign,H=Symbol.for("react.element"),D=Symbol.for("react.transitional.element"),L=Symbol.for("react.portal"),C=Symbol.for("react.fragment"),Q=Symbol.for("react.strict_mode"),X=Symbol.for("react.profiler"),B=Symbol.for("react.provider"),ee=Symbol.for("react.consumer"),P=Symbol.for("react.context"),de=Symbol.for("react.forward_ref"),F=Symbol.for("react.suspense"),Me=Symbol.for("react.suspense_list"),Te=Symbol.for("react.memo"),je=Symbol.for("react.lazy"),be=Symbol.for("react.activity"),Ke=Symbol.for("react.memo_cache_sentinel"),ot=Symbol.iterator;function Qe(e){return e===null||typeof e!="object"?null:(e=ot&&e[ot]||e["@@iterator"],typeof e=="function"?e:null)}var Al=Symbol.for("react.client.reference");function Tl(e){if(e==null)return null;if(typeof e=="function")return e.$$typeof===Al?null:e.displayName||e.name||null;if(typeof e=="string")return e;switch(e){case C:return"Fragment";case X:return"Profiler";case Q:return"StrictMode";case F:return"Suspense";case Me:return"SuspenseList";case be:return"Activity"}if(typeof e=="object")switch(e.$$typeof){case L:return"Portal";case P:return(e.displayName||"Context")+".Provider";case ee:return(e._context.displayName||"Context")+".Consumer";case de:var t=e.render;return e=e.displayName,e||(e=t.displayName||t.name||"",e=e!==""?"ForwardRef("+e+")":"ForwardRef"),e;case Te:return t=e.displayName||null,t!==null?t:Tl(e.type)||"Memo";case je:t=e._payload,e=e._init;try{return Tl(e(t))}catch{}}return null}var ke=Array.isArray,M=s.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,Y=o.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,$={pending:!1,data:null,method:null,action:null},ge=[],v=-1;function w(e){return{current:e}}function G(e){0>v||(e.current=ge[v],ge[v]=null,v--)}function q(e,t){v++,ge[v]=e.current,e.current=t}var V=w(null),ce=w(null),I=w(null),tt=w(null);function xe(e,t){switch(q(I,t),q(ce,e),q(V,null),t.nodeType){case 9:case 11:e=(e=t.documentElement)&&(e=e.namespaceURI)?fd(e):0;break;default:if(e=t.tagName,t=t.namespaceURI)t=fd(t),e=sd(t,e);else switch(e){case"svg":e=1;break;case"math":e=2;break;default:e=0}}G(V),q(V,e)}function Ft(){G(V),G(ce),G(I)}function ci(e){e.memoizedState!==null&&q(tt,e);var t=V.current,l=sd(t,e.type);t!==l&&(q(ce,e),q(V,l))}function Yu(e){ce.current===e&&(G(V),G(ce)),tt.current===e&&(G(tt),Au._currentValue=$)}var ri=Object.prototype.hasOwnProperty,fi=c.unstable_scheduleCallback,si=c.unstable_cancelCallback,m0=c.unstable_shouldYield,y0=c.unstable_requestPaint,Rt=c.unstable_now,g0=c.unstable_getCurrentPriorityLevel,Gr=c.unstable_ImmediatePriority,Qr=c.unstable_UserBlockingPriority,Lu=c.unstable_NormalPriority,v0=c.unstable_LowPriority,kr=c.unstable_IdlePriority,p0=c.log,b0=c.unstable_setDisableYieldValue,_a=null,lt=null;function Pt(e){if(typeof p0=="function"&&b0(e),lt&&typeof lt.setStrictMode=="function")try{lt.setStrictMode(_a,e)}catch{}}var at=Math.clz32?Math.clz32:E0,x0=Math.log,S0=Math.LN2;function E0(e){return e>>>=0,e===0?32:31-(x0(e)/S0|0)|0}var Gu=256,Qu=4194304;function Nl(e){var t=e&42;if(t!==0)return t;switch(e&-e){case 1:return 1;case 2:return 2;case 4:return 4;case 8:return 8;case 16:return 16;case 32:return 32;case 64:return 64;case 128:return 128;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return e&4194048;case 4194304:case 8388608:case 16777216:case 33554432:return e&62914560;case 67108864:return 67108864;case 134217728:return 134217728;case 268435456:return 268435456;case 536870912:return 536870912;case 1073741824:return 0;default:return e}}function ku(e,t,l){var a=e.pendingLanes;if(a===0)return 0;var u=0,n=e.suspendedLanes,i=e.pingedLanes;e=e.warmLanes;var r=a&134217727;return r!==0?(a=r&~n,a!==0?u=Nl(a):(i&=r,i!==0?u=Nl(i):l||(l=r&~e,l!==0&&(u=Nl(l))))):(r=a&~n,r!==0?u=Nl(r):i!==0?u=Nl(i):l||(l=a&~e,l!==0&&(u=Nl(l)))),u===0?0:t!==0&&t!==u&&(t&n)===0&&(n=u&-u,l=t&-t,n>=l||n===32&&(l&4194048)!==0)?t:u}function Da(e,t){return(e.pendingLanes&~(e.suspendedLanes&~e.pingedLanes)&t)===0}function A0(e,t){switch(e){case 1:case 2:case 4:case 8:case 64:return t+250;case 16:case 32:case 128:case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return t+5e3;case 4194304:case 8388608:case 16777216:case 33554432:return-1;case 67108864:case 134217728:case 268435456:case 536870912:case 1073741824:return-1;default:return-1}}function Xr(){var e=Gu;return Gu<<=1,(Gu&4194048)===0&&(Gu=256),e}function Zr(){var e=Qu;return Qu<<=1,(Qu&62914560)===0&&(Qu=4194304),e}function oi(e){for(var t=[],l=0;31>l;l++)t.push(e);return t}function Ua(e,t){e.pendingLanes|=t,t!==268435456&&(e.suspendedLanes=0,e.pingedLanes=0,e.warmLanes=0)}function T0(e,t,l,a,u,n){var i=e.pendingLanes;e.pendingLanes=l,e.suspendedLanes=0,e.pingedLanes=0,e.warmLanes=0,e.expiredLanes&=l,e.entangledLanes&=l,e.errorRecoveryDisabledLanes&=l,e.shellSuspendCounter=0;var r=e.entanglements,y=e.expirationTimes,E=e.hiddenUpdates;for(l=i&~l;0<l;){var O=31-at(l),U=1<<O;r[O]=0,y[O]=-1;var T=E[O];if(T!==null)for(E[O]=null,O=0;O<T.length;O++){var N=T[O];N!==null&&(N.lane&=-536870913)}l&=~U}a!==0&&Vr(e,a,0),n!==0&&u===0&&e.tag!==0&&(e.suspendedLanes|=n&~(i&~t))}function Vr(e,t,l){e.pendingLanes|=t,e.suspendedLanes&=~t;var a=31-at(t);e.entangledLanes|=t,e.entanglements[a]=e.entanglements[a]|1073741824|l&4194090}function Kr(e,t){var l=e.entangledLanes|=t;for(e=e.entanglements;l;){var a=31-at(l),u=1<<a;u&t|e[a]&t&&(e[a]|=t),l&=~u}}function di(e){switch(e){case 2:e=1;break;case 8:e=4;break;case 32:e=16;break;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:case 4194304:case 8388608:case 16777216:case 33554432:e=128;break;case 268435456:e=134217728;break;default:e=0}return e}function hi(e){return e&=-e,2<e?8<e?(e&134217727)!==0?32:268435456:8:2}function Jr(){var e=Y.p;return e!==0?e:(e=window.event,e===void 0?32:Od(e.type))}function N0(e,t){var l=Y.p;try{return Y.p=e,t()}finally{Y.p=l}}var It=Math.random().toString(36).slice(2),Xe="__reactFiber$"+It,$e="__reactProps$"+It,kl="__reactContainer$"+It,mi="__reactEvents$"+It,R0="__reactListeners$"+It,j0="__reactHandles$"+It,$r="__reactResources$"+It,wa="__reactMarker$"+It;function yi(e){delete e[Xe],delete e[$e],delete e[mi],delete e[R0],delete e[j0]}function Xl(e){var t=e[Xe];if(t)return t;for(var l=e.parentNode;l;){if(t=l[kl]||l[Xe]){if(l=t.alternate,t.child!==null||l!==null&&l.child!==null)for(e=md(e);e!==null;){if(l=e[Xe])return l;e=md(e)}return t}e=l,l=e.parentNode}return null}function Zl(e){if(e=e[Xe]||e[kl]){var t=e.tag;if(t===5||t===6||t===13||t===26||t===27||t===3)return e}return null}function Ca(e){var t=e.tag;if(t===5||t===26||t===27||t===6)return e.stateNode;throw Error(f(33))}function Vl(e){var t=e[$r];return t||(t=e[$r]={hoistableStyles:new Map,hoistableScripts:new Map}),t}function Ce(e){e[wa]=!0}var Wr=new Set,Fr={};function Rl(e,t){Kl(e,t),Kl(e+"Capture",t)}function Kl(e,t){for(Fr[e]=t,e=0;e<t.length;e++)Wr.add(t[e])}var z0=RegExp("^[:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD][:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*$"),Pr={},Ir={};function O0(e){return ri.call(Ir,e)?!0:ri.call(Pr,e)?!1:z0.test(e)?Ir[e]=!0:(Pr[e]=!0,!1)}function Xu(e,t,l){if(O0(t))if(l===null)e.removeAttribute(t);else{switch(typeof l){case"undefined":case"function":case"symbol":e.removeAttribute(t);return;case"boolean":var a=t.toLowerCase().slice(0,5);if(a!=="data-"&&a!=="aria-"){e.removeAttribute(t);return}}e.setAttribute(t,""+l)}}function Zu(e,t,l){if(l===null)e.removeAttribute(t);else{switch(typeof l){case"undefined":case"function":case"symbol":case"boolean":e.removeAttribute(t);return}e.setAttribute(t,""+l)}}function Ut(e,t,l,a){if(a===null)e.removeAttribute(l);else{switch(typeof a){case"undefined":case"function":case"symbol":case"boolean":e.removeAttribute(l);return}e.setAttributeNS(t,l,""+a)}}var gi,ef;function Jl(e){if(gi===void 0)try{throw Error()}catch(l){var t=l.stack.trim().match(/\n( *(at )?)/);gi=t&&t[1]||"",ef=-1<l.stack.indexOf(`
    at`)?" (<anonymous>)":-1<l.stack.indexOf("@")?"@unknown:0:0":""}return`
`+gi+e+ef}var vi=!1;function pi(e,t){if(!e||vi)return"";vi=!0;var l=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{var a={DetermineComponentFrameRoot:function(){try{if(t){var U=function(){throw Error()};if(Object.defineProperty(U.prototype,"props",{set:function(){throw Error()}}),typeof Reflect=="object"&&Reflect.construct){try{Reflect.construct(U,[])}catch(N){var T=N}Reflect.construct(e,[],U)}else{try{U.call()}catch(N){T=N}e.call(U.prototype)}}else{try{throw Error()}catch(N){T=N}(U=e())&&typeof U.catch=="function"&&U.catch(function(){})}}catch(N){if(N&&T&&typeof N.stack=="string")return[N.stack,T.stack]}return[null,null]}};a.DetermineComponentFrameRoot.displayName="DetermineComponentFrameRoot";var u=Object.getOwnPropertyDescriptor(a.DetermineComponentFrameRoot,"name");u&&u.configurable&&Object.defineProperty(a.DetermineComponentFrameRoot,"name",{value:"DetermineComponentFrameRoot"});var n=a.DetermineComponentFrameRoot(),i=n[0],r=n[1];if(i&&r){var y=i.split(`
`),E=r.split(`
`);for(u=a=0;a<y.length&&!y[a].includes("DetermineComponentFrameRoot");)a++;for(;u<E.length&&!E[u].includes("DetermineComponentFrameRoot");)u++;if(a===y.length||u===E.length)for(a=y.length-1,u=E.length-1;1<=a&&0<=u&&y[a]!==E[u];)u--;for(;1<=a&&0<=u;a--,u--)if(y[a]!==E[u]){if(a!==1||u!==1)do if(a--,u--,0>u||y[a]!==E[u]){var O=`
`+y[a].replace(" at new "," at ");return e.displayName&&O.includes("<anonymous>")&&(O=O.replace("<anonymous>",e.displayName)),O}while(1<=a&&0<=u);break}}}finally{vi=!1,Error.prepareStackTrace=l}return(l=e?e.displayName||e.name:"")?Jl(l):""}function M0(e){switch(e.tag){case 26:case 27:case 5:return Jl(e.type);case 16:return Jl("Lazy");case 13:return Jl("Suspense");case 19:return Jl("SuspenseList");case 0:case 15:return pi(e.type,!1);case 11:return pi(e.type.render,!1);case 1:return pi(e.type,!0);case 31:return Jl("Activity");default:return""}}function tf(e){try{var t="";do t+=M0(e),e=e.return;while(e);return t}catch(l){return`
Error generating stack: `+l.message+`
`+l.stack}}function dt(e){switch(typeof e){case"bigint":case"boolean":case"number":case"string":case"undefined":return e;case"object":return e;default:return""}}function lf(e){var t=e.type;return(e=e.nodeName)&&e.toLowerCase()==="input"&&(t==="checkbox"||t==="radio")}function _0(e){var t=lf(e)?"checked":"value",l=Object.getOwnPropertyDescriptor(e.constructor.prototype,t),a=""+e[t];if(!e.hasOwnProperty(t)&&typeof l<"u"&&typeof l.get=="function"&&typeof l.set=="function"){var u=l.get,n=l.set;return Object.defineProperty(e,t,{configurable:!0,get:function(){return u.call(this)},set:function(i){a=""+i,n.call(this,i)}}),Object.defineProperty(e,t,{enumerable:l.enumerable}),{getValue:function(){return a},setValue:function(i){a=""+i},stopTracking:function(){e._valueTracker=null,delete e[t]}}}}function Vu(e){e._valueTracker||(e._valueTracker=_0(e))}function af(e){if(!e)return!1;var t=e._valueTracker;if(!t)return!0;var l=t.getValue(),a="";return e&&(a=lf(e)?e.checked?"true":"false":e.value),e=a,e!==l?(t.setValue(e),!0):!1}function Ku(e){if(e=e||(typeof document<"u"?document:void 0),typeof e>"u")return null;try{return e.activeElement||e.body}catch{return e.body}}var D0=/[\n"\\]/g;function ht(e){return e.replace(D0,function(t){return"\\"+t.charCodeAt(0).toString(16)+" "})}function bi(e,t,l,a,u,n,i,r){e.name="",i!=null&&typeof i!="function"&&typeof i!="symbol"&&typeof i!="boolean"?e.type=i:e.removeAttribute("type"),t!=null?i==="number"?(t===0&&e.value===""||e.value!=t)&&(e.value=""+dt(t)):e.value!==""+dt(t)&&(e.value=""+dt(t)):i!=="submit"&&i!=="reset"||e.removeAttribute("value"),t!=null?xi(e,i,dt(t)):l!=null?xi(e,i,dt(l)):a!=null&&e.removeAttribute("value"),u==null&&n!=null&&(e.defaultChecked=!!n),u!=null&&(e.checked=u&&typeof u!="function"&&typeof u!="symbol"),r!=null&&typeof r!="function"&&typeof r!="symbol"&&typeof r!="boolean"?e.name=""+dt(r):e.removeAttribute("name")}function uf(e,t,l,a,u,n,i,r){if(n!=null&&typeof n!="function"&&typeof n!="symbol"&&typeof n!="boolean"&&(e.type=n),t!=null||l!=null){if(!(n!=="submit"&&n!=="reset"||t!=null))return;l=l!=null?""+dt(l):"",t=t!=null?""+dt(t):l,r||t===e.value||(e.value=t),e.defaultValue=t}a=a??u,a=typeof a!="function"&&typeof a!="symbol"&&!!a,e.checked=r?e.checked:!!a,e.defaultChecked=!!a,i!=null&&typeof i!="function"&&typeof i!="symbol"&&typeof i!="boolean"&&(e.name=i)}function xi(e,t,l){t==="number"&&Ku(e.ownerDocument)===e||e.defaultValue===""+l||(e.defaultValue=""+l)}function $l(e,t,l,a){if(e=e.options,t){t={};for(var u=0;u<l.length;u++)t["$"+l[u]]=!0;for(l=0;l<e.length;l++)u=t.hasOwnProperty("$"+e[l].value),e[l].selected!==u&&(e[l].selected=u),u&&a&&(e[l].defaultSelected=!0)}else{for(l=""+dt(l),t=null,u=0;u<e.length;u++){if(e[u].value===l){e[u].selected=!0,a&&(e[u].defaultSelected=!0);return}t!==null||e[u].disabled||(t=e[u])}t!==null&&(t.selected=!0)}}function nf(e,t,l){if(t!=null&&(t=""+dt(t),t!==e.value&&(e.value=t),l==null)){e.defaultValue!==t&&(e.defaultValue=t);return}e.defaultValue=l!=null?""+dt(l):""}function cf(e,t,l,a){if(t==null){if(a!=null){if(l!=null)throw Error(f(92));if(ke(a)){if(1<a.length)throw Error(f(93));a=a[0]}l=a}l==null&&(l=""),t=l}l=dt(t),e.defaultValue=l,a=e.textContent,a===l&&a!==""&&a!==null&&(e.value=a)}function Wl(e,t){if(t){var l=e.firstChild;if(l&&l===e.lastChild&&l.nodeType===3){l.nodeValue=t;return}}e.textContent=t}var U0=new Set("animationIterationCount aspectRatio borderImageOutset borderImageSlice borderImageWidth boxFlex boxFlexGroup boxOrdinalGroup columnCount columns flex flexGrow flexPositive flexShrink flexNegative flexOrder gridArea gridRow gridRowEnd gridRowSpan gridRowStart gridColumn gridColumnEnd gridColumnSpan gridColumnStart fontWeight lineClamp lineHeight opacity order orphans scale tabSize widows zIndex zoom fillOpacity floodOpacity stopOpacity strokeDasharray strokeDashoffset strokeMiterlimit strokeOpacity strokeWidth MozAnimationIterationCount MozBoxFlex MozBoxFlexGroup MozLineClamp msAnimationIterationCount msFlex msZoom msFlexGrow msFlexNegative msFlexOrder msFlexPositive msFlexShrink msGridColumn msGridColumnSpan msGridRow msGridRowSpan WebkitAnimationIterationCount WebkitBoxFlex WebKitBoxFlexGroup WebkitBoxOrdinalGroup WebkitColumnCount WebkitColumns WebkitFlex WebkitFlexGrow WebkitFlexPositive WebkitFlexShrink WebkitLineClamp".split(" "));function rf(e,t,l){var a=t.indexOf("--")===0;l==null||typeof l=="boolean"||l===""?a?e.setProperty(t,""):t==="float"?e.cssFloat="":e[t]="":a?e.setProperty(t,l):typeof l!="number"||l===0||U0.has(t)?t==="float"?e.cssFloat=l:e[t]=(""+l).trim():e[t]=l+"px"}function ff(e,t,l){if(t!=null&&typeof t!="object")throw Error(f(62));if(e=e.style,l!=null){for(var a in l)!l.hasOwnProperty(a)||t!=null&&t.hasOwnProperty(a)||(a.indexOf("--")===0?e.setProperty(a,""):a==="float"?e.cssFloat="":e[a]="");for(var u in t)a=t[u],t.hasOwnProperty(u)&&l[u]!==a&&rf(e,u,a)}else for(var n in t)t.hasOwnProperty(n)&&rf(e,n,t[n])}function Si(e){if(e.indexOf("-")===-1)return!1;switch(e){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}var w0=new Map([["acceptCharset","accept-charset"],["htmlFor","for"],["httpEquiv","http-equiv"],["crossOrigin","crossorigin"],["accentHeight","accent-height"],["alignmentBaseline","alignment-baseline"],["arabicForm","arabic-form"],["baselineShift","baseline-shift"],["capHeight","cap-height"],["clipPath","clip-path"],["clipRule","clip-rule"],["colorInterpolation","color-interpolation"],["colorInterpolationFilters","color-interpolation-filters"],["colorProfile","color-profile"],["colorRendering","color-rendering"],["dominantBaseline","dominant-baseline"],["enableBackground","enable-background"],["fillOpacity","fill-opacity"],["fillRule","fill-rule"],["floodColor","flood-color"],["floodOpacity","flood-opacity"],["fontFamily","font-family"],["fontSize","font-size"],["fontSizeAdjust","font-size-adjust"],["fontStretch","font-stretch"],["fontStyle","font-style"],["fontVariant","font-variant"],["fontWeight","font-weight"],["glyphName","glyph-name"],["glyphOrientationHorizontal","glyph-orientation-horizontal"],["glyphOrientationVertical","glyph-orientation-vertical"],["horizAdvX","horiz-adv-x"],["horizOriginX","horiz-origin-x"],["imageRendering","image-rendering"],["letterSpacing","letter-spacing"],["lightingColor","lighting-color"],["markerEnd","marker-end"],["markerMid","marker-mid"],["markerStart","marker-start"],["overlinePosition","overline-position"],["overlineThickness","overline-thickness"],["paintOrder","paint-order"],["panose-1","panose-1"],["pointerEvents","pointer-events"],["renderingIntent","rendering-intent"],["shapeRendering","shape-rendering"],["stopColor","stop-color"],["stopOpacity","stop-opacity"],["strikethroughPosition","strikethrough-position"],["strikethroughThickness","strikethrough-thickness"],["strokeDasharray","stroke-dasharray"],["strokeDashoffset","stroke-dashoffset"],["strokeLinecap","stroke-linecap"],["strokeLinejoin","stroke-linejoin"],["strokeMiterlimit","stroke-miterlimit"],["strokeOpacity","stroke-opacity"],["strokeWidth","stroke-width"],["textAnchor","text-anchor"],["textDecoration","text-decoration"],["textRendering","text-rendering"],["transformOrigin","transform-origin"],["underlinePosition","underline-position"],["underlineThickness","underline-thickness"],["unicodeBidi","unicode-bidi"],["unicodeRange","unicode-range"],["unitsPerEm","units-per-em"],["vAlphabetic","v-alphabetic"],["vHanging","v-hanging"],["vIdeographic","v-ideographic"],["vMathematical","v-mathematical"],["vectorEffect","vector-effect"],["vertAdvY","vert-adv-y"],["vertOriginX","vert-origin-x"],["vertOriginY","vert-origin-y"],["wordSpacing","word-spacing"],["writingMode","writing-mode"],["xmlnsXlink","xmlns:xlink"],["xHeight","x-height"]]),C0=/^[\u0000-\u001F ]*j[\r\n\t]*a[\r\n\t]*v[\r\n\t]*a[\r\n\t]*s[\r\n\t]*c[\r\n\t]*r[\r\n\t]*i[\r\n\t]*p[\r\n\t]*t[\r\n\t]*:/i;function Ju(e){return C0.test(""+e)?"javascript:throw new Error('React has blocked a javascript: URL as a security precaution.')":e}var Ei=null;function Ai(e){return e=e.target||e.srcElement||window,e.correspondingUseElement&&(e=e.correspondingUseElement),e.nodeType===3?e.parentNode:e}var Fl=null,Pl=null;function sf(e){var t=Zl(e);if(t&&(e=t.stateNode)){var l=e[$e]||null;e:switch(e=t.stateNode,t.type){case"input":if(bi(e,l.value,l.defaultValue,l.defaultValue,l.checked,l.defaultChecked,l.type,l.name),t=l.name,l.type==="radio"&&t!=null){for(l=e;l.parentNode;)l=l.parentNode;for(l=l.querySelectorAll('input[name="'+ht(""+t)+'"][type="radio"]'),t=0;t<l.length;t++){var a=l[t];if(a!==e&&a.form===e.form){var u=a[$e]||null;if(!u)throw Error(f(90));bi(a,u.value,u.defaultValue,u.defaultValue,u.checked,u.defaultChecked,u.type,u.name)}}for(t=0;t<l.length;t++)a=l[t],a.form===e.form&&af(a)}break e;case"textarea":nf(e,l.value,l.defaultValue);break e;case"select":t=l.value,t!=null&&$l(e,!!l.multiple,t,!1)}}}var Ti=!1;function of(e,t,l){if(Ti)return e(t,l);Ti=!0;try{var a=e(t);return a}finally{if(Ti=!1,(Fl!==null||Pl!==null)&&(wn(),Fl&&(t=Fl,e=Pl,Pl=Fl=null,sf(t),e)))for(t=0;t<e.length;t++)sf(e[t])}}function Ha(e,t){var l=e.stateNode;if(l===null)return null;var a=l[$e]||null;if(a===null)return null;l=a[t];e:switch(t){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(a=!a.disabled)||(e=e.type,a=!(e==="button"||e==="input"||e==="select"||e==="textarea")),e=!a;break e;default:e=!1}if(e)return null;if(l&&typeof l!="function")throw Error(f(231,t,typeof l));return l}var wt=!(typeof window>"u"||typeof window.document>"u"||typeof window.document.createElement>"u"),Ni=!1;if(wt)try{var Ba={};Object.defineProperty(Ba,"passive",{get:function(){Ni=!0}}),window.addEventListener("test",Ba,Ba),window.removeEventListener("test",Ba,Ba)}catch{Ni=!1}var el=null,Ri=null,$u=null;function df(){if($u)return $u;var e,t=Ri,l=t.length,a,u="value"in el?el.value:el.textContent,n=u.length;for(e=0;e<l&&t[e]===u[e];e++);var i=l-e;for(a=1;a<=i&&t[l-a]===u[n-a];a++);return $u=u.slice(e,1<a?1-a:void 0)}function Wu(e){var t=e.keyCode;return"charCode"in e?(e=e.charCode,e===0&&t===13&&(e=13)):e=t,e===10&&(e=13),32<=e||e===13?e:0}function Fu(){return!0}function hf(){return!1}function We(e){function t(l,a,u,n,i){this._reactName=l,this._targetInst=u,this.type=a,this.nativeEvent=n,this.target=i,this.currentTarget=null;for(var r in e)e.hasOwnProperty(r)&&(l=e[r],this[r]=l?l(n):n[r]);return this.isDefaultPrevented=(n.defaultPrevented!=null?n.defaultPrevented:n.returnValue===!1)?Fu:hf,this.isPropagationStopped=hf,this}return R(t.prototype,{preventDefault:function(){this.defaultPrevented=!0;var l=this.nativeEvent;l&&(l.preventDefault?l.preventDefault():typeof l.returnValue!="unknown"&&(l.returnValue=!1),this.isDefaultPrevented=Fu)},stopPropagation:function(){var l=this.nativeEvent;l&&(l.stopPropagation?l.stopPropagation():typeof l.cancelBubble!="unknown"&&(l.cancelBubble=!0),this.isPropagationStopped=Fu)},persist:function(){},isPersistent:Fu}),t}var jl={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(e){return e.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},Pu=We(jl),qa=R({},jl,{view:0,detail:0}),H0=We(qa),ji,zi,Ya,Iu=R({},qa,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:Mi,button:0,buttons:0,relatedTarget:function(e){return e.relatedTarget===void 0?e.fromElement===e.srcElement?e.toElement:e.fromElement:e.relatedTarget},movementX:function(e){return"movementX"in e?e.movementX:(e!==Ya&&(Ya&&e.type==="mousemove"?(ji=e.screenX-Ya.screenX,zi=e.screenY-Ya.screenY):zi=ji=0,Ya=e),ji)},movementY:function(e){return"movementY"in e?e.movementY:zi}}),mf=We(Iu),B0=R({},Iu,{dataTransfer:0}),q0=We(B0),Y0=R({},qa,{relatedTarget:0}),Oi=We(Y0),L0=R({},jl,{animationName:0,elapsedTime:0,pseudoElement:0}),G0=We(L0),Q0=R({},jl,{clipboardData:function(e){return"clipboardData"in e?e.clipboardData:window.clipboardData}}),k0=We(Q0),X0=R({},jl,{data:0}),yf=We(X0),Z0={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},V0={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},K0={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function J0(e){var t=this.nativeEvent;return t.getModifierState?t.getModifierState(e):(e=K0[e])?!!t[e]:!1}function Mi(){return J0}var $0=R({},qa,{key:function(e){if(e.key){var t=Z0[e.key]||e.key;if(t!=="Unidentified")return t}return e.type==="keypress"?(e=Wu(e),e===13?"Enter":String.fromCharCode(e)):e.type==="keydown"||e.type==="keyup"?V0[e.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:Mi,charCode:function(e){return e.type==="keypress"?Wu(e):0},keyCode:function(e){return e.type==="keydown"||e.type==="keyup"?e.keyCode:0},which:function(e){return e.type==="keypress"?Wu(e):e.type==="keydown"||e.type==="keyup"?e.keyCode:0}}),W0=We($0),F0=R({},Iu,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0}),gf=We(F0),P0=R({},qa,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:Mi}),I0=We(P0),eh=R({},jl,{propertyName:0,elapsedTime:0,pseudoElement:0}),th=We(eh),lh=R({},Iu,{deltaX:function(e){return"deltaX"in e?e.deltaX:"wheelDeltaX"in e?-e.wheelDeltaX:0},deltaY:function(e){return"deltaY"in e?e.deltaY:"wheelDeltaY"in e?-e.wheelDeltaY:"wheelDelta"in e?-e.wheelDelta:0},deltaZ:0,deltaMode:0}),ah=We(lh),uh=R({},jl,{newState:0,oldState:0}),nh=We(uh),ih=[9,13,27,32],_i=wt&&"CompositionEvent"in window,La=null;wt&&"documentMode"in document&&(La=document.documentMode);var ch=wt&&"TextEvent"in window&&!La,vf=wt&&(!_i||La&&8<La&&11>=La),pf=" ",bf=!1;function xf(e,t){switch(e){case"keyup":return ih.indexOf(t.keyCode)!==-1;case"keydown":return t.keyCode!==229;case"keypress":case"mousedown":case"focusout":return!0;default:return!1}}function Sf(e){return e=e.detail,typeof e=="object"&&"data"in e?e.data:null}var Il=!1;function rh(e,t){switch(e){case"compositionend":return Sf(t);case"keypress":return t.which!==32?null:(bf=!0,pf);case"textInput":return e=t.data,e===pf&&bf?null:e;default:return null}}function fh(e,t){if(Il)return e==="compositionend"||!_i&&xf(e,t)?(e=df(),$u=Ri=el=null,Il=!1,e):null;switch(e){case"paste":return null;case"keypress":if(!(t.ctrlKey||t.altKey||t.metaKey)||t.ctrlKey&&t.altKey){if(t.char&&1<t.char.length)return t.char;if(t.which)return String.fromCharCode(t.which)}return null;case"compositionend":return vf&&t.locale!=="ko"?null:t.data;default:return null}}var sh={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function Ef(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t==="input"?!!sh[e.type]:t==="textarea"}function Af(e,t,l,a){Fl?Pl?Pl.push(a):Pl=[a]:Fl=a,t=Ln(t,"onChange"),0<t.length&&(l=new Pu("onChange","change",null,l,a),e.push({event:l,listeners:t}))}var Ga=null,Qa=null;function oh(e){ud(e,0)}function en(e){var t=Ca(e);if(af(t))return e}function Tf(e,t){if(e==="change")return t}var Nf=!1;if(wt){var Di;if(wt){var Ui="oninput"in document;if(!Ui){var Rf=document.createElement("div");Rf.setAttribute("oninput","return;"),Ui=typeof Rf.oninput=="function"}Di=Ui}else Di=!1;Nf=Di&&(!document.documentMode||9<document.documentMode)}function jf(){Ga&&(Ga.detachEvent("onpropertychange",zf),Qa=Ga=null)}function zf(e){if(e.propertyName==="value"&&en(Qa)){var t=[];Af(t,Qa,e,Ai(e)),of(oh,t)}}function dh(e,t,l){e==="focusin"?(jf(),Ga=t,Qa=l,Ga.attachEvent("onpropertychange",zf)):e==="focusout"&&jf()}function hh(e){if(e==="selectionchange"||e==="keyup"||e==="keydown")return en(Qa)}function mh(e,t){if(e==="click")return en(t)}function yh(e,t){if(e==="input"||e==="change")return en(t)}function gh(e,t){return e===t&&(e!==0||1/e===1/t)||e!==e&&t!==t}var ut=typeof Object.is=="function"?Object.is:gh;function ka(e,t){if(ut(e,t))return!0;if(typeof e!="object"||e===null||typeof t!="object"||t===null)return!1;var l=Object.keys(e),a=Object.keys(t);if(l.length!==a.length)return!1;for(a=0;a<l.length;a++){var u=l[a];if(!ri.call(t,u)||!ut(e[u],t[u]))return!1}return!0}function Of(e){for(;e&&e.firstChild;)e=e.firstChild;return e}function Mf(e,t){var l=Of(e);e=0;for(var a;l;){if(l.nodeType===3){if(a=e+l.textContent.length,e<=t&&a>=t)return{node:l,offset:t-e};e=a}e:{for(;l;){if(l.nextSibling){l=l.nextSibling;break e}l=l.parentNode}l=void 0}l=Of(l)}}function _f(e,t){return e&&t?e===t?!0:e&&e.nodeType===3?!1:t&&t.nodeType===3?_f(e,t.parentNode):"contains"in e?e.contains(t):e.compareDocumentPosition?!!(e.compareDocumentPosition(t)&16):!1:!1}function Df(e){e=e!=null&&e.ownerDocument!=null&&e.ownerDocument.defaultView!=null?e.ownerDocument.defaultView:window;for(var t=Ku(e.document);t instanceof e.HTMLIFrameElement;){try{var l=typeof t.contentWindow.location.href=="string"}catch{l=!1}if(l)e=t.contentWindow;else break;t=Ku(e.document)}return t}function wi(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t&&(t==="input"&&(e.type==="text"||e.type==="search"||e.type==="tel"||e.type==="url"||e.type==="password")||t==="textarea"||e.contentEditable==="true")}var vh=wt&&"documentMode"in document&&11>=document.documentMode,ea=null,Ci=null,Xa=null,Hi=!1;function Uf(e,t,l){var a=l.window===l?l.document:l.nodeType===9?l:l.ownerDocument;Hi||ea==null||ea!==Ku(a)||(a=ea,"selectionStart"in a&&wi(a)?a={start:a.selectionStart,end:a.selectionEnd}:(a=(a.ownerDocument&&a.ownerDocument.defaultView||window).getSelection(),a={anchorNode:a.anchorNode,anchorOffset:a.anchorOffset,focusNode:a.focusNode,focusOffset:a.focusOffset}),Xa&&ka(Xa,a)||(Xa=a,a=Ln(Ci,"onSelect"),0<a.length&&(t=new Pu("onSelect","select",null,t,l),e.push({event:t,listeners:a}),t.target=ea)))}function zl(e,t){var l={};return l[e.toLowerCase()]=t.toLowerCase(),l["Webkit"+e]="webkit"+t,l["Moz"+e]="moz"+t,l}var ta={animationend:zl("Animation","AnimationEnd"),animationiteration:zl("Animation","AnimationIteration"),animationstart:zl("Animation","AnimationStart"),transitionrun:zl("Transition","TransitionRun"),transitionstart:zl("Transition","TransitionStart"),transitioncancel:zl("Transition","TransitionCancel"),transitionend:zl("Transition","TransitionEnd")},Bi={},wf={};wt&&(wf=document.createElement("div").style,"AnimationEvent"in window||(delete ta.animationend.animation,delete ta.animationiteration.animation,delete ta.animationstart.animation),"TransitionEvent"in window||delete ta.transitionend.transition);function Ol(e){if(Bi[e])return Bi[e];if(!ta[e])return e;var t=ta[e],l;for(l in t)if(t.hasOwnProperty(l)&&l in wf)return Bi[e]=t[l];return e}var Cf=Ol("animationend"),Hf=Ol("animationiteration"),Bf=Ol("animationstart"),ph=Ol("transitionrun"),bh=Ol("transitionstart"),xh=Ol("transitioncancel"),qf=Ol("transitionend"),Yf=new Map,qi="abort auxClick beforeToggle cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll toggle touchMove waiting wheel".split(" ");qi.push("scrollEnd");function St(e,t){Yf.set(e,t),Rl(t,[e])}var Lf=new WeakMap;function mt(e,t){if(typeof e=="object"&&e!==null){var l=Lf.get(e);return l!==void 0?l:(t={value:e,source:t,stack:tf(t)},Lf.set(e,t),t)}return{value:e,source:t,stack:tf(t)}}var yt=[],la=0,Yi=0;function tn(){for(var e=la,t=Yi=la=0;t<e;){var l=yt[t];yt[t++]=null;var a=yt[t];yt[t++]=null;var u=yt[t];yt[t++]=null;var n=yt[t];if(yt[t++]=null,a!==null&&u!==null){var i=a.pending;i===null?u.next=u:(u.next=i.next,i.next=u),a.pending=u}n!==0&&Gf(l,u,n)}}function ln(e,t,l,a){yt[la++]=e,yt[la++]=t,yt[la++]=l,yt[la++]=a,Yi|=a,e.lanes|=a,e=e.alternate,e!==null&&(e.lanes|=a)}function Li(e,t,l,a){return ln(e,t,l,a),an(e)}function aa(e,t){return ln(e,null,null,t),an(e)}function Gf(e,t,l){e.lanes|=l;var a=e.alternate;a!==null&&(a.lanes|=l);for(var u=!1,n=e.return;n!==null;)n.childLanes|=l,a=n.alternate,a!==null&&(a.childLanes|=l),n.tag===22&&(e=n.stateNode,e===null||e._visibility&1||(u=!0)),e=n,n=n.return;return e.tag===3?(n=e.stateNode,u&&t!==null&&(u=31-at(l),e=n.hiddenUpdates,a=e[u],a===null?e[u]=[t]:a.push(t),t.lane=l|536870912),n):null}function an(e){if(50<yu)throw yu=0,Vc=null,Error(f(185));for(var t=e.return;t!==null;)e=t,t=e.return;return e.tag===3?e.stateNode:null}var ua={};function Sh(e,t,l,a){this.tag=e,this.key=l,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.refCleanup=this.ref=null,this.pendingProps=t,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=a,this.subtreeFlags=this.flags=0,this.deletions=null,this.childLanes=this.lanes=0,this.alternate=null}function nt(e,t,l,a){return new Sh(e,t,l,a)}function Gi(e){return e=e.prototype,!(!e||!e.isReactComponent)}function Ct(e,t){var l=e.alternate;return l===null?(l=nt(e.tag,t,e.key,e.mode),l.elementType=e.elementType,l.type=e.type,l.stateNode=e.stateNode,l.alternate=e,e.alternate=l):(l.pendingProps=t,l.type=e.type,l.flags=0,l.subtreeFlags=0,l.deletions=null),l.flags=e.flags&65011712,l.childLanes=e.childLanes,l.lanes=e.lanes,l.child=e.child,l.memoizedProps=e.memoizedProps,l.memoizedState=e.memoizedState,l.updateQueue=e.updateQueue,t=e.dependencies,l.dependencies=t===null?null:{lanes:t.lanes,firstContext:t.firstContext},l.sibling=e.sibling,l.index=e.index,l.ref=e.ref,l.refCleanup=e.refCleanup,l}function Qf(e,t){e.flags&=65011714;var l=e.alternate;return l===null?(e.childLanes=0,e.lanes=t,e.child=null,e.subtreeFlags=0,e.memoizedProps=null,e.memoizedState=null,e.updateQueue=null,e.dependencies=null,e.stateNode=null):(e.childLanes=l.childLanes,e.lanes=l.lanes,e.child=l.child,e.subtreeFlags=0,e.deletions=null,e.memoizedProps=l.memoizedProps,e.memoizedState=l.memoizedState,e.updateQueue=l.updateQueue,e.type=l.type,t=l.dependencies,e.dependencies=t===null?null:{lanes:t.lanes,firstContext:t.firstContext}),e}function un(e,t,l,a,u,n){var i=0;if(a=e,typeof e=="function")Gi(e)&&(i=1);else if(typeof e=="string")i=Am(e,l,V.current)?26:e==="html"||e==="head"||e==="body"?27:5;else e:switch(e){case be:return e=nt(31,l,t,u),e.elementType=be,e.lanes=n,e;case C:return Ml(l.children,u,n,t);case Q:i=8,u|=24;break;case X:return e=nt(12,l,t,u|2),e.elementType=X,e.lanes=n,e;case F:return e=nt(13,l,t,u),e.elementType=F,e.lanes=n,e;case Me:return e=nt(19,l,t,u),e.elementType=Me,e.lanes=n,e;default:if(typeof e=="object"&&e!==null)switch(e.$$typeof){case B:case P:i=10;break e;case ee:i=9;break e;case de:i=11;break e;case Te:i=14;break e;case je:i=16,a=null;break e}i=29,l=Error(f(130,e===null?"null":typeof e,"")),a=null}return t=nt(i,l,t,u),t.elementType=e,t.type=a,t.lanes=n,t}function Ml(e,t,l,a){return e=nt(7,e,a,t),e.lanes=l,e}function Qi(e,t,l){return e=nt(6,e,null,t),e.lanes=l,e}function ki(e,t,l){return t=nt(4,e.children!==null?e.children:[],e.key,t),t.lanes=l,t.stateNode={containerInfo:e.containerInfo,pendingChildren:null,implementation:e.implementation},t}var na=[],ia=0,nn=null,cn=0,gt=[],vt=0,_l=null,Ht=1,Bt="";function Dl(e,t){na[ia++]=cn,na[ia++]=nn,nn=e,cn=t}function kf(e,t,l){gt[vt++]=Ht,gt[vt++]=Bt,gt[vt++]=_l,_l=e;var a=Ht;e=Bt;var u=32-at(a)-1;a&=~(1<<u),l+=1;var n=32-at(t)+u;if(30<n){var i=u-u%5;n=(a&(1<<i)-1).toString(32),a>>=i,u-=i,Ht=1<<32-at(t)+u|l<<u|a,Bt=n+e}else Ht=1<<n|l<<u|a,Bt=e}function Xi(e){e.return!==null&&(Dl(e,1),kf(e,1,0))}function Zi(e){for(;e===nn;)nn=na[--ia],na[ia]=null,cn=na[--ia],na[ia]=null;for(;e===_l;)_l=gt[--vt],gt[vt]=null,Bt=gt[--vt],gt[vt]=null,Ht=gt[--vt],gt[vt]=null}var Je=null,Ne=null,fe=!1,Ul=null,jt=!1,Vi=Error(f(519));function wl(e){var t=Error(f(418,""));throw Ka(mt(t,e)),Vi}function Xf(e){var t=e.stateNode,l=e.type,a=e.memoizedProps;switch(t[Xe]=e,t[$e]=a,l){case"dialog":ne("cancel",t),ne("close",t);break;case"iframe":case"object":case"embed":ne("load",t);break;case"video":case"audio":for(l=0;l<vu.length;l++)ne(vu[l],t);break;case"source":ne("error",t);break;case"img":case"image":case"link":ne("error",t),ne("load",t);break;case"details":ne("toggle",t);break;case"input":ne("invalid",t),uf(t,a.value,a.defaultValue,a.checked,a.defaultChecked,a.type,a.name,!0),Vu(t);break;case"select":ne("invalid",t);break;case"textarea":ne("invalid",t),cf(t,a.value,a.defaultValue,a.children),Vu(t)}l=a.children,typeof l!="string"&&typeof l!="number"&&typeof l!="bigint"||t.textContent===""+l||a.suppressHydrationWarning===!0||rd(t.textContent,l)?(a.popover!=null&&(ne("beforetoggle",t),ne("toggle",t)),a.onScroll!=null&&ne("scroll",t),a.onScrollEnd!=null&&ne("scrollend",t),a.onClick!=null&&(t.onclick=Gn),t=!0):t=!1,t||wl(e)}function Zf(e){for(Je=e.return;Je;)switch(Je.tag){case 5:case 13:jt=!1;return;case 27:case 3:jt=!0;return;default:Je=Je.return}}function Za(e){if(e!==Je)return!1;if(!fe)return Zf(e),fe=!0,!1;var t=e.tag,l;if((l=t!==3&&t!==27)&&((l=t===5)&&(l=e.type,l=!(l!=="form"&&l!=="button")||rr(e.type,e.memoizedProps)),l=!l),l&&Ne&&wl(e),Zf(e),t===13){if(e=e.memoizedState,e=e!==null?e.dehydrated:null,!e)throw Error(f(317));e:{for(e=e.nextSibling,t=0;e;){if(e.nodeType===8)if(l=e.data,l==="/$"){if(t===0){Ne=At(e.nextSibling);break e}t--}else l!=="$"&&l!=="$!"&&l!=="$?"||t++;e=e.nextSibling}Ne=null}}else t===27?(t=Ne,gl(e.type)?(e=dr,dr=null,Ne=e):Ne=t):Ne=Je?At(e.stateNode.nextSibling):null;return!0}function Va(){Ne=Je=null,fe=!1}function Vf(){var e=Ul;return e!==null&&(Ie===null?Ie=e:Ie.push.apply(Ie,e),Ul=null),e}function Ka(e){Ul===null?Ul=[e]:Ul.push(e)}var Ki=w(null),Cl=null,qt=null;function tl(e,t,l){q(Ki,t._currentValue),t._currentValue=l}function Yt(e){e._currentValue=Ki.current,G(Ki)}function Ji(e,t,l){for(;e!==null;){var a=e.alternate;if((e.childLanes&t)!==t?(e.childLanes|=t,a!==null&&(a.childLanes|=t)):a!==null&&(a.childLanes&t)!==t&&(a.childLanes|=t),e===l)break;e=e.return}}function $i(e,t,l,a){var u=e.child;for(u!==null&&(u.return=e);u!==null;){var n=u.dependencies;if(n!==null){var i=u.child;n=n.firstContext;e:for(;n!==null;){var r=n;n=u;for(var y=0;y<t.length;y++)if(r.context===t[y]){n.lanes|=l,r=n.alternate,r!==null&&(r.lanes|=l),Ji(n.return,l,e),a||(i=null);break e}n=r.next}}else if(u.tag===18){if(i=u.return,i===null)throw Error(f(341));i.lanes|=l,n=i.alternate,n!==null&&(n.lanes|=l),Ji(i,l,e),i=null}else i=u.child;if(i!==null)i.return=u;else for(i=u;i!==null;){if(i===e){i=null;break}if(u=i.sibling,u!==null){u.return=i.return,i=u;break}i=i.return}u=i}}function Ja(e,t,l,a){e=null;for(var u=t,n=!1;u!==null;){if(!n){if((u.flags&524288)!==0)n=!0;else if((u.flags&262144)!==0)break}if(u.tag===10){var i=u.alternate;if(i===null)throw Error(f(387));if(i=i.memoizedProps,i!==null){var r=u.type;ut(u.pendingProps.value,i.value)||(e!==null?e.push(r):e=[r])}}else if(u===tt.current){if(i=u.alternate,i===null)throw Error(f(387));i.memoizedState.memoizedState!==u.memoizedState.memoizedState&&(e!==null?e.push(Au):e=[Au])}u=u.return}e!==null&&$i(t,e,l,a),t.flags|=262144}function rn(e){for(e=e.firstContext;e!==null;){if(!ut(e.context._currentValue,e.memoizedValue))return!0;e=e.next}return!1}function Hl(e){Cl=e,qt=null,e=e.dependencies,e!==null&&(e.firstContext=null)}function Ze(e){return Kf(Cl,e)}function fn(e,t){return Cl===null&&Hl(e),Kf(e,t)}function Kf(e,t){var l=t._currentValue;if(t={context:t,memoizedValue:l,next:null},qt===null){if(e===null)throw Error(f(308));qt=t,e.dependencies={lanes:0,firstContext:t},e.flags|=524288}else qt=qt.next=t;return l}var Eh=typeof AbortController<"u"?AbortController:function(){var e=[],t=this.signal={aborted:!1,addEventListener:function(l,a){e.push(a)}};this.abort=function(){t.aborted=!0,e.forEach(function(l){return l()})}},Ah=c.unstable_scheduleCallback,Th=c.unstable_NormalPriority,Ue={$$typeof:P,Consumer:null,Provider:null,_currentValue:null,_currentValue2:null,_threadCount:0};function Wi(){return{controller:new Eh,data:new Map,refCount:0}}function $a(e){e.refCount--,e.refCount===0&&Ah(Th,function(){e.controller.abort()})}var Wa=null,Fi=0,ca=0,ra=null;function Nh(e,t){if(Wa===null){var l=Wa=[];Fi=0,ca=Ic(),ra={status:"pending",value:void 0,then:function(a){l.push(a)}}}return Fi++,t.then(Jf,Jf),t}function Jf(){if(--Fi===0&&Wa!==null){ra!==null&&(ra.status="fulfilled");var e=Wa;Wa=null,ca=0,ra=null;for(var t=0;t<e.length;t++)(0,e[t])()}}function Rh(e,t){var l=[],a={status:"pending",value:null,reason:null,then:function(u){l.push(u)}};return e.then(function(){a.status="fulfilled",a.value=t;for(var u=0;u<l.length;u++)(0,l[u])(t)},function(u){for(a.status="rejected",a.reason=u,u=0;u<l.length;u++)(0,l[u])(void 0)}),a}var $f=M.S;M.S=function(e,t){typeof t=="object"&&t!==null&&typeof t.then=="function"&&Nh(e,t),$f!==null&&$f(e,t)};var Bl=w(null);function Pi(){var e=Bl.current;return e!==null?e:pe.pooledCache}function sn(e,t){t===null?q(Bl,Bl.current):q(Bl,t.pool)}function Wf(){var e=Pi();return e===null?null:{parent:Ue._currentValue,pool:e}}var Fa=Error(f(460)),Ff=Error(f(474)),on=Error(f(542)),Ii={then:function(){}};function Pf(e){return e=e.status,e==="fulfilled"||e==="rejected"}function dn(){}function If(e,t,l){switch(l=e[l],l===void 0?e.push(t):l!==t&&(t.then(dn,dn),t=l),t.status){case"fulfilled":return t.value;case"rejected":throw e=t.reason,ts(e),e;default:if(typeof t.status=="string")t.then(dn,dn);else{if(e=pe,e!==null&&100<e.shellSuspendCounter)throw Error(f(482));e=t,e.status="pending",e.then(function(a){if(t.status==="pending"){var u=t;u.status="fulfilled",u.value=a}},function(a){if(t.status==="pending"){var u=t;u.status="rejected",u.reason=a}})}switch(t.status){case"fulfilled":return t.value;case"rejected":throw e=t.reason,ts(e),e}throw Pa=t,Fa}}var Pa=null;function es(){if(Pa===null)throw Error(f(459));var e=Pa;return Pa=null,e}function ts(e){if(e===Fa||e===on)throw Error(f(483))}var ll=!1;function ec(e){e.updateQueue={baseState:e.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null,lanes:0,hiddenCallbacks:null},callbacks:null}}function tc(e,t){e=e.updateQueue,t.updateQueue===e&&(t.updateQueue={baseState:e.baseState,firstBaseUpdate:e.firstBaseUpdate,lastBaseUpdate:e.lastBaseUpdate,shared:e.shared,callbacks:null})}function al(e){return{lane:e,tag:0,payload:null,callback:null,next:null}}function ul(e,t,l){var a=e.updateQueue;if(a===null)return null;if(a=a.shared,(se&2)!==0){var u=a.pending;return u===null?t.next=t:(t.next=u.next,u.next=t),a.pending=t,t=an(e),Gf(e,null,l),t}return ln(e,a,t,l),an(e)}function Ia(e,t,l){if(t=t.updateQueue,t!==null&&(t=t.shared,(l&4194048)!==0)){var a=t.lanes;a&=e.pendingLanes,l|=a,t.lanes=l,Kr(e,l)}}function lc(e,t){var l=e.updateQueue,a=e.alternate;if(a!==null&&(a=a.updateQueue,l===a)){var u=null,n=null;if(l=l.firstBaseUpdate,l!==null){do{var i={lane:l.lane,tag:l.tag,payload:l.payload,callback:null,next:null};n===null?u=n=i:n=n.next=i,l=l.next}while(l!==null);n===null?u=n=t:n=n.next=t}else u=n=t;l={baseState:a.baseState,firstBaseUpdate:u,lastBaseUpdate:n,shared:a.shared,callbacks:a.callbacks},e.updateQueue=l;return}e=l.lastBaseUpdate,e===null?l.firstBaseUpdate=t:e.next=t,l.lastBaseUpdate=t}var ac=!1;function eu(){if(ac){var e=ra;if(e!==null)throw e}}function tu(e,t,l,a){ac=!1;var u=e.updateQueue;ll=!1;var n=u.firstBaseUpdate,i=u.lastBaseUpdate,r=u.shared.pending;if(r!==null){u.shared.pending=null;var y=r,E=y.next;y.next=null,i===null?n=E:i.next=E,i=y;var O=e.alternate;O!==null&&(O=O.updateQueue,r=O.lastBaseUpdate,r!==i&&(r===null?O.firstBaseUpdate=E:r.next=E,O.lastBaseUpdate=y))}if(n!==null){var U=u.baseState;i=0,O=E=y=null,r=n;do{var T=r.lane&-536870913,N=T!==r.lane;if(N?(ie&T)===T:(a&T)===T){T!==0&&T===ca&&(ac=!0),O!==null&&(O=O.next={lane:0,tag:r.tag,payload:r.payload,callback:null,next:null});e:{var W=e,K=r;T=t;var ye=l;switch(K.tag){case 1:if(W=K.payload,typeof W=="function"){U=W.call(ye,U,T);break e}U=W;break e;case 3:W.flags=W.flags&-65537|128;case 0:if(W=K.payload,T=typeof W=="function"?W.call(ye,U,T):W,T==null)break e;U=R({},U,T);break e;case 2:ll=!0}}T=r.callback,T!==null&&(e.flags|=64,N&&(e.flags|=8192),N=u.callbacks,N===null?u.callbacks=[T]:N.push(T))}else N={lane:T,tag:r.tag,payload:r.payload,callback:r.callback,next:null},O===null?(E=O=N,y=U):O=O.next=N,i|=T;if(r=r.next,r===null){if(r=u.shared.pending,r===null)break;N=r,r=N.next,N.next=null,u.lastBaseUpdate=N,u.shared.pending=null}}while(!0);O===null&&(y=U),u.baseState=y,u.firstBaseUpdate=E,u.lastBaseUpdate=O,n===null&&(u.shared.lanes=0),dl|=i,e.lanes=i,e.memoizedState=U}}function ls(e,t){if(typeof e!="function")throw Error(f(191,e));e.call(t)}function as(e,t){var l=e.callbacks;if(l!==null)for(e.callbacks=null,e=0;e<l.length;e++)ls(l[e],t)}var fa=w(null),hn=w(0);function us(e,t){e=Vt,q(hn,e),q(fa,t),Vt=e|t.baseLanes}function uc(){q(hn,Vt),q(fa,fa.current)}function nc(){Vt=hn.current,G(fa),G(hn)}var nl=0,le=null,he=null,_e=null,mn=!1,sa=!1,ql=!1,yn=0,lu=0,oa=null,jh=0;function ze(){throw Error(f(321))}function ic(e,t){if(t===null)return!1;for(var l=0;l<t.length&&l<e.length;l++)if(!ut(e[l],t[l]))return!1;return!0}function cc(e,t,l,a,u,n){return nl=n,le=t,t.memoizedState=null,t.updateQueue=null,t.lanes=0,M.H=e===null||e.memoizedState===null?Gs:Qs,ql=!1,n=l(a,u),ql=!1,sa&&(n=is(t,l,a,u)),ns(e),n}function ns(e){M.H=Sn;var t=he!==null&&he.next!==null;if(nl=0,_e=he=le=null,mn=!1,lu=0,oa=null,t)throw Error(f(300));e===null||He||(e=e.dependencies,e!==null&&rn(e)&&(He=!0))}function is(e,t,l,a){le=e;var u=0;do{if(sa&&(oa=null),lu=0,sa=!1,25<=u)throw Error(f(301));if(u+=1,_e=he=null,e.updateQueue!=null){var n=e.updateQueue;n.lastEffect=null,n.events=null,n.stores=null,n.memoCache!=null&&(n.memoCache.index=0)}M.H=wh,n=t(l,a)}while(sa);return n}function zh(){var e=M.H,t=e.useState()[0];return t=typeof t.then=="function"?au(t):t,e=e.useState()[0],(he!==null?he.memoizedState:null)!==e&&(le.flags|=1024),t}function rc(){var e=yn!==0;return yn=0,e}function fc(e,t,l){t.updateQueue=e.updateQueue,t.flags&=-2053,e.lanes&=~l}function sc(e){if(mn){for(e=e.memoizedState;e!==null;){var t=e.queue;t!==null&&(t.pending=null),e=e.next}mn=!1}nl=0,_e=he=le=null,sa=!1,lu=yn=0,oa=null}function Fe(){var e={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return _e===null?le.memoizedState=_e=e:_e=_e.next=e,_e}function De(){if(he===null){var e=le.alternate;e=e!==null?e.memoizedState:null}else e=he.next;var t=_e===null?le.memoizedState:_e.next;if(t!==null)_e=t,he=e;else{if(e===null)throw le.alternate===null?Error(f(467)):Error(f(310));he=e,e={memoizedState:he.memoizedState,baseState:he.baseState,baseQueue:he.baseQueue,queue:he.queue,next:null},_e===null?le.memoizedState=_e=e:_e=_e.next=e}return _e}function oc(){return{lastEffect:null,events:null,stores:null,memoCache:null}}function au(e){var t=lu;return lu+=1,oa===null&&(oa=[]),e=If(oa,e,t),t=le,(_e===null?t.memoizedState:_e.next)===null&&(t=t.alternate,M.H=t===null||t.memoizedState===null?Gs:Qs),e}function gn(e){if(e!==null&&typeof e=="object"){if(typeof e.then=="function")return au(e);if(e.$$typeof===P)return Ze(e)}throw Error(f(438,String(e)))}function dc(e){var t=null,l=le.updateQueue;if(l!==null&&(t=l.memoCache),t==null){var a=le.alternate;a!==null&&(a=a.updateQueue,a!==null&&(a=a.memoCache,a!=null&&(t={data:a.data.map(function(u){return u.slice()}),index:0})))}if(t==null&&(t={data:[],index:0}),l===null&&(l=oc(),le.updateQueue=l),l.memoCache=t,l=t.data[t.index],l===void 0)for(l=t.data[t.index]=Array(e),a=0;a<e;a++)l[a]=Ke;return t.index++,l}function Lt(e,t){return typeof t=="function"?t(e):t}function vn(e){var t=De();return hc(t,he,e)}function hc(e,t,l){var a=e.queue;if(a===null)throw Error(f(311));a.lastRenderedReducer=l;var u=e.baseQueue,n=a.pending;if(n!==null){if(u!==null){var i=u.next;u.next=n.next,n.next=i}t.baseQueue=u=n,a.pending=null}if(n=e.baseState,u===null)e.memoizedState=n;else{t=u.next;var r=i=null,y=null,E=t,O=!1;do{var U=E.lane&-536870913;if(U!==E.lane?(ie&U)===U:(nl&U)===U){var T=E.revertLane;if(T===0)y!==null&&(y=y.next={lane:0,revertLane:0,action:E.action,hasEagerState:E.hasEagerState,eagerState:E.eagerState,next:null}),U===ca&&(O=!0);else if((nl&T)===T){E=E.next,T===ca&&(O=!0);continue}else U={lane:0,revertLane:E.revertLane,action:E.action,hasEagerState:E.hasEagerState,eagerState:E.eagerState,next:null},y===null?(r=y=U,i=n):y=y.next=U,le.lanes|=T,dl|=T;U=E.action,ql&&l(n,U),n=E.hasEagerState?E.eagerState:l(n,U)}else T={lane:U,revertLane:E.revertLane,action:E.action,hasEagerState:E.hasEagerState,eagerState:E.eagerState,next:null},y===null?(r=y=T,i=n):y=y.next=T,le.lanes|=U,dl|=U;E=E.next}while(E!==null&&E!==t);if(y===null?i=n:y.next=r,!ut(n,e.memoizedState)&&(He=!0,O&&(l=ra,l!==null)))throw l;e.memoizedState=n,e.baseState=i,e.baseQueue=y,a.lastRenderedState=n}return u===null&&(a.lanes=0),[e.memoizedState,a.dispatch]}function mc(e){var t=De(),l=t.queue;if(l===null)throw Error(f(311));l.lastRenderedReducer=e;var a=l.dispatch,u=l.pending,n=t.memoizedState;if(u!==null){l.pending=null;var i=u=u.next;do n=e(n,i.action),i=i.next;while(i!==u);ut(n,t.memoizedState)||(He=!0),t.memoizedState=n,t.baseQueue===null&&(t.baseState=n),l.lastRenderedState=n}return[n,a]}function cs(e,t,l){var a=le,u=De(),n=fe;if(n){if(l===void 0)throw Error(f(407));l=l()}else l=t();var i=!ut((he||u).memoizedState,l);i&&(u.memoizedState=l,He=!0),u=u.queue;var r=ss.bind(null,a,u,e);if(uu(2048,8,r,[e]),u.getSnapshot!==t||i||_e!==null&&_e.memoizedState.tag&1){if(a.flags|=2048,da(9,pn(),fs.bind(null,a,u,l,t),null),pe===null)throw Error(f(349));n||(nl&124)!==0||rs(a,t,l)}return l}function rs(e,t,l){e.flags|=16384,e={getSnapshot:t,value:l},t=le.updateQueue,t===null?(t=oc(),le.updateQueue=t,t.stores=[e]):(l=t.stores,l===null?t.stores=[e]:l.push(e))}function fs(e,t,l,a){t.value=l,t.getSnapshot=a,os(t)&&ds(e)}function ss(e,t,l){return l(function(){os(t)&&ds(e)})}function os(e){var t=e.getSnapshot;e=e.value;try{var l=t();return!ut(e,l)}catch{return!0}}function ds(e){var t=aa(e,2);t!==null&&st(t,e,2)}function yc(e){var t=Fe();if(typeof e=="function"){var l=e;if(e=l(),ql){Pt(!0);try{l()}finally{Pt(!1)}}}return t.memoizedState=t.baseState=e,t.queue={pending:null,lanes:0,dispatch:null,lastRenderedReducer:Lt,lastRenderedState:e},t}function hs(e,t,l,a){return e.baseState=l,hc(e,he,typeof a=="function"?a:Lt)}function Oh(e,t,l,a,u){if(xn(e))throw Error(f(485));if(e=t.action,e!==null){var n={payload:u,action:e,next:null,isTransition:!0,status:"pending",value:null,reason:null,listeners:[],then:function(i){n.listeners.push(i)}};M.T!==null?l(!0):n.isTransition=!1,a(n),l=t.pending,l===null?(n.next=t.pending=n,ms(t,n)):(n.next=l.next,t.pending=l.next=n)}}function ms(e,t){var l=t.action,a=t.payload,u=e.state;if(t.isTransition){var n=M.T,i={};M.T=i;try{var r=l(u,a),y=M.S;y!==null&&y(i,r),ys(e,t,r)}catch(E){gc(e,t,E)}finally{M.T=n}}else try{n=l(u,a),ys(e,t,n)}catch(E){gc(e,t,E)}}function ys(e,t,l){l!==null&&typeof l=="object"&&typeof l.then=="function"?l.then(function(a){gs(e,t,a)},function(a){return gc(e,t,a)}):gs(e,t,l)}function gs(e,t,l){t.status="fulfilled",t.value=l,vs(t),e.state=l,t=e.pending,t!==null&&(l=t.next,l===t?e.pending=null:(l=l.next,t.next=l,ms(e,l)))}function gc(e,t,l){var a=e.pending;if(e.pending=null,a!==null){a=a.next;do t.status="rejected",t.reason=l,vs(t),t=t.next;while(t!==a)}e.action=null}function vs(e){e=e.listeners;for(var t=0;t<e.length;t++)(0,e[t])()}function ps(e,t){return t}function bs(e,t){if(fe){var l=pe.formState;if(l!==null){e:{var a=le;if(fe){if(Ne){t:{for(var u=Ne,n=jt;u.nodeType!==8;){if(!n){u=null;break t}if(u=At(u.nextSibling),u===null){u=null;break t}}n=u.data,u=n==="F!"||n==="F"?u:null}if(u){Ne=At(u.nextSibling),a=u.data==="F!";break e}}wl(a)}a=!1}a&&(t=l[0])}}return l=Fe(),l.memoizedState=l.baseState=t,a={pending:null,lanes:0,dispatch:null,lastRenderedReducer:ps,lastRenderedState:t},l.queue=a,l=qs.bind(null,le,a),a.dispatch=l,a=yc(!1),n=Sc.bind(null,le,!1,a.queue),a=Fe(),u={state:t,dispatch:null,action:e,pending:null},a.queue=u,l=Oh.bind(null,le,u,n,l),u.dispatch=l,a.memoizedState=e,[t,l,!1]}function xs(e){var t=De();return Ss(t,he,e)}function Ss(e,t,l){if(t=hc(e,t,ps)[0],e=vn(Lt)[0],typeof t=="object"&&t!==null&&typeof t.then=="function")try{var a=au(t)}catch(i){throw i===Fa?on:i}else a=t;t=De();var u=t.queue,n=u.dispatch;return l!==t.memoizedState&&(le.flags|=2048,da(9,pn(),Mh.bind(null,u,l),null)),[a,n,e]}function Mh(e,t){e.action=t}function Es(e){var t=De(),l=he;if(l!==null)return Ss(t,l,e);De(),t=t.memoizedState,l=De();var a=l.queue.dispatch;return l.memoizedState=e,[t,a,!1]}function da(e,t,l,a){return e={tag:e,create:l,deps:a,inst:t,next:null},t=le.updateQueue,t===null&&(t=oc(),le.updateQueue=t),l=t.lastEffect,l===null?t.lastEffect=e.next=e:(a=l.next,l.next=e,e.next=a,t.lastEffect=e),e}function pn(){return{destroy:void 0,resource:void 0}}function As(){return De().memoizedState}function bn(e,t,l,a){var u=Fe();a=a===void 0?null:a,le.flags|=e,u.memoizedState=da(1|t,pn(),l,a)}function uu(e,t,l,a){var u=De();a=a===void 0?null:a;var n=u.memoizedState.inst;he!==null&&a!==null&&ic(a,he.memoizedState.deps)?u.memoizedState=da(t,n,l,a):(le.flags|=e,u.memoizedState=da(1|t,n,l,a))}function Ts(e,t){bn(8390656,8,e,t)}function Ns(e,t){uu(2048,8,e,t)}function Rs(e,t){return uu(4,2,e,t)}function js(e,t){return uu(4,4,e,t)}function zs(e,t){if(typeof t=="function"){e=e();var l=t(e);return function(){typeof l=="function"?l():t(null)}}if(t!=null)return e=e(),t.current=e,function(){t.current=null}}function Os(e,t,l){l=l!=null?l.concat([e]):null,uu(4,4,zs.bind(null,t,e),l)}function vc(){}function Ms(e,t){var l=De();t=t===void 0?null:t;var a=l.memoizedState;return t!==null&&ic(t,a[1])?a[0]:(l.memoizedState=[e,t],e)}function _s(e,t){var l=De();t=t===void 0?null:t;var a=l.memoizedState;if(t!==null&&ic(t,a[1]))return a[0];if(a=e(),ql){Pt(!0);try{e()}finally{Pt(!1)}}return l.memoizedState=[a,t],a}function pc(e,t,l){return l===void 0||(nl&1073741824)!==0?e.memoizedState=t:(e.memoizedState=l,e=Co(),le.lanes|=e,dl|=e,l)}function Ds(e,t,l,a){return ut(l,t)?l:fa.current!==null?(e=pc(e,l,a),ut(e,t)||(He=!0),e):(nl&42)===0?(He=!0,e.memoizedState=l):(e=Co(),le.lanes|=e,dl|=e,t)}function Us(e,t,l,a,u){var n=Y.p;Y.p=n!==0&&8>n?n:8;var i=M.T,r={};M.T=r,Sc(e,!1,t,l);try{var y=u(),E=M.S;if(E!==null&&E(r,y),y!==null&&typeof y=="object"&&typeof y.then=="function"){var O=Rh(y,a);nu(e,t,O,ft(e))}else nu(e,t,a,ft(e))}catch(U){nu(e,t,{then:function(){},status:"rejected",reason:U},ft())}finally{Y.p=n,M.T=i}}function _h(){}function bc(e,t,l,a){if(e.tag!==5)throw Error(f(476));var u=ws(e).queue;Us(e,u,t,$,l===null?_h:function(){return Cs(e),l(a)})}function ws(e){var t=e.memoizedState;if(t!==null)return t;t={memoizedState:$,baseState:$,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:Lt,lastRenderedState:$},next:null};var l={};return t.next={memoizedState:l,baseState:l,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:Lt,lastRenderedState:l},next:null},e.memoizedState=t,e=e.alternate,e!==null&&(e.memoizedState=t),t}function Cs(e){var t=ws(e).next.queue;nu(e,t,{},ft())}function xc(){return Ze(Au)}function Hs(){return De().memoizedState}function Bs(){return De().memoizedState}function Dh(e){for(var t=e.return;t!==null;){switch(t.tag){case 24:case 3:var l=ft();e=al(l);var a=ul(t,e,l);a!==null&&(st(a,t,l),Ia(a,t,l)),t={cache:Wi()},e.payload=t;return}t=t.return}}function Uh(e,t,l){var a=ft();l={lane:a,revertLane:0,action:l,hasEagerState:!1,eagerState:null,next:null},xn(e)?Ys(t,l):(l=Li(e,t,l,a),l!==null&&(st(l,e,a),Ls(l,t,a)))}function qs(e,t,l){var a=ft();nu(e,t,l,a)}function nu(e,t,l,a){var u={lane:a,revertLane:0,action:l,hasEagerState:!1,eagerState:null,next:null};if(xn(e))Ys(t,u);else{var n=e.alternate;if(e.lanes===0&&(n===null||n.lanes===0)&&(n=t.lastRenderedReducer,n!==null))try{var i=t.lastRenderedState,r=n(i,l);if(u.hasEagerState=!0,u.eagerState=r,ut(r,i))return ln(e,t,u,0),pe===null&&tn(),!1}catch{}finally{}if(l=Li(e,t,u,a),l!==null)return st(l,e,a),Ls(l,t,a),!0}return!1}function Sc(e,t,l,a){if(a={lane:2,revertLane:Ic(),action:a,hasEagerState:!1,eagerState:null,next:null},xn(e)){if(t)throw Error(f(479))}else t=Li(e,l,a,2),t!==null&&st(t,e,2)}function xn(e){var t=e.alternate;return e===le||t!==null&&t===le}function Ys(e,t){sa=mn=!0;var l=e.pending;l===null?t.next=t:(t.next=l.next,l.next=t),e.pending=t}function Ls(e,t,l){if((l&4194048)!==0){var a=t.lanes;a&=e.pendingLanes,l|=a,t.lanes=l,Kr(e,l)}}var Sn={readContext:Ze,use:gn,useCallback:ze,useContext:ze,useEffect:ze,useImperativeHandle:ze,useLayoutEffect:ze,useInsertionEffect:ze,useMemo:ze,useReducer:ze,useRef:ze,useState:ze,useDebugValue:ze,useDeferredValue:ze,useTransition:ze,useSyncExternalStore:ze,useId:ze,useHostTransitionStatus:ze,useFormState:ze,useActionState:ze,useOptimistic:ze,useMemoCache:ze,useCacheRefresh:ze},Gs={readContext:Ze,use:gn,useCallback:function(e,t){return Fe().memoizedState=[e,t===void 0?null:t],e},useContext:Ze,useEffect:Ts,useImperativeHandle:function(e,t,l){l=l!=null?l.concat([e]):null,bn(4194308,4,zs.bind(null,t,e),l)},useLayoutEffect:function(e,t){return bn(4194308,4,e,t)},useInsertionEffect:function(e,t){bn(4,2,e,t)},useMemo:function(e,t){var l=Fe();t=t===void 0?null:t;var a=e();if(ql){Pt(!0);try{e()}finally{Pt(!1)}}return l.memoizedState=[a,t],a},useReducer:function(e,t,l){var a=Fe();if(l!==void 0){var u=l(t);if(ql){Pt(!0);try{l(t)}finally{Pt(!1)}}}else u=t;return a.memoizedState=a.baseState=u,e={pending:null,lanes:0,dispatch:null,lastRenderedReducer:e,lastRenderedState:u},a.queue=e,e=e.dispatch=Uh.bind(null,le,e),[a.memoizedState,e]},useRef:function(e){var t=Fe();return e={current:e},t.memoizedState=e},useState:function(e){e=yc(e);var t=e.queue,l=qs.bind(null,le,t);return t.dispatch=l,[e.memoizedState,l]},useDebugValue:vc,useDeferredValue:function(e,t){var l=Fe();return pc(l,e,t)},useTransition:function(){var e=yc(!1);return e=Us.bind(null,le,e.queue,!0,!1),Fe().memoizedState=e,[!1,e]},useSyncExternalStore:function(e,t,l){var a=le,u=Fe();if(fe){if(l===void 0)throw Error(f(407));l=l()}else{if(l=t(),pe===null)throw Error(f(349));(ie&124)!==0||rs(a,t,l)}u.memoizedState=l;var n={value:l,getSnapshot:t};return u.queue=n,Ts(ss.bind(null,a,n,e),[e]),a.flags|=2048,da(9,pn(),fs.bind(null,a,n,l,t),null),l},useId:function(){var e=Fe(),t=pe.identifierPrefix;if(fe){var l=Bt,a=Ht;l=(a&~(1<<32-at(a)-1)).toString(32)+l,t="«"+t+"R"+l,l=yn++,0<l&&(t+="H"+l.toString(32)),t+="»"}else l=jh++,t="«"+t+"r"+l.toString(32)+"»";return e.memoizedState=t},useHostTransitionStatus:xc,useFormState:bs,useActionState:bs,useOptimistic:function(e){var t=Fe();t.memoizedState=t.baseState=e;var l={pending:null,lanes:0,dispatch:null,lastRenderedReducer:null,lastRenderedState:null};return t.queue=l,t=Sc.bind(null,le,!0,l),l.dispatch=t,[e,t]},useMemoCache:dc,useCacheRefresh:function(){return Fe().memoizedState=Dh.bind(null,le)}},Qs={readContext:Ze,use:gn,useCallback:Ms,useContext:Ze,useEffect:Ns,useImperativeHandle:Os,useInsertionEffect:Rs,useLayoutEffect:js,useMemo:_s,useReducer:vn,useRef:As,useState:function(){return vn(Lt)},useDebugValue:vc,useDeferredValue:function(e,t){var l=De();return Ds(l,he.memoizedState,e,t)},useTransition:function(){var e=vn(Lt)[0],t=De().memoizedState;return[typeof e=="boolean"?e:au(e),t]},useSyncExternalStore:cs,useId:Hs,useHostTransitionStatus:xc,useFormState:xs,useActionState:xs,useOptimistic:function(e,t){var l=De();return hs(l,he,e,t)},useMemoCache:dc,useCacheRefresh:Bs},wh={readContext:Ze,use:gn,useCallback:Ms,useContext:Ze,useEffect:Ns,useImperativeHandle:Os,useInsertionEffect:Rs,useLayoutEffect:js,useMemo:_s,useReducer:mc,useRef:As,useState:function(){return mc(Lt)},useDebugValue:vc,useDeferredValue:function(e,t){var l=De();return he===null?pc(l,e,t):Ds(l,he.memoizedState,e,t)},useTransition:function(){var e=mc(Lt)[0],t=De().memoizedState;return[typeof e=="boolean"?e:au(e),t]},useSyncExternalStore:cs,useId:Hs,useHostTransitionStatus:xc,useFormState:Es,useActionState:Es,useOptimistic:function(e,t){var l=De();return he!==null?hs(l,he,e,t):(l.baseState=e,[e,l.queue.dispatch])},useMemoCache:dc,useCacheRefresh:Bs},ha=null,iu=0;function En(e){var t=iu;return iu+=1,ha===null&&(ha=[]),If(ha,e,t)}function cu(e,t){t=t.props.ref,e.ref=t!==void 0?t:null}function An(e,t){throw t.$$typeof===H?Error(f(525)):(e=Object.prototype.toString.call(t),Error(f(31,e==="[object Object]"?"object with keys {"+Object.keys(t).join(", ")+"}":e)))}function ks(e){var t=e._init;return t(e._payload)}function Xs(e){function t(x,p){if(e){var S=x.deletions;S===null?(x.deletions=[p],x.flags|=16):S.push(p)}}function l(x,p){if(!e)return null;for(;p!==null;)t(x,p),p=p.sibling;return null}function a(x){for(var p=new Map;x!==null;)x.key!==null?p.set(x.key,x):p.set(x.index,x),x=x.sibling;return p}function u(x,p){return x=Ct(x,p),x.index=0,x.sibling=null,x}function n(x,p,S){return x.index=S,e?(S=x.alternate,S!==null?(S=S.index,S<p?(x.flags|=67108866,p):S):(x.flags|=67108866,p)):(x.flags|=1048576,p)}function i(x){return e&&x.alternate===null&&(x.flags|=67108866),x}function r(x,p,S,_){return p===null||p.tag!==6?(p=Qi(S,x.mode,_),p.return=x,p):(p=u(p,S),p.return=x,p)}function y(x,p,S,_){var k=S.type;return k===C?O(x,p,S.props.children,_,S.key):p!==null&&(p.elementType===k||typeof k=="object"&&k!==null&&k.$$typeof===je&&ks(k)===p.type)?(p=u(p,S.props),cu(p,S),p.return=x,p):(p=un(S.type,S.key,S.props,null,x.mode,_),cu(p,S),p.return=x,p)}function E(x,p,S,_){return p===null||p.tag!==4||p.stateNode.containerInfo!==S.containerInfo||p.stateNode.implementation!==S.implementation?(p=ki(S,x.mode,_),p.return=x,p):(p=u(p,S.children||[]),p.return=x,p)}function O(x,p,S,_,k){return p===null||p.tag!==7?(p=Ml(S,x.mode,_,k),p.return=x,p):(p=u(p,S),p.return=x,p)}function U(x,p,S){if(typeof p=="string"&&p!==""||typeof p=="number"||typeof p=="bigint")return p=Qi(""+p,x.mode,S),p.return=x,p;if(typeof p=="object"&&p!==null){switch(p.$$typeof){case D:return S=un(p.type,p.key,p.props,null,x.mode,S),cu(S,p),S.return=x,S;case L:return p=ki(p,x.mode,S),p.return=x,p;case je:var _=p._init;return p=_(p._payload),U(x,p,S)}if(ke(p)||Qe(p))return p=Ml(p,x.mode,S,null),p.return=x,p;if(typeof p.then=="function")return U(x,En(p),S);if(p.$$typeof===P)return U(x,fn(x,p),S);An(x,p)}return null}function T(x,p,S,_){var k=p!==null?p.key:null;if(typeof S=="string"&&S!==""||typeof S=="number"||typeof S=="bigint")return k!==null?null:r(x,p,""+S,_);if(typeof S=="object"&&S!==null){switch(S.$$typeof){case D:return S.key===k?y(x,p,S,_):null;case L:return S.key===k?E(x,p,S,_):null;case je:return k=S._init,S=k(S._payload),T(x,p,S,_)}if(ke(S)||Qe(S))return k!==null?null:O(x,p,S,_,null);if(typeof S.then=="function")return T(x,p,En(S),_);if(S.$$typeof===P)return T(x,p,fn(x,S),_);An(x,S)}return null}function N(x,p,S,_,k){if(typeof _=="string"&&_!==""||typeof _=="number"||typeof _=="bigint")return x=x.get(S)||null,r(p,x,""+_,k);if(typeof _=="object"&&_!==null){switch(_.$$typeof){case D:return x=x.get(_.key===null?S:_.key)||null,y(p,x,_,k);case L:return x=x.get(_.key===null?S:_.key)||null,E(p,x,_,k);case je:var ae=_._init;return _=ae(_._payload),N(x,p,S,_,k)}if(ke(_)||Qe(_))return x=x.get(S)||null,O(p,x,_,k,null);if(typeof _.then=="function")return N(x,p,S,En(_),k);if(_.$$typeof===P)return N(x,p,S,fn(p,_),k);An(p,_)}return null}function W(x,p,S,_){for(var k=null,ae=null,Z=p,J=p=0,qe=null;Z!==null&&J<S.length;J++){Z.index>J?(qe=Z,Z=null):qe=Z.sibling;var re=T(x,Z,S[J],_);if(re===null){Z===null&&(Z=qe);break}e&&Z&&re.alternate===null&&t(x,Z),p=n(re,p,J),ae===null?k=re:ae.sibling=re,ae=re,Z=qe}if(J===S.length)return l(x,Z),fe&&Dl(x,J),k;if(Z===null){for(;J<S.length;J++)Z=U(x,S[J],_),Z!==null&&(p=n(Z,p,J),ae===null?k=Z:ae.sibling=Z,ae=Z);return fe&&Dl(x,J),k}for(Z=a(Z);J<S.length;J++)qe=N(Z,x,J,S[J],_),qe!==null&&(e&&qe.alternate!==null&&Z.delete(qe.key===null?J:qe.key),p=n(qe,p,J),ae===null?k=qe:ae.sibling=qe,ae=qe);return e&&Z.forEach(function(Sl){return t(x,Sl)}),fe&&Dl(x,J),k}function K(x,p,S,_){if(S==null)throw Error(f(151));for(var k=null,ae=null,Z=p,J=p=0,qe=null,re=S.next();Z!==null&&!re.done;J++,re=S.next()){Z.index>J?(qe=Z,Z=null):qe=Z.sibling;var Sl=T(x,Z,re.value,_);if(Sl===null){Z===null&&(Z=qe);break}e&&Z&&Sl.alternate===null&&t(x,Z),p=n(Sl,p,J),ae===null?k=Sl:ae.sibling=Sl,ae=Sl,Z=qe}if(re.done)return l(x,Z),fe&&Dl(x,J),k;if(Z===null){for(;!re.done;J++,re=S.next())re=U(x,re.value,_),re!==null&&(p=n(re,p,J),ae===null?k=re:ae.sibling=re,ae=re);return fe&&Dl(x,J),k}for(Z=a(Z);!re.done;J++,re=S.next())re=N(Z,x,J,re.value,_),re!==null&&(e&&re.alternate!==null&&Z.delete(re.key===null?J:re.key),p=n(re,p,J),ae===null?k=re:ae.sibling=re,ae=re);return e&&Z.forEach(function(Cm){return t(x,Cm)}),fe&&Dl(x,J),k}function ye(x,p,S,_){if(typeof S=="object"&&S!==null&&S.type===C&&S.key===null&&(S=S.props.children),typeof S=="object"&&S!==null){switch(S.$$typeof){case D:e:{for(var k=S.key;p!==null;){if(p.key===k){if(k=S.type,k===C){if(p.tag===7){l(x,p.sibling),_=u(p,S.props.children),_.return=x,x=_;break e}}else if(p.elementType===k||typeof k=="object"&&k!==null&&k.$$typeof===je&&ks(k)===p.type){l(x,p.sibling),_=u(p,S.props),cu(_,S),_.return=x,x=_;break e}l(x,p);break}else t(x,p);p=p.sibling}S.type===C?(_=Ml(S.props.children,x.mode,_,S.key),_.return=x,x=_):(_=un(S.type,S.key,S.props,null,x.mode,_),cu(_,S),_.return=x,x=_)}return i(x);case L:e:{for(k=S.key;p!==null;){if(p.key===k)if(p.tag===4&&p.stateNode.containerInfo===S.containerInfo&&p.stateNode.implementation===S.implementation){l(x,p.sibling),_=u(p,S.children||[]),_.return=x,x=_;break e}else{l(x,p);break}else t(x,p);p=p.sibling}_=ki(S,x.mode,_),_.return=x,x=_}return i(x);case je:return k=S._init,S=k(S._payload),ye(x,p,S,_)}if(ke(S))return W(x,p,S,_);if(Qe(S)){if(k=Qe(S),typeof k!="function")throw Error(f(150));return S=k.call(S),K(x,p,S,_)}if(typeof S.then=="function")return ye(x,p,En(S),_);if(S.$$typeof===P)return ye(x,p,fn(x,S),_);An(x,S)}return typeof S=="string"&&S!==""||typeof S=="number"||typeof S=="bigint"?(S=""+S,p!==null&&p.tag===6?(l(x,p.sibling),_=u(p,S),_.return=x,x=_):(l(x,p),_=Qi(S,x.mode,_),_.return=x,x=_),i(x)):l(x,p)}return function(x,p,S,_){try{iu=0;var k=ye(x,p,S,_);return ha=null,k}catch(Z){if(Z===Fa||Z===on)throw Z;var ae=nt(29,Z,null,x.mode);return ae.lanes=_,ae.return=x,ae}finally{}}}var ma=Xs(!0),Zs=Xs(!1),pt=w(null),zt=null;function il(e){var t=e.alternate;q(we,we.current&1),q(pt,e),zt===null&&(t===null||fa.current!==null||t.memoizedState!==null)&&(zt=e)}function Vs(e){if(e.tag===22){if(q(we,we.current),q(pt,e),zt===null){var t=e.alternate;t!==null&&t.memoizedState!==null&&(zt=e)}}else cl()}function cl(){q(we,we.current),q(pt,pt.current)}function Gt(e){G(pt),zt===e&&(zt=null),G(we)}var we=w(0);function Tn(e){for(var t=e;t!==null;){if(t.tag===13){var l=t.memoizedState;if(l!==null&&(l=l.dehydrated,l===null||l.data==="$?"||or(l)))return t}else if(t.tag===19&&t.memoizedProps.revealOrder!==void 0){if((t.flags&128)!==0)return t}else if(t.child!==null){t.child.return=t,t=t.child;continue}if(t===e)break;for(;t.sibling===null;){if(t.return===null||t.return===e)return null;t=t.return}t.sibling.return=t.return,t=t.sibling}return null}function Ec(e,t,l,a){t=e.memoizedState,l=l(a,t),l=l==null?t:R({},t,l),e.memoizedState=l,e.lanes===0&&(e.updateQueue.baseState=l)}var Ac={enqueueSetState:function(e,t,l){e=e._reactInternals;var a=ft(),u=al(a);u.payload=t,l!=null&&(u.callback=l),t=ul(e,u,a),t!==null&&(st(t,e,a),Ia(t,e,a))},enqueueReplaceState:function(e,t,l){e=e._reactInternals;var a=ft(),u=al(a);u.tag=1,u.payload=t,l!=null&&(u.callback=l),t=ul(e,u,a),t!==null&&(st(t,e,a),Ia(t,e,a))},enqueueForceUpdate:function(e,t){e=e._reactInternals;var l=ft(),a=al(l);a.tag=2,t!=null&&(a.callback=t),t=ul(e,a,l),t!==null&&(st(t,e,l),Ia(t,e,l))}};function Ks(e,t,l,a,u,n,i){return e=e.stateNode,typeof e.shouldComponentUpdate=="function"?e.shouldComponentUpdate(a,n,i):t.prototype&&t.prototype.isPureReactComponent?!ka(l,a)||!ka(u,n):!0}function Js(e,t,l,a){e=t.state,typeof t.componentWillReceiveProps=="function"&&t.componentWillReceiveProps(l,a),typeof t.UNSAFE_componentWillReceiveProps=="function"&&t.UNSAFE_componentWillReceiveProps(l,a),t.state!==e&&Ac.enqueueReplaceState(t,t.state,null)}function Yl(e,t){var l=t;if("ref"in t){l={};for(var a in t)a!=="ref"&&(l[a]=t[a])}if(e=e.defaultProps){l===t&&(l=R({},l));for(var u in e)l[u]===void 0&&(l[u]=e[u])}return l}var Nn=typeof reportError=="function"?reportError:function(e){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var t=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof e=="object"&&e!==null&&typeof e.message=="string"?String(e.message):String(e),error:e});if(!window.dispatchEvent(t))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",e);return}console.error(e)};function $s(e){Nn(e)}function Ws(e){console.error(e)}function Fs(e){Nn(e)}function Rn(e,t){try{var l=e.onUncaughtError;l(t.value,{componentStack:t.stack})}catch(a){setTimeout(function(){throw a})}}function Ps(e,t,l){try{var a=e.onCaughtError;a(l.value,{componentStack:l.stack,errorBoundary:t.tag===1?t.stateNode:null})}catch(u){setTimeout(function(){throw u})}}function Tc(e,t,l){return l=al(l),l.tag=3,l.payload={element:null},l.callback=function(){Rn(e,t)},l}function Is(e){return e=al(e),e.tag=3,e}function eo(e,t,l,a){var u=l.type.getDerivedStateFromError;if(typeof u=="function"){var n=a.value;e.payload=function(){return u(n)},e.callback=function(){Ps(t,l,a)}}var i=l.stateNode;i!==null&&typeof i.componentDidCatch=="function"&&(e.callback=function(){Ps(t,l,a),typeof u!="function"&&(hl===null?hl=new Set([this]):hl.add(this));var r=a.stack;this.componentDidCatch(a.value,{componentStack:r!==null?r:""})})}function Ch(e,t,l,a,u){if(l.flags|=32768,a!==null&&typeof a=="object"&&typeof a.then=="function"){if(t=l.alternate,t!==null&&Ja(t,l,u,!0),l=pt.current,l!==null){switch(l.tag){case 13:return zt===null?Jc():l.alternate===null&&Re===0&&(Re=3),l.flags&=-257,l.flags|=65536,l.lanes=u,a===Ii?l.flags|=16384:(t=l.updateQueue,t===null?l.updateQueue=new Set([a]):t.add(a),Wc(e,a,u)),!1;case 22:return l.flags|=65536,a===Ii?l.flags|=16384:(t=l.updateQueue,t===null?(t={transitions:null,markerInstances:null,retryQueue:new Set([a])},l.updateQueue=t):(l=t.retryQueue,l===null?t.retryQueue=new Set([a]):l.add(a)),Wc(e,a,u)),!1}throw Error(f(435,l.tag))}return Wc(e,a,u),Jc(),!1}if(fe)return t=pt.current,t!==null?((t.flags&65536)===0&&(t.flags|=256),t.flags|=65536,t.lanes=u,a!==Vi&&(e=Error(f(422),{cause:a}),Ka(mt(e,l)))):(a!==Vi&&(t=Error(f(423),{cause:a}),Ka(mt(t,l))),e=e.current.alternate,e.flags|=65536,u&=-u,e.lanes|=u,a=mt(a,l),u=Tc(e.stateNode,a,u),lc(e,u),Re!==4&&(Re=2)),!1;var n=Error(f(520),{cause:a});if(n=mt(n,l),mu===null?mu=[n]:mu.push(n),Re!==4&&(Re=2),t===null)return!0;a=mt(a,l),l=t;do{switch(l.tag){case 3:return l.flags|=65536,e=u&-u,l.lanes|=e,e=Tc(l.stateNode,a,e),lc(l,e),!1;case 1:if(t=l.type,n=l.stateNode,(l.flags&128)===0&&(typeof t.getDerivedStateFromError=="function"||n!==null&&typeof n.componentDidCatch=="function"&&(hl===null||!hl.has(n))))return l.flags|=65536,u&=-u,l.lanes|=u,u=Is(u),eo(u,e,l,a),lc(l,u),!1}l=l.return}while(l!==null);return!1}var to=Error(f(461)),He=!1;function Ye(e,t,l,a){t.child=e===null?Zs(t,null,l,a):ma(t,e.child,l,a)}function lo(e,t,l,a,u){l=l.render;var n=t.ref;if("ref"in a){var i={};for(var r in a)r!=="ref"&&(i[r]=a[r])}else i=a;return Hl(t),a=cc(e,t,l,i,n,u),r=rc(),e!==null&&!He?(fc(e,t,u),Qt(e,t,u)):(fe&&r&&Xi(t),t.flags|=1,Ye(e,t,a,u),t.child)}function ao(e,t,l,a,u){if(e===null){var n=l.type;return typeof n=="function"&&!Gi(n)&&n.defaultProps===void 0&&l.compare===null?(t.tag=15,t.type=n,uo(e,t,n,a,u)):(e=un(l.type,null,a,t,t.mode,u),e.ref=t.ref,e.return=t,t.child=e)}if(n=e.child,!Dc(e,u)){var i=n.memoizedProps;if(l=l.compare,l=l!==null?l:ka,l(i,a)&&e.ref===t.ref)return Qt(e,t,u)}return t.flags|=1,e=Ct(n,a),e.ref=t.ref,e.return=t,t.child=e}function uo(e,t,l,a,u){if(e!==null){var n=e.memoizedProps;if(ka(n,a)&&e.ref===t.ref)if(He=!1,t.pendingProps=a=n,Dc(e,u))(e.flags&131072)!==0&&(He=!0);else return t.lanes=e.lanes,Qt(e,t,u)}return Nc(e,t,l,a,u)}function no(e,t,l){var a=t.pendingProps,u=a.children,n=e!==null?e.memoizedState:null;if(a.mode==="hidden"){if((t.flags&128)!==0){if(a=n!==null?n.baseLanes|l:l,e!==null){for(u=t.child=e.child,n=0;u!==null;)n=n|u.lanes|u.childLanes,u=u.sibling;t.childLanes=n&~a}else t.childLanes=0,t.child=null;return io(e,t,a,l)}if((l&536870912)!==0)t.memoizedState={baseLanes:0,cachePool:null},e!==null&&sn(t,n!==null?n.cachePool:null),n!==null?us(t,n):uc(),Vs(t);else return t.lanes=t.childLanes=536870912,io(e,t,n!==null?n.baseLanes|l:l,l)}else n!==null?(sn(t,n.cachePool),us(t,n),cl(),t.memoizedState=null):(e!==null&&sn(t,null),uc(),cl());return Ye(e,t,u,l),t.child}function io(e,t,l,a){var u=Pi();return u=u===null?null:{parent:Ue._currentValue,pool:u},t.memoizedState={baseLanes:l,cachePool:u},e!==null&&sn(t,null),uc(),Vs(t),e!==null&&Ja(e,t,a,!0),null}function jn(e,t){var l=t.ref;if(l===null)e!==null&&e.ref!==null&&(t.flags|=4194816);else{if(typeof l!="function"&&typeof l!="object")throw Error(f(284));(e===null||e.ref!==l)&&(t.flags|=4194816)}}function Nc(e,t,l,a,u){return Hl(t),l=cc(e,t,l,a,void 0,u),a=rc(),e!==null&&!He?(fc(e,t,u),Qt(e,t,u)):(fe&&a&&Xi(t),t.flags|=1,Ye(e,t,l,u),t.child)}function co(e,t,l,a,u,n){return Hl(t),t.updateQueue=null,l=is(t,a,l,u),ns(e),a=rc(),e!==null&&!He?(fc(e,t,n),Qt(e,t,n)):(fe&&a&&Xi(t),t.flags|=1,Ye(e,t,l,n),t.child)}function ro(e,t,l,a,u){if(Hl(t),t.stateNode===null){var n=ua,i=l.contextType;typeof i=="object"&&i!==null&&(n=Ze(i)),n=new l(a,n),t.memoizedState=n.state!==null&&n.state!==void 0?n.state:null,n.updater=Ac,t.stateNode=n,n._reactInternals=t,n=t.stateNode,n.props=a,n.state=t.memoizedState,n.refs={},ec(t),i=l.contextType,n.context=typeof i=="object"&&i!==null?Ze(i):ua,n.state=t.memoizedState,i=l.getDerivedStateFromProps,typeof i=="function"&&(Ec(t,l,i,a),n.state=t.memoizedState),typeof l.getDerivedStateFromProps=="function"||typeof n.getSnapshotBeforeUpdate=="function"||typeof n.UNSAFE_componentWillMount!="function"&&typeof n.componentWillMount!="function"||(i=n.state,typeof n.componentWillMount=="function"&&n.componentWillMount(),typeof n.UNSAFE_componentWillMount=="function"&&n.UNSAFE_componentWillMount(),i!==n.state&&Ac.enqueueReplaceState(n,n.state,null),tu(t,a,n,u),eu(),n.state=t.memoizedState),typeof n.componentDidMount=="function"&&(t.flags|=4194308),a=!0}else if(e===null){n=t.stateNode;var r=t.memoizedProps,y=Yl(l,r);n.props=y;var E=n.context,O=l.contextType;i=ua,typeof O=="object"&&O!==null&&(i=Ze(O));var U=l.getDerivedStateFromProps;O=typeof U=="function"||typeof n.getSnapshotBeforeUpdate=="function",r=t.pendingProps!==r,O||typeof n.UNSAFE_componentWillReceiveProps!="function"&&typeof n.componentWillReceiveProps!="function"||(r||E!==i)&&Js(t,n,a,i),ll=!1;var T=t.memoizedState;n.state=T,tu(t,a,n,u),eu(),E=t.memoizedState,r||T!==E||ll?(typeof U=="function"&&(Ec(t,l,U,a),E=t.memoizedState),(y=ll||Ks(t,l,y,a,T,E,i))?(O||typeof n.UNSAFE_componentWillMount!="function"&&typeof n.componentWillMount!="function"||(typeof n.componentWillMount=="function"&&n.componentWillMount(),typeof n.UNSAFE_componentWillMount=="function"&&n.UNSAFE_componentWillMount()),typeof n.componentDidMount=="function"&&(t.flags|=4194308)):(typeof n.componentDidMount=="function"&&(t.flags|=4194308),t.memoizedProps=a,t.memoizedState=E),n.props=a,n.state=E,n.context=i,a=y):(typeof n.componentDidMount=="function"&&(t.flags|=4194308),a=!1)}else{n=t.stateNode,tc(e,t),i=t.memoizedProps,O=Yl(l,i),n.props=O,U=t.pendingProps,T=n.context,E=l.contextType,y=ua,typeof E=="object"&&E!==null&&(y=Ze(E)),r=l.getDerivedStateFromProps,(E=typeof r=="function"||typeof n.getSnapshotBeforeUpdate=="function")||typeof n.UNSAFE_componentWillReceiveProps!="function"&&typeof n.componentWillReceiveProps!="function"||(i!==U||T!==y)&&Js(t,n,a,y),ll=!1,T=t.memoizedState,n.state=T,tu(t,a,n,u),eu();var N=t.memoizedState;i!==U||T!==N||ll||e!==null&&e.dependencies!==null&&rn(e.dependencies)?(typeof r=="function"&&(Ec(t,l,r,a),N=t.memoizedState),(O=ll||Ks(t,l,O,a,T,N,y)||e!==null&&e.dependencies!==null&&rn(e.dependencies))?(E||typeof n.UNSAFE_componentWillUpdate!="function"&&typeof n.componentWillUpdate!="function"||(typeof n.componentWillUpdate=="function"&&n.componentWillUpdate(a,N,y),typeof n.UNSAFE_componentWillUpdate=="function"&&n.UNSAFE_componentWillUpdate(a,N,y)),typeof n.componentDidUpdate=="function"&&(t.flags|=4),typeof n.getSnapshotBeforeUpdate=="function"&&(t.flags|=1024)):(typeof n.componentDidUpdate!="function"||i===e.memoizedProps&&T===e.memoizedState||(t.flags|=4),typeof n.getSnapshotBeforeUpdate!="function"||i===e.memoizedProps&&T===e.memoizedState||(t.flags|=1024),t.memoizedProps=a,t.memoizedState=N),n.props=a,n.state=N,n.context=y,a=O):(typeof n.componentDidUpdate!="function"||i===e.memoizedProps&&T===e.memoizedState||(t.flags|=4),typeof n.getSnapshotBeforeUpdate!="function"||i===e.memoizedProps&&T===e.memoizedState||(t.flags|=1024),a=!1)}return n=a,jn(e,t),a=(t.flags&128)!==0,n||a?(n=t.stateNode,l=a&&typeof l.getDerivedStateFromError!="function"?null:n.render(),t.flags|=1,e!==null&&a?(t.child=ma(t,e.child,null,u),t.child=ma(t,null,l,u)):Ye(e,t,l,u),t.memoizedState=n.state,e=t.child):e=Qt(e,t,u),e}function fo(e,t,l,a){return Va(),t.flags|=256,Ye(e,t,l,a),t.child}var Rc={dehydrated:null,treeContext:null,retryLane:0,hydrationErrors:null};function jc(e){return{baseLanes:e,cachePool:Wf()}}function zc(e,t,l){return e=e!==null?e.childLanes&~l:0,t&&(e|=bt),e}function so(e,t,l){var a=t.pendingProps,u=!1,n=(t.flags&128)!==0,i;if((i=n)||(i=e!==null&&e.memoizedState===null?!1:(we.current&2)!==0),i&&(u=!0,t.flags&=-129),i=(t.flags&32)!==0,t.flags&=-33,e===null){if(fe){if(u?il(t):cl(),fe){var r=Ne,y;if(y=r){e:{for(y=r,r=jt;y.nodeType!==8;){if(!r){r=null;break e}if(y=At(y.nextSibling),y===null){r=null;break e}}r=y}r!==null?(t.memoizedState={dehydrated:r,treeContext:_l!==null?{id:Ht,overflow:Bt}:null,retryLane:536870912,hydrationErrors:null},y=nt(18,null,null,0),y.stateNode=r,y.return=t,t.child=y,Je=t,Ne=null,y=!0):y=!1}y||wl(t)}if(r=t.memoizedState,r!==null&&(r=r.dehydrated,r!==null))return or(r)?t.lanes=32:t.lanes=536870912,null;Gt(t)}return r=a.children,a=a.fallback,u?(cl(),u=t.mode,r=zn({mode:"hidden",children:r},u),a=Ml(a,u,l,null),r.return=t,a.return=t,r.sibling=a,t.child=r,u=t.child,u.memoizedState=jc(l),u.childLanes=zc(e,i,l),t.memoizedState=Rc,a):(il(t),Oc(t,r))}if(y=e.memoizedState,y!==null&&(r=y.dehydrated,r!==null)){if(n)t.flags&256?(il(t),t.flags&=-257,t=Mc(e,t,l)):t.memoizedState!==null?(cl(),t.child=e.child,t.flags|=128,t=null):(cl(),u=a.fallback,r=t.mode,a=zn({mode:"visible",children:a.children},r),u=Ml(u,r,l,null),u.flags|=2,a.return=t,u.return=t,a.sibling=u,t.child=a,ma(t,e.child,null,l),a=t.child,a.memoizedState=jc(l),a.childLanes=zc(e,i,l),t.memoizedState=Rc,t=u);else if(il(t),or(r)){if(i=r.nextSibling&&r.nextSibling.dataset,i)var E=i.dgst;i=E,a=Error(f(419)),a.stack="",a.digest=i,Ka({value:a,source:null,stack:null}),t=Mc(e,t,l)}else if(He||Ja(e,t,l,!1),i=(l&e.childLanes)!==0,He||i){if(i=pe,i!==null&&(a=l&-l,a=(a&42)!==0?1:di(a),a=(a&(i.suspendedLanes|l))!==0?0:a,a!==0&&a!==y.retryLane))throw y.retryLane=a,aa(e,a),st(i,e,a),to;r.data==="$?"||Jc(),t=Mc(e,t,l)}else r.data==="$?"?(t.flags|=192,t.child=e.child,t=null):(e=y.treeContext,Ne=At(r.nextSibling),Je=t,fe=!0,Ul=null,jt=!1,e!==null&&(gt[vt++]=Ht,gt[vt++]=Bt,gt[vt++]=_l,Ht=e.id,Bt=e.overflow,_l=t),t=Oc(t,a.children),t.flags|=4096);return t}return u?(cl(),u=a.fallback,r=t.mode,y=e.child,E=y.sibling,a=Ct(y,{mode:"hidden",children:a.children}),a.subtreeFlags=y.subtreeFlags&65011712,E!==null?u=Ct(E,u):(u=Ml(u,r,l,null),u.flags|=2),u.return=t,a.return=t,a.sibling=u,t.child=a,a=u,u=t.child,r=e.child.memoizedState,r===null?r=jc(l):(y=r.cachePool,y!==null?(E=Ue._currentValue,y=y.parent!==E?{parent:E,pool:E}:y):y=Wf(),r={baseLanes:r.baseLanes|l,cachePool:y}),u.memoizedState=r,u.childLanes=zc(e,i,l),t.memoizedState=Rc,a):(il(t),l=e.child,e=l.sibling,l=Ct(l,{mode:"visible",children:a.children}),l.return=t,l.sibling=null,e!==null&&(i=t.deletions,i===null?(t.deletions=[e],t.flags|=16):i.push(e)),t.child=l,t.memoizedState=null,l)}function Oc(e,t){return t=zn({mode:"visible",children:t},e.mode),t.return=e,e.child=t}function zn(e,t){return e=nt(22,e,null,t),e.lanes=0,e.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null},e}function Mc(e,t,l){return ma(t,e.child,null,l),e=Oc(t,t.pendingProps.children),e.flags|=2,t.memoizedState=null,e}function oo(e,t,l){e.lanes|=t;var a=e.alternate;a!==null&&(a.lanes|=t),Ji(e.return,t,l)}function _c(e,t,l,a,u){var n=e.memoizedState;n===null?e.memoizedState={isBackwards:t,rendering:null,renderingStartTime:0,last:a,tail:l,tailMode:u}:(n.isBackwards=t,n.rendering=null,n.renderingStartTime=0,n.last=a,n.tail=l,n.tailMode=u)}function ho(e,t,l){var a=t.pendingProps,u=a.revealOrder,n=a.tail;if(Ye(e,t,a.children,l),a=we.current,(a&2)!==0)a=a&1|2,t.flags|=128;else{if(e!==null&&(e.flags&128)!==0)e:for(e=t.child;e!==null;){if(e.tag===13)e.memoizedState!==null&&oo(e,l,t);else if(e.tag===19)oo(e,l,t);else if(e.child!==null){e.child.return=e,e=e.child;continue}if(e===t)break e;for(;e.sibling===null;){if(e.return===null||e.return===t)break e;e=e.return}e.sibling.return=e.return,e=e.sibling}a&=1}switch(q(we,a),u){case"forwards":for(l=t.child,u=null;l!==null;)e=l.alternate,e!==null&&Tn(e)===null&&(u=l),l=l.sibling;l=u,l===null?(u=t.child,t.child=null):(u=l.sibling,l.sibling=null),_c(t,!1,u,l,n);break;case"backwards":for(l=null,u=t.child,t.child=null;u!==null;){if(e=u.alternate,e!==null&&Tn(e)===null){t.child=u;break}e=u.sibling,u.sibling=l,l=u,u=e}_c(t,!0,l,null,n);break;case"together":_c(t,!1,null,null,void 0);break;default:t.memoizedState=null}return t.child}function Qt(e,t,l){if(e!==null&&(t.dependencies=e.dependencies),dl|=t.lanes,(l&t.childLanes)===0)if(e!==null){if(Ja(e,t,l,!1),(l&t.childLanes)===0)return null}else return null;if(e!==null&&t.child!==e.child)throw Error(f(153));if(t.child!==null){for(e=t.child,l=Ct(e,e.pendingProps),t.child=l,l.return=t;e.sibling!==null;)e=e.sibling,l=l.sibling=Ct(e,e.pendingProps),l.return=t;l.sibling=null}return t.child}function Dc(e,t){return(e.lanes&t)!==0?!0:(e=e.dependencies,!!(e!==null&&rn(e)))}function Hh(e,t,l){switch(t.tag){case 3:xe(t,t.stateNode.containerInfo),tl(t,Ue,e.memoizedState.cache),Va();break;case 27:case 5:ci(t);break;case 4:xe(t,t.stateNode.containerInfo);break;case 10:tl(t,t.type,t.memoizedProps.value);break;case 13:var a=t.memoizedState;if(a!==null)return a.dehydrated!==null?(il(t),t.flags|=128,null):(l&t.child.childLanes)!==0?so(e,t,l):(il(t),e=Qt(e,t,l),e!==null?e.sibling:null);il(t);break;case 19:var u=(e.flags&128)!==0;if(a=(l&t.childLanes)!==0,a||(Ja(e,t,l,!1),a=(l&t.childLanes)!==0),u){if(a)return ho(e,t,l);t.flags|=128}if(u=t.memoizedState,u!==null&&(u.rendering=null,u.tail=null,u.lastEffect=null),q(we,we.current),a)break;return null;case 22:case 23:return t.lanes=0,no(e,t,l);case 24:tl(t,Ue,e.memoizedState.cache)}return Qt(e,t,l)}function mo(e,t,l){if(e!==null)if(e.memoizedProps!==t.pendingProps)He=!0;else{if(!Dc(e,l)&&(t.flags&128)===0)return He=!1,Hh(e,t,l);He=(e.flags&131072)!==0}else He=!1,fe&&(t.flags&1048576)!==0&&kf(t,cn,t.index);switch(t.lanes=0,t.tag){case 16:e:{e=t.pendingProps;var a=t.elementType,u=a._init;if(a=u(a._payload),t.type=a,typeof a=="function")Gi(a)?(e=Yl(a,e),t.tag=1,t=ro(null,t,a,e,l)):(t.tag=0,t=Nc(null,t,a,e,l));else{if(a!=null){if(u=a.$$typeof,u===de){t.tag=11,t=lo(null,t,a,e,l);break e}else if(u===Te){t.tag=14,t=ao(null,t,a,e,l);break e}}throw t=Tl(a)||a,Error(f(306,t,""))}}return t;case 0:return Nc(e,t,t.type,t.pendingProps,l);case 1:return a=t.type,u=Yl(a,t.pendingProps),ro(e,t,a,u,l);case 3:e:{if(xe(t,t.stateNode.containerInfo),e===null)throw Error(f(387));a=t.pendingProps;var n=t.memoizedState;u=n.element,tc(e,t),tu(t,a,null,l);var i=t.memoizedState;if(a=i.cache,tl(t,Ue,a),a!==n.cache&&$i(t,[Ue],l,!0),eu(),a=i.element,n.isDehydrated)if(n={element:a,isDehydrated:!1,cache:i.cache},t.updateQueue.baseState=n,t.memoizedState=n,t.flags&256){t=fo(e,t,a,l);break e}else if(a!==u){u=mt(Error(f(424)),t),Ka(u),t=fo(e,t,a,l);break e}else{switch(e=t.stateNode.containerInfo,e.nodeType){case 9:e=e.body;break;default:e=e.nodeName==="HTML"?e.ownerDocument.body:e}for(Ne=At(e.firstChild),Je=t,fe=!0,Ul=null,jt=!0,l=Zs(t,null,a,l),t.child=l;l;)l.flags=l.flags&-3|4096,l=l.sibling}else{if(Va(),a===u){t=Qt(e,t,l);break e}Ye(e,t,a,l)}t=t.child}return t;case 26:return jn(e,t),e===null?(l=pd(t.type,null,t.pendingProps,null))?t.memoizedState=l:fe||(l=t.type,e=t.pendingProps,a=Qn(I.current).createElement(l),a[Xe]=t,a[$e]=e,Ge(a,l,e),Ce(a),t.stateNode=a):t.memoizedState=pd(t.type,e.memoizedProps,t.pendingProps,e.memoizedState),null;case 27:return ci(t),e===null&&fe&&(a=t.stateNode=yd(t.type,t.pendingProps,I.current),Je=t,jt=!0,u=Ne,gl(t.type)?(dr=u,Ne=At(a.firstChild)):Ne=u),Ye(e,t,t.pendingProps.children,l),jn(e,t),e===null&&(t.flags|=4194304),t.child;case 5:return e===null&&fe&&((u=a=Ne)&&(a=sm(a,t.type,t.pendingProps,jt),a!==null?(t.stateNode=a,Je=t,Ne=At(a.firstChild),jt=!1,u=!0):u=!1),u||wl(t)),ci(t),u=t.type,n=t.pendingProps,i=e!==null?e.memoizedProps:null,a=n.children,rr(u,n)?a=null:i!==null&&rr(u,i)&&(t.flags|=32),t.memoizedState!==null&&(u=cc(e,t,zh,null,null,l),Au._currentValue=u),jn(e,t),Ye(e,t,a,l),t.child;case 6:return e===null&&fe&&((e=l=Ne)&&(l=om(l,t.pendingProps,jt),l!==null?(t.stateNode=l,Je=t,Ne=null,e=!0):e=!1),e||wl(t)),null;case 13:return so(e,t,l);case 4:return xe(t,t.stateNode.containerInfo),a=t.pendingProps,e===null?t.child=ma(t,null,a,l):Ye(e,t,a,l),t.child;case 11:return lo(e,t,t.type,t.pendingProps,l);case 7:return Ye(e,t,t.pendingProps,l),t.child;case 8:return Ye(e,t,t.pendingProps.children,l),t.child;case 12:return Ye(e,t,t.pendingProps.children,l),t.child;case 10:return a=t.pendingProps,tl(t,t.type,a.value),Ye(e,t,a.children,l),t.child;case 9:return u=t.type._context,a=t.pendingProps.children,Hl(t),u=Ze(u),a=a(u),t.flags|=1,Ye(e,t,a,l),t.child;case 14:return ao(e,t,t.type,t.pendingProps,l);case 15:return uo(e,t,t.type,t.pendingProps,l);case 19:return ho(e,t,l);case 31:return a=t.pendingProps,l=t.mode,a={mode:a.mode,children:a.children},e===null?(l=zn(a,l),l.ref=t.ref,t.child=l,l.return=t,t=l):(l=Ct(e.child,a),l.ref=t.ref,t.child=l,l.return=t,t=l),t;case 22:return no(e,t,l);case 24:return Hl(t),a=Ze(Ue),e===null?(u=Pi(),u===null&&(u=pe,n=Wi(),u.pooledCache=n,n.refCount++,n!==null&&(u.pooledCacheLanes|=l),u=n),t.memoizedState={parent:a,cache:u},ec(t),tl(t,Ue,u)):((e.lanes&l)!==0&&(tc(e,t),tu(t,null,null,l),eu()),u=e.memoizedState,n=t.memoizedState,u.parent!==a?(u={parent:a,cache:a},t.memoizedState=u,t.lanes===0&&(t.memoizedState=t.updateQueue.baseState=u),tl(t,Ue,a)):(a=n.cache,tl(t,Ue,a),a!==u.cache&&$i(t,[Ue],l,!0))),Ye(e,t,t.pendingProps.children,l),t.child;case 29:throw t.pendingProps}throw Error(f(156,t.tag))}function kt(e){e.flags|=4}function yo(e,t){if(t.type!=="stylesheet"||(t.state.loading&4)!==0)e.flags&=-16777217;else if(e.flags|=16777216,!Ad(t)){if(t=pt.current,t!==null&&((ie&4194048)===ie?zt!==null:(ie&62914560)!==ie&&(ie&536870912)===0||t!==zt))throw Pa=Ii,Ff;e.flags|=8192}}function On(e,t){t!==null&&(e.flags|=4),e.flags&16384&&(t=e.tag!==22?Zr():536870912,e.lanes|=t,pa|=t)}function ru(e,t){if(!fe)switch(e.tailMode){case"hidden":t=e.tail;for(var l=null;t!==null;)t.alternate!==null&&(l=t),t=t.sibling;l===null?e.tail=null:l.sibling=null;break;case"collapsed":l=e.tail;for(var a=null;l!==null;)l.alternate!==null&&(a=l),l=l.sibling;a===null?t||e.tail===null?e.tail=null:e.tail.sibling=null:a.sibling=null}}function Ae(e){var t=e.alternate!==null&&e.alternate.child===e.child,l=0,a=0;if(t)for(var u=e.child;u!==null;)l|=u.lanes|u.childLanes,a|=u.subtreeFlags&65011712,a|=u.flags&65011712,u.return=e,u=u.sibling;else for(u=e.child;u!==null;)l|=u.lanes|u.childLanes,a|=u.subtreeFlags,a|=u.flags,u.return=e,u=u.sibling;return e.subtreeFlags|=a,e.childLanes=l,t}function Bh(e,t,l){var a=t.pendingProps;switch(Zi(t),t.tag){case 31:case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return Ae(t),null;case 1:return Ae(t),null;case 3:return l=t.stateNode,a=null,e!==null&&(a=e.memoizedState.cache),t.memoizedState.cache!==a&&(t.flags|=2048),Yt(Ue),Ft(),l.pendingContext&&(l.context=l.pendingContext,l.pendingContext=null),(e===null||e.child===null)&&(Za(t)?kt(t):e===null||e.memoizedState.isDehydrated&&(t.flags&256)===0||(t.flags|=1024,Vf())),Ae(t),null;case 26:return l=t.memoizedState,e===null?(kt(t),l!==null?(Ae(t),yo(t,l)):(Ae(t),t.flags&=-16777217)):l?l!==e.memoizedState?(kt(t),Ae(t),yo(t,l)):(Ae(t),t.flags&=-16777217):(e.memoizedProps!==a&&kt(t),Ae(t),t.flags&=-16777217),null;case 27:Yu(t),l=I.current;var u=t.type;if(e!==null&&t.stateNode!=null)e.memoizedProps!==a&&kt(t);else{if(!a){if(t.stateNode===null)throw Error(f(166));return Ae(t),null}e=V.current,Za(t)?Xf(t):(e=yd(u,a,l),t.stateNode=e,kt(t))}return Ae(t),null;case 5:if(Yu(t),l=t.type,e!==null&&t.stateNode!=null)e.memoizedProps!==a&&kt(t);else{if(!a){if(t.stateNode===null)throw Error(f(166));return Ae(t),null}if(e=V.current,Za(t))Xf(t);else{switch(u=Qn(I.current),e){case 1:e=u.createElementNS("http://www.w3.org/2000/svg",l);break;case 2:e=u.createElementNS("http://www.w3.org/1998/Math/MathML",l);break;default:switch(l){case"svg":e=u.createElementNS("http://www.w3.org/2000/svg",l);break;case"math":e=u.createElementNS("http://www.w3.org/1998/Math/MathML",l);break;case"script":e=u.createElement("div"),e.innerHTML="<script><\/script>",e=e.removeChild(e.firstChild);break;case"select":e=typeof a.is=="string"?u.createElement("select",{is:a.is}):u.createElement("select"),a.multiple?e.multiple=!0:a.size&&(e.size=a.size);break;default:e=typeof a.is=="string"?u.createElement(l,{is:a.is}):u.createElement(l)}}e[Xe]=t,e[$e]=a;e:for(u=t.child;u!==null;){if(u.tag===5||u.tag===6)e.appendChild(u.stateNode);else if(u.tag!==4&&u.tag!==27&&u.child!==null){u.child.return=u,u=u.child;continue}if(u===t)break e;for(;u.sibling===null;){if(u.return===null||u.return===t)break e;u=u.return}u.sibling.return=u.return,u=u.sibling}t.stateNode=e;e:switch(Ge(e,l,a),l){case"button":case"input":case"select":case"textarea":e=!!a.autoFocus;break e;case"img":e=!0;break e;default:e=!1}e&&kt(t)}}return Ae(t),t.flags&=-16777217,null;case 6:if(e&&t.stateNode!=null)e.memoizedProps!==a&&kt(t);else{if(typeof a!="string"&&t.stateNode===null)throw Error(f(166));if(e=I.current,Za(t)){if(e=t.stateNode,l=t.memoizedProps,a=null,u=Je,u!==null)switch(u.tag){case 27:case 5:a=u.memoizedProps}e[Xe]=t,e=!!(e.nodeValue===l||a!==null&&a.suppressHydrationWarning===!0||rd(e.nodeValue,l)),e||wl(t)}else e=Qn(e).createTextNode(a),e[Xe]=t,t.stateNode=e}return Ae(t),null;case 13:if(a=t.memoizedState,e===null||e.memoizedState!==null&&e.memoizedState.dehydrated!==null){if(u=Za(t),a!==null&&a.dehydrated!==null){if(e===null){if(!u)throw Error(f(318));if(u=t.memoizedState,u=u!==null?u.dehydrated:null,!u)throw Error(f(317));u[Xe]=t}else Va(),(t.flags&128)===0&&(t.memoizedState=null),t.flags|=4;Ae(t),u=!1}else u=Vf(),e!==null&&e.memoizedState!==null&&(e.memoizedState.hydrationErrors=u),u=!0;if(!u)return t.flags&256?(Gt(t),t):(Gt(t),null)}if(Gt(t),(t.flags&128)!==0)return t.lanes=l,t;if(l=a!==null,e=e!==null&&e.memoizedState!==null,l){a=t.child,u=null,a.alternate!==null&&a.alternate.memoizedState!==null&&a.alternate.memoizedState.cachePool!==null&&(u=a.alternate.memoizedState.cachePool.pool);var n=null;a.memoizedState!==null&&a.memoizedState.cachePool!==null&&(n=a.memoizedState.cachePool.pool),n!==u&&(a.flags|=2048)}return l!==e&&l&&(t.child.flags|=8192),On(t,t.updateQueue),Ae(t),null;case 4:return Ft(),e===null&&ar(t.stateNode.containerInfo),Ae(t),null;case 10:return Yt(t.type),Ae(t),null;case 19:if(G(we),u=t.memoizedState,u===null)return Ae(t),null;if(a=(t.flags&128)!==0,n=u.rendering,n===null)if(a)ru(u,!1);else{if(Re!==0||e!==null&&(e.flags&128)!==0)for(e=t.child;e!==null;){if(n=Tn(e),n!==null){for(t.flags|=128,ru(u,!1),e=n.updateQueue,t.updateQueue=e,On(t,e),t.subtreeFlags=0,e=l,l=t.child;l!==null;)Qf(l,e),l=l.sibling;return q(we,we.current&1|2),t.child}e=e.sibling}u.tail!==null&&Rt()>Dn&&(t.flags|=128,a=!0,ru(u,!1),t.lanes=4194304)}else{if(!a)if(e=Tn(n),e!==null){if(t.flags|=128,a=!0,e=e.updateQueue,t.updateQueue=e,On(t,e),ru(u,!0),u.tail===null&&u.tailMode==="hidden"&&!n.alternate&&!fe)return Ae(t),null}else 2*Rt()-u.renderingStartTime>Dn&&l!==536870912&&(t.flags|=128,a=!0,ru(u,!1),t.lanes=4194304);u.isBackwards?(n.sibling=t.child,t.child=n):(e=u.last,e!==null?e.sibling=n:t.child=n,u.last=n)}return u.tail!==null?(t=u.tail,u.rendering=t,u.tail=t.sibling,u.renderingStartTime=Rt(),t.sibling=null,e=we.current,q(we,a?e&1|2:e&1),t):(Ae(t),null);case 22:case 23:return Gt(t),nc(),a=t.memoizedState!==null,e!==null?e.memoizedState!==null!==a&&(t.flags|=8192):a&&(t.flags|=8192),a?(l&536870912)!==0&&(t.flags&128)===0&&(Ae(t),t.subtreeFlags&6&&(t.flags|=8192)):Ae(t),l=t.updateQueue,l!==null&&On(t,l.retryQueue),l=null,e!==null&&e.memoizedState!==null&&e.memoizedState.cachePool!==null&&(l=e.memoizedState.cachePool.pool),a=null,t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(a=t.memoizedState.cachePool.pool),a!==l&&(t.flags|=2048),e!==null&&G(Bl),null;case 24:return l=null,e!==null&&(l=e.memoizedState.cache),t.memoizedState.cache!==l&&(t.flags|=2048),Yt(Ue),Ae(t),null;case 25:return null;case 30:return null}throw Error(f(156,t.tag))}function qh(e,t){switch(Zi(t),t.tag){case 1:return e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 3:return Yt(Ue),Ft(),e=t.flags,(e&65536)!==0&&(e&128)===0?(t.flags=e&-65537|128,t):null;case 26:case 27:case 5:return Yu(t),null;case 13:if(Gt(t),e=t.memoizedState,e!==null&&e.dehydrated!==null){if(t.alternate===null)throw Error(f(340));Va()}return e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 19:return G(we),null;case 4:return Ft(),null;case 10:return Yt(t.type),null;case 22:case 23:return Gt(t),nc(),e!==null&&G(Bl),e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 24:return Yt(Ue),null;case 25:return null;default:return null}}function go(e,t){switch(Zi(t),t.tag){case 3:Yt(Ue),Ft();break;case 26:case 27:case 5:Yu(t);break;case 4:Ft();break;case 13:Gt(t);break;case 19:G(we);break;case 10:Yt(t.type);break;case 22:case 23:Gt(t),nc(),e!==null&&G(Bl);break;case 24:Yt(Ue)}}function fu(e,t){try{var l=t.updateQueue,a=l!==null?l.lastEffect:null;if(a!==null){var u=a.next;l=u;do{if((l.tag&e)===e){a=void 0;var n=l.create,i=l.inst;a=n(),i.destroy=a}l=l.next}while(l!==u)}}catch(r){ve(t,t.return,r)}}function rl(e,t,l){try{var a=t.updateQueue,u=a!==null?a.lastEffect:null;if(u!==null){var n=u.next;a=n;do{if((a.tag&e)===e){var i=a.inst,r=i.destroy;if(r!==void 0){i.destroy=void 0,u=t;var y=l,E=r;try{E()}catch(O){ve(u,y,O)}}}a=a.next}while(a!==n)}}catch(O){ve(t,t.return,O)}}function vo(e){var t=e.updateQueue;if(t!==null){var l=e.stateNode;try{as(t,l)}catch(a){ve(e,e.return,a)}}}function po(e,t,l){l.props=Yl(e.type,e.memoizedProps),l.state=e.memoizedState;try{l.componentWillUnmount()}catch(a){ve(e,t,a)}}function su(e,t){try{var l=e.ref;if(l!==null){switch(e.tag){case 26:case 27:case 5:var a=e.stateNode;break;case 30:a=e.stateNode;break;default:a=e.stateNode}typeof l=="function"?e.refCleanup=l(a):l.current=a}}catch(u){ve(e,t,u)}}function Ot(e,t){var l=e.ref,a=e.refCleanup;if(l!==null)if(typeof a=="function")try{a()}catch(u){ve(e,t,u)}finally{e.refCleanup=null,e=e.alternate,e!=null&&(e.refCleanup=null)}else if(typeof l=="function")try{l(null)}catch(u){ve(e,t,u)}else l.current=null}function bo(e){var t=e.type,l=e.memoizedProps,a=e.stateNode;try{e:switch(t){case"button":case"input":case"select":case"textarea":l.autoFocus&&a.focus();break e;case"img":l.src?a.src=l.src:l.srcSet&&(a.srcset=l.srcSet)}}catch(u){ve(e,e.return,u)}}function Uc(e,t,l){try{var a=e.stateNode;nm(a,e.type,l,t),a[$e]=t}catch(u){ve(e,e.return,u)}}function xo(e){return e.tag===5||e.tag===3||e.tag===26||e.tag===27&&gl(e.type)||e.tag===4}function wc(e){e:for(;;){for(;e.sibling===null;){if(e.return===null||xo(e.return))return null;e=e.return}for(e.sibling.return=e.return,e=e.sibling;e.tag!==5&&e.tag!==6&&e.tag!==18;){if(e.tag===27&&gl(e.type)||e.flags&2||e.child===null||e.tag===4)continue e;e.child.return=e,e=e.child}if(!(e.flags&2))return e.stateNode}}function Cc(e,t,l){var a=e.tag;if(a===5||a===6)e=e.stateNode,t?(l.nodeType===9?l.body:l.nodeName==="HTML"?l.ownerDocument.body:l).insertBefore(e,t):(t=l.nodeType===9?l.body:l.nodeName==="HTML"?l.ownerDocument.body:l,t.appendChild(e),l=l._reactRootContainer,l!=null||t.onclick!==null||(t.onclick=Gn));else if(a!==4&&(a===27&&gl(e.type)&&(l=e.stateNode,t=null),e=e.child,e!==null))for(Cc(e,t,l),e=e.sibling;e!==null;)Cc(e,t,l),e=e.sibling}function Mn(e,t,l){var a=e.tag;if(a===5||a===6)e=e.stateNode,t?l.insertBefore(e,t):l.appendChild(e);else if(a!==4&&(a===27&&gl(e.type)&&(l=e.stateNode),e=e.child,e!==null))for(Mn(e,t,l),e=e.sibling;e!==null;)Mn(e,t,l),e=e.sibling}function So(e){var t=e.stateNode,l=e.memoizedProps;try{for(var a=e.type,u=t.attributes;u.length;)t.removeAttributeNode(u[0]);Ge(t,a,l),t[Xe]=e,t[$e]=l}catch(n){ve(e,e.return,n)}}var Xt=!1,Oe=!1,Hc=!1,Eo=typeof WeakSet=="function"?WeakSet:Set,Be=null;function Yh(e,t){if(e=e.containerInfo,ir=Jn,e=Df(e),wi(e)){if("selectionStart"in e)var l={start:e.selectionStart,end:e.selectionEnd};else e:{l=(l=e.ownerDocument)&&l.defaultView||window;var a=l.getSelection&&l.getSelection();if(a&&a.rangeCount!==0){l=a.anchorNode;var u=a.anchorOffset,n=a.focusNode;a=a.focusOffset;try{l.nodeType,n.nodeType}catch{l=null;break e}var i=0,r=-1,y=-1,E=0,O=0,U=e,T=null;t:for(;;){for(var N;U!==l||u!==0&&U.nodeType!==3||(r=i+u),U!==n||a!==0&&U.nodeType!==3||(y=i+a),U.nodeType===3&&(i+=U.nodeValue.length),(N=U.firstChild)!==null;)T=U,U=N;for(;;){if(U===e)break t;if(T===l&&++E===u&&(r=i),T===n&&++O===a&&(y=i),(N=U.nextSibling)!==null)break;U=T,T=U.parentNode}U=N}l=r===-1||y===-1?null:{start:r,end:y}}else l=null}l=l||{start:0,end:0}}else l=null;for(cr={focusedElem:e,selectionRange:l},Jn=!1,Be=t;Be!==null;)if(t=Be,e=t.child,(t.subtreeFlags&1024)!==0&&e!==null)e.return=t,Be=e;else for(;Be!==null;){switch(t=Be,n=t.alternate,e=t.flags,t.tag){case 0:break;case 11:case 15:break;case 1:if((e&1024)!==0&&n!==null){e=void 0,l=t,u=n.memoizedProps,n=n.memoizedState,a=l.stateNode;try{var W=Yl(l.type,u,l.elementType===l.type);e=a.getSnapshotBeforeUpdate(W,n),a.__reactInternalSnapshotBeforeUpdate=e}catch(K){ve(l,l.return,K)}}break;case 3:if((e&1024)!==0){if(e=t.stateNode.containerInfo,l=e.nodeType,l===9)sr(e);else if(l===1)switch(e.nodeName){case"HEAD":case"HTML":case"BODY":sr(e);break;default:e.textContent=""}}break;case 5:case 26:case 27:case 6:case 4:case 17:break;default:if((e&1024)!==0)throw Error(f(163))}if(e=t.sibling,e!==null){e.return=t.return,Be=e;break}Be=t.return}}function Ao(e,t,l){var a=l.flags;switch(l.tag){case 0:case 11:case 15:fl(e,l),a&4&&fu(5,l);break;case 1:if(fl(e,l),a&4)if(e=l.stateNode,t===null)try{e.componentDidMount()}catch(i){ve(l,l.return,i)}else{var u=Yl(l.type,t.memoizedProps);t=t.memoizedState;try{e.componentDidUpdate(u,t,e.__reactInternalSnapshotBeforeUpdate)}catch(i){ve(l,l.return,i)}}a&64&&vo(l),a&512&&su(l,l.return);break;case 3:if(fl(e,l),a&64&&(e=l.updateQueue,e!==null)){if(t=null,l.child!==null)switch(l.child.tag){case 27:case 5:t=l.child.stateNode;break;case 1:t=l.child.stateNode}try{as(e,t)}catch(i){ve(l,l.return,i)}}break;case 27:t===null&&a&4&&So(l);case 26:case 5:fl(e,l),t===null&&a&4&&bo(l),a&512&&su(l,l.return);break;case 12:fl(e,l);break;case 13:fl(e,l),a&4&&Ro(e,l),a&64&&(e=l.memoizedState,e!==null&&(e=e.dehydrated,e!==null&&(l=Jh.bind(null,l),dm(e,l))));break;case 22:if(a=l.memoizedState!==null||Xt,!a){t=t!==null&&t.memoizedState!==null||Oe,u=Xt;var n=Oe;Xt=a,(Oe=t)&&!n?sl(e,l,(l.subtreeFlags&8772)!==0):fl(e,l),Xt=u,Oe=n}break;case 30:break;default:fl(e,l)}}function To(e){var t=e.alternate;t!==null&&(e.alternate=null,To(t)),e.child=null,e.deletions=null,e.sibling=null,e.tag===5&&(t=e.stateNode,t!==null&&yi(t)),e.stateNode=null,e.return=null,e.dependencies=null,e.memoizedProps=null,e.memoizedState=null,e.pendingProps=null,e.stateNode=null,e.updateQueue=null}var Se=null,Pe=!1;function Zt(e,t,l){for(l=l.child;l!==null;)No(e,t,l),l=l.sibling}function No(e,t,l){if(lt&&typeof lt.onCommitFiberUnmount=="function")try{lt.onCommitFiberUnmount(_a,l)}catch{}switch(l.tag){case 26:Oe||Ot(l,t),Zt(e,t,l),l.memoizedState?l.memoizedState.count--:l.stateNode&&(l=l.stateNode,l.parentNode.removeChild(l));break;case 27:Oe||Ot(l,t);var a=Se,u=Pe;gl(l.type)&&(Se=l.stateNode,Pe=!1),Zt(e,t,l),bu(l.stateNode),Se=a,Pe=u;break;case 5:Oe||Ot(l,t);case 6:if(a=Se,u=Pe,Se=null,Zt(e,t,l),Se=a,Pe=u,Se!==null)if(Pe)try{(Se.nodeType===9?Se.body:Se.nodeName==="HTML"?Se.ownerDocument.body:Se).removeChild(l.stateNode)}catch(n){ve(l,t,n)}else try{Se.removeChild(l.stateNode)}catch(n){ve(l,t,n)}break;case 18:Se!==null&&(Pe?(e=Se,hd(e.nodeType===9?e.body:e.nodeName==="HTML"?e.ownerDocument.body:e,l.stateNode),ju(e)):hd(Se,l.stateNode));break;case 4:a=Se,u=Pe,Se=l.stateNode.containerInfo,Pe=!0,Zt(e,t,l),Se=a,Pe=u;break;case 0:case 11:case 14:case 15:Oe||rl(2,l,t),Oe||rl(4,l,t),Zt(e,t,l);break;case 1:Oe||(Ot(l,t),a=l.stateNode,typeof a.componentWillUnmount=="function"&&po(l,t,a)),Zt(e,t,l);break;case 21:Zt(e,t,l);break;case 22:Oe=(a=Oe)||l.memoizedState!==null,Zt(e,t,l),Oe=a;break;default:Zt(e,t,l)}}function Ro(e,t){if(t.memoizedState===null&&(e=t.alternate,e!==null&&(e=e.memoizedState,e!==null&&(e=e.dehydrated,e!==null))))try{ju(e)}catch(l){ve(t,t.return,l)}}function Lh(e){switch(e.tag){case 13:case 19:var t=e.stateNode;return t===null&&(t=e.stateNode=new Eo),t;case 22:return e=e.stateNode,t=e._retryCache,t===null&&(t=e._retryCache=new Eo),t;default:throw Error(f(435,e.tag))}}function Bc(e,t){var l=Lh(e);t.forEach(function(a){var u=$h.bind(null,e,a);l.has(a)||(l.add(a),a.then(u,u))})}function it(e,t){var l=t.deletions;if(l!==null)for(var a=0;a<l.length;a++){var u=l[a],n=e,i=t,r=i;e:for(;r!==null;){switch(r.tag){case 27:if(gl(r.type)){Se=r.stateNode,Pe=!1;break e}break;case 5:Se=r.stateNode,Pe=!1;break e;case 3:case 4:Se=r.stateNode.containerInfo,Pe=!0;break e}r=r.return}if(Se===null)throw Error(f(160));No(n,i,u),Se=null,Pe=!1,n=u.alternate,n!==null&&(n.return=null),u.return=null}if(t.subtreeFlags&13878)for(t=t.child;t!==null;)jo(t,e),t=t.sibling}var Et=null;function jo(e,t){var l=e.alternate,a=e.flags;switch(e.tag){case 0:case 11:case 14:case 15:it(t,e),ct(e),a&4&&(rl(3,e,e.return),fu(3,e),rl(5,e,e.return));break;case 1:it(t,e),ct(e),a&512&&(Oe||l===null||Ot(l,l.return)),a&64&&Xt&&(e=e.updateQueue,e!==null&&(a=e.callbacks,a!==null&&(l=e.shared.hiddenCallbacks,e.shared.hiddenCallbacks=l===null?a:l.concat(a))));break;case 26:var u=Et;if(it(t,e),ct(e),a&512&&(Oe||l===null||Ot(l,l.return)),a&4){var n=l!==null?l.memoizedState:null;if(a=e.memoizedState,l===null)if(a===null)if(e.stateNode===null){e:{a=e.type,l=e.memoizedProps,u=u.ownerDocument||u;t:switch(a){case"title":n=u.getElementsByTagName("title")[0],(!n||n[wa]||n[Xe]||n.namespaceURI==="http://www.w3.org/2000/svg"||n.hasAttribute("itemprop"))&&(n=u.createElement(a),u.head.insertBefore(n,u.querySelector("head > title"))),Ge(n,a,l),n[Xe]=e,Ce(n),a=n;break e;case"link":var i=Sd("link","href",u).get(a+(l.href||""));if(i){for(var r=0;r<i.length;r++)if(n=i[r],n.getAttribute("href")===(l.href==null||l.href===""?null:l.href)&&n.getAttribute("rel")===(l.rel==null?null:l.rel)&&n.getAttribute("title")===(l.title==null?null:l.title)&&n.getAttribute("crossorigin")===(l.crossOrigin==null?null:l.crossOrigin)){i.splice(r,1);break t}}n=u.createElement(a),Ge(n,a,l),u.head.appendChild(n);break;case"meta":if(i=Sd("meta","content",u).get(a+(l.content||""))){for(r=0;r<i.length;r++)if(n=i[r],n.getAttribute("content")===(l.content==null?null:""+l.content)&&n.getAttribute("name")===(l.name==null?null:l.name)&&n.getAttribute("property")===(l.property==null?null:l.property)&&n.getAttribute("http-equiv")===(l.httpEquiv==null?null:l.httpEquiv)&&n.getAttribute("charset")===(l.charSet==null?null:l.charSet)){i.splice(r,1);break t}}n=u.createElement(a),Ge(n,a,l),u.head.appendChild(n);break;default:throw Error(f(468,a))}n[Xe]=e,Ce(n),a=n}e.stateNode=a}else Ed(u,e.type,e.stateNode);else e.stateNode=xd(u,a,e.memoizedProps);else n!==a?(n===null?l.stateNode!==null&&(l=l.stateNode,l.parentNode.removeChild(l)):n.count--,a===null?Ed(u,e.type,e.stateNode):xd(u,a,e.memoizedProps)):a===null&&e.stateNode!==null&&Uc(e,e.memoizedProps,l.memoizedProps)}break;case 27:it(t,e),ct(e),a&512&&(Oe||l===null||Ot(l,l.return)),l!==null&&a&4&&Uc(e,e.memoizedProps,l.memoizedProps);break;case 5:if(it(t,e),ct(e),a&512&&(Oe||l===null||Ot(l,l.return)),e.flags&32){u=e.stateNode;try{Wl(u,"")}catch(N){ve(e,e.return,N)}}a&4&&e.stateNode!=null&&(u=e.memoizedProps,Uc(e,u,l!==null?l.memoizedProps:u)),a&1024&&(Hc=!0);break;case 6:if(it(t,e),ct(e),a&4){if(e.stateNode===null)throw Error(f(162));a=e.memoizedProps,l=e.stateNode;try{l.nodeValue=a}catch(N){ve(e,e.return,N)}}break;case 3:if(Zn=null,u=Et,Et=kn(t.containerInfo),it(t,e),Et=u,ct(e),a&4&&l!==null&&l.memoizedState.isDehydrated)try{ju(t.containerInfo)}catch(N){ve(e,e.return,N)}Hc&&(Hc=!1,zo(e));break;case 4:a=Et,Et=kn(e.stateNode.containerInfo),it(t,e),ct(e),Et=a;break;case 12:it(t,e),ct(e);break;case 13:it(t,e),ct(e),e.child.flags&8192&&e.memoizedState!==null!=(l!==null&&l.memoizedState!==null)&&(kc=Rt()),a&4&&(a=e.updateQueue,a!==null&&(e.updateQueue=null,Bc(e,a)));break;case 22:u=e.memoizedState!==null;var y=l!==null&&l.memoizedState!==null,E=Xt,O=Oe;if(Xt=E||u,Oe=O||y,it(t,e),Oe=O,Xt=E,ct(e),a&8192)e:for(t=e.stateNode,t._visibility=u?t._visibility&-2:t._visibility|1,u&&(l===null||y||Xt||Oe||Ll(e)),l=null,t=e;;){if(t.tag===5||t.tag===26){if(l===null){y=l=t;try{if(n=y.stateNode,u)i=n.style,typeof i.setProperty=="function"?i.setProperty("display","none","important"):i.display="none";else{r=y.stateNode;var U=y.memoizedProps.style,T=U!=null&&U.hasOwnProperty("display")?U.display:null;r.style.display=T==null||typeof T=="boolean"?"":(""+T).trim()}}catch(N){ve(y,y.return,N)}}}else if(t.tag===6){if(l===null){y=t;try{y.stateNode.nodeValue=u?"":y.memoizedProps}catch(N){ve(y,y.return,N)}}}else if((t.tag!==22&&t.tag!==23||t.memoizedState===null||t===e)&&t.child!==null){t.child.return=t,t=t.child;continue}if(t===e)break e;for(;t.sibling===null;){if(t.return===null||t.return===e)break e;l===t&&(l=null),t=t.return}l===t&&(l=null),t.sibling.return=t.return,t=t.sibling}a&4&&(a=e.updateQueue,a!==null&&(l=a.retryQueue,l!==null&&(a.retryQueue=null,Bc(e,l))));break;case 19:it(t,e),ct(e),a&4&&(a=e.updateQueue,a!==null&&(e.updateQueue=null,Bc(e,a)));break;case 30:break;case 21:break;default:it(t,e),ct(e)}}function ct(e){var t=e.flags;if(t&2){try{for(var l,a=e.return;a!==null;){if(xo(a)){l=a;break}a=a.return}if(l==null)throw Error(f(160));switch(l.tag){case 27:var u=l.stateNode,n=wc(e);Mn(e,n,u);break;case 5:var i=l.stateNode;l.flags&32&&(Wl(i,""),l.flags&=-33);var r=wc(e);Mn(e,r,i);break;case 3:case 4:var y=l.stateNode.containerInfo,E=wc(e);Cc(e,E,y);break;default:throw Error(f(161))}}catch(O){ve(e,e.return,O)}e.flags&=-3}t&4096&&(e.flags&=-4097)}function zo(e){if(e.subtreeFlags&1024)for(e=e.child;e!==null;){var t=e;zo(t),t.tag===5&&t.flags&1024&&t.stateNode.reset(),e=e.sibling}}function fl(e,t){if(t.subtreeFlags&8772)for(t=t.child;t!==null;)Ao(e,t.alternate,t),t=t.sibling}function Ll(e){for(e=e.child;e!==null;){var t=e;switch(t.tag){case 0:case 11:case 14:case 15:rl(4,t,t.return),Ll(t);break;case 1:Ot(t,t.return);var l=t.stateNode;typeof l.componentWillUnmount=="function"&&po(t,t.return,l),Ll(t);break;case 27:bu(t.stateNode);case 26:case 5:Ot(t,t.return),Ll(t);break;case 22:t.memoizedState===null&&Ll(t);break;case 30:Ll(t);break;default:Ll(t)}e=e.sibling}}function sl(e,t,l){for(l=l&&(t.subtreeFlags&8772)!==0,t=t.child;t!==null;){var a=t.alternate,u=e,n=t,i=n.flags;switch(n.tag){case 0:case 11:case 15:sl(u,n,l),fu(4,n);break;case 1:if(sl(u,n,l),a=n,u=a.stateNode,typeof u.componentDidMount=="function")try{u.componentDidMount()}catch(E){ve(a,a.return,E)}if(a=n,u=a.updateQueue,u!==null){var r=a.stateNode;try{var y=u.shared.hiddenCallbacks;if(y!==null)for(u.shared.hiddenCallbacks=null,u=0;u<y.length;u++)ls(y[u],r)}catch(E){ve(a,a.return,E)}}l&&i&64&&vo(n),su(n,n.return);break;case 27:So(n);case 26:case 5:sl(u,n,l),l&&a===null&&i&4&&bo(n),su(n,n.return);break;case 12:sl(u,n,l);break;case 13:sl(u,n,l),l&&i&4&&Ro(u,n);break;case 22:n.memoizedState===null&&sl(u,n,l),su(n,n.return);break;case 30:break;default:sl(u,n,l)}t=t.sibling}}function qc(e,t){var l=null;e!==null&&e.memoizedState!==null&&e.memoizedState.cachePool!==null&&(l=e.memoizedState.cachePool.pool),e=null,t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(e=t.memoizedState.cachePool.pool),e!==l&&(e!=null&&e.refCount++,l!=null&&$a(l))}function Yc(e,t){e=null,t.alternate!==null&&(e=t.alternate.memoizedState.cache),t=t.memoizedState.cache,t!==e&&(t.refCount++,e!=null&&$a(e))}function Mt(e,t,l,a){if(t.subtreeFlags&10256)for(t=t.child;t!==null;)Oo(e,t,l,a),t=t.sibling}function Oo(e,t,l,a){var u=t.flags;switch(t.tag){case 0:case 11:case 15:Mt(e,t,l,a),u&2048&&fu(9,t);break;case 1:Mt(e,t,l,a);break;case 3:Mt(e,t,l,a),u&2048&&(e=null,t.alternate!==null&&(e=t.alternate.memoizedState.cache),t=t.memoizedState.cache,t!==e&&(t.refCount++,e!=null&&$a(e)));break;case 12:if(u&2048){Mt(e,t,l,a),e=t.stateNode;try{var n=t.memoizedProps,i=n.id,r=n.onPostCommit;typeof r=="function"&&r(i,t.alternate===null?"mount":"update",e.passiveEffectDuration,-0)}catch(y){ve(t,t.return,y)}}else Mt(e,t,l,a);break;case 13:Mt(e,t,l,a);break;case 23:break;case 22:n=t.stateNode,i=t.alternate,t.memoizedState!==null?n._visibility&2?Mt(e,t,l,a):ou(e,t):n._visibility&2?Mt(e,t,l,a):(n._visibility|=2,ya(e,t,l,a,(t.subtreeFlags&10256)!==0)),u&2048&&qc(i,t);break;case 24:Mt(e,t,l,a),u&2048&&Yc(t.alternate,t);break;default:Mt(e,t,l,a)}}function ya(e,t,l,a,u){for(u=u&&(t.subtreeFlags&10256)!==0,t=t.child;t!==null;){var n=e,i=t,r=l,y=a,E=i.flags;switch(i.tag){case 0:case 11:case 15:ya(n,i,r,y,u),fu(8,i);break;case 23:break;case 22:var O=i.stateNode;i.memoizedState!==null?O._visibility&2?ya(n,i,r,y,u):ou(n,i):(O._visibility|=2,ya(n,i,r,y,u)),u&&E&2048&&qc(i.alternate,i);break;case 24:ya(n,i,r,y,u),u&&E&2048&&Yc(i.alternate,i);break;default:ya(n,i,r,y,u)}t=t.sibling}}function ou(e,t){if(t.subtreeFlags&10256)for(t=t.child;t!==null;){var l=e,a=t,u=a.flags;switch(a.tag){case 22:ou(l,a),u&2048&&qc(a.alternate,a);break;case 24:ou(l,a),u&2048&&Yc(a.alternate,a);break;default:ou(l,a)}t=t.sibling}}var du=8192;function ga(e){if(e.subtreeFlags&du)for(e=e.child;e!==null;)Mo(e),e=e.sibling}function Mo(e){switch(e.tag){case 26:ga(e),e.flags&du&&e.memoizedState!==null&&Nm(Et,e.memoizedState,e.memoizedProps);break;case 5:ga(e);break;case 3:case 4:var t=Et;Et=kn(e.stateNode.containerInfo),ga(e),Et=t;break;case 22:e.memoizedState===null&&(t=e.alternate,t!==null&&t.memoizedState!==null?(t=du,du=16777216,ga(e),du=t):ga(e));break;default:ga(e)}}function _o(e){var t=e.alternate;if(t!==null&&(e=t.child,e!==null)){t.child=null;do t=e.sibling,e.sibling=null,e=t;while(e!==null)}}function hu(e){var t=e.deletions;if((e.flags&16)!==0){if(t!==null)for(var l=0;l<t.length;l++){var a=t[l];Be=a,Uo(a,e)}_o(e)}if(e.subtreeFlags&10256)for(e=e.child;e!==null;)Do(e),e=e.sibling}function Do(e){switch(e.tag){case 0:case 11:case 15:hu(e),e.flags&2048&&rl(9,e,e.return);break;case 3:hu(e);break;case 12:hu(e);break;case 22:var t=e.stateNode;e.memoizedState!==null&&t._visibility&2&&(e.return===null||e.return.tag!==13)?(t._visibility&=-3,_n(e)):hu(e);break;default:hu(e)}}function _n(e){var t=e.deletions;if((e.flags&16)!==0){if(t!==null)for(var l=0;l<t.length;l++){var a=t[l];Be=a,Uo(a,e)}_o(e)}for(e=e.child;e!==null;){switch(t=e,t.tag){case 0:case 11:case 15:rl(8,t,t.return),_n(t);break;case 22:l=t.stateNode,l._visibility&2&&(l._visibility&=-3,_n(t));break;default:_n(t)}e=e.sibling}}function Uo(e,t){for(;Be!==null;){var l=Be;switch(l.tag){case 0:case 11:case 15:rl(8,l,t);break;case 23:case 22:if(l.memoizedState!==null&&l.memoizedState.cachePool!==null){var a=l.memoizedState.cachePool.pool;a!=null&&a.refCount++}break;case 24:$a(l.memoizedState.cache)}if(a=l.child,a!==null)a.return=l,Be=a;else e:for(l=e;Be!==null;){a=Be;var u=a.sibling,n=a.return;if(To(a),a===l){Be=null;break e}if(u!==null){u.return=n,Be=u;break e}Be=n}}}var Gh={getCacheForType:function(e){var t=Ze(Ue),l=t.data.get(e);return l===void 0&&(l=e(),t.data.set(e,l)),l}},Qh=typeof WeakMap=="function"?WeakMap:Map,se=0,pe=null,ue=null,ie=0,oe=0,rt=null,ol=!1,va=!1,Lc=!1,Vt=0,Re=0,dl=0,Gl=0,Gc=0,bt=0,pa=0,mu=null,Ie=null,Qc=!1,kc=0,Dn=1/0,Un=null,hl=null,Le=0,ml=null,ba=null,xa=0,Xc=0,Zc=null,wo=null,yu=0,Vc=null;function ft(){if((se&2)!==0&&ie!==0)return ie&-ie;if(M.T!==null){var e=ca;return e!==0?e:Ic()}return Jr()}function Co(){bt===0&&(bt=(ie&536870912)===0||fe?Xr():536870912);var e=pt.current;return e!==null&&(e.flags|=32),bt}function st(e,t,l){(e===pe&&(oe===2||oe===9)||e.cancelPendingCommit!==null)&&(Sa(e,0),yl(e,ie,bt,!1)),Ua(e,l),((se&2)===0||e!==pe)&&(e===pe&&((se&2)===0&&(Gl|=l),Re===4&&yl(e,ie,bt,!1)),_t(e))}function Ho(e,t,l){if((se&6)!==0)throw Error(f(327));var a=!l&&(t&124)===0&&(t&e.expiredLanes)===0||Da(e,t),u=a?Zh(e,t):$c(e,t,!0),n=a;do{if(u===0){va&&!a&&yl(e,t,0,!1);break}else{if(l=e.current.alternate,n&&!kh(l)){u=$c(e,t,!1),n=!1;continue}if(u===2){if(n=t,e.errorRecoveryDisabledLanes&n)var i=0;else i=e.pendingLanes&-536870913,i=i!==0?i:i&536870912?536870912:0;if(i!==0){t=i;e:{var r=e;u=mu;var y=r.current.memoizedState.isDehydrated;if(y&&(Sa(r,i).flags|=256),i=$c(r,i,!1),i!==2){if(Lc&&!y){r.errorRecoveryDisabledLanes|=n,Gl|=n,u=4;break e}n=Ie,Ie=u,n!==null&&(Ie===null?Ie=n:Ie.push.apply(Ie,n))}u=i}if(n=!1,u!==2)continue}}if(u===1){Sa(e,0),yl(e,t,0,!0);break}e:{switch(a=e,n=u,n){case 0:case 1:throw Error(f(345));case 4:if((t&4194048)!==t)break;case 6:yl(a,t,bt,!ol);break e;case 2:Ie=null;break;case 3:case 5:break;default:throw Error(f(329))}if((t&62914560)===t&&(u=kc+300-Rt(),10<u)){if(yl(a,t,bt,!ol),ku(a,0,!0)!==0)break e;a.timeoutHandle=od(Bo.bind(null,a,l,Ie,Un,Qc,t,bt,Gl,pa,ol,n,2,-0,0),u);break e}Bo(a,l,Ie,Un,Qc,t,bt,Gl,pa,ol,n,0,-0,0)}}break}while(!0);_t(e)}function Bo(e,t,l,a,u,n,i,r,y,E,O,U,T,N){if(e.timeoutHandle=-1,U=t.subtreeFlags,(U&8192||(U&16785408)===16785408)&&(Eu={stylesheets:null,count:0,unsuspend:Tm},Mo(t),U=Rm(),U!==null)){e.cancelPendingCommit=U(Xo.bind(null,e,t,n,l,a,u,i,r,y,O,1,T,N)),yl(e,n,i,!E);return}Xo(e,t,n,l,a,u,i,r,y)}function kh(e){for(var t=e;;){var l=t.tag;if((l===0||l===11||l===15)&&t.flags&16384&&(l=t.updateQueue,l!==null&&(l=l.stores,l!==null)))for(var a=0;a<l.length;a++){var u=l[a],n=u.getSnapshot;u=u.value;try{if(!ut(n(),u))return!1}catch{return!1}}if(l=t.child,t.subtreeFlags&16384&&l!==null)l.return=t,t=l;else{if(t===e)break;for(;t.sibling===null;){if(t.return===null||t.return===e)return!0;t=t.return}t.sibling.return=t.return,t=t.sibling}}return!0}function yl(e,t,l,a){t&=~Gc,t&=~Gl,e.suspendedLanes|=t,e.pingedLanes&=~t,a&&(e.warmLanes|=t),a=e.expirationTimes;for(var u=t;0<u;){var n=31-at(u),i=1<<n;a[n]=-1,u&=~i}l!==0&&Vr(e,l,t)}function wn(){return(se&6)===0?(gu(0),!1):!0}function Kc(){if(ue!==null){if(oe===0)var e=ue.return;else e=ue,qt=Cl=null,sc(e),ha=null,iu=0,e=ue;for(;e!==null;)go(e.alternate,e),e=e.return;ue=null}}function Sa(e,t){var l=e.timeoutHandle;l!==-1&&(e.timeoutHandle=-1,cm(l)),l=e.cancelPendingCommit,l!==null&&(e.cancelPendingCommit=null,l()),Kc(),pe=e,ue=l=Ct(e.current,null),ie=t,oe=0,rt=null,ol=!1,va=Da(e,t),Lc=!1,pa=bt=Gc=Gl=dl=Re=0,Ie=mu=null,Qc=!1,(t&8)!==0&&(t|=t&32);var a=e.entangledLanes;if(a!==0)for(e=e.entanglements,a&=t;0<a;){var u=31-at(a),n=1<<u;t|=e[u],a&=~n}return Vt=t,tn(),l}function qo(e,t){le=null,M.H=Sn,t===Fa||t===on?(t=es(),oe=3):t===Ff?(t=es(),oe=4):oe=t===to?8:t!==null&&typeof t=="object"&&typeof t.then=="function"?6:1,rt=t,ue===null&&(Re=1,Rn(e,mt(t,e.current)))}function Yo(){var e=M.H;return M.H=Sn,e===null?Sn:e}function Lo(){var e=M.A;return M.A=Gh,e}function Jc(){Re=4,ol||(ie&4194048)!==ie&&pt.current!==null||(va=!0),(dl&134217727)===0&&(Gl&134217727)===0||pe===null||yl(pe,ie,bt,!1)}function $c(e,t,l){var a=se;se|=2;var u=Yo(),n=Lo();(pe!==e||ie!==t)&&(Un=null,Sa(e,t)),t=!1;var i=Re;e:do try{if(oe!==0&&ue!==null){var r=ue,y=rt;switch(oe){case 8:Kc(),i=6;break e;case 3:case 2:case 9:case 6:pt.current===null&&(t=!0);var E=oe;if(oe=0,rt=null,Ea(e,r,y,E),l&&va){i=0;break e}break;default:E=oe,oe=0,rt=null,Ea(e,r,y,E)}}Xh(),i=Re;break}catch(O){qo(e,O)}while(!0);return t&&e.shellSuspendCounter++,qt=Cl=null,se=a,M.H=u,M.A=n,ue===null&&(pe=null,ie=0,tn()),i}function Xh(){for(;ue!==null;)Go(ue)}function Zh(e,t){var l=se;se|=2;var a=Yo(),u=Lo();pe!==e||ie!==t?(Un=null,Dn=Rt()+500,Sa(e,t)):va=Da(e,t);e:do try{if(oe!==0&&ue!==null){t=ue;var n=rt;t:switch(oe){case 1:oe=0,rt=null,Ea(e,t,n,1);break;case 2:case 9:if(Pf(n)){oe=0,rt=null,Qo(t);break}t=function(){oe!==2&&oe!==9||pe!==e||(oe=7),_t(e)},n.then(t,t);break e;case 3:oe=7;break e;case 4:oe=5;break e;case 7:Pf(n)?(oe=0,rt=null,Qo(t)):(oe=0,rt=null,Ea(e,t,n,7));break;case 5:var i=null;switch(ue.tag){case 26:i=ue.memoizedState;case 5:case 27:var r=ue;if(!i||Ad(i)){oe=0,rt=null;var y=r.sibling;if(y!==null)ue=y;else{var E=r.return;E!==null?(ue=E,Cn(E)):ue=null}break t}}oe=0,rt=null,Ea(e,t,n,5);break;case 6:oe=0,rt=null,Ea(e,t,n,6);break;case 8:Kc(),Re=6;break e;default:throw Error(f(462))}}Vh();break}catch(O){qo(e,O)}while(!0);return qt=Cl=null,M.H=a,M.A=u,se=l,ue!==null?0:(pe=null,ie=0,tn(),Re)}function Vh(){for(;ue!==null&&!m0();)Go(ue)}function Go(e){var t=mo(e.alternate,e,Vt);e.memoizedProps=e.pendingProps,t===null?Cn(e):ue=t}function Qo(e){var t=e,l=t.alternate;switch(t.tag){case 15:case 0:t=co(l,t,t.pendingProps,t.type,void 0,ie);break;case 11:t=co(l,t,t.pendingProps,t.type.render,t.ref,ie);break;case 5:sc(t);default:go(l,t),t=ue=Qf(t,Vt),t=mo(l,t,Vt)}e.memoizedProps=e.pendingProps,t===null?Cn(e):ue=t}function Ea(e,t,l,a){qt=Cl=null,sc(t),ha=null,iu=0;var u=t.return;try{if(Ch(e,u,t,l,ie)){Re=1,Rn(e,mt(l,e.current)),ue=null;return}}catch(n){if(u!==null)throw ue=u,n;Re=1,Rn(e,mt(l,e.current)),ue=null;return}t.flags&32768?(fe||a===1?e=!0:va||(ie&536870912)!==0?e=!1:(ol=e=!0,(a===2||a===9||a===3||a===6)&&(a=pt.current,a!==null&&a.tag===13&&(a.flags|=16384))),ko(t,e)):Cn(t)}function Cn(e){var t=e;do{if((t.flags&32768)!==0){ko(t,ol);return}e=t.return;var l=Bh(t.alternate,t,Vt);if(l!==null){ue=l;return}if(t=t.sibling,t!==null){ue=t;return}ue=t=e}while(t!==null);Re===0&&(Re=5)}function ko(e,t){do{var l=qh(e.alternate,e);if(l!==null){l.flags&=32767,ue=l;return}if(l=e.return,l!==null&&(l.flags|=32768,l.subtreeFlags=0,l.deletions=null),!t&&(e=e.sibling,e!==null)){ue=e;return}ue=e=l}while(e!==null);Re=6,ue=null}function Xo(e,t,l,a,u,n,i,r,y){e.cancelPendingCommit=null;do Hn();while(Le!==0);if((se&6)!==0)throw Error(f(327));if(t!==null){if(t===e.current)throw Error(f(177));if(n=t.lanes|t.childLanes,n|=Yi,T0(e,l,n,i,r,y),e===pe&&(ue=pe=null,ie=0),ba=t,ml=e,xa=l,Xc=n,Zc=u,wo=a,(t.subtreeFlags&10256)!==0||(t.flags&10256)!==0?(e.callbackNode=null,e.callbackPriority=0,Wh(Lu,function(){return $o(),null})):(e.callbackNode=null,e.callbackPriority=0),a=(t.flags&13878)!==0,(t.subtreeFlags&13878)!==0||a){a=M.T,M.T=null,u=Y.p,Y.p=2,i=se,se|=4;try{Yh(e,t,l)}finally{se=i,Y.p=u,M.T=a}}Le=1,Zo(),Vo(),Ko()}}function Zo(){if(Le===1){Le=0;var e=ml,t=ba,l=(t.flags&13878)!==0;if((t.subtreeFlags&13878)!==0||l){l=M.T,M.T=null;var a=Y.p;Y.p=2;var u=se;se|=4;try{jo(t,e);var n=cr,i=Df(e.containerInfo),r=n.focusedElem,y=n.selectionRange;if(i!==r&&r&&r.ownerDocument&&_f(r.ownerDocument.documentElement,r)){if(y!==null&&wi(r)){var E=y.start,O=y.end;if(O===void 0&&(O=E),"selectionStart"in r)r.selectionStart=E,r.selectionEnd=Math.min(O,r.value.length);else{var U=r.ownerDocument||document,T=U&&U.defaultView||window;if(T.getSelection){var N=T.getSelection(),W=r.textContent.length,K=Math.min(y.start,W),ye=y.end===void 0?K:Math.min(y.end,W);!N.extend&&K>ye&&(i=ye,ye=K,K=i);var x=Mf(r,K),p=Mf(r,ye);if(x&&p&&(N.rangeCount!==1||N.anchorNode!==x.node||N.anchorOffset!==x.offset||N.focusNode!==p.node||N.focusOffset!==p.offset)){var S=U.createRange();S.setStart(x.node,x.offset),N.removeAllRanges(),K>ye?(N.addRange(S),N.extend(p.node,p.offset)):(S.setEnd(p.node,p.offset),N.addRange(S))}}}}for(U=[],N=r;N=N.parentNode;)N.nodeType===1&&U.push({element:N,left:N.scrollLeft,top:N.scrollTop});for(typeof r.focus=="function"&&r.focus(),r=0;r<U.length;r++){var _=U[r];_.element.scrollLeft=_.left,_.element.scrollTop=_.top}}Jn=!!ir,cr=ir=null}finally{se=u,Y.p=a,M.T=l}}e.current=t,Le=2}}function Vo(){if(Le===2){Le=0;var e=ml,t=ba,l=(t.flags&8772)!==0;if((t.subtreeFlags&8772)!==0||l){l=M.T,M.T=null;var a=Y.p;Y.p=2;var u=se;se|=4;try{Ao(e,t.alternate,t)}finally{se=u,Y.p=a,M.T=l}}Le=3}}function Ko(){if(Le===4||Le===3){Le=0,y0();var e=ml,t=ba,l=xa,a=wo;(t.subtreeFlags&10256)!==0||(t.flags&10256)!==0?Le=5:(Le=0,ba=ml=null,Jo(e,e.pendingLanes));var u=e.pendingLanes;if(u===0&&(hl=null),hi(l),t=t.stateNode,lt&&typeof lt.onCommitFiberRoot=="function")try{lt.onCommitFiberRoot(_a,t,void 0,(t.current.flags&128)===128)}catch{}if(a!==null){t=M.T,u=Y.p,Y.p=2,M.T=null;try{for(var n=e.onRecoverableError,i=0;i<a.length;i++){var r=a[i];n(r.value,{componentStack:r.stack})}}finally{M.T=t,Y.p=u}}(xa&3)!==0&&Hn(),_t(e),u=e.pendingLanes,(l&4194090)!==0&&(u&42)!==0?e===Vc?yu++:(yu=0,Vc=e):yu=0,gu(0)}}function Jo(e,t){(e.pooledCacheLanes&=t)===0&&(t=e.pooledCache,t!=null&&(e.pooledCache=null,$a(t)))}function Hn(e){return Zo(),Vo(),Ko(),$o()}function $o(){if(Le!==5)return!1;var e=ml,t=Xc;Xc=0;var l=hi(xa),a=M.T,u=Y.p;try{Y.p=32>l?32:l,M.T=null,l=Zc,Zc=null;var n=ml,i=xa;if(Le=0,ba=ml=null,xa=0,(se&6)!==0)throw Error(f(331));var r=se;if(se|=4,Do(n.current),Oo(n,n.current,i,l),se=r,gu(0,!1),lt&&typeof lt.onPostCommitFiberRoot=="function")try{lt.onPostCommitFiberRoot(_a,n)}catch{}return!0}finally{Y.p=u,M.T=a,Jo(e,t)}}function Wo(e,t,l){t=mt(l,t),t=Tc(e.stateNode,t,2),e=ul(e,t,2),e!==null&&(Ua(e,2),_t(e))}function ve(e,t,l){if(e.tag===3)Wo(e,e,l);else for(;t!==null;){if(t.tag===3){Wo(t,e,l);break}else if(t.tag===1){var a=t.stateNode;if(typeof t.type.getDerivedStateFromError=="function"||typeof a.componentDidCatch=="function"&&(hl===null||!hl.has(a))){e=mt(l,e),l=Is(2),a=ul(t,l,2),a!==null&&(eo(l,a,t,e),Ua(a,2),_t(a));break}}t=t.return}}function Wc(e,t,l){var a=e.pingCache;if(a===null){a=e.pingCache=new Qh;var u=new Set;a.set(t,u)}else u=a.get(t),u===void 0&&(u=new Set,a.set(t,u));u.has(l)||(Lc=!0,u.add(l),e=Kh.bind(null,e,t,l),t.then(e,e))}function Kh(e,t,l){var a=e.pingCache;a!==null&&a.delete(t),e.pingedLanes|=e.suspendedLanes&l,e.warmLanes&=~l,pe===e&&(ie&l)===l&&(Re===4||Re===3&&(ie&62914560)===ie&&300>Rt()-kc?(se&2)===0&&Sa(e,0):Gc|=l,pa===ie&&(pa=0)),_t(e)}function Fo(e,t){t===0&&(t=Zr()),e=aa(e,t),e!==null&&(Ua(e,t),_t(e))}function Jh(e){var t=e.memoizedState,l=0;t!==null&&(l=t.retryLane),Fo(e,l)}function $h(e,t){var l=0;switch(e.tag){case 13:var a=e.stateNode,u=e.memoizedState;u!==null&&(l=u.retryLane);break;case 19:a=e.stateNode;break;case 22:a=e.stateNode._retryCache;break;default:throw Error(f(314))}a!==null&&a.delete(t),Fo(e,l)}function Wh(e,t){return fi(e,t)}var Bn=null,Aa=null,Fc=!1,qn=!1,Pc=!1,Ql=0;function _t(e){e!==Aa&&e.next===null&&(Aa===null?Bn=Aa=e:Aa=Aa.next=e),qn=!0,Fc||(Fc=!0,Ph())}function gu(e,t){if(!Pc&&qn){Pc=!0;do for(var l=!1,a=Bn;a!==null;){if(e!==0){var u=a.pendingLanes;if(u===0)var n=0;else{var i=a.suspendedLanes,r=a.pingedLanes;n=(1<<31-at(42|e)+1)-1,n&=u&~(i&~r),n=n&201326741?n&201326741|1:n?n|2:0}n!==0&&(l=!0,td(a,n))}else n=ie,n=ku(a,a===pe?n:0,a.cancelPendingCommit!==null||a.timeoutHandle!==-1),(n&3)===0||Da(a,n)||(l=!0,td(a,n));a=a.next}while(l);Pc=!1}}function Fh(){Po()}function Po(){qn=Fc=!1;var e=0;Ql!==0&&(im()&&(e=Ql),Ql=0);for(var t=Rt(),l=null,a=Bn;a!==null;){var u=a.next,n=Io(a,t);n===0?(a.next=null,l===null?Bn=u:l.next=u,u===null&&(Aa=l)):(l=a,(e!==0||(n&3)!==0)&&(qn=!0)),a=u}gu(e)}function Io(e,t){for(var l=e.suspendedLanes,a=e.pingedLanes,u=e.expirationTimes,n=e.pendingLanes&-62914561;0<n;){var i=31-at(n),r=1<<i,y=u[i];y===-1?((r&l)===0||(r&a)!==0)&&(u[i]=A0(r,t)):y<=t&&(e.expiredLanes|=r),n&=~r}if(t=pe,l=ie,l=ku(e,e===t?l:0,e.cancelPendingCommit!==null||e.timeoutHandle!==-1),a=e.callbackNode,l===0||e===t&&(oe===2||oe===9)||e.cancelPendingCommit!==null)return a!==null&&a!==null&&si(a),e.callbackNode=null,e.callbackPriority=0;if((l&3)===0||Da(e,l)){if(t=l&-l,t===e.callbackPriority)return t;switch(a!==null&&si(a),hi(l)){case 2:case 8:l=Qr;break;case 32:l=Lu;break;case 268435456:l=kr;break;default:l=Lu}return a=ed.bind(null,e),l=fi(l,a),e.callbackPriority=t,e.callbackNode=l,t}return a!==null&&a!==null&&si(a),e.callbackPriority=2,e.callbackNode=null,2}function ed(e,t){if(Le!==0&&Le!==5)return e.callbackNode=null,e.callbackPriority=0,null;var l=e.callbackNode;if(Hn()&&e.callbackNode!==l)return null;var a=ie;return a=ku(e,e===pe?a:0,e.cancelPendingCommit!==null||e.timeoutHandle!==-1),a===0?null:(Ho(e,a,t),Io(e,Rt()),e.callbackNode!=null&&e.callbackNode===l?ed.bind(null,e):null)}function td(e,t){if(Hn())return null;Ho(e,t,!0)}function Ph(){rm(function(){(se&6)!==0?fi(Gr,Fh):Po()})}function Ic(){return Ql===0&&(Ql=Xr()),Ql}function ld(e){return e==null||typeof e=="symbol"||typeof e=="boolean"?null:typeof e=="function"?e:Ju(""+e)}function ad(e,t){var l=t.ownerDocument.createElement("input");return l.name=t.name,l.value=t.value,e.id&&l.setAttribute("form",e.id),t.parentNode.insertBefore(l,t),e=new FormData(e),l.parentNode.removeChild(l),e}function Ih(e,t,l,a,u){if(t==="submit"&&l&&l.stateNode===u){var n=ld((u[$e]||null).action),i=a.submitter;i&&(t=(t=i[$e]||null)?ld(t.formAction):i.getAttribute("formAction"),t!==null&&(n=t,i=null));var r=new Pu("action","action",null,a,u);e.push({event:r,listeners:[{instance:null,listener:function(){if(a.defaultPrevented){if(Ql!==0){var y=i?ad(u,i):new FormData(u);bc(l,{pending:!0,data:y,method:u.method,action:n},null,y)}}else typeof n=="function"&&(r.preventDefault(),y=i?ad(u,i):new FormData(u),bc(l,{pending:!0,data:y,method:u.method,action:n},n,y))},currentTarget:u}]})}}for(var er=0;er<qi.length;er++){var tr=qi[er],em=tr.toLowerCase(),tm=tr[0].toUpperCase()+tr.slice(1);St(em,"on"+tm)}St(Cf,"onAnimationEnd"),St(Hf,"onAnimationIteration"),St(Bf,"onAnimationStart"),St("dblclick","onDoubleClick"),St("focusin","onFocus"),St("focusout","onBlur"),St(ph,"onTransitionRun"),St(bh,"onTransitionStart"),St(xh,"onTransitionCancel"),St(qf,"onTransitionEnd"),Kl("onMouseEnter",["mouseout","mouseover"]),Kl("onMouseLeave",["mouseout","mouseover"]),Kl("onPointerEnter",["pointerout","pointerover"]),Kl("onPointerLeave",["pointerout","pointerover"]),Rl("onChange","change click focusin focusout input keydown keyup selectionchange".split(" ")),Rl("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" ")),Rl("onBeforeInput",["compositionend","keypress","textInput","paste"]),Rl("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" ")),Rl("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" ")),Rl("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var vu="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),lm=new Set("beforetoggle cancel close invalid load scroll scrollend toggle".split(" ").concat(vu));function ud(e,t){t=(t&4)!==0;for(var l=0;l<e.length;l++){var a=e[l],u=a.event;a=a.listeners;e:{var n=void 0;if(t)for(var i=a.length-1;0<=i;i--){var r=a[i],y=r.instance,E=r.currentTarget;if(r=r.listener,y!==n&&u.isPropagationStopped())break e;n=r,u.currentTarget=E;try{n(u)}catch(O){Nn(O)}u.currentTarget=null,n=y}else for(i=0;i<a.length;i++){if(r=a[i],y=r.instance,E=r.currentTarget,r=r.listener,y!==n&&u.isPropagationStopped())break e;n=r,u.currentTarget=E;try{n(u)}catch(O){Nn(O)}u.currentTarget=null,n=y}}}}function ne(e,t){var l=t[mi];l===void 0&&(l=t[mi]=new Set);var a=e+"__bubble";l.has(a)||(nd(t,e,2,!1),l.add(a))}function lr(e,t,l){var a=0;t&&(a|=4),nd(l,e,a,t)}var Yn="_reactListening"+Math.random().toString(36).slice(2);function ar(e){if(!e[Yn]){e[Yn]=!0,Wr.forEach(function(l){l!=="selectionchange"&&(lm.has(l)||lr(l,!1,e),lr(l,!0,e))});var t=e.nodeType===9?e:e.ownerDocument;t===null||t[Yn]||(t[Yn]=!0,lr("selectionchange",!1,t))}}function nd(e,t,l,a){switch(Od(t)){case 2:var u=Om;break;case 8:u=Mm;break;default:u=vr}l=u.bind(null,t,l,e),u=void 0,!Ni||t!=="touchstart"&&t!=="touchmove"&&t!=="wheel"||(u=!0),a?u!==void 0?e.addEventListener(t,l,{capture:!0,passive:u}):e.addEventListener(t,l,!0):u!==void 0?e.addEventListener(t,l,{passive:u}):e.addEventListener(t,l,!1)}function ur(e,t,l,a,u){var n=a;if((t&1)===0&&(t&2)===0&&a!==null)e:for(;;){if(a===null)return;var i=a.tag;if(i===3||i===4){var r=a.stateNode.containerInfo;if(r===u)break;if(i===4)for(i=a.return;i!==null;){var y=i.tag;if((y===3||y===4)&&i.stateNode.containerInfo===u)return;i=i.return}for(;r!==null;){if(i=Xl(r),i===null)return;if(y=i.tag,y===5||y===6||y===26||y===27){a=n=i;continue e}r=r.parentNode}}a=a.return}of(function(){var E=n,O=Ai(l),U=[];e:{var T=Yf.get(e);if(T!==void 0){var N=Pu,W=e;switch(e){case"keypress":if(Wu(l)===0)break e;case"keydown":case"keyup":N=W0;break;case"focusin":W="focus",N=Oi;break;case"focusout":W="blur",N=Oi;break;case"beforeblur":case"afterblur":N=Oi;break;case"click":if(l.button===2)break e;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":N=mf;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":N=q0;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":N=I0;break;case Cf:case Hf:case Bf:N=G0;break;case qf:N=th;break;case"scroll":case"scrollend":N=H0;break;case"wheel":N=ah;break;case"copy":case"cut":case"paste":N=k0;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":N=gf;break;case"toggle":case"beforetoggle":N=nh}var K=(t&4)!==0,ye=!K&&(e==="scroll"||e==="scrollend"),x=K?T!==null?T+"Capture":null:T;K=[];for(var p=E,S;p!==null;){var _=p;if(S=_.stateNode,_=_.tag,_!==5&&_!==26&&_!==27||S===null||x===null||(_=Ha(p,x),_!=null&&K.push(pu(p,_,S))),ye)break;p=p.return}0<K.length&&(T=new N(T,W,null,l,O),U.push({event:T,listeners:K}))}}if((t&7)===0){e:{if(T=e==="mouseover"||e==="pointerover",N=e==="mouseout"||e==="pointerout",T&&l!==Ei&&(W=l.relatedTarget||l.fromElement)&&(Xl(W)||W[kl]))break e;if((N||T)&&(T=O.window===O?O:(T=O.ownerDocument)?T.defaultView||T.parentWindow:window,N?(W=l.relatedTarget||l.toElement,N=E,W=W?Xl(W):null,W!==null&&(ye=g(W),K=W.tag,W!==ye||K!==5&&K!==27&&K!==6)&&(W=null)):(N=null,W=E),N!==W)){if(K=mf,_="onMouseLeave",x="onMouseEnter",p="mouse",(e==="pointerout"||e==="pointerover")&&(K=gf,_="onPointerLeave",x="onPointerEnter",p="pointer"),ye=N==null?T:Ca(N),S=W==null?T:Ca(W),T=new K(_,p+"leave",N,l,O),T.target=ye,T.relatedTarget=S,_=null,Xl(O)===E&&(K=new K(x,p+"enter",W,l,O),K.target=S,K.relatedTarget=ye,_=K),ye=_,N&&W)t:{for(K=N,x=W,p=0,S=K;S;S=Ta(S))p++;for(S=0,_=x;_;_=Ta(_))S++;for(;0<p-S;)K=Ta(K),p--;for(;0<S-p;)x=Ta(x),S--;for(;p--;){if(K===x||x!==null&&K===x.alternate)break t;K=Ta(K),x=Ta(x)}K=null}else K=null;N!==null&&id(U,T,N,K,!1),W!==null&&ye!==null&&id(U,ye,W,K,!0)}}e:{if(T=E?Ca(E):window,N=T.nodeName&&T.nodeName.toLowerCase(),N==="select"||N==="input"&&T.type==="file")var k=Tf;else if(Ef(T))if(Nf)k=yh;else{k=hh;var ae=dh}else N=T.nodeName,!N||N.toLowerCase()!=="input"||T.type!=="checkbox"&&T.type!=="radio"?E&&Si(E.elementType)&&(k=Tf):k=mh;if(k&&(k=k(e,E))){Af(U,k,l,O);break e}ae&&ae(e,T,E),e==="focusout"&&E&&T.type==="number"&&E.memoizedProps.value!=null&&xi(T,"number",T.value)}switch(ae=E?Ca(E):window,e){case"focusin":(Ef(ae)||ae.contentEditable==="true")&&(ea=ae,Ci=E,Xa=null);break;case"focusout":Xa=Ci=ea=null;break;case"mousedown":Hi=!0;break;case"contextmenu":case"mouseup":case"dragend":Hi=!1,Uf(U,l,O);break;case"selectionchange":if(vh)break;case"keydown":case"keyup":Uf(U,l,O)}var Z;if(_i)e:{switch(e){case"compositionstart":var J="onCompositionStart";break e;case"compositionend":J="onCompositionEnd";break e;case"compositionupdate":J="onCompositionUpdate";break e}J=void 0}else Il?xf(e,l)&&(J="onCompositionEnd"):e==="keydown"&&l.keyCode===229&&(J="onCompositionStart");J&&(vf&&l.locale!=="ko"&&(Il||J!=="onCompositionStart"?J==="onCompositionEnd"&&Il&&(Z=df()):(el=O,Ri="value"in el?el.value:el.textContent,Il=!0)),ae=Ln(E,J),0<ae.length&&(J=new yf(J,e,null,l,O),U.push({event:J,listeners:ae}),Z?J.data=Z:(Z=Sf(l),Z!==null&&(J.data=Z)))),(Z=ch?rh(e,l):fh(e,l))&&(J=Ln(E,"onBeforeInput"),0<J.length&&(ae=new yf("onBeforeInput","beforeinput",null,l,O),U.push({event:ae,listeners:J}),ae.data=Z)),Ih(U,e,E,l,O)}ud(U,t)})}function pu(e,t,l){return{instance:e,listener:t,currentTarget:l}}function Ln(e,t){for(var l=t+"Capture",a=[];e!==null;){var u=e,n=u.stateNode;if(u=u.tag,u!==5&&u!==26&&u!==27||n===null||(u=Ha(e,l),u!=null&&a.unshift(pu(e,u,n)),u=Ha(e,t),u!=null&&a.push(pu(e,u,n))),e.tag===3)return a;e=e.return}return[]}function Ta(e){if(e===null)return null;do e=e.return;while(e&&e.tag!==5&&e.tag!==27);return e||null}function id(e,t,l,a,u){for(var n=t._reactName,i=[];l!==null&&l!==a;){var r=l,y=r.alternate,E=r.stateNode;if(r=r.tag,y!==null&&y===a)break;r!==5&&r!==26&&r!==27||E===null||(y=E,u?(E=Ha(l,n),E!=null&&i.unshift(pu(l,E,y))):u||(E=Ha(l,n),E!=null&&i.push(pu(l,E,y)))),l=l.return}i.length!==0&&e.push({event:t,listeners:i})}var am=/\r\n?/g,um=/\u0000|\uFFFD/g;function cd(e){return(typeof e=="string"?e:""+e).replace(am,`
`).replace(um,"")}function rd(e,t){return t=cd(t),cd(e)===t}function Gn(){}function me(e,t,l,a,u,n){switch(l){case"children":typeof a=="string"?t==="body"||t==="textarea"&&a===""||Wl(e,a):(typeof a=="number"||typeof a=="bigint")&&t!=="body"&&Wl(e,""+a);break;case"className":Zu(e,"class",a);break;case"tabIndex":Zu(e,"tabindex",a);break;case"dir":case"role":case"viewBox":case"width":case"height":Zu(e,l,a);break;case"style":ff(e,a,n);break;case"data":if(t!=="object"){Zu(e,"data",a);break}case"src":case"href":if(a===""&&(t!=="a"||l!=="href")){e.removeAttribute(l);break}if(a==null||typeof a=="function"||typeof a=="symbol"||typeof a=="boolean"){e.removeAttribute(l);break}a=Ju(""+a),e.setAttribute(l,a);break;case"action":case"formAction":if(typeof a=="function"){e.setAttribute(l,"javascript:throw new Error('A React form was unexpectedly submitted. If you called form.submit() manually, consider using form.requestSubmit() instead. If you\\'re trying to use event.stopPropagation() in a submit event handler, consider also calling event.preventDefault().')");break}else typeof n=="function"&&(l==="formAction"?(t!=="input"&&me(e,t,"name",u.name,u,null),me(e,t,"formEncType",u.formEncType,u,null),me(e,t,"formMethod",u.formMethod,u,null),me(e,t,"formTarget",u.formTarget,u,null)):(me(e,t,"encType",u.encType,u,null),me(e,t,"method",u.method,u,null),me(e,t,"target",u.target,u,null)));if(a==null||typeof a=="symbol"||typeof a=="boolean"){e.removeAttribute(l);break}a=Ju(""+a),e.setAttribute(l,a);break;case"onClick":a!=null&&(e.onclick=Gn);break;case"onScroll":a!=null&&ne("scroll",e);break;case"onScrollEnd":a!=null&&ne("scrollend",e);break;case"dangerouslySetInnerHTML":if(a!=null){if(typeof a!="object"||!("__html"in a))throw Error(f(61));if(l=a.__html,l!=null){if(u.children!=null)throw Error(f(60));e.innerHTML=l}}break;case"multiple":e.multiple=a&&typeof a!="function"&&typeof a!="symbol";break;case"muted":e.muted=a&&typeof a!="function"&&typeof a!="symbol";break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"defaultValue":case"defaultChecked":case"innerHTML":case"ref":break;case"autoFocus":break;case"xlinkHref":if(a==null||typeof a=="function"||typeof a=="boolean"||typeof a=="symbol"){e.removeAttribute("xlink:href");break}l=Ju(""+a),e.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href",l);break;case"contentEditable":case"spellCheck":case"draggable":case"value":case"autoReverse":case"externalResourcesRequired":case"focusable":case"preserveAlpha":a!=null&&typeof a!="function"&&typeof a!="symbol"?e.setAttribute(l,""+a):e.removeAttribute(l);break;case"inert":case"allowFullScreen":case"async":case"autoPlay":case"controls":case"default":case"defer":case"disabled":case"disablePictureInPicture":case"disableRemotePlayback":case"formNoValidate":case"hidden":case"loop":case"noModule":case"noValidate":case"open":case"playsInline":case"readOnly":case"required":case"reversed":case"scoped":case"seamless":case"itemScope":a&&typeof a!="function"&&typeof a!="symbol"?e.setAttribute(l,""):e.removeAttribute(l);break;case"capture":case"download":a===!0?e.setAttribute(l,""):a!==!1&&a!=null&&typeof a!="function"&&typeof a!="symbol"?e.setAttribute(l,a):e.removeAttribute(l);break;case"cols":case"rows":case"size":case"span":a!=null&&typeof a!="function"&&typeof a!="symbol"&&!isNaN(a)&&1<=a?e.setAttribute(l,a):e.removeAttribute(l);break;case"rowSpan":case"start":a==null||typeof a=="function"||typeof a=="symbol"||isNaN(a)?e.removeAttribute(l):e.setAttribute(l,a);break;case"popover":ne("beforetoggle",e),ne("toggle",e),Xu(e,"popover",a);break;case"xlinkActuate":Ut(e,"http://www.w3.org/1999/xlink","xlink:actuate",a);break;case"xlinkArcrole":Ut(e,"http://www.w3.org/1999/xlink","xlink:arcrole",a);break;case"xlinkRole":Ut(e,"http://www.w3.org/1999/xlink","xlink:role",a);break;case"xlinkShow":Ut(e,"http://www.w3.org/1999/xlink","xlink:show",a);break;case"xlinkTitle":Ut(e,"http://www.w3.org/1999/xlink","xlink:title",a);break;case"xlinkType":Ut(e,"http://www.w3.org/1999/xlink","xlink:type",a);break;case"xmlBase":Ut(e,"http://www.w3.org/XML/1998/namespace","xml:base",a);break;case"xmlLang":Ut(e,"http://www.w3.org/XML/1998/namespace","xml:lang",a);break;case"xmlSpace":Ut(e,"http://www.w3.org/XML/1998/namespace","xml:space",a);break;case"is":Xu(e,"is",a);break;case"innerText":case"textContent":break;default:(!(2<l.length)||l[0]!=="o"&&l[0]!=="O"||l[1]!=="n"&&l[1]!=="N")&&(l=w0.get(l)||l,Xu(e,l,a))}}function nr(e,t,l,a,u,n){switch(l){case"style":ff(e,a,n);break;case"dangerouslySetInnerHTML":if(a!=null){if(typeof a!="object"||!("__html"in a))throw Error(f(61));if(l=a.__html,l!=null){if(u.children!=null)throw Error(f(60));e.innerHTML=l}}break;case"children":typeof a=="string"?Wl(e,a):(typeof a=="number"||typeof a=="bigint")&&Wl(e,""+a);break;case"onScroll":a!=null&&ne("scroll",e);break;case"onScrollEnd":a!=null&&ne("scrollend",e);break;case"onClick":a!=null&&(e.onclick=Gn);break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"innerHTML":case"ref":break;case"innerText":case"textContent":break;default:if(!Fr.hasOwnProperty(l))e:{if(l[0]==="o"&&l[1]==="n"&&(u=l.endsWith("Capture"),t=l.slice(2,u?l.length-7:void 0),n=e[$e]||null,n=n!=null?n[l]:null,typeof n=="function"&&e.removeEventListener(t,n,u),typeof a=="function")){typeof n!="function"&&n!==null&&(l in e?e[l]=null:e.hasAttribute(l)&&e.removeAttribute(l)),e.addEventListener(t,a,u);break e}l in e?e[l]=a:a===!0?e.setAttribute(l,""):Xu(e,l,a)}}}function Ge(e,t,l){switch(t){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"img":ne("error",e),ne("load",e);var a=!1,u=!1,n;for(n in l)if(l.hasOwnProperty(n)){var i=l[n];if(i!=null)switch(n){case"src":a=!0;break;case"srcSet":u=!0;break;case"children":case"dangerouslySetInnerHTML":throw Error(f(137,t));default:me(e,t,n,i,l,null)}}u&&me(e,t,"srcSet",l.srcSet,l,null),a&&me(e,t,"src",l.src,l,null);return;case"input":ne("invalid",e);var r=n=i=u=null,y=null,E=null;for(a in l)if(l.hasOwnProperty(a)){var O=l[a];if(O!=null)switch(a){case"name":u=O;break;case"type":i=O;break;case"checked":y=O;break;case"defaultChecked":E=O;break;case"value":n=O;break;case"defaultValue":r=O;break;case"children":case"dangerouslySetInnerHTML":if(O!=null)throw Error(f(137,t));break;default:me(e,t,a,O,l,null)}}uf(e,n,r,y,E,i,u,!1),Vu(e);return;case"select":ne("invalid",e),a=i=n=null;for(u in l)if(l.hasOwnProperty(u)&&(r=l[u],r!=null))switch(u){case"value":n=r;break;case"defaultValue":i=r;break;case"multiple":a=r;default:me(e,t,u,r,l,null)}t=n,l=i,e.multiple=!!a,t!=null?$l(e,!!a,t,!1):l!=null&&$l(e,!!a,l,!0);return;case"textarea":ne("invalid",e),n=u=a=null;for(i in l)if(l.hasOwnProperty(i)&&(r=l[i],r!=null))switch(i){case"value":a=r;break;case"defaultValue":u=r;break;case"children":n=r;break;case"dangerouslySetInnerHTML":if(r!=null)throw Error(f(91));break;default:me(e,t,i,r,l,null)}cf(e,a,u,n),Vu(e);return;case"option":for(y in l)if(l.hasOwnProperty(y)&&(a=l[y],a!=null))switch(y){case"selected":e.selected=a&&typeof a!="function"&&typeof a!="symbol";break;default:me(e,t,y,a,l,null)}return;case"dialog":ne("beforetoggle",e),ne("toggle",e),ne("cancel",e),ne("close",e);break;case"iframe":case"object":ne("load",e);break;case"video":case"audio":for(a=0;a<vu.length;a++)ne(vu[a],e);break;case"image":ne("error",e),ne("load",e);break;case"details":ne("toggle",e);break;case"embed":case"source":case"link":ne("error",e),ne("load",e);case"area":case"base":case"br":case"col":case"hr":case"keygen":case"meta":case"param":case"track":case"wbr":case"menuitem":for(E in l)if(l.hasOwnProperty(E)&&(a=l[E],a!=null))switch(E){case"children":case"dangerouslySetInnerHTML":throw Error(f(137,t));default:me(e,t,E,a,l,null)}return;default:if(Si(t)){for(O in l)l.hasOwnProperty(O)&&(a=l[O],a!==void 0&&nr(e,t,O,a,l,void 0));return}}for(r in l)l.hasOwnProperty(r)&&(a=l[r],a!=null&&me(e,t,r,a,l,null))}function nm(e,t,l,a){switch(t){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"input":var u=null,n=null,i=null,r=null,y=null,E=null,O=null;for(N in l){var U=l[N];if(l.hasOwnProperty(N)&&U!=null)switch(N){case"checked":break;case"value":break;case"defaultValue":y=U;default:a.hasOwnProperty(N)||me(e,t,N,null,a,U)}}for(var T in a){var N=a[T];if(U=l[T],a.hasOwnProperty(T)&&(N!=null||U!=null))switch(T){case"type":n=N;break;case"name":u=N;break;case"checked":E=N;break;case"defaultChecked":O=N;break;case"value":i=N;break;case"defaultValue":r=N;break;case"children":case"dangerouslySetInnerHTML":if(N!=null)throw Error(f(137,t));break;default:N!==U&&me(e,t,T,N,a,U)}}bi(e,i,r,y,E,O,n,u);return;case"select":N=i=r=T=null;for(n in l)if(y=l[n],l.hasOwnProperty(n)&&y!=null)switch(n){case"value":break;case"multiple":N=y;default:a.hasOwnProperty(n)||me(e,t,n,null,a,y)}for(u in a)if(n=a[u],y=l[u],a.hasOwnProperty(u)&&(n!=null||y!=null))switch(u){case"value":T=n;break;case"defaultValue":r=n;break;case"multiple":i=n;default:n!==y&&me(e,t,u,n,a,y)}t=r,l=i,a=N,T!=null?$l(e,!!l,T,!1):!!a!=!!l&&(t!=null?$l(e,!!l,t,!0):$l(e,!!l,l?[]:"",!1));return;case"textarea":N=T=null;for(r in l)if(u=l[r],l.hasOwnProperty(r)&&u!=null&&!a.hasOwnProperty(r))switch(r){case"value":break;case"children":break;default:me(e,t,r,null,a,u)}for(i in a)if(u=a[i],n=l[i],a.hasOwnProperty(i)&&(u!=null||n!=null))switch(i){case"value":T=u;break;case"defaultValue":N=u;break;case"children":break;case"dangerouslySetInnerHTML":if(u!=null)throw Error(f(91));break;default:u!==n&&me(e,t,i,u,a,n)}nf(e,T,N);return;case"option":for(var W in l)if(T=l[W],l.hasOwnProperty(W)&&T!=null&&!a.hasOwnProperty(W))switch(W){case"selected":e.selected=!1;break;default:me(e,t,W,null,a,T)}for(y in a)if(T=a[y],N=l[y],a.hasOwnProperty(y)&&T!==N&&(T!=null||N!=null))switch(y){case"selected":e.selected=T&&typeof T!="function"&&typeof T!="symbol";break;default:me(e,t,y,T,a,N)}return;case"img":case"link":case"area":case"base":case"br":case"col":case"embed":case"hr":case"keygen":case"meta":case"param":case"source":case"track":case"wbr":case"menuitem":for(var K in l)T=l[K],l.hasOwnProperty(K)&&T!=null&&!a.hasOwnProperty(K)&&me(e,t,K,null,a,T);for(E in a)if(T=a[E],N=l[E],a.hasOwnProperty(E)&&T!==N&&(T!=null||N!=null))switch(E){case"children":case"dangerouslySetInnerHTML":if(T!=null)throw Error(f(137,t));break;default:me(e,t,E,T,a,N)}return;default:if(Si(t)){for(var ye in l)T=l[ye],l.hasOwnProperty(ye)&&T!==void 0&&!a.hasOwnProperty(ye)&&nr(e,t,ye,void 0,a,T);for(O in a)T=a[O],N=l[O],!a.hasOwnProperty(O)||T===N||T===void 0&&N===void 0||nr(e,t,O,T,a,N);return}}for(var x in l)T=l[x],l.hasOwnProperty(x)&&T!=null&&!a.hasOwnProperty(x)&&me(e,t,x,null,a,T);for(U in a)T=a[U],N=l[U],!a.hasOwnProperty(U)||T===N||T==null&&N==null||me(e,t,U,T,a,N)}var ir=null,cr=null;function Qn(e){return e.nodeType===9?e:e.ownerDocument}function fd(e){switch(e){case"http://www.w3.org/2000/svg":return 1;case"http://www.w3.org/1998/Math/MathML":return 2;default:return 0}}function sd(e,t){if(e===0)switch(t){case"svg":return 1;case"math":return 2;default:return 0}return e===1&&t==="foreignObject"?0:e}function rr(e,t){return e==="textarea"||e==="noscript"||typeof t.children=="string"||typeof t.children=="number"||typeof t.children=="bigint"||typeof t.dangerouslySetInnerHTML=="object"&&t.dangerouslySetInnerHTML!==null&&t.dangerouslySetInnerHTML.__html!=null}var fr=null;function im(){var e=window.event;return e&&e.type==="popstate"?e===fr?!1:(fr=e,!0):(fr=null,!1)}var od=typeof setTimeout=="function"?setTimeout:void 0,cm=typeof clearTimeout=="function"?clearTimeout:void 0,dd=typeof Promise=="function"?Promise:void 0,rm=typeof queueMicrotask=="function"?queueMicrotask:typeof dd<"u"?function(e){return dd.resolve(null).then(e).catch(fm)}:od;function fm(e){setTimeout(function(){throw e})}function gl(e){return e==="head"}function hd(e,t){var l=t,a=0,u=0;do{var n=l.nextSibling;if(e.removeChild(l),n&&n.nodeType===8)if(l=n.data,l==="/$"){if(0<a&&8>a){l=a;var i=e.ownerDocument;if(l&1&&bu(i.documentElement),l&2&&bu(i.body),l&4)for(l=i.head,bu(l),i=l.firstChild;i;){var r=i.nextSibling,y=i.nodeName;i[wa]||y==="SCRIPT"||y==="STYLE"||y==="LINK"&&i.rel.toLowerCase()==="stylesheet"||l.removeChild(i),i=r}}if(u===0){e.removeChild(n),ju(t);return}u--}else l==="$"||l==="$?"||l==="$!"?u++:a=l.charCodeAt(0)-48;else a=0;l=n}while(l);ju(t)}function sr(e){var t=e.firstChild;for(t&&t.nodeType===10&&(t=t.nextSibling);t;){var l=t;switch(t=t.nextSibling,l.nodeName){case"HTML":case"HEAD":case"BODY":sr(l),yi(l);continue;case"SCRIPT":case"STYLE":continue;case"LINK":if(l.rel.toLowerCase()==="stylesheet")continue}e.removeChild(l)}}function sm(e,t,l,a){for(;e.nodeType===1;){var u=l;if(e.nodeName.toLowerCase()!==t.toLowerCase()){if(!a&&(e.nodeName!=="INPUT"||e.type!=="hidden"))break}else if(a){if(!e[wa])switch(t){case"meta":if(!e.hasAttribute("itemprop"))break;return e;case"link":if(n=e.getAttribute("rel"),n==="stylesheet"&&e.hasAttribute("data-precedence"))break;if(n!==u.rel||e.getAttribute("href")!==(u.href==null||u.href===""?null:u.href)||e.getAttribute("crossorigin")!==(u.crossOrigin==null?null:u.crossOrigin)||e.getAttribute("title")!==(u.title==null?null:u.title))break;return e;case"style":if(e.hasAttribute("data-precedence"))break;return e;case"script":if(n=e.getAttribute("src"),(n!==(u.src==null?null:u.src)||e.getAttribute("type")!==(u.type==null?null:u.type)||e.getAttribute("crossorigin")!==(u.crossOrigin==null?null:u.crossOrigin))&&n&&e.hasAttribute("async")&&!e.hasAttribute("itemprop"))break;return e;default:return e}}else if(t==="input"&&e.type==="hidden"){var n=u.name==null?null:""+u.name;if(u.type==="hidden"&&e.getAttribute("name")===n)return e}else return e;if(e=At(e.nextSibling),e===null)break}return null}function om(e,t,l){if(t==="")return null;for(;e.nodeType!==3;)if((e.nodeType!==1||e.nodeName!=="INPUT"||e.type!=="hidden")&&!l||(e=At(e.nextSibling),e===null))return null;return e}function or(e){return e.data==="$!"||e.data==="$?"&&e.ownerDocument.readyState==="complete"}function dm(e,t){var l=e.ownerDocument;if(e.data!=="$?"||l.readyState==="complete")t();else{var a=function(){t(),l.removeEventListener("DOMContentLoaded",a)};l.addEventListener("DOMContentLoaded",a),e._reactRetry=a}}function At(e){for(;e!=null;e=e.nextSibling){var t=e.nodeType;if(t===1||t===3)break;if(t===8){if(t=e.data,t==="$"||t==="$!"||t==="$?"||t==="F!"||t==="F")break;if(t==="/$")return null}}return e}var dr=null;function md(e){e=e.previousSibling;for(var t=0;e;){if(e.nodeType===8){var l=e.data;if(l==="$"||l==="$!"||l==="$?"){if(t===0)return e;t--}else l==="/$"&&t++}e=e.previousSibling}return null}function yd(e,t,l){switch(t=Qn(l),e){case"html":if(e=t.documentElement,!e)throw Error(f(452));return e;case"head":if(e=t.head,!e)throw Error(f(453));return e;case"body":if(e=t.body,!e)throw Error(f(454));return e;default:throw Error(f(451))}}function bu(e){for(var t=e.attributes;t.length;)e.removeAttributeNode(t[0]);yi(e)}var xt=new Map,gd=new Set;function kn(e){return typeof e.getRootNode=="function"?e.getRootNode():e.nodeType===9?e:e.ownerDocument}var Kt=Y.d;Y.d={f:hm,r:mm,D:ym,C:gm,L:vm,m:pm,X:xm,S:bm,M:Sm};function hm(){var e=Kt.f(),t=wn();return e||t}function mm(e){var t=Zl(e);t!==null&&t.tag===5&&t.type==="form"?Cs(t):Kt.r(e)}var Na=typeof document>"u"?null:document;function vd(e,t,l){var a=Na;if(a&&typeof t=="string"&&t){var u=ht(t);u='link[rel="'+e+'"][href="'+u+'"]',typeof l=="string"&&(u+='[crossorigin="'+l+'"]'),gd.has(u)||(gd.add(u),e={rel:e,crossOrigin:l,href:t},a.querySelector(u)===null&&(t=a.createElement("link"),Ge(t,"link",e),Ce(t),a.head.appendChild(t)))}}function ym(e){Kt.D(e),vd("dns-prefetch",e,null)}function gm(e,t){Kt.C(e,t),vd("preconnect",e,t)}function vm(e,t,l){Kt.L(e,t,l);var a=Na;if(a&&e&&t){var u='link[rel="preload"][as="'+ht(t)+'"]';t==="image"&&l&&l.imageSrcSet?(u+='[imagesrcset="'+ht(l.imageSrcSet)+'"]',typeof l.imageSizes=="string"&&(u+='[imagesizes="'+ht(l.imageSizes)+'"]')):u+='[href="'+ht(e)+'"]';var n=u;switch(t){case"style":n=Ra(e);break;case"script":n=ja(e)}xt.has(n)||(e=R({rel:"preload",href:t==="image"&&l&&l.imageSrcSet?void 0:e,as:t},l),xt.set(n,e),a.querySelector(u)!==null||t==="style"&&a.querySelector(xu(n))||t==="script"&&a.querySelector(Su(n))||(t=a.createElement("link"),Ge(t,"link",e),Ce(t),a.head.appendChild(t)))}}function pm(e,t){Kt.m(e,t);var l=Na;if(l&&e){var a=t&&typeof t.as=="string"?t.as:"script",u='link[rel="modulepreload"][as="'+ht(a)+'"][href="'+ht(e)+'"]',n=u;switch(a){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":n=ja(e)}if(!xt.has(n)&&(e=R({rel:"modulepreload",href:e},t),xt.set(n,e),l.querySelector(u)===null)){switch(a){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":if(l.querySelector(Su(n)))return}a=l.createElement("link"),Ge(a,"link",e),Ce(a),l.head.appendChild(a)}}}function bm(e,t,l){Kt.S(e,t,l);var a=Na;if(a&&e){var u=Vl(a).hoistableStyles,n=Ra(e);t=t||"default";var i=u.get(n);if(!i){var r={loading:0,preload:null};if(i=a.querySelector(xu(n)))r.loading=5;else{e=R({rel:"stylesheet",href:e,"data-precedence":t},l),(l=xt.get(n))&&hr(e,l);var y=i=a.createElement("link");Ce(y),Ge(y,"link",e),y._p=new Promise(function(E,O){y.onload=E,y.onerror=O}),y.addEventListener("load",function(){r.loading|=1}),y.addEventListener("error",function(){r.loading|=2}),r.loading|=4,Xn(i,t,a)}i={type:"stylesheet",instance:i,count:1,state:r},u.set(n,i)}}}function xm(e,t){Kt.X(e,t);var l=Na;if(l&&e){var a=Vl(l).hoistableScripts,u=ja(e),n=a.get(u);n||(n=l.querySelector(Su(u)),n||(e=R({src:e,async:!0},t),(t=xt.get(u))&&mr(e,t),n=l.createElement("script"),Ce(n),Ge(n,"link",e),l.head.appendChild(n)),n={type:"script",instance:n,count:1,state:null},a.set(u,n))}}function Sm(e,t){Kt.M(e,t);var l=Na;if(l&&e){var a=Vl(l).hoistableScripts,u=ja(e),n=a.get(u);n||(n=l.querySelector(Su(u)),n||(e=R({src:e,async:!0,type:"module"},t),(t=xt.get(u))&&mr(e,t),n=l.createElement("script"),Ce(n),Ge(n,"link",e),l.head.appendChild(n)),n={type:"script",instance:n,count:1,state:null},a.set(u,n))}}function pd(e,t,l,a){var u=(u=I.current)?kn(u):null;if(!u)throw Error(f(446));switch(e){case"meta":case"title":return null;case"style":return typeof l.precedence=="string"&&typeof l.href=="string"?(t=Ra(l.href),l=Vl(u).hoistableStyles,a=l.get(t),a||(a={type:"style",instance:null,count:0,state:null},l.set(t,a)),a):{type:"void",instance:null,count:0,state:null};case"link":if(l.rel==="stylesheet"&&typeof l.href=="string"&&typeof l.precedence=="string"){e=Ra(l.href);var n=Vl(u).hoistableStyles,i=n.get(e);if(i||(u=u.ownerDocument||u,i={type:"stylesheet",instance:null,count:0,state:{loading:0,preload:null}},n.set(e,i),(n=u.querySelector(xu(e)))&&!n._p&&(i.instance=n,i.state.loading=5),xt.has(e)||(l={rel:"preload",as:"style",href:l.href,crossOrigin:l.crossOrigin,integrity:l.integrity,media:l.media,hrefLang:l.hrefLang,referrerPolicy:l.referrerPolicy},xt.set(e,l),n||Em(u,e,l,i.state))),t&&a===null)throw Error(f(528,""));return i}if(t&&a!==null)throw Error(f(529,""));return null;case"script":return t=l.async,l=l.src,typeof l=="string"&&t&&typeof t!="function"&&typeof t!="symbol"?(t=ja(l),l=Vl(u).hoistableScripts,a=l.get(t),a||(a={type:"script",instance:null,count:0,state:null},l.set(t,a)),a):{type:"void",instance:null,count:0,state:null};default:throw Error(f(444,e))}}function Ra(e){return'href="'+ht(e)+'"'}function xu(e){return'link[rel="stylesheet"]['+e+"]"}function bd(e){return R({},e,{"data-precedence":e.precedence,precedence:null})}function Em(e,t,l,a){e.querySelector('link[rel="preload"][as="style"]['+t+"]")?a.loading=1:(t=e.createElement("link"),a.preload=t,t.addEventListener("load",function(){return a.loading|=1}),t.addEventListener("error",function(){return a.loading|=2}),Ge(t,"link",l),Ce(t),e.head.appendChild(t))}function ja(e){return'[src="'+ht(e)+'"]'}function Su(e){return"script[async]"+e}function xd(e,t,l){if(t.count++,t.instance===null)switch(t.type){case"style":var a=e.querySelector('style[data-href~="'+ht(l.href)+'"]');if(a)return t.instance=a,Ce(a),a;var u=R({},l,{"data-href":l.href,"data-precedence":l.precedence,href:null,precedence:null});return a=(e.ownerDocument||e).createElement("style"),Ce(a),Ge(a,"style",u),Xn(a,l.precedence,e),t.instance=a;case"stylesheet":u=Ra(l.href);var n=e.querySelector(xu(u));if(n)return t.state.loading|=4,t.instance=n,Ce(n),n;a=bd(l),(u=xt.get(u))&&hr(a,u),n=(e.ownerDocument||e).createElement("link"),Ce(n);var i=n;return i._p=new Promise(function(r,y){i.onload=r,i.onerror=y}),Ge(n,"link",a),t.state.loading|=4,Xn(n,l.precedence,e),t.instance=n;case"script":return n=ja(l.src),(u=e.querySelector(Su(n)))?(t.instance=u,Ce(u),u):(a=l,(u=xt.get(n))&&(a=R({},l),mr(a,u)),e=e.ownerDocument||e,u=e.createElement("script"),Ce(u),Ge(u,"link",a),e.head.appendChild(u),t.instance=u);case"void":return null;default:throw Error(f(443,t.type))}else t.type==="stylesheet"&&(t.state.loading&4)===0&&(a=t.instance,t.state.loading|=4,Xn(a,l.precedence,e));return t.instance}function Xn(e,t,l){for(var a=l.querySelectorAll('link[rel="stylesheet"][data-precedence],style[data-precedence]'),u=a.length?a[a.length-1]:null,n=u,i=0;i<a.length;i++){var r=a[i];if(r.dataset.precedence===t)n=r;else if(n!==u)break}n?n.parentNode.insertBefore(e,n.nextSibling):(t=l.nodeType===9?l.head:l,t.insertBefore(e,t.firstChild))}function hr(e,t){e.crossOrigin==null&&(e.crossOrigin=t.crossOrigin),e.referrerPolicy==null&&(e.referrerPolicy=t.referrerPolicy),e.title==null&&(e.title=t.title)}function mr(e,t){e.crossOrigin==null&&(e.crossOrigin=t.crossOrigin),e.referrerPolicy==null&&(e.referrerPolicy=t.referrerPolicy),e.integrity==null&&(e.integrity=t.integrity)}var Zn=null;function Sd(e,t,l){if(Zn===null){var a=new Map,u=Zn=new Map;u.set(l,a)}else u=Zn,a=u.get(l),a||(a=new Map,u.set(l,a));if(a.has(e))return a;for(a.set(e,null),l=l.getElementsByTagName(e),u=0;u<l.length;u++){var n=l[u];if(!(n[wa]||n[Xe]||e==="link"&&n.getAttribute("rel")==="stylesheet")&&n.namespaceURI!=="http://www.w3.org/2000/svg"){var i=n.getAttribute(t)||"";i=e+i;var r=a.get(i);r?r.push(n):a.set(i,[n])}}return a}function Ed(e,t,l){e=e.ownerDocument||e,e.head.insertBefore(l,t==="title"?e.querySelector("head > title"):null)}function Am(e,t,l){if(l===1||t.itemProp!=null)return!1;switch(e){case"meta":case"title":return!0;case"style":if(typeof t.precedence!="string"||typeof t.href!="string"||t.href==="")break;return!0;case"link":if(typeof t.rel!="string"||typeof t.href!="string"||t.href===""||t.onLoad||t.onError)break;switch(t.rel){case"stylesheet":return e=t.disabled,typeof t.precedence=="string"&&e==null;default:return!0}case"script":if(t.async&&typeof t.async!="function"&&typeof t.async!="symbol"&&!t.onLoad&&!t.onError&&t.src&&typeof t.src=="string")return!0}return!1}function Ad(e){return!(e.type==="stylesheet"&&(e.state.loading&3)===0)}var Eu=null;function Tm(){}function Nm(e,t,l){if(Eu===null)throw Error(f(475));var a=Eu;if(t.type==="stylesheet"&&(typeof l.media!="string"||matchMedia(l.media).matches!==!1)&&(t.state.loading&4)===0){if(t.instance===null){var u=Ra(l.href),n=e.querySelector(xu(u));if(n){e=n._p,e!==null&&typeof e=="object"&&typeof e.then=="function"&&(a.count++,a=Vn.bind(a),e.then(a,a)),t.state.loading|=4,t.instance=n,Ce(n);return}n=e.ownerDocument||e,l=bd(l),(u=xt.get(u))&&hr(l,u),n=n.createElement("link"),Ce(n);var i=n;i._p=new Promise(function(r,y){i.onload=r,i.onerror=y}),Ge(n,"link",l),t.instance=n}a.stylesheets===null&&(a.stylesheets=new Map),a.stylesheets.set(t,e),(e=t.state.preload)&&(t.state.loading&3)===0&&(a.count++,t=Vn.bind(a),e.addEventListener("load",t),e.addEventListener("error",t))}}function Rm(){if(Eu===null)throw Error(f(475));var e=Eu;return e.stylesheets&&e.count===0&&yr(e,e.stylesheets),0<e.count?function(t){var l=setTimeout(function(){if(e.stylesheets&&yr(e,e.stylesheets),e.unsuspend){var a=e.unsuspend;e.unsuspend=null,a()}},6e4);return e.unsuspend=t,function(){e.unsuspend=null,clearTimeout(l)}}:null}function Vn(){if(this.count--,this.count===0){if(this.stylesheets)yr(this,this.stylesheets);else if(this.unsuspend){var e=this.unsuspend;this.unsuspend=null,e()}}}var Kn=null;function yr(e,t){e.stylesheets=null,e.unsuspend!==null&&(e.count++,Kn=new Map,t.forEach(jm,e),Kn=null,Vn.call(e))}function jm(e,t){if(!(t.state.loading&4)){var l=Kn.get(e);if(l)var a=l.get(null);else{l=new Map,Kn.set(e,l);for(var u=e.querySelectorAll("link[data-precedence],style[data-precedence]"),n=0;n<u.length;n++){var i=u[n];(i.nodeName==="LINK"||i.getAttribute("media")!=="not all")&&(l.set(i.dataset.precedence,i),a=i)}a&&l.set(null,a)}u=t.instance,i=u.getAttribute("data-precedence"),n=l.get(i)||a,n===a&&l.set(null,u),l.set(i,u),this.count++,a=Vn.bind(this),u.addEventListener("load",a),u.addEventListener("error",a),n?n.parentNode.insertBefore(u,n.nextSibling):(e=e.nodeType===9?e.head:e,e.insertBefore(u,e.firstChild)),t.state.loading|=4}}var Au={$$typeof:P,Provider:null,Consumer:null,_currentValue:$,_currentValue2:$,_threadCount:0};function zm(e,t,l,a,u,n,i,r){this.tag=1,this.containerInfo=e,this.pingCache=this.current=this.pendingChildren=null,this.timeoutHandle=-1,this.callbackNode=this.next=this.pendingContext=this.context=this.cancelPendingCommit=null,this.callbackPriority=0,this.expirationTimes=oi(-1),this.entangledLanes=this.shellSuspendCounter=this.errorRecoveryDisabledLanes=this.expiredLanes=this.warmLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0,this.entanglements=oi(0),this.hiddenUpdates=oi(null),this.identifierPrefix=a,this.onUncaughtError=u,this.onCaughtError=n,this.onRecoverableError=i,this.pooledCache=null,this.pooledCacheLanes=0,this.formState=r,this.incompleteTransitions=new Map}function Td(e,t,l,a,u,n,i,r,y,E,O,U){return e=new zm(e,t,l,i,r,y,E,U),t=1,n===!0&&(t|=24),n=nt(3,null,null,t),e.current=n,n.stateNode=e,t=Wi(),t.refCount++,e.pooledCache=t,t.refCount++,n.memoizedState={element:a,isDehydrated:l,cache:t},ec(n),e}function Nd(e){return e?(e=ua,e):ua}function Rd(e,t,l,a,u,n){u=Nd(u),a.context===null?a.context=u:a.pendingContext=u,a=al(t),a.payload={element:l},n=n===void 0?null:n,n!==null&&(a.callback=n),l=ul(e,a,t),l!==null&&(st(l,e,t),Ia(l,e,t))}function jd(e,t){if(e=e.memoizedState,e!==null&&e.dehydrated!==null){var l=e.retryLane;e.retryLane=l!==0&&l<t?l:t}}function gr(e,t){jd(e,t),(e=e.alternate)&&jd(e,t)}function zd(e){if(e.tag===13){var t=aa(e,67108864);t!==null&&st(t,e,67108864),gr(e,67108864)}}var Jn=!0;function Om(e,t,l,a){var u=M.T;M.T=null;var n=Y.p;try{Y.p=2,vr(e,t,l,a)}finally{Y.p=n,M.T=u}}function Mm(e,t,l,a){var u=M.T;M.T=null;var n=Y.p;try{Y.p=8,vr(e,t,l,a)}finally{Y.p=n,M.T=u}}function vr(e,t,l,a){if(Jn){var u=pr(a);if(u===null)ur(e,t,a,$n,l),Md(e,a);else if(Dm(u,e,t,l,a))a.stopPropagation();else if(Md(e,a),t&4&&-1<_m.indexOf(e)){for(;u!==null;){var n=Zl(u);if(n!==null)switch(n.tag){case 3:if(n=n.stateNode,n.current.memoizedState.isDehydrated){var i=Nl(n.pendingLanes);if(i!==0){var r=n;for(r.pendingLanes|=2,r.entangledLanes|=2;i;){var y=1<<31-at(i);r.entanglements[1]|=y,i&=~y}_t(n),(se&6)===0&&(Dn=Rt()+500,gu(0))}}break;case 13:r=aa(n,2),r!==null&&st(r,n,2),wn(),gr(n,2)}if(n=pr(a),n===null&&ur(e,t,a,$n,l),n===u)break;u=n}u!==null&&a.stopPropagation()}else ur(e,t,a,null,l)}}function pr(e){return e=Ai(e),br(e)}var $n=null;function br(e){if($n=null,e=Xl(e),e!==null){var t=g(e);if(t===null)e=null;else{var l=t.tag;if(l===13){if(e=A(t),e!==null)return e;e=null}else if(l===3){if(t.stateNode.current.memoizedState.isDehydrated)return t.tag===3?t.stateNode.containerInfo:null;e=null}else t!==e&&(e=null)}}return $n=e,null}function Od(e){switch(e){case"beforetoggle":case"cancel":case"click":case"close":case"contextmenu":case"copy":case"cut":case"auxclick":case"dblclick":case"dragend":case"dragstart":case"drop":case"focusin":case"focusout":case"input":case"invalid":case"keydown":case"keypress":case"keyup":case"mousedown":case"mouseup":case"paste":case"pause":case"play":case"pointercancel":case"pointerdown":case"pointerup":case"ratechange":case"reset":case"resize":case"seeked":case"submit":case"toggle":case"touchcancel":case"touchend":case"touchstart":case"volumechange":case"change":case"selectionchange":case"textInput":case"compositionstart":case"compositionend":case"compositionupdate":case"beforeblur":case"afterblur":case"beforeinput":case"blur":case"fullscreenchange":case"focus":case"hashchange":case"popstate":case"select":case"selectstart":return 2;case"drag":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"mousemove":case"mouseout":case"mouseover":case"pointermove":case"pointerout":case"pointerover":case"scroll":case"touchmove":case"wheel":case"mouseenter":case"mouseleave":case"pointerenter":case"pointerleave":return 8;case"message":switch(g0()){case Gr:return 2;case Qr:return 8;case Lu:case v0:return 32;case kr:return 268435456;default:return 32}default:return 32}}var xr=!1,vl=null,pl=null,bl=null,Tu=new Map,Nu=new Map,xl=[],_m="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset".split(" ");function Md(e,t){switch(e){case"focusin":case"focusout":vl=null;break;case"dragenter":case"dragleave":pl=null;break;case"mouseover":case"mouseout":bl=null;break;case"pointerover":case"pointerout":Tu.delete(t.pointerId);break;case"gotpointercapture":case"lostpointercapture":Nu.delete(t.pointerId)}}function Ru(e,t,l,a,u,n){return e===null||e.nativeEvent!==n?(e={blockedOn:t,domEventName:l,eventSystemFlags:a,nativeEvent:n,targetContainers:[u]},t!==null&&(t=Zl(t),t!==null&&zd(t)),e):(e.eventSystemFlags|=a,t=e.targetContainers,u!==null&&t.indexOf(u)===-1&&t.push(u),e)}function Dm(e,t,l,a,u){switch(t){case"focusin":return vl=Ru(vl,e,t,l,a,u),!0;case"dragenter":return pl=Ru(pl,e,t,l,a,u),!0;case"mouseover":return bl=Ru(bl,e,t,l,a,u),!0;case"pointerover":var n=u.pointerId;return Tu.set(n,Ru(Tu.get(n)||null,e,t,l,a,u)),!0;case"gotpointercapture":return n=u.pointerId,Nu.set(n,Ru(Nu.get(n)||null,e,t,l,a,u)),!0}return!1}function _d(e){var t=Xl(e.target);if(t!==null){var l=g(t);if(l!==null){if(t=l.tag,t===13){if(t=A(l),t!==null){e.blockedOn=t,N0(e.priority,function(){if(l.tag===13){var a=ft();a=di(a);var u=aa(l,a);u!==null&&st(u,l,a),gr(l,a)}});return}}else if(t===3&&l.stateNode.current.memoizedState.isDehydrated){e.blockedOn=l.tag===3?l.stateNode.containerInfo:null;return}}}e.blockedOn=null}function Wn(e){if(e.blockedOn!==null)return!1;for(var t=e.targetContainers;0<t.length;){var l=pr(e.nativeEvent);if(l===null){l=e.nativeEvent;var a=new l.constructor(l.type,l);Ei=a,l.target.dispatchEvent(a),Ei=null}else return t=Zl(l),t!==null&&zd(t),e.blockedOn=l,!1;t.shift()}return!0}function Dd(e,t,l){Wn(e)&&l.delete(t)}function Um(){xr=!1,vl!==null&&Wn(vl)&&(vl=null),pl!==null&&Wn(pl)&&(pl=null),bl!==null&&Wn(bl)&&(bl=null),Tu.forEach(Dd),Nu.forEach(Dd)}function Fn(e,t){e.blockedOn===t&&(e.blockedOn=null,xr||(xr=!0,c.unstable_scheduleCallback(c.unstable_NormalPriority,Um)))}var Pn=null;function Ud(e){Pn!==e&&(Pn=e,c.unstable_scheduleCallback(c.unstable_NormalPriority,function(){Pn===e&&(Pn=null);for(var t=0;t<e.length;t+=3){var l=e[t],a=e[t+1],u=e[t+2];if(typeof a!="function"){if(br(a||l)===null)continue;break}var n=Zl(l);n!==null&&(e.splice(t,3),t-=3,bc(n,{pending:!0,data:u,method:l.method,action:a},a,u))}}))}function ju(e){function t(y){return Fn(y,e)}vl!==null&&Fn(vl,e),pl!==null&&Fn(pl,e),bl!==null&&Fn(bl,e),Tu.forEach(t),Nu.forEach(t);for(var l=0;l<xl.length;l++){var a=xl[l];a.blockedOn===e&&(a.blockedOn=null)}for(;0<xl.length&&(l=xl[0],l.blockedOn===null);)_d(l),l.blockedOn===null&&xl.shift();if(l=(e.ownerDocument||e).$$reactFormReplay,l!=null)for(a=0;a<l.length;a+=3){var u=l[a],n=l[a+1],i=u[$e]||null;if(typeof n=="function")i||Ud(l);else if(i){var r=null;if(n&&n.hasAttribute("formAction")){if(u=n,i=n[$e]||null)r=i.formAction;else if(br(u)!==null)continue}else r=i.action;typeof r=="function"?l[a+1]=r:(l.splice(a,3),a-=3),Ud(l)}}}function Sr(e){this._internalRoot=e}In.prototype.render=Sr.prototype.render=function(e){var t=this._internalRoot;if(t===null)throw Error(f(409));var l=t.current,a=ft();Rd(l,a,e,t,null,null)},In.prototype.unmount=Sr.prototype.unmount=function(){var e=this._internalRoot;if(e!==null){this._internalRoot=null;var t=e.containerInfo;Rd(e.current,2,null,e,null,null),wn(),t[kl]=null}};function In(e){this._internalRoot=e}In.prototype.unstable_scheduleHydration=function(e){if(e){var t=Jr();e={blockedOn:null,target:e,priority:t};for(var l=0;l<xl.length&&t!==0&&t<xl[l].priority;l++);xl.splice(l,0,e),l===0&&_d(e)}};var wd=s.version;if(wd!=="19.1.0")throw Error(f(527,wd,"19.1.0"));Y.findDOMNode=function(e){var t=e._reactInternals;if(t===void 0)throw typeof e.render=="function"?Error(f(188)):(e=Object.keys(e).join(","),Error(f(268,e)));return e=b(t),e=e!==null?m(e):null,e=e===null?null:e.stateNode,e};var wm={bundleType:0,version:"19.1.0",rendererPackageName:"react-dom",currentDispatcherRef:M,reconcilerVersion:"19.1.0"};if(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__<"u"){var ei=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!ei.isDisabled&&ei.supportsFiber)try{_a=ei.inject(wm),lt=ei}catch{}}return Ou.createRoot=function(e,t){if(!h(e))throw Error(f(299));var l=!1,a="",u=$s,n=Ws,i=Fs,r=null;return t!=null&&(t.unstable_strictMode===!0&&(l=!0),t.identifierPrefix!==void 0&&(a=t.identifierPrefix),t.onUncaughtError!==void 0&&(u=t.onUncaughtError),t.onCaughtError!==void 0&&(n=t.onCaughtError),t.onRecoverableError!==void 0&&(i=t.onRecoverableError),t.unstable_transitionCallbacks!==void 0&&(r=t.unstable_transitionCallbacks)),t=Td(e,1,!1,null,null,l,a,u,n,i,r,null),e[kl]=t.current,ar(e),new Sr(t)},Ou.hydrateRoot=function(e,t,l){if(!h(e))throw Error(f(299));var a=!1,u="",n=$s,i=Ws,r=Fs,y=null,E=null;return l!=null&&(l.unstable_strictMode===!0&&(a=!0),l.identifierPrefix!==void 0&&(u=l.identifierPrefix),l.onUncaughtError!==void 0&&(n=l.onUncaughtError),l.onCaughtError!==void 0&&(i=l.onCaughtError),l.onRecoverableError!==void 0&&(r=l.onRecoverableError),l.unstable_transitionCallbacks!==void 0&&(y=l.unstable_transitionCallbacks),l.formState!==void 0&&(E=l.formState)),t=Td(e,1,!0,t,l??null,a,u,n,i,r,y,E),t.context=Nd(null),l=t.current,a=ft(),a=di(a),u=al(a),u.callback=null,ul(l,u,a),l=a,t.current.lanes=l,Ua(t,l),_t(t),e[kl]=t.current,ar(e),new In(t)},Ou.version="19.1.0",Ou}var Xd;function Zm(){if(Xd)return Tr.exports;Xd=1;function c(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(c)}catch(s){console.error(s)}}return c(),Tr.exports=Xm(),Tr.exports}var Vm=Zm();const Km=Fd(Vm);var Mu={},Zd;function Jm(){if(Zd)return Mu;Zd=1,Object.defineProperty(Mu,"__esModule",{value:!0}),Mu.parse=A,Mu.serialize=m;const c=/^[\u0021-\u003A\u003C\u003E-\u007E]+$/,s=/^[\u0021-\u003A\u003C-\u007E]*$/,o=/^([.]?[a-z0-9]([a-z0-9-]{0,61}[a-z0-9])?)([.][a-z0-9]([a-z0-9-]{0,61}[a-z0-9])?)*$/i,f=/^[\u0020-\u003A\u003D-\u007E]*$/,h=Object.prototype.toString,g=(()=>{const D=function(){};return D.prototype=Object.create(null),D})();function A(D,L){const C=new g,Q=D.length;if(Q<2)return C;const X=L?.decode||R;let B=0;do{const ee=D.indexOf("=",B);if(ee===-1)break;const P=D.indexOf(";",B),de=P===-1?Q:P;if(ee>de){B=D.lastIndexOf(";",ee-1)+1;continue}const F=z(D,B,ee),Me=b(D,ee,F),Te=D.slice(F,Me);if(C[Te]===void 0){let je=z(D,ee+1,de),be=b(D,de,je);const Ke=X(D.slice(je,be));C[Te]=Ke}B=de+1}while(B<Q);return C}function z(D,L,C){do{const Q=D.charCodeAt(L);if(Q!==32&&Q!==9)return L}while(++L<C);return C}function b(D,L,C){for(;L>C;){const Q=D.charCodeAt(--L);if(Q!==32&&Q!==9)return L+1}return C}function m(D,L,C){const Q=C?.encode||encodeURIComponent;if(!c.test(D))throw new TypeError(`argument name is invalid: ${D}`);const X=Q(L);if(!s.test(X))throw new TypeError(`argument val is invalid: ${L}`);let B=D+"="+X;if(!C)return B;if(C.maxAge!==void 0){if(!Number.isInteger(C.maxAge))throw new TypeError(`option maxAge is invalid: ${C.maxAge}`);B+="; Max-Age="+C.maxAge}if(C.domain){if(!o.test(C.domain))throw new TypeError(`option domain is invalid: ${C.domain}`);B+="; Domain="+C.domain}if(C.path){if(!f.test(C.path))throw new TypeError(`option path is invalid: ${C.path}`);B+="; Path="+C.path}if(C.expires){if(!H(C.expires)||!Number.isFinite(C.expires.valueOf()))throw new TypeError(`option expires is invalid: ${C.expires}`);B+="; Expires="+C.expires.toUTCString()}if(C.httpOnly&&(B+="; HttpOnly"),C.secure&&(B+="; Secure"),C.partitioned&&(B+="; Partitioned"),C.priority)switch(typeof C.priority=="string"?C.priority.toLowerCase():void 0){case"low":B+="; Priority=Low";break;case"medium":B+="; Priority=Medium";break;case"high":B+="; Priority=High";break;default:throw new TypeError(`option priority is invalid: ${C.priority}`)}if(C.sameSite)switch(typeof C.sameSite=="string"?C.sameSite.toLowerCase():C.sameSite){case!0:case"strict":B+="; SameSite=Strict";break;case"lax":B+="; SameSite=Lax";break;case"none":B+="; SameSite=None";break;default:throw new TypeError(`option sameSite is invalid: ${C.sameSite}`)}return B}function R(D){if(D.indexOf("%")===-1)return D;try{return decodeURIComponent(D)}catch{return D}}function H(D){return h.call(D)==="[object Date]"}return Mu}Jm();var Vd="popstate";function $m(c={}){function s(f,h){let{pathname:g,search:A,hash:z}=f.location;return Mr("",{pathname:g,search:A,hash:z},h.state&&h.state.usr||null,h.state&&h.state.key||"default")}function o(f,h){return typeof h=="string"?h:wu(h)}return Fm(s,o,null,c)}function Ee(c,s){if(c===!1||c===null||typeof c>"u")throw new Error(s)}function Tt(c,s){if(!c){typeof console<"u"&&console.warn(s);try{throw new Error(s)}catch{}}}function Wm(){return Math.random().toString(36).substring(2,10)}function Kd(c,s){return{usr:c.state,key:c.key,idx:s}}function Mr(c,s,o=null,f){return{pathname:typeof c=="string"?c:c.pathname,search:"",hash:"",...typeof s=="string"?za(s):s,state:o,key:s&&s.key||f||Wm()}}function wu({pathname:c="/",search:s="",hash:o=""}){return s&&s!=="?"&&(c+=s.charAt(0)==="?"?s:"?"+s),o&&o!=="#"&&(c+=o.charAt(0)==="#"?o:"#"+o),c}function za(c){let s={};if(c){let o=c.indexOf("#");o>=0&&(s.hash=c.substring(o),c=c.substring(0,o));let f=c.indexOf("?");f>=0&&(s.search=c.substring(f),c=c.substring(0,f)),c&&(s.pathname=c)}return s}function Fm(c,s,o,f={}){let{window:h=document.defaultView,v5Compat:g=!1}=f,A=h.history,z="POP",b=null,m=R();m==null&&(m=0,A.replaceState({...A.state,idx:m},""));function R(){return(A.state||{idx:null}).idx}function H(){z="POP";let X=R(),B=X==null?null:X-m;m=X,b&&b({action:z,location:Q.location,delta:B})}function D(X,B){z="PUSH";let ee=Mr(Q.location,X,B);m=R()+1;let P=Kd(ee,m),de=Q.createHref(ee);try{A.pushState(P,"",de)}catch(F){if(F instanceof DOMException&&F.name==="DataCloneError")throw F;h.location.assign(de)}g&&b&&b({action:z,location:Q.location,delta:1})}function L(X,B){z="REPLACE";let ee=Mr(Q.location,X,B);m=R();let P=Kd(ee,m),de=Q.createHref(ee);A.replaceState(P,"",de),g&&b&&b({action:z,location:Q.location,delta:0})}function C(X){return Pm(X)}let Q={get action(){return z},get location(){return c(h,A)},listen(X){if(b)throw new Error("A history only accepts one active listener");return h.addEventListener(Vd,H),b=X,()=>{h.removeEventListener(Vd,H),b=null}},createHref(X){return s(h,X)},createURL:C,encodeLocation(X){let B=C(X);return{pathname:B.pathname,search:B.search,hash:B.hash}},push:D,replace:L,go(X){return A.go(X)}};return Q}function Pm(c,s=!1){let o="http://localhost";typeof window<"u"&&(o=window.location.origin!=="null"?window.location.origin:window.location.href),Ee(o,"No window.location.(origin|href) available to create URL");let f=typeof c=="string"?c:wu(c);return f=f.replace(/ $/,"%20"),!s&&f.startsWith("//")&&(f=o+f),new URL(f,o)}function Pd(c,s,o="/"){return Im(c,s,o,!1)}function Im(c,s,o,f){let h=typeof s=="string"?za(s):s,g=$t(h.pathname||"/",o);if(g==null)return null;let A=Id(c);e1(A);let z=null;for(let b=0;z==null&&b<A.length;++b){let m=o1(g);z=f1(A[b],m,f)}return z}function Id(c,s=[],o=[],f=""){let h=(g,A,z)=>{let b={relativePath:z===void 0?g.path||"":z,caseSensitive:g.caseSensitive===!0,childrenIndex:A,route:g};b.relativePath.startsWith("/")&&(Ee(b.relativePath.startsWith(f),`Absolute route path "${b.relativePath}" nested under path "${f}" is not valid. An absolute child route path must start with the combined path of all its parent routes.`),b.relativePath=b.relativePath.slice(f.length));let m=Jt([f,b.relativePath]),R=o.concat(b);g.children&&g.children.length>0&&(Ee(g.index!==!0,`Index routes must not have child routes. Please remove all child routes from route path "${m}".`),Id(g.children,s,R,m)),!(g.path==null&&!g.index)&&s.push({path:m,score:c1(m,g.index),routesMeta:R})};return c.forEach((g,A)=>{if(g.path===""||!g.path?.includes("?"))h(g,A);else for(let z of e0(g.path))h(g,A,z)}),s}function e0(c){let s=c.split("/");if(s.length===0)return[];let[o,...f]=s,h=o.endsWith("?"),g=o.replace(/\?$/,"");if(f.length===0)return h?[g,""]:[g];let A=e0(f.join("/")),z=[];return z.push(...A.map(b=>b===""?g:[g,b].join("/"))),h&&z.push(...A),z.map(b=>c.startsWith("/")&&b===""?"/":b)}function e1(c){c.sort((s,o)=>s.score!==o.score?o.score-s.score:r1(s.routesMeta.map(f=>f.childrenIndex),o.routesMeta.map(f=>f.childrenIndex)))}var t1=/^:[\w-]+$/,l1=3,a1=2,u1=1,n1=10,i1=-2,Jd=c=>c==="*";function c1(c,s){let o=c.split("/"),f=o.length;return o.some(Jd)&&(f+=i1),s&&(f+=a1),o.filter(h=>!Jd(h)).reduce((h,g)=>h+(t1.test(g)?l1:g===""?u1:n1),f)}function r1(c,s){return c.length===s.length&&c.slice(0,-1).every((f,h)=>f===s[h])?c[c.length-1]-s[s.length-1]:0}function f1(c,s,o=!1){let{routesMeta:f}=c,h={},g="/",A=[];for(let z=0;z<f.length;++z){let b=f[z],m=z===f.length-1,R=g==="/"?s:s.slice(g.length)||"/",H=ui({path:b.relativePath,caseSensitive:b.caseSensitive,end:m},R),D=b.route;if(!H&&m&&o&&!f[f.length-1].route.index&&(H=ui({path:b.relativePath,caseSensitive:b.caseSensitive,end:!1},R)),!H)return null;Object.assign(h,H.params),A.push({params:h,pathname:Jt([g,H.pathname]),pathnameBase:y1(Jt([g,H.pathnameBase])),route:D}),H.pathnameBase!=="/"&&(g=Jt([g,H.pathnameBase]))}return A}function ui(c,s){typeof c=="string"&&(c={path:c,caseSensitive:!1,end:!0});let[o,f]=s1(c.path,c.caseSensitive,c.end),h=s.match(o);if(!h)return null;let g=h[0],A=g.replace(/(.)\/+$/,"$1"),z=h.slice(1);return{params:f.reduce((m,{paramName:R,isOptional:H},D)=>{if(R==="*"){let C=z[D]||"";A=g.slice(0,g.length-C.length).replace(/(.)\/+$/,"$1")}const L=z[D];return H&&!L?m[R]=void 0:m[R]=(L||"").replace(/%2F/g,"/"),m},{}),pathname:g,pathnameBase:A,pattern:c}}function s1(c,s=!1,o=!0){Tt(c==="*"||!c.endsWith("*")||c.endsWith("/*"),`Route path "${c}" will be treated as if it were "${c.replace(/\*$/,"/*")}" because the \`*\` character must always follow a \`/\` in the pattern. To get rid of this warning, please change the route path to "${c.replace(/\*$/,"/*")}".`);let f=[],h="^"+c.replace(/\/*\*?$/,"").replace(/^\/*/,"/").replace(/[\\.*+^${}|()[\]]/g,"\\$&").replace(/\/:([\w-]+)(\?)?/g,(A,z,b)=>(f.push({paramName:z,isOptional:b!=null}),b?"/?([^\\/]+)?":"/([^\\/]+)"));return c.endsWith("*")?(f.push({paramName:"*"}),h+=c==="*"||c==="/*"?"(.*)$":"(?:\\/(.+)|\\/*)$"):o?h+="\\/*$":c!==""&&c!=="/"&&(h+="(?:(?=\\/|$))"),[new RegExp(h,s?void 0:"i"),f]}function o1(c){try{return c.split("/").map(s=>decodeURIComponent(s).replace(/\//g,"%2F")).join("/")}catch(s){return Tt(!1,`The URL path "${c}" could not be decoded because it is a malformed URL segment. This is probably due to a bad percent encoding (${s}).`),c}}function $t(c,s){if(s==="/")return c;if(!c.toLowerCase().startsWith(s.toLowerCase()))return null;let o=s.endsWith("/")?s.length-1:s.length,f=c.charAt(o);return f&&f!=="/"?null:c.slice(o)||"/"}function d1(c,s="/"){let{pathname:o,search:f="",hash:h=""}=typeof c=="string"?za(c):c;return{pathname:o?o.startsWith("/")?o:h1(o,s):s,search:g1(f),hash:v1(h)}}function h1(c,s){let o=s.replace(/\/+$/,"").split("/");return c.split("/").forEach(h=>{h===".."?o.length>1&&o.pop():h!=="."&&o.push(h)}),o.length>1?o.join("/"):"/"}function zr(c,s,o,f){return`Cannot include a '${c}' character in a manually specified \`to.${s}\` field [${JSON.stringify(f)}].  Please separate it out to the \`to.${o}\` field. Alternatively you may provide the full path as a string in <Link to="..."> and the router will parse it for you.`}function m1(c){return c.filter((s,o)=>o===0||s.route.path&&s.route.path.length>0)}function Ur(c){let s=m1(c);return s.map((o,f)=>f===s.length-1?o.pathname:o.pathnameBase)}function wr(c,s,o,f=!1){let h;typeof c=="string"?h=za(c):(h={...c},Ee(!h.pathname||!h.pathname.includes("?"),zr("?","pathname","search",h)),Ee(!h.pathname||!h.pathname.includes("#"),zr("#","pathname","hash",h)),Ee(!h.search||!h.search.includes("#"),zr("#","search","hash",h)));let g=c===""||h.pathname==="",A=g?"/":h.pathname,z;if(A==null)z=o;else{let H=s.length-1;if(!f&&A.startsWith("..")){let D=A.split("/");for(;D[0]==="..";)D.shift(),H-=1;h.pathname=D.join("/")}z=H>=0?s[H]:"/"}let b=d1(h,z),m=A&&A!=="/"&&A.endsWith("/"),R=(g||A===".")&&o.endsWith("/");return!b.pathname.endsWith("/")&&(m||R)&&(b.pathname+="/"),b}var Jt=c=>c.join("/").replace(/\/\/+/g,"/"),y1=c=>c.replace(/\/+$/,"").replace(/^\/*/,"/"),g1=c=>!c||c==="?"?"":c.startsWith("?")?c:"?"+c,v1=c=>!c||c==="#"?"":c.startsWith("#")?c:"#"+c;function p1(c){return c!=null&&typeof c.status=="number"&&typeof c.statusText=="string"&&typeof c.internal=="boolean"&&"data"in c}var t0=["POST","PUT","PATCH","DELETE"];new Set(t0);var b1=["GET",...t0];new Set(b1);var Oa=j.createContext(null);Oa.displayName="DataRouter";var ni=j.createContext(null);ni.displayName="DataRouterState";var l0=j.createContext({isTransitioning:!1});l0.displayName="ViewTransition";var x1=j.createContext(new Map);x1.displayName="Fetchers";var S1=j.createContext(null);S1.displayName="Await";var Nt=j.createContext(null);Nt.displayName="Navigation";var Hu=j.createContext(null);Hu.displayName="Location";var Dt=j.createContext({outlet:null,matches:[],isDataRoute:!1});Dt.displayName="Route";var Cr=j.createContext(null);Cr.displayName="RouteError";function E1(c,{relative:s}={}){Ee(Ma(),"useHref() may be used only in the context of a <Router> component.");let{basename:o,navigator:f}=j.useContext(Nt),{hash:h,pathname:g,search:A}=Bu(c,{relative:s}),z=g;return o!=="/"&&(z=g==="/"?o:Jt([o,g])),f.createHref({pathname:z,search:A,hash:h})}function Ma(){return j.useContext(Hu)!=null}function Wt(){return Ee(Ma(),"useLocation() may be used only in the context of a <Router> component."),j.useContext(Hu).location}var a0="You should call navigate() in a React.useEffect(), not when your component is first rendered.";function u0(c){j.useContext(Nt).static||j.useLayoutEffect(c)}function Hr(){let{isDataRoute:c}=j.useContext(Dt);return c?C1():A1()}function A1(){Ee(Ma(),"useNavigate() may be used only in the context of a <Router> component.");let c=j.useContext(Oa),{basename:s,navigator:o}=j.useContext(Nt),{matches:f}=j.useContext(Dt),{pathname:h}=Wt(),g=JSON.stringify(Ur(f)),A=j.useRef(!1);return u0(()=>{A.current=!0}),j.useCallback((b,m={})=>{if(Tt(A.current,a0),!A.current)return;if(typeof b=="number"){o.go(b);return}let R=wr(b,JSON.parse(g),h,m.relative==="path");c==null&&s!=="/"&&(R.pathname=R.pathname==="/"?s:Jt([s,R.pathname])),(m.replace?o.replace:o.push)(R,m.state,m)},[s,o,g,h,c])}j.createContext(null);function Bu(c,{relative:s}={}){let{matches:o}=j.useContext(Dt),{pathname:f}=Wt(),h=JSON.stringify(Ur(o));return j.useMemo(()=>wr(c,JSON.parse(h),f,s==="path"),[c,h,f,s])}function T1(c,s){return n0(c,s)}function n0(c,s,o,f){Ee(Ma(),"useRoutes() may be used only in the context of a <Router> component.");let{navigator:h}=j.useContext(Nt),{matches:g}=j.useContext(Dt),A=g[g.length-1],z=A?A.params:{},b=A?A.pathname:"/",m=A?A.pathnameBase:"/",R=A&&A.route;{let B=R&&R.path||"";i0(b,!R||B.endsWith("*")||B.endsWith("*?"),`You rendered descendant <Routes> (or called \`useRoutes()\`) at "${b}" (under <Route path="${B}">) but the parent route path has no trailing "*". This means if you navigate deeper, the parent won't match anymore and therefore the child routes will never render.

Please change the parent <Route path="${B}"> to <Route path="${B==="/"?"*":`${B}/*`}">.`)}let H=Wt(),D;if(s){let B=typeof s=="string"?za(s):s;Ee(m==="/"||B.pathname?.startsWith(m),`When overriding the location using \`<Routes location>\` or \`useRoutes(routes, location)\`, the location pathname must begin with the portion of the URL pathname that was matched by all parent routes. The current pathname base is "${m}" but pathname "${B.pathname}" was given in the \`location\` prop.`),D=B}else D=H;let L=D.pathname||"/",C=L;if(m!=="/"){let B=m.replace(/^\//,"").split("/");C="/"+L.replace(/^\//,"").split("/").slice(B.length).join("/")}let Q=Pd(c,{pathname:C});Tt(R||Q!=null,`No routes matched location "${D.pathname}${D.search}${D.hash}" `),Tt(Q==null||Q[Q.length-1].route.element!==void 0||Q[Q.length-1].route.Component!==void 0||Q[Q.length-1].route.lazy!==void 0,`Matched leaf route at location "${D.pathname}${D.search}${D.hash}" does not have an element or Component. This means it will render an <Outlet /> with a null value by default resulting in an "empty" page.`);let X=O1(Q&&Q.map(B=>Object.assign({},B,{params:Object.assign({},z,B.params),pathname:Jt([m,h.encodeLocation?h.encodeLocation(B.pathname).pathname:B.pathname]),pathnameBase:B.pathnameBase==="/"?m:Jt([m,h.encodeLocation?h.encodeLocation(B.pathnameBase).pathname:B.pathnameBase])})),g,o,f);return s&&X?j.createElement(Hu.Provider,{value:{location:{pathname:"/",search:"",hash:"",state:null,key:"default",...D},navigationType:"POP"}},X):X}function N1(){let c=w1(),s=p1(c)?`${c.status} ${c.statusText}`:c instanceof Error?c.message:JSON.stringify(c),o=c instanceof Error?c.stack:null,f="rgba(200,200,200, 0.5)",h={padding:"0.5rem",backgroundColor:f},g={padding:"2px 4px",backgroundColor:f},A=null;return console.error("Error handled by React Router default ErrorBoundary:",c),A=j.createElement(j.Fragment,null,j.createElement("p",null,"💿 Hey developer 👋"),j.createElement("p",null,"You can provide a way better UX than this when your app throws errors by providing your own ",j.createElement("code",{style:g},"ErrorBoundary")," or"," ",j.createElement("code",{style:g},"errorElement")," prop on your route.")),j.createElement(j.Fragment,null,j.createElement("h2",null,"Unexpected Application Error!"),j.createElement("h3",{style:{fontStyle:"italic"}},s),o?j.createElement("pre",{style:h},o):null,A)}var R1=j.createElement(N1,null),j1=class extends j.Component{constructor(c){super(c),this.state={location:c.location,revalidation:c.revalidation,error:c.error}}static getDerivedStateFromError(c){return{error:c}}static getDerivedStateFromProps(c,s){return s.location!==c.location||s.revalidation!=="idle"&&c.revalidation==="idle"?{error:c.error,location:c.location,revalidation:c.revalidation}:{error:c.error!==void 0?c.error:s.error,location:s.location,revalidation:c.revalidation||s.revalidation}}componentDidCatch(c,s){console.error("React Router caught the following error during render",c,s)}render(){return this.state.error!==void 0?j.createElement(Dt.Provider,{value:this.props.routeContext},j.createElement(Cr.Provider,{value:this.state.error,children:this.props.component})):this.props.children}};function z1({routeContext:c,match:s,children:o}){let f=j.useContext(Oa);return f&&f.static&&f.staticContext&&(s.route.errorElement||s.route.ErrorBoundary)&&(f.staticContext._deepestRenderedBoundaryId=s.route.id),j.createElement(Dt.Provider,{value:c},o)}function O1(c,s=[],o=null,f=null){if(c==null){if(!o)return null;if(o.errors)c=o.matches;else if(s.length===0&&!o.initialized&&o.matches.length>0)c=o.matches;else return null}let h=c,g=o?.errors;if(g!=null){let b=h.findIndex(m=>m.route.id&&g?.[m.route.id]!==void 0);Ee(b>=0,`Could not find a matching route for errors on route IDs: ${Object.keys(g).join(",")}`),h=h.slice(0,Math.min(h.length,b+1))}let A=!1,z=-1;if(o)for(let b=0;b<h.length;b++){let m=h[b];if((m.route.HydrateFallback||m.route.hydrateFallbackElement)&&(z=b),m.route.id){let{loaderData:R,errors:H}=o,D=m.route.loader&&!R.hasOwnProperty(m.route.id)&&(!H||H[m.route.id]===void 0);if(m.route.lazy||D){A=!0,z>=0?h=h.slice(0,z+1):h=[h[0]];break}}}return h.reduceRight((b,m,R)=>{let H,D=!1,L=null,C=null;o&&(H=g&&m.route.id?g[m.route.id]:void 0,L=m.route.errorElement||R1,A&&(z<0&&R===0?(i0("route-fallback",!1,"No `HydrateFallback` element provided to render during initial hydration"),D=!0,C=null):z===R&&(D=!0,C=m.route.hydrateFallbackElement||null)));let Q=s.concat(h.slice(0,R+1)),X=()=>{let B;return H?B=L:D?B=C:m.route.Component?B=j.createElement(m.route.Component,null):m.route.element?B=m.route.element:B=b,j.createElement(z1,{match:m,routeContext:{outlet:b,matches:Q,isDataRoute:o!=null},children:B})};return o&&(m.route.ErrorBoundary||m.route.errorElement||R===0)?j.createElement(j1,{location:o.location,revalidation:o.revalidation,component:L,error:H,children:X(),routeContext:{outlet:null,matches:Q,isDataRoute:!0}}):X()},null)}function Br(c){return`${c} must be used within a data router.  See https://reactrouter.com/en/main/routers/picking-a-router.`}function M1(c){let s=j.useContext(Oa);return Ee(s,Br(c)),s}function _1(c){let s=j.useContext(ni);return Ee(s,Br(c)),s}function D1(c){let s=j.useContext(Dt);return Ee(s,Br(c)),s}function qr(c){let s=D1(c),o=s.matches[s.matches.length-1];return Ee(o.route.id,`${c} can only be used on routes that contain a unique "id"`),o.route.id}function U1(){return qr("useRouteId")}function w1(){let c=j.useContext(Cr),s=_1("useRouteError"),o=qr("useRouteError");return c!==void 0?c:s.errors?.[o]}function C1(){let{router:c}=M1("useNavigate"),s=qr("useNavigate"),o=j.useRef(!1);return u0(()=>{o.current=!0}),j.useCallback(async(h,g={})=>{Tt(o.current,a0),o.current&&(typeof h=="number"?c.navigate(h):await c.navigate(h,{fromRouteId:s,...g}))},[c,s])}var $d={};function i0(c,s,o){!s&&!$d[c]&&($d[c]=!0,Tt(!1,o))}j.memo(H1);function H1({routes:c,future:s,state:o}){return n0(c,void 0,o,s)}function c0({to:c,replace:s,state:o,relative:f}){Ee(Ma(),"<Navigate> may be used only in the context of a <Router> component.");let{static:h}=j.useContext(Nt);Tt(!h,"<Navigate> must not be used on the initial render in a <StaticRouter>. This is a no-op, but you should modify your code so the <Navigate> is only ever rendered in response to some user interaction or state change.");let{matches:g}=j.useContext(Dt),{pathname:A}=Wt(),z=Hr(),b=wr(c,Ur(g),A,f==="path"),m=JSON.stringify(b);return j.useEffect(()=>{z(JSON.parse(m),{replace:s,state:o,relative:f})},[z,m,f,s,o]),null}function El(c){Ee(!1,"A <Route> is only ever to be used as the child of <Routes> element, never rendered directly. Please wrap your <Route> in a <Routes>.")}function B1({basename:c="/",children:s=null,location:o,navigationType:f="POP",navigator:h,static:g=!1}){Ee(!Ma(),"You cannot render a <Router> inside another <Router>. You should never have more than one in your app.");let A=c.replace(/^\/*/,"/"),z=j.useMemo(()=>({basename:A,navigator:h,static:g,future:{}}),[A,h,g]);typeof o=="string"&&(o=za(o));let{pathname:b="/",search:m="",hash:R="",state:H=null,key:D="default"}=o,L=j.useMemo(()=>{let C=$t(b,A);return C==null?null:{location:{pathname:C,search:m,hash:R,state:H,key:D},navigationType:f}},[A,b,m,R,H,D,f]);return Tt(L!=null,`<Router basename="${A}"> is not able to match the URL "${b}${m}${R}" because it does not start with the basename, so the <Router> won't render anything.`),L==null?null:j.createElement(Nt.Provider,{value:z},j.createElement(Hu.Provider,{children:s,value:L}))}function q1({children:c,location:s}){return T1(_r(c),s)}function _r(c,s=[]){let o=[];return j.Children.forEach(c,(f,h)=>{if(!j.isValidElement(f))return;let g=[...s,h];if(f.type===j.Fragment){o.push.apply(o,_r(f.props.children,g));return}Ee(f.type===El,`[${typeof f.type=="string"?f.type:f.type.name}] is not a <Route> component. All component children of <Routes> must be a <Route> or <React.Fragment>`),Ee(!f.props.index||!f.props.children,"An index route cannot have child routes.");let A={id:f.props.id||g.join("-"),caseSensitive:f.props.caseSensitive,element:f.props.element,Component:f.props.Component,index:f.props.index,path:f.props.path,loader:f.props.loader,action:f.props.action,hydrateFallbackElement:f.props.hydrateFallbackElement,HydrateFallback:f.props.HydrateFallback,errorElement:f.props.errorElement,ErrorBoundary:f.props.ErrorBoundary,hasErrorBoundary:f.props.hasErrorBoundary===!0||f.props.ErrorBoundary!=null||f.props.errorElement!=null,shouldRevalidate:f.props.shouldRevalidate,handle:f.props.handle,lazy:f.props.lazy};f.props.children&&(A.children=_r(f.props.children,g)),o.push(A)}),o}var li="get",ai="application/x-www-form-urlencoded";function ii(c){return c!=null&&typeof c.tagName=="string"}function Y1(c){return ii(c)&&c.tagName.toLowerCase()==="button"}function L1(c){return ii(c)&&c.tagName.toLowerCase()==="form"}function G1(c){return ii(c)&&c.tagName.toLowerCase()==="input"}function Q1(c){return!!(c.metaKey||c.altKey||c.ctrlKey||c.shiftKey)}function k1(c,s){return c.button===0&&(!s||s==="_self")&&!Q1(c)}var ti=null;function X1(){if(ti===null)try{new FormData(document.createElement("form"),0),ti=!1}catch{ti=!0}return ti}var Z1=new Set(["application/x-www-form-urlencoded","multipart/form-data","text/plain"]);function Or(c){return c!=null&&!Z1.has(c)?(Tt(!1,`"${c}" is not a valid \`encType\` for \`<Form>\`/\`<fetcher.Form>\` and will default to "${ai}"`),null):c}function V1(c,s){let o,f,h,g,A;if(L1(c)){let z=c.getAttribute("action");f=z?$t(z,s):null,o=c.getAttribute("method")||li,h=Or(c.getAttribute("enctype"))||ai,g=new FormData(c)}else if(Y1(c)||G1(c)&&(c.type==="submit"||c.type==="image")){let z=c.form;if(z==null)throw new Error('Cannot submit a <button> or <input type="submit"> without a <form>');let b=c.getAttribute("formaction")||z.getAttribute("action");if(f=b?$t(b,s):null,o=c.getAttribute("formmethod")||z.getAttribute("method")||li,h=Or(c.getAttribute("formenctype"))||Or(z.getAttribute("enctype"))||ai,g=new FormData(z,c),!X1()){let{name:m,type:R,value:H}=c;if(R==="image"){let D=m?`${m}.`:"";g.append(`${D}x`,"0"),g.append(`${D}y`,"0")}else m&&g.append(m,H)}}else{if(ii(c))throw new Error('Cannot submit element that is not <form>, <button>, or <input type="submit|image">');o=li,f=null,h=ai,A=c}return g&&h==="text/plain"&&(A=g,g=void 0),{action:f,method:o.toLowerCase(),encType:h,formData:g,body:A}}function Yr(c,s){if(c===!1||c===null||typeof c>"u")throw new Error(s)}async function K1(c,s){if(c.id in s)return s[c.id];try{let o=await import(c.module);return s[c.id]=o,o}catch(o){return console.error(`Error loading route module \`${c.module}\`, reloading page...`),console.error(o),window.__reactRouterContext&&window.__reactRouterContext.isSpaMode,window.location.reload(),new Promise(()=>{})}}function J1(c){return c==null?!1:c.href==null?c.rel==="preload"&&typeof c.imageSrcSet=="string"&&typeof c.imageSizes=="string":typeof c.rel=="string"&&typeof c.href=="string"}async function $1(c,s,o){let f=await Promise.all(c.map(async h=>{let g=s.routes[h.route.id];if(g){let A=await K1(g,o);return A.links?A.links():[]}return[]}));return I1(f.flat(1).filter(J1).filter(h=>h.rel==="stylesheet"||h.rel==="preload").map(h=>h.rel==="stylesheet"?{...h,rel:"prefetch",as:"style"}:{...h,rel:"prefetch"}))}function Wd(c,s,o,f,h,g){let A=(b,m)=>o[m]?b.route.id!==o[m].route.id:!0,z=(b,m)=>o[m].pathname!==b.pathname||o[m].route.path?.endsWith("*")&&o[m].params["*"]!==b.params["*"];return g==="assets"?s.filter((b,m)=>A(b,m)||z(b,m)):g==="data"?s.filter((b,m)=>{let R=f.routes[b.route.id];if(!R||!R.hasLoader)return!1;if(A(b,m)||z(b,m))return!0;if(b.route.shouldRevalidate){let H=b.route.shouldRevalidate({currentUrl:new URL(h.pathname+h.search+h.hash,window.origin),currentParams:o[0]?.params||{},nextUrl:new URL(c,window.origin),nextParams:b.params,defaultShouldRevalidate:!0});if(typeof H=="boolean")return H}return!0}):[]}function W1(c,s,{includeHydrateFallback:o}={}){return F1(c.map(f=>{let h=s.routes[f.route.id];if(!h)return[];let g=[h.module];return h.clientActionModule&&(g=g.concat(h.clientActionModule)),h.clientLoaderModule&&(g=g.concat(h.clientLoaderModule)),o&&h.hydrateFallbackModule&&(g=g.concat(h.hydrateFallbackModule)),h.imports&&(g=g.concat(h.imports)),g}).flat(1))}function F1(c){return[...new Set(c)]}function P1(c){let s={},o=Object.keys(c).sort();for(let f of o)s[f]=c[f];return s}function I1(c,s){let o=new Set;return new Set(s),c.reduce((f,h)=>{let g=JSON.stringify(P1(h));return o.has(g)||(o.add(g),f.push({key:g,link:h})),f},[])}Object.getOwnPropertyNames(Object.prototype).sort().join("\0");var ey=new Set([100,101,204,205]);function ty(c,s){let o=typeof c=="string"?new URL(c,typeof window>"u"?"server://singlefetch/":window.location.origin):c;return o.pathname==="/"?o.pathname="_root.data":s&&$t(o.pathname,s)==="/"?o.pathname=`${s.replace(/\/$/,"")}/_root.data`:o.pathname=`${o.pathname.replace(/\/$/,"")}.data`,o}function r0(){let c=j.useContext(Oa);return Yr(c,"You must render this element inside a <DataRouterContext.Provider> element"),c}function ly(){let c=j.useContext(ni);return Yr(c,"You must render this element inside a <DataRouterStateContext.Provider> element"),c}var Lr=j.createContext(void 0);Lr.displayName="FrameworkContext";function f0(){let c=j.useContext(Lr);return Yr(c,"You must render this element inside a <HydratedRouter> element"),c}function ay(c,s){let o=j.useContext(Lr),[f,h]=j.useState(!1),[g,A]=j.useState(!1),{onFocus:z,onBlur:b,onMouseEnter:m,onMouseLeave:R,onTouchStart:H}=s,D=j.useRef(null);j.useEffect(()=>{if(c==="render"&&A(!0),c==="viewport"){let Q=B=>{B.forEach(ee=>{A(ee.isIntersecting)})},X=new IntersectionObserver(Q,{threshold:.5});return D.current&&X.observe(D.current),()=>{X.disconnect()}}},[c]),j.useEffect(()=>{if(f){let Q=setTimeout(()=>{A(!0)},100);return()=>{clearTimeout(Q)}}},[f]);let L=()=>{h(!0)},C=()=>{h(!1),A(!1)};return o?c!=="intent"?[g,D,{}]:[g,D,{onFocus:_u(z,L),onBlur:_u(b,C),onMouseEnter:_u(m,L),onMouseLeave:_u(R,C),onTouchStart:_u(H,L)}]:[!1,D,{}]}function _u(c,s){return o=>{c&&c(o),o.defaultPrevented||s(o)}}function uy({page:c,...s}){let{router:o}=r0(),f=j.useMemo(()=>Pd(o.routes,c,o.basename),[o.routes,c,o.basename]);return f?j.createElement(iy,{page:c,matches:f,...s}):null}function ny(c){let{manifest:s,routeModules:o}=f0(),[f,h]=j.useState([]);return j.useEffect(()=>{let g=!1;return $1(c,s,o).then(A=>{g||h(A)}),()=>{g=!0}},[c,s,o]),f}function iy({page:c,matches:s,...o}){let f=Wt(),{manifest:h,routeModules:g}=f0(),{basename:A}=r0(),{loaderData:z,matches:b}=ly(),m=j.useMemo(()=>Wd(c,s,b,h,f,"data"),[c,s,b,h,f]),R=j.useMemo(()=>Wd(c,s,b,h,f,"assets"),[c,s,b,h,f]),H=j.useMemo(()=>{if(c===f.pathname+f.search+f.hash)return[];let C=new Set,Q=!1;if(s.forEach(B=>{let ee=h.routes[B.route.id];!ee||!ee.hasLoader||(!m.some(P=>P.route.id===B.route.id)&&B.route.id in z&&g[B.route.id]?.shouldRevalidate||ee.hasClientLoader?Q=!0:C.add(B.route.id))}),C.size===0)return[];let X=ty(c,A);return Q&&C.size>0&&X.searchParams.set("_routes",s.filter(B=>C.has(B.route.id)).map(B=>B.route.id).join(",")),[X.pathname+X.search]},[A,z,f,h,m,s,c,g]),D=j.useMemo(()=>W1(R,h),[R,h]),L=ny(R);return j.createElement(j.Fragment,null,H.map(C=>j.createElement("link",{key:C,rel:"prefetch",as:"fetch",href:C,...o})),D.map(C=>j.createElement("link",{key:C,rel:"modulepreload",href:C,...o})),L.map(({key:C,link:Q})=>j.createElement("link",{key:C,...Q})))}function cy(...c){return s=>{c.forEach(o=>{typeof o=="function"?o(s):o!=null&&(o.current=s)})}}var s0=typeof window<"u"&&typeof window.document<"u"&&typeof window.document.createElement<"u";try{s0&&(window.__reactRouterVersion="7.6.3")}catch{}function ry({basename:c,children:s,window:o}){let f=j.useRef();f.current==null&&(f.current=$m({window:o,v5Compat:!0}));let h=f.current,[g,A]=j.useState({action:h.action,location:h.location}),z=j.useCallback(b=>{j.startTransition(()=>A(b))},[A]);return j.useLayoutEffect(()=>h.listen(z),[h,z]),j.createElement(B1,{basename:c,children:s,location:g.location,navigationType:g.action,navigator:h})}var o0=/^(?:[a-z][a-z0-9+.-]*:|\/\/)/i,et=j.forwardRef(function({onClick:s,discover:o="render",prefetch:f="none",relative:h,reloadDocument:g,replace:A,state:z,target:b,to:m,preventScrollReset:R,viewTransition:H,...D},L){let{basename:C}=j.useContext(Nt),Q=typeof m=="string"&&o0.test(m),X,B=!1;if(typeof m=="string"&&Q&&(X=m,s0))try{let be=new URL(window.location.href),Ke=m.startsWith("//")?new URL(be.protocol+m):new URL(m),ot=$t(Ke.pathname,C);Ke.origin===be.origin&&ot!=null?m=ot+Ke.search+Ke.hash:B=!0}catch{Tt(!1,`<Link to="${m}"> contains an invalid URL which will probably break when clicked - please update to a valid URL path.`)}let ee=E1(m,{relative:h}),[P,de,F]=ay(f,D),Me=dy(m,{replace:A,state:z,target:b,preventScrollReset:R,relative:h,viewTransition:H});function Te(be){s&&s(be),be.defaultPrevented||Me(be)}let je=j.createElement("a",{...D,...F,href:X||ee,onClick:B||g?s:Te,ref:cy(L,de),target:b,"data-discover":!Q&&o==="render"?"true":void 0});return P&&!Q?j.createElement(j.Fragment,null,je,j.createElement(uy,{page:ee})):je});et.displayName="Link";var fy=j.forwardRef(function({"aria-current":s="page",caseSensitive:o=!1,className:f="",end:h=!1,style:g,to:A,viewTransition:z,children:b,...m},R){let H=Bu(A,{relative:m.relative}),D=Wt(),L=j.useContext(ni),{navigator:C,basename:Q}=j.useContext(Nt),X=L!=null&&vy(H)&&z===!0,B=C.encodeLocation?C.encodeLocation(H).pathname:H.pathname,ee=D.pathname,P=L&&L.navigation&&L.navigation.location?L.navigation.location.pathname:null;o||(ee=ee.toLowerCase(),P=P?P.toLowerCase():null,B=B.toLowerCase()),P&&Q&&(P=$t(P,Q)||P);const de=B!=="/"&&B.endsWith("/")?B.length-1:B.length;let F=ee===B||!h&&ee.startsWith(B)&&ee.charAt(de)==="/",Me=P!=null&&(P===B||!h&&P.startsWith(B)&&P.charAt(B.length)==="/"),Te={isActive:F,isPending:Me,isTransitioning:X},je=F?s:void 0,be;typeof f=="function"?be=f(Te):be=[f,F?"active":null,Me?"pending":null,X?"transitioning":null].filter(Boolean).join(" ");let Ke=typeof g=="function"?g(Te):g;return j.createElement(et,{...m,"aria-current":je,className:be,ref:R,style:Ke,to:A,viewTransition:z},typeof b=="function"?b(Te):b)});fy.displayName="NavLink";var sy=j.forwardRef(({discover:c="render",fetcherKey:s,navigate:o,reloadDocument:f,replace:h,state:g,method:A=li,action:z,onSubmit:b,relative:m,preventScrollReset:R,viewTransition:H,...D},L)=>{let C=yy(),Q=gy(z,{relative:m}),X=A.toLowerCase()==="get"?"get":"post",B=typeof z=="string"&&o0.test(z),ee=P=>{if(b&&b(P),P.defaultPrevented)return;P.preventDefault();let de=P.nativeEvent.submitter,F=de?.getAttribute("formmethod")||A;C(de||P.currentTarget,{fetcherKey:s,method:F,navigate:o,replace:h,state:g,relative:m,preventScrollReset:R,viewTransition:H})};return j.createElement("form",{ref:L,method:X,action:Q,onSubmit:f?b:ee,...D,"data-discover":!B&&c==="render"?"true":void 0})});sy.displayName="Form";function oy(c){return`${c} must be used within a data router.  See https://reactrouter.com/en/main/routers/picking-a-router.`}function d0(c){let s=j.useContext(Oa);return Ee(s,oy(c)),s}function dy(c,{target:s,replace:o,state:f,preventScrollReset:h,relative:g,viewTransition:A}={}){let z=Hr(),b=Wt(),m=Bu(c,{relative:g});return j.useCallback(R=>{if(k1(R,s)){R.preventDefault();let H=o!==void 0?o:wu(b)===wu(m);z(c,{replace:H,state:f,preventScrollReset:h,relative:g,viewTransition:A})}},[b,z,m,o,f,s,c,h,g,A])}var hy=0,my=()=>`__${String(++hy)}__`;function yy(){let{router:c}=d0("useSubmit"),{basename:s}=j.useContext(Nt),o=U1();return j.useCallback(async(f,h={})=>{let{action:g,method:A,encType:z,formData:b,body:m}=V1(f,s);if(h.navigate===!1){let R=h.fetcherKey||my();await c.fetch(R,o,h.action||g,{preventScrollReset:h.preventScrollReset,formData:b,body:m,formMethod:h.method||A,formEncType:h.encType||z,flushSync:h.flushSync})}else await c.navigate(h.action||g,{preventScrollReset:h.preventScrollReset,formData:b,body:m,formMethod:h.method||A,formEncType:h.encType||z,replace:h.replace,state:h.state,fromRouteId:o,flushSync:h.flushSync,viewTransition:h.viewTransition})},[c,s,o])}function gy(c,{relative:s}={}){let{basename:o}=j.useContext(Nt),f=j.useContext(Dt);Ee(f,"useFormAction must be used inside a RouteContext");let[h]=f.matches.slice(-1),g={...Bu(c||".",{relative:s})},A=Wt();if(c==null){g.search=A.search;let z=new URLSearchParams(g.search),b=z.getAll("index");if(b.some(R=>R==="")){z.delete("index"),b.filter(H=>H).forEach(H=>z.append("index",H));let R=z.toString();g.search=R?`?${R}`:""}}return(!c||c===".")&&h.route.index&&(g.search=g.search?g.search.replace(/^\?/,"?index&"):"?index"),o!=="/"&&(g.pathname=g.pathname==="/"?o:Jt([o,g.pathname])),wu(g)}function vy(c,s={}){let o=j.useContext(l0);Ee(o!=null,"`useViewTransitionState` must be used within `react-router-dom`'s `RouterProvider`.  Did you accidentally import `RouterProvider` from `react-router`?");let{basename:f}=d0("useViewTransitionState"),h=Bu(c,{relative:s.relative});if(!o.isTransitioning)return!1;let g=$t(o.currentLocation.pathname,f)||o.currentLocation.pathname,A=$t(o.nextLocation.pathname,f)||o.nextLocation.pathname;return ui(h.pathname,A)!=null||ui(h.pathname,g)!=null}[...ey];const h0=j.createContext(null);function qu(){return j.useContext(h0)}const py=({toggleSidebar:c})=>{const{logout:s,isAuthenticated:o,userInfo:f}=qu(),[h,g]=j.useState(!1),A=j.useRef(null),[z,b]=j.useState(()=>typeof window<"u"?localStorage.getItem("color-theme")==="dark"||!("color-theme"in localStorage)&&window.matchMedia("(prefers-color-scheme: dark)").matches:!1);j.useEffect(()=>{const R=localStorage.getItem("color-theme");R==="dark"||!R&&window.matchMedia("(prefers-color-scheme: dark)").matches?(document.documentElement.classList.add("dark"),b(!0)):(document.documentElement.classList.remove("dark"),b(!1))},[]);const m=()=>{const R=!z;b(R),R?(document.documentElement.classList.add("dark"),localStorage.setItem("color-theme","dark")):(document.documentElement.classList.remove("dark"),localStorage.setItem("color-theme","light")),document.dispatchEvent(new Event("themeChanged"))};return j.useEffect(()=>{function R(H){A.current&&!A.current.contains(H.target)&&g(!1)}return document.addEventListener("mousedown",R),()=>{document.removeEventListener("mousedown",R)}},[]),j.useEffect(()=>{},[o]),d.jsx("nav",{className:"fixed z-30 w-full bg-white border-b border-gray-200 dark:bg-gray-800 dark:border-gray-700 py-3 px-4",children:d.jsxs("div",{className:"flex justify-between items-center max-w-screen-2xl mx-auto",children:[d.jsxs("div",{className:"flex justify-start items-center",children:[d.jsxs("button",{"data-drawer-target":"logo-sidebar","data-drawer-toggle":"logo-sidebar","aria-controls":"logo-sidebar",type:"button",className:"inline-flex items-center p-2 text-sm text-gray-500 rounded-lg md:hidden hover:bg-gray-100 focus:outline-none focus:ring-2 focus:ring-gray-200 dark:text-gray-400 dark:hover:bg-gray-700 dark:focus:ring-gray-600",onClick:c,children:[d.jsx("span",{className:"sr-only",children:"Open sidebar"}),d.jsx("svg",{className:"w-6 h-6","aria-hidden":"true",fill:"currentColor",viewBox:"0 0 20 20",xmlns:"http://www.w3.org/2000/svg",children:d.jsx("path",{clipRule:"evenodd",fillRule:"evenodd",d:"M2 4.75A.75.75 0 012.75 4h14.5a.75.75 0 010 1.5H2.75A.75.75 0 012 4.75zm0 10.5a.75.75 0 01.75-.75h7.5a.75.75 0 010 1.5h-7.5a.75.75 0 01-.75-.75zM2 10a.75.75 0 01.75-.75h14.5a.75.75 0 010 1.5H2.75A.75.75 0 012 10z"})})]}),d.jsxs(et,{to:"/",className:"flex items-center ml-2 md:mr-14",children:[d.jsx("img",{src:"/images/logo.svg",className:"h-8 mr-3",alt:"VitalGraph Logo"}),d.jsx("span",{className:"self-center hidden sm:flex text-2xl font-semibold whitespace-nowrap dark:text-white",children:"VitalGraph"})]}),d.jsx("div",{className:"hidden justify-between items-center w-full lg:flex lg:w-auto lg:order-1",children:d.jsxs("ul",{className:"flex flex-col mt-4 space-x-6 text-sm font-medium lg:flex-row xl:space-x-8 lg:mt-0",children:[d.jsx("li",{children:d.jsx(et,{to:"/",className:"block text-gray-700 hover:text-primary-700 dark:text-gray-400 dark:hover:text-white","aria-current":"page",children:"Home"})}),d.jsx("li",{children:d.jsx(et,{to:"/spaces",className:"block text-gray-700 hover:text-primary-700 dark:text-gray-400 dark:hover:text-white",children:"Spaces"})}),d.jsx("li",{children:d.jsx(et,{to:"/query",className:"block text-gray-700 hover:text-primary-700 dark:text-gray-400 dark:hover:text-white",children:"SPARQL"})})]})})]}),d.jsxs("div",{className:"flex items-center",children:[d.jsx("button",{onClick:m,type:"button",className:"text-gray-500 dark:text-gray-400 hover:bg-gray-100 dark:hover:bg-gray-700 focus:outline-none focus:ring-4 focus:ring-gray-200 dark:focus:ring-gray-700 rounded-lg text-sm p-2.5 mr-1",children:z?d.jsx("svg",{className:"w-5 h-5",fill:"currentColor",viewBox:"0 0 20 20",xmlns:"http://www.w3.org/2000/svg",children:d.jsx("path",{d:"M10 2a1 1 0 011 1v1a1 1 0 11-2 0V3a1 1 0 011-1zm4 8a4 4 0 11-8 0 4 4 0 018 0zm-.464 4.95l.707.707a1 1 0 001.414-1.414l-.707-.707a1 1 0 00-1.414 1.414zm2.12-10.607a1 1 0 010 1.414l-.706.707a1 1 0 11-1.414-1.414l.707-.707a1 1 0 011.414 0zM17 11a1 1 0 100-2h-1a1 1 0 100 2h1zm-7 4a1 1 0 011 1v1a1 1 0 11-2 0v-1a1 1 0 011-1zM5.05 6.464A1 1 0 106.465 5.05l-.708-.707a1 1 0 00-1.414 1.414l.707.707zm1.414 8.486l-.707.707a1 1 0 01-1.414-1.414l.707-.707a1 1 0 011.414 1.414zM4 11a1 1 0 100-2H3a1 1 0 000 2h1z",fillRule:"evenodd",clipRule:"evenodd"})}):d.jsx("svg",{className:"w-5 h-5",fill:"currentColor",viewBox:"0 0 20 20",xmlns:"http://www.w3.org/2000/svg",children:d.jsx("path",{d:"M17.293 13.293A8 8 0 016.707 2.707a8.001 8.001 0 1010.586 10.586z"})})}),d.jsxs("div",{className:"flex items-center ml-3 relative",children:[d.jsx("div",{children:d.jsxs("button",{type:"button",className:"flex text-sm bg-gray-800 rounded-full focus:ring-4 focus:ring-gray-300 dark:focus:ring-gray-600",id:"user-menu-button","aria-expanded":h,onClick:()=>g(!h),children:[d.jsx("span",{className:"sr-only",children:"Open user menu"}),d.jsx("img",{className:"w-8 h-8 rounded-full",src:f?.profile_image||"/images/users/bonnie-green.png",alt:"User photo"})]})}),d.jsxs("div",{ref:A,className:`z-50 absolute right-0 top-10 mt-2 text-base list-none bg-white divide-y divide-gray-100 rounded shadow dark:bg-gray-700 dark:divide-gray-600 ${h?"block":"hidden"}`,id:"user-dropdown",style:{minWidth:"200px"},children:[d.jsxs("div",{className:"px-4 py-3",children:[d.jsx("span",{className:"block text-sm text-gray-900 dark:text-white",children:f?.full_name||"User"}),d.jsx(et,{to:"/profile",className:"block text-sm font-medium text-gray-500 truncate dark:text-gray-400 hover:text-primary-600 dark:hover:text-primary-500",children:f?.email||"email@example.com"})]}),d.jsxs("ul",{className:"py-1","aria-labelledby":"user-menu-button",children:[d.jsx("li",{children:d.jsx(et,{to:"/settings",className:"block px-4 py-2 text-sm text-gray-700 hover:bg-gray-100 dark:hover:bg-gray-600 dark:text-gray-200 dark:hover:text-white",children:"Settings"})}),d.jsx("li",{children:d.jsx("button",{onClick:s,className:"block w-full text-left px-4 py-2 text-sm text-gray-700 hover:bg-gray-100 dark:hover:bg-gray-600 dark:text-gray-200 dark:hover:text-white",children:"Sign out"})})]})]})]})]})]})})},by=({isOpen:c})=>{const s=Wt(),o=h=>h==="/"&&s.pathname==="/"?!0:s.pathname===h,f=h=>{const g="flex items-center p-2 rounded-lg ";return o(h)?{className:`${g} text-white`,style:{backgroundColor:"#1d4ed8",color:"white"}}:{className:`${g} text-gray-900 dark:text-white hover:bg-gray-100 dark:hover:bg-gray-700`,style:{}}};return d.jsx("aside",{id:"logo-sidebar",className:`fixed top-0 left-0 z-20 w-64 h-screen pt-16 transition-transform 
        bg-white border-r border-gray-200 dark:bg-gray-800 dark:border-gray-700 
        ${c?"translate-x-0":"-translate-x-full"} md:translate-x-0`,"aria-label":"Sidebar",children:d.jsxs("div",{className:"h-full px-3 pb-4 overflow-y-auto bg-white dark:bg-gray-800",children:[d.jsxs("ul",{className:"space-y-2 font-medium",children:[d.jsx("li",{children:d.jsxs(et,{to:"/",className:f("/").className,style:f("/").style,children:[d.jsxs("svg",{className:"w-5 h-5 transition duration-75","aria-hidden":"true",xmlns:"http://www.w3.org/2000/svg",fill:"currentColor",viewBox:"0 0 22 21",children:[d.jsx("path",{d:"M16.975 11H10V4.025a1 1 0 0 0-1.066-.998 8.5 8.5 0 1 0 9.039 9.039.999.999 0 0 0-1-1.066h.002Z"}),d.jsx("path",{d:"M12.5 0c-.157 0-.311.01-.565.027A1 1 0 0 0 11 1.02V10h8.975a1 1 0 0 0 1-.935c.013-.188.028-.374.028-.565A8.51 8.51 0 0 0 12.5 0Z"})]}),d.jsx("span",{className:"ml-3",children:"Home"})]})}),d.jsx("li",{children:d.jsxs(et,{to:"/spaces",className:f("/spaces").className,style:f("/spaces").style,children:[d.jsx("svg",{className:"flex-shrink-0 w-5 h-5 transition duration-75","aria-hidden":"true",xmlns:"http://www.w3.org/2000/svg",fill:"currentColor",viewBox:"0 0 18 18",children:d.jsx("path",{d:"M6.143 0H1.857A1.857 1.857 0 0 0 0 1.857v4.286C0 7.169.831 8 1.857 8h4.286A1.857 1.857 0 0 0 8 6.143V1.857A1.857 1.857 0 0 0 6.143 0Zm10 0h-4.286A1.857 1.857 0 0 0 10 1.857v4.286C10 7.169 10.831 8 11.857 8h4.286A1.857 1.857 0 0 0 18 6.143V1.857A1.857 1.857 0 0 0 16.143 0Zm-10 10H1.857A1.857 1.857 0 0 0 0 11.857v4.286C0 17.169.831 18 1.857 18h4.286A1.857 1.857 0 0 0 8 16.143v-4.286A1.857 1.857 0 0 0 6.143 10Zm10 0h-4.286A1.857 1.857 0 0 0 10 11.857v4.286c0 1.026.831 1.857 1.857 1.857h4.286A1.857 1.857 0 0 0 18 16.143v-4.286A1.857 1.857 0 0 0 16.143 10Z"})}),d.jsx("span",{className:"flex-1 ml-3 whitespace-nowrap",children:"Spaces"})]})}),d.jsx("li",{children:d.jsxs(et,{to:"/query",className:f("/query").className,style:f("/query").style,children:[d.jsxs("svg",{className:"flex-shrink-0 w-5 h-5 transition duration-75","aria-hidden":"true",xmlns:"http://www.w3.org/2000/svg",fill:"currentColor",viewBox:"0 0 20 20",children:[d.jsx("path",{d:"M5 5V.13a2.96 2.96 0 0 0-1.293.749L.879 3.707A2.96 2.96 0 0 0 .13 5H5Z"}),d.jsx("path",{d:"M6.737 11.061a2.961 2.961 0 0 1 .81-1.515l6.117-6.116A4.839 4.839 0 0 1 16 2.141V2a1.97 1.97 0 0 0-1.933-2H7v5a2 2 0 0 1-2 2H0v11a1.969 1.969 0 0 0 1.933 2h12.134A1.97 1.97 0 0 0 16 18v-3.093l-1.546 1.546c-.413.413-.94.695-1.513.81l-3.4.679a2.947 2.947 0 0 1-1.85-.227 2.96 2.96 0 0 1-1.635-3.257l.681-3.397Z"}),d.jsx("path",{d:"M8.961 16a.93.93 0 0 0 .189-.019l3.4-.679a.961.961 0 0 0 .49-.263l6.118-6.117a2.884 2.884 0 0 0-4.079-4.078l-6.117 6.117a.96.96 0 0 0-.263.491l-.679 3.4A.961.961 0 0 0 8.961 16Zm7.477-9.8a.958.958 0 0 1 .68-.281.961.961 0 0 1 .682 1.644l-.315.315-1.36-1.36.313-.318Zm-5.911 5.911 4.236-4.236 1.359 1.359-4.236 4.237-1.7.339.341-1.699Z"})]}),d.jsx("span",{className:"flex-1 ml-3 whitespace-nowrap",children:"SPARQL Query"})]})}),d.jsx("li",{children:d.jsxs(et,{to:"/settings",className:f("/settings").className,style:f("/settings").style,children:[d.jsx("svg",{className:"flex-shrink-0 w-5 h-5 transition duration-75","aria-hidden":"true",xmlns:"http://www.w3.org/2000/svg",fill:"currentColor",viewBox:"0 0 20 20",children:d.jsx("path",{d:"M5 4a1 1 0 0 0-2 0v7.268a2 2 0 0 0 0 3.464V16a1 1 0 1 0 2 0v-1.268a2 2 0 0 0 0-3.464V4Zm6 0a1 1 0 0 0-2 0v1.268a2 2 0 0 0 0 3.464V16a1 1 0 1 0 2 0V8.732a2 2 0 0 0 0-3.464V4Zm6 0a1 1 0 1 0-2 0v7.268a2 2 0 0 0 0 3.464V16a1 1 0 1 0 2 0v-1.268a2 2 0 0 0 0-3.464V4Z"})}),d.jsx("span",{className:"flex-1 ml-3 whitespace-nowrap",children:"Settings"})]})})]}),d.jsx("div",{className:"pt-4 mt-4 space-y-2 font-medium border-t border-gray-200 dark:border-gray-700",children:d.jsxs("a",{href:"https://github.com/vital-ai/vital-graph",target:"_blank",rel:"noreferrer",className:"flex items-center p-2 text-gray-900 rounded-lg dark:text-white hover:bg-gray-100 dark:hover:bg-gray-700",children:[d.jsx("svg",{className:"flex-shrink-0 w-5 h-5 transition duration-75","aria-hidden":"true",xmlns:"http://www.w3.org/2000/svg",fill:"currentColor",viewBox:"0 0 20 20",children:d.jsx("path",{fillRule:"evenodd",d:"M10 .333A9.911 9.911 0 0 0 6.866 19.65c.5.092.678-.215.678-.477 0-.237-.01-1.017-.014-1.845-2.757.6-3.338-1.169-3.338-1.169a2.627 2.627 0 0 0-1.1-1.451c-.9-.615.07-.6.07-.6a2.084 2.084 0 0 1 1.518 1.021 2.11 2.11 0 0 0 2.884.823c.044-.503.268-.973.63-1.325-2.2-.25-4.516-1.1-4.516-4.9A3.832 3.832 0 0 1 4.7 7.068a3.56 3.56 0 0 1 .095-2.623s.832-.266 2.726 1.016a9.409 9.409 0 0 1 4.962 0c1.89-1.282 2.717-1.016 2.717-1.016.366.83.402 1.768.1 2.623a3.827 3.827 0 0 1 1.02 2.659c0 3.807-2.319 4.644-4.525 4.889a2.366 2.366 0 0 1 .673 1.834c0 1.326-.012 2.394-.012 2.72 0 .263.18.572.681.475A9.911 9.911 0 0 0 10 .333Z",clipRule:"evenodd"})}),d.jsx("span",{className:"ml-3",children:"GitHub"})]})})]})})},Cu={apiUrl:"http://localhost:8001"};function xy(){const[c,s]=j.useState([]),[o,f]=j.useState(!0),[h,g]=j.useState("");return j.useEffect(()=>{(async()=>{try{const z=localStorage.getItem("authToken"),b=await fetch(`${Cu.apiUrl}/api/spaces`,{headers:{Authorization:`Bearer ${z}`}});if(!b.ok)throw new Error("Failed to fetch spaces");const m=await b.json();s(m)}catch(z){console.error("Error fetching spaces:",z),g("Failed to load spaces")}finally{f(!1)}})()},[]),d.jsxs("div",{children:[d.jsx("div",{className:"p-4 bg-white block sm:flex items-center justify-between border-b border-gray-200 lg:mt-1.5 dark:bg-gray-800 dark:border-gray-700",children:d.jsxs("div",{className:"w-full mb-1",children:[d.jsx("div",{className:"mb-4",children:d.jsx("h1",{className:"text-xl font-semibold text-gray-900 sm:text-2xl dark:text-white",children:"Spaces"})}),d.jsx("div",{className:"items-center justify-between block sm:flex md:divide-x md:divide-gray-100 dark:divide-gray-700",children:d.jsx("div",{className:"flex items-center mb-4 sm:mb-0",children:d.jsx("button",{id:"createSpaceButton",className:"text-white bg-primary-700 hover:bg-primary-800 focus:ring-4 focus:ring-primary-300 font-medium rounded-lg text-sm px-5 py-2.5 dark:bg-primary-600 dark:hover:bg-primary-700 focus:outline-none dark:focus:ring-primary-800",type:"button",children:"Create space"})})})]})}),o&&d.jsx("div",{className:"flex items-center justify-center p-8",children:d.jsx("div",{className:"animate-spin rounded-full h-12 w-12 border-t-2 border-b-2 border-blue-500"})}),h&&d.jsx("div",{className:"p-4 text-sm text-red-700 bg-red-100 rounded-lg dark:bg-red-200 dark:text-red-800",role:"alert",children:h}),d.jsx("div",{className:"grid grid-cols-1 md:grid-cols-2 lg:grid-cols-3 gap-4 p-4 mt-4",children:c.length>0?c.map(A=>d.jsxs("div",{className:"bg-white dark:bg-gray-800 border border-gray-200 dark:border-gray-700 rounded-lg shadow p-4",children:[d.jsx("h3",{className:"text-lg font-medium text-gray-900 dark:text-white",children:A.name}),d.jsx("p",{className:"text-sm text-gray-500 dark:text-gray-400 mb-3",children:A.description||"No description"}),d.jsxs("div",{className:"flex space-x-2",children:[d.jsx("button",{className:"text-white bg-primary-700 hover:bg-primary-800 focus:ring-4 focus:ring-primary-300 font-medium rounded-lg text-xs px-3 py-1.5 dark:bg-primary-600 dark:hover:bg-primary-700 focus:outline-none dark:focus:ring-primary-800",children:"View"}),d.jsx("button",{className:"text-gray-900 bg-white border border-gray-300 hover:bg-gray-100 focus:ring-4 focus:ring-gray-200 font-medium rounded-lg text-xs px-3 py-1.5 dark:bg-gray-800 dark:text-gray-400 dark:border-gray-600 dark:hover:bg-gray-700 dark:hover:border-gray-600 dark:focus:ring-gray-700",children:"Edit"})]})]},A.id)):!o&&d.jsx("p",{className:"text-gray-500 dark:text-gray-400 col-span-3",children:"No spaces found. Create one to get started."})})]})}function Sy(){const[c,s]=j.useState(`# Enter your SPARQL query here
PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>

SELECT ?subject ?predicate ?object
WHERE {
  ?subject ?predicate ?object
}
LIMIT 10`),[o,f]=j.useState(null),[h,g]=j.useState(!1),[A,z]=j.useState(""),b=async()=>{if(!c.trim()){z("Query cannot be empty");return}g(!0),z(""),f(null);try{const m=localStorage.getItem("authToken"),R=await fetch(`${Cu.apiUrl}/api/query`,{method:"POST",headers:{"Content-Type":"application/json",Authorization:`Bearer ${m}`},body:JSON.stringify({query:c})});if(!R.ok){const D=await R.text();throw new Error(D||"Failed to execute query")}const H=await R.json();f(H.results||[])}catch(m){console.error("Error executing query:",m),z(m.message||"An error occurred while executing the query")}finally{g(!1)}};return d.jsxs("div",{children:[d.jsx("div",{className:"p-4 bg-white block sm:flex items-center justify-between border-b border-gray-200 lg:mt-1.5 dark:bg-gray-800 dark:border-gray-700",children:d.jsx("div",{className:"w-full mb-1",children:d.jsx("div",{className:"mb-4",children:d.jsx("h1",{className:"text-xl font-semibold text-gray-900 sm:text-2xl dark:text-white",children:"SPARQL Query"})})})}),d.jsxs("div",{className:"p-4",children:[d.jsxs("div",{className:"mb-4",children:[d.jsx("label",{htmlFor:"query",className:"block mb-2 text-sm font-medium text-gray-900 dark:text-white",children:"Query"}),d.jsx("textarea",{id:"query",rows:10,className:"block p-2.5 w-full text-sm text-gray-900 bg-gray-50 rounded-lg border border-gray-300 focus:ring-primary-500 focus:border-primary-500 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-primary-500 dark:focus:border-primary-500",placeholder:"Enter your SPARQL query...",value:c,onChange:m=>s(m.target.value)})]}),d.jsx("button",{type:"button",className:"text-white bg-primary-700 hover:bg-primary-800 focus:ring-4 focus:ring-primary-300 font-medium rounded-lg text-sm px-5 py-2.5 dark:bg-primary-600 dark:hover:bg-primary-700 focus:outline-none dark:focus:ring-primary-800",onClick:b,disabled:h,children:h?"Running...":"Run Query"}),h&&d.jsx("div",{className:"flex items-center justify-center p-8",children:d.jsx("div",{className:"animate-spin rounded-full h-8 w-8 border-t-2 border-b-2 border-blue-500"})}),A&&d.jsx("div",{className:"p-4 mt-4 text-sm text-red-700 bg-red-100 rounded-lg dark:bg-red-200 dark:text-red-800",role:"alert",children:A}),o&&d.jsxs("div",{className:"mt-6",children:[d.jsx("h2",{className:"text-lg font-semibold text-gray-900 dark:text-white mb-2",children:"Results"}),d.jsx("div",{className:"overflow-x-auto relative shadow-md sm:rounded-lg",children:d.jsxs("table",{className:"w-full text-sm text-left text-gray-500 dark:text-gray-400",children:[d.jsx("thead",{className:"text-xs text-gray-700 uppercase bg-gray-50 dark:bg-gray-700 dark:text-gray-400",children:d.jsx("tr",{children:o.length>0&&Object.keys(o[0]).map((m,R)=>d.jsx("th",{scope:"col",className:"py-3 px-6",children:m},R))})}),d.jsx("tbody",{children:o.map((m,R)=>d.jsx("tr",{className:"bg-white border-b dark:bg-gray-800 dark:border-gray-700",children:Object.values(m).map((H,D)=>d.jsx("td",{className:"py-4 px-6",children:typeof H=="object"?JSON.stringify(H):H},D))},R))})]})})]})]})]})}function Ey(){const[c,s]=j.useState(Cu.apiUrl),[o,f]=j.useState(null),{userInfo:h}=qu(),g=()=>{localStorage.setItem("apiUrl",c),f({type:"success",message:"Settings saved successfully!"}),setTimeout(()=>{f(null)},3e3)};return d.jsxs("div",{children:[d.jsx("div",{className:"p-4 bg-white block sm:flex items-center justify-between border-b border-gray-200 lg:mt-1.5 dark:bg-gray-800 dark:border-gray-700",children:d.jsx("div",{className:"w-full mb-1",children:d.jsx("div",{className:"mb-4",children:d.jsx("h1",{className:"text-xl font-semibold text-gray-900 sm:text-2xl dark:text-white",children:"Settings"})})})}),d.jsxs("div",{className:"p-4",children:[o&&d.jsx("div",{className:`p-4 mb-4 text-sm rounded-lg ${o.type==="success"?"text-green-700 bg-green-100 dark:bg-green-200 dark:text-green-800":"text-red-700 bg-red-100 dark:bg-red-200 dark:text-red-800"}`,role:"alert",children:o.message}),d.jsx("div",{className:"grid gap-6 mb-6 md:grid-cols-2",children:d.jsxs("div",{children:[d.jsx("label",{htmlFor:"api_url",className:"block mb-2 text-sm font-medium text-gray-900 dark:text-white",children:"API URL"}),d.jsx("input",{type:"url",id:"api_url",className:"bg-gray-50 border border-gray-300 text-gray-900 text-sm rounded-lg focus:ring-primary-500 focus:border-primary-500 block w-full p-2.5 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-primary-500 dark:focus:border-primary-500",placeholder:"http://localhost:8001",value:c,onChange:A=>s(A.target.value),required:!0}),d.jsx("p",{className:"mt-1 text-xs text-gray-500 dark:text-gray-400",children:"Backend server URL for API requests"})]})}),d.jsx("h2",{className:"text-xl font-semibold text-gray-900 dark:text-white mb-4",children:"User Information"}),d.jsx("div",{className:"mb-6 p-4 bg-white border border-gray-200 rounded-lg shadow-sm dark:border-gray-700 dark:bg-gray-800",children:d.jsxs("div",{className:"flex items-center space-x-4 mb-4",children:[d.jsx("img",{className:"h-12 w-12 rounded-full",src:h?.profile_image||"/images/users/bonnie-green.png",alt:"User profile"}),d.jsxs("div",{children:[d.jsx("p",{className:"text-base font-medium text-gray-900 dark:text-white",children:h?.full_name||"User"}),d.jsx("p",{className:"text-sm text-gray-500 dark:text-gray-400",children:h?.email||"email@example.com"}),d.jsxs("p",{className:"text-sm text-gray-500 dark:text-gray-400",children:["Role: ",h?.role||"User"]})]})]})}),d.jsx("h2",{className:"text-xl font-semibold text-gray-900 dark:text-white mb-4",children:"User Preferences"}),d.jsx("div",{className:"grid gap-6 mb-6 md:grid-cols-2",children:d.jsxs("div",{className:"flex items-center",children:[d.jsx("input",{id:"default-checkbox",type:"checkbox",className:"w-4 h-4 text-primary-600 bg-gray-100 border-gray-300 rounded focus:ring-primary-500 dark:focus:ring-primary-600 dark:ring-offset-gray-800 focus:ring-2 dark:bg-gray-700 dark:border-gray-600",checked:!0,readOnly:!0}),d.jsx("label",{htmlFor:"default-checkbox",className:"ms-2 text-sm font-medium text-gray-900 dark:text-white",children:"Enable notifications"})]})}),d.jsx("button",{type:"button",className:"text-white bg-primary-700 hover:bg-primary-800 focus:ring-4 focus:ring-primary-300 font-medium rounded-lg text-sm px-5 py-2.5 me-2 mb-2 dark:bg-primary-600 dark:hover:bg-primary-700 focus:outline-none dark:focus:ring-primary-800",onClick:g,children:"Save settings"})]})]})}function Ay(){const{userInfo:c}=qu();return d.jsxs("div",{children:[d.jsx("div",{className:"p-4 bg-white block sm:flex items-center justify-between border-b border-gray-200 lg:mt-1.5 dark:bg-gray-800 dark:border-gray-700",children:d.jsx("div",{className:"w-full mb-1",children:d.jsx("div",{className:"mb-4",children:d.jsx("h1",{className:"text-xl font-semibold text-gray-900 sm:text-2xl dark:text-white",children:"Profile"})})})}),d.jsx("div",{className:"p-4",children:d.jsxs("div",{className:"p-4 mb-4 bg-white border border-gray-200 rounded-lg shadow-sm 2xl:col-span-2 dark:border-gray-700 sm:p-6 dark:bg-gray-800",children:[d.jsxs("div",{className:"flex items-center space-x-4 mb-6",children:[d.jsx("div",{className:"shrink-0",children:d.jsx("img",{className:"h-16 w-16 rounded-full",src:c?.profile_image||"/images/users/bonnie-green.png",alt:"User profile"})}),d.jsxs("div",{className:"min-w-0 flex-1",children:[d.jsx("p",{className:"text-lg font-medium text-gray-900 truncate dark:text-white",children:c?.full_name||"User"}),d.jsx("p",{className:"text-sm text-gray-500 truncate dark:text-gray-400",children:c?.email||"email@example.com"}),d.jsx("p",{className:"text-sm text-gray-500 truncate dark:text-gray-400",children:c?.role||"User"})]}),d.jsx("div",{className:"inline-flex items-center",children:d.jsx("button",{type:"button",className:"py-2 px-3 text-sm font-medium text-center text-white rounded-lg bg-primary-700 hover:bg-primary-800 focus:ring-4 focus:ring-primary-300 dark:bg-primary-600 dark:hover:bg-primary-700 dark:focus:ring-primary-800",children:"Edit Profile"})})]}),d.jsxs("div",{className:"grid grid-cols-1 md:grid-cols-2 gap-6",children:[d.jsxs("div",{children:[d.jsx("label",{htmlFor:"user-bio",className:"block mb-2 text-sm font-medium text-gray-900 dark:text-white",children:"Bio"}),d.jsx("textarea",{id:"user-bio",rows:4,className:"bg-gray-50 border border-gray-300 text-gray-900 text-sm rounded-lg focus:ring-primary-600 focus:border-primary-600 block w-full p-2.5 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-primary-500 dark:focus:border-primary-500",placeholder:"Your biography",value:"System administrator with full access to all features.",readOnly:!0})]}),d.jsxs("div",{children:[d.jsx("h3",{className:"mb-2 text-sm font-medium text-gray-900 dark:text-white",children:"Account Information"}),d.jsxs("dl",{className:"grid grid-cols-1 gap-4 sm:grid-cols-2 text-gray-900 dark:text-white",children:[d.jsxs("div",{children:[d.jsx("dt",{className:"text-sm font-normal text-gray-500 dark:text-gray-400",children:"Member since"}),d.jsx("dd",{className:"text-sm font-semibold",children:"January 2023"})]}),d.jsxs("div",{children:[d.jsx("dt",{className:"text-sm font-normal text-gray-500 dark:text-gray-400",children:"Role"}),d.jsx("dd",{className:"text-sm font-semibold",children:c?.role||"User"})]}),d.jsxs("div",{children:[d.jsx("dt",{className:"text-sm font-normal text-gray-500 dark:text-gray-400",children:"Email verification"}),d.jsxs("dd",{className:"text-sm font-semibold",children:["Verified ",d.jsx("span",{className:"text-green-500",children:"✓"})]})]}),d.jsxs("div",{children:[d.jsx("dt",{className:"text-sm font-normal text-gray-500 dark:text-gray-400",children:"2FA Authentication"}),d.jsx("dd",{className:"text-sm font-semibold text-red-500",children:"Not enabled"})]})]})]})]})]})})]})}const Ty=({children:c})=>{const[s,o]=j.useState(!1),[f,h]=j.useState(!0),[g,A]=j.useState(null);j.useEffect(()=>{(async()=>{if(localStorage.getItem("authToken")){o(!0);const H=localStorage.getItem("userInfo");if(H)try{A(JSON.parse(H))}catch(D){console.error("Error parsing user info:",D)}}h(!1)})()},[]);const z=async(m,R)=>{try{const H=new URLSearchParams;H.append("username",m),H.append("password",R);const D=await fetch(`${Cu.apiUrl}/api/login`,{method:"POST",headers:{"Content-Type":"application/x-www-form-urlencoded"},body:H});if(D.ok){const L=await D.json();localStorage.setItem("authToken",L.access_token),localStorage.setItem("tokenType",L.token_type);const C={username:L.username,full_name:L.full_name,email:L.email,profile_image:L.profile_image,role:L.role};A(C),localStorage.setItem("userInfo",JSON.stringify(C)),o(!0)}else throw new Error("Login failed")}catch(H){throw console.error("Login error:",H),H}},b=async()=>{try{const m=localStorage.getItem("authToken");m&&await fetch(`${Cu.apiUrl}/api/logout`,{method:"POST",headers:{Authorization:`Bearer ${m}`}})}catch(m){console.error("Logout error:",m)}finally{localStorage.removeItem("authToken"),localStorage.removeItem("userInfo"),o(!1),A(null)}};return d.jsx(h0.Provider,{value:{isAuthenticated:s,isLoading:f,userInfo:g,login:z,logout:b},children:f?d.jsx("div",{className:"flex items-center justify-center h-screen bg-gray-50 dark:bg-gray-900",children:d.jsxs("div",{className:"text-center",children:[d.jsx("div",{className:"animate-spin rounded-full h-12 w-12 border-t-2 border-b-2 border-blue-500 mx-auto"}),d.jsx("p",{className:"mt-3 text-gray-600 dark:text-gray-300",children:"Loading..."})]})}):c})},Du=({children:c})=>{const{isAuthenticated:s}=qu();return s?d.jsx(d.Fragment,{children:c}):d.jsx(c0,{to:"/login",replace:!0})},Uu=({children:c})=>{const[s,o]=j.useState(!0),f=()=>{o(!s)};return d.jsxs("div",{className:"flex flex-col h-screen bg-gray-50 dark:bg-gray-900",children:[d.jsx(py,{toggleSidebar:f}),d.jsxs("div",{className:"flex flex-1 overflow-hidden pt-16",children:[" ",d.jsx(by,{isOpen:s}),d.jsxs("main",{className:"md:ml-64 flex-1 overflow-auto p-4",children:[" ",c]})]})]})};function Ny(){const[c,s]=j.useState(""),[o,f]=j.useState(""),[h,g]=j.useState(""),{login:A}=qu(),z=Hr(),b=async m=>{if(m.preventDefault(),g(""),!c||!o){g("Username and password are required");return}try{await A(c,o),z("/")}catch{g("Invalid username or password")}};return d.jsx("div",{className:"bg-gray-50 dark:bg-gray-900",children:d.jsxs("div",{className:"flex flex-col items-center justify-center px-6 pt-8 mx-auto md:h-screen pt:mt-0 dark:bg-gray-900",children:[d.jsxs("a",{href:"/",className:"flex items-center justify-center mb-8 text-2xl font-semibold lg:mb-10 dark:text-white",children:[d.jsx("img",{src:"/images/logo.svg",className:"mr-4 h-11",alt:"VITAL Graph Logo"}),d.jsx("span",{children:"VITAL Graph"})]}),d.jsxs("div",{className:"w-full max-w-xl p-6 space-y-8 sm:p-8 bg-white rounded-lg shadow dark:bg-gray-800",children:[d.jsx("h2",{className:"text-2xl font-bold text-gray-900 dark:text-white",children:"Sign in to VITAL Graph"}),d.jsxs("form",{className:"mt-8 space-y-6",onSubmit:b,children:[h&&d.jsx("div",{className:"p-4 mb-4 text-sm text-red-700 bg-red-100 rounded-lg dark:bg-red-200 dark:text-red-800",role:"alert",children:h}),d.jsxs("div",{children:[d.jsx("label",{htmlFor:"username",className:"block mb-2 text-sm font-medium text-gray-900 dark:text-white",children:"Username"}),d.jsx("input",{type:"text",name:"username",id:"username",value:c,onChange:m=>s(m.target.value),className:"bg-gray-50 border border-gray-300 text-gray-900 sm:text-sm rounded-lg focus:ring-primary-500 focus:border-primary-500 block w-full p-2.5 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-primary-500 dark:focus:border-primary-500",placeholder:"username",required:!0})]}),d.jsxs("div",{children:[d.jsx("label",{htmlFor:"password",className:"block mb-2 text-sm font-medium text-gray-900 dark:text-white",children:"Password"}),d.jsx("input",{type:"password",name:"password",id:"password",value:o,onChange:m=>f(m.target.value),placeholder:"••••••••",className:"bg-gray-50 border border-gray-300 text-gray-900 sm:text-sm rounded-lg focus:ring-primary-500 focus:border-primary-500 block w-full p-2.5 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-primary-500 dark:focus:border-primary-500",required:!0})]}),d.jsx("button",{type:"submit",className:"w-full px-5 py-3 text-base font-medium text-center text-white bg-blue-700 rounded-lg hover:bg-blue-800 focus:ring-4 focus:ring-blue-300 sm:w-auto dark:bg-blue-600 dark:hover:bg-blue-700 dark:focus:ring-blue-800",children:"Login to your account"})]})]})]})})}function Ry(){return d.jsxs("div",{className:"p-4",children:[d.jsx("div",{className:"p-4 bg-white block sm:flex items-center justify-between border-b border-gray-200 lg:mt-1.5 dark:bg-gray-800 dark:border-gray-700",children:d.jsx("div",{className:"w-full mb-1",children:d.jsx("div",{className:"mb-4",children:d.jsx("h1",{className:"text-xl font-semibold text-gray-900 sm:text-2xl dark:text-white",children:"Home"})})})}),d.jsx("div",{className:"mt-4",children:d.jsxs("div",{className:"grid grid-cols-1 gap-4 mb-4 sm:grid-cols-2 lg:grid-cols-4",children:[d.jsxs("div",{className:"p-4 bg-white border border-gray-200 rounded-lg shadow-sm dark:border-gray-700 sm:p-6 dark:bg-gray-800",children:[d.jsxs("div",{className:"flex items-center justify-between mb-4",children:[d.jsx("h3",{className:"text-lg font-semibold text-gray-900 dark:text-white",children:"Spaces"}),d.jsxs(et,{to:"/spaces",className:"inline-flex items-center text-sm font-medium text-primary-600 hover:text-primary-700 dark:text-primary-500 dark:hover:text-primary-400",children:["View all",d.jsx("svg",{className:"w-4 h-4 ml-1",fill:"currentColor",viewBox:"0 0 20 20",xmlns:"http://www.w3.org/2000/svg",children:d.jsx("path",{fillRule:"evenodd",d:"M7.293 14.707a1 1 0 010-1.414L10.586 10 7.293 6.707a1 1 0 011.414-1.414l4 4a1 1 0 010 1.414l-4 4a1 1 0 01-1.414 0z",clipRule:"evenodd"})})]})]}),d.jsxs("div",{className:"flex items-center mb-2",children:[d.jsx("div",{className:"flex-shrink-0 w-8 h-8 text-2xl flex items-center justify-center bg-blue-100 rounded-full dark:bg-blue-900",children:d.jsx("svg",{className:"w-5 h-5 text-blue-600 dark:text-blue-300",fill:"currentColor",viewBox:"0 0 20 20",xmlns:"http://www.w3.org/2000/svg",children:d.jsx("path",{d:"M6.143 0H1.857A1.857 1.857 0 0 0 0 1.857v4.286C0 7.169.831 8 1.857 8h4.286A1.857 1.857 0 0 0 8 6.143V1.857A1.857 1.857 0 0 0 6.143 0Zm10 0h-4.286A1.857 1.857 0 0 0 10 1.857v4.286C10 7.169 10.831 8 11.857 8h4.286A1.857 1.857 0 0 0 18 6.143V1.857A1.857 1.857 0 0 0 16.143 0Zm-10 10H1.857A1.857 1.857 0 0 0 0 11.857v4.286C0 17.169.831 18 1.857 18h4.286A1.857 1.857 0 0 0 8 16.143v-4.286A1.857 1.857 0 0 0 6.143 10Zm10 0h-4.286A1.857 1.857 0 0 0 10 11.857v4.286c0 1.026.831 1.857 1.857 1.857h4.286A1.857 1.857 0 0 0 18 16.143v-4.286A1.857 1.857 0 0 0 16.143 10Z"})})}),d.jsxs("div",{className:"ml-3",children:[d.jsx("p",{className:"text-sm font-medium text-gray-500 dark:text-gray-400",children:"Total Spaces"}),d.jsx("h3",{className:"text-xl font-bold text-gray-900 dark:text-white",children:"4"})]})]})]}),d.jsxs("div",{className:"p-4 bg-white border border-gray-200 rounded-lg shadow-sm dark:border-gray-700 sm:p-6 dark:bg-gray-800",children:[d.jsxs("div",{className:"flex items-center justify-between mb-4",children:[d.jsx("h3",{className:"text-lg font-semibold text-gray-900 dark:text-white",children:"Query"}),d.jsxs(et,{to:"/query",className:"inline-flex items-center text-sm font-medium text-primary-600 hover:text-primary-700 dark:text-primary-500 dark:hover:text-primary-400",children:["Open",d.jsx("svg",{className:"w-4 h-4 ml-1",fill:"currentColor",viewBox:"0 0 20 20",xmlns:"http://www.w3.org/2000/svg",children:d.jsx("path",{fillRule:"evenodd",d:"M7.293 14.707a1 1 0 010-1.414L10.586 10 7.293 6.707a1 1 0 011.414-1.414l4 4a1 1 0 010 1.414l-4 4a1 1 0 01-1.414 0z",clipRule:"evenodd"})})]})]}),d.jsxs("div",{className:"flex items-center mb-2",children:[d.jsx("div",{className:"flex-shrink-0 w-8 h-8 text-2xl flex items-center justify-center bg-green-100 rounded-full dark:bg-green-900",children:d.jsx("svg",{className:"w-5 h-5 text-green-600 dark:text-green-300",fill:"currentColor",viewBox:"0 0 20 20",xmlns:"http://www.w3.org/2000/svg",children:d.jsx("path",{d:"M5 4a1 1 0 0 0-2 0v7.268a2 2 0 0 0 0 3.464V16a1 1 0 1 0 2 0v-1.268a2 2 0 0 0 0-3.464V4Zm6 0a1 1 0 0 0-2 0v1.268a2 2 0 0 0 0 3.464V16a1 1 0 1 0 2 0V8.732a2 2 0 0 0 0-3.464V4Zm6 0a1 1 0 1 0-2 0v7.268a2 2 0 0 0 0 3.464V16a1 1 0 1 0 2 0v-1.268a2 2 0 0 0 0-3.464V4Z"})})}),d.jsxs("div",{className:"ml-3",children:[d.jsx("p",{className:"text-sm font-medium text-gray-500 dark:text-gray-400",children:"SPARQL Query"}),d.jsx("h3",{className:"text-xl font-bold text-gray-900 dark:text-white",children:"Search & Analyze"})]})]})]})]})})]})}function jy(){return d.jsx(ry,{children:d.jsx(Ty,{children:d.jsxs(q1,{children:[d.jsx(El,{path:"/login",element:d.jsx(Ny,{})}),d.jsx(El,{path:"/",element:d.jsx(Du,{children:d.jsx(Uu,{children:d.jsx(Ry,{})})})}),d.jsx(El,{path:"/spaces",element:d.jsx(Du,{children:d.jsx(Uu,{children:d.jsx(xy,{})})})}),d.jsx(El,{path:"/query",element:d.jsx(Du,{children:d.jsx(Uu,{children:d.jsx(Sy,{})})})}),d.jsx(El,{path:"/settings",element:d.jsx(Du,{children:d.jsx(Uu,{children:d.jsx(Ey,{})})})}),d.jsx(El,{path:"/profile",element:d.jsx(Du,{children:d.jsx(Uu,{children:d.jsx(Ay,{})})})}),d.jsx(El,{path:"/dashboard",element:d.jsx(c0,{to:"/",replace:!0})})]})})})}localStorage.getItem("color-theme")==="dark"||!("color-theme"in localStorage)&&window.matchMedia("(prefers-color-scheme: dark)").matches?document.documentElement.classList.add("dark"):document.documentElement.classList.remove("dark");Km.createRoot(document.getElementById("root")).render(d.jsx(Ym.StrictMode,{children:d.jsx(jy,{})}));
